/*
 * Created on Aug 1, 2007
 *
 */
package edu.bu.signstream.codingScheme.ui.importLegacyCodingScheme;

import java.text.CollationKey;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;
import java.util.logging.Level;

//import edu.bu.signstream.common.db.parser.SS2Parser;
import edu.bu.signstream.common.db.parser.SS3Parser;
import edu.bu.signstream.common.util.vo.CodingSchemaField;
import edu.bu.signstream.common.util.vo.CodingSchemaFieldType;
import edu.bu.signstream.common.util.vo.CodingSchemaValue;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class CodingSchemaService {

	private SS3CodingScheme masterSchema;

	// private SS2Parser ss2Parser;
	private SS3Parser ss3Parser;

	private int counter = 0;

	public CodingSchemaService() {
		// this.ss2Parser = new SS2Parser();
		this.ss3Parser = new SS3Parser();
	}

	public void loadMasterSchema(String fileName) {
		this.masterSchema = ss3Parser.parseSS3CodingScheme(fileName);
	}

	public void loadMasterSchema(SS3CodingScheme masterSchema) {
		this.masterSchema = masterSchema;
	}

	public LegacyCodingSchemaFormBean compare(HashMap lCodingSchema) {

		SS3SignStreamApplication.logger.log(Level.INFO,
				"Comparing coding schemas...");

		LegacyCodingSchemaFormBean formBean = new LegacyCodingSchemaFormBean();

		Collection legacyCSFields = lCodingSchema.values();
		Iterator iterator = legacyCSFields.iterator();

		while (iterator.hasNext()) {
			CodingSchemaField csField = (CodingSchemaField) iterator.next();
			LegacyCodingSchemaEntity entity = compareField(csField);

			formBean.addEntity(entity);
		}

		return formBean;
	}

	public SS3CodingScheme translateToCodingSchema(
			LegacyCodingSchemaFormBean fb, HashMap lCodingSchema,
			ArrayList diffList) {
		ArrayList fields = new ArrayList();

		Iterator it = diffList.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			Object obj = lCodingSchema.get(key);

			if (obj != null) {
				fields.add(obj);
			}
		}

		return convertToSS3CodingScheme(fields);
	}

	private SS3CodingScheme convertToSS3CodingScheme(ArrayList codingScheme) {
		SS3CodingScheme ss3CodingScheme = new SS3CodingScheme();

		for (int i = 0; i < codingScheme.size(); i++) {
			CodingSchemaField field = (CodingSchemaField) codingScheme.get(i);

			String legacyID = "" + field.getFieldId();

			SS3Field ss3Field = new SS3Field();
			ss3Field.setId(edu.bu.signstream.common.util.Util.getUniqueNumber());
			ss3Field.setLegacyID(legacyID);
			ss3Field.setLabel(field.getLabel());
			ss3Field.setName(field.getName());
			ss3Field.setPrefix(field.getPrefix());
			CodingSchemaFieldType type = field.getCodingSchemaFieldType();
			ss3Field.setType(getSS3FieldType(type.getType()));

			ArrayList lValues = field.getCodingSchemeValues();
			for (int j = 0; j < lValues.size(); j++) {
				SS3FieldValue value = new SS3FieldValue();
				CodingSchemaValue lValue = (CodingSchemaValue) lValues.get(j);
				value.setId(lValue.getValueIDAsString());
				value.setLabel(lValue.getLabel());
				value.setName(lValue.getName());
				value.setParentFieldID(legacyID);
				ss3Field.addValue(value);
			}

			ss3CodingScheme.addField(ss3Field);
		}

		ss3CodingScheme.setID(edu.bu.signstream.common.util.Util
				.getUniqueNumber());

		return ss3CodingScheme;
	}

	private String getSS3FieldType(String type) {
		if (type.equalsIgnoreCase(Constants.TYPE_NON_MANUAL_OLD)) {
			return Constants.TYPE_NON_MANUAL;
		} else if (type.equalsIgnoreCase(Constants.TYPE_GLOSS_OLD)) {
			return Constants.TYPE_GLOSS_LEGACY;
		} else if (type.equalsIgnoreCase(Constants.TYPE_FREE_TEXT_OLD)) {
			return Constants.TYPE_FREE_TEXT;
		}
		return type;
	}

	public SS3CodingScheme getMasterSchema() {
		return masterSchema;
	}

	private LegacyCodingSchemaEntity compareField(CodingSchemaField csField) {

		// SS3SignStreamApplication.logger.log(Level.INFO,
		// "Comparing coding schema field ["+csField.toString()+"]");

		LegacyCodingSchemaEntity entity = new LegacyCodingSchemaEntity();
		entity.setId("" + counter);
		entity.setFieldID("" + csField.getFieldId());
		counter++;
		entity.setName(csField.getName());
		entity.setLabel(csField.getLabel());

		SS3Field masterField = getSS3FieldByName(csField.getName(),
				masterSchema.getFields().values());

		if (masterField == null) {

			if (csField.getName().equalsIgnoreCase("English mouthing")) {
				masterField = getSS3FieldByName("Mouthing", masterSchema
						.getFields().values());
				if (masterField != null) {
					csField.setName(masterField.getName());
					csField.setLabel(masterField.getLabel());
				}
			} else if (csField.getName().endsWith("LDC POS")) {
				// skip it, do not transfer
				;
			} else {
				// String name = csField.getName();
				entity.setDiffEntityMarker("*");
				entity.setNotFoundInMasterSchemaMarker("*");

				// add children
				ArrayList list = compareValues(csField.getCodingSchemeValues(),
						new ArrayList());
				entity.setChildren(list);
			}

		} else {
			String mName = masterField.getName();
			String mLabel = masterField.getLabel();

			if (!csField.getName().equals(mName)) {
				csField.setName(mName);
			}

			if (!csField.getLabel().equals(mLabel)) {
				csField.setLabel(mLabel);
			}

			// IF FIELD IS "ROLE SHIFT", do not compare values
			if (!mName.equals("role shift")) {

				boolean legacySchemaHasValues = csField.getCodingSchemeValues() != null
						&& !csField.getCodingSchemeValues().isEmpty();
				boolean masterSchemaHasValues = masterField.getValues() != null
						&& !masterField.getValues().isEmpty();

				if (legacySchemaHasValues != masterSchemaHasValues) {
					entity.setDiffEntityMarker("*");
					
					System.out.println("1. field " + mName);
				}

				if (legacySchemaHasValues) {
					ArrayList list = compareValues(
							csField.getCodingSchemeValues(),
							masterField.getValuesAsList());
					entity.setChildren(list);

					if (haveDifferentValues(list))
						entity.setDiffEntityMarker("*");
				}
			}
		}
		return entity;
	}

	private boolean haveDifferentValues(ArrayList valueEntities) {
		Iterator it = valueEntities.iterator();
		while (it.hasNext()) {
			LegacyCodingSchemaEntity vEntity = (LegacyCodingSchemaEntity) it
					.next();
			if (vEntity.getDiffEntityMarker().equals("*")) {
				return true;
			}
		}
		return false;
	}

	private ArrayList compareValues(Collection legacyValues,
			Collection masterValues) {
		int i = 0;

		ArrayList entities = new ArrayList();
		Iterator legacyIter = legacyValues.iterator();
		while (legacyIter.hasNext()) {
			CodingSchemaValue lValue = (CodingSchemaValue) legacyIter.next();

			LegacyCodingSchemaEntity entity = new LegacyCodingSchemaEntity();
			entity.setId("" + i);
			i++;
			entity.setName(lValue.getName());
			entity.setLabel(lValue.getLabel());
			entity.setDiffEntityMarker("");
			entity.setNotFoundInMasterSchemaMarker("");

			SS3FieldValue mValue = getSS3ValueByName(lValue.getName(), masterValues);

			if (mValue == null) {
				mValue = getSS3ValueByLabel(lValue.getLabel(), masterValues);
				if(mValue!= null) {
					lValue.setName(mValue.getName());
				} else {	
					entity.setDiffEntityMarker("*");
					entity.setNotFoundInMasterSchemaMarker("*");
				}
			} else {
				String mName = mValue.getName();
				String mLabel = mValue.getLabel();

				if (!lValue.getName().equals(mName)) {
					entity.setMasterName(mName);
					entity.setDiffEntityMarker("*");
					//
					System.out.println("3. field value " + mValue);
					//
				}

				if (!lValue.getLabel().equals(mLabel)) {
					lValue.setLabel(mLabel);
				}
			}
			entities.add(entity);
		}
		return entities;
	}

	private SS3FieldValue getSS3ValueByName(String name, Collection masterValues) {
		Iterator iter = masterValues.iterator();
		while (iter.hasNext()) {
			SS3FieldValue value = (SS3FieldValue) iter.next();
			if (name.equals(value.getName()))
				return value;
		}
		return null;
	}
	
	private SS3FieldValue getSS3ValueByLabel(String label, Collection masterValues) {
		Iterator iter = masterValues.iterator();
		while (iter.hasNext()) {
			SS3FieldValue value = (SS3FieldValue) iter.next();
			if (label.equals(value.getLabel()))
				return value;
		}
		return null;
	}

	private SS3Field getSS3FieldByName(String name, Collection fields) {
		Iterator iterator = fields.iterator();
		while (iterator.hasNext()) {
			SS3Field field = (SS3Field) iterator.next();

			if (field.getName().equalsIgnoreCase(name))
				return field;
		}
		return null;
	}

	private SS3FieldValue getSS3FieldValueByName(String name,
			Collection fieldValues) {
		Iterator iterator = fieldValues.iterator();
		while (iterator.hasNext()) {
			SS3FieldValue fieldValue = (SS3FieldValue) iterator.next();

			if (fieldValue.getName().equalsIgnoreCase(name))
				return fieldValue;
		}
		return null;
	}

	public ArrayList sortByFiedName(Collection fields) {
		ArrayList list = new ArrayList();

		Collator collator = Collator.getInstance();
		Set set = new TreeSet();

		Iterator cIterator = fields.iterator();
		while (cIterator.hasNext()) {
			SS3Field field = (SS3Field) cIterator.next();
			set.add(collator.getCollationKey(field.getName()));
		}

		Iterator iterator = set.iterator();
		while (iterator.hasNext()) {
			CollationKey key = (CollationKey) iterator.next();
			String fieldName = key.getSourceString();
			SS3Field cfield = getSS3FieldByName(fieldName, fields);
			list.add(cfield);
		}

		return list;
	}

	public ArrayList sortByValueName(Collection values) {
		ArrayList list = new ArrayList();

		Collator collator = Collator.getInstance();
		Set set = new TreeSet();

		Iterator cIterator = values.iterator();
		while (cIterator.hasNext()) {
			SS3FieldValue value = (SS3FieldValue) cIterator.next();
			set.add(collator.getCollationKey(value.getName()));
		}

		Iterator iterator = set.iterator();
		while (iterator.hasNext()) {
			CollationKey key = (CollationKey) iterator.next();
			String valueName = key.getSourceString();
			SS3FieldValue cValue = getSS3FieldValueByName(valueName, values);
			list.add(cValue);
		}

		return list;
	}
}
