package edu.bu.signstream.common.db;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.Reference;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3DBProfile;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Element;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItemNote;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3StatementField;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaLibrary;
import edu.bu.signstream.common.util.vo.ss3.note.SS3Note;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.SerchableDirectory;

/**
 * Temporary class to save SS3 database.
 */
public class SS3DatabaseWriter {

	private Element createDistributorElement(SS3DBProfile prf, Document document) {
		Element root = document.createElement("DISTRIBUTOR");
		root.appendChild(document.createTextNode(prf.getDistributor()));
		return root;
	}

	private Element createUtilsElement(SS3Database db, Document document) {
		Element root = document.createElement("UTILS");
		root.setAttribute("TP_PIXEL_IN_TIME_FRAME",
				"" + db.getTemporalPartitionPixelInTimeFrame());
		root.setAttribute("TP_ONSCREEN_LOCATION",
				"" + db.getTemporalPartitionOnScreenLocation());
		root.setAttribute("TP_MOVIE_TIME",
				"" + db.getTemporalPartitionMovieTime());
		return root;
	}

	private Element createReferencesElement(SS3Database db, Document document) {
		Element root = document.createElement("REFERENCES");
		root.appendChild(createCodingSchemasElement(db, document));
		root.appendChild(createParticipantsElement(db, document));
		root.appendChild(createAuthorsElement(db, document));
		return root;
	}

	private Element createCodingSchemasElement(SS3Database db, Document document) {
		Element root = document.createElement("CODING-SCHEMES");
		ArrayList csRef = db.getCodingSchemeRef();
		for (int i = 0; i < csRef.size(); i++) {
			Reference ref = (Reference) csRef.get(i);
			Element rEl = document.createElement("REF");
			rEl.setAttribute("ID", "" + ref.getId());
			rEl.setAttribute("FILE_NAME", "" + ref.getFileName());
			rEl.setAttribute("SIGNATURE", "" + ref.getSignature());
			root.appendChild(rEl);
		}
		return root;
	}

	private Element createParticipantsElement(SS3Database db, Document document) {
		Element root = document.createElement("PARTICIPANTS");
		ArrayList<SS3Participant> participants = db.getSelectedParticipants();

		for (SS3Participant participant : participants) {
			Element pEl = document.createElement("REF");
			pEl.setAttribute("ID", "" + participant.getId());
			pEl.setAttribute("LABEL", "" + participant.getLabel());
			root.appendChild(pEl);
		}
		return root;
	}

	private Element createAuthorsElement(SS3Database db, Document document) {
		Element root = document.createElement("ANNOTATORS");
		ArrayList<SS3Participant> annotators = db.getSelectedAnnotators();

		for (SS3Participant annotator : annotators) {
			Element aEl = document.createElement("REF");
			aEl.setAttribute("ID", "" + annotator.getId());
			aEl.setAttribute("LABEL", "" + annotator.getLabel());
			root.appendChild(aEl);
		}
		return root;
	}

	private Element createCodingSchemeElement(SS3Database db, Document document) {

		Element root = document.createElement("LOCAL-CODING-SCHEME");
		SS3CodingScheme scheme = db.getLocalSS3CodingScheme();

		if (scheme == null)
			return root;

		HashMap fields = scheme.getFields();

		Set keys = fields.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {
			Object obj = fields.get(it.next());
			SS3Field field = (SS3Field) obj;

			String name = field.getName() == null ? "" : field.getName();
			String label = field.getLabel() == null ? "" : field.getLabel();
			String color = field.getColor() == null ? "" : ""
					+ field.getColor().getRGB();
			String type = field.getType() == null ? "" : field.getType();
			String group = field.getGroup() == null ? "" : field.getGroup();

			Element fldEl = document.createElement("FIELD");
			fldEl.setAttribute("ID", field.getId());
			fldEl.setAttribute("NAME", name);
			fldEl.setAttribute("LABEL", label);
			fldEl.setAttribute("COLOR", color);
			fldEl.setAttribute("TYPE", type);
			fldEl.setAttribute("GROUP", group);

			ArrayList<SS3FieldValue> values = field.getValuesAsList();
			for (SS3FieldValue value : values) {
				Element valEl = document.createElement("VALUE");

				String vName = value.getName() == null ? "" : value.getName();
				String vLabel = value.getLabel() == null ? "" : value
						.getLabel();
				String iid = value.getIid() == null ? "" : value.getIid();

				valEl.setAttribute("ID", value.getId());
				valEl.setAttribute("NAME", vName);
				valEl.setAttribute("LABEL", vLabel);
				valEl.setAttribute("IID", iid);
				fldEl.appendChild(valEl);
			}

			root.appendChild(fldEl);
		}
		return root;
	}

	private Element createMediaFilesOnScreenLocationElement(SS3Element element,
			Document document) {
		Element root = document.createElement("MEDIA_FILES_ON_SCREEN_LOCATION");
		HashMap<String, Point> locations = element.getVideoOnScreenLocations();
		Set<String> keySet = locations.keySet();
		Iterator<String> keyIter = keySet.iterator();

		while (keyIter.hasNext()) {
			String key = keyIter.next();
			Point point = locations.get(key);
			Element mlEl = document.createElement("MEDIA-FILE");
			mlEl.setAttribute("FILE_NAME", key);
			mlEl.setAttribute("LOCATION_X", "" + point.x);
			mlEl.setAttribute("LOCATION_Y", "" + point.y);
			root.appendChild(mlEl);
		}

		return root;
	}

	private Element createMediaFilesElement(SS3Database db, Document document) {
		Element root = document.createElement("MEDIA-FILES");
		root.setAttribute("MOVIE_DURATION", "" + db.getMovieDuration());
		root.setAttribute("PACKET_DURATION", "" + db.getPacketDuration());

		ArrayList<SS3MediaFile> mFiles = db.getMediaFilesCollection();

		for (SS3MediaFile mFile : mFiles) {
			Element mfEl = document.createElement("MEDIA-FILE");
			mfEl.setAttribute("ID", mFile.getId());
			mfEl.setAttribute("PATH", mFile.getFile().getPath());
			root.appendChild(mfEl);
		}
		return root;
	}

	private Element createGlossWindowElement(SS3Database db, Document document) {
		Element root = document.createElement("GLOSS-WINDOW");
		Element dtEl = document.createElement("DISPLAY-TIME-PERIOD");
		root.appendChild(dtEl);

		SS3GlossWindow window = db.getSS3GlossWindow();
		ArrayList<SS3GlossWindowItem> items = window.getDisplayTimePeriod()
				.getSS3GlossWindowItems();

		for (SS3GlossWindowItem item : items) {
			item.synchronize();

			Element sEl = document.createElement("GLOSS-WINDOW-ITEM");
			sEl.setAttribute("ID", item.getId());
			sEl.setAttribute("LABEL", item.getLabel());
			sEl.setAttribute("S", "" + item.getStartTimeInfo().getMovieTime());
			sEl.setAttribute("E", "" + item.getEndTimeInfo().getMovieTime());
			sEl.setAttribute("PARTICIPANT-ID", ""
					+ item.getSelectedParticipant().getId());
			sEl.setAttribute("ANNOTATOR-ID", "" + item.getSelectedAnnotatorID());

			dtEl.appendChild(sEl);
		}

		Element tpEl = document.createElement("TEMPORAL-PARTITIONS");
		root.appendChild(tpEl);
		
		ArrayList<SS3TemporalPartition> temporalPartitions = window
				.getTemporalPartitions();

		for (SS3TemporalPartition temporalPartition : temporalPartitions) {
			Element tp1El = document.createElement("TEMPORAL-PARTITION");
			tp1El.setAttribute("ID", temporalPartition.getID());
			tp1El.setAttribute("LABEL", temporalPartition.getLabel());
			tpEl.appendChild(tp1El);

			ArrayList<SS3GlossWindowSegment> segments = temporalPartition
					.getTemporalPartitionSegments();
			for (SS3GlossWindowSegment segment : segments) {
				Element sEl = document
						.createElement("TEMPORAL-PARTITION-SEGMENT");
				sEl.setAttribute("ID", segment.getID());
				sEl.setAttribute("LABEL", segment.getLabel());
				sEl.setAttribute("ISDEFAULT", "" + segment.isDefault());
				String participantID = segment.getParticipant() != null ? segment.getParticipant().getId() : "";
				sEl.setAttribute("PARTICIPANT_ID", "" + participantID);

				SS3GlossWindowItemNote note = segment.getNote();
				if (note != null) {
					String noteStr = note.getNote();
					if (noteStr != null && !noteStr.isEmpty()) {
						sEl.setAttribute("NOTE", noteStr);
					}
				}
				
				tp1El.appendChild(sEl);

				ArrayList<SS3GlossWindowItem> segmentItems = segment
						.getSS3GlossWindowItems();
				for (SS3GlossWindowItem segmentItem : segmentItems) {
					segmentItem.synchronize();

					SS3Participant annotator = segmentItem
							.getSelectedAnotator();
					SS3Participant participant = segmentItem
							.getSelectedParticipant();

					Element wEl = document.createElement("GLOSS-WINDOW-ITEM");
					wEl.setAttribute("ID", segmentItem.getId());
					wEl.setAttribute("LABEL", segmentItem.getLabel());
					wEl.setAttribute("S", ""
							+ segmentItem.getStartTimeInfo().getMovieTime());
					wEl.setAttribute("E", ""
							+ segmentItem.getEndTimeInfo().getMovieTime());

					if (participant != null) {
						wEl.setAttribute("SELECTED_PARTICIPANT_ID",
								participant.getId());
					}

					if (annotator != null) {
						wEl.setAttribute("SELECTED_ANNOTATOR_ID",
								annotator.getId());
					}

					sEl.appendChild(wEl);

					ArrayList<SS3StatementField> statementFields = segmentItem
							.getStatementFields();
					for (SS3StatementField statementField : statementFields) {
						// STATEMENT-FIELD
						Element stmtEl = document
								.createElement("STATEMENT-FIELD");
						stmtEl.setAttribute("FIELD-ID", statementField.getId());

						Color fieldColor = statementField.getColor();
						if (fieldColor != null) {
							stmtEl.setAttribute("COLOR",
									"" + fieldColor.getRGB());
						}
						
						ArrayList<String> entityIDs = new ArrayList<String>();
						
						ArrayList<SS3Entity> ss3Entities = statementField.getSS3Entities();
						for (SS3Entity ss3Entity: ss3Entities) {
							entityIDs.add(ss3Entity.getID());
						}

						StringBuffer bf = new StringBuffer();

						for (int counter = 0; counter < entityIDs.size(); counter++) {
							bf.append(entityIDs.get(counter));
							if (counter < entityIDs.size() - 1)
								bf.append(", ");
						}

						stmtEl.setAttribute("ENTITY-IDS", bf.toString());
						wEl.appendChild(stmtEl);
					}
				}
			}
		}
		return root;
	}

	private Element createDatabaseElement(SS3Database db, Document document) {
		Element root = document.createElement("DATABASE");

		root.appendChild(createTracksElement(db, document));
		root.appendChild(createNotesElement(db, document));
		root.appendChild(createGlossWindowElement(db, document));
		// root.appendChild(createStatementsElement(db, document));
		return root;
	}

	private Element createTracksElement(SS3Database db, Document document) {
		Element root = document.createElement("TRACKS");
		ArrayList tracks = db.getTracksList();
		for (int i = 0; i < tracks.size(); i++) {
			SS3Track track = (SS3Track) tracks.get(i);

			Element tEl = document.createElement("TRACK");
			tEl.setAttribute("FID", "" + track.getId());
			ArrayList<SS3Entity> entities = track.getEntities();
			for (SS3Entity entity : entities) {
				Element eEl = document.createElement("ENTITY");
				eEl.setAttribute("ID", entity.getID());
				eEl.setAttribute("PARTICIPANT-ID", entity.getParticipantID());
				eEl.setAttribute("CODER-ID", entity.getCoderID());
				eEl.setAttribute("TIME_CREATED", "");
				eEl.setAttribute("TIME_UPDATED", "");
				eEl.setAttribute("REF_ENTITY_ID", entity.getRefEntityId());
				eEl.setAttribute("PARENT_ENTITY_ID", entity.getParentEntityID());
				eEl.setAttribute("COMP_START_ENTITY_ID",
						entity.getStartCompaundEntityID());
				eEl.setAttribute("COMP_END_ENTITY_ID",
						entity.getEndCompaundEntityID());
				// eEl.setAttribute("VALUE_START_LOCKED", Util
				// .getBooleanAsString(entity.isValueStartTimeLocked()));
				// eEl.setAttribute("VALUE_END_LOCKED", Util
				// .getBooleanAsString(entity.isValueEndTimeLocked()));
				// eEl.setAttribute("INITIAL_HOLD_LOCKED", Util
				// .getBooleanAsString(entity
				// .isInitialHoldStartTimeLocked()));
				// eEl.setAttribute("FINAL_HOLD_LOCKED", Util
				// .getBooleanAsString(entity.isFinalHoldEndTimeLocked()));

				GlossEntityProperties properties = entity
						.getGlossEntityProperties();

				if (track.getId().equalsIgnoreCase("1000")
						|| track.getId().equalsIgnoreCase("2000")
						|| track.getId().equalsIgnoreCase("10501")
						|| track.getId().equalsIgnoreCase("10502")) {
					
					boolean locked = entity.isLockedGlossPhoneAlignments();					
					
					if(locked) {
						eEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "T");
					} else {
						eEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "F");
					}
				}

				if ((track.getId().equalsIgnoreCase("10000") || track.getId()
						.equalsIgnoreCase("10001"))) {
					if (properties != null) {
						// sign bank flags
						boolean isVerifiedFromSignBank = properties.isVerifiedFromSignBank();
						boolean isDifferingFromOccurrence = properties.isDifferingFromOccurrence();
						String sourceResource = properties.getSourceResource();
						String mainGlossID = properties.getMainGlossID();
						String variantID = properties.getVariantID();
						String occurrenceID = properties.getOccurrenceID();
						
						if (isVerifiedFromSignBank) {
							Element sbEl = document.createElement("SIGNBANKDATA");
							
							sbEl.setAttribute("VERIFIED", "T");
						
							if (isDifferingFromOccurrence)
								sbEl.setAttribute("DIFF_FROM_OCCURRENCE", "T");
	//						else
	//							sbEl.setAttribute("DIFF_FROM_OCCURRENCE", "F");
							
							if (sourceResource != null)
								sbEl.setAttribute("RESOURCE", sourceResource);
							
							if (mainGlossID != null)
								sbEl.setAttribute("MAIN_GLOSS_ID", mainGlossID);
							
							if (variantID != null)
								sbEl.setAttribute("VARIANT_ID", variantID);
														
							if (occurrenceID != null)
								sbEl.setAttribute("OCCURRENCE_ID", occurrenceID);
							
							eEl.appendChild(sbEl);
						}
//						else
//							sbEl.setAttribute("VERIFIED", "F");	// if we want to implement this, then need to add SIGNBANKDATA element to all entity elements	
						
						Element pEl = document.createElement("PROPERTIES");
						pEl.setAttribute("SIGN", properties.getClassifier());
						pEl.setAttribute("NAME_SIGN", properties.getNameSign());
						pEl.setAttribute("PASSIVE_BASE_ARM", properties.getPassiveBaseArm());
						pEl.setAttribute("NO_OF_HANDS", properties.getNumberOfHands());

						pEl.setAttribute("TWOHANDED_HADSHAPES", properties.getTwoHandedHandshapesClassification());
						pEl.setAttribute("START_END_HANDSHAPES", properties.getStartEndHandshape());
						pEl.setAttribute("LEFT_HAND_START_END_HANDSHAPE", properties.getStartEndHandshapeLeftHand());
						pEl.setAttribute("RIGHT_HAND_START_END_HANDSHAPE",
								properties.getStartEndHandshapeRightHand());

						eEl.appendChild(pEl);

						eEl.setAttribute("DOMINANCE",
								properties.getDominamtHand());
						eEl.setAttribute("MARKED_NUMBER_OF_HANDS",
								properties.getMarkedNumberOfHands());
						eEl.setAttribute("CORRESPONDING_GLOSS_ENTITY_ID",
								entity.getDependentGlossEntityID());
					}
					
					boolean lockedGlossPhoneAlignments = entity
							.isLockedGlossPhoneAlignments();
					boolean lockedLeftRightAlignments = entity
							.isLockedLeftRightAlignments();

					String lockedGlossPhonAlignment = lockedGlossPhoneAlignments ? "T"
							: "F";
					String lockedLeftRightAlignment = lockedLeftRightAlignments ? "T"
							: "F";

					String depGlossID = entity.getDependentGlossEntityID();
					String depHandID = entity.getDependentHandEntityID();
					String depHandShapeID = entity
							.getDependentHandShapeEntityID();
				
					eEl.setAttribute("LOCKED_LEF_RIGHT_ALIGNMENT",
							lockedLeftRightAlignment);
					eEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT",
							lockedGlossPhonAlignment);

					eEl.setAttribute("DEPENDENT_GLOSS_ID", depGlossID);
					eEl.setAttribute("DEPENDENT_HAND_ID", depHandID);
					eEl.setAttribute("DEPENDENT_HAND_SHAPE_ID", depHandShapeID);
				} else if (track.getId().equalsIgnoreCase(Util.FREE_TEXT_FIELD_ID)) {
					if (entity.isValidTime()) {
						eEl.setAttribute("VALID_TIME", "T");
					}
				}

				/**
				 * <ENTITY> <PROPERTIES
				 * SIGN="FINGERSPELLED/LOAN/LEXICAL/CLASSIFIER/GESTURE/NUMBER"
				 * NAME_SIGN="T/F" PASSIVE_BASE_ARM="T/F" NO_OF_HANDS="1/2"
				 * 
				 * TWOHANDED_HADSHAPES="SAME/ALTERNATING_SAME/DIFFERENT"
				 * START_END_HANDSHAPES="SAME/DIFFERENT"
				 * LEFT_HAND_START_END_HANDSHAPE="SAME/DIFFERENT"
				 * RIGHT_HAND_START_END_HANDSHAPE="SAME/DIFFERENT" />
				 * <DOMINANCE="L/R"/> <MARKED_NUMBER_OF_HANDS="T/F"/> </ENTITY>
				 */

				ArrayList<SS3Item> items = entity.getItemsAsList();
				for (SS3Item item : items) {
					Element iEl = document.createElement("ITEM");
					iEl.setAttribute("VID", item.getId());

					if (item.isTextOnly()) {
						iEl.setAttribute("TEXT_ONLY", "T");
					}

					if (item.getNoteID() != null
							&& item.getNoteID().length() > 0) {
						iEl.setAttribute("NOTE_ID", item.getNoteID());
					}
					iEl.setAttribute("S", ""
							+ item.getStartTimeInfo().getMovieTime());
					iEl.setAttribute("E", ""
							+ item.getEndTimeInfo().getMovieTime());
					if (item.getText() != null && item.getText().length() > 0) {

						// String str = item.getText();
						// System.out.println("TEXT = [" + str + "]");

						iEl.appendChild(document.createTextNode(item.getText()));
					}
					eEl.appendChild(iEl);
				}
				tEl.appendChild(eEl);
			}

			ArrayList<SS3Item> nonCompleteEventsList = track
					.getNonCompleteEvents();
			if (nonCompleteEventsList.size() > 0) {
				Element neEl = document.createElement("NON_COMPLETE_EVENTS");
				for (int n = 0; n < nonCompleteEventsList.size(); n++) {
					SS3Item item = nonCompleteEventsList.get(n);

					Element iEl = document.createElement("ITEM");
					iEl.setAttribute("S", ""
							+ item.getStartTimeInfo().getMovieTime());
					iEl.setAttribute("E", ""
							+ item.getEndTimeInfo().getMovieTime());
					String sf = item.isStartTimeFinal() ? "T" : "F";
					iEl.setAttribute("SF", sf);
					String ef = item.isEndTimeFinal() ? "T" : "F";
					iEl.setAttribute("EF", ef);
					if (item.getText() != null && item.getText().length() > 0) {
						iEl.appendChild(document.createTextNode(item.getText()));
					}

					iEl.setAttribute("COMP_START_ENTITY_ID",
							item.getStartCompaundEntityID());
					iEl.setAttribute("COMP_END_ENTITY_ID",
							item.getEndCompaundEntityID());

					neEl.appendChild(iEl);
				}
				tEl.appendChild(neEl);
			}

			ArrayList<SS3Item> enteredTxtList = track.getEnteredTextEvents();
			if (enteredTxtList.size() > 0) {
				Element etEl = document.createElement("ENTERED_TEXT");
				for (int t = 0; t < enteredTxtList.size(); t++) {
					SS3Item item1 = enteredTxtList.get(t);

					Element iEl = document.createElement("ITEM");
					iEl.setAttribute("S", ""
							+ item1.getStartTimeInfo().getMovieTime());
					iEl.setAttribute("E", "");
					if (item1.getText() != null && item1.getText().length() > 0) {
						iEl.appendChild(document.createTextNode(item1.getText()));
					}
					etEl.appendChild(iEl);
				}
				tEl.appendChild(etEl);
			}
			root.appendChild(tEl);
		}
		return root;
	}

	private Element createNotesElement(SS3Database db, Document document) {
		Element root = document.createElement("NOTES");
		ArrayList<SS3Note> notes = db.getNotesList();
		for (SS3Note note : notes) {
			Element nEl = document.createElement("NOTE");
			nEl.setAttribute("ID", note.getID());
			nEl.setAttribute("CODER-ID", note.getCoderID());
			nEl.setAttribute("TIME_CREATED", note.getTimeCreated());
			nEl.setAttribute("TIME_UPDATED", note.getTimeUpdated());
			Element sEl = document.createElement("SUBJECT");
			sEl.appendChild(document.createTextNode(note.getSubject()));
			nEl.appendChild(sEl);
			Element bEl = document.createElement("BODY");
			bEl.appendChild(document.createTextNode(note.getBody()));
			nEl.appendChild(bEl);
			root.appendChild(nEl);
		}
		return root;
	}

	public void saveMediaLibrary(SS3MediaLibrary library) throws Exception {
		Document document = XMLWriter.createNewXMLDocument(library
				.getFileName());

		try {
			Element root = document.createElement("SIGNSTREAM-DATABASE");
			Element distributorElement = document.createElement("DISTRIBUTOR");
			distributorElement
					.appendChild(document
							.createTextNode("American Sign Language Linguistic Research Project, Boston University, Boston, MA."));
			// document.createTextNode(prf.getDistributor()));
			root.appendChild(distributorElement);

			Element searchLocationElement = document
					.createElement("SEARCH_LOCATIONS");
			root.appendChild(searchLocationElement);

			int priority = 1;
			ArrayList<SerchableDirectory> searchLocations = library
					.getDirectoriesToSearch();
			for (SerchableDirectory directory : searchLocations) {
				Element searchElement = document
						.createElement("SEARCH_LOCATION");
				searchElement.setAttribute("PATH", directory.getDirectory());
				String searchable = directory.isSearchable() ? "Y" : "N";
				searchElement.setAttribute("LOOKUP", searchable);
				searchElement.setAttribute("PRIORITY", "" + priority);
				int numberOfSearches = directory.isSearchable() ? directory
						.getNumberOfSearches() + 1 : directory
						.getNumberOfSearches();
				searchElement.setAttribute("SEARCH_COUNT", ""
						+ numberOfSearches);
				priority++;
				searchLocationElement.appendChild(searchElement);
			}

			XMLWriter.saveDocument(root, library.getFileName());

		} catch (Exception ex) {

			SS3SignStreamApplication.logger.log(Level.INFO,
					"Media library [" + library.getFileName()
							+ "] had NOT been saved. " + ex.getStackTrace());

			// ex.printStackTrace();

			throw new Exception("Media library [" + library.getFileName()
					+ "] had NOT been saved.");
		}
	}

	public boolean deleteDatabase(SS3Database db) {
		String directory = "SignStreamDatabases" + File.separator
				+ db.getDirectory();
		String fileName = db.getFileName();

		if (db.getReadFromXMLFile() != null) {
			fileName = db.getReadFromXMLFile().getPath();
		} else {
			fileName = directory + File.separator + db.getFileName();
		}

		File file = new File(fileName);
		return file.delete();
	}

	public boolean saveDocument(SS3Database db) throws Exception {
		String dbDirectory = db.getParentDirectoryPath();
		
		String dir = db.getDirectory();
		if(dir.endsWith(".ss3")) {
			int lenght = dir.length();
			int lenghtToHave = lenght - 4;
			dir = dir.substring(0, lenghtToHave);
		}
		
		String directory = dbDirectory + File.separator + dir;


		SS3SignStreamApplication.logger.log(Level.INFO, "Saving database ["
				+ directory + "]...");

		String collectionName = SS3Singleton.getSignStreamApplication().getTitle();

		File databasesFile = new File(dbDirectory);
		if (!databasesFile.exists()) {
			try {
				boolean createDirectory = databasesFile.mkdir();

				if (!createDirectory) {
					String[] str = new String[4];
					str[0] = "Collection " + collectionName
							+ " could not be saved ";
					str[1] = "because its parent directory could not be created.";
					str[2] = "Please make sure that only legal characters are used.";
					str[3] = "Also try saving the collection in a different directory.";

					SS3Singleton.getErrorMessages()
							.showSaveCollectionErrorMessage(str);
					return false;
				}
			} catch (SecurityException sEx) {
				String[] str = new String[4];
				str[0] = "Collection " + collectionName
						+ " could not be saved ";
				str[1] = "The Security manager does not permit the named";
				str[2] = "directory to be created.  Please verify write";
				str[3] = "permissions and try again.";

				SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(
						str);
				return false;
			}
		}

		File directoryFile = new File(directory);

		if (!directoryFile.exists()) {
			try {
				boolean createFile = directoryFile.mkdir();
				if (!createFile) {

					String[] str = new String[4];
					str[0] = "Collection " + collectionName
							+ " could not be saved ";
					str[1] = "because collection directory was not created. ";
					str[2] = "Please make sure that only legal characters had been used.";
					str[3] = "Also try saving the collection in different directory.";

					SS3Singleton.getErrorMessages()
							.showSaveCollectionErrorMessage(str);
					return false;
				}
			} catch (SecurityException sEx) {

				String[] str = new String[4];
				str[0] = "Collection " + collectionName
						+ " could not be saved. ";
				str[1] = "The Security manager does not permit the named";
				str[2] = "directory to be created.  Please verify write";
				str[3] = "permissions and try again.";

				SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(
						str);
				return false;
			}
		}

		String name0 = db.getFileName();

		int t = name0.indexOf(".3.xml");
		if (t == -1) {
			t = name0.indexOf(".xml");
		}

		String name = name0;
		if (t > -1) {
			name = name0.substring(0, t);
		}

		// check if filename is valid
		String newFileName = "";
		char[] chars = name.toCharArray();

		for (int i = 0; i < chars.length; i++) {
			char c = chars[i];
			if (Character.isLetterOrDigit(c)) {
				newFileName = newFileName + c;
			} else if (Character.isWhitespace(c)) {
				newFileName.trim();
				newFileName = newFileName + c;
			} else {
				String str = new String("" + c);
				if (str.equalsIgnoreCase("-") || str.equalsIgnoreCase("+")) {
					newFileName = newFileName + str;
				} else if (str.equalsIgnoreCase("#")) {
					newFileName = newFileName + "loan";
				} else if (str.equalsIgnoreCase(File.pathSeparator)) {
					newFileName = newFileName + "or";
				}
			}
		}

		String fileName = directory + File.separator + newFileName + ".3.xml";
		
		File file = new File(fileName);
		
		if(file.exists()) {
			file.delete();
		}

		Document document = null;
		try {
			document = XMLWriter.createNewXMLDocument(fileName);
		} catch (Exception ex) {

			String[] str = new String[4];
			str[0] = "Collection " + collectionName + " could not be saved. ";
			str[1] = ex.getMessage();
			str[2] = "";
			str[3] = "";

			SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(str);
			return false;
		}

		try {
			SS3DBProfile prf = db.getProfile();
			Element root = document.createElement("SIGNSTREAM-DATABASE");
			root.setAttribute("SIGNSTREAM-VERSION",
					"" + prf.getSignStreamVersion());
			root.setAttribute("SOURCE", "" + prf.getSource());
			root.setAttribute("DATABASE-VERSION", "" + prf.getDatabaseVersion());
			root.setAttribute("EXCERPT", db.getExcerpt());
			root.setAttribute("EXCERPT_NUMBER", db.getNumber());

			root.setAttribute("S", "" + db.getStartTimeInfo().getMovieTime());
			root.setAttribute("E", "" + db.getEndTimeInfo().getMovieTime());
			root.setAttribute("PARTICIPANT", db.getParticipantName());

			root.appendChild(createUtilsElement(db, document));
			root.appendChild(createDistributorElement(prf, document));
			root.appendChild(createReferencesElement(db, document));
			// root.appendChild(createAuthorsElement(db, document));
			root.appendChild(createMediaFilesElement(db, document));
			root.appendChild(createMediaFilesOnScreenLocationElement(db,
					document));
			root.appendChild(createCodingSchemeElement(db, document));
			root.appendChild(createDatabaseElement(db, document));

			boolean savedCoolection = XMLWriter.saveDocument(root, fileName);
			if (!savedCoolection) {

				String[] str = new String[4];
				str[0] = "Collection " + collectionName
						+ " could not be saved. ";
				str[1] = "Please manually delete the directory and/or file";
				str[2] = "with the collection name and try again.";
				str[3] = "";

				SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(
						str);
				return false;
			}

			// showCollectionSavedMessage();
			return true;
		} catch (Exception ex) {
			SS3SignStreamApplication.logger.log(Level.INFO, "Collection ["
					+ dbDirectory
					+ "] had NOT been saved. Error in Macro Unit [" + fileName
					+ "]" + ex.getStackTrace());

			String[] str = new String[4];
			str[0] = "Collection " + collectionName + " could not be saved. ";
			str[1] = ex.getMessage();
			str[2] = "";
			str[3] = "";

			SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(str);
			return false;
		}
	}

	private JPopupMenu collectionSavedPopup = null;
	private JLabel label = new JLabel("Collection had been saved.");

	private void showCollectionSavedMessage() {

		if (collectionSavedPopup != null) {
			UserInterfaceUtil.setLabelLNF(label);
			collectionSavedPopup = new JPopupMenu();

			final JButton okBtn = new JButton("OK");

			JPanel msgPanel = new JPanel();
			GridBagConstraints gbc = new GridBagConstraints();

			msgPanel.setLayout(new GridBagLayout());
			gbc.anchor = GridBagConstraints.WEST;

			gbc.gridwidth = 3;
			gbc.gridx = 0;
			gbc.gridy = 0;
			msgPanel.add(label, gbc);

			gbc.gridy = 1;
			gbc.gridx = 0;
			msgPanel.add(new JLabel(" "), gbc);

			gbc.gridwidth = 1;
			gbc.anchor = GridBagConstraints.EAST;
			gbc.gridy = 3;
			gbc.gridx = 0;
			msgPanel.add(new JLabel(" "), gbc);
			gbc.gridx = 2;
			msgPanel.add(okBtn, gbc);

			okBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					collectionSavedPopup.setVisible(false);
				}
			});

			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					okBtn.requestFocus();
				}
			});

			Dimension d = new Dimension();
			int width = msgPanel.getPreferredSize().width + 30;
			int height = msgPanel.getPreferredSize().height + 20;

			d.setSize(width, height);

			collectionSavedPopup.add(msgPanel);
			collectionSavedPopup.setPopupSize(d);
		}

		JPanel panel = SS3Singleton.getSignStreamApplication()
				.getSignStreamSegmentPanel().getUtteranceView();
		// popup.show(field.getSignStreamSegmentPanel().getUtteranceView(), 250,
		// 250);
		if (panel != null) {
			panel = SS3Singleton.getSignStreamApplication()
					.getSignStreamSegmentPanel().getDatabaseView();
		}
		collectionSavedPopup.show(panel, 250, 250);
		collectionSavedPopup.setVisible(true);
	}
}