package edu.bu.signstream.common.db.parser;

import java.awt.Color;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Stack;
import java.util.logging.Level;

import edu.bu.signstream.common.filesLocators.MediaFilesLocator;
import edu.bu.signstream.common.util.vo.CodingSchemaField;
import edu.bu.signstream.common.util.vo.CodingSchemaFieldType;
import edu.bu.signstream.common.util.vo.CodingSchemaValue;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.MediaFile;
import edu.bu.signstream.common.util.vo.Participant;
import edu.bu.signstream.common.util.vo.Profile;
import edu.bu.signstream.common.util.vo.Segment;
import edu.bu.signstream.common.util.vo.SignStreamDocument;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.Utterance;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.FrameNumberToMediaTimeConvertor;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SS2Parser extends Parser {
	private HashMap fields = new HashMap();
	private File file;

	/**
	 * maintains a stack of file markers in the {@link DataInputStream} passed
	 * to {@link // #readFromStream}, for cases of multiply nested
	 * {@link // #REFERENCED_CHUNK}s
	 */
	private static Stack fileMarkers = new Stack();

	private int utteranceOffset = 0;

	public SS2Parser() {
		try {
			// File file = new
			// File("/Users/irene/Desktop/ben_story_439_small_0.mov");
			// convertor = new FrameToTimeConvertor(file);
		} catch (Exception ex) {
			SS3SignStreamApplication.logger.log(Level.INFO, ex.getMessage());
		}
	}

	@Override
	public SignStreamDocument loadSSDocument(String fileName)
			throws IllegalArgumentException {
		SignStreamDocument ssDoc = null;
		try {
			this.file = new File(fileName);
			ssDoc = parseDBFileData(file);

			refactorSegmentValues(ssDoc);

		} catch (IllegalArgumentException ilEx) {
			SS3SignStreamApplication.logger.log(Level.INFO, ilEx.getMessage());
			throw ilEx;
		} catch (Exception ex) {
			SS3SignStreamApplication.logger.log(Level.INFO, ex.getMessage());
		}
		return ssDoc;
	}

	public SignStreamDocument parseDBFileData(File file)
			throws IllegalArgumentException {
		
		this.file = file;

		DataInputStream dis = null;
		ByteArrayInputStream bais = null;
		int filePointer = 0;

		HashMap codingScheme = null;
		HashMap participants = null;
		HashMap mediaFiles = null;
		HashMap utterances = null;
		Profile profile = null;

		try {
			InputStream is = new FileInputStream(file);
			byte[] ramFile = new byte[(int) file.length()];
			is.read(ramFile);
			is.close();

			bais = new ByteArrayInputStream(ramFile);
			dis = new DataInputStream(bais);
			String signature = readSignature(dis, filePointer);

			if (!signature.equals("sASL")) {
				throw new IllegalArgumentException("File not in valid format");
			}

			int version = readU16(dis, filePointer);
			int fileSize = readU32(dis, filePointer);
			int nextSegmentID = readU32(dis, filePointer);
			int indexOffset = readU32(dis, filePointer);
			int indexSize = readU32(dis, filePointer);
			int fieldSpecOffset = readU32(dis, filePointer);
			int fieldSpecSize = readU32(dis, filePointer);
			utteranceOffset = readU32(dis, filePointer);
			int utteranceSize = readU32(dis, filePointer);
			int fontListOffset = readU32(dis, filePointer);
			int fontListSize = readU32(dis, filePointer);
			int openSegmentsOffset = readU32(dis, filePointer);
			int openSegmentsSize = readU32(dis, filePointer);
			int participantsOffset = readU32(dis, filePointer);
			int participantsSize = readU32(dis, filePointer);
			int dbProfileOffset = readU32(dis, filePointer);
			int dbProfileSize = readU32(dis, filePointer);
			int windowLocationOffset = readU32(dis, filePointer);
			int windowLocationSize = readU32(dis, filePointer);
			int mediaRefsOffset = readU32(dis, filePointer);
			int mediaRefsSize = readU32(dis, filePointer);

			// // READ THE CODING SCHEME
			jumpToFilePosition(dis, fieldSpecOffset, filePointer);
			codingScheme = parseFieldSpec(dis, filePointer);
			// System.out.println("CODING SCHEME");

			// // READ PARTICIPANT DATA ++
			jumpToFilePosition(dis, participantsOffset, filePointer);
			participants = parseParticipants(dis, filePointer);
			// System.out.println("PARTICIPANT DATA");

			// // READ "DB Profile" META-DATA ++
			jumpToFilePosition(dis, dbProfileOffset, filePointer);
			profile = parseDBProfile(dis, filePointer);
			// System.out.println("Profile");

			// / READ MEDIA REF DATA ++
			jumpToFilePosition(dis, mediaRefsOffset, filePointer);
			mediaFiles = parseMediaRefs(dis, filePointer);

			// System.out.println("MEDIA REF DATA");

			// / READ UTTERANCES ++
			// System.err.println("READ UTTERANCES: ");
			jumpToFilePosition(dis, utteranceOffset, filePointer);
			utterances = parseUtterances(dis, filePointer);
			// System.out.println("UTTERANCES");

			// removeUnusedMediaRefs();

			dis.close();
			bais.close();
			// } catch(EOFException ex) {
			// ex.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.INFO, ioe.getMessage());

			try {
				dis.close();
				bais.close();
			} catch (IOException ioe2) {
				SS3SignStreamApplication.logger.log(Level.INFO,
						ioe2.getMessage());
			}
			throw new IllegalArgumentException("Can't read file");
		}
		SignStreamDocument ssDocument = new SignStreamDocument(profile,
				participants, codingScheme, mediaFiles, utterances,
				file.getName());
		return ssDocument;
	}

	private HashMap parseParticipants(DataInputStream dis, int filePointer)
			throws IOException {
		int version = readU16(dis, filePointer);
		int languageListVersion = readU16(dis, filePointer);
		int numLanguages = readU16(dis, filePointer);
		HashMap participants = new HashMap();

		String[] languages = new String[numLanguages];
		for (int i = 0; i < numLanguages; i++) {
			String name = readPascalString(dis, filePointer);
			languages[i] = name;
		}

		int nextParticipantID = readU16(dis, filePointer); // discard
		int numParticipants = readU16(dis, filePointer);

		int offset = 0; // for when participant IDs are not sequential from 0
		for (int i = 0; i < numParticipants; i++) {
			int participantScrapVersion = readU16(dis, filePointer);

			Participant participant = new Participant();
			int participantId = readU16(dis, filePointer);
			participant.setParticipantId(participantId);
			boolean everPaned = readBoolean(dis, filePointer); // discard
			participant.setGender(readU16(dis, filePointer) == 1 ? "female"
					: "male");
			participant.setLanguage(languages[readU16(dis, filePointer)]);
			participant.setLabel(readPascalString(dis, filePointer));
			participant.setName(readPascalString(dis, filePointer));
			participant.setAge(readPascalString(dis, filePointer));
			participant.setComments(readLongPascalString(dis, filePointer));
			participant.setParents(readLongPascalString(dis, filePointer));
			ArrayList background = new ArrayList();
			background.add(readLongPascalString(dis, filePointer));
			participant.setBackground(background);

			participants.put("" + participantId, participant);
		}
		return participants;
	}

	private Profile parseDBProfile(DataInputStream dis, int filePointer)
			throws IOException {
		Profile profile = new Profile();

		profile.setSignStreamVersion("" + readU16(dis, filePointer));

		boolean defaultFilename = readBoolean(dis, filePointer);
		// profile.setDefaultFilename(defaultFilename);

		// if (!defaultFilename)
		// profile.setFilename(readPascalString(dis, filePointer));

		// profile.setVersionText(readPascalString(dis, filePointer));
		profile.setDatabaseVersion(readPascalString(dis, filePointer));
		
		//try {
			profile.setDistributor(readLongPascalString(dis, filePointer));
		//} catch(Exception ex) {
		//	String msg = ex.getMessage();
		//	System.out.println(""+msg);
		//}
 
		profile.setCitation(readLongPascalString(dis, filePointer));
		profile.setNotes(readLongPascalString(dis, filePointer));
		// System.err.println(profile.toString());
		return profile;
	}

	private HashMap parseMediaRefs(DataInputStream dis, int filePointer)
			throws IOException {
		HashMap mediaRefs = new HashMap();
		int version = readU16(dis, filePointer);
		int nextID = readU16(dis, filePointer);
		int numRefs = readU16(dis, filePointer);

		for (int i = 0; i < numRefs; i++) {
			int mediaScrapVersion = readU16(dis, filePointer);
			int id = readU16(dis, filePointer);
			int numUsers = readU16(dis, filePointer);
			int pid = readU16(dis, filePointer);
			boolean gotStats = readBoolean(dis, filePointer);

			if (gotStats) {
				String path = readPascalString(dis, filePointer);
				
				//
				//

				String[] result = path.split(":");
				System.out.println("1 " + path);
				System.out.println("2 " + file.getParent());
				System.out.println("3 " + File.separator);
				System.out.println("4 " + result[result.length - 1]);
				String fullPath = ""; // + File.separator; // +
										// result[result.length-1];

				for (int k = 1; k < result.length; k++) {
					fullPath += File.separator;
					fullPath += result[k];
				}

				System.out.println("fullPath = " + fullPath);

				int created = readU32(dis, filePointer);
				// SHOULD figure out format or ignore
				int modified = readU32(dis, filePointer);
				int fileBytes = readU32(dis, filePointer);
				int trackCount = readU16(dis, filePointer);
				int timeScale = readU32(dis, filePointer);

				int duration = readU32(dis, filePointer);
				int prefWidth = readU16(dis, filePointer);
				int prefHeight = readU16(dis, filePointer);

				String fileParent = file == null ? "" : file.getParent();

				MediaFile mediaFile = new MediaFile("" + id, path, fullPath,
						file.getParent());

				mediaRefs.put("" + id, mediaFile);
				// if(frameRate < 0) {
				// String fileName = "_974_small_0.mov";
				// frameRate = MovieUtil.getMovieFrameRate(fileName);
				// }
			}
		}
		return mediaRefs;
	}

	private HashMap parseUtterances(DataInputStream dis, int filePointer)
			throws IOException {
		HashMap utterances = new HashMap();
		int numUtterances = readU16(dis, filePointer);

		for (int u = 0; u < numUtterances; u++) {
			jumpToFilePosition(dis, utteranceOffset + (u * 38), filePointer);
			readU16(dis, filePointer);

			Utterance utterance = new Utterance();

			int fileOffset = readU32(dis, filePointer);
			int size = readU32(dis, filePointer); // discard
			int id = readU32(dis, filePointer);
			String excerpt = parseUtteranceExcerpt(dis, filePointer);
			utterances.put("" + id, utterance);

			jumpToFilePosition(dis, fileOffset, filePointer);

			int utteranceVersion = readU16(dis, filePointer);
			int id2 = readU32(dis, filePointer);

			boolean overrideFieldOrder = readBoolean(dis, filePointer);
			boolean overrideFieldVisibility = readBoolean(dis, filePointer);
			boolean overrideFieldColor = readBoolean(dis, filePointer);

			readWindowStateInfo(dis, filePointer); // discard window state info
			String note = readNotes(dis, filePointer);
			ArrayList notes = new ArrayList();
			notes.add(note);

			HashMap mediaRefs = parseUtteranceMediaRefRefs(dis, filePointer);

			TimeInfo[] timeInfos = readClipTimes(dis, false, filePointer);
			TimeInfo startTimeInfo = timeInfos[0];
			TimeInfo endTimeInfo = timeInfos[1];

			// int currentUtteranceDuration = (int) (endTime - startTime);
			readWindowStateInfo(dis, filePointer); // discard
			int numParticipants = readU16(dis, filePointer);

			ArrayList segments = new ArrayList();
			for (int i = 0; i < numParticipants; i++) {
				Segment segment = parseParticipantSegment(dis, startTimeInfo,
						endTimeInfo, filePointer);
				// segment.removeEmptyStart();
				segments.add(segment);
			}
			id = id + 1;
			utterance.setId("" + id);
			utterance.setExcerpt(excerpt);
			utterance.setStartTimeInfo(startTimeInfo);
			utterance.setEndTimeInfo(endTimeInfo);
			utterance.setNotes(notes);
			utterance.setMediaFiles(mediaRefs);
			utterance.setSegments(segments);

			// System.err.println(utterance.toString());
			restoreLastFilePosition(dis, filePointer);
		}

		return utterances;
	}

	private void restoreLastFilePosition(DataInputStream dis, int filePointer)
			throws IOException {
		filePointer = ((Integer) fileMarkers.pop()).intValue();
		dis.reset();
		dis.skipBytes(filePointer);
	}

	/** Corresponds to a ParticipantPane in the DB file */
	private Segment parseParticipantSegment(DataInputStream dis,
			TimeInfo utteranceStartTimeInfo, TimeInfo utteranceEndTimeInfo,
			// int currentUtteranceDuration,
			int filePointer) throws IOException {
		// ArrayList tracks = new ArrayList();
		HashMap<String, Track> tracks = new HashMap<String, Track>();
		Segment segment = new Segment();
		int GROUP_FIXED = 1;
		int GROUP_LOWER = 2;
		int GROUP_FIXED_ID_OFFSET = 10000;
		int GROUP_LOWER_ID_OFFSET = 20000;

		int trackNumber = 0;
		// used to set track's ordering to match SS2 display
		int participantID = readU16(dis, filePointer);
		boolean primary = readBoolean(dis, filePointer);
		boolean visible = readBoolean(dis, filePointer); // discard

		if (visible) {
			int height = readU16(dis, filePointer); // discard
			int upperHeight = readU16(dis, filePointer); // discard
			int fixedHeight = readU16(dis, filePointer); // discard
			int lowerHeight = readU16(dis, filePointer); // discard
		}

		segment.setParticipantId("" + participantID);
		segment.setPrimaryParticipant(primary);

		int numTracks = readU16(dis, filePointer);

		for (int i = 0, n = numTracks; i < n; i++) {
			Track track = new Track();
			int version = readU16(dis, filePointer);
			int group = readU16(dis, filePointer);
			int fieldID = readU16(dis, filePointer);
			track.setFieldId("" + fieldID);

			int order = trackNumber++;

			if (group == GROUP_FIXED) {
				fieldID += GROUP_FIXED_ID_OFFSET;
				order += GROUP_FIXED_ID_OFFSET;
			} else if (group == GROUP_LOWER) {
				fieldID += GROUP_LOWER_ID_OFFSET;
				order += GROUP_LOWER_ID_OFFSET;
			}

			visible = readBoolean(dis, filePointer);
			int dataLength = readU32(dis, filePointer);

			CodingSchemaField field = (CodingSchemaField) fields.get(""
					+ fieldID);

			String type = field.getCodingSchemaFieldType().getType();
			track.setFieldType(field.getCodingSchemaFieldType());

			String name = field.getName();
			if (dataLength > 0) {

				if (type.equalsIgnoreCase(Constants.TYPE_NON_MANUAL_OLD)
						|| type.equalsIgnoreCase(Constants.TYPE_NON_MANUAL)) {
					track.addTrackValues(readNonManualTrackData(dis, field,
							utteranceStartTimeInfo, filePointer));

					CodingSchemaFieldType nType = new CodingSchemaFieldType(
							Constants.TYPE_NON_MANUAL, "non manual",
							"non manual field");

					track.setFieldType(nType);
				} else if (type.equalsIgnoreCase(Constants.TYPE_GLOSS_OLD)
						|| type.equalsIgnoreCase(Constants.TYPE_GLOSS)) {
					ArrayList values = readGlossTrackData(dis, fieldID,
							utteranceStartTimeInfo, filePointer);
					track.addTrackValues(values);
					track.setFieldId("" + fieldID);

					CodingSchemaFieldType nType = new CodingSchemaFieldType(
							Constants.TYPE_GLOSS, "gloss", "gloss field field");

					track.setFieldType(nType);
				} else if (type.equalsIgnoreCase(Constants.TYPE_FREE_TEXT_OLD)
						|| type.equalsIgnoreCase(Constants.TYPE_FREE_TEXT)) {
					int version1 = readU16(dis, filePointer);
					String text = readLongPascalString(dis, filePointer);

					TrackValue value = new TrackValue();
					value.setTextValue(text);
					value.setStartTimeInfo(utteranceStartTimeInfo);
					value.setEndTimeInfo(utteranceEndTimeInfo);
					track.addTrackValue(value);

					track.setFieldId("" + fieldID);

					CodingSchemaFieldType nType = new CodingSchemaFieldType(
							Constants.TYPE_FREE_TEXT, "free text",
							"free text field");

					track.setFieldType(nType);
					// break;
				} else {
					// default :
					SS3SignStreamApplication.logger.log(Level.INFO,
							"Track doesn't match any known type. Type = "
									+ type);
				}
			}
			tracks.put(track.getFieldId(), track);
		}
		segment.setTracks(tracks);
		return segment;
	}

	private ArrayList readGlossTrackData(DataInputStream dis, int fieldID,
			TimeInfo utteranceStartTimeInfo, int filePointer)
			throws IOException {

		ArrayList trackValues = new ArrayList();
		int version = readU16(dis, filePointer);
		int numAnnotations = readU16(dis, filePointer);

		for (int i = 0; i < numAnnotations; i++) {
			Object value = readLongPascalString(dis, filePointer);

			if (value != null && ((String) value).length() > 0) {
				String styleScrap = readLongPascalString(dis, filePointer);
				// discard
				TrackValue trackValue = new TrackValue();

				boolean hold = false;
				TimeInfo[] times = readClipTimes(dis, false, filePointer);

				TimeInfo startTimeInfo = times[0]; // - utteranceStartTime;
				TimeInfo endTimeInfo = times[1]; // - utteranceStartTime;
				// int startFrame = (int) times[2];
				// int endFrame = (int) times[3];

				// trackValue.setStartFrame(startFrame);
				// trackValue.setEndFrame(endFrame);

				if (endTimeInfo.getFrameNumber() >= 0)
					hold = readBoolean(dis, filePointer);

				if (startTimeInfo.getFrameNumber() >= 0
						|| endTimeInfo.getFrameNumber() >= 0) {
					if (startTimeInfo.getFrameNumber() < 0)
						startTimeInfo.setFrameNumber(endTimeInfo
								.getFrameNumber());
					else if (endTimeInfo.getFrameNumber() < 0)
						endTimeInfo.setFrameNumber(startTimeInfo
								.getFrameNumber());
					if (hold) {
						// trackValue.setTextValue("HOLD");
						trackValue.setValueId(Constants.HOLD_CODE);

						// HACK: set start time to end time of last annotation
						if (trackValues.size() > 0) {
							TrackValue trValue = (TrackValue) trackValues
									.get(trackValues.size() - 1);
							startTimeInfo.setFrameNumber(trValue
									.getStartTimeInfo().getFrameNumber());
						}
					} else {
						if (startTimeInfo.getFrameNumber() == endTimeInfo
								.getFrameNumber()) {
							endTimeInfo.setFrameNumber(endTimeInfo
									.getFrameNumber() + 1);
							// assume 30 fps
						}
					}

					// currentUtteranceDuration =
					// Math.max(currentUtteranceDuration, endTime);

					trackValue.setEndTimeInfo(endTimeInfo);
					trackValue.setStartTimeInfo(startTimeInfo);

					String sValue = (String) value;

					if (!hold) {
						trackValue.setTextValue(sValue);
					}

					trackValues.add(trackValue);
				}
			}
		}
		return trackValues;
	}

	// change start time to start frame number
	private ArrayList readNonManualTrackData(DataInputStream dis,
			CodingSchemaField field, TimeInfo utteranceStartTimeInfo,
			int filePointer) throws IOException {
		ArrayList trackValues = new ArrayList();
		int version = readU16(dis, filePointer);

		// many annotations in the DB file are empty placeholders
		int numAnnotations = readU16(dis, filePointer);
		for (int i = 0; i < numAnnotations; i++) {
			TimeInfo startTimeInfo = readTimePoint(dis, filePointer);
			TimeInfo endTimeInfo = readTimePoint(dis, filePointer);

			// int startFrame = readU32(dis, filePointer);
			// int startTime =
			// (int) (readTimePoint(dis, filePointer, startFrame)
			// - utteranceStartTime);
			// int endFrame = readU32(dis, filePointer);
			// int endTime =
			// (int) (readTimePoint(dis, filePointer, endFrame)
			// - utteranceStartTime);
			int valueID = readU16(dis, filePointer);

			TrackValue value = new TrackValue();
			value.setValueId(valueID);
			// CodingSchemeField field = (CodingSchemeField)
			// fields.get(""+valueID);

			if (startTimeInfo.getFrameNumber() == endTimeInfo.getFrameNumber()) {
				endTimeInfo.setFrameNumber(endTimeInfo.getFrameNumber() + 1);
			}

			value.setUtteranceTimeInfo(utteranceStartTimeInfo);
			value.setStartTimeInfo(startTimeInfo);
			value.setEndTimeInfo(endTimeInfo);

			// if (startTime == endTime) {
			// endTime += 33; // assume a frame rate of 30fps
			// }
			// //currentUtteranceDuration = Math.max(currentUtteranceDuration,
			// endTime);
			//
			// value.setStartTime(startTime);
			// value.setEndTime(endTime);
			// value.setStartFrame(startFrame);
			// value.setEndFrame(endFrame);

			trackValues.add(value);

			int legacyOnsetID = field.getLegacyOnsetID();
			int legacyOffsetID = field.getLegacyOffsetID();

			if (valueID == legacyOnsetID) {
				value.setValueId(Constants.ONSET_CODE);
			} else if (valueID == legacyOffsetID) {
				value.setValueId(Constants.OFFSET_CODE);
			}
		}
		return trackValues;
	}

	private HashMap parseFieldSpec(DataInputStream dis, int filePointer)
			throws IOException {
		HashMap fields = new HashMap();

		int version = readU16(dis, filePointer);
		int numFields = readU16(dis, filePointer);
		int nextUpperID = readU16(dis, filePointer); // discard
		int nextFixedID = readU16(dis, filePointer); // discard
		int nextLowerID = readU16(dis, filePointer); // discard

		for (int i = 0; i < numFields; i++) {
			int fieldChunkVersion = readU16(dis, filePointer);
			int group = readU16(dis, filePointer);
			String type = "" + readU16(dis, filePointer);
			int id = readU16(dis, filePointer);

			boolean predefined = readBoolean(dis, filePointer); // discard??
			boolean visible = readBoolean(dis, filePointer);
			int red = readU16(dis, filePointer) & 0x00FF;
			int green = readU16(dis, filePointer) & 0x00FF;
			int blue = readU16(dis, filePointer) & 0x00FF;
			Color color = new Color(red, green, blue);

			String prefix = null;
			String name = readPascalString(dis, filePointer);
			String label = readPascalString(dis, filePointer);
			int legacyOnsetID = -1, legacyOffsetID = -1;
			ArrayList csValues = new ArrayList();

			switch (group) {
			case Constants.GROUP_FIXED:
				id += Constants.GROUP_FIXED_ID_OFFSET;
				break;
			case Constants.GROUP_LOWER:
				id += Constants.GROUP_LOWER_ID_OFFSET;
				break;
			}

			CodingSchemaFieldType fieldType = null;
			int numValues = 0;
			if (type.equalsIgnoreCase(Constants.TYPE_NON_MANUAL_OLD)) {
				fieldType = new CodingSchemaFieldType(
						Constants.TYPE_NON_MANUAL, "non manual",
						"non-manual field");
				prefix = readPascalString(dis, filePointer);
				numValues = readU16(dis, filePointer);
				int nextValueID = readU16(dis, filePointer); // discard
			} else if (type.equalsIgnoreCase(Constants.TYPE_FREE_TEXT_OLD)) {
				fieldType = new CodingSchemaFieldType(Constants.TYPE_FREE_TEXT,
						"free text", "free text field");
			} else if (type.equalsIgnoreCase(Constants.TYPE_GLOSS_OLD)) {
				fieldType = new CodingSchemaFieldType(Constants.TYPE_GLOSS,
						"gloss", "gloss field");
			} else if (type.equalsIgnoreCase(Constants.TYPE_HAND_SHAPE_OLD)) {
				fieldType = new CodingSchemaFieldType(
						Constants.TYPE_RIGHT_HAND_SHAPE,
						// .TYPE_DOM_HAND_SHAPE,
						"hand shapes", "hand shapes field");
			} else if (type.equalsIgnoreCase(Constants.TYPE_POS_TAG_OLD)) {
				fieldType = new CodingSchemaFieldType(
						Constants.TYPE_DOM_POS_TAG, "part of speech",
						"part of speech field");
			}

			for (int v = 0; v < numValues; v++) {
				int valueID = readU16(dis, filePointer);
				String valueName = readPascalString(dis, filePointer);
				String valueLabel = readPascalString(dis, filePointer);

				if (valueLabel.equals("s")) {
					legacyOnsetID = valueID;
				} else if (valueLabel.equals("e")) {
					legacyOffsetID = valueID;
				} else {
					CodingSchemaValue val = new CodingSchemaValue();
					val.setValueId(valueID);
					val.setName(valueName);
					val.setLabel(valueLabel);
					csValues.add(val);
				}
			}

			CodingSchemaField csField = new CodingSchemaField();

			csField.setFieldId(id);
			csField.setName(name);
			csField.setLabel(label);
			csField.setCodingSchemaFieldType(fieldType);

			if (prefix != null && !prefix.equalsIgnoreCase(""))
				csField.setPrefix(prefix);

			if (type != Constants.TYPE_NON_MANUAL_OLD) {
				csField.setConstraint("free-text");
				if (type == Constants.TYPE_FREE_TEXT_OLD) {
					csField.setTimeAlignment("none");
				}
			}

			csField.setColor(color);
			csField.setCodingSchemeValues(csValues);

			if (legacyOnsetID > -1) {
				csField.setLegacyOnsetID(legacyOnsetID);
			}
			if (legacyOffsetID > -1) {
				csField.setLegacyOffsetID(legacyOffsetID);
			}

			fields.put("" + id, csField);
		}
		this.fields = fields;
		return fields;
	}

	private HashMap parseUtteranceMediaRefRefs(DataInputStream dis,
			int filePointer) throws IOException {
		HashMap mediaRefs = new HashMap();

		int mediaDispatchScrapVersion = readU16(dis, filePointer);
		readWindowStateInfo(dis, filePointer); // discard
		boolean displayMD = readBoolean(dis, filePointer); // discard
		int numVideoRefs = readU16(dis, filePointer);
		int numAudioRefs = readU16(dis, filePointer);
		int numRefs = numVideoRefs + numAudioRefs;

		for (int i = 0; i < numRefs; i++) {
			int mRefVersion = readU16(dis, filePointer);

			int mediaType = readU16(dis, filePointer);
			boolean selected = readBoolean(dis, filePointer);
			int mediaID = -1;
			boolean visible = true;
			if (selected) {
				mediaID = readU16(dis, filePointer);
				boolean volumeSet = readBoolean(dis, filePointer);
				if (volumeSet) {
					int volume = readU16(dis, filePointer); // discard
				}
				readWindowStateInfo(dis, filePointer); // discard
				visible = readBoolean(dis, filePointer);

				String filePath = file == null ? "" : file.getParent();

				MediaFile mFile = new MediaFile("" + mediaID, "", "", filePath);
				// System.err.println("mediaID = " + mFile);
				mediaRefs.put("" + mediaID, mFile);
			}
		}
		return mediaRefs;
	}

	private TimeInfo[] readClipTimes(DataInputStream dis,
			boolean unconditional, int filePointer) throws IOException {

		TimeInfo[] timeInfos = new TimeInfo[2];
		timeInfos[0] = new TimeInfo();
		timeInfos[1] = new TimeInfo();

		if (unconditional || readBoolean(dis, filePointer)) {
			timeInfos[0] = readTimePoint(dis, filePointer);
		}
		if (unconditional || readBoolean(dis, filePointer)) {
			timeInfos[1] = readTimePoint(dis, filePointer);
		}
		return timeInfos;
	}

	private int readFrameNumber(DataInputStream dis, int filePointer)
			throws IOException {
		return readU32(dis, filePointer);
	}

	private TimeInfo readTimePoint(DataInputStream dis, int filePointer)
			throws IOException {
		int frame = readU32(dis, filePointer);
		long value = readU64(dis, filePointer);
		int scale = readU32(dis, filePointer);
		int base = readU32(dis, filePointer);

		TimeInfo info = new TimeInfo();
		info.setFrameNumber(frame);
		info.setMovieTime(frame);
		info.setTimeBase(base);
		info.setTimeScale(scale);

		// System.out.println("TIME: FRAME = " + frame + ", value = " + value +
		// ", scale = " + scale + ", base = " + base);
		//

		// long time1 = frame * 1000 / scale;

		/*
		 * 
		 * QuickTime doesn't deal in frames. It deals in movie time. A QuickTime
		 * Movie object (as well as particular tracks and so forth) has a
		 * duration and a timescale, both integers. So, a movie may have a
		 * duration of 1500 and a timescale of 600, which means it's 2.5 seconds
		 * long. But any timescale is possible, eg. people often use a timescale
		 * of 2997 and frame durations of 100 to represent NTSC drop frame (or a
		 * timescale of 30001 and a frame duration of 1001).
		 * 
		 * Don't get too excited at the mention of frame durations. QuickTime
		 * movies can have a variable frame rate, so they don't directly store a
		 * frame duration, nor do they deal in frame numbers for the same
		 * reason. To jump to a particular frame in a movie, e.g. 0.5sec from
		 * the start, you'd have to set the time to 300 (assuming a timescale of
		 * 600), rather than jumping to frame 15 (assuming 30 fps).
		 * 
		 * I suspect what you want is something like the following therefore:
		 * 
		 * movie time = frame number / frame rate * movie timescale
		 */

		return info;
		// return convertor.convertFrameNumberToTime((int)value); //value *1000
		// /2997;
	}

	private String parseUtteranceExcerpt(DataInputStream dis, int filePointer)
			throws IOException {
		// SS2 code comments say excerpt length is 1 byte, but we need to
		// right-shift read length value by one byte for unknown reasons
		int excerptLength = readU16(dis, filePointer) >> 8;
		String excerpt = readString24Byte(dis, filePointer);

		if (excerptLength > 0 && excerptLength <= 24)
			excerpt = excerpt.substring(0, excerptLength - 1);
		else if (excerptLength == 0)
			excerpt = "";
		return excerpt;
	}

	private void readWindowStateInfo(DataInputStream dis, int filePointer)
			throws IOException {
		// FROM "Window State scrap.h": #define CURRENT_WIN_STATE_SCRAP_VERSION
		// ( 0 )
		int version = readU16(dis, filePointer);
		// Assert.check(version == 0);
		int size = readU32(dis, filePointer);
		boolean windowSized = readBoolean(dis, filePointer);
		int left, top, right, bottom;
		boolean maximized;
		if (windowSized) {
			maximized = readBoolean(dis, filePointer);
			left = readU16(dis, filePointer);
			top = readU16(dis, filePointer);
			right = readU16(dis, filePointer);
			bottom = readU16(dis, filePointer);
		}
	}

	private String readNotes(DataInputStream dis, int filePointer)
			throws IOException {
		String text = readLongPascalString(dis, filePointer);
		if (text != null && text.length() > 0) {
			String styleScrap = readLongPascalString(dis, filePointer);
			// discard
			int selStart = readU16(dis, filePointer); // discard
			int selEnd = readU16(dis, filePointer); // discard
		}
		return text;
	}

	/** Reads 2 bytes from the supplied stream, returning an unsigned int. */
	private int readU16(DataInputStream dis, int filePointer)
			throws IOException {
		filePointer += 2;
		return dis.readShort();
	}

	/** Reads 4 bytes from the supplied stream, returning an unsigned int. */
	private int readU32(DataInputStream dis, int filePointer)
			throws IOException {
		filePointer += 4;
		return dis.readInt();
	}

	/** Reads 24 ASCII characters from the supplied stream and returns a String. */
	private String readString24Byte(DataInputStream dis, int filePointer)
			throws IOException {
		byte[] buffer = new byte[24];
		dis.readFully(buffer);
		String b = new String(buffer);
		filePointer += 24;
		return b;
	}

	/** Reads 8 bytes from the supplied stream, returning an unsigned long. */
	private long readU64(DataInputStream dis, int filePointer)
			throws IOException {
		filePointer += 8;
		return dis.readLong();
	}

	/** Reads 2 bytes into a boolean value. */
	private boolean readBoolean(DataInputStream dis, int filePointer)
			throws IOException {
		filePointer += 2;
		return dis.readShort() != 0;
	}

	/**
	 * Reads a 4-character ASCII string, used as the file signature in SS2 files
	 * (and many other binary files). For SignStream 2 files, this string must
	 * be "sASL".
	 * 
	 * // @see SS2File
	 */
	private String readSignature(DataInputStream dis, int filePointer)
			throws IOException {
		filePointer += 4;
		byte[] buffer = new byte[4];
		dis.readFully(buffer);
		return new String(buffer);
	}

	private void jumpToFilePosition(DataInputStream dis, int offset,
			int filePointer) throws IOException {
		fileMarkers.push(new Integer(filePointer));
		dis.reset();
		dis.skipBytes(offset);
		filePointer = offset;
	}

	/**
	 * Reads a 1 byte length and then the corresponding number of ASCII
	 * characters, returning these as a String.
	 */
	private String readPascalString(DataInputStream dis, int filePointer)
			throws IOException {
		int length = dis.readByte();
		if (length < 0)
			throw new IOException("Read length byte of " + length);
		byte[] buffer = new byte[length];
		dis.readFully(buffer);
		filePointer += length + 1;
		return new String(buffer);
	}

	/** Same as {@link #readPascalString}, only uses a 4 byte length header. */
	private String readLongPascalString(DataInputStream dis, int filePointer)
			throws IOException {
		try {
		int length = dis.readInt();
		//try {
			byte[] buffer = new byte[length];
			dis.readFully(buffer);
			filePointer += length + 4;
			return new String(buffer);
		} catch (EOFException eofe) {
			// Log.error("readLongPascalString() read length "+length);
			throw eofe;
		}
	}

	private void convertFrameNumbersToMovieTime(SignStreamDocument ssDoc) {
		HashMap utterances = ssDoc.getUtterances();
		Set uttSet = utterances.keySet();
		Iterator uttrIt = uttSet.iterator();

		while (uttrIt.hasNext()) {
			String utteranceKey = (String) uttrIt.next();
			Utterance utterance = (Utterance) utterances.get(utteranceKey);
			HashMap uMediaFiles = utterance.getMediaFiles();

			if (uMediaFiles.size() > 0) {
				Set keys = uMediaFiles.keySet();
				Iterator iter = keys.iterator();

				MediaFilesLocator locator = SS3Singleton.getMediaFilesLocator();

				ArrayList mediaFiles = new ArrayList();

				while (iter.hasNext()) {
					Object obj1 = iter.next();
					String id = (String) obj1;
					MediaFile mFile = ssDoc.getMediaFile(id);

					File file = locator.locateMediaFile(mFile.getPath());
					mediaFiles.add(file);
				}

				FrameNumberToMediaTimeConvertor conv = new FrameNumberToMediaTimeConvertor(
						mediaFiles);

				ArrayList segments = utterance.getSegments();
				Iterator segmIt = segments.iterator();

				while (segmIt.hasNext()) {
					Segment segment = (Segment) segmIt.next();
					HashMap tracks = segment.getTracks();
					Set trackSet = tracks.keySet();
					Iterator trackIt = trackSet.iterator();

					while (trackIt.hasNext()) {
						String trackKey = (String) trackIt.next();
						Track track = (Track) tracks.get(trackKey);
						ArrayList trackValues = track.getTrackValues();
						for (int i = 0; i < trackValues.size(); i++) {
							TrackValue value = (TrackValue) trackValues.get(i);
							conv.convertFrameNumberToMovieTime(value
									.getStartTimeInfo());
							conv.convertFrameNumberToMovieTime(value
									.getEndTimeInfo());
							conv.convertFrameNumberToMovieTime(value
									.getUtteranceTimeInfo());
						}
					}
				}
			}
		}
	}

	private void refactorSegmentValues(SignStreamDocument ssDoc) {
		HashMap utterances = ssDoc.getUtterances();
		Set uttSet = utterances.keySet();
		Iterator uttrIt = uttSet.iterator();

		while (uttrIt.hasNext()) {
			String utteranceKey = (String) uttrIt.next();
			Utterance utterance = (Utterance) utterances.get(utteranceKey);
			ArrayList segments = utterance.getSegments();
			Iterator segmIt = segments.iterator();

			while (segmIt.hasNext()) {
				Segment segment = (Segment) segmIt.next();
				HashMap tracks = segment.getTracks();
				Set trackSet = tracks.keySet();
				Iterator trackIt = trackSet.iterator();

				while (trackIt.hasNext()) {
					String trackKey = (String) trackIt.next();
					Track track = (Track) tracks.get(trackKey);
					ArrayList trackValues = track.getTrackValues();

					for (int i = 0; i < trackValues.size(); i++) {
						TrackValue trackValue = (TrackValue) trackValues.get(i);
						int valueId = trackValue.getValueId();
						if (valueId == Constants.HOLD_CODE) {
							TrackValue prTrackValue = (TrackValue) trackValues
									.get(i - 1);

							int duration = prTrackValue.getEndTimeInfo()
									.getFrameNumber()
									- prTrackValue.getStartTimeInfo()
											.getFrameNumber();

							TimeInfo info = trackValue.getStartTimeInfo();

							int correctedStartFrameNumber = info
									.getFrameNumber() + duration;

							info.setFrameNumber(correctedStartFrameNumber);
							info.setMovieTime(correctedStartFrameNumber);

						} else if (valueId == Constants.ONSET_CODE
								&& i + 1 < trackValues.size()) {
							TrackValue postTrackValue = (TrackValue) trackValues
									.get(i + 1);
							trackValue.getEndTimeInfo().setFrameNumber(
									postTrackValue.getStartTimeInfo()
											.getFrameNumber());
							trackValue.getEndTimeInfo().setMovieTime(
									postTrackValue.getStartTimeInfo()
											.getMovieTime());
						} else if (valueId == Constants.OFFSET_CODE && i > 0) {
							TrackValue preTrackValue = (TrackValue) trackValues
									.get(i - 1);
							trackValue.getStartTimeInfo().setFrameNumber(
									preTrackValue.getEndTimeInfo()
											.getFrameNumber());
							trackValue.getStartTimeInfo().setMovieTime(
									preTrackValue.getEndTimeInfo()
											.getMovieTime());
						}
					}
				}
			}
		}
	}
}
