package edu.bu.signstream.common.db.parser;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.logging.Level;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.CharacterData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXParseException;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.Author;
import edu.bu.signstream.common.util.vo.ss3.Reference;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldType;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.ValueCollection;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.ValueCollections;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeItem;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapePalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeTemplate;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeTemplates;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Selection;
import edu.bu.signstream.common.util.vo.ss3.db.SS3StatementField;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaLibrary;
import edu.bu.signstream.common.util.vo.ss3.note.SS3Note;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.SerchableDirectory;

public class SS3Parser {

	public SS3CodingScheme parseSS3CodingScheme(String fileName) {
		SS3CodingScheme cScheme = new SS3CodingScheme();

		try {
			DocumentBuilder builder = DocumentBuilderFactory.newInstance()
					.newDocumentBuilder();
			Document doc = builder.parse(fileName);

			NodeList nodes1 = doc.getElementsByTagName("SIGNSTREAM-DATABASE");
			for (int i = 0; i < nodes1.getLength(); i++) {
				Element el1 = (Element) nodes1.item(i);
				NodeList refList = el1.getElementsByTagName("REFERENCES");

				for (int j = 0; j < refList.getLength(); j++) {
					Element el2 = (Element) refList.item(j);
					NodeList cSchRefList = el2
							.getElementsByTagName("CODING-SCHEMES");
					NodeList prtRefList = el2
							.getElementsByTagName("PARTICIPANTS");

					cScheme.setCodingSchemeRef(parseReferenses(cSchRefList));
					cScheme.setParticipantRef(parseReferenses(prtRefList));
				}

				NodeList authorList = el1.getElementsByTagName("AUTHORS");
				cScheme.setAuthors(parseAuthors(authorList));

				NodeList csList = el1.getElementsByTagName("CODING-SCHEME");

				for (int l = 0; l < csList.getLength(); l++) {
					Element el10 = (Element) csList.item(l);

					NodeList imgsList = el10.getElementsByTagName("IMGS");
					ArrayList<SS3FieldTypeImg> imgs = parseImages(imgsList);

					for (SS3FieldTypeImg img : imgs) {
						cScheme.addImage(img);
					}

					NodeList tmpltsList = el10
							.getElementsByTagName("TEMPLATES");
					Element tmpltsEl = (Element) tmpltsList.item(0);
					CSHandShapeTemplates templates = parseHandShapeTemplates(tmpltsEl);
					cScheme.setHandShapeTemplates(templates);

					NodeList fldsList = el10
							.getElementsByTagName("FIELD_TYPES");
					ArrayList<SS3FieldType> flds = parseFieldTypes(fldsList);
					for (int k = 0; k < flds.size(); k++) {
						cScheme.addFieldType(flds.get(k));
					}

					NodeList vcNode = el10
							.getElementsByTagName("VALUE_COLLECTIONS");
					ValueCollections col = parseValueCollections(vcNode);
					cScheme.setValueCollections(col);

					NodeList stmtList = el10.getElementsByTagName("FIELDS");

					Element valEl = (Element) stmtList.item(0);
					cScheme.setColor(valEl.getAttribute("DFLT_COLOR"));
					cScheme.setFields(parseFields(stmtList));

					NodeList dList = el10
							.getElementsByTagName("DEFAULT-SEGMENT");

					ArrayList<String> defaultSegmentList = new ArrayList<String>();

					Element pEl = (Element) dList.item(0);
					NodeList vNode = pEl.getElementsByTagName("F");

					for (int j = 0; j < vNode.getLength(); j++) {
						Element vEl = (Element) vNode.item(j);
						String id = vEl.getAttribute("ID");
						defaultSegmentList.add(id);
					}
					
					cScheme.setDefaultSegment(defaultSegmentList);
				}
			}
		} catch (SAXParseException e) {
			SS3SignStreamApplication.logger.setLevel(Level.SEVERE);
			SS3SignStreamApplication.logger.info(e.getMessage());
		} catch (Exception ex) {
			SS3SignStreamApplication.logger.setLevel(Level.SEVERE);
			SS3SignStreamApplication.logger.info(ex.getMessage());
		}

		return cScheme;
	}

	/**
	 * <TEMPLATES> <TEMPLATE ID="0" NAME="American Sign Language" LABEL = "ASL">
	 * <PALETTE ID="1" NAME="Letters"> 
	 * <ITEM ID="HSH_014" DESCRIPTION="A" UNMARKED="Y" FINGERSPELLED="Y"/> 
	 * <ITEM ID="HSH_020" DESCRIPTION="B" UNMARKED="Y" FINGERSPELLED="Y"/> 
	 * </PALETTE>
	 */
	private CSHandShapeTemplates parseHandShapeTemplates(Element element) {
		CSHandShapeTemplates templates = new CSHandShapeTemplates();

		NodeList templateList = element.getElementsByTagName("TEMPLATE");

		for (int n = 0; n < templateList.getLength(); n++) {
			Element templateEl = (Element) templateList.item(n);

			CSHandShapeTemplate template = new CSHandShapeTemplate();

			template.setId(templateEl.getAttribute("ID"));
			template.setName(templateEl.getAttribute("NAME"));
			template.setLabel(templateEl.getAttribute("LABEL"));

			NodeList paletteList = templateEl.getElementsByTagName("PALETTE");

			for (int p = 0; p < paletteList.getLength(); p++) {
				Element paletteEl = (Element) paletteList.item(p);

				CSHandShapePalette palette = new CSHandShapePalette();

				palette.setId(paletteEl.getAttribute("ID"));
				palette.setName(paletteEl.getAttribute("NAME"));

				NodeList itemList = paletteEl.getElementsByTagName("ITEM");

				for (int i = 0; i < itemList.getLength(); i++) {
					Element itemEl = (Element) itemList.item(i);

					CSHandShapeItem item = new CSHandShapeItem();

					item.setId(itemEl.getAttribute("ID"));
					item.setDescription(itemEl.getAttribute("DESCRIPTION"));
					item.setUnmarked(Util.isTrue(itemEl
							.getAttribute("UNMARKED")));
					item.setFingerspelled(Util.isTrue(itemEl
							.getAttribute("FINGERSPELLED")));
					palette.addHandShapeItem(item);
				}
				template.addHandShapePalette(palette);
			}
			templates.addHandShapeTemplate(template);
		}

		return templates;
	}

	// point to VALUE_COLLECTIONS
	private ValueCollections parseValueCollections(NodeList cNode) {
		ValueCollections col = new ValueCollections();

		Element pEl = (Element) cNode.item(0);
		NodeList vNode = pEl.getElementsByTagName("VALUE_COLLECTION");

		for (int i = 0; i < vNode.getLength(); i++) {
			Element vEl = (Element) vNode.item(i);
			String id = vEl.getAttribute("ID");
			ValueCollection vc = new ValueCollection(id);

			NodeList node = vEl.getElementsByTagName("VALUE");
			for (int j = 0; j < node.getLength(); j++) {
				Element e = (Element) node.item(j);
				SS3FieldValue value = new SS3FieldValue();
				value.setId(e.getAttribute("ID"));
				value.setName(e.getAttribute("NAME"));
				value.setLabel(e.getAttribute("LABEL"));
				vc.addValue(value);
			}

			col.addCollection(vc);
		}
		return col;
	}

	// point to STATEMENTS
	private ArrayList<SS3GlossWindowItem> parseStatements(NodeList nodes) {
		ArrayList<SS3GlossWindowItem> statements = new ArrayList<SS3GlossWindowItem>();

		Element element = (Element) nodes.item(0);
		NodeList tNode = element.getElementsByTagName("STATEMENT");

		for (int i = 0; i < tNode.getLength(); i++) {
			SS3GlossWindowItem statement = new SS3GlossWindowItem("", "");

			Element tEl = (Element) tNode.item(i);

			NodeList mediaOnScreenLocationList = tEl
					.getElementsByTagName("MEDIA_FILES_ON_SCREEN_LOCATION");
			statement
					.setVideoOnScreenLocations(parseMFilesOnScreenLocation(mediaOnScreenLocationList));
			statement.setId(tEl.getAttribute("ID"));
			statement.setNoteID(tEl.getAttribute("NOTE_ID"));

			statement.setLabel(tEl.getAttribute("EXCERPT"));
			TimeInfo strTimeInfo = new TimeInfo();
			strTimeInfo.setMovieTime(Integer.parseInt(tEl.getAttribute("S")));
			TimeInfo endTimeInfo = new TimeInfo();
			endTimeInfo.setMovieTime(Integer.parseInt(tEl.getAttribute("E")));
			statement.setStartTimeInfo(strTimeInfo);
			statement.setEndTimeInfo(endTimeInfo);

			NodeList pnl = tEl.getElementsByTagName("PARTICIPANT");
			ArrayList<String> participants = new ArrayList<String>();
			for (int r = 0; r < pnl.getLength(); r++) {
				Element pel = (Element) pnl.item(r);
				participants.add(pel.getAttribute("ID"));
			}
			statement.setParticipants(participants);

			NodeList cnl = tEl.getElementsByTagName("CODER");
			ArrayList<String> coders = new ArrayList<String>();
			for (int c = 0; c < cnl.getLength(); c++) {
				Element cel = (Element) cnl.item(c);
				coders.add(cel.getAttribute("ID"));
			}
			statement.setCoders(coders);

			NodeList fnls = tEl.getElementsByTagName("FIELDS");
			for (int j = 0; j < fnls.getLength(); j++) {
				Element fel = (Element) fnls.item(j);
				NodeList fnl = fel.getElementsByTagName("FIELD");

				for (int l = 0; l < fnl.getLength(); l++) {
					Element fEl = (Element) fnl.item(l);
					SS3StatementField field = new SS3StatementField();
					field.setId(fEl.getAttribute("ID"));

					NodeList ent = fEl.getElementsByTagName("ENTITIES");
					if (ent != null && ent.getLength() > 0) {
						Element eEl = (Element) ent.item(0);
						NodeList eNl = eEl.getElementsByTagName("ENTITY");
						for (int e = 0; e < eNl.getLength(); e++) {
							// TODO: copy to here the functionality from SS3ZipXmlFilesParser.parseSS3GlossWindow()
							Element entEl = (Element) eNl.item(e);
							SS3Entity ss3Entity = new SS3Entity();
							ss3Entity.setID(entEl.getAttribute("ID"));
							field.addSS3Entity(ss3Entity);
						}
					}
					statement.addField(field);
				}
			}
			statements.add(statement);
		}
		return statements;
	}

	// point to PRESENTATIONS
	private ArrayList<SS3Selection> parseSelections(NodeList nodes) {
		ArrayList<SS3Selection> selections = new ArrayList<SS3Selection>();

		Element element = (Element) nodes.item(0);
		NodeList sNode = element.getElementsByTagName("SELECTION");

		for (int i = 0; i < sNode.getLength(); i++) {
			SS3Selection selection = new SS3Selection();

			Element sEl = (Element) sNode.item(i);
			selection.setId(sEl.getAttribute("ID"));
			selection.setLabel(sEl.getAttribute("LABEL"));

			NodeList pnl = sEl.getElementsByTagName("STATEMENT");
			for (int r = 0; r < pnl.getLength(); r++) {
				Element pel = (Element) pnl.item(r);
				selection.addStatementID(pel.getAttribute("ID"));
			}
			selections.add(selection);
		}
		return selections;
	}

	private HashMap<String, SS3Field> parseFields(NodeList nodes) {
		HashMap<String, SS3Field> fields = new HashMap<String, SS3Field>();
		Element element = (Element) nodes.item(0);
		NodeList fNode = element.getElementsByTagName("FIELD");

		for (int i = 0; i < fNode.getLength(); i++) {
			SS3Field fld = new SS3Field();
			Element fEl = (Element) fNode.item(i);
			fld.setId(fEl.getAttribute("ID"));
			fld.setColor(fEl.getAttribute("COLOR"));
			fld.setLabel(fEl.getAttribute("LABEL"));
			fld.setName(fEl.getAttribute("NAME"));
			fld.setPrefix(fEl.getAttribute("PREFIX"));
			fld.setType(fEl.getAttribute("TYPE"));			
			fld.setGroup(fEl.getAttribute("GROUP"));

			NodeList valNode = fEl.getElementsByTagName("VALUE");
			for (int j = 0; j < valNode.getLength(); j++) {
				SS3FieldValue val = new SS3FieldValue();
				Element valEl = (Element) valNode.item(j);
				val.setId(valEl.getAttribute("ID"));
				val.setIid(valEl.getAttribute("IID"));
				val.setLabel(valEl.getAttribute("LABEL"));
				val.setName(valEl.getAttribute("NAME"));
				val.setType(valEl.getAttribute("TYPE"));
				fld.addValue(val);
			}
			fields.put(fld.getId(), fld);
			
		}
		return fields;
	}

	// point to IMGS
	/**
	 * <IMG ID="HSH_001" PATH1="big-1.jpg" PATH2="1.jpg"> <SELECTED_FIINGERS
	 * ID="1"> <BASE_JOINTS ID="1"> <SPREAD ID="1"/> </BASE_JOINTS>
	 * <NONBASE_JOINTS ID="1"/> </SELECTED_FIINGERS> <NONSELECTED_FINGERS
	 * ID="1"/> <THUMB_CONTACT ID="1"/> </IMG>
	 * 
	 * <IMG ID="HSH_001" PATH1="big-1.jpg" PATH2="1.jpg" slct_fingers="1"
	 * base_joints="1" spread="1" non_base_joints="1" non_slct_fingers="1"
	 * thumb_contact="1"/>
	 */
	private ArrayList<SS3FieldTypeImg> parseImages(NodeList imgsNode) {
		ArrayList<SS3FieldTypeImg> images = new ArrayList<SS3FieldTypeImg>();

		Element element = (Element) imgsNode.item(0);
		NodeList imgNode = element.getElementsByTagName("IMG");

		for (int j = 0; j < imgNode.getLength(); j++) {
			SS3FieldTypeImg img = new SS3FieldTypeImg();
			Element imgEl = (Element) imgNode.item(j);
			img.setId(imgEl.getAttribute("ID"));
			img.setPath1(imgEl.getAttribute("PATH1"));
			img.setPath2(imgEl.getAttribute("PATH2"));
			img.setSelectedFingersID(imgEl.getAttribute("slct_fingers"));
			img.setBaseJointsID(imgEl.getAttribute("base_joints"));
			img.setSpreadID(imgEl.getAttribute("spread"));
			img.setNonBaseJointsID(imgEl.getAttribute("non_base_joints"));
			img.setNonSelectedFingersID(imgEl.getAttribute("non_slct_fingers"));
			img.setThumbContactID(imgEl.getAttribute("thumb_contact"));

			images.add(img);
		}
		return images;
	}

	// nodes should referense FIELD_TYPES
	private ArrayList<SS3FieldType> parseFieldTypes(NodeList nodes) {
		ArrayList<SS3FieldType> fTypes = new ArrayList<SS3FieldType>();
		HashMap<String, SS3FieldType> hTypes = new HashMap<String, SS3FieldType>();

		Element element = (Element) nodes.item(0);
		NodeList tNode = element.getElementsByTagName("FIELD_TYPE");

		for (int i = 0; i < tNode.getLength(); i++) {
			SS3FieldType ft = new SS3FieldType();
			Element tEl = (Element) tNode.item(i);
			ft.setType(tEl.getAttribute("TYPE"));
			ft.setLabel(tEl.getAttribute("LABEL"));
			ft.setDecsription(tEl.getAttribute("DESCRIPTION"));
			ft.setRefType(tEl.getAttribute("REF"));
			ft.setDepType(tEl.getAttribute("DEP_TYPE"));

			fTypes.add(ft);
			hTypes.put(ft.getType(), ft);
		}

		for (int j = 0; j < fTypes.size(); j++) {
			SS3FieldType ft = fTypes.get(j);
			String refType = ft.getRefType();
			Object obj = hTypes.get(refType);
			if (obj != null) {
				SS3FieldType ftm = (SS3FieldType) obj;
				ftm.addDependentField(ft);
			}
		}

		return fTypes;
	}

	public SS3Database parseSS3Database(String fileName) {
		try {
			DocumentBuilder builder = DocumentBuilderFactory.newInstance()
					.newDocumentBuilder();
			Document doc = null;

			try {
				doc = builder.parse(fileName);
			} catch (java.net.MalformedURLException ex) {
				String space = "%20";
				StringBuffer sb = new StringBuffer();
				StringTokenizer tokenizer = new StringTokenizer(fileName);
				sb.append(tokenizer.nextElement());

				while (tokenizer.hasMoreTokens()) {
					sb.append(space);
					sb.append(tokenizer.nextElement());
				}
				doc = builder.parse(sb.toString());
			}

			NodeList nodes1 = doc.getElementsByTagName("SIGNSTREAM-DATABASE");
			Element excEl = (Element) nodes1.item(0);

			String sStr = excEl.getAttribute("S");
			String eStr = excEl.getAttribute("E");
			String partName = excEl.getAttribute("PARTICIPANT");

			TimeInfo startTimeInfo = new TimeInfo();
			TimeInfo endTimeInfo = new TimeInfo();
			startTimeInfo.setTimeScale(600);
			startTimeInfo.setMovieTime(Integer.parseInt(sStr));
			endTimeInfo.setTimeScale(600);
			endTimeInfo.setMovieTime(Integer.parseInt(eStr));

			SS3Database db = new SS3Database(startTimeInfo, endTimeInfo);
			db.setFileName(fileName);
			db.setExcerpt(excEl.getAttribute("EXCERPT"));
			db.setNumber(excEl.getAttribute("EXCERPT_NUMBER"));

			db.setStartTimeInfo(startTimeInfo);
			db.setEndTimeInfo(endTimeInfo);

			db.setParticipantName(partName);

			for (int i = 0; i < nodes1.getLength(); i++) {
				Element el1 = (Element) nodes1.item(i);
				NodeList refList = el1.getElementsByTagName("REFERENSES");

				for (int j = 0; j < refList.getLength(); j++) {
					Element el2 = (Element) refList.item(j);
					NodeList cSchRefList = el2
							.getElementsByTagName("CODING-SCHEMES");
					NodeList prtRefList = el2
							.getElementsByTagName("PARTICIPANTS");

					ArrayList<Reference> cShRef = parseReferenses(cSchRefList);
					ArrayList<Reference> prtRef = parseReferenses(prtRefList);

					db.setCodingSchemeRef(cShRef);
					//db.setParticipantRef(prtRef);
				}

				NodeList authorList = el1.getElementsByTagName("AUTHORS");
				//db.setAuthors(parseAuthors(authorList));

				NodeList mediaList = el1.getElementsByTagName("MEDIA-FILES");

				Element element = (Element) mediaList.item(0);
				String mds = element.getAttribute("MOVIE_DURATION");
				String pds = element.getAttribute("PACKET_DURATION");
				int movieDuration = 0, packetDuration = 0;
				
				try {
					movieDuration = Integer.parseInt(mds);
					packetDuration = Integer.parseInt(pds);
				} catch(NumberFormatException ex) {
					;
				}
				
				db.setMovieDuration(movieDuration);
				db.setPacketDuration(packetDuration);
				
				db.setMediaFiles(parseMFiles(mediaList));
				

				NodeList mediaOnScreenLocationList = el1
						.getElementsByTagName("MEDIA_FILES_ON_SCREEN_LOCATION");
				db
						.setVideoOnScreenLocations(parseMFilesOnScreenLocation(mediaOnScreenLocationList));

				SS3CodingScheme cScheme = new SS3CodingScheme();
				NodeList lCodingSchemeList = el1
						.getElementsByTagName("LOCAL-CODING-SCHEME");
				cScheme.setFields(parseFields(lCodingSchemeList));
				db.setLocalSS3CodingScheme(cScheme);

				NodeList dbList = el1.getElementsByTagName("DATABASE");

				for (int l = 0; l < dbList.getLength(); l++) {
					Element el10 = (Element) dbList.item(i);

					NodeList tracksList = el10.getElementsByTagName("TRACKS");
					ArrayList<SS3Track> tracks = parseTracks(tracksList);
					for (int k = 0; k < tracks.size(); k++) {
						db.addTrack(tracks.get(k));
					}

					NodeList stmtList = el10.getElementsByTagName("STATEMENTS");
					ArrayList<SS3GlossWindowItem> statements = parseStatements(stmtList);
					for (int n = 0; n < statements.size(); n++) {
						SS3GlossWindowItem statement = statements
								.get(n);
						db.addStatement(statement);
					}

					NodeList prsList = el10
							.getElementsByTagName("PRESENTATIONS");
					ArrayList<SS3Selection> selections = parseSelections(prsList);
					for (int m = 0; m < selections.size(); m++) {
						SS3Selection selection = selections
								.get(m);
						db.addSelection(selection);
					}

					NodeList ndsList = el10.getElementsByTagName("NOTES");

					ArrayList<SS3Note> notes = parseNotes(ndsList);
					for (int k = 0; k < notes.size(); k++) {
						SS3Note note = notes.get(k);
						db.addNote(note);
					}
				}
			}
			return db;
		} catch (Exception e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.setLevel(Level.SEVERE);
			SS3SignStreamApplication.logger.info(e.getMessage());
		}
		return null;
	}

	private ArrayList<SS3Note> parseNotes(NodeList nodes) {
		ArrayList<SS3Note> list = new ArrayList<SS3Note>();
		Element element = (Element) nodes.item(0);
		NodeList noteNode = element.getElementsByTagName("NOTE");

		for (int i = 0; i < noteNode.getLength(); i++) {
			SS3Note note = new SS3Note();
			Element line = (Element) noteNode.item(i);
			note.setID(line.getAttribute("ID"));
			note.setCoderID(line.getAttribute("CODER-ID"));
			note.setTimeCreated(line.getAttribute("TIME_CREATED"));
			note.setTimeUpdated(line.getAttribute("TIME_UPDATED"));

			NodeList sNode = line.getElementsByTagName("SUBJECT");
			for (int j = 0; j < sNode.getLength(); j++) {
				Element sEl = (Element) sNode.item(j);
				note.setSubject(getCharacterDataFromElement(sEl));
			}

			NodeList bNode = line.getElementsByTagName("BODY");
			for (int l = 0; l < bNode.getLength(); l++) {
				Element bEl = (Element) bNode.item(l);
				note.setBody(getCharacterDataFromElement(bEl));
			}

			list.add(note);
		}
		return list;
	}

	public SS3MediaLibrary parseMediaLibrary(String fileName) {
		SS3MediaLibrary library = new SS3MediaLibrary(fileName);

		SS3SignStreamApplication.logger.log(Level.INFO, "Started parsing media library...[" + fileName + "]");
		
		if(fileName == null || fileName.trim().length()==0) {
			SS3SignStreamApplication.logger.log(Level.INFO,"Media library was not found. Created new library.");
			return library;
		}

		try {
			
			File testFile = new File(fileName);
			boolean isFileExist = testFile.exists();
			
			SS3SignStreamApplication.logger.log(Level.INFO, "Parsing Media library: file was found: " + isFileExist);
			
			DocumentBuilder builder = DocumentBuilderFactory.newInstance()
					.newDocumentBuilder();
			Document doc = builder.parse(fileName);
			
			NodeList dbNodes = doc.getElementsByTagName("SIGNSTREAM-DATABASE");
			
			for (int i = 0; i < dbNodes.getLength(); i++) {
				Element dbEl = (Element) dbNodes.item(i);				
				NodeList locationsList = dbEl.getElementsByTagName("SEARCH_LOCATIONS");
				
				for (int l = 0; l < locationsList.getLength(); l++) {
					Element locationsEl = (Element) locationsList.item(l);
					NodeList locationList = locationsEl.getElementsByTagName("SEARCH_LOCATION");
					
					for (int f = 0; f < locationList.getLength(); f++) {
						Element El = (Element) locationList.item(f);
						SerchableDirectory directory = new SerchableDirectory();
						directory.setDirectory(El.getAttribute("PATH"));
						directory.setSearchable(El.getAttribute("LOOKUP").equalsIgnoreCase("Y"));

						int priority = 0;
						try {
							priority = Integer.parseInt(El.getAttribute("PRIORITY"));
							SS3SignStreamApplication.logger.log(Level.INFO,"Parsing Media library");
						} catch(NumberFormatException ex) {
							SS3SignStreamApplication.logger.log(Level.INFO,"Parsing Media library");
							;
						}
						
						directory.setPriority(priority);
						
						int searchCount = 0;
						try {
							searchCount = Integer.parseInt(El.getAttribute("SEARCH_COUNT"));
							
						} catch(NumberFormatException ex) {
							;
						}
						
						directory.setNumberOfSearches(searchCount);
						
						library.addDirectoryToSearch(directory);
						
						SS3SignStreamApplication.logger.log(Level.INFO, "Media Library was parsed");
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			
			SS3SignStreamApplication.logger.log(Level.INFO, "Parsing Media library: exception");
			SS3SignStreamApplication.logger.log(Level.INFO, "" + e.getStackTrace());
			
			//SS3SignStreamApplication.logger.setLevel(Level.SEVERE);
			//SS3SignStreamApplication.logger.info("Exception: " + e.getStackTrace());
			
			return library;
		}

		SS3SignStreamApplication.logger.log(Level.INFO, "Media library parsed");
		return library;
	}

	private ArrayList<SS3Participant> parseAnnotators(String fileName) throws Exception {
		ArrayList<SS3Participant> annotators = new ArrayList<SS3Participant>();

		DocumentBuilder builder = DocumentBuilderFactory.newInstance()
				.newDocumentBuilder();
		Document glbDoc = builder.parse(fileName);

		NodeList glbNodes = glbDoc.getElementsByTagName("SIGNSTREAM-DATABASE");
		for (int i = 0; i < glbNodes.getLength(); i++) {
			Element glbDbEl = (Element) glbNodes.item(i);
			NodeList glbRefList = glbDbEl.getElementsByTagName("ANNOTATORS");

			for (int j = 0; j < glbRefList.getLength(); j++) {
				Element glbPrtEl = (Element) glbRefList.item(j);
				NodeList glbPrtList = glbPrtEl
						.getElementsByTagName("ANNOTATOR");

				for (int l = 0; l < glbPrtList.getLength(); l++) {
					SS3Participant glbParticipant = new SS3Participant();
					Element glbEl = (Element) glbPrtList.item(l);
					glbParticipant.setId(glbEl.getAttribute("ID"));
					glbParticipant.setFullName(glbEl.getAttribute("FULL_NAME"));
					glbParticipant.setLabel(glbEl.getAttribute("LABEL"));
					glbParticipant.setComments(glbEl.getAttribute("COMMENTS"));

					annotators.add(glbParticipant);
				}
			}
		}
		return annotators;
	}

	public ParticipantsLibrary parseAnnotatorsLibrary(
			String globalAnnotatorsFileName, String localAnnotatorsFileName) {

		ParticipantsLibrary lib = new ParticipantsLibrary();
		lib.setGlobalParticipantsFileName(globalAnnotatorsFileName);
		lib.setLocalParticipantsFileName(localAnnotatorsFileName);

		try {
			ArrayList<SS3Participant> glbAnnotators = parseAnnotators(globalAnnotatorsFileName);			
			for(SS3Participant glbAnnotator : glbAnnotators) {
				lib.addGlobalParticipant(glbAnnotator);
			}
			
			ArrayList<SS3Participant> lclAnnotators = parseAnnotators(localAnnotatorsFileName);			
			for(SS3Participant lclAnnotator : lclAnnotators) {
				lib.addLocalParticipant(lclAnnotator);
			}
			
		} catch (Exception e) {
			SS3SignStreamApplication.logger.log(Level.INFO, e.getMessage());
			e.printStackTrace();
		}
		return lib;
	}

	private ArrayList<SS3Participant> parseParticipants(String fileName)
			throws Exception {
		ArrayList<SS3Participant> participants = new ArrayList<SS3Participant>();
		DocumentBuilder builder = DocumentBuilderFactory.newInstance()
				.newDocumentBuilder();
		Document glbDoc = builder.parse(fileName);

		NodeList glbNodes = glbDoc.getElementsByTagName("SIGNSTREAM-DATABASE");
		for (int i = 0; i < glbNodes.getLength(); i++) {
			Element glbDbEl = (Element) glbNodes.item(i);
			NodeList glbRefList = glbDbEl.getElementsByTagName("PARTICIPANTS");

			for (int j = 0; j < glbRefList.getLength(); j++) {
				Element glbPrtEl = (Element) glbRefList.item(j);
				NodeList glbPrtList = glbPrtEl
						.getElementsByTagName("PARTICIPANT");

				for (int l = 0; l < glbPrtList.getLength(); l++) {
					SS3Participant glbParticipant = new SS3Participant();
					Element glbEl = (Element) glbPrtList.item(l);
					glbParticipant.setId(glbEl.getAttribute("ID"));
					glbParticipant.setFullName(glbEl.getAttribute("FULL_NAME"));
					glbParticipant.setLabel(glbEl.getAttribute("LABEL"));
					glbParticipant.setGender(glbEl.getAttribute("GENDER"));
					glbParticipant.setDominantHand(glbEl
							.getAttribute("DOMINANT_HAND"));
					glbParticipant.setYearOfBirth(glbEl
							.getAttribute("YEAR_OF_BIRTH"));
					glbParticipant.setNativeLanguage(glbEl
							.getAttribute("NATIVE_LANGUAGE"));
					glbParticipant.setComments(glbEl.getAttribute("COMMENTS"));
					glbParticipant.setParentsInfo(glbEl
							.getAttribute("PARENT_INFO"));
					glbParticipant.setBackground(glbEl
							.getAttribute("BACKGROUND"));

					participants.add(glbParticipant);
				}
			}
		}
		return participants;
	}

	public ParticipantsLibrary parseParticipantsLibrary(
			String globalParticipantsFileName, String localParticipantsFileName) {

		ParticipantsLibrary lib = new ParticipantsLibrary();
		lib.setGlobalParticipantsFileName(globalParticipantsFileName);
		lib.setLocalParticipantsFileName(localParticipantsFileName);
		
		try {
			ArrayList<SS3Participant> globalParticipants = parseParticipants(globalParticipantsFileName);
			for (SS3Participant glbParticipant : globalParticipants) {
				lib.addGlobalParticipant(glbParticipant);
			}

			ArrayList<SS3Participant> localParticipants = parseParticipants(localParticipantsFileName);
			for (SS3Participant lclParticipant : localParticipants) {
				lib.addLocalParticipant(lclParticipant);
			}
		} catch (Exception e) {
			SS3SignStreamApplication.logger.log(Level.INFO, e.getMessage());

			e.printStackTrace();
			
			return lib;
		}

		return lib;
	}

	private ArrayList<Reference> parseReferenses(NodeList nodes) {
		ArrayList<Reference> list = new ArrayList<Reference>();
		Element element = (Element) nodes.item(0);
		NodeList prtcpNode = element.getElementsByTagName("REF");

		for (int i = 0; i < prtcpNode.getLength(); i++) {
			Reference ref = new Reference();
			Element line = (Element) prtcpNode.item(i);
			ref.setId(line.getAttribute("ID"));
			ref.setFileName(line.getAttribute("FILE_NAME"));
			ref.setSignature(line.getAttribute("SIGNATURE"));
			list.add(ref);
		}
		return list;
	}

	private ArrayList<Author> parseAuthors(NodeList nodes) {
		ArrayList<Author> list = new ArrayList<Author>();
		Element element = (Element) nodes.item(0);
		NodeList authorNode = element.getElementsByTagName("AUTHOR");

		for (int i = 0; i < authorNode.getLength(); i++) {
			Author author = new Author();
			Element line = (Element) authorNode.item(i);
			author.setParticipantID(line.getAttribute("PARTICIPANT_ID"));
			list.add(author);
		}
		return list;
	}

	private HashMap<String, java.awt.Point> parseMFilesOnScreenLocation(NodeList nodes) {
		HashMap<String, java.awt.Point> locations = new HashMap<String, java.awt.Point>();

		SS3SignStreamApplication.logger.log(Level.INFO, "Starting parsing media on screen locations");

		Object obj = nodes.item(0);
		if (obj != null) {
			Element element = (Element) obj;
			NodeList mfNode = element.getElementsByTagName("MEDIA-FILE");

			for (int i = 0; i < mfNode.getLength(); i++) {
				Element line = (Element) mfNode.item(i);
				String fileName = line.getAttribute("FILE_NAME");
				String x = line.getAttribute("LOCATION_X");
				String y = line.getAttribute("LOCATION_Y");

				try {
					int X = Integer.parseInt(x);
					int Y = Integer.parseInt(y);
					locations.put(fileName, new java.awt.Point(X, Y));
				} catch (NullPointerException ex) {
					SS3SignStreamApplication.logger.log(Level.INFO, "Parsing media on screen location: fileName = "
									+ fileName + " could not parse properly");
				}

				SS3SignStreamApplication.logger.log(Level.INFO, "Parsed media on screen location: fileName = "
								+ fileName + ", x = " + x + ", y = " + y);
			}
		}

		SS3SignStreamApplication.logger.log(Level.INFO, "Media on screen locations parsed");
		return locations;
	}

	private HashMap<String, SS3MediaFile> parseMFiles(NodeList nodes) {
		HashMap<String, SS3MediaFile> mFiles = new HashMap<String, SS3MediaFile>();

		SS3SignStreamApplication.logger.log(Level.INFO, "Starting parsing video references");

		Element element = (Element) nodes.item(0);
		NodeList mfNode = element.getElementsByTagName("MEDIA-FILE");

		for (int i = 0; i < mfNode.getLength(); i++) {
			SS3MediaFile mFile = new SS3MediaFile();
			Element line = (Element) mfNode.item(i);
			mFile.setId(line.getAttribute("ID"));
			mFile.setFile(line.getAttribute("PATH"));
			mFiles.put(mFile.getId(), mFile);

			SS3SignStreamApplication.logger.log(Level.INFO, "Parsed video reference: " + mFile);
		}
		SS3SignStreamApplication.logger.log(Level.INFO, "Video references parsed");
		return mFiles;
	}

	// point nodes to "TRACKS"
	private ArrayList<SS3Track> parseTracks(NodeList nodes) {
		ArrayList<SS3Track> tracks = new ArrayList<SS3Track>();
		Element element = (Element) nodes.item(0);
		NodeList tNode = element.getElementsByTagName("TRACK");

		for (int i = 0; i < tNode.getLength(); i++) {
			SS3Track track = new SS3Track();
			Element tEl = (Element) tNode.item(i);
			track.setId(tEl.getAttribute("FID"));

			NodeList ncNode = tEl.getElementsByTagName("NON_COMPLETE_EVENTS");
			for (int nc = 0; nc < ncNode.getLength(); nc++) {
				Element entEl = (Element) ncNode.item(nc);
				NodeList ncItmNode = entEl.getElementsByTagName("ITEM");

				for (int nci = 0; nci < ncItmNode.getLength(); nci++) {
					SS3Item ncItem = new SS3Item();
					Element ncItmEl = (Element) ncItmNode.item(nci);
					TimeInfo ncStartTimeInfo = new TimeInfo();
					ncStartTimeInfo.setMovieTime(Integer.parseInt(ncItmEl
							.getAttribute("S")));
					ncItem.setStartTimeInfo(ncStartTimeInfo);
					TimeInfo ncEndTimeInfo = new TimeInfo();
					ncEndTimeInfo.setMovieTime(Integer.parseInt(ncItmEl
							.getAttribute("E")));
					ncItem.setEndTimeInfo(ncEndTimeInfo);
					ncItem.setText(getCharacterDataFromElement(ncItmEl));
					String sf = ncItmEl.getAttribute("SF");
					String ef = ncItmEl.getAttribute("EF");
					ncItem.setStartTimeFinal(Util.isTrue(sf));
					ncItem.setEndTimeFinal(Util.isTrue(ef));
					String startEntID = ncItmEl
							.getAttribute("COMP_START_ENTITY_ID");
					String endEntID = ncItmEl
							.getAttribute("COMP_END_ENTITY_ID");
					ncItem.setStartCompaundEntityID(startEntID);
					ncItem.setEndCompaundEntityID(endEntID);
					track.addNonCompleteEvent(ncItem);
				}
			}

			NodeList etNode = tEl.getElementsByTagName("ENTERED_TEXT");
			String fid = tEl.getAttribute("FID");
			for (int et = 0; et < etNode.getLength(); et++) {
				Element etEl = (Element) etNode.item(et);
				NodeList etItmNode = etEl.getElementsByTagName("ITEM");

				for (int eti = 0; eti < etItmNode.getLength(); eti++) {
					Element etItmEl = (Element) etItmNode.item(eti);

					TimeInfo etStartTimeInfo = new TimeInfo();
					etStartTimeInfo.setMovieTime(Integer.parseInt(etItmEl
							.getAttribute("S")));

					String enteredTxt = getCharacterDataFromElement(etItmEl);
					
					if(enteredTxt.trim().length()!=0) {
						track.addEnteredTextEvent(new SS3Item(enteredTxt, etStartTimeInfo));
					}
				}
			}

			NodeList entNode = tEl.getElementsByTagName("ENTITY");
			for (int j = 0; j < entNode.getLength(); j++) {
				SS3Entity entity = new SS3Entity();
				Element entEl = (Element) entNode.item(j);
				String value = "";
				entity.setParticipantID(entEl.getAttribute("PARTICIPANT-ID"));
				entity.setID(entEl.getAttribute("ID"));
				entity.setCoderID(entEl.getAttribute("CODER-ID"));
				entity.setTimeCreated(entEl.getAttribute("TIME_CREATED"));
				entity.setTimeUpdated(entEl.getAttribute("TIME_UPDATED"));
				entity.setRefEntityId(entEl.getAttribute("REF_ENTITY_ID"));

				String compStartEntityID = entEl
						.getAttribute("COMP_START_ENTITY_ID");
				if (compStartEntityID != null) {
					entity.setStartCompaundEntityID(compStartEntityID);
				}

				String compEndEntityID = entEl
						.getAttribute("COMP_END_ENTITY_ID");
				if (compEndEntityID != null) {
					entity.setEndCompaundEntityID(compEndEntityID);
				}

				entity.setReference(entEl.getAttribute("REF"));

				NodeList itmNode = entEl.getElementsByTagName("ITEM");

				for (int l = 0; l < itmNode.getLength(); l++) {
					SS3Item item = new SS3Item();
					Element itmEl = (Element) itmNode.item(l);
					item.setId(itmEl.getAttribute("VID"));
					item.setNoteID(itmEl.getAttribute("NOTE_ID"));
					TimeInfo startTimeInfo = new TimeInfo();
					startTimeInfo.setMovieTime(Integer.parseInt(itmEl
							.getAttribute("S")));
					item.setStartTimeInfo(startTimeInfo);
					TimeInfo endTimeInfo = new TimeInfo();
					endTimeInfo.setMovieTime(Integer.parseInt(itmEl
							.getAttribute("E")));
					item.setEndTimeInfo(endTimeInfo);
					String txt = getCharacterDataFromElement(itmEl);
					item.setText(txt);
					entity.addItem(item);
					
					if(fid.equalsIgnoreCase("10000")) {
						value = value + txt;
					}
					
				}				
				track.addEntity(entity);
			}			
			tracks.add(track);
		}
		return tracks;
	}

	public String getCharacterDataFromElement(Element e) {
		Node child = e.getFirstChild();
		if (child instanceof CharacterData) {
			CharacterData cd = (CharacterData) child;
			return cd.getData();
		}
		return "";
	}
}