package edu.bu.signstream.common.db.parser;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Level;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.CharacterData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import edu.bu.signstream.common.util.vo.CodingSchemaField;
import edu.bu.signstream.common.util.vo.CodingSchemaFieldImg;
import edu.bu.signstream.common.util.vo.CodingSchemaFieldType;
import edu.bu.signstream.common.util.vo.CodingSchemaValue;
import edu.bu.signstream.common.util.vo.MediaFile;
import edu.bu.signstream.common.util.vo.Participant;
import edu.bu.signstream.common.util.vo.Profile;
import edu.bu.signstream.common.util.vo.Segment;
import edu.bu.signstream.common.util.vo.SignStreamDocument;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.Utterance;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class XMLParser extends Parser {
   /*
   	public static void main(String str[]) {
   		XMLParser parser = new XMLParser();
   		String filename = "C:/test/accident/accident.ss2.xml";
   		SignStreamDocument doc = parser.loadSSDocument(filename);
   	}
   */

   private File file;

   @Override
public SignStreamDocument loadSSDocument(String fileName) {
      SignStreamDocument ssDocument = null;
      try {
         DocumentBuilder builder =
            DocumentBuilderFactory.newInstance().newDocumentBuilder();

         Document doc = builder.parse(fileName);

         this.file = new File(fileName);

         NodeList nodes = doc.getElementsByTagName("SIGNSTREAM-DATABASE");
         for (int i = 0; i < nodes.getLength(); i++) {
            Element element = (Element) nodes.item(i);

            Profile profile = parseProfile(nodes);
            //				
            System.err.println("parsing participants");

            NodeList prtspList = element.getElementsByTagName("PARTICIPANTS");
            HashMap participants = parseParticipants(prtspList);
            //				
            System.err.println("parsing CODING-SCHEME");

            NodeList csList = element.getElementsByTagName("CODING-SCHEME");
            HashMap codingScheme = parseCodingScheme(csList);
            //				
            System.err.println("parsing MEDIA-FILES");

            NodeList mfList = element.getElementsByTagName("MEDIA-FILES");
            HashMap mediaFiles = parseMediaFiles(mfList);
            //				
            System.err.println("parsing UTTERANCES");

            NodeList uttrList = element.getElementsByTagName("UTTERANCES");
            HashMap utterances = parseUtterances(uttrList);
            //				
            System.err.println("DONE DONE DONE");

            ssDocument =
               new SignStreamDocument(
                  profile,
                  participants,
                  codingScheme,
                  mediaFiles,
                  utterances,
                  fileName);
         }
      } catch (Exception e) {
    	  SS3SignStreamApplication.logger.log(Level.INFO, e.getMessage());
      }
      return ssDocument;
   }

   private HashMap parseParticipants(NodeList nodes) {
      HashMap participants = new HashMap();
      Element element = (Element) nodes.item(0);
      NodeList prtcpNode = element.getElementsByTagName("PARTICIPANT");

      for (int i = 0; i < prtcpNode.getLength(); i++) {
         Element line = (Element) prtcpNode.item(i);
         String id = line.getAttribute("ID");
         String name = line.getAttribute("NAME");
         String label = line.getAttribute("LABEL");
         String age = line.getAttribute("AGE");
         String gender = line.getAttribute("GENDER");
         String language = line.getAttribute("LANGUAGE");
         String comments = line.getAttribute("COMMENTS");
         String parents = line.getAttribute("PARENTS");

         ArrayList backgrounds = new ArrayList();
         NodeList bckgrdList = element.getElementsByTagName("BACKGROUND");
         for (int j = 0; j < bckgrdList.getLength(); j++) {
            Element el = (Element) bckgrdList.item(j);
            backgrounds.add(getCharacterDataFromElement(el));
         }

         Participant participant =
            new Participant(
               id,
               name,
               label,
               age,
               gender,
               language,
               comments,
               parents,
               backgrounds);

         participants.put(id, participant);
      }
      return participants;
   }

   private HashMap getCodingSchemaFieldType(NodeList nodes) {
      HashMap csFieldsType = new HashMap();

      Element csElement = (Element) nodes.item(0);
      NodeList csNodes = csElement.getElementsByTagName("FIELD_TYPES");
      Element cstElement = (Element) csNodes.item(0);
      NodeList cstNodes = cstElement.getElementsByTagName("FIELD_TYPE");

      for (int i = 0; i < cstNodes.getLength(); i++) {
         Element element = (Element) cstNodes.item(i);
         String type = element.getAttribute("TYPE");
         String label = element.getAttribute("LABEL");
         String description = element.getAttribute("DESCRIPTION");

         CodingSchemaFieldType csFieldType =
            new CodingSchemaFieldType(type, label, description);

         NodeList csList = element.getElementsByTagName("IMG");

         for (int j = 0; j < csList.getLength(); j++) {
            Element el = (Element) csList.item(j);
            String id = el.getAttribute("ID");
            String path = el.getAttribute("PATH");
            String descr = el.getAttribute("DESCRIPTION");

            CodingSchemaFieldImg fieldImg =
               new CodingSchemaFieldImg(id, path, descr);

            csFieldType.addFieldImg(fieldImg);
         }
         csFieldsType.put(type, csFieldType);
      }
      return csFieldsType;
   }

   private HashMap parseCodingScheme(NodeList nodes) {
      HashMap codingScheme = new HashMap();
      HashMap csFieldTypes = getCodingSchemaFieldType(nodes);

      Element csElement = (Element) nodes.item(0);
      NodeList csNodes = csElement.getElementsByTagName("FIELD");

      for (int i = 0; i < csNodes.getLength(); i++) {

         Element element = (Element) csNodes.item(i);
         String fieldId = element.getAttribute("ID");
         String name = element.getAttribute("NAME");
         String label = element.getAttribute("LABEL");
         String color = element.getAttribute("COLOR");
         String constraint = element.getAttribute("CONSTRAINT");
         String timeAlignment = element.getAttribute("TIME-ALIGNMENT");
         String prefix = element.getAttribute("PREFIX");
         String type = element.getAttribute("TYPE");

         ArrayList csValues = new ArrayList();
         NodeList csList = element.getElementsByTagName("VALUE");

         for (int j = 0; j < csList.getLength(); j++) {
            Element el = (Element) csList.item(j);
            String vId = el.getAttribute("ID");
            String imageID = el.getAttribute("IID");
            String vName = el.getAttribute("NAME");
            String vLabel = el.getAttribute("LABEL");
            String vColor = el.getAttribute("COLOR");

            CodingSchemaValue csValue =
               new CodingSchemaValue(vId, vName, vLabel, vColor);
            csValue.setImageID(imageID);

            csValues.add(csValue);
         }

         CodingSchemaField csField =
            new CodingSchemaField(
               fieldId,
               name,
               label,
               constraint,
               timeAlignment,
               prefix,
               color,
               csValues);

         CodingSchemaFieldType fType =
            (CodingSchemaFieldType) csFieldTypes.get(type);
         csField.setCodingSchemaFieldType(fType);

         codingScheme.put(fieldId, csField);
      }

      return codingScheme;
   }

   private HashMap parseMediaFiles(NodeList nodes) {
      HashMap mediaFiles = new HashMap();

      Element element = (Element) nodes.item(0);
      NodeList csNode = element.getElementsByTagName("MEDIA-FILE");

      for (int i = 0; i < csNode.getLength(); i++) {
         Element line = (Element) csNode.item(i);

         NamedNodeMap map = line.getAttributes();
         Node n = map.getNamedItem("LEGACY-PATH");

         String legacyPath = n.getNodeValue();
         String id = line.getAttribute("ID");

         MediaFile mediaFile = new MediaFile(id, legacyPath, legacyPath, file.getParent());

         mediaFiles.put(id, mediaFile);
      }
      return mediaFiles;
   }

   private HashMap parseUtterances(NodeList nodes) {
      // nodes = SIGNSTREAM-DATABASE/UTTERANCES
      HashMap utterances = new HashMap();

      Element element = (Element) nodes.item(0);
      NodeList uttreranceNodes = element.getElementsByTagName("UTTERANCE");

      for (int i = 0; i < uttreranceNodes.getLength(); i++) {
         Element utteranceElement = (Element) uttreranceNodes.item(i);
         String id = utteranceElement.getAttribute("ID");
         String excerpt = utteranceElement.getAttribute("EXCERPT");
         String start = utteranceElement.getAttribute("S");
         String end = utteranceElement.getAttribute("E");

         ArrayList notes = new ArrayList();
         NodeList ntsList = utteranceElement.getElementsByTagName("NOTES");
         for (int j = 0; j < ntsList.getLength(); j++) {
            Element el = (Element) ntsList.item(j);
            String note = getCharacterDataFromElement(el);

            notes.add(note);
         }

         HashMap mediaRefs = new HashMap();
         NodeList mList = utteranceElement.getElementsByTagName("MEDIA-REF");

         for (int m = 0; m < mList.getLength(); m++) {
            Element meEl = (Element) mList.item(m);
            String mediaId = meEl.getAttribute("ID");
            MediaFile mFile = new MediaFile("" + mediaId, "", "", file.getParent());
            mediaRefs.put("" + mediaId, mFile);
         }
         // parse segments
         ArrayList segments = new ArrayList();
         NodeList sList = utteranceElement.getElementsByTagName("SEGMENT");

         for (int l = 0; l < sList.getLength(); l++) {
            Element m = (Element) sList.item(l);
            String participantId = m.getAttribute("PARTICIPANT-ID");
            String primaryParticipant = m.getAttribute("PRIMARY");

            HashMap tracks = new HashMap();
            NodeList trList = m.getElementsByTagName("TRACK");

            for (int t = 0; t < trList.getLength(); t++) {
               Track track = new Track();
               Element ml = (Element) trList.item(t);
               String fId = ml.getAttribute("FID");
               track.setFieldId(fId);

               NodeList vList = ml.getElementsByTagName("A");
               for (int v = 0; v < vList.getLength(); v++) {
                  Element vl = (Element) vList.item(v);
                  String s = vl.getAttribute("S");
                  String e = vl.getAttribute("E");
                  int startMovieTime = Integer.parseInt(s);
                  int endMovieTime = Integer.parseInt(e);
                  TimeInfo startMovieTimeInfo = new TimeInfo();
                  startMovieTimeInfo.setMovieTime(startMovieTime);
                  TimeInfo endMovieTimeInfo = new TimeInfo();
                  endMovieTimeInfo.setMovieTime(endMovieTime);

                  String vid = vl.getAttribute("VID");
                  String text = getCharacterDataFromElement(vl);
                  TrackValue value =
                     new TrackValue(
                        vid,
                        text,
                        startMovieTimeInfo,
                        endMovieTimeInfo);
                  track.addTrackValue(value);
               }

               tracks.put(fId, track);
            }
            Segment segment =
               new Segment(participantId, primaryParticipant, tracks);
            segments.add(segment);
         }

         int utStartMovieTime = Integer.parseInt(start);
         int utEndMovieTime = Integer.parseInt(end);
         TimeInfo utStartMovieTimeInfo = new TimeInfo();
         utStartMovieTimeInfo.setMovieTime(utStartMovieTime);
         TimeInfo utEndMovieTimeInfo = new TimeInfo();
         utEndMovieTimeInfo.setMovieTime(utEndMovieTime);

         Utterance utterance =
            new Utterance(
               id,
               utStartMovieTimeInfo,
               utEndMovieTimeInfo,
               excerpt,
               mediaRefs,
               null,
               segments,
               notes);

         utterances.put(id, utterance);
      }
      return utterances;
   }

   private Profile parseProfile(NodeList nodes) {
      Element element = (Element) nodes.item(0);
      String ssVersion = element.getAttribute("SIGNSTREAM-VERSION");
      String dbVersion = element.getAttribute("DATABASE-VERSION");
      String sourcePath = element.getAttribute("SOURCE");

      NodeList dstrList = element.getElementsByTagName("DISTRIBUTOR");
      Element elDstr = (Element) dstrList.item(0);
      String distributor = getCharacterDataFromElement(elDstr);

      NodeList authorList = element.getElementsByTagName("AUTHOR");
      Element elAuthor = (Element) authorList.item(0);
      String author = getCharacterDataFromElement(elAuthor);

      NodeList citationList = element.getElementsByTagName("CITATION");
      Element elCitation = (Element) authorList.item(0);
      String citation = getCharacterDataFromElement(elCitation);

      NodeList notesList = element.getElementsByTagName("NOTES");
      Element elNotes = (Element) authorList.item(0);
      String notes = getCharacterDataFromElement(elNotes);

      return new Profile(
         ssVersion,
         dbVersion,
         sourcePath,
         distributor,
         author,
         citation,
         notes);
   }

   public String getCharacterDataFromElement(Element e) {
      Node child = e.getFirstChild();
      if (child instanceof CharacterData) {
         CharacterData cd = (CharacterData) child;
         return cd.getData();
      }
      return "";
   }
}
