package edu.bu.signstream.common.util;

import java.awt.Color;
import java.awt.FontMetrics;
import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;

import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;

public class Util {
	private static int counter;
	private static int INTEGER_MAX_VALUE = Integer.MAX_VALUE;
	
	public static String GLOSS1_FIELD_ID = "10000";
	public static String GLOSS2_FIELD_ID = "10001";
	
	public static String RIGHT_HAND_FIELD_ID = "10501";
	public static String LEFT_HAND_FIELD_ID = "10502";
	
	public static String RIGHT_HANDSHAPE_FIELD_ID = "1000";
	public static String LEFT_HANDSHAPE_FIELD_ID = "2000";
	
	public static String FREE_TEXT_FIELD_ID = "30000";
	
	public static int MILLISECONDS_WAIT_LISTENERS = 300;

	public static String VIDEO_FILE_MOV = "mov";
	public static String VIDEO_FILE_MPG = "mpg";
	public static String VIDEO_FILE_AVI = "avi";
	
	public final static char SEGMENT_TIER_SEPARATOR = ':';
	
	public static String SIGN_BANK_DIR = "signbank";
	public static String SIGN_BANK_PRIORITIES_FILE = "priorityList.xml";
	public static String SIGN_BANK_DEFAULT_LOCAL_FILE = "localSignBank.xml";
	
	public static int MTB_TEXT_BUFFER = 5;
   
   private static SearchPaths searchPaths = new SearchPaths();
   
   public static SearchPaths getSearchPaths() {
      return searchPaths;    
   }

   public static String getCurrentDayNTime() {
	      Calendar c = Calendar.getInstance();
	      StringBuffer sb = new StringBuffer();
	      sb.append(c.get(Calendar.MONTH));
	      sb.append("/");
	      sb.append(c.get(Calendar.DAY_OF_MONTH));
	      sb.append("/");
	      sb.append(c.get(Calendar.YEAR));
	      sb.append(" ");
	      sb.append(c.get(Calendar.HOUR_OF_DAY));
	      sb.append(":");
	      sb.append(c.get(Calendar.MINUTE));
	      int am = c.get(Calendar.AM_PM);
	      if (am == 0) {
	         sb.append(" AM");
	      } else {
	         sb.append(" PM");
	      }
	      return sb.toString();
   }
   
   public static boolean isNotEmpty(String str) {
      return str != null && str.trim().length() > 0;
   }

	public static String getUniqueNumber() {
		Calendar calendar = Calendar.getInstance();
		counter++;
		
		long longtime = calendar.getTimeInMillis() + counter;
		long originallongtime = longtime;
		String timeStr;
		String substr;
		
		// keep truncating the long until its less than INTEGER_MAX_VALUE
		do {	
			timeStr = "" + longtime;
			substr = timeStr.substring(5);
			longtime = Long.parseLong(substr);
			substr = "" + longtime;		// re-cast to a string in case parseLong chopped off any leading zero's
		} while (longtime > INTEGER_MAX_VALUE);
		
		return substr;
	}

	public static String getLongUniqueNumber() {
		Calendar calendar = Calendar.getInstance();
		// do not remove counter!!! Should be here to ensure that number is unique
		counter++;
		return "" + calendar.getTimeInMillis() + counter;
	}

	public static ArrayList<String> getAdjustedLabels(String str,
			FontMetrics fontMetrics, int maxLength) {
		ArrayList<String> list = new ArrayList<String>();

		String str0 = str;
		String str1 = "";

		while (str0.length() > 1) {
			str1 = getAdjustedLabel(str0, fontMetrics, maxLength);
			list.add(str1);
			str0 = str0.substring(str1.length());
		}

		return list;
	}

	public static String getAdjustedLabel(String str, FontMetrics fontMetrics,
			int maxLength) {

		if (str == null)
			return "";

		str = str.trim();

		if (fontMetrics.stringWidth(str) <= maxLength)
			return str;

		String strToDraw = str;

		while (fontMetrics.stringWidth(strToDraw) > maxLength) {
//		- fontMetrics.stringWidth("...")) {
			if (strToDraw.length() > 1) {
				strToDraw = strToDraw.substring(0, strToDraw.length() - 1);
			} else {
				break;
			}
		}

		return strToDraw;
	}

	public static boolean str2Bool(String str) {
		return str.equalsIgnoreCase("true");
	}

	public static String color2HexString(Color color) {
		StringBuffer sbColor = new StringBuffer();

		sbColor.append(toHexString(color.getRed()));
		sbColor.append(toHexString(color.getGreen()));
		sbColor.append(toHexString(color.getBlue()));

		return sbColor.toString();
	}

	private static String toHexString(int i) {
		if (i > 0xFF || i < 0)
			throw new RuntimeException(
					"Color component is outside expected range: " + i);
		if (i <= 0x0F)
			return "0" + Integer.toHexString(i);
		else
			return Integer.toHexString(i);
	}

	public static Color hexString2Color(String hexColor) {
		if (hexColor.length() < 6)
			return Color.BLACK;
		// throw new RuntimeException("Color component is outside expected
		// range: 6");

		String red = hexColor.substring(0, 2);
		String green = hexColor.substring(2, 4);
		String blue = hexColor.substring(4, 6);

      Color color = Color.BLACK;
      
      try {
         color = new Color(Integer.parseInt(red, 16), Integer.parseInt(green, 16), Integer.parseInt(blue, 16));
      } catch(Exception ex) {
         ;
      }

		return color;
	}
	
	public static boolean isXML3File(String fileName) {
		return (fileName.lastIndexOf(".3.xml") > -1);
	}

	public static boolean isXMLFile(String fileName) {
		String ext = getExtension(fileName);

		if ("xml".equalsIgnoreCase(ext))
			return true;
		return false;
	}

	public static boolean isMovieFile(File file) {
		String ext = getExtension(file);

		if (VIDEO_FILE_MOV.equals(ext) || VIDEO_FILE_MPG.equals(ext)
				|| VIDEO_FILE_AVI.equals(ext))
			return true;
		return false;
	}

	public static boolean isMovieFile(String fileName) {
		String ext = getExtension(fileName);

		if (VIDEO_FILE_MOV.equals(ext) || VIDEO_FILE_MPG.equals(ext)
				|| VIDEO_FILE_AVI.equals(ext))
			return true;
		return false;
	}

	/*
	 * Get the extension of a file.
	 */
	public static String getExtension(String fileName) {
		String ext = null;
		String s = getName(fileName);
		int i = s.lastIndexOf('.');

		if (i > 0 && i < s.length() - 1) {
			ext = s.substring(i + 1).toLowerCase();
		}
		return ext;
	}

	public static String getExtension(File file) {
		String ext = null;
		String s = file.getName();
		int i = s.lastIndexOf('.');

		if (i > 0 && i < s.length() - 1) {
			ext = s.substring(i + 1).toLowerCase();
		}
		return ext;
	}

	public static boolean isTextFile(String fileName) {
		String extension = getExtension(fileName);
		return "txt".equals(extension);
	}

	public static String getName(String fileName) {
		File file = new File(fileName);
		return file.getName();
	}

	public static String getNameNoExtension(String fileName) {
		String name = getName(fileName);
		String extension = getExtension(fileName);
		int s = name.length() - extension.length() - 1;
		String fileNameNoExtension = name.substring(0, s);
		return fileNameNoExtension;
	}

	public static boolean isBinaryFile(String fileName) {
		if (isMovieFile(fileName) && isTextFile(fileName))
			return true;

		return false;
	}
   
   public static boolean isTrue(String y) {
      return (y.equalsIgnoreCase("y") || y.equalsIgnoreCase("yes") || y.equalsIgnoreCase("true") || y.equalsIgnoreCase("1") || y.equalsIgnoreCase("t"));
   }
   
   public static String getBooleanAsString(boolean b) {
      if(b) {
         return "T";
      }
      return "F";
   }
   

	/**
	 * Checks for a match between occurrenceLabel and any in the list of SS3FieldValue objects.
	 * Intended for use only by clients of sign bank classes/functionality.
	 * @param fieldValues The field values taken from one field of the coding schema
	 * @param occurrenceLabel The label from the occurrence to compare to the coding schema labels
	 * @return SS3FieldValue The SS3FieldValue whose label matches occurrenceLabel, or null if none
	 */
	public static SS3FieldValue matchSignBank(ArrayList<SS3FieldValue> fieldValues, String occurrenceLabel) {
		
		// the coding scheme does not find the "big-" images in the directory, so use the others of similar name
		// hack for now
		String alternativeHSImageTag = "big-";
		int tagSize = alternativeHSImageTag.length();
		
		if (occurrenceLabel.length() > tagSize && occurrenceLabel.substring(0, tagSize).equals(alternativeHSImageTag)) {
			occurrenceLabel = occurrenceLabel.substring(tagSize);
		}
		
		// exact match check
		for (SS3FieldValue value: fieldValues) {
			String valueLabel = value.getLabel();

			if (valueLabel != null && !valueLabel.isEmpty()) {	
				if (valueLabel.equals(occurrenceLabel)) {
					return value;
				}
			}
		}
		
		// case where labels have a slash
		for (SS3FieldValue value: fieldValues) {
			String valueLabel = value.getLabel();
			
			if (valueLabel != null && !valueLabel.isEmpty()) {			
				
				if (valueLabel.contains("/")) {
					int index = valueLabel.indexOf("/");
					String valueLabel1 = null, valueLabel2 = null;
					valueLabel1 = valueLabel.substring(0, index);
					valueLabel2 = valueLabel.substring(index + 1);
					
					if (valueLabel1.equals(occurrenceLabel) || valueLabel2.equals(occurrenceLabel)) {
						return value;
					}
				}
				
				if (occurrenceLabel.contains("/")) {
					int index = occurrenceLabel.indexOf("/");
					String occurrenceLabel1 = null, occurrenceLabel2 = null;
					occurrenceLabel1 = occurrenceLabel.substring(0, index);
					occurrenceLabel2 = occurrenceLabel.substring(index + 1);
					
					if (valueLabel.equals(occurrenceLabel1) || valueLabel.equals(occurrenceLabel2)) {
						return value;
					}
				}
			}			
		}
		
		return null;
	}
	
	/**
	 * Checks for a match between schemaLabel and occurrenceLabel
	 * Intended for use only by clients of sign bank classes/functionality.
	 * @param schemaLabel The label based on the coding schema, which could contain two labels separated by a slash (e.g. "F/9")
	 * @param occurrenceLabel The label from the occurrence to compare to the coding schema labels
	 * @return boolean whether the labels match or schemaLabel contains occurrenceLabel
	 */
	public static boolean matchSignBank(String schemaLabel, String occurrenceLabel) {
		
		if (schemaLabel != null && !schemaLabel.isEmpty()) {			
			if (schemaLabel.contains("/")) {
				int index = schemaLabel.indexOf("/");
				String valueLabel1 = null, valueLabel2 = null;
				valueLabel1 = schemaLabel.substring(0, index);
				valueLabel2 = schemaLabel.substring(index + 1);
				
				if ( !valueLabel1.equals(occurrenceLabel) && !valueLabel2.equals(occurrenceLabel)) {
					return false;
				}
			} else {
				if ( !schemaLabel.equals(occurrenceLabel)) {
					return false;
				}
			}
		} else {
//			return false;
		}
		
		return true;
	}
	
	public static String trimAsterisksAndPlusSigns(String input) {
		if (input == null)
			return null;

		String output = input;
		
		//Remove leading + and * symbols
		while(output.length() > 0 && (output.charAt(0) == '+' || output.charAt(0) == '*')) {
			output = output.substring(1);
		}
		
		//Remove trailing + and * symbols
		while (output.length() > 0 && (output.charAt(output.length()-1) == '+' || output.charAt(output.length()-1) == '*')) {
			output = output.substring(0, output.length()-1);
		}
				
		return output;
	}
}
