package edu.bu.signstream.common.util.signbank;

public class SignBankResource {
	
	private final int INVALID_PRIORITY = -999;
	
	private int priority;
	
	private String signBankFileName = null;
	
	private String signBankAbsolutePath = null;
	
	protected boolean nonEditableResource = false;
	
	private String signBankAuthor = null;
	private String dateCreated = null;
	private String dateModified = null;
	private String redirect = null;
	
	public SignBankResource(String signBankFileName, String signBankAbsolutePath) {
		this.priority = INVALID_PRIORITY;
		this.signBankFileName = signBankFileName;
		this.signBankAbsolutePath = signBankAbsolutePath;
	}
	
	public String getSignBankFileName() {
		return signBankFileName;
	}
	
	public String getSignBankAbsolutePath() {
		return signBankAbsolutePath;
	}
	
	public boolean isNonEditableResource() {
		return nonEditableResource;
	}
	
	/**
	 * Set the priority of this SignBankResource.
	 * This method is intended to be called only by SignBankResourceCollection.setResourcesList(...)
	 * @param priority The priority to reset the SignBankResource to
	 */
	protected void setPriority(int priority) {
		this.priority = priority;
	}
	
	/**
	 * Get the priority of this SIgnBankResource.
	 * This method is intended to be called only by SIgnBankResource.ResourceComparator.compare(...)
	 * @return The priority of this resource
	 */
	protected int getPriority() {
		return priority;
	}
	
	/**
	 * Check whether this resource is valid as determined by whether its priority has been properly set
	 * @return A boolean representing whether this resource is valid
	 */
	protected boolean isValidPriority() {
		return priority != INVALID_PRIORITY;
	}
	
	public String getSignBankAuthor() {
		return signBankAuthor;
	}
	
	public void setSignBankAuthor(String signBankAuthor) {
		this.signBankAuthor = signBankAuthor;
	}
	
	public String getDateCreated() {
		return dateCreated;
	}
	
	public void setDateCreated(String dateCreated) {
		this.dateCreated = dateCreated;
	}
	
	public String getDateModified() {
		return dateModified;
	}
	
	public void setDateModified(String dateModified) {
		this.dateModified = dateModified;
	}
	
	public String  getRedirect() {
		return redirect;
	}
	
	public void setRedirect(String redirect) {
		this.redirect = redirect;
	}
	
	@Override
	public SignBankResource clone() {
		SignBankResource clone = new SignBankResource(signBankFileName, signBankAbsolutePath);
		clone.priority = priority;
		clone.nonEditableResource = nonEditableResource;		
		clone.signBankAuthor = signBankAuthor;
		clone.dateCreated = dateCreated;
		clone.dateModified = dateModified;
		clone.redirect = redirect;
		return clone;
	}
	
	@Override
	public String toString() {
		String ret = "SignBankResource[";
		ret += "SignBankFileName=" + signBankFileName;
		ret += ", Priority=" + priority;
		ret += ", SignBankAbsolutePath=" +signBankAbsolutePath;
		ret += ", Editabable=" + !nonEditableResource;
		ret += "]";
		return ret;
	}

}
