package edu.bu.signstream.common.util.signbank;

import java.util.ArrayList;
import java.util.Comparator;

import edu.bu.signstream.common.util.Util;

public class SignBankResourceCollection {
	
	public static final String NON_EDITABLE_FILENAME_STRING = "global";
	
	private boolean CAN_ACCESS_SIGN_BANK = false;
	
	private ResourceComparator resourceComparator = null;
	private ArrayList<SignBankResource> resourcesPriorityList = null;
	
	public SignBankResourceCollection() {
		resourceComparator = new ResourceComparator();
		resourcesPriorityList = new ArrayList<SignBankResource>();
	}
	
	/**
	 * Get a cloned list of resources tracked in this collection, ordered by their relative priorities
	 * @return An ArrayList containing the ordered resources from this collection 
	 */
	public ArrayList<SignBankResource> getResourcesList() {
		ArrayList<SignBankResource> resourcesList = new ArrayList<SignBankResource>();
		
		for (SignBankResource resource: resourcesPriorityList) {
			SignBankResource clonedResource = resource.clone();
			resourcesList.add(clonedResource);
		}
		
		return resourcesList;
	}
	
	/**
	 * Sets the resources in this collection to clones of those found in resourcesList in this specific order
	 * @param resourcesList An ArrayList of SignBankResource objects to set in this collection
	 */
	public void setResourcesList(ArrayList<SignBankResource> resourcesList) {
		if (resourcesList == null || resourcesList.isEmpty()) {
			
			// can access sign bank should be set to false here too TODO
			
			return;			
		}
		
		boolean foundDefaultLocalFile = false;
		
		resourcesPriorityList.clear();
		
		SignBankResource resource = null;
		SignBankResource clonedResource = null;
		for (int i = 0; i < resourcesList.size(); i++) {
			resource = resourcesList.get(i);
			
			clonedResource = resource.clone();
			
			clonedResource.setPriority(i);

			String signBankFileName = clonedResource.getSignBankFileName();
			if (signBankFileName != null) {
				
				if (signBankFileName.toLowerCase().contains(
								NON_EDITABLE_FILENAME_STRING.toLowerCase())) {
					clonedResource.nonEditableResource = true;
				}
				
				if (signBankFileName.equals(Util.SIGN_BANK_DEFAULT_LOCAL_FILE)) {
					foundDefaultLocalFile = true;
				}
			}
			
			resourcesPriorityList.add(clonedResource);
		}

		if (resourcesPriorityList.size() > 0 && foundDefaultLocalFile)
			CAN_ACCESS_SIGN_BANK = true;
		else
			CAN_ACCESS_SIGN_BANK = false;
		
	}

	/**
	 * Gets a list of all the editable (local) resources in this collection.
	 * @return An ArrayList containing the editable (local) resources from this collection.
	 */
	public ArrayList<SignBankResource> getEditableResourcesList() {
		ArrayList<SignBankResource> editableResourcesList = new ArrayList<SignBankResource>();
		
		for (SignBankResource resource: resourcesPriorityList) {
			if ( !resource.isNonEditableResource()) {
				SignBankResource clonedResource = resource.clone();
				editableResourcesList.add(clonedResource);
			}
		}
		
		return editableResourcesList;
	}

	/**
	 * Gets a list of the non-editable (global) resources in this collection
	 * @return An ArrayList containing the non-editable (global) resources from this collection.
	 */
	public ArrayList<SignBankResource> getNonEditableResourcesList() {
		ArrayList<SignBankResource> nonEditableResourcesList = new ArrayList<SignBankResource>();
		
		for (SignBankResource resource: resourcesPriorityList) {
			if ( resource.isNonEditableResource()) {
				SignBankResource clonedResource = resource.clone();
				nonEditableResourcesList.add(clonedResource);
			}
		}
		
		return nonEditableResourcesList;
	}
	
	public SignBankResource getResourceByFileName(String fileName) {
		
		SignBankResource ret = null;
		
		if (fileName == null || fileName.isEmpty())
			return ret;

		for (SignBankResource resource: resourcesPriorityList) {
			String resourceFileName = resource.getSignBankFileName();
			if (fileName.equals(resourceFileName)) {
				ret = resource;
				break;
			}
		}
		
		return ret;
	}
	
	// if there are no resources in this collection, then user should be disallowed from accessing the sign bank
	public boolean canAccessSignBank() {
		return CAN_ACCESS_SIGN_BANK;
	}
	
	private class ResourceComparator implements Comparator {
		@Override
		public int compare(Object o1, Object o2) {
			SignBankResource r1 = (SignBankResource) o1;
			SignBankResource r2 = (SignBankResource) o2;
			
			int r1priority = r1.getPriority();
			int r2priority = r2.getPriority();

			if (r1priority < r2priority) {
				return -1;
			} else if (r1priority > r2priority) {
				return 1;
			} else {
				return 0;
			}
		}
	}
	
}
