package edu.bu.signstream.common.util.signbank;

import java.util.ArrayList;
import java.util.HashMap;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import edu.bu.signstream.common.db.XMLWriter;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankMainGloss;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankVariant;

public class SignBankWriter {
	
	public boolean writeCollectionToSignBank(SignBankCollection collection) {
		return true;
	}

	// to be called only by SignBankResultListPanel on insert/delete
	public boolean writeResourceToSignBank(SignBankCollection collection, String resourceFileName) {
		if (collection == null || resourceFileName == null || resourceFileName.isEmpty())
			return false;
		
		SignBankResourceCollection resourceCollection = collection.getSignBankResourceCollection();
		SignBankResource resource = resourceCollection.getResourceByFileName(resourceFileName);
		
		if (resource == null)
			return false;
		
		String signBankAuthor = resource.getSignBankAuthor();
		String dateCreated = resource.getDateCreated();
		String dateModified = resource.getDateModified();
		String redirect = resource.getRedirect();

		HashMap<String, SignBankMainGloss> mainGlosses = collection.getMainGlossesList();
		HashMap<String, SignBankVariant> variants;
		HashMap<String, SignBankOccurrence> occurrences;
		
		String defaultVariantID = null;
		String defaultOccurrenceID = null;
		
		boolean foundDefaultVariant = false;
		boolean foundDefaultOccurrence = false;
		
		Document doc = null;
		try {
			doc = XMLWriter.createNewXMLDocument(resource.getSignBankAbsolutePath());
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Failed to save sign bank resource file: " + resourceFileName);
			return false;
		}
		
		Element root = doc.createElement("SIGNBANK");
		if (signBankAuthor != null)
			root.setAttribute("AUTHOR", signBankAuthor);
		if (dateCreated != null)
			root.setAttribute("DATE_CREATED", dateCreated);
		if (dateModified != null)
			root.setAttribute("DATE_MODIFIED",  dateModified);
		if (redirect != null)
			root.setAttribute("REDIRECT",  redirect);
		
		Element mainGlossesListEl = doc.createElement("MAIN_GLOSSES");
		Element variantsListEl = doc.createElement("VARIANTS");
		Element occurrencesListEl = doc.createElement("OCCURRENCES");
		
		Element mainGlossEl = null, variantEl = null, occurrenceEl = null;

		ArrayList<SignBankMainGloss> matchingMainGlosses = new ArrayList<SignBankMainGloss>();
		for (SignBankMainGloss mainGloss: mainGlosses.values()) {
			variants = mainGloss.getVariantsList();
			defaultVariantID = mainGloss.getDefaultVariantID();
			foundDefaultVariant = false;
			
			mainGlossEl = doc.createElement("MAIN_GLOSS");
			mainGlossEl.setAttribute("ID", mainGloss.getMainGlossID());
			
			variantsListEl = doc.createElement("VARIANTS");
			mainGlossEl.appendChild(variantsListEl);
			
			ArrayList<SignBankVariant> matchingVariants = new ArrayList<SignBankVariant>();
			for (SignBankVariant variant: variants.values()) {
				occurrences = variant.getOccurrencesList();
				defaultOccurrenceID = variant.getDefaultOccurenceID();
				foundDefaultOccurrence = false;
				
				variantEl = doc.createElement("VARIANT");
				variantEl.setAttribute("ID", variant.getVariantID());
				
				occurrencesListEl = doc.createElement("OCCURRENCES");
				variantEl.appendChild(occurrencesListEl);
				
				ArrayList<SignBankOccurrence> matchingOccurrences = new ArrayList<SignBankOccurrence>();
				for (SignBankOccurrence occurrence: occurrences.values()) {
					String occResFileName = occurrence.getSIgnBankResourceFileName();
					if (resourceFileName.equals(occResFileName)) {
						matchingOccurrences.add(occurrence);
						
						if (occurrence.getOccurrenceID().equals(defaultOccurrenceID))
							foundDefaultOccurrence = true;
						
						occurrenceEl = doc.createElement("OCCURRENCE");
						occurrenceEl.setAttribute("ID", occurrence.getOccurrenceID());

						// surround this in a try/catch, some mechanism to not break for these (optional) things TODO
						Element domStartHSEl = doc.createElement("D_START_HS");
						domStartHSEl.setTextContent(occurrence.getDomStartHS());
						occurrenceEl.appendChild(domStartHSEl);
						Element nondomStartHSEl = doc.createElement("ND_START_HS");
						nondomStartHSEl.setTextContent(occurrence.getNondomStartHS());
						occurrenceEl.appendChild(nondomStartHSEl);
						Element domEndHSEl = doc.createElement("D_END_HS");
						domEndHSEl.setTextContent(occurrence.getDomEndHS());
						occurrenceEl.appendChild(domEndHSEl);
						Element nondomEndHSEl = doc.createElement("ND_END_HS");
						nondomEndHSEl.setTextContent(occurrence.getNondomEndHS());
						occurrenceEl.appendChild(nondomEndHSEl);
						
						ArrayList<String> frameImageURLs = occurrence.getFrameImageURLs();
						Element imagesEl = doc.createElement("IMAGES");
						int counter = 1;
						for (String url: frameImageURLs) {
							Element imageEl = doc.createElement("IMAGE");
							imageEl.setAttribute("URL", url);
							imageEl.setAttribute("ORDER", "" + counter);
							imagesEl.appendChild(imageEl);
							counter++;
						}
						occurrenceEl.appendChild(imagesEl);
						
						ArrayList<String> signVideoURLs = occurrence.getSignVideoURLs();
						Element videosEl = doc.createElement("VIDEOS");
						for (String url: signVideoURLs) {
							Element videoEl = doc.createElement("VIDEO");
							videoEl.setAttribute("URL", url);
							videosEl.appendChild(videoEl);
						}
						occurrenceEl.appendChild(videosEl);
						
						Element participantEl = doc.createElement("PARTICIPANT");
						participantEl.setTextContent(occurrence.getParticipant());
						occurrenceEl.appendChild(participantEl);
						Element signTypeEl = doc.createElement("SIGN_TYPE");
						signTypeEl.setTextContent(occurrence.getSignType());
						occurrenceEl.appendChild(signTypeEl);
						Element twoHandedEl = doc.createElement("TWO_HANDED");
						twoHandedEl.setTextContent(occurrence.getTwoHanded());
						occurrenceEl.appendChild(twoHandedEl);
						Element passiveBaseArmEl = doc.createElement("PASSIVE_BASE_ARM");
						passiveBaseArmEl.setTextContent(occurrence.getPassiveBaseArm());
						occurrenceEl.appendChild(passiveBaseArmEl);
						Element markedHandsEl = doc.createElement("MARKED_HANDS");
						markedHandsEl.setTextContent(occurrence.getMarkedHands());
						occurrenceEl.appendChild(markedHandsEl);
						Element leftRightHSEl = doc.createElement("LEFT_RIGHT_HS");
						leftRightHSEl.setTextContent(occurrence.getLeftRightHS());
						occurrenceEl.appendChild(leftRightHSEl);
						Element startEndHSEl = doc.createElement("START_END_HS");
						startEndHSEl.setTextContent(occurrence.getStartEndHS());
						occurrenceEl.appendChild(startEndHSEl);
						Element startEndLeftHSEl = doc.createElement("START_END_LEFT_HS");
						startEndLeftHSEl.setTextContent(occurrence.getStartEndLeftHS());
						occurrenceEl.appendChild(startEndLeftHSEl);
						Element startEndRightHSEl = doc.createElement("START_END_RIGHT_HS");
						startEndRightHSEl.setTextContent(occurrence.getStartEndRightHS());
						occurrenceEl.appendChild(startEndRightHSEl);
						
						occurrencesListEl.appendChild(occurrenceEl);
						
					}
				}
				
				if ( !matchingOccurrences.isEmpty()) {
					matchingVariants.add(variant);
					
					if (foundDefaultOccurrence) {
						// set the attr for default occ id on the variant, only if its in this list of matching occurrences
						variantEl.setAttribute("DEFAULT_OCCURRENCE_ID", variant.getDefaultOccurenceID());
					}
					
					Element variantLabelEl = doc.createElement("LABEL");
					variantLabelEl.setTextContent(variant.getVariantLabel());
					variantEl.appendChild(variantLabelEl);
					variantEl.appendChild(occurrencesListEl);
					
					variantsListEl.appendChild(variantEl);
					
				} else {
					continue;		// go to next variant and try again to find matching occurrences
				}
			}
			
			if ( !matchingVariants.isEmpty()) {
				matchingMainGlosses.add(mainGloss);
				
				if (foundDefaultVariant) {
					// set the attr for default var id on the main gloss
					mainGlossEl.setAttribute("DEFAULT_VARIANT_ID", defaultVariantID);
				}
				
				Element glossLabelEl = doc.createElement("LABEL");
				glossLabelEl.setTextContent(mainGloss.getMainGlossLabel());
				mainGlossEl.appendChild(glossLabelEl);
				mainGlossEl.appendChild(variantsListEl);
				
				mainGlossesListEl.appendChild(mainGlossEl);
				
			} else {
				continue;		// go to the next main gloss and try again to find matching variants
			}
		}
		
		if ( !matchingMainGlosses.isEmpty()) {
			root.appendChild(mainGlossesListEl);
		}
		
		// write it now
		boolean savedCoolection = false;
		try {
			savedCoolection = XMLWriter.saveDocument(root, resource.getSignBankAbsolutePath());
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Failed to save sign bank resource file: " + resourceFileName);
			return false;
		}
		
		if (!savedCoolection) {

			String[] str = new String[4];
			str[0] = "Sign Bank Resource " + resourceFileName
					+ " could not be saved. ";
			str[1] = "Please manually delete the directory and/or file";
			str[2] = "with the file name and try again.";
			str[3] = "";

			SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(str);
			return false;
		}
		
		return true;
	}
	
	// to be called only by CUMU
	public boolean writePrioritiesFile(/*what here*/) {
		return true;
	}
	
	// to be called only by CUMU
	public boolean createNewSIgnBank(SignBankResource resource) {
		return true;
	}

}
