package edu.bu.signstream.common.util.vo;

import java.util.ArrayList;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class TimeInfo {

   private int frameNumber = -1;
   private int movieTime = -1;
   private int timeBase = -1;
   private int timeScale = -1;
   
   private ArrayList<TimeInfo> dependentMovieTimes = new ArrayList<TimeInfo>();

   public TimeInfo() {}
   
   public TimeInfo(int frameNumber, int movieTime, int timeBase, int timeScale) {
	   this.frameNumber = frameNumber;
	   this.movieTime = movieTime;
	   this.timeBase = timeBase;
	   this.timeScale = timeScale;
	   
	   long packetDuration = -1;
	   
	   try {
		   packetDuration = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getMultipleMovieController().getPacketDuration();
	   } catch(NullPointerException ex) {
		   ;
	   }
	   
	   if(packetDuration > 0) {
		   this.frameNumber = (int)(movieTime / packetDuration);
		   this.movieTime = (int)(frameNumber * packetDuration);
	   }
   }
   
   public void addDependentMovieTime(TimeInfo timeInfo) {
	   dependentMovieTimes.add(timeInfo);
   }
   
   public void removeTimeInfo(TimeInfo timeInfo) {
	   dependentMovieTimes.remove(timeInfo);
   }
   
   public int getFrameNumber() {
      return frameNumber;
   }

   public int getMovieTime() {
      return movieTime;
      //return frameNumber;
   }

   public int getTimeBase() {
      return timeBase;
   }

   public int getTimeScale() {
      return timeScale;
   }

   public void setFrameNumber(int i) {
      frameNumber = i;
      updateDependentMovieTimes();
   }

   public void setMovieTime(int i) {
      movieTime = i;
      updateDependentMovieTimes();
   }

   public void setTimeBase(int i) {
      timeBase = i;
      updateDependentMovieTimes();
   }

   public void setTimeScale(int i) {
      timeScale = i;
      updateDependentMovieTimes();
   }
   
   private void updateDependentMovieTimes() {
	   for(TimeInfo dTimeInfo : dependentMovieTimes) {
		   dTimeInfo.setFrameNumber(frameNumber);
		   dTimeInfo.setMovieTime(movieTime);
		   dTimeInfo.setTimeBase(timeBase);
		   dTimeInfo.setTimeScale(timeScale);
	   }
   }
   
   public TimeInfo cloneTimeInfo() {
      TimeInfo info = new TimeInfo();
      info.setMovieTime(getMovieTime());
      info.setTimeBase(getTimeBase());
      info.setTimeScale(getTimeScale());
      return info;
   } 
   
   @Override
public String toString() {
      return "TimeInfo: frame number = " + frameNumber + ", movie time = " + movieTime;
   }
}
