package edu.bu.signstream.common.util.vo.ss3.codingScheme;

import java.util.*;

public class CSLanguage {
   public static final int LETTERS = 0;
   public static final int NUMBERS = 1;
   public static final int ALL = 2;

   // collection of HandShapeImgItem objects
   private ArrayList imgItems = new ArrayList();

   private String id = "";
   private String name = "";
   private String label = "";

   public CSLanguage() {}

   public void addImmageItems(ArrayList items) {
      this.imgItems.addAll(items);
   }

   public void addIImmageItem(Object object) {
      imgItems.add(object);
   }

   public int getNumberOfItems() {
      return imgItems.size();
   }
   
   

   /**
    * @returns list of HandShapeImgItem objects
    * */
   public ArrayList getImmageItems(
      int type,
      boolean unmarked,
      boolean fingerspelled) {
      
      ArrayList items = getImmageItemsWithType(type);


      if (unmarked) {
         items = unmarkedOnly(items);
         items.trimToSize();
      }

      if (fingerspelled) {
         items = fingerspelledOnly(items);
         items.trimToSize();
      }

      return items;
   }

   private ArrayList unmarkedOnly(ArrayList items) {
      ArrayList list = new ArrayList();
      
      for (int i = 0; i < items.size(); i++) {
         HandShapeImgItem item = (HandShapeImgItem) items.get(i);

         if (item.isUnmarked())
            list.add(item);
      }
      return list;
   }

   private ArrayList fingerspelledOnly(ArrayList items) {
      ArrayList list = new ArrayList();
      
      for (int i = 0; i < items.size(); i++) {
         HandShapeImgItem item = (HandShapeImgItem) items.get(i);

         if (item.isFingerspelled()) {
            list.add(item);
         }
      }
      return list;
   }

   private ArrayList getImmageItemsWithType(int type) {
      ArrayList list = new ArrayList();
      
      if (type == CSLanguage.ALL) {
         for (int j = 0; j < imgItems.size(); j++) {
            list.add(imgItems.get(j));
         }  
      } else {   
         for (int i = 0; i < imgItems.size(); i++) {
            HandShapeImgItem item = (HandShapeImgItem) imgItems.get(i);
            if (item.getType() == type) {
               list.add(item);
            }
         }
      }
      
      //
      System.err.println("-----> type = " + type + ", imgItems.size() = " + imgItems.size());
      //

      return list;
   }

   public String getId() {
      return id;
   }

   public String getLabel() {
      return label;
   }

   public String getName() {
      return name;
   }

   public void setId(String string) {
      id = string;
   }

   public void setLabel(String string) {
      label = string;
   }

   public void setName(String string) {
      name = string;
   }

   @Override
public String toString() {
      StringBuffer sb = new StringBuffer(name);
      sb.append(" (");
      sb.append(label);
      sb.append(")");
      return sb.toString();
   }

}
