package edu.bu.signstream.common.util.vo.ss3.db;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.tree.DefaultMutableTreeNode;

import edu.bu.signstream.common.db.SS3DatabaseWriter;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.Reference;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.note.SS3Note;
import edu.bu.signstream.common.util.vo.ss3.note.SS3NoteCollection;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEvent;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class SS3Database extends SS3Element {
	// contains name of the original file + _utteranceID in format 001
	private String fileName = "";
	private String directory = "";
	private String parentDirectoryPath = "SignStreamDatabases";
	private DefaultMutableTreeNode dbNavigationPanelNode;
	
	private int movieDuration = 0;
	private int packetDuration = 0;
	
	private double temporalPartitionPixelInTimeFrame = 0; 
	private int temporalPartitionMovieTime = 0;
	private int temporalPartitionOnScreenLocation = 0;

	public double getTemporalPartitionPixelInTimeFrame() {
		return temporalPartitionPixelInTimeFrame;
	}

	public void setTemporalPartitionPixelInTimeFrame(
			double temporalPartitionPixelInTimeFrame) {
		this.temporalPartitionPixelInTimeFrame = temporalPartitionPixelInTimeFrame;
	}

	public int getTemporalPartitionMovieTime() {
		return temporalPartitionMovieTime;
	}

	public void setTemporalPartitionMovieTime(int temporalPartitionMovieTime) {
		this.temporalPartitionMovieTime = temporalPartitionMovieTime;
	}

	public int getTemporalPartitionOnScreenLocation() {
		return temporalPartitionOnScreenLocation;
	}

	public void setTemporalPartitionOnScreenLocation(
			int temporalPartitionOnScreenLocation) {
		this.temporalPartitionOnScreenLocation = temporalPartitionOnScreenLocation;
	}

	private File readFromXMLFile = null;

	private String excerpt = "";
	private String number = "";

	private SS3DBProfile profile = new SS3DBProfile();

	private ArrayList<Reference> cSchemeRef = new ArrayList<Reference>();

	private ArrayList<SS3Participant> selectedParticipants = new ArrayList<SS3Participant>();
	private ArrayList<SS3Participant> selectedAnnotators = new ArrayList<SS3Participant>();

	private HashMap<String, SS3MediaFile> mFiles = new HashMap<String, SS3MediaFile>();

	private final SS3ElementCollection tracks = new SS3ElementCollection();
	private final SS3NoteCollection notes = new SS3NoteCollection();

	private final SS3ElementCollection selections = new SS3ElementCollection();
	
	private SS3GlossWindow glsWindow = new SS3GlossWindow();

	private SS3CodingScheme scheme = null;
	private SS3CodingScheme localScheme = null;

	private TimeInfo startTimeInfo = null;
	private TimeInfo endTimeInfo = null;

	private String participantName = null;
	
	private Excerpt excerptObj = null;

	public SS3Database(TimeInfo startTimeInfo, TimeInfo endTimeInfo) {
		this.startTimeInfo = startTimeInfo;
		this.endTimeInfo = endTimeInfo;
	}
	
	public Excerpt getExcerptObj() {
		if(excerptObj == null && excerpt != null && excerpt.length() == 0) {
			excerptObj = new Excerpt(number, excerpt);
		}
		return excerptObj;
	}
		
	public int getMovieDuration() {
		return movieDuration;
	}

	public void setMovieDuration(int movieDuration) {
		this.movieDuration = movieDuration;
	}

	public int getPacketDuration() {
		return packetDuration;
	}

	public void setPacketDuration(int packetDuration) {
		this.packetDuration = packetDuration;
	}
	
	public void setExcerptObj(Excerpt excerptObj) {
		this.excerptObj = excerptObj;
	}
	
	public SS3GlossWindow getSS3GlossWindow() {
		return glsWindow;
	}
	
	public void setSS3GlossWindow(SS3GlossWindow glsWindow) {
		this.glsWindow = glsWindow;
	}

	public boolean addSelectedParticipant(SS3Participant participant) {
		for (SS3Participant part: selectedParticipants) {
			if (part.getId().equals(participant.getId()))
					return false;
		}
		
		selectedParticipants.add(participant);
		return true;
	}

	public ArrayList<SS3Participant> getSelectedParticipants() {
		return selectedParticipants;
	}
	
	public SS3Participant getSelectedParticipant(String participantID) {
		for(SS3Participant participant : selectedParticipants) {
			if(participant.getId().equalsIgnoreCase(participantID))
				return participant;
		}
		return null;
	}
	
	public SS3Participant getSelectedAnnotator(String annotatorID) {
		for(SS3Participant annotator : selectedAnnotators) {
			if(annotator.getId().equalsIgnoreCase(annotatorID))
				return annotator;
		}
		return null;
	}
	
	public void setSelectedParticipants(ArrayList<SS3Participant> participants) {
		this.selectedParticipants = participants;
	}
	
	public void setSelectedAnnotators(ArrayList<SS3Participant> annotators) {
		this.selectedAnnotators = annotators;
	}

	public boolean addSelectedAnnotator(SS3Participant participant) {
		for (SS3Participant part: selectedAnnotators) {
			if (part.getId().equals(participant.getId()))
					return false;
		}
		
		selectedAnnotators.add(participant);
		return true;
	}

	public ArrayList<SS3Participant> getSelectedAnnotators() {
		return selectedAnnotators;
	}

	public String getParentDirectoryPath() {
		return parentDirectoryPath;
	}

	public void setParentDirectoryPath(String parentDirectoryPath) {
		this.parentDirectoryPath = parentDirectoryPath;
	}

	public DefaultMutableTreeNode getDbNavigationPanelNode() {
		return dbNavigationPanelNode;
	}

	public void setDbNavigationPanelNode(DefaultMutableTreeNode dbNavigationPanelNode) {
		this.dbNavigationPanelNode = dbNavigationPanelNode;
	}

	public void setStartTimeInfo(TimeInfo startTimeInfo) {
		this.startTimeInfo = startTimeInfo;
	}

	public TimeInfo getStartTimeInfo() {
		if (startTimeInfo.getMovieTime() >= 0)
			return startTimeInfo;

		if (glsWindow == null && glsWindow.getTemporalPartitions().isEmpty()) {
			startTimeInfo.setMovieTime(0);
		} else {
			int movieTime = -1;
			ArrayList<SS3TemporalPartition> partitions = glsWindow.getTemporalPartitions();

			for (SS3TemporalPartition partition : partitions) {
				ArrayList<SS3GlossWindowSegment> partitionSegments = partition.getTemporalPartitionSegments();
				for (SS3GlossWindowSegment partitionSegment : partitionSegments) {
					ArrayList<SS3GlossWindowItem> items = partitionSegment.getSS3GlossWindowItems();
					for (SS3GlossWindowItem item : items) {
						int newMovieTime = item.getStartTimeInfo().getMovieTime();
						if (movieTime == -1)
							movieTime = newMovieTime;

						if (newMovieTime >= 0 && newMovieTime < movieTime)
							movieTime = newMovieTime;
					}
				}
			}
			startTimeInfo.setMovieTime(movieTime);
		}

		return startTimeInfo;
	}

	public void setEndTimeInfo(TimeInfo endTimeInfo) {
		this.endTimeInfo = endTimeInfo;
	}

	public TimeInfo getEndTimeInfo() {
		if (endTimeInfo.getMovieTime() >= 0)
			return endTimeInfo;

		if (glsWindow == null && glsWindow.getTemporalPartitions().isEmpty()) {
			endTimeInfo.setMovieTime(0);
		} else {
			int movieTime = -1;
			ArrayList<SS3TemporalPartition> partitions = glsWindow.getTemporalPartitions();

			for (SS3TemporalPartition partition : partitions) {
				ArrayList<SS3GlossWindowSegment> partitionSegments = partition.getTemporalPartitionSegments();
				for (SS3GlossWindowSegment partitionSegment : partitionSegments) {
					ArrayList<SS3GlossWindowItem> items = partitionSegment.getSS3GlossWindowItems();
					for (SS3GlossWindowItem item : items) {
						int newMovieTime = item.getEndTimeInfo().getMovieTime();

						if (movieTime == -1)
							movieTime = newMovieTime;

						if (newMovieTime >= 0 && newMovieTime > movieTime)
							movieTime = newMovieTime;
					}
				}
			}
			endTimeInfo.setMovieTime(movieTime);
		}
		return endTimeInfo;
	}

	public String getDirectory() {
		return directory;
	}

	public void setDirectory(String directory) {
		this.directory = directory;
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	public String getFileName() {
		return fileName;
	}

	public void addNote(SS3Note note) {
		notes.addNote(note);
	}

	public ArrayList<SS3Note> getNotesList() {
		return notes.getNotesList();
	}

	public SS3Note getNote(String noteID) {
		return notes.getNote(noteID);
	}

	public void addStatement(SS3GlossWindowItem statement) {
	}

	public void addSelection(SS3Selection selection) {
		selections.addElement(selection);
	}

	public void addTrack(SS3Track track) {
		tracks.addElement(track);
	}

	public ArrayList<Object> getTracksList() {
		return tracks.getObjectsList();
	}

	public SS3Track getTrack(String trackID) {		
		Object obj = tracks.getElement(trackID);
		if (obj != null) {
			return (SS3Track) obj;
		} else {
			SS3Track track = new SS3Track();
			track.setId(trackID);
			tracks.addElement(track);
			return track;
		}
	}

	public SS3Track getTrack(SS3StatementField field, 
			String fieldID, 
			int startMovieTime, int endMovieTime) {

		if(field != null) {
			fieldID = field.getId();
		}
		
		SS3Track newTrack = new SS3Track();
		newTrack.setId(fieldID);
		
		SS3Track track = getTrack(fieldID);

		if (field == null || track == null)
			return newTrack;

		ArrayList<SS3Entity> ss3Entities = field.getSS3Entities();
		ArrayList<String> entIDs = new ArrayList<String>();
		for (SS3Entity ss3Entity: ss3Entities) {
			entIDs.add(ss3Entity.getID());
		}
		for(String entID : entIDs) {
			SS3Entity entity = track.getSS3Entity(entID);
			
			if(entity == null) {
				System.out.println("HERE!!! entID = " + entID + ", fieldID = " + fieldID);
			} else {
			
			if(fieldID.equalsIgnoreCase("30000")) {
				newTrack.addEntity(entity);
			} else if(startMovieTime <= entity.getStartTime() && endMovieTime >= entity.getEndTime()) {
				newTrack.addEntity(entity);
			 }
			}
		}

		return newTrack;
	}
	
	public SS3Track getTrack(String fieldID, ArrayList<SS3GlossWindowItem> items) {
		
		SS3Track newTrack = new SS3Track();
		newTrack.setId(fieldID);
		
		if (items.isEmpty())
			return newTrack;
		
		int itemsSize = items.size();
		
		SS3GlossWindowItem firstItem = items.get(0);
		SS3GlossWindowItem lastItem = items.get(itemsSize - 1);
		// first and last items could be the same reference if itemsSize == 1
		
		int startTime = firstItem.getStartTimeInfo().getMovieTime();
		int endTime = lastItem.getEndTimeInfo().getMovieTime();
		
		ArrayList<SS3StatementField> statementFields = new ArrayList<SS3StatementField>();
		ArrayList<PresentationEvent> presentationEvents = new ArrayList<PresentationEvent>();
		for (SS3GlossWindowItem item: items) {
			if (item.getSS3StatementField(fieldID) != null) {
				statementFields.add(item.getSS3StatementField(fieldID));
				presentationEvents.add(item.getEvent());
			}
		}

		for (int i = 0; i < statementFields.size(); i++) {
			SS3StatementField statementField = statementFields.get(i);
			PresentationEvent presentationEvent = presentationEvents.get(i);
			
			ArrayList<SS3Entity> ss3Entities = statementField.getSS3Entities();
			
			for (SS3Entity ss3Entity: ss3Entities) {
				if (startTime <= ss3Entity.getStartTime() && ss3Entity.getEndTime() <= endTime) {
					ss3Entity.setParentEntityID(presentationEvent.getID());
					newTrack.addEntity(ss3Entity);
				}
			}
		}
		
		return newTrack;
	}

	public SS3TracksCollection getTracks(SS3GlossWindowItem statement) {
		SS3TracksCollection tracks = new SS3TracksCollection();
		ArrayList<SS3StatementField> fields = statement.getStatementFields();

		int startTime = statement.getStartTimeInfo().getMovieTime();
		int endTime = statement.getEndTimeInfo().getMovieTime();

		for (SS3StatementField field : fields) {
			SS3Track newTrack = new SS3Track();
			newTrack.setId(field.getId());
			SS3Track track = getTrack(field.getId());
			
			ArrayList<SS3Entity> ss3Entities = field.getSS3Entities();
			ArrayList<String> entIDs = new ArrayList<String>();
			for (SS3Entity ss3Entity: ss3Entities) {
				entIDs.add(ss3Entity.getID());
			}

			if (entIDs.size() > 0) {
				for (String id : entIDs) {
					SS3Entity ent = track.getSS3Entity(id);
					if (ent.getStartTime() >= startTime && ent.getEndTime() <= endTime)
						newTrack.addEntity(ent);
				}
			} else {
				ArrayList<SS3Entity> entities = track.getEntities();
				for (SS3Entity entity:entities) {
					if (entity.getStartTime() >= startTime && entity.getEndTime() <= endTime)
						newTrack.addEntity(entity);
				}
			}
			tracks.addTrack(newTrack);
		}
		return tracks;
	}

	public ArrayList<Reference> getCodingSchemeRef() {
		return cSchemeRef;
	}

	public void setCodingSchemeRef(ArrayList<Reference> list) {
		cSchemeRef = list;
	}

	public HashMap<String, SS3MediaFile> getMediaFiles() {
		return mFiles;
	}

	public void setMediaFiles(HashMap<String, SS3MediaFile> map) {
		mFiles = map;
	}

	public void addMediaFile(SS3MediaFile mFile) {
		mFiles.put(mFile.getId(), mFile);
	}

	public SS3MediaFile getMediaFile(String id) {
		return mFiles.get(id);
	}

	public ArrayList<SS3MediaFile> getMediaFilesCollection() {
		ArrayList<SS3MediaFile> list = new ArrayList<SS3MediaFile>();
		Set<Map.Entry<String, SS3MediaFile>> set = mFiles.entrySet();
        for (Map.Entry<String, SS3MediaFile> entry : set) {
            list.add(entry.getValue());
        }
		return list;
	}

	public SS3DBProfile getProfile() {
		return profile;
	}

	public void deleteFile() {
		SS3DatabaseWriter dbWriter = new SS3DatabaseWriter();
		dbWriter.deleteDatabase(this);
	}

	public boolean save() throws Exception {
		this.movieDuration = SS3Singleton.getVideoControlManager().getMultipleMovieController().getMovieDuration();		
		this.packetDuration = (int)SS3Singleton.getVideoControlManager().getMultipleMovieController().getPacketDuration();	
		SS3DatabaseWriter dbWriter = new SS3DatabaseWriter();
		return dbWriter.saveDocument(this);
	}

	public void setSS3CodingScheme(SS3CodingScheme scheme) {
		this.scheme = scheme;
	}

	public SS3CodingScheme getSS3CodingScheme() {
		return scheme;
	}

	public void setLocalSS3CodingScheme(SS3CodingScheme scheme) {
		this.localScheme = scheme;
	}

	public SS3CodingScheme getLocalSS3CodingScheme() {
		if(localScheme == null) {
			localScheme = new SS3CodingScheme();
		}
		return localScheme;
	}

	@Override
	public String toString() {
		return super.toString();
	}

	public String getShortDescription() {
		return "S3Database " + excerpt + " has [" + tracks.size() + "] fields.";
	}

	public String getExcerpt() {
		return excerpt;
	}

	public void setExcerpt(String string) {
		excerpt = string;
	}

	public File getReadFromXMLFile() {
		return readFromXMLFile;
	}

	public void setReadFromXMLFile(File file) {
		readFromXMLFile = file;
	}

	public String getParticipantName() {
		return participantName;
	}

	public void setParticipantName(String string) {
		participantName = string;
	}

	public String getNumber() {
		return number;
	}

	public void setNumber(String string) {
		number = string;
	}
}