package edu.bu.signstream.common.util.vo.ss3.db;

import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class SS3DatabaseCollection {

	// key ===> Excerpt, value ===> S3Database
	private final HashMap<Excerpt, SS3Database> databases = new HashMap<Excerpt, SS3Database>();

	public void addSS3SDatabase(SS3Database database) {
		Excerpt excerpt = database.getExcerptObj();
		databases.put(excerpt, database);
	}

	public HashMap<Excerpt, SS3Database> getDatabases() {
		return databases;
	}

	public boolean save() {
		Collection<SS3Database> collection = databases.values();

		if (!collection.isEmpty()) {
			Iterator<SS3Database> iter = collection.iterator();
			while (iter.hasNext()) {
				SS3Database db = iter.next();

				try {
					boolean save = db.save();
					if(!save) {
						return false;
					}
				} catch (Exception e) {
					String msg = "Warning: " + e.getMessage();
					Object[] options = { "OK" };
					String errorMsg = "Warning: The Collection had not been saved /n."
							+ e.getMessage();

					if (msg.startsWith("Could not create a directory")) {
						StringBuffer sb = new StringBuffer("");
						sb.append("Warning: Please rename a colection and save. /n");
						sb.append("Use any character for a name, except for the following: /n");
						sb.append("- reserved characters: < > : \" / \\| ? * /n");
						sb.append("- integer value zero /n");
						sb.append("- Do not use the following reserved device names: /n");
						sb.append("CON, PRN, AUX, NUL, COM1, COM2, COM3, COM4, COM5, /n");
						sb.append("COM6, COM7, COM8, COM9, LPT1, LPT2, LPT3, LPT4, /n");
						sb.append("LPT5, LPT6, LPT7, LPT8, and LPT9");
						errorMsg = sb.toString();
					} else {
						errorMsg = "Warning: " + e.getMessage();
					}

					SS3Singleton.getSignStreamApplication()
							.confirmationDialog("Save Resource", msg);
					
					return false;
				}
			}
		}
		return true;
	}

	public boolean isSS3DatabaseLoaded(File file) {
		Collection<SS3Database> collection = databases.values();

		if (collection.isEmpty())
			return false;

		Iterator<SS3Database> iter = collection.iterator();
		while (iter.hasNext()) {
			SS3Database db = iter.next();

			if (file.getParentFile().getAbsolutePath()
					.equals(db.getParentDirectoryPath())
					&& file.getName().equals(db.getDirectory())) {
				return true;
			}
		}
		return false;
	}

	public SS3Database getSS3Database(Excerpt excerpt) {
		return databases.get(excerpt);
	}

	public void deleteSS3Database(Excerpt excerpt) {
		databases.remove(excerpt);
	}

	public void removeAll() {
		databases.clear();
	}

	public int getSize() {
		return databases.size();
	}

	public Excerpt getExcerpt(int excerptNumber) {
		Iterator<Excerpt> iter = databases.keySet().iterator();
		while (iter.hasNext()) {
			Excerpt excerpt = iter.next();

			if (Integer.parseInt(excerpt.getID()) == excerptNumber) {
				return excerpt;
			}
		}
		return null;
	}
}
