package edu.bu.signstream.common.util.vo.ss3.db;

import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEvent;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class SS3GlossWindowItem extends SS3Element {
	
	private TimeInfo startTimeInfo;
	private TimeInfo endTimeInfo;
	private String label = "";
	private String noteID = "";
	private ArrayList<String> participantIDs = new ArrayList<String>();
	private ArrayList<String> anotatorIDs = new ArrayList<String>();

	private String selectedParticipantID = null;
	private String selectedAnnotatorID = null;
	private SS3Participant selectedAnnotator = null;
	private SS3Participant selectedParticipant = null;
	
	private String partitionLabel = null;
	private String segmentLabel = null;
	private String utteranceLabel = "";

	private PresentationEvent event = null;

	// Collection of SS3StatementField objects
	private ArrayList<SS3StatementField> fields = new ArrayList<SS3StatementField>();

	// Segment level items. Must contain SS3StatementField collection. 
	// Must be added to Wrappers and updated in updateDatabase method.
	private ArrayList<SS3GlossWindowItem> segmentItems = new ArrayList<SS3GlossWindowItem>();
	
	//private ArrayList<SS3GlossWindowItemNote> notes = new ArrayList<SS3GlossWindowItemNote>();

	public SS3GlossWindowItem(String partitionLabel, String segmentLabel) {
		this.partitionLabel = partitionLabel;
		this.segmentLabel = segmentLabel;
		this.startTimeInfo = new TimeInfo();
		this.endTimeInfo = new TimeInfo();
		
		SS3Database db = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database();
		
		String currentAnnotator = SS3Singleton.getCurrentAnnotator(); 
		SS3Participant selectedAnnotator = null;
		ArrayList<SS3Participant> selectedAnnotators = db.getSelectedAnnotators();
		for (SS3Participant annotator: selectedAnnotators) {
			if (annotator.getLabel().equals(currentAnnotator)) {
				selectedAnnotator = annotator;
			}
		}
		this.setSelectedAnnotator(selectedAnnotator);
		
		SS3Participant selectedParticipant = null;		
		SS3TemporalPartition partition = db.getSS3GlossWindow().getTemporalPartitionByLabel(partitionLabel);		
		if (partition != null) {
			SS3GlossWindowSegment segment = partition.getTemporalPartitionSegmentByLabel(segmentLabel);
			if (segment != null) {
				selectedParticipant = segment.getParticipant();
			}
		}
		this.setSelectedParticipant(selectedParticipant);
	}
	
	public SS3Participant getSelectedAnotator() {
		return selectedAnnotator;
	}
	
	public SS3Participant getSelectedParticipant() {
		return selectedParticipant;
	}
	
	public void setSelectedParticipant(SS3Participant selectedParticipant) {
		this.selectedParticipant = selectedParticipant;
		if(selectedParticipant != null) {
			this.selectedParticipantID = selectedParticipant.getId();
		}
	}
	
	public void setSelectedAnnotator(SS3Participant selectedAnnotators) {
		this.selectedAnnotator = selectedAnnotators;
		if(selectedAnnotator != null) {
			this.selectedAnnotatorID = selectedAnnotators.getId();
		}
	}

	public String getSelectedParticipantID() {
		return selectedParticipantID;
	}

	public void setSelectedParticipantID(String selectedParticipant) {
		this.selectedParticipantID = selectedParticipant;
	}

	public String getSelectedAnnotatorID() {
		return selectedAnnotatorID;
	}

	public void setSelectedAnnotatorID(String selectedAnnotator) {
		this.selectedAnnotatorID = selectedAnnotator;
	}

	public void addSS3GlossWindowItem(SS3GlossWindowItem item) {
		segmentItems.add(item);
	}

	public SS3GlossWindowItem getSS3GlossWindowItem(String id) {
		for (SS3GlossWindowItem item : segmentItems) {
			if (item.getId().equalsIgnoreCase(id))
				return item;
		}
		return null;
	}

	public ArrayList<SS3GlossWindowItem> getSS3GlossWindowItems() {
		return segmentItems;
	}
	
	public PresentationEvent getEvent() {
		return event;
	}

	public void setEvent(PresentationEvent event) {
		this.event = event;
	}

	public void synchronize() {
		if (event != null) {
			this.startTimeInfo.setMovieTime(event.getStartTimeInfo()
					.getMovieTime());
			this.endTimeInfo
					.setMovieTime(event.getEndTimeInfo().getMovieTime());
			this.setId(event.getID());
			this.setLabel(event.getText());
		}
	}

	public void setUtteranceLabel(String utteranceLabel) {
		this.utteranceLabel = utteranceLabel;
	}

	public String getUtteranceLabel() {
		return utteranceLabel;
	}

	public void setPartitionLabel(String partitionLabel) {
		this.partitionLabel = partitionLabel;
	}

	public String getPartitionLabel() {
		return partitionLabel;
	}

	public void setSegmentLabel(String segmentLabel) {
		this.segmentLabel = segmentLabel;
	}

	public String getSegmentLabel() {
		return segmentLabel;
	}

	@Override
	public SS3GlossWindowItem clone() {
		SS3GlossWindowItem item = new SS3GlossWindowItem(partitionLabel,
				segmentLabel);
		String label1 = label;
		item.setLabel(label1);
		item.setNoteID(noteID);
		item.setStartTimeInfo(startTimeInfo.cloneTimeInfo());
		item.setEndTimeInfo(endTimeInfo.cloneTimeInfo());
		item.setSelectedAnnotator(selectedAnnotator);
		item.setSelectedParticipant(selectedParticipant);

		ArrayList<String> clonedParticipants = new ArrayList<String>();
		for (String participant : participantIDs) {
			clonedParticipants.add(participant);
		}

		ArrayList<String> clonedCoders = new ArrayList<String>();
		for (String coder : anotatorIDs) {
			clonedCoders.add(coder);
		}

		ArrayList<SS3StatementField> statementFields = new ArrayList<SS3StatementField>();
		for (SS3StatementField field : fields) {
			statementFields.add(field);
		}

		item.setCoders(clonedCoders);
		item.setParticipants(clonedParticipants);
		item.setFields(statementFields);

		return item;
	}

	public String getNoteID() {
		return noteID;
	}

	public void setNoteID(String noteID) {
		this.noteID = noteID;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String string) {
		label = string;
	}

	public void setParticipants(ArrayList<String> participants) {
		this.participantIDs = participants;
	}

	public void addParticipant(String participantID) {
		participantIDs.add(participantID);
	}

	public void setCoders(ArrayList<String> coders) {
		this.anotatorIDs = coders;
	}

	public void setFields(ArrayList<SS3StatementField> fields) {
		this.fields = fields;
	}

	public void addField(SS3StatementField field) {
		fields.add(field);
	}

	public void removeAllFields() {
		fields.clear();
	}

	public ArrayList<String> getParticipants() {
		return participantIDs;
	}

	public ArrayList<String> setCoders() {
		return anotatorIDs;
	}

	public ArrayList<SS3StatementField> getStatementFields() {
		return fields;
	}

	public SS3StatementField getSS3StatementField(String fieldID) {
		for (int i = 0; i < fields.size(); i++) {
			SS3StatementField field = fields.get(i);
			if (fieldID.equalsIgnoreCase(field.getId())) {
				return field;
			}
		}
		return null;
	}

	public TimeInfo getEndTimeInfo() {
		return endTimeInfo;
	}

	public TimeInfo getStartTimeInfo() {
		return startTimeInfo;
	}

	public void setEndTimeInfo(TimeInfo info) {
		endTimeInfo = info;
	}

	public void setStartTimeInfo(TimeInfo info) {
		startTimeInfo = info;
	}
	
	public int getMaximalStartTime() {
		int maximalStartTime = Integer.MAX_VALUE;
		
		for (SS3StatementField field: fields) {
//			if (field.getId().equalsIgnoreCase(Util.FREE_TEXT_FIELD_ID))
//				continue;
			for (SS3Entity entity: field.getSS3Entities()) {
				int entityStartTime = entity.getStartTime();
				if (entityStartTime != -1 && entityStartTime < maximalStartTime)
					maximalStartTime = entityStartTime;
			}
		}
		
		return maximalStartTime;
	}
	
	public int getMinimalEndTime() {
		int minimalEndTime = Integer.MIN_VALUE;
		
		for (SS3StatementField field: fields) {
//			if (field.getId().equalsIgnoreCase(Util.FREE_TEXT_FIELD_ID))
//				continue;
			for (SS3Entity entity: field.getSS3Entities()) {
				int entityEndTime = entity.getEndTime();
				if (entityEndTime != -1 && entityEndTime > minimalEndTime)
					minimalEndTime = entityEndTime;
			}
		}
		
		return minimalEndTime;
	}
	
//	public void setTranslationFieldEntityStartTime(int startTime) {
//		SS3StatementField translationStatementField = getSS3StatementField(Util.FREE_TEXT_FIELD_ID);
//		int translationEntitiesSize = translationStatementField.getSS3Entities().size();
//		if (translationEntitiesSize != 1) {
//			// i dont know here
//		} else {
//			SS3Entity translationEntity = translationStatementField.getSS3Entities().get(0);
//			SS3Item translationItem = translationEntity.getItems().get(0);
//			TimeInfo clonedStartTimeInfo = translationItem.getStartTimeInfo().cloneTimeInfo();
//			clonedStartTimeInfo.setMovieTime(startTime);
//			translationItem.setStartTimeInfo(clonedStartTimeInfo);
//		}
//	}
//	
//	public void setTranslationFieldEntityEndTime(int endTime) {
//		SS3StatementField translationStatementField = getSS3StatementField(Util.FREE_TEXT_FIELD_ID);
//		int translationEntitiesSize = translationStatementField.getSS3Entities().size();
//		if (translationEntitiesSize != 1) {
//			// i dont know here
//		} else {
//			SS3Entity translationEntity = translationStatementField.getSS3Entities().get(0);
//			SS3Item translationItem = translationEntity.getItems().get(0);
//			TimeInfo clonedEndTimeInfo = translationItem.getEndTimeInfo().cloneTimeInfo();
//			clonedEndTimeInfo.setMovieTime(endTime);
//			translationItem.setEndTimeInfo(clonedEndTimeInfo);
//		}
//	}

	@Override
	public String toString() {
		return label;
	}
}
