package edu.bu.signstream.grepresentation.fields;

import java.awt.*;
import java.util.*;

import edu.bu.signstream.grepresentation.fields.nonManualField.*;

public class FieldEntitiesCollection {
	private ArrayList collection = new ArrayList();

	public FieldEntitiesCollection() {
	}

	public void addTrackEntity(NonManualField entity) {
		collection.add(entity);
		sortTracks();
	}

	public void addTrackEntities(ArrayList entities) {
		for (int i = 0; i < entities.size(); i++) {
			collection.add(entities.get(i));
		}
		sortTracks();
	}

	public int getStartTime() {
		int startTime = 0;
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			startTime =
				startTime < track.getStartTime()
					? startTime
					: track.getStartTime();
		}
		return startTime;
	}

	public int getEndTime() {
		int endTime = 0;
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			endTime =
				endTime > track.getEndTime() ? endTime : track.getEndTime();
		}
		return endTime;
	}

	public int getStartTimeCoordinate() {
		int startTimeCoord = 0;
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			startTimeCoord =
				startTimeCoord < track.getStartTimeCoordinate()
					? startTimeCoord
					: track.getStartTimeCoordinate();
		}
		return startTimeCoord;
	}

	public int getEndTimeCoordinate() {
		int endTimeCoord = 0;
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			endTimeCoord =
				endTimeCoord > track.getEndTimeCoordinate()
					? endTimeCoord
					: track.getEndTimeCoordinate();
		}
		return endTimeCoord;
	}

	public void selectEntity(Point point) {
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			if (track.contains(point.y))
				track.selectEntity(point.x);
		}
	}
	
	public NonManualField getSelectedTrackEntity() {
		for (int i = 0; i < collection.size(); i++) { 
			NonManualField track = (NonManualField) collection.get(i);
			
			if(track.getSelectedEvent() != null)
				return track;		
		}
		return null;
	}

	public void unselect() {
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			track.unselect();
		}
	}

	public void reset() {
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			track.reset();
		}
	}

//	public void addNewChainedEvent(Point point) {
//		for (int i = 0; i < collection.size(); i++) {
//			NonManualField track = (NonManualField) collection.get(i);
//
//			if (track.contains(point.y))
//				track.addNewChainedEvent(point.x);
//		}
//	}

	public Event getSelectedEvent() {
		Event event = null;
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			if ((event = track.getSelectedEvent()) != null)
				return event;
		}
		return null;
	}

	public ChainedEventsEntity getSelectedEntity() {
		ChainedEventsEntity entity = null;
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			if ((entity = (ChainedEventsEntity)track.getSelectedEntity()) != null)
				return entity;
		}
		return null;
	}

	public int getSize() {
		return collection.size();
	}

	public ArrayList getTracks() {
		return collection;
	}

	public boolean canDrawNewEvent(Point point) {
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);

			if (track.contains(point.y))
				if (track.canDrawNewEvent(point.x))
					return true;
		}
		return false;
	}

	// throw Exception!!!
	public NonManualField getTrackEntityAt(int y) {
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);

			if (track.contains(y))
				return track;
		}
		return null;
	}

	public NewChainedEvent getNewChainedEvent() {
		NewChainedEvent newEvent = null;
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			if ((newEvent = track.getNewChainedEvent()) != null)
				return newEvent;
		}
		return newEvent;
	}

	/*	
		public ArrayList getEventsToAdd(Point point) {
			TrackEntity entity = getTrackEntityAt(point);
			return entity.getEventsToAdd(point.x);
		}
	*/
	public boolean contains(int y) {
		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			if (track.contains(y))
				return true;
		}
		return false;
	}

	private void sortTracks() {
		int keys[] = new int[collection.size()];
		HashMap map = new HashMap();

		for (int i = 0; i < collection.size(); i++) {
			NonManualField track = (NonManualField) collection.get(i);
			String fieldID = track.getTrack().getFieldId();

			map.put(fieldID, track);
			keys[i] = Integer.parseInt(fieldID);
		}

		Arrays.sort(keys);

		collection.removeAll(collection);
		for (int j = 0; j < keys.length; j++) {
			collection.add(map.get("" + keys[j]));
		}
	}
}
