package edu.bu.signstream.grepresentation.fields;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.StringTokenizer;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.glossField.Compoundable;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredWord;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.Text;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEvent;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeSubEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapesImageManager;
import edu.bu.signstream.grepresentation.fields.handShapeField.NewHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapes.joints.JointsEvent;
import edu.bu.signstream.grepresentation.fields.handShapes.joints.JointsEventsEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEvent;
import edu.bu.signstream.grepresentation.fields.movementField.MovementEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.NewChainedEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class Painter {

	private Color entitySelectedColor = Color.GREEN;
	private Color eventSelectedColor = Color.RED;
	private Color eventHighlightedColor = Color.BLUE;
	private Color onsetUnselectedColor = new Color(14, 140, 58);
	private Color offsetUnselectedColor = Color.BLUE;

	public Painter() {
	}

	public void drawNewEvent(NewChainedEvent event, Graphics2D g2d, double Y) {
		int X1 = event.getStartTimeCoordinate();
		int X2 = event.getEndTimeCoordinate();

		g2d.fillOval(X1, (int) Y - 5, 4, 4);
		if (X2 > -1)
			g2d.draw(new Line2D.Double(X1, Y - 5, X2, Y - 5));
	}

	public void drawEnteredWord(EnteredWord word, Graphics2D g2d, double Y,
			FontMetrics fontMetrics) {
		g2d.setColor(Color.darkGray);
		g2d.drawString(word.getText(), word.getStartCoord(), (int) Y - 1);
	}
	
	public void drawEnteredGlossText(ArrayList<Text> textObjects,
			Graphics2D g2d, double Y, FontMetrics fontMetrics,
			GlossField glsField) {

		g2d.setFont(fontMetrics.getFont());

		for (Text text : textObjects) {
			if (text.isSelected()) {
				g2d.setColor(Color.RED);
			} else {
				g2d.setColor(Color.DARK_GRAY);
			}

			int endCoord = text.getStartCoord() + fontMetrics.stringWidth(text.getText());
			text.setEndCoord(endCoord);
			g2d.drawString(text.getText(), text.getStartCoord(), (int) Y - 1);
		}
	}
// @ TODO:  
	public void drawEnteredGlossTextNew(ArrayList<Text> textObjects,
			Graphics2D g2d, double Y, FontMetrics fontMetrics,
			GlossField glsField) {

		g2d.setFont(fontMetrics.getFont());

		for (Text text : textObjects) {
			if (text.isSelected()) {
				g2d.setColor(Color.RED);
			} else {
				g2d.setColor(Color.DARK_GRAY);
			}
			
			int textStartCoord = text.getStartCoord();
			
			int textWidth = fontMetrics.stringWidth(text.getText());

			int nextEventStartCoord = glsField
					.getNextEntityOrTextStartCoord(text.getStartCoord());			
			
			if(nextEventStartCoord < 0) {
				int endCoord = text.getStartCoord() + fontMetrics.stringWidth(text.getText());
				text.setEndCoord(endCoord);
				g2d.drawString(text.getText(), text.getStartCoord(), (int) Y - 1);
				return;
			}
			
			int availableWidth = nextEventStartCoord - textStartCoord;
//			
			System.out.println("textStartCoord = " + textStartCoord + 
					", textWidth = " + textWidth + ", nextEventStartCoord = " + nextEventStartCoord + 
					", availableWidth = " + availableWidth);
//			
			if(availableWidth < textWidth) {
				//
				System.err.println("availableWidth = " + availableWidth);
				//
				
				int prevEventStartCoord = glsField.getPrevEntityOrTextStartCoord(text.getStartCoord());
				
				if(prevEventStartCoord < text.getStartCoord()) {
					int newStartCoord = prevEventStartCoord + 5;
					text.setStartCoord(newStartCoord);
					
					//if() {
						
					//}
					
				} else {				
					String shortTxt = "&";
					int endCoord = text.getStartCoord() + fontMetrics.stringWidth(shortTxt);
					int endShortCoord = text.getStartCoord() + endCoord;
					text.setEndCoord(endShortCoord);
					g2d.drawString(shortTxt, text.getStartCoord(), (int) Y - 1);
				}

			} else {
				int endCoord = text.getStartCoord() + fontMetrics.stringWidth(text.getText());
				text.setEndCoord(endCoord);
				g2d.drawString(text.getText(), text.getStartCoord(), (int) Y - 1);
			}
		}
	}

	public void drawGlossEntity(GlossChainedEventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		Field field = entity.getField();

		ChainedEventsCollection collection = entity
				.getChainedEventsCollection();
		ChainedEvent onset = collection.getOnsetEvent();
		ChainedEvent initialHold = collection.getInitialHoldEvent();
		ChainedEvent value = collection.getValueEvent();
		GlossChainedEvent glossValue = collection.getTextValueEvent();
		ChainedEvent finalHold = collection.getFinalHoldEvent();
		ChainedEvent offset = collection.getOffsetEvent();

		boolean selected = entity.isSelected();

		// paint compounds
		if (entity.isStartCompound()) {
			Compoundable cmp = entity.getStartCompound();
			int coord = cmp.getEndTimeCoordinate();

			int x = coord + (entity.getStartTimeCoordinate() - coord) / 2;
            g2d.setColor(entity.getField().getColor());
			g2d.drawString("+", x - 4, (int) Y);
		}

		if (onset != null) {
			g2d.setColor(getHoldColor(onset, selected));
			drawOnset(onset, g2d, Y);
		}

		if (initialHold != null) {
			g2d.setColor(getHoldColor(initialHold, selected));
			drawInitialHold(initialHold, g2d, Y);
		}

		if (glossValue != null) {

			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				g2d.setColor(entity.getField().getColor());
			} else {
				g2d.setColor(getValueColor(glossValue, selected, entity
						.isHighlighted(), fieldID, field));
			}
			drawTextValue(entity, glossValue, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);

			boolean isLockedLeftRightAlignments = entity
					.isLockedLeftRightAlignments();
			// boolean isLockedGlossPhonAlignments =
			// entity.isLockedGlossPhonAlignments();
			// drawLock(g2d, glossValue.getStartTimeCoordinate(), (int) Y,
			// Color.BLUE, isLockedLeftRightAlignments,
			// isLockedGlossPhonAlignments);
			// drawLock(g2d, glossValue.getEndTimeCoordinate(), (int) Y,
			// Color.BLUE, isLockedLeftRightAlignments,
			// isLockedGlossPhonAlignments);

			drawLock(g2d, glossValue.getStartTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedLeftRightAlignments);
			drawLock(g2d, glossValue.getEndTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedLeftRightAlignments);
		}

		if (value != null) {
			Color color = null;
			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				color = entity.getField().getColor();
			} else {
				color = getValueColor(value, selected, entity.isHighlighted(),
						fieldID, field);
			}
			g2d.setColor(color);
			drawValue(value, g2d, Y);
			drawValueLabel(value, entity, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);

			boolean isLockedLeftRightAlignments = entity
					.isLockedLeftRightAlignments();
			boolean isLockedGlossPhonAlignments = entity
					.isLockedGlossPhonAlignments();

			drawLock(g2d, glossValue.getStartTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedLeftRightAlignments,
					isLockedGlossPhonAlignments);
			drawLock(g2d, glossValue.getEndTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedLeftRightAlignments,
					isLockedGlossPhonAlignments);

			// if (entity.isLockedLeftRightAlignments()) {
			// drawLock(g2d, value.getStartTimeCoordinate(), (int) Y,
			// eventHighlightedColor);
			// drawLock(g2d, value.getEndTimeCoordinate(), (int) Y,
			// eventHighlightedColor);
			// }
		}

		if (finalHold != null) {
			g2d.setColor(getHoldColor(finalHold, selected));
			drawFinalHold(finalHold, g2d, Y);
		}

		if (offset != null) {
			g2d.setColor(getHoldColor(offset, selected));
			drawOffset(offset, g2d, Y);
		}
	}

	public void drawEntity(ChainedEventsEntity entity, Graphics2D g2d,
			double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		Field field = entity.getField();

		ChainedEventsCollection collection = entity
				.getChainedEventsCollection();
		ChainedEvent onset = collection.getOnsetEvent();
		ChainedEvent initialHold = collection.getInitialHoldEvent();
		ChainedEvent value = collection.getValueEvent();
		GlossChainedEvent glossValue = collection.getTextValueEvent();
		ChainedEvent finalHold = collection.getFinalHoldEvent();
		ChainedEvent offset = collection.getOffsetEvent();

		boolean selected = entity.isSelected();

		if (onset != null) {
			g2d.setColor(getHoldColor(onset, selected));
			drawOnset(onset, g2d, Y);
		}

		if (initialHold != null) {
			g2d.setColor(getHoldColor(initialHold, selected));
			drawInitialHold(initialHold, g2d, Y);
		}

		if (glossValue != null) {

			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				g2d.setColor(entity.getField().getColor());
			} else {
				g2d.setColor(getValueColor(glossValue, selected, entity
						.isHighlighted(), fieldID, field));
			}
			drawTextValue(entity, glossValue, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);
		}

		if (value != null) {
			Color color = null;
			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				color = entity.getField().getColor();
			} else {
				color = getValueColor(value, selected, entity.isHighlighted(),
						fieldID, field);
			}
			g2d.setColor(color);
			drawValue(value, g2d, Y);
			drawValueLabel(value, entity, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);
		}

		if (finalHold != null) {
			g2d.setColor(getHoldColor(finalHold, selected));
			drawFinalHold(finalHold, g2d, Y);
		}

		if (offset != null) {
			g2d.setColor(getHoldColor(offset, selected));
			drawOffset(offset, g2d, Y);
		}

		 if((fieldID.equalsIgnoreCase("10501") ||
		 fieldID.equalsIgnoreCase("10502")) &&
		 entity.isLockedGlossPhonAlignments()) {
			 
			 Event v = entity.getValueOrTextValueEvent();
			 if(v != null) {}
			 	drawLock(g2d, v.getStartTimeCoordinate(), (int) Y, Color.BLUE,
			 			entity.isLockedGlossPhonAlignments());
			 	drawLock(g2d, v.getEndTimeCoordinate(), (int) Y, Color.BLUE,
			 			entity.isLockedGlossPhonAlignments());
		 }
	}

	public void drawLock(Graphics2D g2d, int x, int y, Color color,
			boolean locked) {
		if (locked) {
			g2d.setColor(color);
			g2d.drawLine(x, y - 14, x, y - 8);

			g2d.drawLine(x - 3, y - 11, x, y - 8);
			g2d.drawLine(x + 3, y - 11, x, y - 8);

			g2d.drawLine(x - 3, y - 11, x, y - 14);
			g2d.drawLine(x + 3, y - 11, x, y - 14);
		}
	}

	public void drawLock(Graphics2D g2d, int x, int y, Color color,
			boolean isLockedLeftRightAlignments,
			boolean isLockedGlossPhonAlignments) {

		if (isLockedLeftRightAlignments || isLockedGlossPhonAlignments) {
			g2d.setColor(color);
			g2d.drawLine(x, y - 14, x, y - 8);
		}

		if (isLockedLeftRightAlignments) {
			g2d.drawLine(x - 3, y - 11, x, y - 8);
			g2d.drawLine(x + 3, y - 11, x, y - 8);
		}

		if (isLockedGlossPhonAlignments) {
			g2d.drawLine(x - 3, y - 11, x, y - 14);
			g2d.drawLine(x + 3, y - 11, x, y - 14);
		}
	}

	public void drawPresentationEntity(PresentationEventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics) {

		PresentationEvent event = entity.getEvent();
		g2d.setColor(Color.BLACK);

		if (entity.isSelected())
			g2d.setColor(Color.RED);

		int X1 = event.getStartTimeCoordinate();
		int X2 = event.getEndTimeCoordinate();

		g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		g2d.draw(new Line2D.Double(X1, Y + 1, X2, Y + 1));
		g2d.draw(new Line2D.Double(X1, Y - 2, X1, Y + 3));
		g2d.draw(new Line2D.Double(X2, Y - 2, X2, Y + 3));

		String label = event.getText();
		label = Util.getAdjustedLabel(event.getText(), fontMetrics, X2 - X1);

		g2d.setColor(Color.DARK_GRAY);

		label = label == null ? "" : label;

		int lenght = fontMetrics.stringWidth(label);
		double dX = (X2 - X1 - lenght) / 2 + X1;
		g2d.drawString(label, (int) dX, (int) Y - 1);
	}

	public void drawTextValue(EventsEntity entity,
			GlossChainedEvent glossValue, Graphics2D g2d, double Y,
			FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		Field field = entity.getField();

		Color color = getValueColor(glossValue, entity.isSelected(), entity
				.isHighlighted(), fieldID, field);
		g2d.setColor(color);

		drawValue(glossValue, g2d, Y);
		drawValueLabel(glossValue, entity, g2d, Y, fontMetrics, fieldID,
				previousEntity, followingEntity);
	}

	public void drawMovementEntity(MovementEntity entity, Graphics2D g2d,
			FontMetrics fontMetrics, double Y, String fieldID) {

		boolean selected = entity.isSelected();

		if (entity.getFirstEvent() == null)
			return;

		Color color = getValueColor(entity, selected, entity.isHighlighted(),
				fieldID, entity.getField());

		g2d.setColor(color);

		ArrayList events = entity.getEvents();
		for (int i = 0; i < events.size(); i++) {
			ChainedEvent event = (ChainedEvent) events.get(i);
			String id = event.getID();
			StringBuffer sbLabel = new StringBuffer();
			StringTokenizer tk = new StringTokenizer(id, "#");
			String vID = "0";

			while (tk.hasMoreTokens()) {
				String token = tk.nextToken();
				StringTokenizer tk1 = new StringTokenizer(token, ".");
				String cID = tk1.nextToken();
				if (cID.equalsIgnoreCase(Constants.VC_MOVEMENT_CONTOUR)) {
					vID = tk1.nextToken();
					break;
				}
			}

			SS3CodingScheme codingScheme = SS3Singleton
					.getSS3CodingSchemeCollection().getDefaultCodingScheme();

			SS3FieldValue value = codingScheme.getValueCollectionSS3FieldValue(
					Constants.VC_MOVEMENT_CONTOUR, vID);
			sbLabel.append(value.getLabelOrName());

			if (event.isSelected()) {
				g2d.setColor(eventSelectedColor);
			} else {
				g2d.setColor(color);
			}

			int X1 = event.getStartTimeCoordinate();
			int X2 = event.getEndTimeCoordinate();
			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
			g2d.draw(new Line2D.Double(X1, Y + 1, X2, Y + 1));
			g2d.draw(new Line2D.Double(X1, Y - 2, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y - 2, X2, Y + 3));

			g2d.setColor(Color.DARK_GRAY);

			String label = sbLabel.toString();

			int lenght = fontMetrics.stringWidth(label);
			double dX = (X2 - X1 - lenght) / 2 + X1;
			g2d.drawString(label, (int) dX, (int) Y - 1);
		}
	}

	public void drawLocationEntity(LocationEntity entity, Graphics2D g2d,
			double Y, String fieldID) {

		boolean selected = entity.isSelected();

		if (entity.getFirstEvent() == null)
			return;

		int X1 = entity.getStartTimeCoordinate();
		int X2 = entity.getEndTimeCoordinate();

		SS3CodingScheme codingScheme = SS3Singleton
				.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		Color color = selected ? eventSelectedColor : getValueColor(entity,
				selected, entity.isHighlighted(), fieldID, entity.getField());

		g2d.setColor(color);

		g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		g2d.draw(new Line2D.Double(X1, Y + 1, X2, Y + 1));
		g2d.draw(new Line2D.Double(X1, Y - 2, X1, Y + 3));
		g2d.draw(new Line2D.Double(X2, Y - 2, X2, Y + 3));

		ArrayList events = entity.getEvents();
		for (int i = 0; i < events.size(); i++) {
			LocationEvent event = (LocationEvent) events.get(i);
			String id = event.getID();
			// VID="1.4#2.1#3.3"
			StringBuffer label = new StringBuffer();
			StringBuffer toolTip = new StringBuffer();

			StringTokenizer tk = new StringTokenizer(id, "#");

			while (tk.hasMoreTokens()) {
				String token = tk.nextToken();

				StringTokenizer tk1 = new StringTokenizer(token, ".");

				String cID = tk1.nextToken();
				String vID = tk1.nextToken();
				SS3FieldValue value = codingScheme.getValueCollectionSS3FieldValue(cID, vID);
				label.append(value.getLabel());
				label.append(";");

				toolTip.append(value.getName());
				toolTip.append("; ");
			}

			event.setToolTipText(toolTip.toString());

			g2d.setColor(Color.BLACK);
			g2d.drawString(label.toString(),
					event.getStartTimeCoordinate() - 5, (int) Y - 5);
		}
	}

	public void drawEntityWithMultipleEvents(JointsEventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID) {

		ArrayList events = entity.getEvents();
		boolean selected = entity.isSelected();

		SS3CodingScheme codingScheme = SS3Singleton
				.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		Color color = entity.getColor();
		if (color != null)
			g2d.setColor(color);

		g2d.draw(new Line2D.Double(entity.getStartTimeCoordinate(), Y + 1,
				entity.getEndTimeCoordinate(), Y + 1));

		for (int i = 0; i < events.size(); i++) {
			JointsEvent event = (JointsEvent) events.get(i);

			if (event != null) {
				color = getValueColor(event, selected, entity.isHighlighted(),
						fieldID, entity.getField());
				g2d.setColor(color);

				int X1 = event.getStartTimeCoordinate();
				int X2 = X1 + 10;

				if (i == events.size() - 1)
					X1 = X1 - 10;

				if (event.isSelected()) {
					g2d.setColor(Color.red);
				} else {
					g2d.setColor(Color.DARK_GRAY);
				}

				g2d.fillRect(X1, (int) Y - 1, 10, 3);
				event.setRectangle(new Rectangle(X1 - 1, 1, 10, 30));

				String valueID = event.getID();

				SS3FieldValue csValue = codingScheme.getField(fieldID) != null ? codingScheme
						.getField(fieldID).getFieldValue(valueID)
						: null;

				if (csValue != null) {
					event.setText(csValue.getLabelOrName());
					event.setToolTipText(csValue.getName());
					g2d.setColor(Color.BLACK);
				}

				String label = event.getText() == null ? "" : event.getText();

				int lenght = fontMetrics.stringWidth(label);
				double dX = (X2 - X1 - lenght) / 2 + X1;
				g2d.drawString(label, (int) dX, (int) Y - 2);
			}
		}
	}

	public void drawHandShapeEntity(HandShapeEntity entity, Graphics2D g2d,
			double Y, String fieldID) {

		if (entity.getFirstEvent() == null)
			return;

		if (!entity.hasSubEntities())
			return;

		// LOAD IMAGES
		loadHandShapeIcons(entity, fieldID);

        // check if one framed entity.
        if(entity.getFirstEvent().getStartTimeInfo().getMovieTime() == entity.getLastHandShapeEvent().getEndTimeInfo().getMovieTime()) {

            ArrayList<HandShapeSubEntity> subEntities = entity.getSubEntities();

            // iterator returns sorted events
            HandShapeSubEntity superEntity = subEntities.get(0);

            if (superEntity.getEvents().isEmpty())
                return;

            Color entityColor = getValueColor(entity, entity.isSelected(),
                    entity.isHighlighted(), fieldID, entity.getField());

            Color color = entityColor;

            if (superEntity.isSelected()) {
                color = Color.RED;
            }

            superEntity.getFirstEvent();

            HandShapeEvent event = (HandShapeEvent) superEntity.getFirstEvent();
            g2d.setColor(color);

            drawHandShapeEvent(event, entity, entity.getEventsIterator(), LINE_UP_HANDSHAPE_ICON_MIDDLE_TIME, g2d, Y);

            return;
        }

		if (!drawTwoIdenticalHandshapes(entity, fieldID, g2d, Y)) {
			EventsIterator eventsIterator = entity.getEventsIterator();

			HandShapeEvent magnifiedEvent = null;
			Color entityColor = getValueColor(entity, entity.isSelected(),
					entity.isHighlighted(), fieldID, entity.getField());

			Color color = entityColor;
			g2d.setColor(color);

			ArrayList<HandShapeSubEntity> subEntities = entity.getSubEntities();

			// iterator returns sorted events
			HandShapeSubEntity superEntity = subEntities.get(0);

			if (superEntity.getEvents().isEmpty())
				return;

			if (superEntity.isSelected()) {
				color = Color.RED;
			} else {
				color = entityColor;
			}

			g2d.draw(new Line2D.Double(superEntity.getStartTimeCoordinate(),
					Y - 5, superEntity.getEndTimeCoordinate(), Y - 5));

			// draw entity base line
			g2d.draw(new Line2D.Double(
					superEntity.getStartTimeCoordinate() + 2, Y - 5,
					superEntity.getEndTimeCoordinate() - 2, Y - 5));

			// draw super entity icons
			ArrayList<HandShapeEvent> superEvents = superEntity.getEvents();

			HandShapeEvent superEntityLastEvent = (HandShapeEvent) superEntity
					.getLastEvent();

			g2d.setColor(color);

			for (int i = 0; i < superEvents.size() - 1; i++) {
				HandShapeEvent event = superEvents.get(i);
				// boolean lineByStartCoord = true;
				// if (event == superEntityLastEvent)
				// lineByStartCoord = false;
				// g2d.setColor(color);

				boolean drawImageOriginalSize = drawHandShapeEvent(event,
						entity, eventsIterator, LINE_UP_HANDSHAPE_ICON_START_TIME, g2d, Y);

				if (event.isMagnifiedImage() && !drawImageOriginalSize) {
					magnifiedEvent = event;
				}
			}

			// g2d.setColor(color);
			boolean drawImageOriginalSize1 = drawHandShapeEvent(
					superEntityLastEvent, entity, eventsIterator, LINE_UP_HANDSHAPE_ICON_END_TIME, g2d, Y);

			if (superEntityLastEvent.isMagnifiedImage()
					&& !drawImageOriginalSize1) {
				magnifiedEvent = superEntityLastEvent;
			}

			// draw new sub entity
			NewHandShapeEntity newEntity = entity.getNewHandShapeEntity();

			if (newEntity != null) {
				g2d.setColor(color);

				drawSubEntityBaseLine(newEntity.getStartTimeCoord(), newEntity
						.getEndTimeCoord(), (int) Y, g2d);
			}

			for (int i = 1; i < subEntities.size(); i++) {
				HandShapeSubEntity subEntity = subEntities.get(i);

				if (subEntity.isSelected()) {
					color = Color.RED;
				} else {
					color = entityColor;
				}

				g2d.setColor(color);

				// draw sub entity base line
				drawSubEntityBaseLine(subEntity.getStartTimeCoordinate(),
						subEntity.getEndTimeCoordinate(), (int) Y, g2d);

				Event subEntityLastEvent = subEntity.getLastEvent();
				ArrayList subEvents = subEntity.getEvents();

				for (int j = 0; j < subEvents.size(); j++) {
					HandShapeEvent sEvent = (HandShapeEvent) subEvents.get(j);
					int lineByStartCoord = LINE_UP_HANDSHAPE_ICON_START_TIME;
					if (sEvent == subEntityLastEvent)
						lineByStartCoord = LINE_UP_HANDSHAPE_ICON_END_TIME;

                    g2d.setColor(color);

					boolean drawImageOriginalSize = drawHandShapeEvent(sEvent,
							entity, eventsIterator, lineByStartCoord, g2d, Y);

					if (sEvent.isMagnifiedImage() && !drawImageOriginalSize) {
						magnifiedEvent = sEvent;
					}
				}
			}

			if (magnifiedEvent != null) {
				Image img = magnifiedEvent.getLargeImage();

				Image image = img.getScaledInstance(img.getWidth(entity), 23,
						Image.SCALE_AREA_AVERAGING);

				int width1 = image.getWidth(entity) + 1;
				int height1 = image.getHeight(entity) + 1;

				int w = magnifiedEvent.getEndTimeCoordinate()
						- magnifiedEvent.getStartTimeCoordinate();
				int w1 = magnifiedEvent.getStartTimeCoordinate() + (w / 2);
				int startCoord = w1 - width1 / 2;

				g2d.drawRect(startCoord - 1, (int) Y - 18 - 1, width1, height1);
				g2d.drawImage(image, startCoord, (int) Y - 18, Color.WHITE,
						entity);
			}
		}
	}

	private boolean drawTwoIdenticalHandshapes(HandShapeEntity entity,
			String fieldID, Graphics2D g2d, double Y) {
		ArrayList events = entity.getEvents();

		if (events.size() != 2) {
			return false;
		}
		// start here!
		HandShapeEvent event1 = (HandShapeEvent) events.get(0);
		HandShapeEvent event2 = (HandShapeEvent) events.get(1);

		if (!event1.getImageID().equalsIgnoreCase(event2.getImageID())) {
			return false;
		}

		// LOAD IMAGES
		loadHandShapeIcons(entity, fieldID);

		int lenght = entity.getEndTimeCoordinate()
				- entity.getStartTimeCoordinate();

		Image imgL = event1.getLargeImage();
		Image imgS = event1.getSmallImage();
		int lL = imgL.getWidth(entity);
		int lS = imgS.getWidth(entity);
		// int lLR1 = (int)(lL*0.8);
		int lLR2 = (int) (lL * 1.3);
		int lSR1 = (int) (lS * 0.8);
		int lSR2 = (int) (lS * 1.2);

		if (lenght == lS) {
			// draw one small handshape
			g2d.drawImage(imgS, entity.getStartTimeCoordinate(), (int) Y - 18,
					Color.WHITE, entity);
			event1.setRectangle(new Rectangle(
					entity.getStartTimeCoordinate() - 1, 1, lenght, 23));
			event2.setRectangle(new Rectangle(
					entity.getStartTimeCoordinate() - 1, 1, lenght, 23));
			return true;
		} else if (lenght < lS) {
			if (lenght >= lSR1) {
				// draw resized small image
				Image resizedSmallImage = imgS.getScaledInstance(lenght, 23,
						Image.SCALE_AREA_AVERAGING);
				g2d.drawImage(resizedSmallImage, entity
						.getStartTimeCoordinate(), (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				event2.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				return true;
			} else {
				// draw rectangles
				return false;
			}
		} else if (lenght < lL) {
			if (lenght <= lSR2) {
				Image resizedSmallImage = imgS.getScaledInstance(lenght, 23,
						Image.SCALE_AREA_AVERAGING);
				g2d.drawImage(resizedSmallImage, entity
						.getStartTimeCoordinate(), (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				event2.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				return true;
			} else {
				Image resizedLargeImage = imgL.getScaledInstance(lenght, 23,
						Image.SCALE_AREA_AVERAGING);
				g2d.drawImage(resizedLargeImage, entity
						.getStartTimeCoordinate(), (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				event2.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				return true;
			}
		} else if (lenght == lL) {
			// draw large handshape
			g2d.drawImage(imgL, entity.getStartTimeCoordinate(), (int) Y - 18,
					Color.WHITE, entity);
			event1.setRectangle(new Rectangle(
					entity.getStartTimeCoordinate() - 1, 1, lenght, 23));
			return true;
		} else if (lenght < 2 * lS) {
			if (lenght <= lLR2) {
				Image resizedLargeImage = imgL.getScaledInstance(lenght, 23,
						Image.SCALE_AREA_AVERAGING);
				g2d.drawImage(resizedLargeImage, entity
						.getStartTimeCoordinate(), (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				event2.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				return true;
			} else {
				// draw a rectangle (black)
				int height = imgL.getHeight(null);
				g2d.setColor(Color.black);
				g2d.fillRect(entity.getStartTimeCoordinate(), (int) Y - 18,
						lenght, height); // 23);

				int startCoord = entity.getStartTimeCoordinate()
						+ (lenght - lL) / 2;
				g2d.drawImage(imgL, startCoord, (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				event2.setRectangle(new Rectangle(entity
						.getStartTimeCoordinate() - 1, 1, lenght, 23));
				return true;
			}
		}

		return false;
		/*
		 * else if(lenght > 2*lL) { // draw two large handshapes } else
		 * if(lenght > 2*lS) { // draw two small handshapes }
		 * 
		 * // if (lenght < img.getWidth(entity)) { if (lenght == 0) lenght = 1;
		 * 
		 * Image image = event1.getLargeImage(); //.getSmallImage(); //Image
		 * image = img2.getScaledInstance(lenght, 23, //
		 * Image.SCALE_AREA_AVERAGING
		 * 
		 * g2d.drawImage(image, entity.getStartTimeCoordinate(), (int) Y - 18,
		 * Color.WHITE, entity);
		 * 
		 * event1.setRectangle(new Rectangle(entity.getStartTimeCoordinate() -
		 * 1, 1, lWidth, 23)); event2.setRectangle(new
		 * Rectangle(entity.getStartTimeCoordinate() - 1 + lWidth, 1, lWidth,
		 * 23)); // g2d.setColor(Color.RED); //
		 * g2d.drawRect(entity.getStartTimeCoordinate() - 1, (int) Y - 18, //
		 * lWidth, 23); // g2d.drawRect(entity.getStartTimeCoordinate() - 1 +
		 * lWidth, (int) Y - // 18, lWidth, 23); return true;
		 */
	}

	private void drawSubEntityBaseLine(int x1, int x2, int Y, Graphics2D g2d) {
		g2d.draw(new Line2D.Double(x1 + 2, Y - 2, x2 - 2, Y - 2));
		g2d.draw(new Line2D.Double(x1 + 2, Y - 8, x2 - 2, Y - 8));
	}

	private static final int LINE_UP_HANDSHAPE_ICON_START_TIME = 0;
    private static final int LINE_UP_HANDSHAPE_ICON_END_TIME = 1;
    private static final int LINE_UP_HANDSHAPE_ICON_MIDDLE_TIME = 2;

    private boolean drawHandShapeEvent(HandShapeEvent event,
			HandShapeEntity entity, EventsIterator eventsIterator, int lineUp, Graphics2D g2d, double Y) {

		boolean drawSmallImage = false;

		if (event.isSelected())
			g2d.setColor(Color.RED);

		int startTimeCoordinate = event.getStartTimeCoordinate();

        Event previousEvent = null;
        Event nextEvent = null;

        if(eventsIterator != null) {
            previousEvent = eventsIterator.previous(event);
            nextEvent = eventsIterator.next(event);
        }

		Image img = event.getLargeImage();

		Image image = img.getScaledInstance(img.getWidth(entity), 23,
				Image.SCALE_AREA_AVERAGING);

		if (lineUp == LINE_UP_HANDSHAPE_ICON_START_TIME) {
			int w = 10000;
			if (nextEvent != null) {
				w = nextEvent.getStartTimeCoordinate()
						- event.getStartTimeCoordinate() - 5;
			}
			if (w < event.getSmallImage().getWidth(entity)) {
				g2d.drawRect(startTimeCoordinate - 1, (int) Y - 18 - 1, 2, 24);
				event.setRectangle(new Rectangle(startTimeCoordinate - 1, 1, 2, 24));
				return false;
			} else if (w < image.getWidth(entity)) {
				img = event.getSmallImage();
				image = img.getScaledInstance(img.getWidth(entity), 23,
						Image.SCALE_AREA_AVERAGING);
			}
		} else if(lineUp == LINE_UP_HANDSHAPE_ICON_END_TIME) {

			int w = event.getLargeImage().getWidth(entity);

			if (previousEvent != null) {
				w = event.getStartTimeCoordinate()
						- previousEvent.getStartTimeCoordinate() - 25;
			}

			if (w < event.getSmallImage().getWidth(entity)) {
				g2d.drawRect(startTimeCoordinate - 1, (int) Y - 18 - 1, 2, 24);
				event.setRectangle(new Rectangle(startTimeCoordinate - 1, 1, 2, 24));
				return false;
			} else if (w < image.getWidth(entity)) {
				image = event.getSmallImage();
				drawSmallImage = true;
			}
            startTimeCoordinate = event.getStartTimeCoordinate()
					- image.getWidth(entity);
		} else if(lineUp == LINE_UP_HANDSHAPE_ICON_MIDDLE_TIME) {
            // used only for one framed events
            int w = event.getLargeImage().getWidth(entity);

            if (previousEvent != null) {
                w = event.getStartTimeCoordinate()
                        - previousEvent.getStartTimeCoordinate() - 25;
            }

            if (w < event.getSmallImage().getWidth(entity)) {
                g2d.drawRect(startTimeCoordinate - 1, (int) Y - 18 - 1, 2, 24);
                event.setRectangle(new Rectangle(startTimeCoordinate - 1, 1, 2, 24));
                return false;
            } else if (w < image.getWidth(entity)) {
                image = event.getSmallImage();
                drawSmallImage = true;
            }
            startTimeCoordinate = event.getStartTimeCoordinate()
                    - image.getWidth(entity)/2;
        }

		int width = image.getWidth(entity) + 1;
		int height = image.getHeight(entity) + 1;

		g2d.drawRect(startTimeCoordinate - 1, (int) Y - 18 - 1, width, height);
		event.setRectangle(new Rectangle(startTimeCoordinate, 1, width, height));
		g2d.drawImage(image, startTimeCoordinate, (int) Y - 18, Color.WHITE, entity);
		return !drawSmallImage;
	}

	private void loadHandShapeIcons(HandShapeEntity entity, String fieldID) {
		ArrayList hShapes = entity.getHandShapeEvents();
		HandShapesImageManager hsManager = SS3Singleton
				.getHandShapesImageManager();

		SS3CodingScheme codingScheme = SS3Singleton
				.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		SS3Field csField = codingScheme.getField(fieldID);
		for (int i = 0; i < hShapes.size(); i++) {
			HandShapeEvent event = (HandShapeEvent) hShapes.get(i);

			if (event.getLargeImage() == null || event.getSmallImage() == null) {
				String id = event.getID();
				SS3FieldValue value = csField.getFieldValue(id);

				SS3FieldTypeImg ftImg = codingScheme.getImage(value.getIid());

				if (event.getLargeImage() == null) {

					Image lImage = hsManager.getImage(ftImg.getPath1());
					Image imgL = lImage.getScaledInstance(lImage
							.getWidth(entity), 23, Image.SCALE_AREA_AVERAGING);
					event.setLargeImage(imgL);
				}

				if (event.getSmallImage() == null) {

					Image sImage = hsManager.getImage(ftImg.getPath2());
					Image imgS = sImage.getScaledInstance(sImage
							.getWidth(entity), 23, Image.SCALE_AREA_AVERAGING);
					event.setSmallImage(imgS);
				}
			}
		}
	}

	public void drawOnset(ChainedEvent event, Graphics2D g2d, double Y) {
		int X1 = event.getStartTimeCoordinate();
		int X2 = event.getEndTimeCoordinate();

		Color color = g2d.getColor();
		
		if(!event.isSelected()) {
			g2d.setColor(onsetUnselectedColor);
		}
		
		if (X2 - X1 <= 6) {
			int X3 = X2 - 5;
			double Y1 = Y - 8;
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y - 3));

			g2d.draw(new Line2D.Double(X2, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 + 3, Y1, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 + 3, Y1 + 6, X3, Y1 + 3));
		} else {
			g2d.draw(new Line2D.Double(X1 + 5, Y - 3, X1, Y));
			g2d.draw(new Line2D.Double(X1 + 5, Y + 3, X1, Y));

			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		}
		
		if(!event.isSelected()) {
			g2d.setColor(color);
		}
	}

	public void drawOffset(ChainedEvent event, Graphics2D g2d, double Y) {
		int X1 = event.getStartTimeCoordinate();
		int X2 = event.getEndTimeCoordinate();

		Color color = g2d.getColor();
		
		if(!event.isSelected()) {
			g2d.setColor(offsetUnselectedColor);
		}
		
		if (X2 - X1 <= 6) {
			int X3 = X1 + 5;
			double Y1 = Y - 8;
			g2d.draw(new Line2D.Double(X1, Y1, X1, Y - 3));

			g2d.draw(new Line2D.Double(X1, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 - 3, Y1, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 - 3, Y1 + 6, X3, Y1 + 3));
		} else {
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		}

		if(!event.isSelected()) {
			g2d.setColor(color);
		}
	}

	public void drawHold(ChainedEvent event, Graphics2D g2d, double Y) {
		int X1 = event.getStartTimeCoordinate();
		int X2 = event.getEndTimeCoordinate();

		double cX = X1;
		while (cX < X2 - 5) {
			g2d.draw(new Line2D.Double(cX, Y, cX + 5, Y));
			cX += 10;
		}

		g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
		g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
		g2d.draw(new Line2D.Double(X1, Y + 3, X2, Y + 3));
		g2d.draw(new Line2D.Double(X1, Y + 5, X1, Y - 2));
		g2d.draw(new Line2D.Double(X2, Y + 5, X2, Y - 3));
	}

	private Color getValueColor(Event event, boolean selected,
			boolean eventHighlighted, String fieldID, Field field) {

		SS3CodingScheme codingScheme = SS3Singleton
				.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		if (selected) {
			return entitySelectedColor;
		} else if (event != null && event.isSelected()) {
			return eventSelectedColor;
		} else if (eventHighlighted || event.isHighlighted()) {
			return eventHighlightedColor;
		} else if (event.getColor() != null) {
			return event.getColor();
		} else if (field != null && field.getColor() != null) {
			return field.getColor();
		} else {
			SS3Field csField = codingScheme.getField(fieldID);
			if (csField != null)
				return csField.getColor();
			return codingScheme.getColor();
		}
	}

	private Color getHoldColor(Event event, boolean entitySelected) {
		SS3CodingScheme codingScheme = SS3Singleton
				.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		if (entitySelected) {
			return entitySelectedColor;
		} else if (event.isSelected()) {
			return eventSelectedColor;
		} else if (event.isHighlighted()) {
			return eventHighlightedColor;
		} else {
			return codingScheme.getColor();
		}
	}

	public void drawInitialHold(ChainedEvent event, Graphics2D g2d, double Y) {

		int X1 = event.getStartTimeCoordinate();
		int X2 = event.getEndTimeCoordinate();

		if (X2 - X1 <= 6) {
			int X3 = X1 + 5;
			double Y1 = Y - 6;

			g2d.draw(new Line2D.Double(X1, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X1, Y1 + 2, X3, Y1 + 2));
			g2d.draw(new Line2D.Double(X1, Y1 + 1, X3, Y1 + 1));

			g2d.draw(new Line2D.Double(X1, Y1, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y + 3));
		} else {

			double cX = X1;
			while (cX < X2 - 5) {
				g2d.draw(new Line2D.Double(cX, Y, cX + 5, Y));
				cX += 10;
			}

			g2d.draw(new Line2D.Double(X1, Y, X1 + 5, Y - 3));
			g2d.draw(new Line2D.Double(X1, Y, X1 + 5, Y + 3));

			g2d.draw(new Line2D.Double(X1, Y + 3, X2, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y + 5, X1, Y - 2));
			g2d.draw(new Line2D.Double(X2, Y + 5, X2, Y - 3));
		}
	}

	public void drawFinalHold(ChainedEvent event, Graphics2D g2d, double Y) {

		int X1 = event.getStartTimeCoordinate();
		int X2 = event.getEndTimeCoordinate();

		if (X2 - X1 <= 6) {
			int X3 = X2 - 5;
			double Y1 = Y - 6;

			g2d.draw(new Line2D.Double(X2, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X2, Y1 + 2, X3, Y1 + 2));
			g2d.draw(new Line2D.Double(X2, Y1 + 1, X3, Y1 + 1));

			g2d.draw(new Line2D.Double(X1, Y1, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y + 3));
		} else {

			double cX = X1;
			while (cX < X2 - 5) {
				g2d.draw(new Line2D.Double(cX, Y, cX + 5, Y));
				cX += 10;
			}

			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y + 3, X2, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y + 5, X1, Y - 2));
			g2d.draw(new Line2D.Double(X2, Y + 5, X2, Y - 3));
		}
	}

	public void drawValue(ChainedEvent event, Graphics2D g2d, double Y) {
		int X1 = event.getStartTimeCoordinate();
		int X2 = event.getEndTimeCoordinate();

		if (event.isSelected()) {
			g2d.setColor(Color.red);
		} else {
			g2d.setColor(event.getColor());
		}

		g2d.draw(new Line2D.Double(X1, Y, X2, Y));

		if (X1 == X2) {
			g2d.draw(new Line2D.Double(X1, Y + 1, X2 + 10, Y + 1));
			g2d.fillOval(X1, (int) Y - 1, 6, 4);
		} else {
			g2d.draw(new Line2D.Double(X1, Y + 1, X2, Y + 1));
			g2d.draw(new Line2D.Double(X1, Y - 2, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y - 2, X2, Y + 3));
		}
	}

	private void drawValueLabel(ChainedEvent event, EventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		int X1 = event.getStartTimeCoordinate();
		int X2 = event.getEndTimeCoordinate();

		SS3CodingScheme codingScheme = SS3Singleton
				.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		String valueID = event.getID();

		SS3FieldValue csValue = codingScheme.getField(fieldID) != null ? codingScheme
				.getField(fieldID).getFieldValue(valueID)
				: null;

		if (csValue != null && !(event instanceof GlossChainedEvent)) {
			event.setText(csValue.getLabelOrName());
			event.setToolTipText(csValue.getName());
			g2d.setColor(Color.BLACK);
		}

		String label = event.getText();

		g2d.setColor(Color.DARK_GRAY);

		label = label == null ? "" : label;

		int lenght = fontMetrics.stringWidth(label);
		double dX = (X2 - X1 - lenght) / 2 + X1;

		if (dX <= 0)
			dX = 1;

		g2d.drawString(label, (int) dX, (int) Y - 1);
	}

	public void drawFreeTextLabel(ChainedEvent event, Graphics2D g2d, double Y,
			FontMetrics fontMetrics, String fieldID) {

		GlossChainedEvent ftEvent = (GlossChainedEvent) event;

        int x = ftEvent.getStartTimeCoordinate();

        if(x < 5) {
           x = 10;
           ftEvent.setStartTimeCoordinate(10);
        }
        g2d.setColor(Color.BLACK);
		g2d.drawString(ftEvent.getText(), x, (int)Y - 1);
		return;
	}

	public Color getHighlightedColor() {
		return Color.BLUE;
	}
}
