package edu.bu.signstream.grepresentation.fields;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.note.SS3CompleteNoteInformation;
import edu.bu.signstream.common.util.vo.ss3.note.SS3CompleteNoteInformationCollection;
import edu.bu.signstream.common.util.vo.ss3.note.SS3Note;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.FieldsFactory;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.RootLabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class Segment {
   private FieldWrappersCollection wrappers;

   private RootLabelObject rootLabelObject;

   private SignStreamSegmentPanel segmentPanel;

//   private NotesRepresentation notes;
   
   private ArrayList<String> noteIDs = new ArrayList<String>();   

//   private SS3GlossWindowItem windowItem = null;
   
   private ArrayList<SS3GlossWindowItem> windowItems = null;
   
   private PresentationField parentField = null;
   
   private String id = "";

   private String title = "SEGMENT";

   // @TODO: implement!!!
   private String prtcpID = "0";

   private String coderID = "0";

   private int timeStarted = 0;

   private int duration = -1;

   private boolean collapsed = false;

   public Segment(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
	   wrappers = new FieldWrappersCollection();
      this.segmentPanel = segmentPanel;
      this.parentField = parentField;
   }
   
   public void setNotesIDs(ArrayList<String> newNoteIDs) {
	  noteIDs.clear();
	  noteIDs.addAll(newNoteIDs);
   }
    
   public ArrayList<String> getNoteIDs() {
	   return noteIDs;
   }
   
   public void addNoteID(String id) {
	   noteIDs.add(id);
   }
   
   public void setID(String id) {
	   this.id = id;
   }
   
   public String getID() {
	   return id;
   }

   public void setRootLabelObject(RootLabelObject rootLabelObject) {
      this.rootLabelObject = rootLabelObject;
   }
   
   public ArrayList<SS3GlossWindowItem> getSS3GlossWindowItems() {
	   return windowItems;
   }
   
   public void setSS3GlossWindowItems(ArrayList<SS3GlossWindowItem> windowItems) {
	   this.windowItems = windowItems;
   }
   
   public PresentationField getParentField() {
	   return parentField;
   }
   
   public void setParentField(PresentationField parentFIeld) {
	   this.parentField = parentField;
   }

   public RootLabelObject getRootLabelObject() {
      return rootLabelObject;
   }

   private FieldWrapper getFieldWrapper(String type, RootLabelObject lObj) {
	   ArrayList<LabelObject> objects = lObj.getAllLabelObjects();

      for (LabelObject obj : objects) {
         FieldWrapper wrapper = obj.getWrapper();
         if (type.equalsIgnoreCase(wrapper.getType())) {
            return wrapper;
         }
      }
      return null;
   }

   public SignStreamSegmentPanel getSignStreamSegmentPanel() {
      return segmentPanel;
   }

   public void selectItem() {
      try {
         EventsEntity selectedEntity = wrappers.getSelectedEntity();
         selectedEntity.selectEventIfEntitySelected();
      } catch (NullPointerException ex) {
         // do nothing
      }
   }

   public void selectEntity() {
      try {
         EventsEntity selectedEntity = wrappers.getSelectedEntity();
         selectedEntity.selectEntityIfEventSelected();
      } catch (NullPointerException ex) {
         // do nothing
      }
   }
   
   public FieldWrapper getUpdatableFieldWrapper() {
      return wrappers.getUpdatableFieldWrapper();
   }

   public FieldWrapper getFieldWrapper(String type) {
      return getFieldWrapper(type, rootLabelObject); 
   }

   public int getTimeStarted() {
      return timeStarted;
   }

   public void setTimeStarted(int timeStarted) {
      this.timeStarted = timeStarted;
   }

   public SS3CompleteNoteInformation getSegmentNoteInformation() {

      SS3CompleteNoteInformation noteInfo = new SS3CompleteNoteInformation();
      SS3Note note1 = new SS3Note();
      note1.setSubject("Test Note 1");
      note1.setBody("Test Body 1");
      noteInfo.setNote(note1);
      
      SS3Note note2 = new SS3Note();
      note2.setSubject("Test Note 2");
      note2.setBody("Test Body 2");
      noteInfo.setNote(note2);

      SS3Note note3 = new SS3Note();
      note3.setSubject("Test Note 3");
      note3.setBody("Test Body 3");
      noteInfo.setNote(note3);
     
      return noteInfo;
   }

   public Field getField(String fieldID) {
	   ArrayList<LabelObject> list = rootLabelObject.getAllLabelObjects();

       for (LabelObject obj : list) {
           FieldWrapper wrapper = obj.getWrapper();

           if (fieldID.equalsIgnoreCase(wrapper.getField().getFieldID())) {
        	   return wrapper.getField();
           }
       }
       return null;
   }      

   public void setParticipantID(String prtcpID) {
      this.prtcpID = prtcpID;
   }

   public String getParticipantID() {
      return prtcpID;
   }

   public void setCoderID(String coderID) {
      this.coderID = coderID;
   }

   public String getCoderID() {
      return coderID;
   }

   public int getSize() {
      return wrappers.getSize();
   }

   public FieldWrapper getFieldWrapperAt(int Y) {
      return wrappers.getFieldWrapperAt(Y);
   }
   public void addFieldWrapper(FieldWrapper wrapper) {
      wrappers.addFieldWrapper(wrapper);
   }

   public FieldWrappersCollection getFieldWrappersCollection() {
      return wrappers;
   }

   public void unselectAll() {
      ArrayList<FieldWrapper> wrps = wrappers.getWrappers();
      for (FieldWrapper wrp : wrps) {
         wrp.unselectAll();
         wrp.setUpdatable(false);
      }
   }

   public void deleteEmptyEntities() {
      ArrayList<FieldWrapper> wrps = wrappers.getWrappers();
      for (FieldWrapper wrp : wrps) {
         wrp.getField().deleteEmptyEntities();
      }
   }

   public boolean isCollapsed() {
      return collapsed;
   }

   public void collapse(boolean collapsed) {
      this.collapsed = collapsed;
   }

   public String getTitle() {
      return title;
   }

   public void setDuration(int duration) {
      this.duration = duration;
   }

   public int getStartTime() {
	   if (windowItems.isEmpty())
		   return -1;
	   
	   SS3GlossWindowItem firstItem = windowItems.get(0);
	   
	   if (firstItem != null && firstItem.getEndTimeInfo() != null)
		   return firstItem.getStartTimeInfo().getMovieTime();
	   
	   return wrappers.getStartTime();
   }

   public int getEndTime() {
	   if (windowItems.isEmpty())
		   return -1;
	   
	   SS3GlossWindowItem lastItem = windowItems.get(windowItems.size() - 1);
	   
	   if (lastItem != null && lastItem.getEndTimeInfo() != null)
		   return lastItem.getEndTimeInfo().getMovieTime();
	   
	   return wrappers.getEndTime();
   }

}
