package edu.bu.signstream.grepresentation.fields.freeTextField;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.ConfirmationDialog;
import edu.bu.signstream.ui.SignStreamDialogInterface;

public class UpdateTextFieldDialog
   extends JDialog
   implements ActionListener, WindowListener, SignStreamDialogInterface {

   private JButton enter = new JButton("Enter");
   private JButton cancel = new JButton("Cancel");

   private final int ENTER = 0;
   private final int CANCEL = 1;

   private final SignStreamSegmentPanel segmentPanel;
   private final GlossChainedEvent event;
   private final JTextField txtFld = new JTextField(24);

   public UpdateTextFieldDialog(GlossChainedEvent glsEvent, final SignStreamSegmentPanel segmentPanel) {
      super(SS3Singleton.getSignStreamApplication(), "Update Text Field Value");
      super.setLocationRelativeTo(SS3Singleton.getSignStreamApplication());
      this.segmentPanel = segmentPanel;
      this.event = glsEvent;

      JPanel panel = new JPanel();

      enter.setActionCommand("" + ENTER);
      cancel.setActionCommand("" + CANCEL);
      enter.addActionListener(this);
      cancel.addActionListener(this);

      enter.setSelected(true);
      getRootPane().setDefaultButton(enter);

      JLabel title = new JLabel(" ");
      JLabel lbl = new JLabel("Text field value: ");

      txtFld.setText(event.getText());

      Action updateFreeTextEvent = new AbstractAction() {
         @Override
		public void actionPerformed(ActionEvent e) {
            event.setText(txtFld.getText());
            segmentPanel.repaint();
            dispose();
         }
      };

      txtFld.getInputMap().put(
         KeyStroke.getKeyStroke("pressed ENTER"),
         "updateFreeTextEvent");
      txtFld.getActionMap().put("updateFreeTextEvent", updateFreeTextEvent);

      GridBagLayout gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      panel.setLayout(gbl);
      gbc.anchor = GridBagConstraints.NORTHWEST;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      panel.add(title, gbc);

      gbc.anchor = GridBagConstraints.WEST;
      gbc.gridwidth = 1;
      gbc.insets = new Insets(0, 0, 0, 0);
      panel.add(lbl);

      panel.add(Box.createHorizontalStrut(10));
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      panel.add(txtFld, gbc);

      ButtonPanel buttonPanel = new ButtonPanel();
      buttonPanel.add(cancel);
      buttonPanel.add(enter);

      gbc.anchor = GridBagConstraints.SOUTH;
      gbc.insets = new Insets(15, 0, 0, 0);
      gbc.fill = GridBagConstraints.HORIZONTAL;
      gbc.gridwidth = 7;
      panel.add(buttonPanel, gbc);

      addWindowListener(this);

      Dimension d = panel.getPreferredSize();
      Dimension nD = new Dimension(d.width + 10, d.height + 10);
      panel.setPreferredSize(nD);

      Container contentPane = getContentPane();
      contentPane.setLayout(new BorderLayout(50, 50));
      contentPane.add(panel, BorderLayout.CENTER);

      this.pack();
   }

   @Override
public void windowClosing(WindowEvent e) {
      String str1 = event.getText().trim();
      String str2 = txtFld.getText().trim();

      if (!str1.equals(str2)) {
         ConfirmationDialog dialog = new ConfirmationDialog(this);
         dialog.setLocationRelativeTo(this);
         dialog.setModal(true);
         dialog.show();
      }

      dispose();
   }

   @Override
public void saveChanges() {
      event.setText(txtFld.getText());
      segmentPanel.repaint();
   }

   @Override
public void discardChanges() {
      dispose();
   }
   
   @Override
public void actionPerformed(ActionEvent ae) {
      String str = ae.getActionCommand();

      switch (Integer.parseInt(str)) {
         case ENTER :
            saveChanges();
            break;
         case CANCEL :
            dispose();
            break;
      }

      dispose();
   }

   @Override
public void windowClosed(WindowEvent e) {
      //This will only be seen on standard output.
      //displayMessage("WindowListener method called: windowClosed.");
   }

   @Override
public void windowOpened(WindowEvent e) {
      //displayMessage("WindowListener method called: windowOpened.");
   }

   @Override
public void windowIconified(WindowEvent e) {
      //displayMessage("WindowListener method called: windowIconified.");
   }

   @Override
public void windowDeiconified(WindowEvent e) {
      //displayMessage("WindowListener method called: windowDeiconified.");
   }

   @Override
public void windowActivated(WindowEvent e) {
      //displayMessage("WindowListener method called: windowActivated.");
   }

   @Override
public void windowDeactivated(WindowEvent e) {
      //displayMessage("WindowListener method called: windowDeactivated.");
   }
}
