package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.FontMetrics;
import java.awt.geom.Line2D;
import java.util.StringTokenizer;

import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GlossText implements Compoundable {
	private int startCoord = -1;
	private FontMetrics fontMetrics;
	private SignStreamSegmentPanel segmentPanel;

	private StringBuffer bf = new StringBuffer();

	// position in the text, number of character. If cursorPosition >-1, enter/update data 
	private int cursorPosition = -1;

	private String id = "";

	private Compoundable startCompound = null;
	private Compoundable endCompound = null;

	private TimeInfo startTimeInfo = new TimeInfo();
	private TimeInfo endTimeInfo = new TimeInfo();

	public GlossText(String text, int startCoord,
			SignStreamSegmentPanel segmentPanel) {
		this.startCoord = startCoord;
		this.segmentPanel = segmentPanel;

		if (text != null) {
			this.bf.append(text);
			this.cursorPosition = text.length();
		} else {
			this.cursorPosition = 0;
		}
	}

	@Override
	public String getID() {
		return id;
	}

	@Override
	public void setID(String id) {
		this.id = id;
	}

	@Override
	public boolean isEndCompound() {
		return endCompound != null;
	}

	@Override
	public boolean isStartCompound() {
		return startCompound != null;
	}

	@Override
	public Compoundable getEndCompound() {
		return endCompound;
	}

	@Override
	public Compoundable getStartCompound() {
		return startCompound;
	}

	@Override
	public void setEndCompound(Compoundable endCompound) {
		this.endCompound = endCompound;
	}

	@Override
	public void setStartCompound(Compoundable startCompound) {
		this.startCompound = startCompound;
	}

	@Override
	public int getStartTimeCoordinate() {
		return startCoord;
	}

	@Override
	public int getEndTimeCoordinate() {
		return startCoord + fontMetrics.stringWidth(bf.toString());
	}

	@Override
	public TimeInfo getStartTimeInfo() {
		startTimeInfo.setMovieTime(segmentPanel.getZoomer()
				.getCoordinateTimeConvertor().convertCoordinateToTime(
						startCoord));
		return startTimeInfo;
	}

	@Override
	public TimeInfo getEndTimeInfo() {
		endTimeInfo.setMovieTime(segmentPanel.getZoomer()
				.getCoordinateTimeConvertor().convertCoordinateToTime(
						getEndTimeCoordinate()));
		return endTimeInfo;
	}

	public void setFontMetrics(FontMetrics fontMetrics) {
		this.fontMetrics = fontMetrics;
	}

	public boolean isInRange(int coord) {
		try {
			int endCoord = startCoord + fontMetrics.stringWidth(bf.toString());
			return startCoord <= coord && endCoord >= coord;
		} catch (NullPointerException e) {
			return false;
		}
	}

	public void setCursorAtCoord(int coord) {
		cursorPosition = getCoordPosition(coord);
	}

	private int getCoordPosition(int coord) {
		if (bf.length() <= 1)
			return 0;

		int dCoord = coord - startCoord;

		for (int i = 1; i < bf.length(); i++) {
			int w1 = fontMetrics.stringWidth(bf.substring(0, i));
			int w2 = fontMetrics.stringWidth(bf.substring(0, i - 1));

			if (w1 >= dCoord && w2 <= dCoord) {
				if (dCoord > w2 - 3)
					return i;
				return i - 1;
			}
		}
		return 0;
	}

	public String getTextBefore(int coord) {
		return bf.substring(0, getCoordPosition(coord));
	}

	public String getTextAfter(int coord) {
		return bf.substring(getCoordPosition(coord));
	}

	// returns deleted substring
	public String deleteCharactersBeforeCoord(int coord) {
		return bf.delete(0, getCoordPosition(coord)).toString();
	}

	// returns deleted substring
	public String deleteCharactersAfterCoord(int coord) {
		return bf.delete(getCoordPosition(coord), bf.length()).toString();
	}

	public void characterTyped(char typedCharacter) {
		bf.insert(cursorPosition, typedCharacter);
		cursorPosition += 1;
	}

	public void keyPressed(int keyCode) {
		int deleteBtn = 8;
		int deleteBtn2 = 127;
		// int enterBtn = 10;
		int backBtn = 37;
		int forwardBtn = 39;

		if (keyCode == deleteBtn || keyCode == deleteBtn2) {
			if (cursorPosition > 0 && cursorPosition < bf.length() + 1) {
				bf = bf.deleteCharAt(cursorPosition - 1);
				cursorPosition -= 1;
			}
		} else if (keyCode == backBtn) {
			if (cursorPosition > 0) {
				cursorPosition -= 1;
			}
		} else if (keyCode == forwardBtn) {
			if (cursorPosition < bf.length()) {
				cursorPosition += 1;
			}
		}
	}

	public int getStartCoord() {
		return startCoord;
	}

	public FontMetrics getFontMetrics() {
		return fontMetrics;
	}

	public String getText() {
		return bf.toString();
	}
	
	@Override
	public String getText(boolean start) {
		StringTokenizer tk = new StringTokenizer(bf.toString());
		if(start) {
			return tk.nextToken();
		} else {
			String lastTk = "";
			while(tk.hasMoreTokens()) {
				lastTk = tk.nextToken();
			}
			return lastTk;
		}
	}

	public void setStartCoord(int i) {
		startCoord = i;
	}

	public void setText(String string) {
		bf.delete(0, bf.length());
		bf.append(string);
	}

	public int getCursorPosition() {
		return cursorPosition;
	}

	public void setCursorPosition(int i) {
		cursorPosition = i;
	}

	public TextInInterval getNDeleteTextInInterval(int selStartCoord,
			int selEndCoord) {

		// add " " to prevent last character from chopping off
		if (bf.toString().charAt(bf.length() - 1) != ' ')
			bf.append(" ");

		if (!Line2D.linesIntersect(startCoord, 0, startCoord
				+ fontMetrics.stringWidth(bf.toString()), 0, selStartCoord, 0,
				selEndCoord, 0)) {
			return null;
		}

		int delFrom = -1;
		int delTo = -1;

		StringBuffer hsb = new StringBuffer();

		for (int i = 1; i < bf.length(); i++) {
			int sStartCoord = startCoord
					+ fontMetrics.stringWidth(bf.substring(0, i - 1));
			int sEndCoord = sStartCoord
					+ fontMetrics.stringWidth(bf.substring(i - 1, i));

			if (Line2D.linesIntersect(sStartCoord, 0, sEndCoord, 0,
					selStartCoord, 0, selEndCoord, 0)) {
				hsb.append(bf.substring(i - 1, i));
				if (delFrom < 0) {
					delFrom = i - 1;
				} else {
					delTo = i;
				}
			}
		}

		if (delFrom > -1 && delTo > -1) {
			TextInInterval txtInInt = new TextInInterval();
			txtInInt.setText(hsb.toString());

			if (delFrom == 0) {
				bf.replace(delFrom, delTo, "");
				cursorPosition = cursorPosition == 0 ? 0 : cursorPosition
						- delTo;
				this.setStartCoord(this.getStartCoord()
						+ fontMetrics.stringWidth(txtInInt.getText()));
				txtInInt.setAfterGlsText(this);
			} else if (delTo == bf.length()) {
				bf.replace(delFrom, delTo, "");
				cursorPosition = cursorPosition >= bf.length() ? bf.length()
						: cursorPosition - (delTo - delFrom);
				txtInInt.setBeforeGlsText(this);
			} else {
				String before = bf.substring(0, delFrom);
				String after = bf.substring(delTo);
				int startAfterCoord = this.getStartCoord()
						+ fontMetrics.stringWidth(bf.substring(0, delTo));
				bf.replace(0, bf.length(), before);
				txtInInt.setBeforeGlsText(this);
				GlossText txt = new GlossText(after, startAfterCoord,
						segmentPanel);
				txtInInt.setAfterGlsText(txt);
				cursorPosition = -1;
			}
			return txtInInt;
		}
		return null;
	}

	@Override
	public String toString() {
		return bf.toString();
	}
}

class TextInInterval {
	private String text = null;
	private GlossText beforeGlsText = null;
	private GlossText afterGlsText = null;

	public GlossText getAfterGlsText() {
		return afterGlsText;
	}

	public GlossText getBeforeGlsText() {
		return beforeGlsText;
	}

	public String getText() {
		return text;
	}

	public void setAfterGlsText(GlossText afterGlsText) {
		this.afterGlsText = afterGlsText;
	}

	public void setBeforeGlsText(GlossText beforeGlsText) {
		this.beforeGlsText = beforeGlsText;
	}

	public void setText(String string) {
		text = string;
	}

	@Override
	public String toString() {
		StringBuffer bf = new StringBuffer();
		bf.append("text = ");
		bf.append(text);

		if (beforeGlsText != null) {
			bf.append(", beforeGlsText = ");
			bf.append(beforeGlsText.getText());
		}

		if (afterGlsText != null) {
			bf.append(", afterGlsText = ");
			bf.append(afterGlsText.getText());
		}

		return bf.toString();
	}
}
