package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossClassifications;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEvent;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeSubEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.AccessSignBankActionListener;
import edu.bu.signstream.grepresentation.fields.rightLeftField.RightLeftHandFieldWrapper;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.GlossNRelatedView;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankMainGloss;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankVariant;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.signbank.SignBankDialog;

public class AssociatedFieldsPanel extends JPanel implements ItemListener, ActionListener {

	private static final long serialVersionUID = 1L;

	private final JDialog parent;
	
	private SignBankDialog signBankDialog = null;

	private GlossFieldWrapper wrapper = null;
	private HandShapePanel hshPanel = null;

	private final JLabel primaryEntryLb = new JLabel("Primary Entry ");
	private final JTextField primaryEntryField = new JTextField();

	private final JLabel variantLb = new JLabel("Entry/Variant ");
	private final JTextField variantField = new JTextField();	
	private final JButton searchSignBank = new JButton("Search Sign Bank");
	
	private final JButton addNewSign = new JButton("Add To Sign Bank");

	private final JButton enter = new JButton("Enter");
	private final JButton cancel = new JButton("Cancel");
	private final JButton bypass = new JButton("Bypass Sign Bank Check");

	private final JRadioButton fingerspelled = new JRadioButton("Fingerspelled");
	private final JRadioButton loan = new JRadioButton("Loan sign");
	private final JRadioButton number = new JRadioButton("Number");
	private final JRadioButton lexical = new JRadioButton("Lexical sign");
	private final JRadioButton classifier = new JRadioButton("Classifier");
	private final JRadioButton gesture = new JRadioButton("Gesture");
	private final JCheckBox nameSign = new JCheckBox("Name sign");
	
	private final JLabel donHndLbl = new JLabel("Dominant Hand   ");
	private final JRadioButton domHndR = new JRadioButton("R");
	private final JRadioButton domHndL = new JRadioButton("L");

	private final int ENTER_BYPASS = -1;
	private final int ENTER = 0;
	private final int CANCEL = 1;
	private final int FINGERSPELLED = 2;
	private final int LOAN = 3;
	private final int LEXICAL = 4;
	private final int CLASSIFIER = 5;
	private final int GESTURE = 6;
	private final int NUMBER = 7;
	private final int SIGN_BANK = 8;
	private final int ADD_NEW = 9;

	private GlossNRelatedView view = null;

	private HandShapePreprocessor processor = null;
	private HandShapeFormBean hshFormBean = null;

	private final LoadedProperties loadedProperties = new LoadedProperties();
	
	private AccessSignBankActionListener accessSignBankListener = null;
	
	public AssociatedFieldsPanel(GlossFieldWrapper wrapper, GlossNRelatedView view, JDialog parent) {  
		this.wrapper = wrapper;
		this.view = view;
		this.parent = parent;
		this.processor = new HandShapePreprocessor(wrapper, view.getSegment());

		ButtonGroup grp = new ButtonGroup();
		grp.add(fingerspelled);
		grp.add(loan);
		grp.add(number);
		grp.add(lexical);
		grp.add(classifier);
		grp.add(gesture);

		enter.setActionCommand("" + ENTER);
		cancel.setActionCommand("" + CANCEL);
		bypass.setActionCommand("" + ENTER_BYPASS);
		fingerspelled.setActionCommand("" + FINGERSPELLED);
		loan.setActionCommand("" + LOAN);
		lexical.setActionCommand("" + LEXICAL);
		classifier.setActionCommand("" + CLASSIFIER);
		gesture.setActionCommand("" + GESTURE);
		number.setActionCommand("" + NUMBER);
		addGlossFieldDocumentListener();

		enter.addActionListener(new WeakActionListener(this, enter));
		cancel.addActionListener(new WeakActionListener(this, cancel));
		bypass.addActionListener(new WeakActionListener(this, bypass));
		fingerspelled.addActionListener(new WeakActionListener(this, fingerspelled));
		loan.addActionListener(new WeakActionListener(this, loan));
		number.addActionListener(new WeakActionListener(this, number));
		lexical.addActionListener(new WeakActionListener(this, lexical));
		classifier.addActionListener(new WeakActionListener(this, classifier));
		gesture.addActionListener(new WeakActionListener(this, gesture));
		
		searchSignBank.addActionListener(this);
		searchSignBank.setActionCommand("" + SIGN_BANK);
		
		addNewSign.addActionListener(this);
		addNewSign.setActionCommand("" + ADD_NEW);

		update(wrapper, view);

		this.hshPanel = new HandShapePanel(this);

        ButtonGroup domHndGrp = new ButtonGroup();
        domHndGrp.add(domHndR);
		domHndGrp.add(domHndL);

		enter.setEnabled(false);

		initUI();
	}
	
	private void addGlossFieldDocumentListener() {
		variantField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				changed();
			}
			@Override
			public void removeUpdate(DocumentEvent e) {
				changed();
			}
			@Override
			public void changedUpdate(DocumentEvent e) {
			}
			
			public void changed() {
				if (variantField.getText().equals(""))
					enter.setEnabled(false);
				else
					enter.setEnabled(true);
			}
			});		
	}

	private void populateHandshapeFormBean(GlossChainedEventsEntity entity) {
		// retrieve selected handshapes for right and left hands
		HandShapeEntity hshEntity1 = entity.getDependentHandShapeEntity();
		// 1000, 2000
		SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		ArrayList<HandShapeImage> selectedHandImagesRight = new ArrayList<HandShapeImage>();
		ArrayList<HandShapeImage> selectedHandImagesLeft = new ArrayList<HandShapeImage>();

		if (hshEntity1 != null) {
			String fID = hshEntity1.getField().getFieldID();
			boolean isDomRight = fID.equalsIgnoreCase("1000");
			SS3Field field = cs.getField(fID);
            ArrayList<HandShapeEvent> hShapes = hshEntity1.getHandShapeEvents();

			for (HandShapeEvent event : hShapes) {
				SS3FieldValue value = field.getFieldValue(event.getID());
				HandShapeImage image = new HandShapeImage(value, cs);
				if (isDomRight) {
					selectedHandImagesRight.add(image);
				} else {
					selectedHandImagesLeft.add(image);
				}
			}
		}

		GlossChainedEventsEntity ndEntity = entity.getDependentGlossEntity();

		if (ndEntity != null && ndEntity.getDependentHandShapeEntity() != null) {
			HandShapeEntity hshEntity2 = ndEntity.getDependentHandShapeEntity();
			String ndfID = hshEntity2.getField().getFieldID();

			boolean isNdDomRight = ndfID.equalsIgnoreCase("1000");
			SS3Field ndfield = cs.getField(ndfID);
            ArrayList<HandShapeEvent> ndHShapes = hshEntity2.getHandShapeEvents();

			for (HandShapeEvent event : ndHShapes) {
				SS3FieldValue value = ndfield.getFieldValue(event.getID());
				HandShapeImage image = new HandShapeImage(value, cs);
				if (isNdDomRight) {
					selectedHandImagesRight.add(image);
				} else {
					selectedHandImagesLeft.add(image);
				}
			}
		}

		hshFormBean.setSelectedRightHandShapeImages(selectedHandImagesRight);
		hshFormBean.setSelectedLeftHandShapeImages(selectedHandImagesLeft);
	}

	protected void update(GlossFieldWrapper wrapper, GlossNRelatedView glsView) {
		this.wrapper = wrapper;
		this.view = glsView;

		loadedProperties.reset();

		EventsEntity ent = SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();

		if (ent == null)
			return;

 		GlossChainedEventsEntity entity = (GlossChainedEventsEntity) ent;

		hshFormBean = new HandShapeFormBean();
		processor.populateFormBean(hshFormBean);
		hshFormBean.setGlsFieldWrapper(wrapper);

		populateHandshapeFormBean(entity);

		GlossChainedEvent event = entity.getGlossChainedEvent();

		variantField.setText(event.getText());
		
		primaryEntryField.setText(null);

		// get gloss classifications
		GlossClassifications classifications = event.getGlossClassifications();

		domHndR.setSelected(classifications.isRightHandDominant());
		domHndL.setSelected(!classifications.isRightHandDominant());

		String fieldID = entity.getField().getFieldID();
		String frstGlsID = "10000";
		String ndGlsID = "10001";

		GlossEntityProperties properties = entity.getGlossEntityProperties();

        ArrayList<HandShapeImage> rhList = new ArrayList<HandShapeImage>(hshFormBean.getSelectedRightHandShapeImages());
        ArrayList<HandShapeImage> lhList = new ArrayList<HandShapeImage>(hshFormBean.getSelectedLeftHandShapeImages());

        if((rhList.size() > 0 && lhList.size() == 0) || (rhList.size() == 0 && lhList.size() > 0)) {
            properties.setTwoHanded(false);
        }

		if (classifications.isRightHandDominant()) {
			properties.setDominamtHand("R");
			domHndR.setSelected(true);
		} else {
			properties.setDominamtHand("L");
			domHndL.setSelected(true);
		}

		if (fieldID.equalsIgnoreCase(frstGlsID)) {
			// if dominant hand right (right handed signer), check right hand;
			// otherwise check left
			if (domHndR.isSelected()) {
				properties.setOnehandedSignHand("R");
			} else {
				properties.setOnehandedSignHand("L");
			}
		} else {
			if (domHndR.isSelected()) {
				properties.setOnehandedSignHand("L");
			} else {
				properties.setOnehandedSignHand("R");
			}
		}
		
		int entityStartTime = entity.getStartTimeInfo().getMovieTime();
		int entityEndTime = entity.getEndTimeInfo().getMovieTime();
		if (entityStartTime == entityEndTime) {
			properties.setOneFrameGloss(true);
		} else {
			properties.setOneFrameGloss(false);
		}		

		FieldWrappersCollection collection = view.getSegment()
				.getFieldWrappersCollection();
		GlossFieldWrapper ndWrp;

		String classifierStr = properties.getClassifier();
		if (classifierStr == null || classifierStr.length() == 0) {

			boolean rhEmpty = rhList.isEmpty();
			boolean lhEmpty = lhList.isEmpty();

			if (rhEmpty && !lhEmpty) {
				properties.setTwoHanded(false);
				boolean rIdentical = isIdenticalStartEndHandshapes(lhList);

				if (rIdentical) {
					properties
							.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties
							.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}

			} else {
                if (!rhEmpty && lhEmpty) {
                    properties.setTwoHanded(false);
                    boolean lIdentical = isIdenticalStartEndHandshapes(rhList);

                    if (lIdentical) {
                        properties
                                .setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
                    } else {
                        properties
                                .setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
                    }
                } else if (!rhEmpty && !lhEmpty) {
                    properties.setTwoHanded(true);

                    boolean isIdenticalLeftRight = isIdenticalHandshapes(rhList, lhList);

                    if (isIdenticalLeftRight) {
                        properties.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_LR);
                        boolean lIdentical = isIdenticalStartEndHandshapes(rhList);

                        if (lIdentical) {
                            properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
                        } else {
                            properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
                        }
                    } else {
                        properties.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR);

                        if (isIdenticalStartEndHandshapes(lhList)) {
                            properties
                                    .setStartEndHandshapeLeftHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
                        } else {
                            properties
                                    .setStartEndHandshapeLeftHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
                        }

                        if (isIdenticalStartEndHandshapes(rhList)) {
                            properties
                                    .setStartEndHandshapeRightHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
                        } else {
                            properties
                                    .setStartEndHandshapeRightHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
                        }
                    }

                    properties.setTwoHandedEnabled(true);

                    String gls = entity.getText();

                    String handednessPattern = "(!)?(neu:|([a-z_1-3][p:]))?((.)([12][h])(.))(alt.)?(.*)?";
                    Pattern.compile(handednessPattern);
                    Pattern handednessR = Pattern.compile(handednessPattern);
                    Matcher handednessMatcher = handednessR.matcher(gls);

                    // handedness
                    if (handednessMatcher.find()) {

                        String oneHandedPattern = "(!)?(neu:)?((1h)){1}(.*)?";
                        Pattern oneHandedR = Pattern.compile(oneHandedPattern);
                        Matcher oneHandedMatcher = oneHandedR.matcher(gls);

                        if (oneHandedMatcher.find()) {
                            hshPanel.selectOneHanded();
                            properties.setTwoHanded(false);

                        } else {
                            properties.setTwoHanded(true);

                            String twoHandedAltPattern = "(!)?(neu:)?((2h).alt)(.*)?";
                            Pattern twoHandedAltR = Pattern
                                    .compile(twoHandedAltPattern);
                            Matcher twoHandedAltMatcher = twoHandedAltR
                                    .matcher(gls);

                            if (twoHandedAltMatcher.find()) {
                                properties
                                        .setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR);
                            }
                        }
                    }
                }
            }

			String gls = entity.getText();
			// name sign
			String nameSignPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(ns-){1}(.*)?";
			Pattern nameSignR = Pattern.compile(nameSignPattern);
			Matcher nameSignMatcher = nameSignR.matcher(gls);

			if (nameSignMatcher.find()) {
				nameSign.setSelected(true);
			}

			String fingerspelledPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(fs-){1}(.*)?";
			Pattern fingerspelledR = Pattern.compile(fingerspelledPattern);
			Matcher fingerspelledMatcher = fingerspelledR.matcher(gls);

			if (fingerspelledMatcher.find()) {
				fingerspelled.setSelected(true);
			} else {
				// loan sign
				String loanSignPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(#){1}(.*)?";
				Pattern loanSignR = Pattern.compile(loanSignPattern);
				Matcher loanSignMatcher = loanSignR.matcher(gls);

				if (loanSignMatcher.find()) {
					loan.setSelected(true);
				} else {
					String classifierPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(ns-)?((CL)|(DCL)|(LCL)|(SCL)|(BCL)|(BPCL)|(PCL)|(ICL)){1}(.*)?";
					Pattern classifierR = Pattern.compile(classifierPattern);
					Matcher classifierMatcher = classifierR.matcher(gls);

					if (classifierMatcher.find()) {
						classifier.setSelected(true);
					} else {

						String gesturePattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(ns-)?((\"){1}(.){1,}(\"){1})(.*)?";
						Pattern gestureR = Pattern.compile(gesturePattern);
						Matcher gestureMatcher = gestureR.matcher(gls);

						if (gestureMatcher.find()) {
							gesture.setSelected(true);
						} else {
							lexical.setSelected(true);

							if (fieldID.equalsIgnoreCase(frstGlsID)) {
								ndWrp = (GlossFieldWrapper) collection
										.getFieldWrapperWithFieldID(ndGlsID);
							} else {
								ndWrp = (GlossFieldWrapper) collection
										.getFieldWrapperWithFieldID(frstGlsID);
							}

							properties.setTwoHanded(true);
						}
					}
				}

                boolean lexicalProperty = properties.isLexical();
                if(lexicalProperty) {
                    properties.setTwoHanded(true);
                }
			}
		} else {
			fingerspelled.setSelected(properties.isFingerspelled());
			loan.setSelected(properties.isLoan());
			number.setSelected(properties.isNumber());
			lexical.setSelected(properties.isLexical());
			classifier.setSelected(properties.isClassifier());
			gesture.setSelected(properties.isGesture());
			nameSign.setSelected(properties.isNameSign());

			if (fieldID.equalsIgnoreCase(frstGlsID)) {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(ndGlsID);
			} else {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(frstGlsID);
			}

			NonManualField ndField = (NonManualField) ndWrp.getField();
			boolean isPossibleToAddEvent = ndField.isEmptyInterval(entity
					.getStartTimeInfo().getMovieTime(), entity.getEndTimeInfo()
					.getMovieTime());

			if (!isPossibleToAddEvent) {
				String correspondingID = entity.getCorrespondingGlossEntityID();
				EventsEntity correspondingEntity = ndField
						.getEntity(correspondingID);
				if (correspondingEntity != null) {
					isPossibleToAddEvent = true;
				}
			}			

			properties.setTwoHandedEnabled(isPossibleToAddEvent);
			
		}

		properties.setTwoHandedEnabled(true);
		
		hshFormBean.setGlossEntityProperties(properties);

		if (this.hshPanel != null) {
			this.hshPanel.update();
		}

		checkSignTypeAndUpdateSignBankFunctionality();

		// populate LoadedProperties object 
		loadedProperties.setGloss(variantField.getText());
		loadedProperties.setFingerspelled(fingerspelled.isSelected());
		loadedProperties.setLoan(loan.isSelected());
		loadedProperties.setNumber(number.isSelected());
		loadedProperties.setLexical(lexical.isSelected());
		loadedProperties.setClassifier(classifier.isSelected());
		loadedProperties.setGesture(gesture.isSelected());
		loadedProperties.setNameSign(nameSign.isSelected());
		
		loadedProperties.setTwoHanded(properties.isTwoHanded());
		loadedProperties.setMarkedNumberOfHands(properties.isMarkedNumberOfHands());
		loadedProperties.setPassiveBaseArm(properties.isPassiveBaseArm());	
		loadedProperties.setOneFrameGloss(properties.isOneFrameGloss());

		if(hshPanel != null){
			loadedProperties.setSameDifHandshapes(hshPanel.getSelectedSameDifLeftRightHandshapes());
			loadedProperties.setSameDIfferentLeftRightHandshapes(hshPanel.getSelectedSameDifStartEndHandshapesR()); 
			loadedProperties.setSameDifStartEndHandshapesL(hshPanel.getSelectedSameDifStartEndHandshapesL()); 
			loadedProperties.setSameDifStartEndHandshapesR(hshPanel.getSelectedSameDifStartEndHandshapesR()); 
		}
		
		loadedProperties.setRightHandSelectedHandshapes(hshFormBean.getSelectedRightHandShapeImages());
		loadedProperties.setLeftHandSelectedHandshapes(hshFormBean.getSelectedLeftHandShapeImages());
		
		revalidate();
		parent.pack();
	}

	public HandShapeFormBean getHandShapeFormBean() {
		return hshFormBean;
	}

	private boolean isIdenticalHandshapes(ArrayList<HandShapeImage> rhList,
			ArrayList<HandShapeImage> lhList) {
		if (rhList.size() == lhList.size()) {
			for (int i = 0; i < rhList.size(); i++) {
				HandShapeImage rImg = rhList.get(i);
				HandShapeImage lImg = lhList.get(i);
				String rId = rImg.getSS3FieldValue().getId();
				String lId = lImg.getSS3FieldValue().getId();

				if (!rId.equalsIgnoreCase(lId)) {
					return false;
				}
			}
			return true;
		} else {
			return false;
		}
	}

	private boolean isIdenticalStartEndHandshapes(ArrayList<HandShapeImage> list) {
		if (list.isEmpty()) {
			return true;
		}

		HandShapeImage fImg = list.get(0);
		HandShapeImage lImg = list.get(list.size() - 1);

		String rId = fImg.getSS3FieldValue().getId();
		String lId = lImg.getSS3FieldValue().getId();

        return rId.equalsIgnoreCase(lId);

    }

	private JPanel createGlossPanel() {
		JPanel panel = new JPanel();

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		panel.setLayout(gbl);

		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(13, 3, 3, 3);
		gbc.anchor = GridBagConstraints.WEST;
		
		gbc.gridy = 1;
		gbc.gridx = 1;
		gbc.gridwidth = 1;
		panel.add(primaryEntryLb, gbc);
		
		gbc.gridx = 2;
		gbc.gridwidth = 3;
		panel.add(primaryEntryField, gbc);
		
		gbc.gridy++;
		gbc.gridx = 1;
		gbc.insets = new Insets(0, 20, 5, 0);
		gbc.gridwidth = 1;
		panel.add(variantLb, gbc);

		gbc.gridx = 2;
		gbc.gridwidth = 3;
		panel.add(variantField, gbc);
		
		gbc.gridx = 5;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(1, 1, 1, 1);
		panel.add(searchSignBank, gbc);

		gbc.insets = new Insets(1, 1, 1, 1);
		gbc.gridy++;
		gbc.gridx = 1;
		gbc.gridwidth = 1;
		panel.add(fingerspelled, gbc);
		gbc.gridx = 2;
		panel.add(loan, gbc);
		gbc.gridx = 3;
		panel.add(number, gbc);
		gbc.gridx = 4;
		panel.add(new JLabel(), gbc);

		gbc.gridx = 5;
		gbc.gridwidth = 1;
		panel.add(addNewSign, gbc);

		gbc.gridy++;
		gbc.gridx = 1;
		gbc.gridwidth = 1;
		panel.add(lexical, gbc);
		gbc.gridx = 2;
		panel.add(classifier, gbc);
		gbc.gridx = 3;
		panel.add(gesture, gbc);
		gbc.gridx = 4;
		panel.add(nameSign, gbc);

		Dimension dd = new Dimension(128, 27);

		domHndR.setMinimumSize(dd);
		domHndR.setSize(dd);
		domHndR.setPreferredSize(dd);
		domHndR.setHorizontalAlignment(SwingConstants.CENTER);

		Dimension ddd = new Dimension(190, 27);

		domHndL.setMinimumSize(ddd);
		domHndL.setSize(ddd);
		domHndL.setPreferredSize(ddd);
		domHndL.setHorizontalAlignment(SwingConstants.CENTER);

		domHndR.setEnabled(false);
		domHndL.setEnabled(false);

		return panel;
	}

	private JPanel createPosPanel() {
		JPanel panel = new JPanel();

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		panel.setLayout(gbl);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0, 0, 10, 0);

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(cancel);
		buttonPanel.add(enter);
		buttonPanel.add(bypass);

		gbc.anchor = GridBagConstraints.SOUTH;
		gbc.insets = new Insets(15, 0, 0, 0);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		panel.add(buttonPanel, gbc);

		return panel;
	}

	private JPanel createCenterPanel() {
		JPanel panel = new JPanel();

		LocationPanel lPanel = new LocationPanel(new LocationFormBean());
		OrientationPanel oPanel = new OrientationPanel(new OrientationBean());
		MovementPanel mPanel = new MovementPanel(new MovementFormBean());

		JTabbedPane tp = new JTabbedPane();
		tp.add(hshPanel, "Hand Shapes");
		tp.add(lPanel, "Location");
		tp.add(oPanel, "Orientation");
		tp.add(mPanel, "Movement");

		panel.setLayout(new BorderLayout());

		GridBagConstraints gbc1 = new GridBagConstraints();

		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new GridBagLayout());

		gbc1.anchor = GridBagConstraints.NORTHWEST;
		gbc1.gridwidth = GridBagConstraints.REMAINDER;

		gbc1.gridy = 1;
		gbc1.gridx = 1;
		gbc1.gridwidth = 2;
		gbc1.insets = new Insets(1, 1, 1, 1);
		gbc1.anchor = GridBagConstraints.NORTHWEST;
		centerPanel.add(donHndLbl, gbc1);

		gbc1.gridwidth = 1;
		gbc1.gridx = 3;
		gbc1.anchor = GridBagConstraints.CENTER;
		centerPanel.add(domHndL, gbc1);
		gbc1.gridx = 4;
		gbc1.anchor = GridBagConstraints.CENTER;
		centerPanel.add(domHndR, gbc1);

		panel.add(centerPanel, BorderLayout.NORTH);

		panel.add(tp, BorderLayout.CENTER);

		return panel;
	}

	private void initUI() {
		JPanel glsPanel = createGlossPanel();
		JPanel posPanel = createPosPanel();
		JPanel cPanel = createCenterPanel();

		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(new GridBagLayout());

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.insets = new Insets(0, 0, 10, 0);
		add(glsPanel, gbc);

		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0, 0, 10, 0);
		add(new JSeparator(), gbc);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		add(cPanel, gbc);

		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0, 0, 10, 0);
		add(posPanel, gbc);
	}

	@Override
	public void itemStateChanged(ItemEvent event) {
		removeAll();
		initUI();
		revalidate();
		parent.pack();
	}

	public boolean isChanged() {

		String glsText = variantField.getText();
		if (!glsText.equalsIgnoreCase(loadedProperties.getGloss())) {
			return true;
		}
		
		if(fingerspelled.isSelected() != loadedProperties.isFingerspelled()) {
			return true;
		}
		
		if(loan.isSelected() != loadedProperties.isLoan()) {
			return true;
		}
		
		if(number.isSelected() != loadedProperties.isNumber()) {
			return true;
		}
		
		if(lexical.isSelected() != loadedProperties.isLexical()) {
			return true;
		}
		
		if(classifier.isSelected() != loadedProperties.isClassifier()) {
			return true;
		}
		
		if(gesture.isSelected() !=  loadedProperties.isGesture()) {
			return true;
		}
		
		if(nameSign.isSelected() != loadedProperties.isNameSign()) {
			return true;
		}
		
		if(hshPanel.isPassiveBaseArm() != loadedProperties.isPassiveBaseArm()) {
			return true;
		}
		
		if(hshPanel.isMarkedNumberOfHands() != loadedProperties.isMarkedNumberOfHands()) {
			return true;
		}

		if(hshPanel.isTwoHandedSelected() != loadedProperties.isTwoHanded()) {
			return true;
		}

		int sameDifHandshapes = hshPanel.getSelectedSameDifLeftRightHandshapes();
		int sameDifStartEndHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
		int sameDifStartEndHandshapesL = hshPanel.getSelectedSameDifStartEndHandshapesL();
		int sameDifStartEndHandshapesR = hshPanel.getSelectedSameDifStartEndHandshapesR();

		int sameDifHandshapes0 = loadedProperties.getSameDifHandshapes(); 
		int sameDifStartEndHandshapes0 = loadedProperties.getSameDIfferentLeftRightHandshapes(); 
		int sameDifStartEndHandshapesR0 = loadedProperties.getSameDifStartEndHandshapesR(); 
		int sameDifStartEndHandshapesL0 = loadedProperties.getSameDifStartEndHandshapesL(); 
		
		if(sameDifHandshapes != sameDifHandshapes0) {
			return true;
		}

		if(sameDifStartEndHandshapes != sameDifStartEndHandshapes0) {
			return true;
		}
		
		if(sameDifStartEndHandshapesL != sameDifStartEndHandshapesL0) {
			return true;
		}
		
		if(sameDifStartEndHandshapesR != sameDifStartEndHandshapesR0) {
			return true;
		}
		
		ArrayList<HandShapeImage> selLeftHandShapes = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
		ArrayList<HandShapeImage> selRightHandShapes = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
		
		ArrayList<HandShapeImage> selLeftHandShapes1 = loadedProperties.getLeftHandSelectedHandshapes();
		ArrayList<HandShapeImage> selRightHandShapes1 = loadedProperties.getRightHandSelectedHandshapes();
		
		if(selLeftHandShapes.isEmpty() != selLeftHandShapes1.isEmpty()) {
			return true;
		}
		
		if(selRightHandShapes.isEmpty() != selRightHandShapes1.isEmpty()) {
			return true;
		}
		
		int rSize1 = selRightHandShapes1.size();
		int rSize = selRightHandShapes.size();
		
		if(rSize1 != rSize) {
			if(rSize1 == 2 && rSize == 1) {
				HandShapeImage rHandShape11 = selRightHandShapes1.get(0);
				HandShapeImage rHandShape12 = selRightHandShapes1.get(1);
				HandShapeImage rHandShape = selRightHandShapes.get(0);
				String rHandShape11ID = rHandShape11.getSS3FieldValue().getId();
				String rHandShape12ID = rHandShape12.getSS3FieldValue().getId();
				if(!rHandShape11ID.equalsIgnoreCase(rHandShape12ID)) {
					return true;
				}
				String rHandShapeID = rHandShape.getSS3FieldValue().getId();
				if(!rHandShape11ID.equalsIgnoreCase(rHandShapeID)) {
					return true;
				}
			} else {
				return true;
			} 
		}
			
		int i = 0 ;
		for(HandShapeImage rHandShape : selRightHandShapes) {
			HandShapeImage rHandShape1 = selRightHandShapes1.get(i);
			String rhShapeID = rHandShape.getSS3FieldValue().getId();
			String rhShapeID1 = rHandShape1.getSS3FieldValue().getId();
			if(!rhShapeID.equalsIgnoreCase(rhShapeID1)) {
				return true;
			}
			i++;
		}
		
		int lSize1 = selLeftHandShapes1.size();
		int lSize = selLeftHandShapes.size();
		
		if(lSize1 != lSize) {
			if(lSize1 == 2 && lSize == 1) {
				HandShapeImage lHandShape11 = selLeftHandShapes1.get(0);
				HandShapeImage lHandShape12 = selLeftHandShapes1.get(1);
				HandShapeImage lHandShape = selLeftHandShapes.get(0);
				String lHandShape11ID = lHandShape11.getSS3FieldValue().getId();
				String lHandShape12ID = lHandShape12.getSS3FieldValue().getId();
				if(!lHandShape11ID.equalsIgnoreCase(lHandShape12ID)) {
					return true;
				}
				String lHandShapeID = lHandShape.getSS3FieldValue().getId();
				if(!lHandShape11ID.equalsIgnoreCase(lHandShapeID)) {
					return true;
				}
			} else {
				return true;
			}
		}
	
		int j = 0 ;
		for(HandShapeImage lHandShape : selLeftHandShapes) {
			HandShapeImage lHandShape1 = selLeftHandShapes1.get(j);
			String lhShapeID = lHandShape.getSS3FieldValue().getId();
			String lhShapeID1 = lHandShape1.getSS3FieldValue().getId();
			if(!lhShapeID.equalsIgnoreCase(lhShapeID1)) {
				return true;
			}
			j++;
		}

		return false;
	}

	public void saveChanges() {
		updateChanges();
		SS3Singleton.getApplicationStateController()
				.setLoadedCollectionUpdated(true);
	}
	
	public void saveChanges(SignBankVariant variant) {
		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);
		GlossChainedEventsEntity sEntity = (GlossChainedEventsEntity) mediaToolBar.getSelectedEventsEntity();

		if (sEntity == null) {
			sEntity = wrapper.getGlossField()
					.getGlossChainedEventsEntityCollection()
					.getSelectedEntity();
		}
		
		GlossEntityProperties properties = sEntity.getGlossEntityProperties();
		
		properties.setVerifiedFromSignBank(true);
		
		// compare against the defaultOccurrence - from which we get the source resource
		SignBankOccurrence defaultOccurrence = variant.getDefaultOccurrence();
		String sourceResource = defaultOccurrence.getSIgnBankResourceFileName();
		properties.setSourceResource(sourceResource);
		
		String mainGlossID = variant.getParentMainGloss().getMainGlossID();
		String variantID = variant.getVariantID();
		String occurrenceID = defaultOccurrence.getOccurrenceID();		
		properties.setMainGlossID(mainGlossID);
		properties.setVariantID(variantID);
		properties.setOccurrenceID(occurrenceID);

		ArrayList<HandShapeImage> domImages = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
		int domImagesSize = domImages.size();
		ArrayList<HandShapeImage> nondomImages = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
		int nondomImagesSize = nondomImages.size();
		
		String domStartHS = null;
		String nondomStartHS = null;
		String domEndHS = null;
		String nondomEndHS = null;
		
		if (domImagesSize > 0) {
			domStartHS = domImages.get(0).getSS3FieldValue().getLabel();
			domEndHS = domImages.get(domImagesSize - 1).getSS3FieldValue().getLabel();
		}
		
		if (nondomImagesSize > 0) {
			nondomStartHS = nondomImages.get(0).getSS3FieldValue().getLabel();
			nondomEndHS = nondomImages.get(nondomImagesSize - 1).getSS3FieldValue().getLabel();
		}
		
		String occDomStartHS = defaultOccurrence.getDomStartHS();
		String occNondomStartHS = defaultOccurrence.getNondomStartHS();
		String occDomEndHS = defaultOccurrence.getDomEndHS();
		String occNondomEndHS = defaultOccurrence.getNondomEndHS();
		
		String occLeftRightHS = defaultOccurrence.getLeftRightHS();
		String occStartEndHS = defaultOccurrence.getStartEndHS();
		String occStartEndLeftHS = defaultOccurrence.getStartEndLeftHS();
		String occStartEndRightHS = defaultOccurrence.getStartEndRightHS();

		boolean occTwoHanded = defaultOccurrence.isTwoHanded();
		boolean occMarkedHands = defaultOccurrence.isMarkedHands();
		boolean occPassiveBaseArm = defaultOccurrence.isPassiveBaseArm();
		
		String occSignType = defaultOccurrence.getSignType();
		
		boolean differingFromOccurrence = false;
		
		if (occDomStartHS != null && domStartHS != null && !occDomStartHS.equals(domStartHS))
			differingFromOccurrence = true;
		if (occNondomStartHS != null && nondomStartHS != null && !occNondomStartHS.equals(nondomStartHS))
			differingFromOccurrence = true;
		if (occDomEndHS != null && domEndHS != null && !occDomEndHS.equals(domEndHS))
			differingFromOccurrence = true;
		if (occNondomEndHS != null && nondomEndHS != null && !occNondomEndHS.equals(nondomEndHS))
			differingFromOccurrence = true;
		
		int leftRightHS = hshPanel.getSelectedSameDifLeftRightHandshapes();
		int startEndHS = hshPanel.getSelectedSameDifStartEndHandshapes();
		int startEndLeftHS = hshPanel.getSelectedSameDifStartEndHandshapesL();
		int startEndRightHS = hshPanel.getSelectedSameDifStartEndHandshapesR();
		
		boolean twoHanded = hshPanel.isTwoHandedSelected();
		boolean markedHands = hshPanel.isMarkedNumberOfHands();
		boolean passiveBaseArm = hshPanel.isPassiveBaseArm();
		
		String signType = null;
		if (isLexical())
			signType = "LEXICAL";
		if (isFingerspelled()) 
			signType = "FINGERSPELLED";
		if (isLoanSign())
			signType = "LOAN";
		if (isNumber())
			signType = "NUMBER";
		if (isClassifier())
			signType = "CLASSIFER";
		if (isGesture()) 
			signType = "GESTURE";

		if (occLeftRightHS != null) {
			if (occLeftRightHS.equals("SAME") && leftRightHS != 0) {
				differingFromOccurrence = true;
				
				if (occStartEndHS != null) {
					if (occStartEndHS.equals("SAME") && startEndHS != 0)
						differingFromOccurrence = true;
					else if (occStartEndHS.equals("DIFFERENT") && startEndHS != 1)
						differingFromOccurrence = true;
				}
			} else if (occLeftRightHS.equals("DIFFERENT") && leftRightHS != 2) {
				differingFromOccurrence = true;
				
				if (occStartEndLeftHS != null) {
					if (occStartEndLeftHS.equals("SAME") && startEndLeftHS != 0)
						differingFromOccurrence = true;
					else if (occStartEndLeftHS.equals("DIFFERENT") && startEndLeftHS != 1)
						differingFromOccurrence = true;
				}
				
				if (occStartEndRightHS != null) {
					if (occStartEndRightHS.equals("SAME") && startEndRightHS != 0)
						differingFromOccurrence = true;
					else if (occStartEndRightHS.equals("DIFFERENT") && startEndRightHS != 1)
						differingFromOccurrence = true;
				}
			}
		}
		
		if (occTwoHanded != twoHanded)
			differingFromOccurrence = true;
		
		if (occMarkedHands != markedHands)
			differingFromOccurrence = true;
		
		if (occPassiveBaseArm != passiveBaseArm)
			differingFromOccurrence = true;
		
		if ( !occSignType.equals(signType))
			differingFromOccurrence = true;
		
		properties.setDifferingFromOccurrence(differingFromOccurrence);
		
		// we need to set the same sign bank properties on the dependent (other hand) gloss entity
		// because on "file open" both hands (both SS3Track objects) will be need to be parsed separately
		// (there could be one handed signs on the nondominant hand - an unlikely but possible use case)
		GlossChainedEventsEntity dependentEntity = sEntity.getDependentGlossEntity();
		if (dependentEntity != null) {
			GlossEntityProperties dependentProperties = dependentEntity.getGlossEntityProperties();
			dependentProperties.setVerifiedFromSignBank(true);
			dependentProperties.setSourceResource(sourceResource);
			dependentProperties.setDifferingFromOccurrence(differingFromOccurrence);
			dependentProperties.setMainGlossID(mainGlossID);
			dependentProperties.setVariantID(variantID);
			dependentProperties.setOccurrenceID(occurrenceID);
		}
				
		updateChanges();
		SS3Singleton.getApplicationStateController()
				.setLoadedCollectionUpdated(true);
	}

	protected void updateChanges() {

		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);
		GlossChainedEventsEntity sEntity = (GlossChainedEventsEntity) mediaToolBar.getSelectedEventsEntity();
		
		
		if (sEntity == null) {
			sEntity = wrapper.getGlossField()
					.getGlossChainedEventsEntityCollection()
					.getSelectedEntity();
		}

		GlossChainedEventsEntity ndGlsEntity = sEntity.getDependentGlossEntity();
		GlossChainedEvent selectedTextEvent = (GlossChainedEvent) sEntity.getTextValue();
		GlossClassifications glsClassification = selectedTextEvent.getGlossClassifications();
		
		GlossEntityProperties properties = sEntity.getGlossEntityProperties();
		
		if(fingerspelled.isSelected()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_FINGERSPELLED);
		} else if(loan.isSelected()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_LOAN);
		} else if(number.isSelected()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_NUMBERS);
		} else if(lexical.isSelected()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_LEXICAL);
		} else if(classifier.isSelected()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_CLASSIFIER);
		} else if(gesture.isSelected()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_GESTURE);
		}
		
		glsClassification.setNameSign(nameSign.isSelected());
		glsClassification.setOneFrameGloss(hshPanel.isOneFrameGloss());
		glsClassification.setPassiveBaseArm(hshPanel.isPassiveBaseArm());
		glsClassification.setMarkedNamberOfHands(hshPanel.isMarkedNumberOfHands());
		
		properties.setOneFrameGloss(hshPanel.isOneFrameGloss());
		
		if(hshPanel.isTwoHandedSelected()) {
			glsClassification.setNumberOfHands(2);
            properties.setTwoHanded(true);
		} else {
			glsClassification.setNumberOfHands(1);
            properties.setTwoHanded(false);
		}
		
		glsClassification.setRightHandDominant(domHndR.isSelected());
		
		int handshapeClassification = hshPanel.getSelectedSameDifLeftRightHandshapes();
		boolean sameHandshapes = handshapeClassification == 0;

		glsClassification.setSameHandshapes(sameHandshapes);
		
		if(sameHandshapes) {
			int sameDiffStartEndHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();			
			glsClassification.setDomHandShapesSameDIfferent(sameDiffStartEndHandshapes);
			glsClassification.setNondomHandShapesSameDIfferent(sameDiffStartEndHandshapes);
		} else {
			int sameDifStartEndHandshapesL = hshPanel.getSelectedSameDifStartEndHandshapesL();
			int sameDifStartEndHandshapesR = hshPanel.getSelectedSameDifStartEndHandshapesR();
			
			if(domHndR.isSelected()) {
				glsClassification.setDomHandShapesSameDIfferent(sameDifStartEndHandshapesR);
				glsClassification.setNondomHandShapesSameDIfferent(sameDifStartEndHandshapesL);
			} else {
				glsClassification.setDomHandShapesSameDIfferent(sameDifStartEndHandshapesL);
				glsClassification.setNondomHandShapesSameDIfferent(sameDifStartEndHandshapesR);
			}
		}

        String fieldID = null;

        Field field = SS3Singleton.getMediaToolBar(null).getSelectedField();
        if(field != null) {
            fieldID = field.getFieldID();
        }

        if(fieldID == null) {
            EventsEntity ent = SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();

            if (ent == null)
                return;

            GlossChainedEventsEntity entity = (GlossChainedEventsEntity) ent;
            fieldID = entity.getField().getFieldID();
        }

		FieldWrappersCollection collection = view.getSegment().getFieldWrappersCollection();

        String newID = fieldID.equalsIgnoreCase("10000") ? "10001" : "10000";

        GlossFieldWrapper ndWrp = (GlossFieldWrapper) collection.getFieldWrapperWithFieldID(newID);

        if (hshPanel.isTwoHandedSelected()) {
			if(ndGlsEntity != null) {
				String text = sEntity.getText();
				ndGlsEntity.getTextValue().setText(text);
			} else {
				int startCoord = sEntity.getStartTimeCoordinate();
				int endCoord = sEntity.getEndTimeCoordinate();

				GlossField glsField = (GlossField) ndWrp.getField();

				ArrayList<GlossChainedEventsEntity> entities = glsField
						.getEntitiesInInterval(startCoord, endCoord);

				if (entities.isEmpty()) {
					ChainedEventsCollection chainedEventsCollection = new ChainedEventsCollection();

					int startMovieTime = selectedTextEvent.getStartTimeInfo()
							.getMovieTime();
					TimeInfo ndStartTimeInfo = new TimeInfo();
					ndStartTimeInfo.setMovieTime(startMovieTime);

					int endMovieTime = selectedTextEvent.getEndTimeInfo()
							.getMovieTime();
					TimeInfo ndEndTimeInfo = new TimeInfo();
					ndEndTimeInfo.setMovieTime(endMovieTime);

					GlossChainedEvent ndTextEvent = new GlossChainedEvent(
							sEntity.getSignStreamSegmentPanel());
					ndTextEvent.setStartTimeInfo(ndStartTimeInfo);
					ndTextEvent.setEndTimeInfo(ndEndTimeInfo);
					ndTextEvent.setText(selectedTextEvent.getText());

					chainedEventsCollection.setTextValueEvent(ndTextEvent);
					ndGlsEntity = new GlossChainedEventsEntity(
							chainedEventsCollection,
							sEntity.getSignStreamSegmentPanel(), glsField);
					ndGlsEntity.determineParentEntity();
					ndGlsEntity.setCorrespondingGlossEntityID(sEntity.getID());
					sEntity.setCorrespondingGlossEntityID(ndGlsEntity.getID());
					sEntity.setDependentGlossEntity(ndGlsEntity);
					ndGlsEntity.setDependentGlossEntity(sEntity);

					sEntity.setLockedLeftRightAlignments(true);
					ndGlsEntity.setLockedLeftRightAlignments(true);
					ndWrp.getGlossField().addGlossChainedEventsEntity(ndGlsEntity);
				} else {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CHAINS_CANNOT_OVERLAP);
					return;
				}
			}
		} else {
            // delete second gloss and related
			if (ndGlsEntity != null) {
                ndWrp.getGlossField().deleteEntity(ndGlsEntity);

                ChainedEventsEntity ndHandEntity = ndGlsEntity.getDependentHandEntity();
                HandShapeEntity ndHandShapeEntity = ndGlsEntity.getDependentHandShapeEntity();

                RightLeftHandFieldWrapper ndHandWrapper;

                if(ndWrp.getGlossField().getFieldID().equalsIgnoreCase("10000")) {
                    // right hand
                    ndHandWrapper = (RightLeftHandFieldWrapper)collection.getFieldWrapperWithFieldID("10501");
                } else {
                    // left hand
                    ndHandWrapper = (RightLeftHandFieldWrapper)collection.getFieldWrapperWithFieldID("10502");
                }

                if(ndHandEntity != null) {
                    NonManualField rl = (NonManualField)ndHandWrapper.getField();
                    rl.deleteEntity(ndHandEntity);
                }

                if(ndHandShapeEntity != null) {
                    ndHandShapeEntity.getField().deleteEntity(ndHandShapeEntity);
                }
                ndGlsEntity = null;
			}
		}
		
		GlossEntityProperties properties2 = null;

		if (ndGlsEntity != null)
			properties2 = ndGlsEntity.getGlossEntityProperties();

		if (isFingerspelled()) {
			properties.setFingerspelled();
			if (properties2 != null)
				properties2.setFingerspelled();
		} else if (isLoanSign()) {
			properties.setLoan();
			if (properties2 != null)
				properties2.setLoan();
		} else if (isNumber()) {
			properties.setNumber();
			if (properties2 != null)
				properties2.setNumber();
		} else if (isLexical()) {
			properties.setLexical();
			if (properties2 != null)
				properties2.setLexical();
		} else if (isClassifier()) {
			properties.setClassifier();
			if (properties2 != null)
				properties2.setClassifier();
		} else if (isGesture()) {
			properties.setGesture();
			if (properties2 != null)
				properties2.setGesture();
		}

		properties.setNameSign(isNameSign());
		properties.setDominamtHand(getDominantHand());
		properties.setMarkedNumberOfHands(hshPanel.isMarkedNumberOfHands());
		properties.setPassiveBaseArm(hshPanel.isPassiveBaseArm());

		if (properties2 != null) {
			properties2.setNameSign(isNameSign());
			properties2.setDominamtHand(getDominantHand());
			properties2
					.setMarkedNumberOfHands(hshPanel.isMarkedNumberOfHands());
			properties2.setPassiveBaseArm(hshPanel.isPassiveBaseArm());
		}

		String text = variantField.getText();

		sEntity.getGlossChainedEvent().setText(text);

		if (ndGlsEntity != null) {
			ndGlsEntity.getGlossChainedEvent().setText(text);
		}

        HandShapeBean hspBean = hshPanel.getHandShapeBean();

	    addOrUpdateHandShapes(hspBean, sEntity, collection);

		if (ndGlsEntity != null) {
			Event event = sEntity.getValueOrTextValueEvent();
			ChainedEvent onset = sEntity.getOnset();
			ChainedEvent initialHold = sEntity.getInitialHold();
			ChainedEvent offset = sEntity.getOffset();
			ChainedEvent finalHold = sEntity.getFinalHold();

			Event ndEvent = ndGlsEntity.getValueOrTextValueEvent();
			ChainedEvent ndOnset = ndGlsEntity.getOnset();
			ChainedEvent ndInitialHold = ndGlsEntity.getInitialHold();
			ChainedEvent ndOffset = ndGlsEntity.getOffset();
			ChainedEvent ndFinalHold = ndGlsEntity.getFinalHold();

			if (sEntity.isLockedLeftRightAlignments() && ndGlsEntity.isLockedLeftRightAlignments()) {
				ndEvent.getStartTimeInfo().setMovieTime(
						event.getStartTimeInfo().getMovieTime());
				ndEvent.getEndTimeInfo().setMovieTime(
						event.getEndTimeInfo().getMovieTime());

				SignStreamSegmentPanel panel = wrapper.getField()
						.getSignStreamSegmentPanel();

				if (initialHold != null) {
					if (ndInitialHold != null) {
						ndInitialHold.getStartTimeInfo().setMovieTime(
								initialHold.getStartTimeInfo().getMovieTime());
						ndInitialHold.getEndTimeInfo().setMovieTime(
								initialHold.getEndTimeInfo().getMovieTime());
					} else {
						ndInitialHold = new ChainedEvent(panel);
						ndInitialHold.setStartTimeInfo(initialHold
								.getStartTimeInfo().cloneTimeInfo());
						ndInitialHold.setEndTimeInfo(initialHold.getEndTimeInfo()
								.cloneTimeInfo());
						ndGlsEntity.setInitialHoldEvent(ndInitialHold);
					}
				} else {
					ndGlsEntity.setInitialHoldEvent(null);
				}

				if (onset != null) {
					if (ndOnset != null) {
						ndOnset.getStartTimeInfo().setMovieTime(
								onset.getStartTimeInfo().getMovieTime());
						ndOnset.getEndTimeInfo().setMovieTime(
								onset.getEndTimeInfo().getMovieTime());
					} else {
						ndOnset = new ChainedEvent(panel);
						ndOnset.setStartTimeInfo(onset.getStartTimeInfo()
								.cloneTimeInfo());
						ndOnset.setEndTimeInfo(onset.getEndTimeInfo()
								.cloneTimeInfo());
						ndGlsEntity.setOnsetEvent(ndOnset);
					}
				} else {
					ndGlsEntity.setOnsetEvent(null);
				}

				if (finalHold != null) {
					if (ndFinalHold != null) {
						ndFinalHold.getStartTimeInfo().setMovieTime(
								finalHold.getStartTimeInfo().getMovieTime());
						ndFinalHold.getEndTimeInfo().setMovieTime(
								finalHold.getEndTimeInfo().getMovieTime());
					} else {
						ndFinalHold = new ChainedEvent(panel);
						ndFinalHold.setStartTimeInfo(finalHold.getStartTimeInfo()
								.cloneTimeInfo());
						ndFinalHold.setEndTimeInfo(finalHold.getEndTimeInfo()
								.cloneTimeInfo());
						ndGlsEntity.setFinalHoldEvent(ndFinalHold);
					}
				} else {
					ndGlsEntity.setFinalHoldEvent(null);
				}

				if (offset != null) {
					if (ndOffset != null) {
						ndOffset.getStartTimeInfo().setMovieTime(
								offset.getStartTimeInfo().getMovieTime());
						ndOffset.getEndTimeInfo().setMovieTime(
								offset.getEndTimeInfo().getMovieTime());
					} else {
						ndOffset = new ChainedEvent(panel);
						ndOffset.setStartTimeInfo(offset.getStartTimeInfo()
								.cloneTimeInfo());
						ndOffset.setEndTimeInfo(offset.getEndTimeInfo()
								.cloneTimeInfo());
						ndGlsEntity.setOffsetEvent(ndOffset);
					}
				} else {
					ndGlsEntity.setOffsetEvent(null);
				}

			}
		}		
		
		properties.clearHandshapeInformation();
		
		if(hshPanel.isTwoHandedSelected()) {
			properties2.clearHandshapeInformation();
			
			if(handshapeClassification == 0) {
				properties.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_LR);
				properties2.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_LR);
				int sameDiffHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
				if(sameDiffHandshapes == 0) {
					properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
			} else if(handshapeClassification == 1) {
				properties.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR);
				properties2.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR);
				int sameDiffHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
				if(sameDiffHandshapes == 0) {
					properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
			} else {
				properties.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR);
				properties2.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR);
				
				int selectedStartEndHandshapeR = hshPanel.getSelectedSameDifStartEndHandshapesR();
				int selectedStartEndHandshapeL = hshPanel.getSelectedSameDifStartEndHandshapesL();
				
				if(selectedStartEndHandshapeR == 0) {
					properties.setStartEndHandshapeRightHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeRightHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshapeRightHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeRightHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
				
				if(selectedStartEndHandshapeL == 0) {
					properties.setStartEndHandshapeLeftHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeLeftHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshapeLeftHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeLeftHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
			} 
		} else {
			int sameDiffHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
			if(sameDiffHandshapes == 0) {
				properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
			} else {
				properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
			}
		}

        GlossEntityProperties tProperties = sEntity.getGlossEntityProperties();

        if(!tProperties.isTwoHanded()) {
            sEntity.setDependentGlossEntity(null);
            sEntity.setCorrespondingGlossEntityID(null);
        }

        if (ndGlsEntity != null && sEntity.isLockedGlossPhonAlignments()) { //check if locked to hand shape
            Event initialHold = ndGlsEntity.getInitialHold();
            Event finalHold = ndGlsEntity.getFinalHold();
//            
            if(initialHold != null) {
                int initialStartMovieTime = initialHold.getStartTimeInfo().getMovieTime();
                ndGlsEntity.getDependentHandEntity().getStartTimeInfo().setMovieTime(initialStartMovieTime);
                ndGlsEntity.getDependentHandShapeEntity().getFirstHandShapeEvent().getStartTimeInfo().setMovieTime(initialStartMovieTime);
                ndGlsEntity.getDependentHandShapeEntity().getFirstHandShapeEvent().getEndTimeInfo().setMovieTime(initialStartMovieTime);
            }
            if(finalHold != null) {
                int finalEndMovieTime = finalHold.getEndTimeInfo().getMovieTime();
                ndGlsEntity.getDependentHandEntity().getEndTimeInfo().setMovieTime(finalEndMovieTime);
                ndGlsEntity.getDependentHandShapeEntity().getLastHandShapeEvent().getStartTimeInfo().setMovieTime(finalEndMovieTime);
                ndGlsEntity.getDependentHandShapeEntity().getLastHandShapeEvent().getEndTimeInfo().setMovieTime(finalEndMovieTime);
            }
        }

        hshPanel.getSelectedRightHandshapesPane().removeAll();
		hshPanel.getSelectedLeftHandshapesPane().removeAll();

		wrapper.getSignStreamSegmentPanel().repaint();
		wrapper.getSignStreamSegmentPanel().updateLoadedDatabase();
		wrapper.getSignStreamSegmentPanel().getUtteranceView().updateUtteranceLists();
	}

	private void addOrUpdateHandShapes(HandShapeBean bean,
                                       GlossChainedEventsEntity entity, FieldWrappersCollection collection) {

		GlossChainedEventsEntity ndEntity = entity.getDependentGlossEntity();
		String fieldID = entity.getField().getFieldID();
		
		// 10000 - right handshapes, 10001 - left handshapes

		ArrayList<HandShapeImage> rHndShapeImgs = new ArrayList<HandShapeImage>(bean
				.getSelectedRightHandShapeImages());
		ArrayList<HandShapeImage> lHndShapeImgs = new ArrayList<HandShapeImage>(bean
				.getSelectedLeftHandShapeImages());

		ChainedEventsEntity rhEntity, lhEntity;
		HandShapeEntity rHshEntity, lHshEntity;

		SignStreamSegmentPanel segmentPanel = entity.getSignStreamSegmentPanel();
		
		ChainedEventsEntity handEntity = entity.getDependentHandEntity();
		HandShapeEntity handShapeEntity = entity.getDependentHandShapeEntity();
		GlossChainedEventsEntity ndGlsEntity = entity.getDependentGlossEntity();
		ChainedEventsEntity ndHandEntity = null;
		HandShapeEntity ndHandShapeEntity = null;
		
		if(ndGlsEntity != null) {
			ndHandEntity = ndGlsEntity.getDependentHandEntity();
			ndHandShapeEntity = ndGlsEntity.getDependentHandShapeEntity();
		}
		
		RightLeftHandFieldWrapper rightHandWrapper = (RightLeftHandFieldWrapper) collection
				.getFieldWrapper(Constants.TYPE_RIGHT_HAND);
		HandShapeFieldWrapper rightHandShapeWrapper = (HandShapeFieldWrapper) collection
				.getFieldWrapper(Constants.TYPE_RIGHT_HAND_SHAPE);

		RightLeftHandFieldWrapper leftHandWrapper = (RightLeftHandFieldWrapper) collection
				.getFieldWrapper(Constants.TYPE_LEFT_HAND);
		HandShapeFieldWrapper leftHandShapeWrapper = (HandShapeFieldWrapper) collection
				.getFieldWrapper(Constants.TYPE_LEFT_HAND_SHAPE);
		
		
		if(!rHndShapeImgs.isEmpty()) {
			// add/update handshapes on right hand
			if(fieldID.equals("10000")) {
				if(handEntity != null) {
					handEntity.getTextValue().setText(entity.getText());
				}
				if(handShapeEntity != null) {
					ChainedEvent value = entity.getTextValue();
					ChainedEvent initHold = entity.getInitialHold();
					ChainedEvent finalHold = entity.getFinalHold();

					TimeInfo startTimeInfo; // = null;
					TimeInfo endTimeInfo; // = null;

					if (initHold != null) {
						startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
					} else {
						startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
					}

					if (finalHold != null) {
						endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
					} else {
						endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
					}

					SS3Entity ent = createSS3Entity(rHndShapeImgs, startTimeInfo, endTimeInfo);	
					handShapeEntity.resetEntity(ent);					
				} else {
					// add entity to Right Hand field
					rhEntity = addHandEntity(rightHandWrapper, entity, segmentPanel);

					rHshEntity = createHandShapeEntity(bean, rHndShapeImgs, rhEntity,
							rightHandShapeWrapper.getHandShapeField());

					rightHandShapeWrapper.getHandShapeField().addHandShapeEntity(rHshEntity);
					entity.setDependentHandEntity(rhEntity);
					entity.setDependentHandShapeEntity(rHshEntity);
					
					entity.setLockedGlossPhonAlignments(true);
					rhEntity.setLockedGlossPhonAlignments(true);
					rHshEntity.setLockedGlossPhonAlignments(true);
				}
			} else {
				if(ndHandEntity != null) {
					ndEntity.getTextValue().setText(entity.getText());
				}
				if(ndHandShapeEntity != null) {
					ChainedEvent value = ndEntity.getTextValue();
					ChainedEvent initHold = ndEntity.getInitialHold();
					ChainedEvent finalHold = ndEntity.getFinalHold();

					TimeInfo startTimeInfo; // = null;
					TimeInfo endTimeInfo; // = null;

					if (initHold != null) {
						startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
					} else {
						startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
					}

					if (finalHold != null) {
						endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
					} else {
						endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
					}
					
					SS3Entity ent = createSS3Entity(rHndShapeImgs, startTimeInfo, endTimeInfo);	
					ndHandShapeEntity.resetEntity(ent);					
				} else {
					// add entity to Right Hand field
					rhEntity = addHandEntity(rightHandWrapper, ndEntity, segmentPanel);

					rHshEntity = createHandShapeEntity(bean, rHndShapeImgs, rhEntity,
							rightHandShapeWrapper.getHandShapeField());

					rightHandShapeWrapper.getHandShapeField().addHandShapeEntity(rHshEntity);
					ndEntity.setDependentHandEntity(rhEntity);
					ndEntity.setDependentHandShapeEntity(rHshEntity);
					
					ndEntity.setLockedGlossPhonAlignments(true);
					rhEntity.setLockedGlossPhonAlignments(true);
					rHshEntity.setLockedGlossPhonAlignments(true);
				}
			}
		} else {
			// delete handshapes
			if(fieldID.equals("10000")) {
				if(handEntity != null) {
					rightHandWrapper.getField().deleteEntity(handEntity);
				}
			
				if(handShapeEntity != null) {
					rightHandShapeWrapper.getHandShapeField().deleteEntity(handShapeEntity);	
				}
				
				entity.setDependentHandEntity(null);
				entity.setDependentHandShapeEntity(null);
			} else {
				if(ndHandEntity != null) {
					rightHandWrapper.getField().deleteEntity(handEntity);
				}
				if(ndHandShapeEntity != null) {
					rightHandShapeWrapper.getHandShapeField().deleteEntity(handShapeEntity);	
				}
				
				if(ndEntity != null) {
					ndEntity.setDependentHandEntity(null);
					ndEntity.setDependentHandShapeEntity(null);
				}
			}
		}	
		
		if(!lHndShapeImgs.isEmpty()) {
			// add/update handshapes on right hand
			if(fieldID.equals("10001")) {
				if(handEntity != null) {
					handEntity.getTextValue().setText(entity.getText());
				}
				if(handShapeEntity != null) {
					ChainedEvent value = entity.getTextValue();
					ChainedEvent initHold = entity.getInitialHold();
					ChainedEvent finalHold = entity.getFinalHold();

					TimeInfo startTimeInfo; // = null;
					TimeInfo endTimeInfo; // = null;

					if (initHold != null) {
						startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
					} else {
						startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
					}

					if (finalHold != null) {
						endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
					} else {
						endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
					}
					
					SS3Entity ent = createSS3Entity(lHndShapeImgs, startTimeInfo, endTimeInfo);	
					handShapeEntity.resetEntity(ent);					
				} else {
					// add entity to Right Hand field
					lhEntity = addHandEntity(leftHandWrapper, entity, segmentPanel);

					lHshEntity = createHandShapeEntity(bean, lHndShapeImgs, lhEntity,
							leftHandShapeWrapper.getHandShapeField());

					leftHandShapeWrapper.getHandShapeField().addHandShapeEntity(lHshEntity);
					entity.setDependentHandEntity(lhEntity);
					entity.setDependentHandShapeEntity(lHshEntity);
					
					entity.setLockedGlossPhonAlignments(true);
					lhEntity.setLockedGlossPhonAlignments(true);
					lHshEntity.setLockedGlossPhonAlignments(true);
				}
			} else {
                // assert ndEntity != null;
                if(ndHandEntity != null) {
					ndHandEntity.getTextValue().setText(entity.getText());
				}
				if(ndHandShapeEntity != null) {
					ChainedEvent value = ndEntity.getTextValue();
					ChainedEvent initHold = ndEntity.getInitialHold();
					ChainedEvent finalHold = ndEntity.getFinalHold();

					TimeInfo startTimeInfo; // = null;
					TimeInfo endTimeInfo; // = null;

					if (initHold != null) {
						startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
					} else {
						startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
					}

					if (finalHold != null) {
						endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
					} else {
						endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
					}
					
					SS3Entity ent = createSS3Entity(lHndShapeImgs, startTimeInfo, endTimeInfo);	
					ndHandShapeEntity.resetEntity(ent);					
				} else {
					// add entity to Right Hand field
					lhEntity = addHandEntity(leftHandWrapper, ndEntity, segmentPanel);

					lHshEntity = createHandShapeEntity(bean, lHndShapeImgs, lhEntity,
							leftHandShapeWrapper.getHandShapeField());

					leftHandShapeWrapper.getHandShapeField().addHandShapeEntity(lHshEntity);

                    if(ndEntity != null) {
					    ndEntity.setDependentHandEntity(lhEntity);
					    ndEntity.setDependentHandShapeEntity(lHshEntity);
					    ndEntity.setLockedGlossPhonAlignments(true);
                    }

					lhEntity.setLockedGlossPhonAlignments(true);
					lHshEntity.setLockedGlossPhonAlignments(true);
				}
			}
		} else {
			// delete handshapes
			if(fieldID.equals("10001")) {
				if(handEntity != null) {
					leftHandWrapper.getField().deleteEntity(handEntity);
				}
			
				if(handShapeEntity != null) {
					leftHandShapeWrapper.getHandShapeField().deleteEntity(handShapeEntity);	
				}
				
				entity.setDependentHandEntity(null);
				entity.setDependentHandShapeEntity(null);
			} else {
				if(ndHandEntity != null) {
					leftHandWrapper.getField().deleteEntity(ndHandEntity);
				}
				if(ndHandShapeEntity != null) {
					leftHandShapeWrapper.getHandShapeField().deleteEntity(ndHandShapeEntity);	
				}
				
				if(ndEntity != null) {
					ndEntity.setDependentHandEntity(null);
					ndEntity.setDependentHandShapeEntity(null);
				}
			}
		}	
	}

	private SS3Entity createSS3Entity(ArrayList<HandShapeImage> handShapeImgs,
			TimeInfo startTimeInfo, TimeInfo endTimeInfo) {
		SS3Entity ent = new SS3Entity();

		if (handShapeImgs.size() == 1) {
			HandShapeImage img = handShapeImgs.get(0);
			SS3FieldValue value = img.getSS3FieldValue();

			SS3Item item1 = new SS3Item(value.getId(), value.getIid(),
					startTimeInfo, endTimeInfo);
			ent.addItem(item1);
		} else {

			int startTime = startTimeInfo.getMovieTime();
			HandShapeImage img1 = handShapeImgs.get(0);
			SS3FieldValue value1 = img1.getSS3FieldValue();

			SS3Item item1 = new SS3Item(value1.getId(), value1.getIid(),
					startTimeInfo, startTimeInfo.cloneTimeInfo());
			ent.addItem(item1);

			int step = (endTimeInfo.getMovieTime() - startTimeInfo
					.getMovieTime()) / (handShapeImgs.size() - 1);

			for (int i = 1; i < handShapeImgs.size() - 1; i++) {
				HandShapeImage img = handShapeImgs.get(i);
				SS3FieldValue value = img.getSS3FieldValue();
				TimeInfo startTimeInfo1 = startTimeInfo.cloneTimeInfo();
				startTime = startTime + step;
				startTimeInfo1.setMovieTime(startTime);
				SS3Item item3 = new SS3Item(value.getId(), value.getIid(),
						startTimeInfo1, startTimeInfo1.cloneTimeInfo());
				ent.addItem(item3);
			}

			HandShapeImage img2 = handShapeImgs.get(handShapeImgs.size() - 1);
			SS3FieldValue value2 = img2.getSS3FieldValue();
			TimeInfo endTimeInfo2 = endTimeInfo.cloneTimeInfo();
			SS3Item item4 = new SS3Item(value2.getId(), value2.getIid(),
					endTimeInfo2, endTimeInfo2.cloneTimeInfo());
			ent.addItem(item4);
		}
		return ent;
	}

	private HandShapeEntity createHandShapeEntity(HandShapeBean bean,
			ArrayList<HandShapeImage> handShapeImages,
			ChainedEventsEntity superEntity, Field field) {

		SignStreamSegmentPanel segmentPanel = superEntity
				.getSignStreamSegmentPanel();

		HandShapeEntity hsEntity = new HandShapeEntity(segmentPanel, field);
		hsEntity.setID(Util.getUniqueNumber());

		hsEntity.setField(field);
		hsEntity.setHandShapeType(bean.getType());
		hsEntity.setRefEntityId(superEntity.getID());
		HandShapeSubEntity subEntity = new HandShapeSubEntity(segmentPanel, field);
		subEntity.setID(hsEntity.getID());
		hsEntity.addHandShapeSubEntity(subEntity);

		ChainedEvent event = superEntity.getTextValue();

		int startTime = event.getStartTimeInfo().getMovieTime();

		if (handShapeImages.size() == 1) {
			HandShapeImage img = handShapeImages.get(0);
			SS3FieldValue value = img.getSS3FieldValue();
			String vid = value.getId();

			SS3Item item1 = new SS3Item();
			item1.setId(vid);
			item1.setImageID(value.getIid());

			TimeInfo startTimeInfo1 = event.getStartTimeInfo().cloneTimeInfo();
			TimeInfo startTimeInfo2 = event.getEndTimeInfo().cloneTimeInfo();
			item1.setStartTimeInfo(startTimeInfo1);
			item1.setEndTimeInfo(startTimeInfo2);
			subEntity.addHandShape(item1);
		} else if (handShapeImages.size() > 1) {
			HandShapeImage img1 = handShapeImages.get(0);
			SS3FieldValue value1 = img1.getSS3FieldValue();
			String vid1 = value1.getId();

			SS3Item item1 = new SS3Item();
			item1.setId(vid1);
			item1.setImageID(value1.getIid());

			TimeInfo startTimeInfo1 = event.getStartTimeInfo().cloneTimeInfo();
			item1.setStartTimeInfo(startTimeInfo1);
			item1.setEndTimeInfo(startTimeInfo1.cloneTimeInfo());
			subEntity.addHandShape(item1);

			int step = (event.getEndTimeInfo().getMovieTime() - event
					.getStartTimeInfo().getMovieTime())
					/ (handShapeImages.size() - 1);

			for (int i = 1; i < handShapeImages.size() - 1; i++) {
				HandShapeImage img = handShapeImages.get(i);
				SS3FieldValue value = img.getSS3FieldValue();
				String vid = value.getId();
				SS3Item item = new SS3Item();
				item.setId(vid);
				item.setImageID(value.getIid());

				TimeInfo startTimeInfo = event.getStartTimeInfo()
						.cloneTimeInfo();
				startTimeInfo.setMovieTime(startTime + i * step);
				item.setStartTimeInfo(startTimeInfo);
				item.setEndTimeInfo(startTimeInfo.cloneTimeInfo());

				subEntity.addHandShape(item);
			}

			HandShapeImage img2 = handShapeImages
					.get(handShapeImages.size() - 1);
			SS3FieldValue value2 = img2.getSS3FieldValue();
			String vid2 = value2.getId();

			SS3Item item2 = new SS3Item();
			item2.setId(vid2);
			item2.setImageID(value2.getIid());

			TimeInfo startTimeInfo2 = event.getEndTimeInfo().cloneTimeInfo();
			item2.setStartTimeInfo(startTimeInfo2);
			item2.setEndTimeInfo(startTimeInfo2.cloneTimeInfo());
			subEntity.addHandShape(item2);
		}

		hsEntity.setLockedGlossPhonAlignments(true);
		hsEntity.highlight();

		subEntity.determineParentEntity();
		hsEntity.determineParentEntity();

		return hsEntity;
	}

	private ChainedEventsEntity addHandEntity(RightLeftHandFieldWrapper wrp,
			GlossChainedEventsEntity glsEntity, SignStreamSegmentPanel panel) {

		ChainedEvent value = glsEntity.getTextValue();
		ChainedEvent initHold = glsEntity.getInitialHold();
		ChainedEvent finalHold = glsEntity.getFinalHold();

		TimeInfo startTimeInfo; // = null;
		TimeInfo endTimeInfo; // = null;

		glsEntity.setLockedGlossPhonAlignments(true);

		if (initHold != null) {
			startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
		} else {
			startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
		}

		if (finalHold != null) {
			endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
		} else {
			endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
		}

		ChainedEventsCollection collection = new ChainedEventsCollection();

		GlossChainedEvent gEvent = new GlossChainedEvent(panel);
		gEvent.setStartTimeInfo(startTimeInfo);
		gEvent.setEndTimeInfo(endTimeInfo);
		gEvent.setText(value.getText());
		collection.setTextValueEvent(gEvent);

		ChainedEventsEntity handEntity = new ChainedEventsEntity(collection,
				panel, wrp.getField());

		handEntity.setID(Util.getUniqueNumber());
		handEntity.setField(wrp.getField());
		handEntity.determineParentEntity();

		wrp.addEventsEntity(handEntity);
		handEntity.setRefEntityId(glsEntity.getID());
		handEntity.highlight();

		handEntity.setLockedGlossPhonAlignments(true);
		return handEntity;
	}
	
	public void addToSignBank(String mainGlossStr, String variantStr) {
		SignBankCollection collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
		

		// check first if this sign exists in collection already - we will disallow if so
		SignBankVariant matchingVariant = collection.contains(mainGlossStr, variantStr);
		if (matchingVariant != null) {
			String warningDialogText =
					"Sign already found in bank.\nPlease enter a different set of values";
			String warningTitleText = "Duplicate Sign";
			int optionType = JOptionPane.YES_NO_OPTION;
			int messageType = JOptionPane.ERROR_MESSAGE;
			String options[] = {"OK"};
			
			try {
				JOptionPane.showOptionDialog(this, 
						warningDialogText, 
						warningTitleText, 
						optionType, 
						messageType, 
						null, 
						options, 
						options[0]);
			} catch (HeadlessException he) {
				System.out.println("");
			}
			
			return;
		}
			
		// next check that there is at least one handshape on the dominant (right) hand before allowing the addition
		ArrayList<HandShapeImage> rightHandShapeImages = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
		if (rightHandShapeImages.size() < 1) {
			String warningDialogText =
					"You must enter at least one handshape for the dominant hand before adding to the Sign Bank.";
			String warningTitleText = "Select Handshapes";
			int optionType = JOptionPane.YES_NO_OPTION;
			int messageType = JOptionPane.ERROR_MESSAGE;
			String options[] = {"OK"};
			
			try {
				JOptionPane.showOptionDialog(this, 
						warningDialogText, 
						warningTitleText, 
						optionType, 
						messageType, 
						null, 
						options, 
						options[0]);
			} catch (HeadlessException he) {
				System.out.println("");
			}
			
			return;
		}
		
		// and when two-handed is selected, it means you must be intending to add both hands, so check nondominant (left) handshapes as well
		if (hshPanel.isTwoHandedSelected()) {
			ArrayList<HandShapeImage> leftHandShapeImages = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
			if (leftHandShapeImages.size() < 1) {
				String warningDialogText =
						"You must enter at least one handshape for the nondominant hand before adding to the Sign Bank.";
				String warningTitleText = "Select Handshapes";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String options[] = {"OK"};
				
				try {
					JOptionPane.showOptionDialog(this, 
							warningDialogText, 
							warningTitleText, 
							optionType, 
							messageType, 
							null, 
							options, 
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}
				
				return;
			}
		}
		
		String resourceFilename = Util.SIGN_BANK_DEFAULT_LOCAL_FILE;
		
//		SignBankResource resource;
//		String resourceFilename = (String) resourceSelector.getSelectedItem();
//		resource = resourceCollection.getResourceByFileName(resourceFilename);
		
		String mgID = Util.getUniqueNumber();
		String varID = Util.getUniqueNumber();
		String occID = Util.getUniqueNumber();
		
		// check whether ID already taken in SB collection - and if so get a new one
		while (collection.getOccurrenceWithID(mgID, varID, occID) != null) {
			occID = Util.getUniqueNumber();
		}
		
		// if there is not an existing main gloss with the label given, then create a new main gloss object to save
		SignBankMainGloss mainGloss = collection.getMainGlossesList().get(mainGlossStr);
		if (mainGloss == null) {
			mainGloss = new SignBankMainGloss(mgID, mainGlossStr.toUpperCase(), varID);
		}
		SignBankVariant newVariant = new SignBankVariant(mainGloss, varID, variantStr, occID);
		SignBankOccurrence newOccurrence = new SignBankOccurrence(newVariant, occID);
		
		// get data points to set in the occurrence object

		ArrayList<HandShapeImage> domImages = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
		int domImagesSize = domImages.size();
		ArrayList<HandShapeImage> nondomImages = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
		int nondomImagesSize = nondomImages.size();
		
		String domStartHS = null;
		String nondomStartHS = null;
		String domEndHS = null;
		String nondomEndHS = null;
		
		if (domImagesSize > 0) {
			domStartHS = domImages.get(0).getSS3FieldValue().getLabel();
			domEndHS = domImages.get(domImagesSize - 1).getSS3FieldValue().getLabel();
		}
		
		if (nondomImagesSize > 0) {
			nondomStartHS = nondomImages.get(0).getSS3FieldValue().getLabel();
			nondomEndHS = nondomImages.get(nondomImagesSize - 1).getSS3FieldValue().getLabel();
		}
		
		String participant = null;
		
		String signType = null;
		if (isLexical())
			signType = "LEXICAL";
		if (isFingerspelled()) 
			signType = "FINGERSPELLED";
		if (isLoanSign())
			signType = "LOAN";
		if (isNumber())
			signType = "NUMBER";
		if (isClassifier())
			signType = "CLASSIFER";
		if (isGesture()) 
			signType = "GESTURE";

		String twoHanded = hshPanel.isTwoHandedSelected() ? "T" : "F";

		String passiveBaseArm = hshPanel.isPassiveBaseArm() ? "T" : "F";

		String markedHands = hshPanel.isMarkedNumberOfHands() ? "T" : "F";
		
		int leftRightHSIndex = hshPanel.getSelectedSameDifLeftRightHandshapes();
		int startEndHSIndex = hshPanel.getSelectedSameDifStartEndHandshapes();
		int startEndLeftHSIndex = hshPanel.getSelectedSameDifStartEndHandshapesL();
		int startEndRightHSIndex = hshPanel.getSelectedSameDifStartEndHandshapesR();
		
		String leftRightHS = leftRightHSIndex == 0 ? "SAME" : "DIFFERENT";
		String startEndHS = "", startEndLeftHS = "", startEndRightHS = "";
		
		if (leftRightHSIndex == 0) {	// if left right is the same
			startEndHS = startEndHSIndex == 0 ? "SAME" : "DIFFERENT";
		} else {	// if left right is different
			startEndLeftHS = startEndLeftHSIndex == 0 ? "SAME" : "DIFFERENT";
			startEndRightHS = startEndRightHSIndex == 0 ? "SAME" : "DIFFERENT";
		}
		
		
		newOccurrence.setSignBankResourceFileName(resourceFilename);
		newOccurrence.setDomStartHS(domStartHS);
		newOccurrence.setNondomStartHS(nondomStartHS);
		newOccurrence.setDomEndHS(domEndHS);
		newOccurrence.setNondomEndHS(nondomEndHS);
		newOccurrence.setParticipant(participant);
		newOccurrence.setSignType(signType);
		newOccurrence.setTwoHanded(twoHanded);
		newOccurrence.setPassiveBaseArm(passiveBaseArm);
		newOccurrence.setMarkedHands(markedHands);
		newOccurrence.setLeftRightHS(leftRightHS);
		newOccurrence.setStartEndHS(startEndHS);
		newOccurrence.setStartEndLeftHS(startEndLeftHS);
		newOccurrence.setStartEndRightHS(startEndRightHS);
		
		newVariant.getOccurrencesList().put(newOccurrence.getOccurrenceID(), newOccurrence);
		mainGloss.getVariantsList().put(newVariant.getVariantLabel(), newVariant);
		
		// no harm in overwriting the same main gloss with an additional variant
		collection.getMainGlossesList().put(mainGloss.getMainGlossLabel(), mainGloss);
		
		// write file now
		SS3Singleton.getSignBankWriter().writeResourceToSignBank(collection, resourceFilename);
		
		// re-parse the entire sign bank collection fresh, application class reference will be set
		SS3Singleton.getSignStreamApplication().parseSignBank();
		
		// notify the user of a successful addition to the sign bank
		String warningDialogText =
				"Successfully added to sign bank.";
		String warningTitleText = "Added Sign";
		int optionType = JOptionPane.YES_NO_OPTION;
		int messageType = JOptionPane.ERROR_MESSAGE;
		String options[] = {"OK"};
		
		try {
			JOptionPane.showOptionDialog(this, 
					warningDialogText, 
					warningTitleText, 
					optionType, 
					messageType, 
					null, 
					options, 
					options[0]);
		} catch (HeadlessException he) {
			System.out.println("");
		}
	}
	
	public void addToSignBank() {
		String mainGlossStr = primaryEntryField.getText();
		String untrimmedVariantStr = variantField.getText();
		
		addToSignBank(mainGlossStr, untrimmedVariantStr);
	}
	
	/**
	 * To be called by the sign bank panel exclusively.
	 * Take the occurrence object passed in and set the values on this panel accordingly.
	 * @param signBankOccurrence The sign bank occurrence object from the sign bank results list
	 */
	public void populateFromSignBankOccurrence(SignBankOccurrence signBankOccurrence) {
		SignBankVariant parentVariant = signBankOccurrence.getParentVariant();
		SignBankMainGloss parentMainGloss = parentVariant.getParentMainGloss();
		
		String variantLabel = parentVariant.getVariantLabel();	
		String mainGlossLabel = parentMainGloss.getMainGlossLabel();
		
		String domStartHS = signBankOccurrence.getDomStartHS();
		String nondomStartHS = signBankOccurrence.getNondomStartHS();
		String domEndHS = signBankOccurrence.getDomEndHS();
		String nondomEndHS = signBankOccurrence.getNondomEndHS();
		String signType = signBankOccurrence.getSignType();
		boolean twoHanded = signBankOccurrence.isTwoHanded();
		boolean passiveBaseArm = signBankOccurrence.isPassiveBaseArm();
		boolean markedHands = signBankOccurrence.isMarkedHands();
		String leftRightHS = signBankOccurrence.getLeftRightHS();
		String startEndHS = signBankOccurrence.getStartEndHS();
		String startEndLeftHS = signBankOccurrence.getStartEndLeftHS();
		String startEndRightHS = signBankOccurrence.getStartEndRightHS();
		
		variantField.setText(variantLabel);
		
		primaryEntryField.setText(mainGlossLabel);
		
		if (signType != null) {
			if (signType.equals("LEXICAL"))
				lexical.doClick();
			if (signType.equals("FINGERSPELLED"))
				fingerspelled.doClick();
			if (signType.equals("LOAN"))
				loan.doClick();
			if (signType.equals("NUMBER"))
				number.doClick();
			if (signType.equals("CLASSIFIER"))
				classifier.doClick();
			if (signType.equals("GESTURE"))
				gesture.doClick();
		}
		
		if (twoHanded)
			hshPanel.selectTwoHanded();
		else
			hshPanel.selectOneHanded();
		
		hshPanel.setPassiveBaseArmSelected(passiveBaseArm);
		
		hshPanel.setMarkedNumberOfHands(markedHands);
		
		if (leftRightHS != null) {
			if (leftRightHS.equals("SAME")) {
				hshPanel.setSelectedSameDifLeftRightHandshapes(0);
				
				if (startEndHS != null) {
					if (startEndHS.equals("SAME"))
						hshPanel.setSelectedSameDifStartEndHandShapesIndex(0);
					else
						hshPanel.setSelectedSameDifStartEndHandShapesIndex(1);
				}
			} else if (leftRightHS.equals("DIFFERENT")) {
				hshPanel.setSelectedSameDifLeftRightHandshapes(2);
				
				if (startEndLeftHS != null) {
					if (startEndLeftHS.equals("SAME"))
						hshPanel.setSelectedSameDifStartEndHandshapesLeftIndex(0);
					else
						hshPanel.setSelectedSameDifStartEndHandshapesLeftIndex(1);
				}
				
				if (startEndRightHS != null) {
					if (startEndRightHS.equals("SAME"))
						hshPanel.setSelectedSameDifStartEndHandshapesRightIndex(0);
					else
						hshPanel.setSelectedSameDifStartEndHandshapesRightIndex(1);
				}
			}
		}

		SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		ArrayList<HandShapeImage> selectedHandImagesRight = new ArrayList<HandShapeImage>();
		ArrayList<HandShapeImage> selectedHandImagesLeft = new ArrayList<HandShapeImage>();
		
		ArrayList<SS3FieldValue> rightFieldValues = cs.getFieldImg(Util.RIGHT_HANDSHAPE_FIELD_ID);
		ArrayList<SS3FieldValue> leftFieldValues = cs.getFieldImg(Util.LEFT_HANDSHAPE_FIELD_ID);
        
        SS3FieldValue rightHandStartValue = null, rightHandEndValue = null, leftHandStartValue = null, leftHandEndValue = null;

		// right start hs		
		rightHandStartValue = Util.matchSignBank(rightFieldValues, domStartHS);		
		
		if (rightHandStartValue != null) {
			HandShapeImage image = new HandShapeImage(rightHandStartValue, cs);
			selectedHandImagesRight.add(image);
		}
		
		// right end hs
		rightHandEndValue = Util.matchSignBank(rightFieldValues, domEndHS);
		
		
		if (rightHandEndValue != null) {
			HandShapeImage image = new HandShapeImage(rightHandEndValue, cs);
			selectedHandImagesRight.add(image);
		}

		// left start hs
		leftHandStartValue = Util.matchSignBank(leftFieldValues, nondomStartHS);
		
		if (leftHandStartValue != null) {
			HandShapeImage image = new HandShapeImage(leftHandStartValue, cs);
			selectedHandImagesLeft.add(image);
		}

		// left end hs
		leftHandEndValue = Util.matchSignBank(leftFieldValues, nondomEndHS);
		
		if (leftHandEndValue != null) {
			HandShapeImage image = new HandShapeImage(leftHandEndValue, cs);
			selectedHandImagesLeft.add(image);
		}
		
		// set the arraylists of handshapes
		hshPanel.getHandShapeBean().setUpdated(true);
		hshPanel.getHandShapeBean().setSelectedRightHandShapeImages(selectedHandImagesRight);
		hshPanel.getHandShapeBean().setSelectedLeftHandShapeImages(selectedHandImagesLeft );
		hshPanel.updateSelectedHandshapesPanels(selectedHandImagesRight, selectedHandImagesLeft);
		hshPanel.repaint();
		
		revalidate();
		repaint();		
	}
	
	/**
	 * To be called by the sign bank panel exclusively.
	 * Set the main gloss text and variant text to the labels passed in.
	 * @param mainGlossLabel The main gloss label from the sign bank panel results list
	 * @param variantLabel The variant label from the sign bank panel results list
	 */
	public void populateMainGlossAndVariantLabels(String mainGlossLabel, String variantLabel) {
		if (mainGlossLabel == null || variantLabel == null)
			return;
		
		primaryEntryField.setText(mainGlossLabel);
		variantField.setText(variantLabel);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();
		
		switch (Integer.parseInt(str)) {
		case ENTER_BYPASS: 
			saveChanges();
			view.update();
			SS3Singleton.getSignStreamApplication().repaint();
			parent.setVisible(false);
			System.gc();
			break;
		case ENTER:
			
			SignBankCollection collection = null;
			String mainGlossStr = "", variantStr = "", untrimmedVariantStr = "";
			
			// first, before closing out, check that the user didn't edit the labels of main gloss & variant - to ensure that those labels are in the SB
			if (isLexical() || isNumber()) {	
				collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
				
				mainGlossStr = primaryEntryField.getText();
				untrimmedVariantStr = variantField.getText();
				variantStr = Util.trimAsterisksAndPlusSigns(untrimmedVariantStr);
				// check first if this sign DOES NOT exist in collection already - we will disallow if so
				SignBankVariant matchingVariant = collection.contains(mainGlossStr, variantStr);
				SignBankVariant matchingUntrimmedVariant = collection.contains(mainGlossStr, untrimmedVariantStr);
				if (matchingVariant == null && matchingUntrimmedVariant == null) {
					String warningDialogText =
							"Sign was not found in the sign bank.\nPlease either search for an existing sign, or add a new one.";
					String warningTitleText = "Sign Not Found";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String options[] = {"OK"};
					
					try {
						JOptionPane.showOptionDialog(this, 
								warningDialogText, 
								warningTitleText, 
								optionType, 
								messageType, 
								null, 
								options, 
								options[0]);
					} catch (HeadlessException he) {
						System.out.println("");
					}
					
					return;
				} else if (matchingUntrimmedVariant == null) {
					addToSignBank(mainGlossStr, untrimmedVariantStr);
					
					// reset this variant in the case we just added to sign bank 
					// (inside addToSignBank() the sign bank itself got re-parsed into `collection`)
					matchingUntrimmedVariant = collection.contains(mainGlossStr, untrimmedVariantStr);
				} 
				
				saveChanges(matchingUntrimmedVariant);				
			} else {
				saveChanges();
			}
			
			view.update();
			SS3Singleton.getSignStreamApplication().repaint();
			parent.setVisible(false);
			System.gc();
			break;
		case CANCEL:
			parent.setVisible(false);
			break;
		case SIGN_BANK:
			collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
			if ( !collection.canAccessSignBank()) {
				// an error occurred - please fix the error
				String warningDialogText =
					"An error occurred in accessing the sign bank.\n" +
					"Either a sign bank file is missing, or a local file was not found.\n" +
					"Make sure files are named exactly globalSignBank.xml and localSignBank.xml";
				String warningTitleText = "Cannot Access Sign Bank";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String options[] = {"OK"};
				
				try {
					JOptionPane.showOptionDialog(this, 
							warningDialogText, 
							warningTitleText, 
							optionType, 
							messageType, 
							null, 
							options, 
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}
						
				return;
			}
			
			String variantLabel = variantField.getText();
			
			signBankDialog = new SignBankDialog(this, variantLabel);			
			signBankDialog.setLocation(new Point(this.getWidth(), 0));
			signBankDialog.setVisible(true);
			
			break;			
		case ADD_NEW:
			
			// disallow non-sign-bank signs from being added to the sign bank
			if (fingerspelled.isSelected() || gesture.isSelected() || loan.isSelected() || classifier.isSelected()) {
				return;
			}
			
			collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
			if ( !collection.canAccessSignBank()) {
				// an error occurred - please fix the error
				String warningDialogText =
					"An error occurred in accessing the sign bank.\n" +
					"Either a sign bank file is missing, or a local file was not found.\n" +
					"Make sure files are named exactly globalSignBank.xml and localSignBank.xml";
				String warningTitleText = "Cannot Access Sign Bank";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String options[] = {"OK"};
				
				try {
					JOptionPane.showOptionDialog(this, 
							warningDialogText, 
							warningTitleText, 
							optionType, 
							messageType, 
							null, 
							options, 
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}
						
				return;
			}
			
			mainGlossStr = primaryEntryField.getText().toUpperCase();
			variantStr = variantField.getText().toUpperCase();
			
			// first check empty labels
			if (mainGlossStr == null || mainGlossStr.isEmpty() || variantStr == null || variantStr.isEmpty()) {

				String warningDialogText =
						"Enter values for both variant and main gloss";
				String warningTitleText = "Enter Values";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String options[] = {"OK"};
				
				try {
					JOptionPane.showOptionDialog(this, 
							warningDialogText, 
							warningTitleText, 
							optionType, 
							messageType, 
							null, 
							options, 
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}
				
				primaryEntryField.requestFocus();				
				return;
			}
			
			// then check to sanitize input text
			// disallow these characters:  "  '  ,  !  <  >
			String regex = ".*[\"',!<>].*";	
			
			if (mainGlossStr.matches(regex) || variantStr.matches(regex)) {
				String warningDialogText =
						"Cannot enter these characters for main gloss or variant label:\n" +
						" +  \"  '  ,  !  <  >";
				String warningTitleText = "Invalid Label";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String options[] = {"OK"};
				
				try {
					JOptionPane.showOptionDialog(this, 
							warningDialogText, 
							warningTitleText, 
							optionType, 
							messageType, 
							null, 
							options, 
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}

				primaryEntryField.requestFocus();
				return;
			}
			
			addToSignBank();			
			break;
		case FINGERSPELLED:
		case LOAN:
		case NUMBER:            
            hshPanel.selectOneHanded();
           	hshPanel.setPassiveBaseArmEnabled(false);
           	hshPanel.setSelectedSameDifStartEndHandshapesVisible(false);
           	hshPanel.setSelectedSameDifStartEndHandShapesIndex(0);
           	hshPanel.setSelectedSameDifStartEndHandshapesLeftVisible(false);
           	hshPanel.setSelectedSameDifStartEndHandshapesLeftIndex(0);
           	hshPanel.setSelectedSameDifStartEndHandshapesRightVisible(false);
           	hshPanel.setSelectedSameDifStartEndHandshapesRightIndex(0);
           	break;
		case GESTURE:
            hshPanel.selectOneHanded();
           	hshPanel.setPassiveBaseArmEnabled(false);
           	hshPanel.setMarkedNumberOfHands(false);
            hshPanel.setSelectedSameDifStartEndHandshapesVisible(true);
			hshPanel.setSelectedSameDifStartEndHandShapesIndex(0);
			hshPanel.setSelectedSameDifStartEndHandshapesLeftVisible(true);
			hshPanel.setSelectedSameDifStartEndHandshapesLeftIndex(0);
			hshPanel.setSelectedSameDifStartEndHandshapesRightVisible(true);
			hshPanel.setSelectedSameDifStartEndHandshapesRightIndex(0);
			break;
		case LEXICAL:
		case CLASSIFIER:
			hshPanel.selectTwoHanded();
			hshPanel.setSelectedSameDifStartEndHandshapesVisible(true);
			hshPanel.setSelectedSameDifLeftRightHandshapes(0);
			hshPanel.setMarkedNumberOfHands(false);
			hshPanel.setPassiveBaseArmSelected(false);
			break;
		}
		
		checkSignTypeAndUpdateSignBankFunctionality();
	}
	
	public void checkSignTypeAndUpdateSignBankFunctionality() {
		if (isLexical() || isNumber())
			enableSignBankFunctionality(true);
		else if (isFingerspelled() || isGesture() || isLoanSign() || isClassifier())
			enableSignBankFunctionality(false);
	}
	
	public void enableSignBankFunctionality(boolean enabled) {
		primaryEntryField.setEnabled(enabled);
		searchSignBank.setEnabled(enabled);
		addNewSign.setEnabled(enabled);
	}

	public boolean isFingerspelled() {
		return fingerspelled.isSelected();
	}

	protected boolean isLoanSign() {
		return loan.isSelected();
	}

	public boolean isNumber() {
		return number.isSelected();
	}

	protected boolean isLexical() {
		return lexical.isSelected();
	}

	public boolean isClassifier() {
		return classifier.isSelected();
	}

	public boolean isGesture() {
		return gesture.isSelected();
	}

	public boolean isNameSign() {
		return nameSign.isSelected();
	}
	
	public boolean isOneHandedSelected() {
		return hshPanel.isOneHandedSelected();
	}
	
	public boolean isTwoHandedSelected() {
		return hshPanel.isTwoHandedSelected();
	}

	public String getDominantHand() {
		if (domHndR.isSelected()) {
			return "R";
		} else {
			return "L";
		}
	}

	public JDialog getParentDialog() {
		return parent;
	}

	public JButton getEnterButton() {
		return enter;
	}
}

class LoadedProperties {
	
	private String gloss = "";
	
	private boolean fingerspelled = false;
	private boolean loan = false;
	private boolean number = false;
	private boolean lexical = false;
	private boolean classifier = false;
	private boolean gesture = false;
	private boolean nameSign = false;
	private boolean twoHanded = false;
	private boolean oneFrameGloss = false;
	private boolean markedNumberOfHands = false;
	private boolean passiveBaseArm = false;
	
	private int sameDifHandshapes = 0;
	private int sameDIfferentLeftRightHandshapes = 0;
	private int sameDifStartEndHandshapesL = 0;
	private int sameDifStartEndHandshapesR = 0; 
	
	private ArrayList<HandShapeImage> rightHandSelectedHandshapes = new ArrayList<HandShapeImage>();
	private ArrayList<HandShapeImage> leftHandSelectedHandshapes = new ArrayList<HandShapeImage>();
	
	public int getSameDifHandshapes() {
		return sameDifHandshapes;
	}
	public void setSameDifHandshapes(int sameDifHandshapes) {
		this.sameDifHandshapes = sameDifHandshapes;
	}
	public int getSameDIfferentLeftRightHandshapes() {
		return sameDIfferentLeftRightHandshapes;
	}
	public void setSameDIfferentLeftRightHandshapes(
			int sameDIfferentLeftRightHandshapes) {
		this.sameDIfferentLeftRightHandshapes = sameDIfferentLeftRightHandshapes;
	}
	public int getSameDifStartEndHandshapesL() {
		return sameDifStartEndHandshapesL;
	}
	public void setSameDifStartEndHandshapesL(int sameDifStartEndHandshapesL) {
		this.sameDifStartEndHandshapesL = sameDifStartEndHandshapesL;
	}
	public int getSameDifStartEndHandshapesR() {
		return sameDifStartEndHandshapesR;
	}
	public void setSameDifStartEndHandshapesR(int sameDifStartEndHandshapesR) {
		this.sameDifStartEndHandshapesR = sameDifStartEndHandshapesR;
	}
	public boolean isFingerspelled() {
		return fingerspelled;
	}
	public void setFingerspelled(boolean fingerspelled) {
		this.fingerspelled = fingerspelled;
	}
	public boolean isLoan() {
		return loan;
	}
	public void setLoan(boolean loan) {
		this.loan = loan;
	}
	public boolean isNumber() {
		return number;
	}
	public void setNumber(boolean number) {
		this.number = number;
	}
	public boolean isLexical() {
		return lexical;
	}
	public void setLexical(boolean lexical) {
		this.lexical = lexical;
	}
	public boolean isClassifier() {
		return classifier;
	}
	public void setClassifier(boolean classifier) {
		this.classifier = classifier;
	}
	public boolean isGesture() {
		return gesture;
	}
	public void setGesture(boolean gesture) {
		this.gesture = gesture;
	}
	public boolean isNameSign() {
		return nameSign;
	}
	public void setNameSign(boolean nameSign) {
		this.nameSign = nameSign;
	}
	public boolean isTwoHanded() {
		return twoHanded;
	}
	public void setTwoHanded(boolean twoHanded) {
		this.twoHanded = twoHanded;
	}
	public boolean isOneFrameGloss() {
		return oneFrameGloss;
	}
	public void setOneFrameGloss(boolean oneFrameGloss) {
		this.oneFrameGloss = oneFrameGloss;
	}
	public boolean isMarkedNumberOfHands() {
		return markedNumberOfHands;
	}
	public void setMarkedNumberOfHands(boolean markedNumberOfHands) {
		this.markedNumberOfHands = markedNumberOfHands;
	}
	public boolean isPassiveBaseArm() {
		return passiveBaseArm;
	}
	public void setPassiveBaseArm(boolean passiveBaseArm) {
		this.passiveBaseArm = passiveBaseArm;
	}
	public ArrayList<HandShapeImage> getRightHandSelectedHandshapes() {
		return rightHandSelectedHandshapes;
	}
	public void setRightHandSelectedHandshapes(
			ArrayList<HandShapeImage> rightHandSelectedHandshapes) {
		this.rightHandSelectedHandshapes.clear();
		this.rightHandSelectedHandshapes.addAll(rightHandSelectedHandshapes);
	}
	public ArrayList<HandShapeImage> getLeftHandSelectedHandshapes() {
		return leftHandSelectedHandshapes;
	}
	public void setLeftHandSelectedHandshapes(
			ArrayList<HandShapeImage> leftHandSelectedHandshapes) {
		this.leftHandSelectedHandshapes.clear();
		this.leftHandSelectedHandshapes.addAll(leftHandSelectedHandshapes);
	}	
	public String getGloss() {
		return gloss;
	}
	public void setGloss(String gloss) {
		this.gloss = gloss;
	}
	public void reset() {
		this.gloss = "";
		this.fingerspelled = false;
		this.loan = false;
		this.number = false;
		this.lexical = false;
		this.classifier = false;
		this.gesture = false;
		this.nameSign = false;
		this.twoHanded = false;
		this.oneFrameGloss = false;
		this.markedNumberOfHands = false;
		this.passiveBaseArm = false;
		this.sameDifHandshapes = 0;
		this.sameDIfferentLeftRightHandshapes = 0;
		this.sameDifStartEndHandshapesL = 0;
		this.sameDifStartEndHandshapesR = 0; 

		rightHandSelectedHandshapes.clear();
		leftHandSelectedHandshapes.clear();
	}
}
