package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeItem;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeTemplate;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeTemplates;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.DblHandshapeDialog;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandshapeDialog;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedHandshapesPanel;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.listener.WeakItemListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.ui.ButtonPanel;

public class HandShapePanel extends JPanel implements ItemListener, ActionListener {

	private static final long serialVersionUID = 1L;

	public static final String RIGHT_HAND_TITLE = "Right Hand";
	public static final String LEFT_HAND_TITLE = "Left Hand";
	public static final String RIGHT_N_LEFT_HANDS_TITLE = "Right and Left Hands";

	private JDialog parentDialog = null;

	// private JLabel donHndLbl = new JLabel(" Dominant Hand ");
	private JRadioButton domHndR = new JRadioButton("R");
	private JRadioButton domHndL = new JRadioButton("L");

	private JRadioButton oneHanded = new JRadioButton("One-handed");
	private JRadioButton twoHanded = new JRadioButton("Two-handed");

	private JRadioButton oneHndR = new JRadioButton("R");
	private JRadioButton oneHndL = new JRadioButton("L");

	private JButton selectHandshapes = new JButton("Select handshapes");
	private JButton clearHandshapes = new JButton("Clear");

	private AssociatedFieldsPanel associatedFieldsPanel = null;
	
	private JCheckBox oneFrameGloss = new JCheckBox("One-Frame Gloss");

	private JCheckBox markedForm = new JCheckBox("Marked # of Hands");
	private JCheckBox passiveBaseArm = new JCheckBox("Passive base arm");

	private String[] sameDifLeftRightHandshapesStr = { "Same handshapes L & R",
			"Alt same handshapes L & R", "Diff't handshapes L & R" };
	private JComboBox<String> sameDifLeftRightHandshapes = new JComboBox<String>(sameDifLeftRightHandshapesStr);

	private String[] sameDifStartEndHandshapesSrt = { "Same START/END hs",
			"Diff't START/END hs" };
	private JComboBox<String> sameDifStartEndHandshapes = new JComboBox<String>(sameDifStartEndHandshapesSrt);

	private String[] sameDifStartEndHandshapesLSrt = { "Same START/END hs (L)",
			"Diff't START/END hs (L)" };
	private JComboBox<String> sameDifStartEndHandshapesL = new JComboBox<String>(sameDifStartEndHandshapesLSrt);

	private String[] sameDifStartEndHandshapesRSrt = { "Same START/END hs (R)",
			"Diff't START/END hs (R)" };
	private JComboBox<String> sameDifStartEndHandshapesR = new JComboBox<String>(sameDifStartEndHandshapesRSrt);

	private final int DOM_HND_R = -1;
	private final int DOM_HND_L = -2;
	private final int ONE_HANDED = 1;
	private final int TWO_HANDED = 2;
	private final int ONE_HND_R = 3;
	private final int ONE_HND_L = 4;

	private final int SELECT_HANDSHAPES = 0;
	private final int CLEAR_HANDSHAPES = 1;

	private ButtonGroup grp1 = new ButtonGroup();
	private ButtonGroup oneHndGrp = new ButtonGroup();
	private ButtonGroup domHndGrp = new ButtonGroup();

	// contains entered information. Loaded from preprocessor
	private HandShapeFormBean hshFormBean;
	// contains updated information. Will be used by postprocessor
	private HandShapeBean bean;

	private GlossChainedEventsEntity selectedEntity = null;
	private GlossFieldWrapper wrapper = null;
	// private Segment segment = null;

	private JPanel southPanel = null;

    private SelectedHandshapesPanel lhPanel = null;
	private SelectedHandshapesPanel rhPanel = null;

	private int selectedIndex = -1;

	private SameHandshapesLeftRightComboBoxListener sameDifLeftRightHandshapesComboBoxListener = null;
	private SameHandshapesStartEndComboBoxListener sameDifStartEndHandshapesComboBoxListener = null;
	private SameHandshapesStartEndRightComboBoxListener sameDifStartEndHandshapesRightComboBoxListener = null;
	private SameHandshapesStartEndLeftComboBoxListener sameDifStartEndHandshapesLeftComboBoxListener = null;

	public HandShapePanel(AssociatedFieldsPanel associatedFieldsPanel) {
		this.associatedFieldsPanel = associatedFieldsPanel;
		this.parentDialog = associatedFieldsPanel.getParentDialog();
		
		associatedFieldsPanel.getHandShapeFormBean();

		this.hshFormBean = associatedFieldsPanel.getHandShapeFormBean();
		this.bean = new HandShapeBean();

		this.bean.setSelectedRightHandShapeImages(hshFormBean
				.getSelectedRightHandShapeImages());
		this.bean.setSelectedLeftHandShapeImages(hshFormBean
				.getSelectedLeftHandShapeImages());
		this.bean.setType(hshFormBean.getType());
		this.bean.setRightHandSelected(hshFormBean.isRightHandSelected());

		this.wrapper = hshFormBean.getGlsFieldWrapper();
		this.selectedEntity = (GlossChainedEventsEntity) SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();
		//associatedFieldsPanel.getEnterButton().setEnabled(false);
		parentDialog.getRootPane().setDefaultButton(selectHandshapes);

		oneHanded.addItemListener(new WeakItemListener(this, oneHanded));
		twoHanded.addItemListener(new WeakItemListener(this, twoHanded));
		domHndR.addItemListener(new WeakItemListener(this, domHndR));
		domHndL.addItemListener(new WeakItemListener(this, domHndL));
		oneHndR.addItemListener(new WeakItemListener(this, oneHndR));
		oneHndL.addItemListener(new WeakItemListener(this, oneHndL));

		domHndR.setActionCommand("" + DOM_HND_R);
		domHndL.setActionCommand("" + DOM_HND_L);
		oneHndR.setActionCommand("" + ONE_HND_R);
		oneHndL.setActionCommand("" + ONE_HND_L);
		oneHanded.setActionCommand("" + ONE_HANDED);
		twoHanded.setActionCommand("" + TWO_HANDED);

		selectHandshapes.setActionCommand("" + SELECT_HANDSHAPES);
		clearHandshapes.setActionCommand("" + CLEAR_HANDSHAPES);
		selectHandshapes.addActionListener(new WeakActionListener(this, selectHandshapes));
		clearHandshapes.addActionListener(new WeakActionListener(this, clearHandshapes));

		sameDifLeftRightHandshapes.setRenderer(new CenteredTextCellRenderer());
		sameDifStartEndHandshapes.setRenderer(new CenteredTextCellRenderer());
		sameDifStartEndHandshapesL.setRenderer(new CenteredTextCellRenderer());
		sameDifStartEndHandshapesR.setRenderer(new CenteredTextCellRenderer());

		java.awt.Font cFont = sameDifLeftRightHandshapes.getFont();
		java.awt.Font font = new java.awt.Font(cFont.getName(), cFont.getStyle(), 10);

		sameDifLeftRightHandshapes.setFont(font);
		sameDifStartEndHandshapes.setFont(font);
		sameDifStartEndHandshapesL.setFont(font);
		sameDifStartEndHandshapesR.setFont(font);

		Dimension d1 = sameDifLeftRightHandshapes.getPreferredSize();

		int w0 = sameDifStartEndHandshapesR.getPreferredSize().width;
		int w1 = sameDifStartEndHandshapesL.getPreferredSize().width;

		int w = w0 > w1 ? w0 : w1;

		Dimension d = new Dimension(2 * w + 10, d1.height);
		Dimension dd = new Dimension(w + 5, d1.height);

		sameDifLeftRightHandshapes.setPreferredSize(d);
		sameDifLeftRightHandshapes.setMaximumSize(d);
		sameDifLeftRightHandshapesComboBoxListener = new SameHandshapesLeftRightComboBoxListener(this);
		sameDifLeftRightHandshapes.addActionListener(new WeakActionListener(sameDifLeftRightHandshapesComboBoxListener, sameDifLeftRightHandshapes));
		
		sameDifStartEndHandshapes.setPreferredSize(d);
		sameDifStartEndHandshapes.setMinimumSize(d);		
		sameDifStartEndHandshapesComboBoxListener = new SameHandshapesStartEndComboBoxListener(this);
		sameDifStartEndHandshapes.addActionListener(new WeakActionListener(sameDifStartEndHandshapesComboBoxListener, sameDifStartEndHandshapes));

		sameDifStartEndHandshapesR.setMinimumSize(dd);
		sameDifStartEndHandshapesR.setSize(dd);
		sameDifStartEndHandshapesR.setPreferredSize(dd);
		sameDifStartEndHandshapesRightComboBoxListener = new SameHandshapesStartEndRightComboBoxListener(this);
		sameDifStartEndHandshapesR.addActionListener(new WeakActionListener(sameDifStartEndHandshapesRightComboBoxListener, sameDifStartEndHandshapesR));

		sameDifStartEndHandshapesL.setMinimumSize(dd);
		sameDifStartEndHandshapesL.setSize(dd);
		sameDifStartEndHandshapesL.setPreferredSize(dd);
		sameDifStartEndHandshapesLeftComboBoxListener = new SameHandshapesStartEndLeftComboBoxListener(this);
		sameDifStartEndHandshapesL.addActionListener(new WeakActionListener(sameDifStartEndHandshapesLeftComboBoxListener, sameDifStartEndHandshapesL));

		domHndR.setMinimumSize(dd);
		domHndR.setSize(dd);
		domHndR.setPreferredSize(dd);
		domHndR.setHorizontalAlignment(SwingConstants.CENTER);

		domHndL.setMinimumSize(dd);
		domHndL.setSize(dd);
		domHndL.setPreferredSize(dd);
		domHndL.setHorizontalAlignment(SwingConstants.CENTER);

		oneHndR.setMinimumSize(dd);
		oneHndR.setSize(dd);
		oneHndR.setPreferredSize(dd);
		oneHndR.setHorizontalAlignment(SwingConstants.CENTER);

		oneHndL.setMinimumSize(dd);
		oneHndL.setSize(dd);
		oneHndL.setPreferredSize(dd);
		oneHndL.setHorizontalAlignment(SwingConstants.CENTER);

		grp1.add(oneHanded);
		grp1.add(twoHanded);

		oneHndGrp.add(oneHndL);
		oneHndGrp.add(oneHndR);

		domHndGrp.add(domHndR);
		domHndGrp.add(domHndL);

		setPreferredSize(new Dimension(500, 340));

		populate();
		init();
	}

	public void update() {
		this.hshFormBean = associatedFieldsPanel.getHandShapeFormBean();
		this.bean.setSelectedRightHandShapeImages(hshFormBean
				.getSelectedRightHandShapeImages());
		this.bean.setSelectedLeftHandShapeImages(hshFormBean
				.getSelectedLeftHandShapeImages());
		this.bean.setType(hshFormBean.getType());
		this.bean.setRightHandSelected(hshFormBean.isRightHandSelected());

		this.selectedEntity = (GlossChainedEventsEntity) SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();

        ArrayList<HandShapeImage> rhImages =  new ArrayList<HandShapeImage>(hshFormBean.getSelectedRightHandShapeImages());
        ArrayList<HandShapeImage> lhImages =  new ArrayList<HandShapeImage>(hshFormBean.getSelectedLeftHandShapeImages());

        lhPanel.setHandshapes(lhImages);
		lhPanel.repaint();
		lhPanel.revalidate();

        rhPanel.setHandshapes(rhImages);
		rhPanel.repaint();
		rhPanel.revalidate();

		populate();
		removeAll();
		init();

		repaint();
		revalidate();
	}

    public void updateSelectedHandshapesPanels(ArrayList<HandShapeImage> rightHandshapes, ArrayList<HandShapeImage> leftHandshapes) {
		rhPanel.setHandshapes(rightHandshapes);
		rhPanel.repaint();
		rhPanel.revalidate();
		
		lhPanel.setHandshapes(leftHandshapes);
		lhPanel.repaint();
		lhPanel.revalidate();
		
		hshFormBean.setSelectedRightHandShapeImages(rightHandshapes);
		hshFormBean.setSelectedLeftHandShapeImages(leftHandshapes);
		selectedHandshapesPanelUpdated();
    }

	public void clean() {
		rhPanel.clean();
		lhPanel.clean();

		oneHanded.removeItemListener(this);
		twoHanded.removeItemListener(this);
		domHndR.removeItemListener(this);
		domHndL.removeItemListener(this);
		oneHndR.removeItemListener(this);
		oneHndL.removeItemListener(this);

		sameDifLeftRightHandshapes.removeActionListener(sameDifLeftRightHandshapesComboBoxListener);
	}
	
	public String getSelectedSameDifLeftRightHandshapesStr() {
		return (String)sameDifLeftRightHandshapes.getSelectedItem(); //.getSelectedIndex();
	} 

	public int getSelectedSameDifLeftRightHandshapes() {
		return sameDifLeftRightHandshapes.getSelectedIndex();
	}
	
	public void setSelectedSameDifLeftRightHandshapes(int index) {
		sameDifLeftRightHandshapes.setSelectedIndex(index);
	}

	public int getSelectedSameDifStartEndHandshapes() {
		return sameDifStartEndHandshapes.getSelectedIndex();
	}
	
	public void setSelectedSameDifStartEndHandShapesIndex(int index) {
		sameDifStartEndHandshapes.setSelectedIndex(index);
	}
	
	public void setSelectedSameDifStartEndHandshapesEnabled(boolean enable) {
		sameDifStartEndHandshapes.setEnabled(enable); 
	}
	
	public void setSelectedSameDifStartEndHandshapesVisible(boolean visible) {
		sameDifStartEndHandshapes.setVisible(visible);
	}

	public int getSelectedSameDifStartEndHandshapesL() {
		return sameDifStartEndHandshapesL.getSelectedIndex();
	}
	
	public void setSelectedSameDifStartEndHandshapesLeftIndex(int index) {
		sameDifStartEndHandshapesL.setSelectedIndex(index);
	}
	
	public void setSelectedSameDifStartEndHandshapesLeftEnabled(boolean enable) {
		sameDifStartEndHandshapesL.setEnabled(enable);
	}
	
	public void setSelectedSameDifStartEndHandshapesLeftVisible(boolean visible) {
		sameDifStartEndHandshapesL.setVisible(visible);
	}

	public int getSelectedSameDifStartEndHandshapesR() {
		return sameDifStartEndHandshapesR.getSelectedIndex();
	}
	
	public void setSelectedSameDifStartEndHandshapesRightIndex(int index) {
		sameDifStartEndHandshapesR.setSelectedIndex(index);
	}
	
	public void setSelectedSameDifStartEndHandshapesRightEnabled(boolean enable) {
		sameDifStartEndHandshapesR.setEnabled(enable);
	}
	
	public void setSelectedSameDifStartEndHandshapesRightVisible(boolean visible) {
		sameDifStartEndHandshapesR.setVisible(visible);
	}
	
	public void setOneFrameGloss(boolean selected) {
		oneFrameGloss.setSelected(selected);
	}
	
	public boolean isOneFrameGloss() {
		return oneFrameGloss.isSelected();
	}
	
	public void setMarkedNumberOfHands(boolean selected) {
		markedForm.setSelected(selected);
	}

	public boolean isMarkedNumberOfHands() {
		return markedForm.isSelected();
	}

	public void setPassiveBaseArmEnabled(boolean enable) {
		passiveBaseArm.setEnabled(enable);
	}
	
	public void setPassiveBaseArmSelected(boolean selected) {
		passiveBaseArm.setSelected(selected);
	}
	
	public boolean isPassiveBaseArm() {
		return passiveBaseArm.isSelected();
	}

	public boolean isTwoHanded() {
		return twoHanded.isSelected();
	}

	public void setSelectedIndex(int selectedIndex) {
		this.selectedIndex = selectedIndex;
	}

	//Selects one-handed programatically (Prevents warning dialogs)
	public void selectOneHanded() {
		//Temporarily remove the item listener to prevent warning dialogs from appearing.
		ItemListener temp = oneHanded.getItemListeners()[0];
		oneHanded.removeItemListener(temp);
		oneHanded.setSelected(true);
		//Add back item listener.
		oneHanded.addItemListener(temp);
	}

	public void selectTwoHanded() {
		twoHanded.setSelected(true);
		// sameDifHandshapes.setSelectedIndex(0);
		// sameDifStartEndHandshapes.setSelectedIndex(0);
	}

	public HandShapeFormBean getHandShapeFormBean() {
		return hshFormBean;
	}

	public SelectedHandshapesPanel getSelectedLeftHandshapesPane() {
		return lhPanel;
	}

	public SelectedHandshapesPanel getSelectedRightHandshapesPane() {
		return rhPanel;
	}

	private JPanel selectedImagesPanel() {
		SS3CodingScheme scheme = hshFormBean.getSS3CodingScheme();

		JLabel rhShapeLbl = new JLabel(" Right Hand");
		JLabel lhShapeLbl = new JLabel(" Left Hand");

		ArrayList<HandShapeImage> rhList = bean
				.getSelectedRightHandShapeImages();
		rhPanel = new SelectedHandshapesPanel(rhList, scheme, this);
		rhPanel.setBorder(BorderFactory.createLineBorder(Color.black));

		ArrayList<HandShapeImage> lhList = bean
				.getSelectedLeftHandShapeImages();
		lhPanel = new SelectedHandshapesPanel(lhList, scheme, this);
		lhPanel.setBorder(BorderFactory.createLineBorder(Color.black));

		JPanel panel = new JPanel();
		GridBagLayout gridBag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		panel.setLayout(gridBag);

		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.ipady = 0;
		c.weightx = 0.5;
		gridBag.setConstraints(rhShapeLbl, c);
		panel.add(rhShapeLbl, c);
		c.gridx = 1;

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(clearHandshapes);

		c.fill = GridBagConstraints.NORTHEAST;
		panel.add(buttonPanel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.ipady = 28; // make this component tall
		c.ipadx = 300;
		c.weightx = 0.0;
		c.gridwidth = 3;
		gridBag.setConstraints(rhPanel, c);
		panel.add(rhPanel);

		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.ipady = 0; // reset to default
		c.weighty = 1.0; // request any extra vertical space
		c.ipady = 10;
		c.weightx = 0.0;
		c.gridwidth = 3;
		gridBag.setConstraints(lhShapeLbl, c);
		panel.add(lhShapeLbl);

		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.ipady = 28; // make this component tall
		c.ipadx = 300;
		c.weightx = 0.0;
		c.gridwidth = 3;
		gridBag.setConstraints(lhPanel, c);
		panel.add(lhPanel);

		panel.setBorder(BorderFactory.createTitledBorder("Selected Handshapes"));
		return panel;
	}

	/**
     * Updates and returns HandShapeBean object.
     * @return HandShapeBean object that has selected HandShapeImage objects for both hands
     */
    public HandShapeBean getHandShapeBean() {
		ArrayList<HandShapeImage> rhList = rhPanel.getHandshapes();
		ArrayList<HandShapeImage> lhList = lhPanel.getHandshapes();
		
        bean.setSelectedRightHandShapeImages(rhList);
        bean.setSelectedLeftHandShapeImages(lhList);

		// bean.setMarkedForm(markedForm.isSelected());
		return bean;
	}

	private void populate() {
		// this is done so that setSelectedIndex does not trigger a removal of nondominant handshape images
		// that should only occur with a mouse click on the sameDifHandshapes combobox
		// this is undone at the end of this function
		sameDifLeftRightHandshapesComboBoxListener.setIgnoreFlag(true);

		String dominantHand = associatedFieldsPanel.getDominantHand();
		String fieldID = wrapper.getField().getFieldID();

		EventsEntity ent = SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();

		GlossChainedEventsEntity entity = (GlossChainedEventsEntity) ent;
		
		GlossEntityProperties properties = entity.getGlossEntityProperties();
		//String prClassification = properties.getTwoHandedHandshapesClassification();
		String startEndHandshape = properties.getStartEndHandshape();
		String startEndHandshapeLeft = properties.getStartEndHandshapeLeftHand();
		String startEndHandshapeRight = properties.getStartEndHandshapeRightHand();

		twoHanded.setEnabled(true);

		if (fieldID.equalsIgnoreCase("10000")) {
			// if dominant hand right (right handed signer), check right hand;
			// otherwise check left
			if (dominantHand.equalsIgnoreCase("R")) {
				oneHndR.setSelected(true);
			} else {
				oneHndL.setSelected(true);
			}
		} else {
			if (dominantHand.equalsIgnoreCase("R")) {
				oneHndL.setSelected(true);
			} else {
				oneHndR.setSelected(true);
			}
		}

        GlossEntityProperties prp = hshFormBean.getGlossEntityProperties();
        String oneHandedSignHand = prp.getOnehandedSignHand();

        if("L".equalsIgnoreCase(oneHandedSignHand)) {
            oneHndL.setSelected(true);
        } else {
            oneHndR.setSelected(true);
        }

		oneHndR.setEnabled(false);
		oneHndL.setEnabled(false);
		
		oneFrameGloss.setSelected(properties.isOneFrameGloss());
		oneFrameGloss.setEnabled(false);

		String twoHandedHandshapesClassification = properties
				.getTwoHandedHandshapesClassification();

		if (properties.isTwoHanded()) {
			twoHanded.setSelected(true);
			if (twoHandedHandshapesClassification == null
					|| twoHandedHandshapesClassification.trim().length() == 0) {
				sameDifLeftRightHandshapes.setSelectedIndex(0);
				sameDifStartEndHandshapes.setSelectedIndex(0);
			} else if (twoHandedHandshapesClassification
					.equalsIgnoreCase(GlossEntityProperties.SAME_HANDSHAPES_LR)) {
				sameDifLeftRightHandshapes.setSelectedIndex(0);
				if (startEndHandshape
						.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapes.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapes.setSelectedIndex(0);
				}
			} else if (twoHandedHandshapesClassification
					.equalsIgnoreCase(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR)) {
				sameDifLeftRightHandshapes.setSelectedIndex(1);
				if (startEndHandshape
						.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapes.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapes.setSelectedIndex(0);
				}
			} else if (twoHandedHandshapesClassification
					.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR)) {
				sameDifLeftRightHandshapes.setSelectedIndex(2);
				if (startEndHandshapeRight
						.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapesR.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapesR.setSelectedIndex(0);
				}

				if (startEndHandshapeLeft
						.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapesL.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapesL.setSelectedIndex(0);
				}
			}
		} else {
			oneHanded.setSelected(true);
			if (startEndHandshape
					.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
				sameDifStartEndHandshapes.setSelectedIndex(1);
			} else {
				// startEndHandshape
				sameDifStartEndHandshapes.setSelectedIndex(0);
			}
		}

		markedForm.setSelected(properties.isMarkedNumberOfHands());
		passiveBaseArm.setSelected(properties.isPassiveBaseArm());
		
		// undo the ignore on the sameDifHandshapes listener
		sameDifLeftRightHandshapesComboBoxListener.setIgnoreFlag(false);
	}

	public void init() {

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(gbl);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		gbc.gridy = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(1, 1, 1, 1);
		gbc.anchor = GridBagConstraints.NORTHWEST;
		centerPanel.add(new JLabel(), gbc);

		gbc.anchor = GridBagConstraints.CENTER;
		centerPanel.add(new JLabel(), gbc);
		gbc.anchor = GridBagConstraints.CENTER;
		centerPanel.add(new JLabel(), gbc);

		gbc.gridy = 2;
		gbc.gridwidth = 4;
		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		// centerPanel.add(new JSeparator(), gbc);
		centerPanel.add(new JLabel(), gbc);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.NONE;

		gbc.gridy = 3;
		gbc.gridwidth = 1;
		gbc.anchor = GridBagConstraints.NORTHWEST;

		centerPanel.add(oneHanded, gbc);
		gbc.anchor = GridBagConstraints.CENTER;
		centerPanel.add(oneHndL, gbc);
		gbc.anchor = GridBagConstraints.CENTER;
		centerPanel.add(oneHndR, gbc);

		gbc.gridy = 5;
		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		centerPanel.add(twoHanded, gbc);

		
		if(oneHanded.isSelected()) {
			
			gbc.gridwidth = 2;
			centerPanel.add(sameDifStartEndHandshapes, gbc);
			
			gbc.gridy = 6;
			gbc.gridwidth = 1;
			centerPanel.add(new JLabel(" "), gbc);

			JPanel p = new JPanel();
			p.setLayout(new GridLayout(1, 1));
			p.add(new JLabel());
			p.add(new JLabel());
			gbc.gridwidth = 2;
			centerPanel.add(p, gbc);
		} else if (twoHanded.isSelected()) {
			
			gbc.gridwidth = 2;
			centerPanel.add(sameDifLeftRightHandshapes, gbc);
			
			if (sameDifLeftRightHandshapes.getSelectedIndex() == 0
					|| sameDifLeftRightHandshapes.getSelectedIndex() == 1) {
				gbc.gridy = 6;
				gbc.gridwidth = 1;
				centerPanel.add(new JLabel(" "), gbc);
				gbc.gridwidth = 2;
				centerPanel.add(sameDifStartEndHandshapes, gbc);
			} else {
				gbc.gridy = 6;
				gbc.gridwidth = 1;
				centerPanel.add(new JLabel(" "), gbc);

				JPanel p = new JPanel();
				p.setLayout(new GridLayout(1, 1));
				p.add(sameDifStartEndHandshapesL);
				p.add(sameDifStartEndHandshapesR);
				gbc.gridwidth = 2;
				centerPanel.add(p, gbc);
			}
		}

		gbc.gridy = 8;
		gbc.insets = new Insets(0, 0, 0, 0);
		gbc.gridwidth = 1;
		centerPanel.add(markedForm, gbc);
		
		gbc.gridy = 8;
		gbc.insets = new Insets(0, 0, 0, 0);
		gbc.gridwidth = 1;
		centerPanel.add(oneFrameGloss, gbc);

		gbc.gridy = 9;
		gbc.insets = new Insets(0, 0, 0, 0);
		gbc.gridwidth = 1;
		centerPanel.add(passiveBaseArm, gbc);

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(selectHandshapes);

		gbc.anchor = GridBagConstraints.SOUTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		// gbc.gridx = gbc.gridx+1;
		gbc.gridwidth = 2;
		//
		gbc.fill = GridBagConstraints.NORTHEAST;
		centerPanel.add(buttonPanel, gbc);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		//
		// centerPanel.add(buttonPanel, gbc);

		setLayout(new BorderLayout());
		add(centerPanel, BorderLayout.CENTER);

		southPanel = selectedImagesPanel();
		add(southPanel, BorderLayout.SOUTH);

		invalidate();
		revalidate();
		repaint();
		parentDialog.pack();
	}

	public void updateSelectedIconsPanel() {
		remove(southPanel);
		southPanel = selectedImagesPanel();
		add(southPanel, BorderLayout.SOUTH);
		invalidate();
		revalidate();
		repaint();
	}

	private JDialog proceedRemoveDialog = null;
	private JButton proceedRemoveBtn = null;
	private JButton cancelBtn = null;

	private void removeHandshapes() {

		if (proceedRemoveDialog == null) {
			proceedRemoveDialog = new JDialog(parentDialog, "Handshapes ", true);

			cancelBtn = new JButton("Cancel");
			cancelBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					twoHanded.setSelected(true);
					
					// ensure reverse-switch parameters on cancel
					if(associatedFieldsPanel.isFingerspelled() || associatedFieldsPanel.isNumber() 
							|| associatedFieldsPanel.isLoanSign()) {
						sameDifStartEndHandshapes.setVisible(true);
						setSelectedSameDifStartEndHandShapesIndex(0);
						sameDifStartEndHandshapesL.setVisible(true);
						setSelectedSameDifStartEndHandshapesLeftIndex(0);
						sameDifStartEndHandshapesR.setVisible(true);
						setSelectedSameDifStartEndHandshapesRightIndex(0);
						markedForm.setSelected(true);
					}
					
					passiveBaseArm.setEnabled(true);
					
					proceedRemoveDialog.dispose();
				}
			});

			proceedRemoveBtn = new JButton("Proceed");
			proceedRemoveBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
                    boolean rhDominant = hshFormBean.isRightHandDominant();

					if (oneHanded.isSelected()) {
						selectOnehandedSign();
					}
					
					// ensure switch parameters on proceed
					if(associatedFieldsPanel.isFingerspelled() || associatedFieldsPanel.isNumber() || associatedFieldsPanel.isLoanSign()) {
						sameDifStartEndHandshapes.setVisible(false);
						setSelectedSameDifStartEndHandShapesIndex(0);
						sameDifStartEndHandshapesL.setVisible(false);
						setSelectedSameDifStartEndHandshapesLeftIndex(0);
						sameDifStartEndHandshapesR.setVisible(false);
						setSelectedSameDifStartEndHandshapesRightIndex(0);
						markedForm.setSelected(false);
					}

					// default for all types for proceeding to switch to one-handed is same handshapes on left/right
					sameDifLeftRightHandshapes.setSelectedIndex(0);
					
					passiveBaseArm.setSelected(false);
					passiveBaseArm.setEnabled(false);

					if (rhPanel != null && lhPanel != null) {
                        String fieldID = null;

                        Field field = SS3Singleton.getMediaToolBar(null).getSelectedField();
                        if(field != null) {
                            fieldID = field.getFieldID();
                        }

                        if(fieldID == null) {
                            EventsEntity ent = SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();

                            if (ent == null)
                                return;

                            GlossChainedEventsEntity entity = (GlossChainedEventsEntity) ent;
                            fieldID = entity.getField().getFieldID();
                        }

                        String glsID1 = "10000";

						if (rhDominant) {
                            if(fieldID.equalsIgnoreCase(glsID1)) {
                                lhPanel.setHandshapes(new ArrayList<HandShapeImage>());
                                lhPanel.repaint();
                                lhPanel.revalidate();
                                oneHndR.setSelected(true);
                                hshFormBean.setSelectedLeftHandShapeImages(new ArrayList<HandShapeImage>());
                            } else {
                                rhPanel.setHandshapes(new ArrayList<HandShapeImage>());
                                rhPanel.repaint();
                                rhPanel.revalidate();
                                oneHndL.setSelected(true);
                                hshFormBean.setSelectedRightHandShapeImages(new ArrayList<HandShapeImage>());
                            }
						} else {
                            if(fieldID.equalsIgnoreCase(glsID1)) {
                                rhPanel.setHandshapes(new ArrayList<HandShapeImage>());
                                rhPanel.repaint();
                                rhPanel.revalidate();
                                oneHndL.setSelected(true);
                                hshFormBean.setSelectedRightHandShapeImages(new ArrayList<HandShapeImage>());
                            } else {
                                lhPanel.setHandshapes(new ArrayList<HandShapeImage>());
                                lhPanel.repaint();
                                lhPanel.revalidate();
                                oneHndR.setSelected(true);
                                hshFormBean.setSelectedLeftHandShapeImages(new ArrayList<HandShapeImage>());
                            }
						}
					}

					proceedRemoveDialog.dispose();
				}
			});

			proceedRemoveDialog.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosing(WindowEvent we) {
					twoHanded.setSelected(true);
				}
			});

			JLabel l1 = new JLabel(" If you complete this operation,");
			JLabel l2 = new JLabel(
					"previously selected handshapes will be discarded.");
			JLabel l3 = new JLabel("");
			JLabel l4 = new JLabel("(This operation cannot be undone.)");
			JLabel l5 = new JLabel("");

			JPanel warningPanel = new JPanel();

			GridBagConstraints c = new GridBagConstraints();
			warningPanel.setLayout(new GridBagLayout());

			c.anchor = GridBagConstraints.WEST;
			c.insets = new Insets(1, 1, 1, 1);

			c.gridy = 0;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l1, c);

			c.gridy = 1;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l2, c);

			c.gridy = 2;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l3, c);

			c.gridy = 3;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l4, c);

			c.gridy = 4;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l5, c);

			ButtonPanel buttonPanel = new ButtonPanel();
			buttonPanel.add(cancelBtn);
			buttonPanel.add(proceedRemoveBtn);

			c.gridy = 5;
			c.gridx = 2;
			c.gridwidth = 1;
			c.anchor = GridBagConstraints.EAST;
			warningPanel.add(buttonPanel, c);

			c.anchor = GridBagConstraints.WEST;
			c.gridy = 6;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(new JLabel(""), c);

			warningPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10,
					10));

			JPanel panel = new JPanel(new BorderLayout(20, 20));
			panel.add(warningPanel, BorderLayout.CENTER);

			Container contentPane = proceedRemoveDialog.getContentPane();
			contentPane.setLayout(new BorderLayout());
			contentPane.add(panel, BorderLayout.CENTER);
			proceedRemoveDialog.pack();
		}

		Point p = parentDialog.getLocation();
		int dx = (parentDialog.getWidth() - proceedRemoveDialog.getWidth()) / 2;
		int dy = (parentDialog.getHeight() - proceedRemoveDialog.getHeight()) / 2;

		int x = (p.x + dx);
		int y = (p.y + dy);

		proceedRemoveDialog.setLocation(x, y);
		proceedRemoveDialog.setVisible(true);
	}

	@Override
	public void itemStateChanged(ItemEvent event) {
		boolean rightHandEnabled = hshFormBean.isRightHandEnabled();
		boolean leftHandEnabled = hshFormBean.isLeftHansEnabled();

		JRadioButton obj = (JRadioButton) event.getSource();
		String str = obj.getActionCommand();

		switch (Integer.parseInt(str)) {
		case DOM_HND_R:
			if (rightHandEnabled) {
				oneHndL.setSelected(false);
				oneHndR.setSelected(true);
			}
			break;
		case DOM_HND_L:
			if (leftHandEnabled) {
				oneHndL.setSelected(true);
				oneHndR.setSelected(false);
			}
			break;

		case ONE_HANDED:
			parentDialog.getRootPane().setDefaultButton(selectHandshapes);
			selectHandshapes.requestFocus();
			selectHandshapes.requestFocus(true);
			selectHandshapes.requestFocusInWindow();

			if (rhPanel != null && lhPanel != null) {

				ArrayList<HandShapeImage> rhList1 = rhPanel.getHandshapes();
				ArrayList<HandShapeImage> lhList1 = lhPanel.getHandshapes();

				if (oneHanded.isSelected() && !rhList1.isEmpty()
						&& !lhList1.isEmpty()) {
					removeHandshapes();
				} else if (oneHanded.isSelected()) {
					selectOnehandedSign();
				}
			} else if (oneHanded.isSelected()) {
				selectOnehandedSign();
				
				if(associatedFieldsPanel.isFingerspelled() || associatedFieldsPanel.isNumber() 
						|| associatedFieldsPanel.isLoanSign()) {
					sameDifStartEndHandshapes.setVisible(false);
					setSelectedSameDifStartEndHandShapesIndex(0);
					sameDifStartEndHandshapesL.setVisible(false);
					setSelectedSameDifStartEndHandshapesLeftIndex(0);
					sameDifStartEndHandshapesR.setVisible(false);
					setSelectedSameDifStartEndHandshapesRightIndex(0);
				}
				
				//Setting marked # hands to false as this is the default option in all cases.
				markedForm.setSelected(false);
			}
			
			break;

		case TWO_HANDED:
			//
			// parentDialog.getRootPane().setDefaultButton(cancel);
			//

			if (twoHanded.isSelected()) {
				// oneHndR.setEnabled(false);
				// oneHndL.setEnabled(false);
				selectHandshapes.setEnabled(true);

				if(associatedFieldsPanel.isFingerspelled() || associatedFieldsPanel.isNumber() 
						|| associatedFieldsPanel.isLoanSign()) {
					sameDifStartEndHandshapes.setVisible(false);
					setSelectedSameDifStartEndHandShapesIndex(0);
					sameDifStartEndHandshapesL.setVisible(false);
					setSelectedSameDifStartEndHandshapesLeftIndex(0);
					sameDifStartEndHandshapesR.setVisible(false);
					setSelectedSameDifStartEndHandshapesRightIndex(0);
				} else {
					setSelectedSameDifStartEndHandShapesIndex(0);
					sameDifStartEndHandshapes.setVisible(true);
					setSelectedSameDifStartEndHandshapesLeftIndex(0);
					sameDifStartEndHandshapesL.setVisible(true);
					setSelectedSameDifStartEndHandshapesRightIndex(0);
					sameDifStartEndHandshapesR.setVisible(true);
					
				}
				

                //
                // boolean changed = associatedFieldsPanel.isChanged();
                //
                associatedFieldsPanel.getEnterButton().setEnabled(true);

                //

				// if same handshapes L&R are selected and L or R handshape is
				// populated, double the selection
				removeAll();
				init();

				// if(sameDifHandshapes.getSelectedIndex() == 0) {
				ArrayList<HandShapeImage> rhList = rhPanel.getHandshapes();
				ArrayList<HandShapeImage> lhList = lhPanel.getHandshapes();

				if (rhList.isEmpty() && !lhList.isEmpty()) {
					rhPanel.setHandshapes(lhList);
					rhPanel.repaint();
					rhPanel.revalidate();
                    hshFormBean.setSelectedRightHandShapeImages(lhList);
				}

				if (lhList.isEmpty() && !rhList.isEmpty()) {
					lhPanel.setHandshapes(rhList);
					lhPanel.repaint();
					lhPanel.revalidate();
                    hshFormBean.setSelectedLeftHandShapeImages(rhList);
				}
				
				if(associatedFieldsPanel.isFingerspelled() || associatedFieldsPanel.isNumber() 
						|| associatedFieldsPanel.isLoanSign()) {
					markedForm.setSelected(true);
					sameDifStartEndHandshapes.setVisible(false);
					setSelectedSameDifStartEndHandShapesIndex(0);
					sameDifStartEndHandshapesL.setVisible(false);
					setSelectedSameDifStartEndHandshapesLeftIndex(0);
					sameDifStartEndHandshapesR.setVisible(false);
					setSelectedSameDifStartEndHandshapesRightIndex(0);
				}
				else {
					//For all non finger, number and loan signs, set marked # of hands to false.  This is to preserve the default behavior
					markedForm.setSelected(false);

					sameDifStartEndHandshapes.setVisible(true);
					sameDifStartEndHandshapesL.setVisible(true);
					sameDifStartEndHandshapesR.setVisible(true);
				}
				
				passiveBaseArm.setEnabled(true);
				passiveBaseArm.setSelected(false);
			}
			break;

		case ONE_HND_R:
		case ONE_HND_L:
			if (oneHndR.isEnabled() && rhPanel != null && lhPanel != null) {
				ArrayList<HandShapeImage> rhList = rhPanel.getHandshapes();
				ArrayList<HandShapeImage> lhList = lhPanel.getHandshapes();

				if (oneHndR.isSelected()) {
					if (rhList.isEmpty() && !lhList.isEmpty()) {
						rhPanel.setHandshapes(lhList);
						lhPanel.setHandshapes(new ArrayList<HandShapeImage>());
						rhPanel.repaint();
						rhPanel.revalidate();
						lhPanel.repaint();
						lhPanel.revalidate();
					}
				} else {
					if (lhList.isEmpty() && !rhList.isEmpty()) {
						lhPanel.setHandshapes(rhList);
						rhPanel.setHandshapes(new ArrayList<HandShapeImage>());
						rhPanel.repaint();
						rhPanel.revalidate();
						lhPanel.repaint();
						lhPanel.revalidate();
					}
				}
			}
			break;
		}
	}

	private void selectOnehandedSign() {
		// oneHndR.setEnabled(true);
		// oneHndL.setEnabled(true);
		selectHandshapes.setEnabled(true);

		removeAll();
		init();
		
		hshFormBean.setSelectedLeftHandShapeImages(new ArrayList());
		lhPanel.setHandshapes(new ArrayList());
	}
	
	// private JDialog d = null;

    private HandshapeDialog sameHandDialog = null;
    private DblHandshapeDialog twoHandedDialog = null;

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {

		case SELECT_HANDSHAPES:
			HandShapesPaletteFormBean palletFormBean = new HandShapesPaletteFormBean();

			SS3CodingScheme scheme = hshFormBean.getSS3CodingScheme();
			palletFormBean.setCodingScheme(scheme);

			CSHandShapeTemplates templates = scheme.getHandShapeTemplates();
			CSHandShapeTemplate handShapeTemplate = templates.getSelectedHandShapeTemplate();

			if (twoHanded.isSelected()) {
				// 2-handed
				int sameDifHandshapesSelectedIndex = sameDifLeftRightHandshapes.getSelectedIndex();

				if (sameDifHandshapesSelectedIndex == 0 || sameDifHandshapesSelectedIndex == 1) {
					// one palette for both hands
					String hndType = Constants.TYPE_RIGHT_HAND_SHAPE;

					if (oneHndR.isSelected()) {
						palletFormBean.setSelectedHshImages(bean
								.getSelectedRightHandShapeImages());
					} else {
						palletFormBean.setSelectedHshImages(bean
								.getSelectedLeftHandShapeImages());
						hndType = Constants.TYPE_LEFT_HAND_SHAPE;
					}

					if (associatedFieldsPanel.isFingerspelled()) {
						selectedIndex = 1;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.FINGERSPELLED);
					} else if (associatedFieldsPanel.isNumber()) {
						selectedIndex = 2;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.NUMBERS);
					} else {
						selectedIndex = 0;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.ALL);
					}

					palletFormBean.setRightHandSelected(oneHndR.isSelected());

					ArrayList<CSHandShapeItem> items = handShapeTemplate
							.getHandShapePalette(handShapeTemplate.getSelectedPaletteID()).getHandShapeItems();
					ArrayList<SS3FieldValue> list = scheme.getCorrespondingFieldValues(items, hndType);

					palletFormBean.setValues(list);

					palletFormBean.setTitle(RIGHT_N_LEFT_HANDS_TITLE);

					if (oneHndR.isSelected()) {
						palletFormBean.setSelectedHshImages(bean
								.getSelectedRightHandShapeImages());
					} else {
						palletFormBean.setSelectedHshImages(bean
								.getSelectedLeftHandShapeImages());
					}

 					if (selectedIndex < 0)
                        selectedIndex = 0;

                    loadOrUpdateSameHandshapeDialog(palletFormBean, bean, this,
                            hshFormBean.getSS3CodingScheme(), parentDialog,
                            selectedIndex);
				} else {
					// two palettes
					HandShapesPaletteFormBean palletFormBean2 = new HandShapesPaletteFormBean();

					String dominantHand = associatedFieldsPanel
							.getDominantHand();
					boolean isDominantHandRight = dominantHand
							.equalsIgnoreCase("R");

					boolean rHndUnmarked = !(isDominantHandRight && oneHndR.isSelected());
					String rid, lid = "";

					if (associatedFieldsPanel.isFingerspelled()) {
						if (rHndUnmarked) {
							rid = CSHandShapeTemplate.FINGERSPELLED_UNMARKED;
							lid = CSHandShapeTemplate.FINGERSPELLED;
							palletFormBean.setPaletteID(3);
							palletFormBean2.setPaletteID(1);
						} else {
							rid = CSHandShapeTemplate.FINGERSPELLED;
							lid = CSHandShapeTemplate.FINGERSPELLED_UNMARKED;
							palletFormBean.setPaletteID(1);
							palletFormBean2.setPaletteID(3);
						}
					} else if (associatedFieldsPanel.isNumber()) {
						// handShapeTemplate.selectPalette(CSHandShapeTemplate.NUMBERS);

						if (rHndUnmarked) {
							rid = CSHandShapeTemplate.NUMBERS_UNMARKED;
							lid = CSHandShapeTemplate.NUMBERS;
							palletFormBean.setPaletteID(3);
							palletFormBean2.setPaletteID(2);
						} else {
							rid = CSHandShapeTemplate.NUMBERS;
							lid = CSHandShapeTemplate.NUMBERS_UNMARKED;
							palletFormBean.setPaletteID(2);
							palletFormBean2.setPaletteID(3);
						}
					} else {
						// handShapeTemplate.selectPalette(CSHandShapeTemplate.ALL);
						if (rHndUnmarked) {
							rid = CSHandShapeTemplate.ALL_UNMARKED;
							lid = CSHandShapeTemplate.ALL;
							palletFormBean.setPaletteID(3);
							palletFormBean2.setPaletteID(0);
						} else {
							rid = CSHandShapeTemplate.ALL;
							lid = CSHandShapeTemplate.ALL_UNMARKED;
							palletFormBean.setPaletteID(0);
							palletFormBean2.setPaletteID(3);
						}
					}

					ArrayList<CSHandShapeItem> rhUnmarkedItems = handShapeTemplate
							.getHandShapePalette(rid).getHandShapeItems();
					ArrayList<SS3FieldValue> rhList = scheme
							.getCorrespondingFieldValues(rhUnmarkedItems,
									Constants.TYPE_RIGHT_HAND_SHAPE);
					handShapeTemplate.selectPalette(rid);
					handShapeTemplate.selectPalette2(lid);

					palletFormBean.setSelectedHshImages(bean
							.getSelectedRightHandShapeImages());
					palletFormBean.setValues(rhList);
					palletFormBean.setRightHandSelected(domHndR.isSelected());
					palletFormBean.setTitle(RIGHT_HAND_TITLE);

					ArrayList<CSHandShapeItem> lhUnmarkedItems = handShapeTemplate
							.getHandShapePalette(lid).getHandShapeItems();

					ArrayList<SS3FieldValue> lhList = scheme
							.getCorrespondingFieldValues(lhUnmarkedItems,
									Constants.TYPE_LEFT_HAND_SHAPE);

					palletFormBean2.setSelectedHshImages(bean
							.getSelectedLeftHandShapeImages());
					palletFormBean2.setValues(lhList);
					palletFormBean2.setRightHandSelected(!domHndR.isSelected());
					palletFormBean2.setTitle(LEFT_HAND_TITLE);

					//DblHandshapeDialog 
                    twoHandedDialog = new DblHandshapeDialog(
							palletFormBean, palletFormBean2, bean, this,
							hshFormBean.getSS3CodingScheme());

                    twoHandedDialog.setLocationRelativeTo(null); //parentDialog);
                    twoHandedDialog.setLocation(parentDialog.getWidth() + 15, 10);
                    twoHandedDialog.setVisible(true);
                    twoHandedDialog.toFront();
				}
			} else {
					if (associatedFieldsPanel.isFingerspelled()) {
						selectedIndex = 1;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.FINGERSPELLED);
					} else if (associatedFieldsPanel.isNumber()) {
						selectedIndex = 2;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.NUMBERS);
					} else {
						selectedIndex = 0;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.ALL);
					}

				ArrayList<CSHandShapeItem> items = handShapeTemplate.getHandShapePalette(handShapeTemplate.getSelectedPaletteID()).getHandShapeItems();

				if (oneHndR.isSelected()) {

					ArrayList<SS3FieldValue> list = scheme
							.getCorrespondingFieldValues(items,
									Constants.TYPE_RIGHT_HAND_SHAPE);

					palletFormBean.setValues(list);
					palletFormBean.setRightHandSelected(true);
					palletFormBean.setTitle(RIGHT_HAND_TITLE);
					palletFormBean.setSelectedHshImages(bean
							.getSelectedRightHandShapeImages());

				} else if (oneHndL.isSelected()) {

					ArrayList<SS3FieldValue> list = scheme
							.getCorrespondingFieldValues(items,
									Constants.TYPE_RIGHT_HAND_SHAPE);

					palletFormBean.setValues(list);
					palletFormBean.setRightHandSelected(false);
					palletFormBean.setTitle(LEFT_HAND_TITLE);
					palletFormBean.setSelectedHshImages(bean
							.getSelectedLeftHandShapeImages());
				}

                loadOrUpdateSameHandshapeDialog(palletFormBean, bean,
                        this, hshFormBean.getSS3CodingScheme(), parentDialog,
                        selectedIndex);
			}
			break;

		case CLEAR_HANDSHAPES:
			bean.clearRightHandShapeImages();
			bean.clearLeftHandShapeImages();
			hshFormBean.setSelectedLeftHandShapeImages(new ArrayList());
			hshFormBean.setSelectedRightHandShapeImages(new ArrayList());
			if (sameHandDialog != null && associatedFieldsPanel.isFingerspelled()) {
				sameHandDialog.getPalette().getPalette().actionPerformed(null);	// this is to clear the text inside the palette window
			}
			updateSelectedIconsPanel();
			break;
		}
	}

    private void loadOrUpdateSameHandshapeDialog(HandShapesPaletteFormBean formBean,
                                                 HandShapeBean bean,
                                                 HandShapePanel panel,
                                                 SS3CodingScheme scheme,
                                                 JDialog dialog, int selectedIndex) {
        if(twoHandedDialog != null)
            twoHandedDialog.setVisible(false);

        if(sameHandDialog == null) {
             sameHandDialog = new HandshapeDialog(formBean, bean, panel, scheme,dialog, selectedIndex);
             sameHandDialog.setLocationRelativeTo(null); //parentDialog);
             sameHandDialog.setLocation(parentDialog.getWidth() + 15, 10);
        } else {
             sameHandDialog.update(formBean, bean, selectedIndex);
        }
        sameHandDialog.setVisible(true);
        sameHandDialog.toFront();
    }

	private boolean justLoaded = true;

	public void selectedHandshapesPanelUpdated() {
		ArrayList<HandShapeImage> lhImages =  new ArrayList<HandShapeImage>(bean
				.getSelectedLeftHandShapeImages());
		ArrayList<HandShapeImage> rhImages =  new ArrayList<HandShapeImage>(bean
				.getSelectedRightHandShapeImages());

		if (justLoaded) {
			parentDialog.getRootPane().setDefaultButton(selectHandshapes);
			justLoaded = false;
		} else {

			JButton enterHandshapes = associatedFieldsPanel.getEnterButton();

			if (twoHanded.isSelected()) {
				if (!lhImages.isEmpty() && !rhImages.isEmpty()) {
					enterHandshapes.setEnabled(true);
					parentDialog.getRootPane()
							.setDefaultButton(enterHandshapes);
				} else {
					enterHandshapes.setEnabled(false);
					parentDialog.getRootPane().setDefaultButton(selectHandshapes);
				}
			} else if (oneHanded.isSelected()) {
				if (lhImages.isEmpty() && rhImages.isEmpty()) {
					enterHandshapes.setEnabled(false);
					parentDialog.getRootPane().setDefaultButton(selectHandshapes);
				} else {
					enterHandshapes.setEnabled(true);
					parentDialog.getRootPane()
							.setDefaultButton(enterHandshapes);
				}
			}
		}
	}

	public boolean isOneHandedSelected() {
		return oneHanded.isSelected();
	}

	public boolean isTwoHandedSelected() {
		return twoHanded.isSelected();
	}

	public String getSelectedSameDifferentLeftRightHandshapes() {
		int selectedIndex = sameDifLeftRightHandshapes.getSelectedIndex();
		if (selectedIndex == 0) {
			return GlossEntityProperties.SAME_HANDSHAPES_LR;
		} else if (selectedIndex == 1) {
			return GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR;
		} else {
			return GlossEntityProperties.DIFFERENT_HANDSHAPES_LR;
		}
	}

	public String getSelectedSameDifferentStartEndHandshapes() {
		int selectedIndex = sameDifStartEndHandshapes.getSelectedIndex();
		if (selectedIndex == 0) {
			return GlossEntityProperties.SAME_START_END_HANDSHAPE;
		} else {
			return GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE;
		}
	}

	public String getSelectedSameDifferentLeftHandshape() {
		int selectedIndex = sameDifStartEndHandshapesL.getSelectedIndex();
		if (selectedIndex == 0) {
			return GlossEntityProperties.SAME_START_END_HANDSHAPE;
		} else {
			return GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE;
		}
	}

	public String getSelectedSameDifferentRightHandshape() {
		int selectedIndex = sameDifStartEndHandshapesR.getSelectedIndex();
		if (selectedIndex == 0) {
			return GlossEntityProperties.SAME_START_END_HANDSHAPE;
		} else {
			return GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE;
		}
	}
}

class CenteredTextCellRenderer extends JLabel implements ListCellRenderer<Object> {
	private static final long serialVersionUID = 1L;

	public CenteredTextCellRenderer() {
	}

	@Override
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus) {

		setHorizontalAlignment(SwingConstants.CENTER);
		setText(value.toString());
		return this;
	}
}

class SameHandshapesLeftRightComboBoxListener implements ActionListener {
	private HandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesLeftRightComboBoxListener(HandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {
			panel.removeAll();
			panel.init();
			
			int selectedSameDifLeftRightHandshapesIndex = panel.getSelectedSameDifLeftRightHandshapes();
			HandShapeBean bean = panel.getHandShapeBean();
			
			if (selectedSameDifLeftRightHandshapesIndex == 2) {
				bean.clearLeftHandShapeImages();
				panel.updateSelectedIconsPanel();
				panel.getHandShapeFormBean().setSelectedLeftHandShapeImages(new ArrayList());
			}
			
			if (selectedSameDifLeftRightHandshapesIndex == 0 || selectedSameDifLeftRightHandshapesIndex == 1) {
				bean.setSelectedLeftHandShapeImages(bean.getSelectedRightHandShapeImages());
				panel.updateSelectedIconsPanel();
				panel.getHandShapeFormBean().setSelectedLeftHandShapeImages(bean.getSelectedRightHandShapeImages());
			}
		}
	}
}

class SameHandshapesStartEndComboBoxListener implements ActionListener {
	private HandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesStartEndComboBoxListener(HandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {			
			int selectedSameDifStartEndHandshapesIndex = panel.getSelectedSameDifStartEndHandshapes();
			
			// disallow 1-frame and different start/end handshapes
			if (selectedSameDifStartEndHandshapesIndex == 1 && panel.isOneFrameGloss()) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SET_DIF_START_END_ONE_FRAME);
					panel.setSelectedSameDifStartEndHandShapesIndex(0);
			}
		}
	}
}

class SameHandshapesStartEndRightComboBoxListener implements ActionListener {
	private HandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesStartEndRightComboBoxListener(HandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {			
			int selectedSameDifStartEndHandshapesRightIndex = panel.getSelectedSameDifStartEndHandshapesR();
			
			// disallow 1-frame and different start/end handshapes
			if (selectedSameDifStartEndHandshapesRightIndex == 1 && panel.isOneFrameGloss()) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SET_DIF_START_END_ONE_FRAME);
					panel.setSelectedSameDifStartEndHandshapesRightIndex(0);
			}
		}
	}
}

class SameHandshapesStartEndLeftComboBoxListener implements ActionListener {
	private HandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesStartEndLeftComboBoxListener(HandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {			
			int selectedSameDifStartEndHandshapesLeftIndex = panel.getSelectedSameDifStartEndHandshapesL();
			
			// disallow 1-frame and different start/end handshapes
			if (selectedSameDifStartEndHandshapesLeftIndex == 1 && panel.isOneFrameGloss()) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SET_DIF_START_END_ONE_FRAME);
					panel.setSelectedSameDifStartEndHandshapesLeftIndex(0);
			}
		}
	}
}
