package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.DependentEntitiesManager;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntityTypes;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEvent;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.rightLeftField.RightLeftHandFieldWrapper;

public class HandShapePreprocessor {
	private GlossFieldWrapper wrapper = null;
	private Segment segment = null;

	private boolean rHandSelected = true;
	private boolean lHandSelected = false;

	public HandShapePreprocessor(GlossFieldWrapper wrapper, Segment segment) {
		this.wrapper = wrapper;
		this.segment = segment;

		boolean rHndDom = isRightHandDominant();

		this.rHandSelected = rHndDom;
		this.lHandSelected = !rHndDom;
	}

	public void populateFormBean(HandShapeFormBean formBean) {
		char type = getSelectedHandShapeType();

		// boolean rHandDom = isRightHandDominant();
		boolean rHandSelected = isRightHandSelected();
		boolean twoHndEnbl = isTwoHandedHandShapeEnabled();

		boolean rightHandFieldsUpdatable = isRightHandFieldUpdatable();
		boolean leftHandFieldsUpdatable = isLeftHandFieldUpdatable();
		boolean fingerspelledSelected = isFingerspelledSelected(type);
		boolean fingerspelledEnabled = isFingerspelledEnabled();
		boolean oneHandedSelected = isOneHandedSelected(type);
		boolean oneHandedEnabled = isOneHandedEnabled();

		boolean oneHandedRightEnabled = isOneHandedRightEnabled(
				oneHandedEnabled, oneHandedSelected, rightHandFieldsUpdatable);

		boolean oneHandedLeftEnabled = isOneHandedLeftEnabled(oneHandedEnabled,
				oneHandedSelected, leftHandFieldsUpdatable);

		boolean fingerspelledRightEnabled = isFingerspelledRightEnabled(
				fingerspelledEnabled, fingerspelledSelected,
				rightHandFieldsUpdatable);

		boolean fingerspelledLeftEnabled = isFingerspelledLeftEnabled(
				fingerspelledEnabled, fingerspelledSelected,
				leftHandFieldsUpdatable);

		boolean oneHandedLeftSelected = isOneHandedLeftSelected(
				leftHandFieldsUpdatable, rightHandFieldsUpdatable);

		boolean oneHandedRightSelected = isOneHandedRightSelected(
				leftHandFieldsUpdatable, rightHandFieldsUpdatable);

		SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection()
				.getDefaultCodingScheme();

		formBean.setSS3CodingScheme(cs);

		formBean.setRightHandEnabled(rightHandFieldsUpdatable);
		formBean.setLeftHansEnabled(leftHandFieldsUpdatable);

		formBean.setType(type);

		formBean.setTwoHandedEnabled(twoHndEnbl);
		formBean.setSameHndshapesEnabled(twoHndEnbl);
		formBean.setDifHndshapesEnabled(twoHndEnbl);

		formBean.setFingerspelledEnabled(fingerspelledEnabled);
		formBean.setFingerspelledREnabled(fingerspelledRightEnabled);
		formBean.setFingerspelledLEnabled(fingerspelledLeftEnabled);
		formBean.setOneHandedEnabled(isOneHandedEnabled());
		formBean.setOneHandedREnabled(oneHandedRightEnabled);
		formBean.setOneHandedLEnabled(oneHandedLeftEnabled);

		formBean.setDifHndshapesSelected(isDifHndshapesSelected(type));

		formBean.setFingerspelledL(oneHandedLeftSelected);
		formBean.setFingerspelledR(oneHandedRightSelected);
		formBean.setFingerspelledSelected(fingerspelledSelected);

		formBean.setOneHandedL(oneHandedLeftSelected);
		formBean.setOneHandedR(oneHandedRightSelected);
		formBean.setOneHandedSelected(oneHandedSelected);
		formBean.setSameHndshapesSelected(isSameHndshapesSelected(type));

		formBean.setTwoHandedL(!rHandSelected);
		formBean.setTwoHandedR(rHandSelected);
		formBean.setTwoHandedSelected(isTwoHandedSelected(type));

		formBean.setRightHandDominant(isRightHandDominant());

		//formBean.setSelectedRightHandShapeImages(getSelectedRightHandShapeImages());
		//formBean.setSelectedLeftHandShapeImages(getSelectedLeftHandShapeImages());

		formBean.setMarkedForm(isMarkedFormChecked());
	}

	private boolean isOneHandedLeftSelected(boolean leftHandFieldsUpdatable,
			boolean rightHandFieldsUpdatable) {
		if (leftHandFieldsUpdatable) {
			if (!rHandSelected) {
				return true;
			} else if (!rightHandFieldsUpdatable) {
				return true;
			}
		}
		return false;
	}

	private boolean isOneHandedRightSelected(boolean leftHandFieldsUpdatable,
			boolean rightHandFieldsUpdatable) {
		if (rightHandFieldsUpdatable) {
			if (rHandSelected) {
				return true;
			} else if (!leftHandFieldsUpdatable) {
				return true;
			}
		}
		return false;
	}

	private boolean isOneHandedRightEnabled(boolean oneHandedEnabled,
			boolean oneHandedSelected, boolean rightHandFieldsUpdatable) {
		return (oneHandedEnabled && oneHandedSelected && rightHandFieldsUpdatable);
	}

	private boolean isOneHandedLeftEnabled(boolean oneHandedEnabled,
			boolean oneHandedSelected, boolean leftHandFieldsUpdatable) {
		return (oneHandedEnabled && oneHandedSelected && leftHandFieldsUpdatable);
	}

	private boolean isFingerspelledRightEnabled(boolean fingerspelledEnabled,
			boolean fingerspelledSelected, boolean rightHandFieldsUpdatable) {
		return (fingerspelledEnabled && fingerspelledSelected && rightHandFieldsUpdatable);
	}

	private boolean isFingerspelledLeftEnabled(boolean fingerspelledEnabled,
			boolean fingerspelledSelected, boolean leftHandFieldsUpdatable) {
		return (fingerspelledEnabled && fingerspelledSelected && leftHandFieldsUpdatable);
	}

	private ArrayList<HandShapeImage> getSelectedRightHandShapeImages() {
		//GlsDependentFields fields = wrapper.getDependentFields();
		
		HandShapeFieldWrapper rhWrp = (HandShapeFieldWrapper)segment.getFieldWrappersCollection()
				.getFieldWrapper(Constants.TYPE_RIGHT_HAND_SHAPE);
		
		RightLeftHandFieldWrapper rightHandWrapper = (RightLeftHandFieldWrapper)segment.getFieldWrappersCollection()
				.getFieldWrapper(Constants.TYPE_RIGHT_HAND);


		ArrayList<HandShapeImage> images = new ArrayList<HandShapeImage>();

		GlossChainedEventsEntity entity = (GlossChainedEventsEntity) SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity(); 
	
		EventsEntity rhEntity = rightHandWrapper.getField().getEntityWithRefID(
				entity.getID());

		if (rhEntity == null) {
			FieldWrappersCollection collection = segment
					.getFieldWrappersCollection();
			String fieldID = wrapper.getField().getFieldID();
			String frstGlsID = "10000";
			String ndGlsID = "10001";

			GlossFieldWrapper ndWrp;
			if (fieldID.equalsIgnoreCase(frstGlsID)) {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(ndGlsID);
			} else {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(frstGlsID);
			}

			EventsEntity ndEntity = ndWrp.getField().getEntityWithRefID(
					entity.getID());
			if (ndEntity == null)
				return images;

			rhEntity = rightHandWrapper.getField().getEntityWithRefID(
					ndEntity.getID());
			if (rhEntity == null)
				return images;

			EventsEntity ent = rhWrp.getEntityWithRefID(rhEntity.getID());

			String rhFieldID = rhWrp.getField().getFieldID();

			SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection()
					.getDefaultCodingScheme();
			SS3Field field = cs.getField(rhFieldID);

			if (ent != null) {
				HandShapeEntity hshEntity = (HandShapeEntity) ent;

                ArrayList<HandShapeEvent> events = hshEntity.getHandShapeEvents();

				for (HandShapeEvent event : events) {
					SS3FieldValue value = field.getFieldValue(event.getID());
					HandShapeImage image = new HandShapeImage(value, cs);
					images.add(image);
				}
			}

			return images;
		}

		EventsEntity ent = rhWrp.getEntityWithRefID(rhEntity.getID());

		String fieldID = rhWrp.getField().getFieldID();

		SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection()
				.getDefaultCodingScheme();
		SS3Field field = cs.getField(fieldID);

		if (ent != null) {
			HandShapeEntity hshEntity = (HandShapeEntity) ent;

			ArrayList hShapes = hshEntity.getHandShapeEvents();
			// HandShapesImageManager hsManager = SS3Singleton
			//		.getHandShapesImageManager();

            for (Object hShape : hShapes) {
                HandShapeEvent event = (HandShapeEvent) hShape;

                SS3FieldValue value = field.getFieldValue(event.getID());
                HandShapeImage image = new HandShapeImage(value, cs);
                images.add(image);
            }
		}
		return images;
	}

	private ArrayList<HandShapeImage> getSelectedLeftHandShapeImages() {
		
		HandShapeFieldWrapper lhWrp = (HandShapeFieldWrapper)segment.getFieldWrappersCollection()
				.getFieldWrapper(Constants.TYPE_LEFT_HAND_SHAPE);
		
		RightLeftHandFieldWrapper leftHandWrapper = (RightLeftHandFieldWrapper)segment.getFieldWrappersCollection()
				.getFieldWrapper(Constants.TYPE_LEFT_HAND);

		ArrayList<HandShapeImage> images = new ArrayList<HandShapeImage>();

		GlossChainedEventsEntity entity = (GlossChainedEventsEntity) SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();
		
		EventsEntity lhEntity = leftHandWrapper.getField().getEntityWithRefID(
				entity.getID());

		if (lhEntity == null) {
			FieldWrappersCollection collection = segment
					.getFieldWrappersCollection();
			String fieldID = wrapper.getField().getFieldID();
			String frstGlsID = "10000";
			String ndGlsID = "10001";

			GlossFieldWrapper ndWrp;
			if (fieldID.equalsIgnoreCase(frstGlsID)) {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(ndGlsID);
			} else {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(frstGlsID);
			}

			EventsEntity ndEntity = ndWrp.getField().getEntityWithRefID(
					entity.getID());
			if (ndEntity == null)
				return images;

			lhEntity = leftHandWrapper.getField().getEntityWithRefID(
					ndEntity.getID());
			if (lhEntity == null)
				return images;

			EventsEntity ent = lhWrp.getEntityWithRefID(lhEntity.getID());

			String lhFieldID = lhWrp.getField().getFieldID();

			SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection()
					.getDefaultCodingScheme();
			SS3Field field = cs.getField(lhFieldID);

			if (ent != null) {
				HandShapeEntity hshEntity = (HandShapeEntity) ent;
				ArrayList hShapes = hshEntity.getHandShapeEvents();

                for (Object hShape : hShapes) {
                    HandShapeEvent event = (HandShapeEvent) hShape;

                    SS3FieldValue value = field.getFieldValue(event.getID());
                    HandShapeImage image = new HandShapeImage(value, cs);
                    images.add(image);
                }
			}

		} else {
			EventsEntity ent = lhWrp.getEntityWithRefID(lhEntity.getID());

			String fieldID = lhWrp.getField().getFieldID();

			SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection()
					.getDefaultCodingScheme();
			SS3Field field = cs.getField(fieldID);

			if (ent != null) {
				HandShapeEntity hshEntity = (HandShapeEntity) ent;
				ArrayList hShapes = hshEntity.getHandShapeEvents();
				// HandShapesImageManager hsManager =
				// SS3Singleton.getHandShapesImageManager();

                for (Object hShape : hShapes) {
                    HandShapeEvent event = (HandShapeEvent) hShape;

                    SS3FieldValue value = field.getFieldValue(event.getID());
                    HandShapeImage image = new HandShapeImage(value, cs);
                    images.add(image);
                }
			}
		}
		return images;
	}

	private boolean isOneHandedEnabled() {
		return (isRightHandFieldUpdatable() || isLeftHandFieldUpdatable());
	}

	private boolean isTwoHandedHandShapeEnabled() {
		GlossChainedEventsEntity glossEntity = (GlossChainedEventsEntity) SS3Singleton
				.getMediaToolBar(null).getSelectedEventsEntity();

		GlossChainedEventsEntity depGlossEntity = glossEntity
				.getDependentGlossEntity();

		if (depGlossEntity != null) {
			return true;
		} else {
			GlossFieldWrapper wrp = (GlossFieldWrapper) segment.getFieldWrapper(Constants.TYPE_DOM_GLOSS);
			
			if(wrapper.equals(wrp)) {
				wrp = (GlossFieldWrapper) segment.getFieldWrapper(Constants.TYPE_NON_DOM_GLOSS);
			}
			
			int startTime = glossEntity.getStartTimeInfo().getMovieTime();
			int endTime = glossEntity.getEndTimeInfo().getMovieTime();

			return wrp.getField().isEmptyInterval(startTime, endTime);
		}
	}

	private boolean isFingerspelledEnabled() {
        return rHandSelected && isRightHandFieldUpdatable() || lHandSelected && isLeftHandFieldUpdatable();
    }

	private boolean isRightHandFieldUpdatable() {

		GlossChainedEventsEntity glossEntity = (GlossChainedEventsEntity) SS3Singleton
				.getMediaToolBar(null).getSelectedEventsEntity();

		FieldWrapper rhWrapper = segment.getFieldWrappersCollection()
				.getFieldWrapper(Constants.TYPE_RIGHT_HAND);
		String rhID = rhWrapper.getField().getFieldID();

		ChainedEventsEntity depHandEntity = glossEntity
				.getDependentHandEntity();
		if (depHandEntity != null) {
			String id = depHandEntity.getField().getFieldID();
			if (rhID.equalsIgnoreCase(id)) {
				return true;
			}
		}

		GlossChainedEventsEntity depGlossEntity = glossEntity
				.getDependentGlossEntity();
		if (depGlossEntity != null) {
			ChainedEventsEntity depHandEntity1 = depGlossEntity
					.getDependentHandEntity();
			if (depHandEntity1 != null) {
				String id1 = depHandEntity1.getField().getFieldID();
				if (rhID.equalsIgnoreCase(id1)) {
					return true;
				}
			}
		}

		RightLeftHandFieldWrapper rlhWrp = (RightLeftHandFieldWrapper) rhWrapper;
		int startTime = glossEntity.getStartTimeInfo().getMovieTime();
		int endTime = glossEntity.getEndTimeInfo().getMovieTime();

		return rlhWrp.getField().isEmptyInterval(startTime, endTime);
	}

	private boolean isLeftHandFieldUpdatable() {
		GlossChainedEventsEntity glossEntity = (GlossChainedEventsEntity) SS3Singleton
				.getMediaToolBar(null).getSelectedEventsEntity();

		FieldWrapper lhWrapper = segment.getFieldWrappersCollection()
				.getFieldWrapper(Constants.TYPE_LEFT_HAND);
		String lhID = lhWrapper.getField().getFieldID();

		ChainedEventsEntity depHandEntity = glossEntity
				.getDependentHandEntity();
		if (depHandEntity != null) {
			String id = depHandEntity.getField().getFieldID();
			if (lhID.equalsIgnoreCase(id)) {
				return true;
			}
		}

		GlossChainedEventsEntity depGlossEntity = glossEntity
				.getDependentGlossEntity();
		if (depGlossEntity != null) {
			ChainedEventsEntity depHandEntity1 = depGlossEntity
					.getDependentHandEntity();
			if (depHandEntity1 != null) {
				String id1 = depHandEntity1.getField().getFieldID();
				if (lhID.equalsIgnoreCase(id1)) {
					return true;
				}
			}
		}

		RightLeftHandFieldWrapper lhWrp = (RightLeftHandFieldWrapper) lhWrapper;
		int startTime = glossEntity.getStartTimeInfo().getMovieTime();
		int endTime = glossEntity.getEndTimeInfo().getMovieTime();

		return lhWrp.getField().isEmptyInterval(startTime, endTime);
	}

	private boolean isRightHandDominant() {
		String pID = segment.getParticipantID();
		ParticipantsLibrary lib = SS3Singleton.getManager()
				.getLoadedParticipantsLibrary();
		SS3Participant prt = lib.getParticipant(pID);

		return prt.isDominantHandRight();
	}

	private boolean isRightHandSelected() {
		ArrayList<HandShapeImage> rList = getSelectedRightHandShapeImages();
		ArrayList<HandShapeImage> lList = getSelectedLeftHandShapeImages();

		if (rList != null && lList != null)
			return isRightHandDominant();

		if (rList != null && !rList.isEmpty())
			return true;

        return !(lList != null && !lList.isEmpty()) && isRightHandDominant();
    }

	private boolean isFingerspelledSelected(char type) {
		return type == HandShapeEntityTypes.FINGERSPELLED;
	}

	private boolean isOneHandedSelected(char type) {
		return type == HandShapeEntityTypes.ONE_HANDED;
	}

	private boolean isTwoHandedSelected(char type) {
		return type == HandShapeEntityTypes.TWO_HANDED_SAME
				|| type == HandShapeEntityTypes.TWO_HANDED_DIF;
	}

	private boolean isSameHndshapesSelected(char type) {
		return type == HandShapeEntityTypes.TWO_HANDED_SAME;
	}

	private boolean isDifHndshapesSelected(char type) {
		return type == HandShapeEntityTypes.TWO_HANDED_DIF;
	}

	private boolean isMarkedFormChecked() {

		GlossChainedEventsEntity entity = (GlossChainedEventsEntity) SS3Singleton
				.getMediaToolBar(null).getSelectedEventsEntity();

		ArrayList<EventsEntity> dEntities = entity.getDependentEntities();

		DependentEntitiesManager depMngr = new DependentEntitiesManager(
				dEntities);

		HandShapeFieldWrapper rHshWrp = (HandShapeFieldWrapper) segment
				.getFieldWrappersCollection().getFieldWrapper(
						Constants.TYPE_RIGHT_HAND_SHAPE);

		if (rHshWrp != null) {
			EventsEntity rEnt = depMngr.getEventsEntityWithFieldID(rHshWrp
					.getField().getFieldID());

			if (rEnt != null) {
				HandShapeEntity rHshEnt = (HandShapeEntity) rEnt;
				return rHshEnt.isMarkedHand();
			}
		}

		HandShapeFieldWrapper lHshWrp = (HandShapeFieldWrapper) segment
				.getFieldWrappersCollection().getFieldWrapper(
						Constants.TYPE_LEFT_HAND_SHAPE);

		if (lHshWrp != null) {
			EventsEntity lEnt = depMngr.getEventsEntityWithFieldID(lHshWrp
					.getField().getFieldID());

			if (lEnt != null) {
				HandShapeEntity lHshEnt = (HandShapeEntity) lEnt;
				return lHshEnt.isMarkedHand();
			}
		}
		return false;
	}

	private char getSelectedHandShapeType() {

		GlossChainedEventsEntity entity = (GlossChainedEventsEntity) SS3Singleton
				.getMediaToolBar(null).getSelectedEventsEntity();

		ArrayList<EventsEntity> dEntities = entity.getDependentEntities();

		DependentEntitiesManager depMngr = new DependentEntitiesManager(
				dEntities);

		HandShapeFieldWrapper rHshWrp = (HandShapeFieldWrapper) segment
				.getFieldWrappersCollection().getFieldWrapper(
						Constants.TYPE_RIGHT_HAND_SHAPE);

		if (rHshWrp != null) {
			EventsEntity rEnt = depMngr.getEventsEntityWithFieldID(rHshWrp
					.getField().getFieldID());

			if (rEnt != null) {
				HandShapeEntity rHshEnt = (HandShapeEntity) rEnt;
				return rHshEnt.getHandShapeType();
			}
		}

		HandShapeFieldWrapper lHshWrp = (HandShapeFieldWrapper) segment
				.getFieldWrappersCollection().getFieldWrapper(
						Constants.TYPE_LEFT_HAND_SHAPE);
		if (lHshWrp != null) {
			EventsEntity lEnt = depMngr.getEventsEntityWithFieldID(lHshWrp
					.getField().getFieldID());

			if (lEnt != null) {
				HandShapeEntity lHshEnt = (HandShapeEntity) lEnt;
				return lHshEnt.getHandShapeType();
			}
		}
		return HandShapeEntityTypes.ONE_HANDED;
	}
}
