package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.text.BadLocationException;
import javax.swing.text.DocumentFilter;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;

public class LettersHandshapesPalette extends PalettePanel {

	private static final long serialVersionUID = 1L;

	private HashMap<String, String> idNLetter = new HashMap<String, String>();
	
	private PaletteUtil paletteUtil = null;

	public LettersHandshapesPalette(ArrayList<SS3FieldValue> handshapes, PaletteUtil paletteUtil, ArrayList<HandShapeImage> selectedHshImages) {
		this.paletteUtil = paletteUtil;
		
		paletteUtil.setLetter(true);

		for (int i = 0; i < handshapes.size(); i++) {
			Object obj = handshapes.get(i);
			SS3FieldValue value = (SS3FieldValue) obj;
			paletteUtil.addLetter(value.getId(), value);
		}

		populate();
		initUI();

		StringBuffer sb = new StringBuffer();
		for(HandShapeImage handshape:selectedHshImages) {
			SS3FieldValue value = handshape.getSS3FieldValue();
			String iid = value.getIid();
			String letter = idNLetter.get(iid);
			if(paletteUtil.containsLetter(letter)) {
				sb.append(letter);
			}
		}		
		paletteUtil.getLettersTxtfld().setText(sb.toString());
	}
	
	public void synchronizeHandshapeWithTextfield(ArrayList<HandShapeImage> handshapes) {
		StringBuffer sb = new StringBuffer();
		for(HandShapeImage handshape:handshapes) {
			SS3FieldValue value = handshape.getSS3FieldValue();
			String iid = value.getIid();
			String letter = idNLetter.get(iid);
			if(paletteUtil.containsLetter(letter)) {
				sb.append(letter);
			}
		}
		paletteUtil.getLettersTxtfld().setText(sb.toString());
	}

	private JPanel handshapesPanel = new JPanel();
	GridBagConstraints c = new GridBagConstraints();
	JScrollPane scrollPane = new JScrollPane(handshapesPanel);

	private void initUI() {
		setLayout(new GridBagLayout());

		handshapesPanel.setLayout(new GridBagLayout());

		JPanel controlsPanel = new JPanel();
		GridBagLayout gridbag1 = new GridBagLayout();
		GridBagConstraints c1 = new GridBagConstraints();
		controlsPanel.setLayout(gridbag1);

		c1.insets = new Insets(1, 1, 1, 1);

		populateHahdshapesPanel();

		int h = paletteUtil.getLettersTxtfld().getPreferredSize().height;
		paletteUtil.getLettersTxtfld().setPreferredSize(new Dimension(450, h));

		scrollPane.setPreferredSize(new Dimension(450, 385 - h));
		scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		c.gridwidth = 1;
		c.gridy = 1;
		c.gridx = 1;
		add(controlsPanel, c);
		
		c.gridwidth = 1;
		c.gridy = 3;
		c.gridx = 1;
		add(scrollPane, c);

		c.gridy = 4;
		c.gridx = 1;
		add(paletteUtil.getLettersTxtfld(), c);
	}

	private void populateHahdshapesPanel() {
		handshapesPanel.removeAll();
		handshapesPanel.setBackground(UIPalette.getBackground());

		c.insets = new Insets(0, 0, 0, 0);

		int longestHandshapeeList = sortedHandshapesCollectionLeft.size();
		if (longestHandshapeeList < sortedHandshapesCollectionCenter.size()) {
			longestHandshapeeList = sortedHandshapesCollectionRight.size();
		}

		if (longestHandshapeeList < sortedHandshapesCollectionRight.size()) {
			longestHandshapeeList = sortedHandshapesCollectionRight.size();
		}

		int y = 0;
		int height = 10;
		for (int i = 0; i < longestHandshapeeList; i++) {

			c.gridwidth = 1;
			c.gridy = y;
			c.gridx = 1;
			c.anchor = GridBagConstraints.NORTHEAST;

			if (sortedHandshapesCollectionLeft.size() > i) {
				FieldValueWrapper wrapperLeft = sortedHandshapesCollectionLeft.get(i);
				SS3FieldValue valueLeft = wrapperLeft.getSS3FieldValue();
				
				JLabel l1 = paletteUtil.getHandhsapeVisualRepresentation(valueLeft, wrapperLeft.getBackground());
				handshapesPanel.add(new JLabel(" "), c);

				c.gridx = 2;
				c.weightx = 1;
				c.anchor = GridBagConstraints.NORTHWEST;
				handshapesPanel.add(l1, c);
			}

			c.gridx = 3;

			if (sortedHandshapesCollectionCenter.size() > i) {
				final FieldValueWrapper wrapperCenter = sortedHandshapesCollectionCenter
						.get(i);
				final SS3FieldValue valueCenter = wrapperCenter
						.getSS3FieldValue();

				c.anchor = GridBagConstraints.NORTHEAST;

				JLabel l2 = paletteUtil.getHandhsapeVisualRepresentation(valueCenter, wrapperCenter.getBackground());
				handshapesPanel.add(new JLabel(" "), c);

				c.gridx = 4;
				c.weightx = 1;
				c.anchor = GridBagConstraints.NORTHWEST; 
				handshapesPanel.add(l2, c);
			}

			c.gridx = 5;

			if (sortedHandshapesCollectionRight.size() > i) {
				final FieldValueWrapper wrapperRight = sortedHandshapesCollectionRight
						.get(i);
				final SS3FieldValue valueRight = wrapperRight
						.getSS3FieldValue();

				c.anchor = GridBagConstraints.NORTHEAST;

				JLabel l3 = paletteUtil.getHandhsapeVisualRepresentation(valueRight, wrapperRight.getBackground());
				handshapesPanel.add(new JLabel(" "), c);

				c.gridx = 6;
				c.weightx = 1;
				c.anchor = GridBagConstraints.NORTHWEST;
				handshapesPanel.add(l3, c);
			}
			y++;
		}

		int j = 20 - y;
		for (int k = 0; k < j; k++) {
			c.gridwidth = 1;
			c.gridy = GridBagConstraints.RELATIVE;
			c.gridy = y;
			c.gridx = 2;
			JLabel label = createPlainLabel(height);
			handshapesPanel.add(label, c);
			y++;
		}
	}

	ArrayList<FieldValueWrapper> sortedHandshapesCollectionLeft = new ArrayList<FieldValueWrapper>();
	ArrayList<FieldValueWrapper> sortedHandshapesCollectionCenter = new ArrayList<FieldValueWrapper>();
	ArrayList<FieldValueWrapper> sortedHandshapesCollectionRight = new ArrayList<FieldValueWrapper>();

	private void populate() {
		Color color2 = new Color(255, 255, 255);
		Color color1 = new Color(123, 166, 172);

		FieldValueWrapper aw = createFieldValueWrapper("14", "HSH_014", "A",
				false, false, color1);
		paletteUtil.addLetter("A", aw.getSS3FieldValue());
		idNLetter.put("HSH_014", "A");
		sortedHandshapesCollectionLeft.add(aw);

		FieldValueWrapper bw = createFieldValueWrapper("20", "HSH_020", "B",
				false, false, color2);
		paletteUtil.addLetter("B", bw.getSS3FieldValue());
		idNLetter.put("HSH_020", "B");
		sortedHandshapesCollectionLeft.add(bw);

		FieldValueWrapper cw = createFieldValueWrapper("5", "HSH_005", "C",
				false, false, color1);
		paletteUtil.addLetter("C", cw.getSS3FieldValue());
		idNLetter.put("HSH_005", "C");
		sortedHandshapesCollectionLeft.add(cw);

		FieldValueWrapper dw = createFieldValueWrapper("42", "HSH_042", "D",
				false, false, color2);
		paletteUtil.addLetter("D", dw.getSS3FieldValue());
		idNLetter.put("HSH_042", "D");
		sortedHandshapesCollectionLeft.add(dw);

		FieldValueWrapper ew = createFieldValueWrapper("43", "HSH_043", "E",
				false, false, color1);
		paletteUtil.addLetter("E", ew.getSS3FieldValue());
		idNLetter.put("HSH_043", "E");
		sortedHandshapesCollectionLeft.add(ew);

		FieldValueWrapper fw = createFieldValueWrapper("44", "HSH_044", "F/9", "F", 
				false, false, color2);
		paletteUtil.addLetter("F", fw.getSS3FieldValue());
		idNLetter.put("HSH_044", "F");
		sortedHandshapesCollectionLeft.add(fw);

		FieldValueWrapper gw = createFieldValueWrapper("52", "HSH_053", "G/Q", "G",
				false, false, color1);
		paletteUtil.addLetter("G", gw.getSS3FieldValue());
		idNLetter.put("HSH_053", "G");
		sortedHandshapesCollectionLeft.add(gw);

		FieldValueWrapper hw = createFieldValueWrapper("76", "HSH_076", "U/H", "H",
				false, false, color2);
		paletteUtil.addLetter("H", hw.getSS3FieldValue());
		idNLetter.put("HSH_076", "H");
		sortedHandshapesCollectionLeft.add(hw);

		FieldValueWrapper iw = createFieldValueWrapper("57", "HSH_057", "I",
				false, false, color1);
		paletteUtil.addLetter("I", iw.getSS3FieldValue());
		idNLetter.put("HSH_057", "I");
		sortedHandshapesCollectionLeft.add(iw);

		FieldValueWrapper jw = createFieldValueWrapper("97", "HSH_097", "J",
				false, false, color1);
		paletteUtil.addLetter("J", jw.getSS3FieldValue());
		idNLetter.put("HSH_097", "J");
		sortedHandshapesCollectionCenter.add(jw);

		FieldValueWrapper kw = createFieldValueWrapper("58", "HSH_058", "P/K", "K",
				false, false, color2);
		paletteUtil.addLetter("K", kw.getSS3FieldValue());
		idNLetter.put("HSH_058", "K");
		sortedHandshapesCollectionCenter.add(kw);

		FieldValueWrapper lw = createFieldValueWrapper("60", "HSH_060", "L",
				false, false, color1);
		paletteUtil.addLetter("L", lw.getSS3FieldValue());
		idNLetter.put("HSH_060", "L");
		sortedHandshapesCollectionCenter.add(lw);

		FieldValueWrapper mw = createFieldValueWrapper("62", "HSH_062", "M",
				false, false, color2);
		paletteUtil.addLetter("M", mw.getSS3FieldValue());
		idNLetter.put("HSH_062", "M");
		sortedHandshapesCollectionCenter.add(mw);

		FieldValueWrapper nw = createFieldValueWrapper("63", "HSH_063", "N",
				false, false, color2);
		paletteUtil.addLetter("N", nw.getSS3FieldValue());
		idNLetter.put("HSH_063", "N");
		sortedHandshapesCollectionCenter.add(nw);

		FieldValueWrapper ow = createFieldValueWrapper("65", "HSH_065", "O",
				false, false, color2);
		paletteUtil.addLetter("O", ow.getSS3FieldValue());
		idNLetter.put("HSH_065", "O");
		sortedHandshapesCollectionCenter.add(ow);

		FieldValueWrapper pw = createFieldValueWrapper("58", "HSH_058", "P/K", "P",
				false, false, color1);
		paletteUtil.addLetter("P", pw.getSS3FieldValue());
		idNLetter.put("HSH_058", "P");
		sortedHandshapesCollectionCenter.add(pw);

		FieldValueWrapper qw = createFieldValueWrapper("53", "HSH_053", "G/Q", "Q",
				false, false, color2);
		paletteUtil.addLetter("Q", qw.getSS3FieldValue());
		idNLetter.put("HSH_053", "Q");
		sortedHandshapesCollectionCenter.add(qw);

		FieldValueWrapper rw = createFieldValueWrapper("70", "HSH_070", "R",
				false, false, color1);
		paletteUtil.addLetter("R", rw.getSS3FieldValue());
		idNLetter.put("HSH_070", "R");
		sortedHandshapesCollectionCenter.add(rw);

		FieldValueWrapper sw = createFieldValueWrapper("82", "HSH_082", "S",
				false, false, color1);
		paletteUtil.addLetter("S", sw.getSS3FieldValue());
		idNLetter.put("HSH_082", "S");
		sortedHandshapesCollectionRight.add(sw);

		FieldValueWrapper tw = createFieldValueWrapper("72", "HSH_072", "T",
				false, false, color2);
		paletteUtil.addLetter("T", tw.getSS3FieldValue());
		idNLetter.put("HSH_072", "T");
		sortedHandshapesCollectionRight.add(tw);

		FieldValueWrapper uw = createFieldValueWrapper("76", "HSH_076", "U/H", "U", 
				false, false, color1);
		paletteUtil.addLetter("U", uw.getSS3FieldValue());
		idNLetter.put("HSH_076", "U");
		sortedHandshapesCollectionRight.add(uw);

		FieldValueWrapper vw = createFieldValueWrapper("77", "HSH_077", "V/2", "V",
				false, false, color2);
		paletteUtil.addLetter("V", vw.getSS3FieldValue());
		idNLetter.put("HSH_077", "V");
		sortedHandshapesCollectionRight.add(vw);

		FieldValueWrapper ww = createFieldValueWrapper("78", "HSH_078", "W",
				false, false, color1);
		paletteUtil.addLetter("W", ww.getSS3FieldValue());
		idNLetter.put("HSH_078", "W");
		sortedHandshapesCollectionRight.add(ww);

		FieldValueWrapper xw = createFieldValueWrapper("79", "HSH_079", "X",
				false, false, color2);
		paletteUtil.addLetter("X", xw.getSS3FieldValue());
		idNLetter.put("HSH_079", "X");
		sortedHandshapesCollectionRight.add(xw);

		FieldValueWrapper yw = createFieldValueWrapper("81", "HSH_081", "Y",
				false, false, color1);
		paletteUtil.addLetter("Y", yw.getSS3FieldValue());
		idNLetter.put("HSH_081", "Y");
		sortedHandshapesCollectionRight.add(yw);

		FieldValueWrapper zw = createFieldValueWrapper("96", "HSH_096", "Z",
				false, false, color2);
		paletteUtil.addLetter("Z", zw.getSS3FieldValue());
		idNLetter.put("HSH_096", "Z");
		sortedHandshapesCollectionRight.add(zw);
	}

	public void clearEnteredIcons() {
		paletteUtil.getLettersTxtfld().setText("");
	}
}

class LetterTypedListener extends KeyAdapter {

	private PaletteUtil paletteUtil = null;

	public LetterTypedListener(PaletteUtil paletteUtil) {
		this.paletteUtil = paletteUtil;
	}
	
	@Override
	public void keyReleased(KeyEvent e) {
		JTextField tf = (JTextField)e.getSource();
		
		paletteUtil.getHandShapePalletClient().removeAll();
		char[] c = tf.getText().toUpperCase().toCharArray();
		for (int i = 0; i < c.length; i++) {
			char cc = c[i];
			if (Character.isLetter(cc)) {
				String s = "" + cc;
				SS3FieldValue obj = paletteUtil.getFieldValueForLetter(s);
				if (obj != null)
					paletteUtil.getHandShapePalletClient().setItem(obj);
			}
		}
	}
}

class NonNumericFilter extends DocumentFilter {

	@Override
	public void insertString(DocumentFilter.FilterBypass fb, int offset,
			String text, javax.swing.text.AttributeSet attr)
			throws BadLocationException {
		
		fb.insertString(offset, text.replaceAll("\\d", ""), attr);
	}

	@Override
	public void replace(DocumentFilter.FilterBypass fb, int offset, int length,
			String text, javax.swing.text.AttributeSet attr)
			throws BadLocationException {
		
		if (text.matches("^[a-zA-Z]*$")) {
			fb.replace(offset, length, text.replaceAll("\\d", ""), attr);
		}
	}
}
