package edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapePanel;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.ui.ButtonPanel;

public class HandshapePalette extends JPanel implements ActionListener {

	private HandShapesPaletteFormBean formBean = null;
	private HandShapeBean bean = null;
	private GeneralHandshapePalette palette = null;
    private HandShapePanel hsPanel = null;

	private JDialog dialog = null;

	private final int ENTER = 0;
	private final int CANCEL = 1;

    private final JComboBox<String> options = new JComboBox<String>(new String[] {"All", "Letters", "Numbers", "Unmarked"});

	public HandshapePalette(HandShapesPaletteFormBean formBean,
			HandShapeBean bean, SS3CodingScheme scheme, JDialog dialog, int selectedIndex, HandShapePanel hsPanel) {

		this.formBean = formBean;
		this.bean = bean;
		this.dialog = dialog;
        this.hsPanel = hsPanel;

		palette = new GeneralHandshapePalette(formBean.getValues(),
				formBean.getSelectedHshImages(), scheme, selectedIndex, hsPanel);
		palette.setBorder(new TitledBorder(formBean.getTitle()));

//        options.addItemListener(new WeakItemListener(new ItemListener() {
//            @Override
//            public void itemStateChanged(ItemEvent itemEvent) {
//                int selectedIndex = options.getSelectedIndex();
//                palette.setPalette(selectedIndex);
//            }
//        }, options));

        options.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent itemEvent) {
                int selectedIndex = options.getSelectedIndex();
                palette.setPalette(selectedIndex);
            }
        });

        options.setSelectedIndex(selectedIndex);

        setLayout(new BorderLayout());

        JPanel languagePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        languagePanel.add(new JLabel("Palettes: "));
        languagePanel.add(new JComboBox<String>(new String[] {"American Sign Language"}));
        languagePanel.add(options);

        JButton enter = new JButton("Enter");
        JButton cancel = new JButton("Cancel");

        ButtonPanel buttonPanel = new ButtonPanel();
        buttonPanel.add(cancel);
        buttonPanel.add(enter);
        add(languagePanel, BorderLayout.NORTH);
        add(palette, BorderLayout.CENTER);
        add(buttonPanel, BorderLayout.SOUTH);

		enter.setActionCommand("" + ENTER);
		cancel.setActionCommand("" + CANCEL);
		enter.addActionListener(new WeakActionListener(this, enter));
		cancel.addActionListener(new WeakActionListener(this, cancel));

		enter.setSelected(true);
		dialog.getRootPane().setDefaultButton(enter);
	}
	
	public void update(HandShapesPaletteFormBean formBean, HandShapeBean bean, int selectedIndex) {
		this.bean = bean;
		this.formBean = formBean;
        options.setSelectedIndex(selectedIndex);

        palette.setSelectedHshImages(formBean.getSelectedHshImages());
        palette.repaint();
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case ENTER:
            ArrayList<HandShapeImage> hsImages = new ArrayList<HandShapeImage>(palette.getSelectedHandshapes());

			// first check if this is a one-frame gloss, and if it is then disallow anything more than one handshape in either hand
	    		if (hsPanel.isOneFrameGloss()) {
	    			if (hsImages.size() > 1) {
	    				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_HANDSHAPES_EXACTLY_ONE);
	    				break;
	    			}
	    		}
	    		// conversely, if this is not a one-frame gloss and you have dif start/end selected, 
	    		// then disallow a selection of just one handshape in either hand
	    		else if (hsPanel.getSelectedSameDifStartEndHandshapes() == 1 && (hsImages.size() == 1)) {
    				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_HANDSHAPES_MULTIPLE);
    				break;
	    		}
	    		
			bean.setUpdated(true);

			if (formBean.getTitle().equalsIgnoreCase(
					HandShapePanel.RIGHT_HAND_TITLE)) {
				bean.setSelectedRightHandShapeImages(palette
						.getSelectedHandshapes());
			}

			if (formBean.getTitle().equalsIgnoreCase(
					HandShapePanel.LEFT_HAND_TITLE)) {
				bean.setSelectedLeftHandShapeImages(palette
						.getSelectedHandshapes());
			}

			if (formBean.getTitle().equalsIgnoreCase(
					HandShapePanel.RIGHT_N_LEFT_HANDS_TITLE)) {
				bean.setSelectedRightHandShapeImages(palette
						.getSelectedHandshapes());
				bean.setSelectedLeftHandShapeImages(palette
						.getSelectedHandshapes());
			}	    	

			hsPanel.updateSelectedHandshapesPanels(bean.getSelectedRightHandShapeImages(), 
					bean.getSelectedLeftHandShapeImages());

			dialog.setVisible(false);
			break;
		case CANCEL:
			dialog.setVisible(false);
			break;
		}
	}

	public ArrayList<HandShapeImage> getSelectedHandshapes() {
		return palette.getSelectedHandshapes();
	}
	
	public GeneralHandshapePalette getPalette() {
		return palette;
	}
}
