package edu.bu.signstream.grepresentation.fields.handShapeField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsIterator;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.HandshapeEntityPainter;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class HandShapeEntity extends HandShapeSubEntity {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private NewHandShapeEntity newEntity = null;
	private char type;
	private boolean markedHand = false;

	private boolean isLockedGlossPhonAlignments = true;

	private HandShapeSubEntitiesManager manager = new HandShapeSubEntitiesManager();

	public HandShapeEntity(SignStreamSegmentPanel segmentPanel, Field field) {
		super(segmentPanel, field);
	}

	public HandShapeEntity(String fieldID, SS3Entity entity,
			SignStreamSegmentPanel segmentPanel, Field field) {
		super(segmentPanel, field);
		//this.updDelegate = new EntityUpdateDelegate(this, segmentPanel);

		HandShapeSubEntity subEntity = new HandShapeSubEntity(segmentPanel,
				field);
		this.addHandShapeSubEntity(subEntity);

        ArrayList<SS3Item> items = entity.getItemsAsList();
		for (SS3Item item : items) {
			subEntity.addHandShape(item);
		}
		subEntity.determineParentEntity();
	}
	
	@Override
	public void setID(String id) {
		super.setID(id);
	}
	
	public void resetEntity(SS3Entity entity) {
		ArrayList<HandShapeSubEntity> entities = this.getSubEntities();
		
		if(entities.size() < 1)
			return;
		
		HandShapeSubEntity subEntity = entities.get(0);		
		HandShapesManager manager = subEntity.getHandShapesManager();
		manager.deleteEntity();
        ArrayList<SS3Item> items = entity.getItemsAsList();
		for (SS3Item item : items) {
			subEntity.addHandShape(item);
		}
	}

    public void cloneEvents(int time) {
        Event firstEvent = getFirstHandShapeEvent();
        Event lastEvent = getLastHandShapeEvent();

        if(firstEvent.equals(lastEvent)) {
            HandShapeEvent hshEvent = (HandShapeEvent)firstEvent;

            TimeInfo timeInfo = firstEvent.getStartTimeInfo().cloneTimeInfo();
            timeInfo.setMovieTime(time);

            SS3Entity entity = new SS3Entity();
            entity.addItem(hshEvent.getSS3Item());
            SS3Item item = hshEvent.getSS3Item();
            SS3Item nextItem = new SS3Item(item.getId(), item.getImageID(), timeInfo, timeInfo);

            nextItem.setImageID(item.getImageID());

            entity.addItem(nextItem);
            resetEntity(entity);
        }
    }
    
    	public boolean mergeEvents(int time) {
    		Event firstEvent = getFirstHandShapeEvent();
    		Event lastEvent = getLastHandShapeEvent();
    		
		HandShapeEvent firstHshEvent = (HandShapeEvent)firstEvent;
		HandShapeEvent lastHshEvent = (HandShapeEvent)lastEvent;
		
		String firstImageID = firstHshEvent.getImageID();
		String lastImageID = lastHshEvent.getImageID();
		
		if ( firstImageID.equals(lastImageID)) {
			SS3Entity entity = new SS3Entity();
			
			TimeInfo timeInfo = firstHshEvent.getStartTimeInfo().cloneTimeInfo();
			timeInfo.setMovieTime(time);

            SS3Item firstItem = firstHshEvent.getSS3Item();
			SS3Item newItem = new SS3Item(firstItem.getId(), firstItem.getImageID(), timeInfo, timeInfo);
			
			entity.addItem(newItem);
			resetEntity(entity);
			
			return true;
		}
    		
    		return false;
    	}
    	
    	public boolean isPossibleToMergeEvents() {
		Event firstEvent = getFirstHandShapeEvent();
		Event lastEvent = getLastHandShapeEvent();
		
		HandShapeEvent firstHshEvent = (HandShapeEvent)firstEvent;
		HandShapeEvent lastHshEvent = (HandShapeEvent)lastEvent;
		
		String firstImageID = firstHshEvent.getImageID();
		String lastImageID = lastHshEvent.getImageID();
		
		if ( firstImageID.equals(lastImageID)) {
			return true;
		}
    		
    		return false;
    	}
		
	public boolean isLockedGlossPhonAlignments() {
		return isLockedGlossPhonAlignments;
	}

	public void setLockedGlossPhonAlignments(boolean isLockedGlossPhonAlignments) {
		this.isLockedGlossPhonAlignments = isLockedGlossPhonAlignments;
	}

	public boolean minifyAllIcons() {
		boolean minified = false;
        ArrayList<HandShapeEvent> events = manager.getEvents();
		for (HandShapeEvent event : events) {
			if (event.isMagnifiedImage()) {
				event.setMagnifiedImage(false);
				minified = true;
			}
		}
		return minified;
	}

	public void setHandShapeType(char type) {
		this.type = type;
	}

	public char getHandShapeType() {
		return type;
	}

	public NewHandShapeEntity getNewHandShapeEntity() {
		return newEntity;
	}

	public void setNewHandShapeEntity(NewHandShapeEntity newEntity) {
		this.newEntity = newEntity;
	}

	public EventsIterator getEventsIterator() {
		return new EventsIterator(getEvents());
	}

	public boolean hasSubEntities() {
		return !manager.getSubEntities().isEmpty();
	}

	public void addHandShapeSubEntity(HandShapeSubEntity subEntity) {
		manager.addSubEntity(subEntity);
	}

	@Override
	public boolean contains(int x) {
		if (getFirstEvent() == null || getLastEvent() == null) {
			return false;
		}
		int startTimeCoord = getFirstEvent().getStartTimeCoordinate();
		int endTimeCoord = getLastEvent().getStartTimeCoordinate();

		return (x >= startTimeCoord && x <= endTimeCoord);
	}

	@Override
	public SS3Entity getSS3Entity() {
		SS3Entity entity = super.getSS3Entity();

        ArrayList<HandShapeEvent> col = manager.getEvents(); // .getHandShapesCollection();
		for (HandShapeEvent event : col) {
			if (event != null) {
				SS3Item item = new SS3Item();
				item.setId(event.getID());
				item.setStartTimeInfo(event.getStartTimeInfo());
				item.setEndTimeInfo(event.getEndTimeInfo());
				item.setText(event.getText());
				entity.addItem(item);
			}
		}
		
		entity.setID(getID());
		entity.setRefEntityId(getRefEntityId());
		entity.setLockedGlossPhoneAlignments(isLockedGlossPhonAlignments);

		return entity;
	}

	@Override
	public boolean isSelected() {
		return false;
	}

	@Override
	public boolean isOverlapping(int coordx1, int coordx2) {
		int startCoord = getStartTimeCoordinate();
		int endCoord = getEndTimeCoordinate();

		return (coordx1 <= startCoord && coordx2 >= startCoord)
				|| (coordx1 <= endCoord && coordx2 >= endCoord);
	}

	@Override
	public int getStartTimeCoordinate() {
		// if (initialHold != null)
		// return initialHold.getStartTimeCoordinate();
		// return manager.getStartTimeCoordinate();
		return getFirstEvent().getStartTimeCoordinate();
	}

	@Override
	public Event getFirstEvent() {
		// if (initialHold != null)
		// return initialHold;
		return manager.getFirstEvent();
	}

	@Override
	public Event getLastEvent() {
		// if (finalHold != null)
		// return finalHold;
		return manager.getLastEvent();
	}

	public Event getFirstHandShapeEvent() {
		return manager.getFirstEvent();
	}

	public Event getLastHandShapeEvent() {
		return manager.getLastEvent();
	}

	@Override
	public int getEndTimeCoordinate() {
		// if (finalHold != null)
		// return finalHold.getEndTimeCoordinate();
		// return manager.getEndTimeCoordinate();
		return getLastHandShapeEvent().getEndTimeCoordinate();
	}

	@Override
	public void addHandShape(SS3Item item) {
		// HandShapeSubEntity ent = manager.getSuperEntity();
		// ent.addHandShape(item);
		// manager.addHandShape(item);
	}

	public void addHandShapeEvent(HandShapeEvent event) {
		HandShapeSubEntity entity = manager.getSuperEntity();
		entity.addHandShape(event);
	}

	@Override
	public void addHandShape(TrackValue value) {
		// hsEventsManager.addHandShape(value);
	}

	@Override
	public void setStartTime(int startTime) {
		Event event = manager.getFirstEvent();
		event.getStartTimeInfo().setMovieTime(startTime);
		event.getEndTimeInfo().setMovieTime(startTime);
	}

	@Override
	public void setEndTime(int endTime) {
		Event event = manager.getLastEvent();
		event.getEndTimeInfo().setMovieTime(endTime);
		event.getStartTimeInfo().setMovieTime(endTime);
	}

	@Override
	public int getEndTime() {
		Event event = manager.getLastEvent();

		if (event == null)
			return 0;

		return event.getEndTimeInfo().getMovieTime();
	}

	@Override
	public int getStartTime() {
		Event event = manager.getFirstEvent();

		if (event == null)
			return 0;

		return event.getEndTimeInfo().getMovieTime();
	}

	@Override
	public TimeInfo getStartTimeInfo() {
		return getFirstEvent().getStartTimeInfo();
	}

    public void setStartTime() {
        // check if entity has only one handshape, create another handshape
        if(getFirstEvent().equals(getLastEvent())) {
            getLastEvent();
        }
    }

	@Override
	public TimeInfo getEndTimeInfo() {
		return getLastEvent().getStartTimeInfo();
		// .getEndTimeInfo();
	}

	@Override
	public boolean selectEntityIfEventSelected() {
		if (manager.selectEntityIfEventSelected())
			select();
		return selected;
	}

	@Override
	public boolean selectEventIfEntitySelected() {
        return selected && manager.selectEventIfEntitySelected();
    }

	@Override
	public boolean select() {
		// hsEventsManager.highlightEntity(Palette.HIGHLIGHTED_ENTITY_COLOR);

		// QTMultiMovieController qtControl =
		// SSSingleton.getMultiMovieController();
		// qtControl.setStartTime((int)
		// hsEventsManager.getFirstEvent().getStartTime());
		// qtControl.setEndTime((int)
		// hsEventsManager.getLastEvent().getEndTime());
		selected = true;
		// mToolBar.setEnable(true);
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		// mToolBar.setEnable(false);
		manager.unselectEntity();
		// if (initialHold != null)
		// initialHold.unselect();
		//
		// if (finalHold != null)
		// finalHold.unselect();

		return selected;
	}

	// public boolean adjustEntity(int x1, int x2) {
	// return manager.adjustEntity(x1, x2);
	// }

	@Override
	public Event getSelectedEvent() {
		// if (initialHold != null && initialHold.isSelected())
		// return initialHold;
		//
		// if (finalHold != null && finalHold.isSelected())
		// return finalHold;

		return manager.getSelectedEvent();
	}

	@Override
	public void reset() {
		manager.reset();
	}

	@Override
	public boolean delete() {
		// this.initialHold = null;
		// this.finalHold = null;
		manager.clear();
		// Field field =
		// SS3Singleton.getManager().getLoadedSegment().getField(fieldID);
		// field.deleteEmptyEntities();
		return true;
	}

    @Override
	public Event getEventAt(int x) {
		// if (initialHold != null && initialHold.contains(x))
		// return initialHold;
		//
		// if (finalHold != null && finalHold.contains(x))
		// return finalHold;

		return manager.getEventAt(x);
	}

	@Override
	public ArrayList getEventsToAdd() {
		ArrayList list = new ArrayList();
		list.add("Add HandShape");
		return list;
	}

	@Override
	public String getToolTipText(int x) {
		HandShapeEvent event = (HandShapeEvent) manager.getEventAt(x);

		if (event == null)
			return "";

		StringBuilder toolTipText = new StringBuilder();
		toolTipText.append(event.getText());

		if (event.equals(getFirstEvent())) {
			toolTipText.append(", starts at ");
		} else if (event.equals(getLastEvent())) {
			toolTipText.append(", ends at ");
		} else {
			toolTipText.append(", at ");
		}

		toolTipText.append(event.getStartTimeInfo().getMovieTime());

		return toolTipText.toString();
	}

	@Override
	public boolean selectEvent(int x) {
		Event event = manager.getEventAt(x);
        return event != null && event.select();
    }

	public ArrayList<HandShapeSubEntity> getSubEntities() {
		return manager.getSubEntities();
	}

	@Override
	public ArrayList getEvents() {
		ArrayList events = new ArrayList();
		// if (getInitialHold() != null)
		// events.add(getInitialHold());
		events.addAll(manager.getEvents());
		// if (getFinalHold() != null)
		// events.add(getFinalHold());
		return events;
	}

	public ArrayList<HandShapeEvent> getHandShapeEvents() {
		return manager.getEvents();
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
        HandshapeEntityPainter.drawHandShapeEntity(this, g2d, Y, getField().getFieldID());
	}

	public boolean isMarkedHand() {
		return markedHand;
	}
}
