package edu.bu.signstream.grepresentation.fields.handShapeField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class HandShapeSubEntity extends EventsEntity {

	// collection of hand shapes
	private HandShapesManager manager = new HandShapesManager(segmentPanel);
	//private Color lineColor = Color.GRAY;

	public HandShapeSubEntity(SignStreamSegmentPanel segmentPanel, Field field) {
		super(segmentPanel, field);
	}

	@Override
	public SS3Entity getSS3Entity() {
		SS3Entity entity = new SS3Entity();
        ArrayList<HandShapeEvent> events = manager.getHandShapesCollection();

		for (HandShapeEvent event : events) {
			if (event != null) {
				SS3Item item = new SS3Item();
				item.setId(event.getID());
				item.setStartTimeInfo(event.getStartTimeInfo().cloneTimeInfo());
				item.setEndTimeInfo(event.getEndTimeInfo().cloneTimeInfo());
				item.setText(event.getText());
				entity.addItem(item);
			}
		}
		return entity;
	}

	public HandShapesManager getHandShapesManager() {
		return manager;
	}

	@Override
	public boolean isSelected() {
		return false;
	}

	@Override
	public boolean isOverlapping(int coordx1, int coordx2) {
		int startCoord = getStartTimeCoordinate();
		int endCoord = getEndTimeCoordinate();

		return (coordx1 <= startCoord && coordx2 >= startCoord)
			|| (coordx1 <= endCoord && coordx2 >= endCoord);
	}

	@Override
	public int getStartTimeCoordinate() {
        ArrayList<HandShapeEvent> events = manager.getHandShapesCollection();

        if(events.size() < 1)
            return 0;

        int startTimeCoord = events.get(0).getStartTimeCoordinate();
		
		for (HandShapeEvent event : events) {
			startTimeCoord =
				startTimeCoord < event.getStartTimeCoordinate()
					? startTimeCoord
					: event.getStartTimeCoordinate();
		}
		return startTimeCoord;
	}

	@Override
	public Event getFirstEvent() {
		return manager.getFirstEvent();
	}

	@Override
	public Event getLastEvent() {
		return manager.getLastEvent();
	}

	@Override
	public int getEndTimeCoordinate() {
        ArrayList<HandShapeEvent> events = manager.getHandShapesCollection();

		int endTimeCoord = 0;
		for (HandShapeEvent event : events) {
			endTimeCoord =
				endTimeCoord > event.getStartTimeCoordinate()
					? endTimeCoord
					: event.getStartTimeCoordinate();
		}
		return endTimeCoord;
	}

	public void addHandShape(SS3Item item) {
		manager.addHandShape(item);
	}

	public void addHandShape(HandShapeEvent event) {
		manager.addHandShape(event);
	}

	public void addHandShape(TrackValue value) {
		manager.addHandShape(value);
	}

	public void setStartTime(int startTime) {
		Event event = manager.getFirstEvent();
		event.getStartTimeInfo().setMovieTime(startTime);
		event.getEndTimeInfo().setMovieTime(startTime);
	}

	public void setEndTime(int endTime) {
		Event event = manager.getLastEvent();
		event.getEndTimeInfo().setMovieTime(endTime);
		event.getStartTimeInfo().setMovieTime(endTime);
	}

	public int getEndTime() {
		return manager.getLastEvent().getEndTimeInfo().getMovieTime();
	}

	public int getStartTime() {
		return manager.getFirstEvent().getStartTimeInfo().getMovieTime();
	}

	@Override
	public boolean selectEntityIfEventSelected() {
		if (manager.getSelectedEvent() != null)
			select();
		return selected;
	}

	@Override
	public boolean selectEventIfEntitySelected() {
        return selected && manager.getFirstEvent().select();
    }

	@Override
	public boolean select() {
		//manager.highlightEntity(Palette.HIGHLIGHTED_ENTITY_COLOR);

		//QTMultiMovieController qtControl =
		// SSSingleton.getMultiMovieController();
		//qtControl.setStartTime((int) manager.getFirstEvent().getStartTime());
		//qtControl.setEndTime((int) manager.getLastEvent().getEndTime());
		selected = true;
     // segmentPanel.getToolBarsPanel().getMediaToolBar().enable();
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		//mToolBar.setEnable(false);
		manager.unselectEntity();
		return selected;
	}

	@Override
	public boolean adjustEntity(int x1, int x2) {
		return manager.adjustEntity(x1, x2);
	}

	@Override
	public Event getSelectedEvent() {
		return manager.getSelectedEvent();
	}

	@Override
	public void reset() {
		manager.resetAll();
	}

	@Override
	public boolean delete() {
		manager.deleteEntity();
		return true;
	}

	@Override
	public Event getEventAt(int x) {
		return manager.getEventAt(x);
	}

	@Override
	public ArrayList getEventsToAdd() {
		ArrayList list = new ArrayList();
		list.add("Add HandShape");
		return list;
	}

	@Override
	public String getToolTipText(int x) {
		HandShapeEvent event = (HandShapeEvent) manager.getEventAt(x);

		if (event == null)
			return "";

		StringBuilder toolTipText = new StringBuilder();
		toolTipText.append(event.getText());

		if (event.equals(getFirstEvent())) {
			toolTipText.append(", starts at ");
		} else if (event.equals(getLastEvent())) {
			toolTipText.append(", ends at ");
		} else {
			toolTipText.append(", at ");
		}

		toolTipText.append(event.getStartTimeInfo().getMovieTime());

		return toolTipText.toString();
	}

	@Override
	public boolean selectEvent(int x) {
		Event event = manager.getEventAt(x);
		if (event != null)
			return event.select();
		return false;
	}

	public ArrayList<HandShapeEvent> getEvents() {
		return manager.getHandShapesCollection();
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
	}


   @Override
public TimeInfo getStartTimeInfo() {  	   
      return manager.getFirstEvent().getStartTimeInfo();
   }

   @Override
public TimeInfo getEndTimeInfo() {
      return manager.getLastEvent().getEndTimeInfo();
   }

   @Override
public void setStartTimeInfo(TimeInfo startTimeInfo) {
      manager.getFirstEvent().setStartTimeInfo(startTimeInfo);
      
   }

   @Override
public void setEndTimeInfo(TimeInfo endTimeInfo) {
      manager.getLastEvent().setEndTimeInfo(endTimeInfo);
   }
}
