package edu.bu.signstream.grepresentation.fields.locationField;

import java.util.*;
import java.awt.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;

public class LocationEntityCollection {
   private ArrayList entities = new ArrayList();

   public LocationEntityCollection(ArrayList entities) {
      this.entities = entities;
   }

   public EventsEntity getEntity(String entityID) {
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         if (entity.getID() == entityID)
            return entity;
      }
      return null;
   }

   public EventsEntity getEntityWithRefID(String refEntityId) {
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         if (refEntityId.equalsIgnoreCase(entity.getRefEntityId()))
            return entity;
      }
      return null;
   }

   public int size() {
      return entities.size();
   }

   public void deleteEntity(EventsEntity entity) {
      entities.remove(entity);
   }

   public ArrayList getEntities() {
      return entities;
   }

   // removes deleted entities from collection
   public void clean() {
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         if (entity.getEvents().size() == 0)
            entities.remove(i);
      }
   }

   public int getStartTime() {
      int startTime = 0;
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         startTime =
            startTime < entity.getStartTime()
               ? startTime
               : entity.getStartTime();
      }
      return startTime;
   }

   public int getEndTime() {
      int endTime = 0;
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         endTime =
            endTime > entity.getEndTime() ? endTime : entity.getEndTime();
      }
      return endTime;
   }

   public int getStartTimeCoordinate() {
      int startTimeCoord = 0;
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         startTimeCoord =
            startTimeCoord < entity.getStartTimeCoordinate()
               ? startTimeCoord
               : entity.getStartTimeCoordinate();
      }
      return startTimeCoord;
   }

   public int getEndTimeCoordinate() {
      int endTimeCoord = 0;
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         endTimeCoord =
            endTimeCoord > entity.getEndTimeCoordinate()
               ? endTimeCoord
               : entity.getEndTimeCoordinate();
      }
      return endTimeCoord;
   }

   public void unselect() {
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         entity.unselect();
      }
   }

   public void reset() {
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         entity.reset();
      }
   }

   public void addEntity(LocationEntity entity) {
      this.entities.add(entity);
   }

   public void selectEntity(int x) {
      EventsEntity entity = getEntityAt(x);
      if (entity != null)
         entity.select();
      //		for (int i = 0; i < entities.size(); i++) {
      //			ChainedEventsEntity entity = (ChainedEventsEntity) entities.get(i);
      //			if (entity.contains(x))
      //				entity.select();
      //		}
   }

   public LocationEntity getEntityAt(int x) {
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);
         if (entity.contains(x))
            return entity;
      }
      return null;
   }

   public Event getSelectedEvent() {
      Event event = null;
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);

         if ((event = entity.getSelectedEvent()) != null)
            return event;
      }
      return event;
   }

   public LocationEntity getSelectedEntity() {
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);

         if (entity.getSelectedEvent() != null)
            return entity;
      }
      return null;
   }

   public LocationEvent getEventAt(int x) {
      for (int i = 0; i < entities.size(); i++) {
         LocationEntity entity = (LocationEntity) entities.get(i);

         if (entity.contains(x))
            return (LocationEvent) entity.getEventAt(x);
      }
      return null;
   }

   public ArrayList getEntitiesInInterval(int x1, int x2) {
      ArrayList list = new ArrayList();

      for (int j = 0; j < entities.size(); j++) {
         LocationEntity entity = (LocationEntity) entities.get(j);
         if ((entity.isEntityInsideElement(x1, x2))
            || (entity.contains(x1) || entity.contains(x2))) {
            list.add(entity);
         }
      }

      return list;
   }

   public ArrayList getEntitiesInside(int X1, int X2) {
      ArrayList list = new ArrayList();

      for (int j = 0; j < entities.size(); j++) {
         LocationEntity entity = (LocationEntity) entities.get(j);
         if (entity.isOverlapping(X1, X2))
            list.add(entity);
      }

      return list;
   }

   // ---------------------------------------------------------- //
   public boolean isOverlappingNotSelectedEvent(int coordx1, int coordx2) {
      for (int j = 0; j < entities.size(); j++) {
         LocationEntity entity = (LocationEntity) entities.get(j);
         if (entity.getSelectedEvent() == null) {
            if (entity.isOverlapping(coordx1, coordx2))
               return true;
         }
      }
      return false;
   }
   // ----------------------------------------------------------- //

   public void paintEntities(Graphics2D g2d, double Y, Font font) {
      for (int j = 0; j < entities.size(); j++) {
         LocationEntity entity = (LocationEntity) entities.get(j);
         entity.paint(g2d, Y, font);
      }
   }
}
