package edu.bu.signstream.grepresentation.fields.locationField;

import java.awt.Color;

import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class LocationEvent implements Event {
	// <ITEM VID="1.4#2.1#3.3" S="800" E=""/>
	protected String id;

	protected String text;

	protected String ttText;

	protected TimeInfo startTimeInfo, endTimeInfo;

	private String noteID;

	private Color color = null;

	private boolean highlighted = false;

	protected boolean selected = false;

	// protected MediaToolBar1 mToolBar = SSSingleton.getMediaToolBar();

	private SignStreamSegmentPanel segmentPanel = null;

	public LocationEvent(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

	@Override
	public void setColor(Color color) {
		this.color = color;
	}

	@Override
	public void highlight() {
		this.highlighted = true;
	}

	@Override
	public void unhighlight() {
		this.highlighted = false;
	}

	@Override
	public boolean isHighlighted() {
		return highlighted;
	}

	@Override
	public Color getColor() {
		return color;
	}

	@Override
	public TimeInfo getStartTimeInfo() {
		return startTimeInfo;
	}

	@Override
	public TimeInfo getEndTimeInfo() {
		return this.endTimeInfo;
	}

	@Override
	public void setStartTimeInfo(TimeInfo startTimeInfo) {
		this.startTimeInfo = startTimeInfo;
	}

	@Override
	public void setEndTimeInfo(TimeInfo endTimeInfo) {
		this.startTimeInfo = endTimeInfo;
	}

	// Event interface
	@Override
	public int getStartTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(getStartTimeInfo().getMovieTime());
	}

	// Event interface
	@Override
	public int getEndTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(getEndTimeInfo().getMovieTime());
	}

	// Event interface
	@Override
	public void setStartTimeCoordinate(int coord) {
		int startMovieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(coord);
      getStartTimeInfo().setMovieTime(startMovieTime);
	}

	// Event interface
	@Override
	public void setEndTimeCoordinate(int coord) {
		int endMovieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(coord); 
      getEndTimeInfo().setMovieTime(endMovieTime);
	}

	public boolean contains(int x) {
		return (getStartTimeCoordinate() - 20 <= x && x <= getEndTimeCoordinate() + 20);
	}

	@Override
	public String getToolTipText() {
		return ttText;
	}

	public void setToolTipText(String ttText) {
		this.ttText = ttText;
	}

	@Override
	public boolean select() {
		// highlight(true);
		selected = true;
		// mToolBar.setEnable(true);
		return selected;
	}

	@Override
	public boolean unselect() {
		this.selected = false;
		// highlight(false);
		// mToolBar.setEnable(true);
		return true;
	}

	@Override
	public boolean isSelected() {
		return selected;
	}

	public void reset() {
		unselect();
		selected = false;
		// mToolBar.setEnable(true);
	}

	public void setTrackValue(TrackValue value) {
		if (value != null) {
			this.id = "" + value.getValueId();
			this.text = value.getTextValue();
			this.startTimeInfo = value.getStartTimeInfo();
			this.endTimeInfo = value.getEndTimeInfo();
		}
	}

	public void setSS3Item(SS3Item item) {
		this.id = item.getId();
		this.noteID = item.getNoteID();
		this.text = item.getText();
		this.startTimeInfo = item.getStartTimeInfo();
		this.endTimeInfo = item.getEndTimeInfo();
	}

	public SS3Item getSS3Item() {
		SS3Item item = new SS3Item();
		item.setId(id);
		item.setNoteID(noteID);
		item.setStartTimeInfo(startTimeInfo);
		item.setEndTimeInfo(endTimeInfo);
		item.setText(text);
		return item;
	}

	public String getID() {
		return id;
	}

	public void setID(String id) {
		this.id = id;
	}
}
