package edu.bu.signstream.grepresentation.fields.locationField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class LocationField extends Field {
   private LocationEntityCollection entities = null;

   public LocationField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
      super(segmentPanel, parentField);
      this.track = track;
      createTrackEntity(track);
   }

   public LocationField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
      super(segmentPanel, parentField);
      this.ss3Track = ss3track;
      createTrackEntity(ss3track);
   }
   
   @Override
public void deleteEmptyEntities(){}

   @Override
public boolean isEmptyInterval(int startTime, int endTime) {
      return true;
   }

   @Override
public void addEventsEntity(EventsEntity entity) {
      LocationEntity lEntity = (LocationEntity) entity;
      entities.addEntity(lEntity);
   }

   @Override
public EventsEntity getEntity(String entityID) {
      return entities.getEntity(entityID);
   }

   @Override
public ArrayList getEntities() {
      return entities.getEntities();
   }

   @Override
public EventsEntity getEntityWithRefID(String refEntityId) {
      return entities.getEntityWithRefID(refEntityId);
   }

   @Override
public void highlight(boolean highlight, String refID) {
      if (!highlight) {
         ArrayList ents = entities.getEntities();
         for (int i = 0; i < ents.size(); i++) {
            EventsEntity entity = (EventsEntity) ents.get(i);
            entity.highlight(false);
         }
      } else {
         EventsEntity ent = entities.getEntityWithRefID(refID);

         if (ent != null)
            ent.highlight(true);
      }
   }

   public void createTrackEntity(Track track) {
      ArrayList entity = new ArrayList();
      entities = new LocationEntityCollection(entity);
   }

   private void createTrackEntity(SS3Track track) {
      ArrayList lEntities = new ArrayList();
      ArrayList ent = track.getEntities();
      for (int j = 0; j < ent.size(); j++) {
         SS3Entity entity = (SS3Entity) ent.get(j);
         LocationEventsCollection collection = new LocationEventsCollection();

         ArrayList items = entity.getItemsAsList();
         for (int i = 0; i < items.size(); i++) {
            SS3Item item = (SS3Item) items.get(i);
            LocationEvent event = new LocationEvent(segmentPanel);
            event.setSS3Item(item);

            collection.addEvent(event);
         }

         LocationEntity lEntity = new LocationEntity(collection, track.getId(), segmentPanel, this);

         lEntity.setID(entity.getID());
         lEntity.setRefEntityId(entity.getRefEntityId());
         lEntity.setParentEntity((PresentationEventsEntity) getParentField().getEntity(entity.getParentEntityID()));
        // lEntity.setReference(entity.getReference());

         lEntities.add(lEntity);
      }
      entities = new LocationEntityCollection(lEntities);
   }

   @Override
public EventsEntity getEntityAt(int entityAt) {
      ArrayList list = entities.getEntities();
      for (int i = 0; i < list.size(); i++) {
         LocationEntity entity = (LocationEntity) list.get(i);
         int startTimeCoord = entity.getFirstEvent().getStartTimeCoordinate();
         int endTimeCoord = entity.getLastEvent().getStartTimeCoordinate();

         if (startTimeCoord <= entityAt && endTimeCoord >= entityAt)
            return entity;
      }
      return null;
   }

   @Override
public Event getEventAt(int x) {
      return entities.getEventAt(x);
   }

   @Override
public void deleteEntity(EventsEntity entity) {}

   @Override
public int getFieldSize() {
      return 0;
   }

   @Override
public Event selectEvent(int x) {
      Event event = getEventAt(x);
      if (event != null)
         event.select();
      return event;
   }

   @Override
public boolean contains(int y) {
      return y > Y - 14 && y < Y + 5;
   }

   @Override
public int getStartTime() {
      return ss3Track.getStartTime();
   }

   @Override
public int getEndTime() {
      return ss3Track.getEndTime();
   }

   @Override
public int getStartTimeCoordinate() {
      return 0;
   }

   @Override
public int getEndTimeCoordinate() {
      return 0;
   }

   @Override
public void unselect() {
      entities.unselect();
   }

   @Override
public Event getSelectedEvent() {
      return null;
   }

   @Override
public EventsEntity getSelectedEntity() {
      return entities.getSelectedEntity();
   }

   public boolean setSelectedEventStartTime(int time) {
      return false;
   }

   public boolean setSelectedEventEndTime(int time) {
      return false;
   }

   @Override
public void paint(Graphics2D g2d, double Y, Font font) {
      super.paint(g2d, Y, font);
      entities.paintEntities(g2d, Y, font);
   }
}
