package edu.bu.signstream.grepresentation.fields.locationField;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.ArrayList;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SpringLayout;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.ValueCollection;


public class NewLocationPanel extends JPanel {
   private ArrayList locValues = new ArrayList();
   private ArrayList proxValues = new ArrayList();
   private ArrayList latValues = new ArrayList();

   private JList locationList = null;
   private JList proximityList = null;
   private JList lateralizationList = null;

   public NewLocationPanel(SS3CodingScheme scheme) {
      createUI(scheme);
   }

   private void createUI(SS3CodingScheme scheme) {
      JPanel lPanel = new JPanel();
      JPanel rPanel = new JPanel();

      // body location
      ValueCollection locValCol =
         scheme.getValueCollection(Constants.VC_BODY_LOCATION);
      locValues = locValCol.getFieldValues();
      DefaultListModel locModel = new DefaultListModel();
      for (int i = 0; i < locValues.size(); i++) {
         SS3FieldValue value = (SS3FieldValue) locValues.get(i);
         locModel.addElement(value.getName());
      }

      locationList = new JList(locModel);
      locationList.setSelectionMode(
         ListSelectionModel.SINGLE_INTERVAL_SELECTION);
      locationList.setLayoutOrientation(JList.VERTICAL);
      locationList.setVisibleRowCount(-1);

      JScrollPane locationScroller = new JScrollPane(locationList);
      locationScroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
      locationScroller.setPreferredSize(new Dimension(150, 80));

      // proximity
      ValueCollection proxValCol =
         scheme.getValueCollection(Constants.VC_PROXIMITY);
      proxValues = proxValCol.getFieldValues();
      DefaultListModel proxModel = new DefaultListModel();
      for (int i = 0; i < proxValues.size(); i++) {
         SS3FieldValue value = (SS3FieldValue) proxValues.get(i);
         proxModel.addElement(value.getName());
      }

      proximityList = new JList(proxModel);
      proximityList.setSelectionMode(
         ListSelectionModel.SINGLE_INTERVAL_SELECTION);
      proximityList.setLayoutOrientation(JList.VERTICAL);
      proximityList.setVisibleRowCount(-1);

      JScrollPane proximityScroller = new JScrollPane(proximityList);
      proximityScroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
      proximityScroller.setPreferredSize(new Dimension(150, 80));

      // lateralization
      ValueCollection latValCol =
         scheme.getValueCollection(Constants.VC_LATERALIZATION);
      latValues = latValCol.getFieldValues();
      DefaultListModel latModel = new DefaultListModel();
      for (int i = 0; i < latValues.size(); i++) {
         SS3FieldValue value = (SS3FieldValue) latValues.get(i);
         latModel.addElement(value.getName());
      }

      lateralizationList = new JList(latModel);
      lateralizationList.setSelectionMode(
         ListSelectionModel.SINGLE_INTERVAL_SELECTION);
      lateralizationList.setLayoutOrientation(JList.VERTICAL);
      lateralizationList.setVisibleRowCount(-1);

      JScrollPane lateralizationScroller = new JScrollPane(lateralizationList);
      lateralizationScroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
      lateralizationScroller.setPreferredSize(new Dimension(150, 80));

      lPanel.setLayout(new GridLayout(0, 1, 5, 5));
      JPanel lctPanel = new JPanel();
      lctPanel.setLayout(new BorderLayout());
      lctPanel.add(new JLabel("Body Location"), BorderLayout.NORTH);
      lctPanel.add(locationScroller, BorderLayout.CENTER);
      lPanel.add(lctPanel);

      JPanel prxPanel = new JPanel();
      prxPanel.setLayout(new BorderLayout());
      prxPanel.add(new JLabel("Proximity"), BorderLayout.NORTH);
      prxPanel.add(proximityScroller, BorderLayout.CENTER);
      lPanel.add(prxPanel);

      JPanel ltrPanel = new JPanel();
      ltrPanel.setLayout(new BorderLayout());
      ltrPanel.add(new JLabel("Proximity"), BorderLayout.NORTH);
      ltrPanel.add(lateralizationScroller, BorderLayout.CENTER);
      lPanel.add(ltrPanel);

      JPanel sPanel = new JPanel();
      sPanel.setLayout(new SpringLayout());
      sPanel.add(new JLabel("Selection: "));
      JTextField selTxt = new JTextField(26);
      sPanel.add(selTxt);
      sPanel.add(new JButton("Info"));

      rPanel.setLayout(new BorderLayout());
      rPanel.add(sPanel, BorderLayout.NORTH);
      rPanel.add(
         new JLabel("front, side, up images will be here"),
         BorderLayout.CENTER);

      setLayout(new BorderLayout(10, 10));
      add(sPanel, BorderLayout.NORTH);
      add(lPanel, BorderLayout.WEST);
      add(rPanel, BorderLayout.CENTER);
   }

   public String getSelecteedIDs() {
      StringBuffer bf = new StringBuffer();
      int lctIndex = locationList.getSelectedIndex();
      int prxIndex = proximityList.getSelectedIndex();
      int ltrIndex = lateralizationList.getSelectedIndex();

      if (lctIndex > -1) {
         SS3FieldValue locValue = (SS3FieldValue) locValues.get(lctIndex);
         bf.append(Constants.VC_BODY_LOCATION);
         bf.append(".");
         bf.append(locValue.getId());
      }

      if (prxIndex > -1) {
         SS3FieldValue prxValue = (SS3FieldValue) proxValues.get(prxIndex);

         if (bf.length() > 0)
            bf.append("#");

         bf.append(Constants.VC_PROXIMITY);
         bf.append(".");
         bf.append(prxValue.getId());
      }

      if (prxIndex > -1) {
         SS3FieldValue ltrValue = (SS3FieldValue) latValues.get(ltrIndex);

         if (bf.length() > 0)
            bf.append("#");

         bf.append(Constants.VC_LATERALIZATION);
         bf.append(".");
         bf.append(ltrValue.getId());
      }

      return bf.toString();
   }
}
