package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.CollectionManager;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.ConfirmationPanel;

/**
 * Manages collection of ChainedEventsEntity objects.
 */
public class ChainedEventsCollectionManager extends CollectionManager {

	/**
	 * collection of ChainedEventsEntity objects to manage
	 */
	private ChainedEventsCollection collection = null;
	private SignStreamSegmentPanel segmentPanel = null;
	private CoordinateTimeConverter convertor = null;

	private int minFrameDurationToAdd = 0;

	/**
	 * 
	 * @param collection
	 *            must not be null
	 */
	public ChainedEventsCollectionManager(ChainedEventsCollection collection,
			SignStreamSegmentPanel segmentPanel) {
		this.collection = collection;
		this.segmentPanel = segmentPanel;

		this.convertor = segmentPanel.getZoomer().getCoordinateTimeConvertor();
		this.minFrameDurationToAdd = convertor.getMovieFrameDuration();
		// this.minFrameDurationToAdd = 2*conv.getMovieFrameDuration();
	}
	
	public void selectChainedEvent(ChainedEvent event) {
		if (event != null)
			event.select();
	}

	public void unselectChainedEvent(ChainedEvent event) {
		event.unselect();
	}

	@Override
	public void unselectEntity() {
		if (collection.getOnsetEvent() != null)
			collection.getOnsetEvent().unselect();
		if (collection.getInitialHoldEvent() != null)
			collection.getInitialHoldEvent().unselect();
		if (collection.getValueEvent() != null)
			collection.getValueEvent().unselect();
		if (collection.getTextValueEvent() != null)
			collection.getTextValueEvent().unselect();
		if (collection.getFinalHoldEvent() != null)
			collection.getFinalHoldEvent().unselect();
		if (collection.getOffsetEvent() != null)
			collection.getOffsetEvent().unselect();
	}

	@Override
	public Event getSelectedEvent() {
		if (collection.getOnsetEvent() != null
				&& collection.getOnsetEvent().isSelected())
			return collection.getOnsetEvent();
		if (collection.getInitialHoldEvent() != null
				&& collection.getInitialHoldEvent().isSelected())
			return collection.getInitialHoldEvent();
		if (collection.getValueEvent() != null
				&& collection.getValueEvent().isSelected())
			return collection.getValueEvent();
		if (collection.getTextValueEvent() != null
				&& collection.getTextValueEvent().isSelected())
			return collection.getTextValueEvent();
		if (collection.getFinalHoldEvent() != null
				&& collection.getFinalHoldEvent().isSelected())
			return collection.getFinalHoldEvent();
		if (collection.getOffsetEvent() != null
				&& collection.getOffsetEvent().isSelected())
			return collection.getOffsetEvent();
		return null; // throw Exception
	}

//	public void setStartTimeOrRemoveEvent(int time) {
//		Event firstEvent = getFirstEvent();
//		int event1StartTime = firstEvent.getStartTimeInfo().getMovieTime();
//
//		if (time < event1StartTime) {
//			firstEvent.getStartTimeInfo().setMovieTime(time);
//		} else {
//			if (collection.getOnsetEvent() != null) {
//				Event onset = collection.getOnsetEvent();
//				if (onset.getEndTimeInfo().getMovieTime() < time) {
//					collection.setOnsetEvent(null);
//					setStartTimeOrRemoveEvent(time);
//				} else if (onset.getEndTimeInfo().getMovieTime() == time) {
//					collection.setOnsetEvent(null);
//				} else {
//					collection.getOnsetEvent().getStartTimeInfo()
//							.setMovieTime(time);
//				}
//			} else if (collection.getInitialHoldEvent() != null) {
//				Event hold = collection.getInitialHoldEvent();
//				if (hold.getEndTimeInfo().getMovieTime() < time) {
//					collection.setInitialHoldEvent(null);
//					setStartTimeOrRemoveEvent(time);
//				} else if (hold.getEndTimeInfo().getMovieTime() == time) {
//					collection.setInitialHoldEvent(null);
//				} else {
//					collection.getInitialHoldEvent().getStartTimeInfo()
//							.setMovieTime(time);
//				}
//			} else {
//				Event value = collection.getValueEvent();
//				if (value.getEndTimeInfo().getMovieTime() <= time) {
//					;
//				}
//			}
//		}
//	}

//	public void setEndTimeOrRemoveEvent(int time) {
//		Event lastEvent = getLastEvent();
//		int event1EndTime = lastEvent.getStartTimeInfo().getMovieTime();
//
//		if (time > event1EndTime) {
//			lastEvent.getEndTimeInfo().setMovieTime(time);
//		} else {
//			if (collection.getOffsetEvent() != null) {
//				Event offset = collection.getOffsetEvent();
//				if (offset.getStartTimeInfo().getMovieTime() > time) {
//					collection.setOffsetEvent(null);
//					setEndTimeOrRemoveEvent(time);
//				} else if (offset.getStartTimeInfo().getMovieTime() == time) {
//					collection.setOffsetEvent(null);
//				} else {
//					collection.getOffsetEvent().getEndTimeInfo()
//							.setMovieTime(time);
//				}
//			} else if (collection.getFinalHoldEvent() != null) {
//				Event hold = collection.getFinalHoldEvent();
//				if (hold.getStartTimeInfo().getMovieTime() > time) {
//					collection.setFinalHoldEvent(null);
//					setEndTimeOrRemoveEvent(time);
//				} else if (hold.getStartTimeInfo().getMovieTime() == time) {
//					collection.setFinalHoldEvent(null);
//				} else {
//					collection.getFinalHoldEvent().getEndTimeInfo()
//							.setMovieTime(time);
//				}
//			} else {
//				Event value = collection.getValueEvent();
//				if (value.getEndTimeInfo().getMovieTime() <= time) {
//					;
//				}
//			}
//		}
//	}

	// public Event getEventAt(int x) {
	@Override
	public Event getEventAt(int x) {
		deleteUnattachedEvents();

		ChainedEvent event = null;
		if ((event = collection.getOnsetEvent()) != null && event.contains(x))
			return event;
		if ((event = collection.getInitialHoldEvent()) != null
				&& event.contains(x))
			return event;
		if ((event = collection.getValueEvent()) != null && event.contains(x))
			return event;
		if ((event = collection.getTextValueEvent()) != null
				&& event.contains(x))
			return event;
		if ((event = collection.getFinalHoldEvent()) != null
				&& event.contains(x))
			return event;
		if ((event = collection.getOffsetEvent()) != null && event.contains(x))
			return event;
		return null;
	}

	/**
	 * @return left existing ChainedEvent of the ChainedEventsEntity
	 */
	@Override
	public Event getFirstEvent() {

		deleteUnattachedEvents();

		ChainedEvent event = null;

		if ((event = collection.getOnsetEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getInitialHoldEvent()) != null
				&& event.isExist())
			return event;

		if ((event = collection.getValueEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getTextValueEvent()) != null && event.isExist())
			return event;
		return null;
	}

	private void deleteUnattachedEvents() {
		ChainedEvent txtEvent = collection.getValueOrTextValueEvent();

		if (txtEvent == null)
			return;

		int txtStartMovieTime = txtEvent.getStartTimeInfo().getMovieTime();
		int txtEndMovieTime = txtEvent.getEndTimeInfo().getMovieTime();

		if (collection.getInitialHoldEvent() != null) {
			int ihEndMovieTime = collection.getInitialHoldEvent()
					.getEndTimeInfo().getMovieTime();
			if (ihEndMovieTime != txtStartMovieTime) {
				collection.setInitialHoldEvent(null);
				collection.setOnsetEvent(null);
			} else if (collection.getOnsetEvent() != null) {
				int ihStartMovieTime = collection.getInitialHoldEvent()
						.getStartTimeInfo().getMovieTime();
				int onsetEndMovieTime = collection.getOnsetEvent()
						.getEndTimeInfo().getMovieTime();
				if (ihStartMovieTime != onsetEndMovieTime) {
					collection.setOnsetEvent(null);
				}
			}
		} else if (collection.getOnsetEvent() != null) {
			int onsetEndMovieTime = collection.getOnsetEvent().getEndTimeInfo()
					.getMovieTime();
			if (onsetEndMovieTime != txtStartMovieTime) {
				collection.setOnsetEvent(null);
			}
		}

		if (collection.getFinalHoldEvent() != null) {
			int fhStartMovieTime = collection.getFinalHoldEvent()
					.getStartTimeInfo().getMovieTime();
			if (txtEndMovieTime != fhStartMovieTime) {
				collection.setFinalHoldEvent(null);
				collection.setOffsetEvent(null);
			} else {
				int offsetStartMovieTime = collection.getFinalHoldEvent()
						.getStartTimeInfo().getMovieTime();
				if (txtEndMovieTime != offsetStartMovieTime) {
					collection.setOffsetEvent(null);
				}
			}
		} else if (collection.getOffsetEvent() != null) {
			int offsetStartMovieTime = collection.getOffsetEvent()
					.getStartTimeInfo().getMovieTime();
			if (txtEndMovieTime != offsetStartMovieTime) {
				collection.setOffsetEvent(null);
			}
		}
	}

	/**
	 * @return right existing ChainedEvent of the ChainedEventsEntity
	 */
	@Override
	public Event getLastEvent() {
		deleteUnattachedEvents();
		ChainedEvent event = null;

		// clean entity - sometimes it may have wrong HOLD transfered from
		// Legacy file

		if ((event = collection.getOffsetEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getFinalHoldEvent()) != null && event.isExist()) {
			if (collection.getValueOrTextValueEvent().getEndTimeInfo()
					.getMovieTime() != collection.getFinalHoldEvent()
					.getStartTimeInfo().getMovieTime()) {
				collection.setFinalHoldEvent(null);
			} else {
				return event;
			}
		}

		if ((event = collection.getValueEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getTextValueEvent()) != null && event.isExist())
			return event;

		return null;
	}

	/**
	 * delete entire entity
	 */
	public boolean deleteEntity() {
		collection.setOnsetEvent(null);
		collection.setInitialHoldEvent(null);
		collection.setValueEvent(null);
		collection.setTextValueEvent(null);
		collection.setFinalHoldEvent(null);
		collection.setOffsetEvent(null);
		return true;
	}

	/**
	 * @param event
	 *            ChainedEvent to be deleted
	 */
	public void deleteChainedEvent(ChainedEvent event) {
		// if(event != null)
		// event.delete();
		event = null;
	}

	public ChainedEventsCollection getChainedEventsCollection() {
		return collection;
	}

	@Override
	public void resetAll() {
		ChainedEvent event;
		if ((event = collection.getOnsetEvent()) != null)
			event.reset();
		if ((event = collection.getInitialHoldEvent()) != null)
			event.reset();
		if ((event = collection.getValueEvent()) != null)
			event.reset();
		if ((event = collection.getTextValueEvent()) != null)
			event.reset();
		if ((event = collection.getFinalHoldEvent()) != null)
			event.reset();
		if ((event = collection.getOffsetEvent()) != null)
			event.reset();
	}

	public boolean isAnyEventSelected() {
		return getSelectedEvent() != null ? true : false;
	}

	@Override
	public boolean adjustEntity(int startTime, int endTime) {
		ChainedEvent onset = collection.getOnsetEvent();
		ChainedEvent offset = collection.getOffsetEvent();
		ChainedEvent finalHold = collection.getFinalHoldEvent();
		ChainedEvent value = collection.getValueOrTextValueEvent();

		if (value == null)
			return false;

		if (startTime < value.getStartTimeInfo().getMovieTime()
				&& endTime > value.getEndTimeInfo().getMovieTime()) {
			deleteEntity();
			return true;
		} else if (startTime < value.getStartTimeInfo().getMovieTime()
				&& endTime < value.getEndTimeInfo().getMovieTime()) {
			deleteOnsetEvent();
			deleteInitialHoldEvent();
			collection.getValueEvent().getStartTimeInfo().setMovieTime(endTime);
			return true;
		} else if (endTime < value.getEndTimeInfo().getMovieTime()
				&& endTime > value.getStartTimeInfo().getMovieTime()) {
			deleteFinalHoldEvent();
			deleteOffsetEvent();
			collection.getValueEvent().getEndTimeInfo().setMovieTime(endTime);
			return true;
		} else if (offset != null
				&& startTime > offset.getStartTimeInfo().getMovieTime()) {
			deleteOffsetEvent();
			collection.getInitialHoldEvent().getStartTimeInfo()
					.setMovieTime(startTime);
			return true;
		} else if (onset != null
				&& startTime > onset.getStartTimeInfo().getMovieTime()) {
			collection.getOnsetEvent().getStartTimeInfo()
					.setMovieTime(startTime);
			return true;
		} else if (finalHold != null
				&& endTime < finalHold.getEndTimeInfo().getMovieTime()) {
			deleteFinalHoldEvent();
			collection.getFinalHoldEvent().getStartTimeInfo()
					.setMovieTime(endTime);
			return true;
		} else if (offset != null
				&& endTime < onset.getEndTimeInfo().getMovieTime()) {
			collection.getOffsetEvent().getStartTimeInfo()
					.setMovieTime(endTime);
			return true;
		}
		return false;
	}

	public boolean deleteOnsetEvent() {
		String message = "<HTML>This action will delete ONSET value. <BR> Do you want to proceed? </HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				collection.setOnsetEvent(null);
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteOnsetEventNoWarning() {
		collection.setOnsetEvent(null);
		return true;
	}

	public boolean deleteOffsetEventNoWarning() {
		collection.setOffsetEvent(null);
		return true;
	}

	public boolean deleteFinalHoldEventNoWarning() {
		collection.setFinalHoldEvent(null);

		if (collection.getOffsetEvent() != null) {
			Event value = collection.getValueOrTextValueEvent();
			collection.getOffsetEvent().getStartTimeInfo()
					.setMovieTime(value.getStartTimeInfo().getMovieTime());
		}
		return true;
	}

	public boolean deleteInitialHoldEventNoWarning() {
		collection.setInitialHoldEvent(null);

		if (collection.getOnsetEvent() != null) {
			Event value = collection.getValueOrTextValueEvent();
			collection.getOnsetEvent().getEndTimeInfo()
					.setMovieTime(value.getEndTimeInfo().getMovieTime());
		}
		return true;
	}

	public boolean deleteInitialHoldEvent() {
		String message = "<HTML>This action will delete INITIAL HOLD value. <BR> Do you want to proceed? </HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				collection
						.getValueOrTextValueEvent()
						.getStartTimeInfo()
						.setMovieTime(
								collection.getInitialHoldEvent()
										.getStartTimeInfo().getMovieTime());
				deleteInitialHoldEventNoWarning();
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteValueEvent() {
		String message = "<HTML>This action will delete selected event. <BR> Do you want to proceed?</HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				deleteEntity();
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteTextValueEvent() {
		String message = "<HTML>This action will delete selected event. <BR> Do you want to proceed?</HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				deleteEntity();
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteFinalHoldEvent() {
		String message = "<HTML>This action will delete FINAL HOLD value. <BR> Do you want to proceed? </HTML>";

		final JDialog dialog = new JDialog(SS3Singleton.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (collection.getOffsetEvent() != null)
					collection
							.getOffsetEvent()
							.getStartTimeInfo()
							.setMovieTime(
									collection.getFinalHoldEvent()
											.getStartTimeInfo().getMovieTime());
				// collection.getFinalHoldEvent().delete();
				collection.setFinalHoldEvent(null);
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteOffsetEvent() {
		String message = "<HTML>This action will delete OFFSET value. <BR> Do you want to proceed?</HTML>";

		final JDialog dialog = new JDialog(SS3Singleton.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				collection.setOffsetEvent(null);
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

//	public boolean setInitialHoldStartMovieTimeNoWarning(int movieTime) {
//		ChainedEvent initialHold = collection.getInitialHoldEvent();
//		ChainedEvent onset = collection.getOnsetEvent();
//
//		if (initialHold == null)
//			return false;
//
//		int maxHoldStartMovieTime = convertor
//				.getPreviousFrameMovieTime(initialHold.getEndTimeInfo()
//						.getMovieTime());
//
//		if (movieTime <= maxHoldStartMovieTime) {
//			if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//
//				if (movieTime >= onsetMinEndMovieTime) {
//					onset.getEndTimeInfo().setMovieTime(movieTime);
//					initialHold.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					onset.getEndTimeInfo().setMovieTime(onsetMinEndMovieTime);
//					initialHold.getStartTimeInfo().setMovieTime(
//							onsetMinEndMovieTime);
//					// collection.setOnsetEvent(null);
//					// initialHold.setStartTimeCoordinate(coord);
//					return true;
//				}
//			} else {
//				initialHold.getStartTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			initialHold.getStartTimeInfo().setMovieTime(maxHoldStartMovieTime);
//			// collection.setInitialHoldEvent(null);
//			if (onset != null)
//				onset.getEndTimeInfo().setMovieTime(
//						collection.getValueOrTextValueEvent()
//								.getStartTimeInfo().getMovieTime());
//			return true;
//		}
//	}
//
//	public boolean setValueStartMovieTimeNoWarning(int movieTime) {
//		ChainedEvent onset = collection.getOnsetEvent();
//		ChainedEvent initialHold = collection.getInitialHoldEvent();
//		ChainedEvent value = collection.getValueOrTextValueEvent();
//
//		if (value == null)
//			return false;
//
//		int valueMaxStartMovieTime = convertor.getPreviousFrameMovieTime(value
//				.getEndTimeInfo().getMovieTime());
//
//		if (movieTime <= valueMaxStartMovieTime) {
//			if (initialHold != null) {
//				int holdMinEndMovieTime = convertor
//						.getNextFrameMovieTime(initialHold.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime >= holdMinEndMovieTime) {
//					initialHold.getEndTimeInfo().setMovieTime(movieTime);
//					value.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					// collection.setInitialHoldEvent(null);
//					// value.setStartTimeCoordinate(coord);
//					initialHold.getEndTimeInfo().setMovieTime(
//							holdMinEndMovieTime);
//					value.getStartTimeInfo().setMovieTime(holdMinEndMovieTime);
//					return true;
//				}
//			} else if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime > onsetMinEndMovieTime) {
//					onset.getEndTimeInfo().setMovieTime(movieTime);
//					value.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					onset.getEndTimeInfo().setMovieTime(onsetMinEndMovieTime);
//					value.getStartTimeInfo().setMovieTime(onsetMinEndMovieTime);
//					// collection.setOnsetEvent(null);
//					// value.setStartTimeCoordinate(coord);
//					return true;
//				}
//			} else {
//				value.getStartTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			return deleteEntity();
//		}
//	}
//
//	public boolean setOnsetStartMovieTimeNoWarning(int movieTime) {
//		ChainedEvent onset = collection.getOnsetEvent();
//
//		if (movieTime > onset.getStartTimeInfo().getMovieTime()) {
//
//			int maxStartMovieTime = convertor.getPreviousFrameMovieTime(onset
//					.getEndTimeInfo().getMovieTime());
//
//			if (movieTime <= maxStartMovieTime) {
//				onset.getStartTimeInfo().setMovieTime(movieTime);
//			} else {
//				onset.getStartTimeInfo().setMovieTime(maxStartMovieTime);
//			}
//		} else {
//			onset.getStartTimeInfo().setMovieTime(movieTime);
//		}
//		return true;
//	}
//
//	public Response checkOnsetStartMovieTimeUpdatePossibility(int movieTime) {
//		Response response = new Response();
//		ChainedEvent onset = collection.getOnsetEvent();
//
//		response.setUpdatePossible(true);
//		response.setMovieTime(movieTime);
//		response.setState(Response.OK);
//
//		if (movieTime > onset.getStartTimeInfo().getMovieTime()) {
//			int maxStartTime = convertor.getPreviousFrameMovieTime(onset
//					.getEndTimeInfo().getMovieTime());
//
//			if (movieTime > maxStartTime) {
//				response.setUpdatePossible(true);
//				response.setState(Response.HOLD_DELETE);
//				response.setErrorMessage("Onset event will be deleted.");
//				response.setMovieTime(maxStartTime);
//			}
//		}
//
//		return response;
//	}
//
//	public Response checkValueStartMovieTimeUpdatePossibility(int movieTime) {
//		ChainedEvent onset = collection.getOnsetEvent();
//		ChainedEvent initialHold = collection.getInitialHoldEvent();
//		ChainedEvent value = collection.getValueOrTextValueEvent();
//
//		Response response = new Response();
//		response.setState(Response.OK);
//		response.setUpdatePossible(true);
//		response.setMovieTime(movieTime);
//
//		int valueMaxStartMovieTime = convertor.getPreviousFrameMovieTime(value
//				.getEndTimeInfo().getMovieTime());
//
//		if (movieTime <= valueMaxStartMovieTime) {
//			if (initialHold != null) {
//				int holdMinEndMovieTime = convertor
//						.getNextFrameMovieTime(initialHold.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime >= holdMinEndMovieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Initial Hold will be deleted");
//					response.setMovieTime(holdMinEndMovieTime);
//				}
//			} else if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime > onsetMinEndMovieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Onset will be deleted");
//					response.setMovieTime(onsetMinEndMovieTime);
//				}
//			} else {
//				;
//			}
//		} else {
//			response.setState(Response.ENTITY_DELETE);
//			response.setErrorMessage("Entity will be deleted");
//			response.setMovieTime(valueMaxStartMovieTime);
//		}
//		return response;
//	}
//
//	public Response checkValueEndMovieTimeUpdatePossibility(int movieTime) {
//		ChainedEvent offset = collection.getOffsetEvent();
//		ChainedEvent finalHold = collection.getFinalHoldEvent();
//		ChainedEvent value = collection.getValueOrTextValueEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int valueMinEndMovieTime = convertor.getNextFrameMovieTime(value
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= valueMinEndMovieTime) {
//			if (finalHold != null) {
//				int maxHoldStartMovieTime = convertor
//						.getPreviousFrameMovieTime(finalHold.endTimeInfo
//								.getMovieTime());
//
//				if (maxHoldStartMovieTime >= movieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Final Hold will be deleted");
//					response.setMovieTime(maxHoldStartMovieTime);
//				}
//
//			} else if (offset != null) {
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Offset will be deleted");
//					response.setMovieTime(maxOffsetStartMovieTime);
//				}
//			} else {
//				;
//			}
//		} else {
//			response.setState(Response.ENTITY_DELETE);
//			response.setErrorMessage("Entity will be deleted");
//			response.setMovieTime(valueMinEndMovieTime);
//		}
//		return response;
//	}
//
//	public boolean setOffsetEndMovieTimeNoWarning(int movieTime) {
//		ChainedEvent offset = collection.getOffsetEvent();
//
//		int offsetMinEndMovieTime = convertor.getNextFrameMovieTime(offset
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= offsetMinEndMovieTime) {
//			offset.getEndTimeInfo().setMovieTime(movieTime);
//			return true;
//		} else {
//			offset.getEndTimeInfo().setMovieTime(offsetMinEndMovieTime);
//			// collection.setOffsetEvent(null);
//			return false;
//		}
//	}

//	public Response checkOffsetEndMovieTimeUpdatePossibility(int movieTime) {
//		ChainedEvent offset = collection.getOffsetEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int offsetMinEndMovieTime = convertor.getNextFrameMovieTime(offset
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime <= offsetMinEndMovieTime) {
//			response.setErrorMessage("Offset will be deleted");
//			response.setState(Response.HOLD_DELETE);
//			response.setMovieTime(offsetMinEndMovieTime);
//		}
//
//		return response;
//	}
//
//	public Response checkInitialHoldStartMovieTimeUpdatePossibility(
//			int movieTime) {
//		ChainedEvent onset = collection.getOnsetEvent();
//		ChainedEvent initialHold = collection.getInitialHoldEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int maxHoldStartMovieTime = convertor
//				.getPreviousFrameMovieTime(initialHold.getEndTimeInfo()
//						.getMovieTime());
//
//		if (movieTime < maxHoldStartMovieTime) {
//			if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime >= onsetMinEndMovieTime) {
//					;
//				} else {
//					response.setErrorMessage("Onset will be deleted");
//					response.setState(Response.HOLD_DELETE);
//					response.setMovieTime(onsetMinEndMovieTime);
//				}
//			} else {
//				;
//			}
//		} else {
//			response.setErrorMessage("Initial Hold will be deleted");
//			response.setState(Response.HOLD_DELETE);
//			response.setMovieTime(maxHoldStartMovieTime);
//		}
//
//		return response;
//	}
//
//	public Response checkFinalHoldEndMovieTimeUpdatePossibility(int movieTime) {
//		ChainedEvent offset = collection.getOffsetEvent();
//		ChainedEvent finalHold = collection.getFinalHoldEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int holdMinEndMovieTime = convertor.getNextFrameMovieTime(finalHold
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime > holdMinEndMovieTime) {
//			if (offset != null) {
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					;
//				} else {
//					response.setErrorMessage("Offset will be deleted");
//					response.setState(Response.HOLD_DELETE);
//					// response.setMovieTime();
//				}
//			}
//		} else {
//			response.setErrorMessage("Final Hold will be deleted");
//			response.setState(Response.HOLD_DELETE);
//			response.setMovieTime(holdMinEndMovieTime);
//		}
//
//		return response;
//	}
//
//	public boolean setFinalHoldEndMovieTimeNoWarning(int movieTime) {
//		ChainedEvent finalHold = collection.getFinalHoldEvent();
//
//		if (finalHold == null)
//			return true;
//
//		ChainedEvent offset = collection.getOffsetEvent();
//
//		int holdMinEndMovieTime = convertor.getNextFrameMovieTime(finalHold
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= holdMinEndMovieTime) {
//			if (offset != null) {
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					finalHold.getEndTimeInfo().setMovieTime(movieTime);
//					offset.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					finalHold.getEndTimeInfo().setMovieTime(
//							maxOffsetStartMovieTime);
//					offset.getStartTimeInfo().setMovieTime(
//							maxOffsetStartMovieTime);
//					return true;
//				}
//			} else {
//				finalHold.getEndTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			finalHold.getEndTimeInfo().setTimeBase(holdMinEndMovieTime);
//			return true;
//		}
//	}
//
//	public boolean setValueEndMovieTimeNoWarning(int movieTime) {
//		ChainedEvent offset = collection.getOffsetEvent();
//		ChainedEvent finalHold = collection.getFinalHoldEvent();
//		ChainedEvent value = collection.getValueOrTextValueEvent();
//
//		if (value == null)
//			return false;
//
//		int valueMinEndMovieTime = convertor.getNextFrameMovieTime(value
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= valueMinEndMovieTime) {
//			if (finalHold != null) {
//				int maxHoldStartMovieTime = convertor
//						.getPreviousFrameMovieTime(finalHold.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxHoldStartMovieTime >= movieTime) {
//					value.getEndTimeInfo().setMovieTime(movieTime);
//					finalHold.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					value.getEndTimeInfo().setMovieTime(maxHoldStartMovieTime);
//					finalHold.getStartTimeInfo().setMovieTime(
//							maxHoldStartMovieTime);
//					return true;
//				}
//
//			} else if (offset != null) {
//
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					value.getEndTimeInfo().setMovieTime(movieTime);
//					offset.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					try {
//						finalHold.getStartTimeInfo().setMovieTime(
//								maxOffsetStartMovieTime);
//						value.getEndTimeInfo().setMovieTime(
//								maxOffsetStartMovieTime);
//					} catch (NullPointerException ex) {
//						ex.printStackTrace();
//					}
//					return true;
//				}
//			} else {
//				value.getEndTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			return deleteEntity();
//		}
//	}

	// add onset, one frame long approximately 1/30 second
	public void addOnsetEvent() {
		ChainedEvent event = collection.getInitialHoldEvent();
		if (event == null)
			event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.ONSET_CODE;
			
			TimeInfo tm = event.getStartTimeInfo();

			int timeBase = tm.getTimeBase();
			int timeScale = tm.getTimeScale();
			int endTime = tm.getMovieTime();
			int startTime = endTime - minFrameDurationToAdd;
			int endFrameNumber = endTime/minFrameDurationToAdd;
			int startFrameNumber = endFrameNumber - 1;
			  
			TimeInfo endTimeInfo = new TimeInfo();
			endTimeInfo.setMovieTime(endTime);
			endTimeInfo.setTimeBase(timeBase);
			endTimeInfo.setTimeScale(timeScale);
			//   endFrameNumber, endTime, timeBase, timeScale);
			TimeInfo startTimeInfo = new TimeInfo(); 
			//startFrameNumber, startTime, timeBase, timeScale);
			startTimeInfo.setMovieTime(startTime);
			startTimeInfo.setTimeBase(timeBase);
			startTimeInfo.setTimeScale(timeScale);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setOnsetEvent(new ChainedEvent(newValue, segmentPanel));
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedEventsCollectionManager");
		}
	}

	public void addOffsetEvent() {
		ChainedEvent event = collection.getFinalHoldEvent();
		if (event == null)
			event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.OFFSET_CODE;
			TimeInfo eventStartTimeInfo = event.getEndTimeInfo().cloneTimeInfo();
			TimeInfo endTimeInfo = eventStartTimeInfo.cloneTimeInfo();
			endTimeInfo.setMovieTime(event.getEndTimeInfo().getMovieTime() + minFrameDurationToAdd);
			TrackValue newValue = new TrackValue(id, "", eventStartTimeInfo, endTimeInfo);
			collection.setOffsetEvent(new ChainedEvent(newValue, segmentPanel));
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedEventsCollectionManager");
		}
	}

	public void addInitialHoldEvent() {
		ChainedEvent onset = collection.getOnsetEvent();
		ChainedEvent value = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.HOLD_CODE;
			TimeInfo valueStartTimeInfo = value.getStartTimeInfo()
					.cloneTimeInfo();
			TimeInfo holdStartTimeInfo = valueStartTimeInfo.cloneTimeInfo();
			
			int holdStartTime = valueStartTimeInfo.getMovieTime() - minFrameDurationToAdd;
			
			holdStartTimeInfo.setMovieTime(holdStartTime);

			TrackValue newValue = new TrackValue(id, "", holdStartTimeInfo,
					valueStartTimeInfo);
			collection.setInitialHoldEvent(new ChainedEvent(newValue,
					segmentPanel));

			if (onset != null) {
				onset.getEndTimeInfo().setMovieTime(holdStartTime);
				// onset.getEndTimeInfo().setMovieTime(onset.getEndTimeInfo().getMovieTime()-minFrameDurationToAdd);
			}
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedEventsCollectionManager");
		}
	}

	public void addInitialHoldEvent(ChainedEvent hold) {
		collection.setInitialHoldEvent(hold);
	}

	public void addFinalHoldEvent(ChainedEvent hold) {
		collection.setFinalHoldEvent(hold);
	}

	public void addFinalHoldEvent() {
		ChainedEvent offset = collection.getOffsetEvent();
		ChainedEvent value = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.HOLD_CODE;
			TimeInfo valueEndTimeInfo = value.getEndTimeInfo();
			TimeInfo holdStartTimeInfo = valueEndTimeInfo.cloneTimeInfo();
			TimeInfo holdEndTimeInfo = valueEndTimeInfo.cloneTimeInfo();
			holdEndTimeInfo.setMovieTime(valueEndTimeInfo.getMovieTime()
					+ minFrameDurationToAdd);

			TrackValue newValue = new TrackValue(id, "", holdStartTimeInfo,
					holdEndTimeInfo);
			collection.setFinalHoldEvent(new ChainedEvent(newValue,
					segmentPanel));

			if (offset != null) {
				offset.getStartTimeInfo().setMovieTime(
						offset.getStartTimeInfo().getMovieTime()
								+ minFrameDurationToAdd);
				offset.getEndTimeInfo().setMovieTime(
						offset.getEndTimeInfo().getMovieTime());
			}
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedEventsCollectionManager");
		}
	}
}
