package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.HeadlessException;
import java.util.ArrayList;

import javax.swing.JOptionPane;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.freeTextField.FreeTextEntity;
import edu.bu.signstream.grepresentation.fields.glossField.Compoundable;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class ChainedEventsEntity extends EventsEntity {

	private static final long serialVersionUID = 1L;

	protected ChainedEventsCollectionManager manager;

	private ChainedEventsEntity previousEntity = null;
	private ChainedEventsEntity nextEntity = null;
	
	private boolean isLockedGlossPhonAlignments = false;

	public ChainedEventsEntity(ChainedEventsCollection collection,
			SignStreamSegmentPanel segmentPanel, Field field) {

		super(segmentPanel, field);
		setID(Util.getLongUniqueNumber());
		this.manager = new ChainedEventsCollectionManager(collection,
				segmentPanel);
	}
	
	public boolean isLockedGlossPhonAlignments() {
		return isLockedGlossPhonAlignments;
	}

	public void setLockedGlossPhonAlignments(boolean isLockedGlossPhonAlignments) {
		this.isLockedGlossPhonAlignments = isLockedGlossPhonAlignments;
	}
	
    // returns an error code that will be matched with an error message or null if success. 
	public String setSelectedEventStartMovieTime(int movieTime) {
		Event selectedEvent =  SS3Singleton.getMediaToolBar(null).getSelectedEvent();
		
        TimeInfo originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo().cloneTimeInfo();
        TimeInfo originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo().cloneTimeInfo();

		int packetDuration = (int) getField().getSignStreamSegmentPanel()
				.getMultipleMovieController().getPacketDuration();
		int maxStartMovieTime = selectedEvent.getEndTimeInfo().getMovieTime()
				- packetDuration;

		// gloss value events are allowed to have the same start/end time
		if (this instanceof GlossChainedEventsEntity
				&& selectedEvent.equals(manager.getChainedEventsCollection().getValueOrTextValueEvent())) {
			maxStartMovieTime += packetDuration;
		}

		if (maxStartMovieTime < movieTime) {
			return ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME;
		}
		
		Event prevEvent = manager.getChainedEventsCollection().getPreviousEvent(selectedEvent);
		
		TimeInfo originalPrevEndTimeInfo = null;
		
		if (prevEvent != null)
			originalPrevEndTimeInfo = prevEvent.getEndTimeInfo().cloneTimeInfo();
		
		if (prevEvent == null) {
			// |<--
			if (selectedEvent.getStartTimeInfo().getMovieTime() > movieTime) {
				if (previousEntity == null
						|| previousEntity.getEndTimeInfo().getMovieTime() <= movieTime) {
		            TimeInfo timeInfo1 = selectedEvent.getStartTimeInfo().cloneTimeInfo();
		            timeInfo1.setMovieTime(movieTime);
		            selectedEvent.setStartTimeInfo(timeInfo1);
//                    // selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
				} else {
					return ErrorMessages.CHAINS_CANNOT_OVERLAP;
				}
			} else {
	            TimeInfo timeInfo1 = selectedEvent.getStartTimeInfo().cloneTimeInfo();
	            timeInfo1.setMovieTime(movieTime);
	            selectedEvent.setStartTimeInfo(timeInfo1);
//                // selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
			}			
		} else if (selectedEvent.getStartTimeInfo().getMovieTime() > movieTime) {
			int prevStartTime = prevEvent.getStartTimeInfo().getMovieTime();
			double prevEventStartFrame = prevStartTime / packetDuration;
			double goToMovie = movieTime / packetDuration;
			double diff = goToMovie - prevEventStartFrame;
			
			if (diff < 1) { 
				
				//Event onsetEvent = manager.getChainedEventsCollection().getOnsetEvent();
				Event initialHoldEvent = manager.getChainedEventsCollection().getInitialHoldEvent();
				Event event = manager.getChainedEventsCollection().getValueOrTextValueEvent();
				Event finalHoldEvent = manager.getChainedEventsCollection().getFinalHoldEvent();
				Event offsetEvent = manager.getChainedEventsCollection().getOffsetEvent();
				
				if(selectedEvent.equals(initialHoldEvent)) {
					return ErrorMessages.SET_START_INITIAL_HOLD_CANT_DELETE_ONSET;
				} else if(selectedEvent.equals(event)) {
					if(initialHoldEvent != null && prevEvent.equals(initialHoldEvent)) {
						return ErrorMessages.SET_START_VALUE__CANT_DELETE_INITIAL_HOLD;
					} else  {
						return ErrorMessages.SET_START_VALUE_CANT_DELETE_ONSET;
					}
				} else if(selectedEvent.equals(finalHoldEvent)) {
					return ErrorMessages.SET_START_FINAL_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(offsetEvent)) {
					if(finalHoldEvent != null && prevEvent.equals(event)) {
						return ErrorMessages.SET_START_OFFSET_CANT_DELETE_FINAL_HOLD; 
					} else {
						return ErrorMessages.SET_START_OFFSET_CANT_DELETE_VALUE;
					}
				}
			} else {
	            TimeInfo timeInfo1 = selectedEvent.getStartTimeInfo().cloneTimeInfo();
	            timeInfo1.setMovieTime(movieTime);
	            selectedEvent.setStartTimeInfo(timeInfo1);
	            
                TimeInfo timeInfo2 = prevEvent.getEndTimeInfo().cloneTimeInfo();
                timeInfo2.setMovieTime(movieTime);
				prevEvent.setEndTimeInfo(timeInfo2);
			}
		} else {
			// selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
			// prevEvent.getEndTimeInfo().setMovieTime(movieTime);
            //
            TimeInfo timeInfo1 = selectedEvent.getStartTimeInfo().cloneTimeInfo();
            timeInfo1.setMovieTime(movieTime);
            selectedEvent.setStartTimeInfo(timeInfo1);
            
            TimeInfo timeInfo2 = prevEvent.getEndTimeInfo().cloneTimeInfo();
            timeInfo2.setMovieTime(movieTime);
            prevEvent.setEndTimeInfo(timeInfo2);
		}

		// check for one-frame logic of gloss events
		if(this instanceof GlossChainedEventsEntity
				&& selectedEvent.equals(manager.getChainedEventsCollection().getValueOrTextValueEvent())) {
			GlossChainedEventsEntity  glsEntity = (GlossChainedEventsEntity)this;
			
			int startMovieTime = originalSelectedStartTimeInfo.getMovieTime();
			int endMovieTime = originalSelectedEndTimeInfo.getMovieTime();
			GlossEntityProperties properties = glsEntity.getGlossEntityProperties();
			
			// current start and end times same? it means we are moving to a 1 or more frame difference, so set oneFrameGloss to false
			if (startMovieTime == endMovieTime) {
				properties.setOneFrameGloss(false);
				
				if (glsEntity.getDependentHandShapeEntity() != null)
					glsEntity.getDependentHandShapeEntity().cloneEvents(movieTime);
				
				if (glsEntity.getDependentGlossEntity() != null && glsEntity.getDependentGlossEntity().getDependentHandShapeEntity() != null)
					glsEntity.getDependentGlossEntity().getDependentHandShapeEntity().cloneEvents(movieTime);
			}
			else {
				// start is moving into end - intention is to recreate a one frame gloss
				if (movieTime == endMovieTime) {	
					if (glsEntity.getDependentHandShapeEntity() != null
							|| (glsEntity.getDependentGlossEntity() != null && glsEntity.getDependentGlossEntity().getDependentHandShapeEntity() != null) ) {
						boolean isPossibleToMerge = false;	// can we merge handshapes at all? (i.e. is the first the same as the last handshape?)
						
						if (glsEntity.getDependentHandShapeEntity() != null) {
							isPossibleToMerge = glsEntity.getDependentHandShapeEntity().isPossibleToMergeEvents();
						}
						
						if (isPossibleToMerge) {
							if (glsEntity.getDependentGlossEntity() != null && glsEntity.getDependentGlossEntity().getDependentHandShapeEntity() != null) {
								isPossibleToMerge = glsEntity.getDependentGlossEntity().getDependentHandShapeEntity().isPossibleToMergeEvents();
							}
						}

						int optionPaneResult = -1;
						if (isPossibleToMerge) {
							String warningDialogText = "Warning: this action will create a one-frame gloss\n\n"
									+ "and merge the start and end handshapes into one handshape.\n\n"
									+ "If this is okay, please press Proceed";
							String warningTitleText = "WARNING";
							int optionType = JOptionPane.YES_NO_OPTION;
							int messageType = JOptionPane.ERROR_MESSAGE;
							String options[] = {"Proceed", "Cancel"};
							// indeed this action is not an error, and is in fact a warning;
							// but nonetheless a red stop sign messageType is louder than a yellow triangle
							
							try {
								optionPaneResult = JOptionPane.showOptionDialog(this, warningDialogText, warningTitleText, optionType, 
										messageType, null, options,	options[1]);
							} catch (HeadlessException he) {
								System.out.println(he.getMessage());
							}
						} else {				            
							// first undo time moves, always for selectedEvent
				            TimeInfo timeInfo1 = originalSelectedStartTimeInfo.cloneTimeInfo();
				            selectedEvent.setStartTimeInfo(timeInfo1);

				            // and check if we should undo move for prevEvent as well 
				            if (prevEvent != null) {	            
				                TimeInfo timeInfo2 = originalPrevEndTimeInfo.cloneTimeInfo();
								prevEvent.setEndTimeInfo(timeInfo2);            	
				            }					
				            
							return ErrorMessages.SET_ONE_FRAME_GLOSS_DIFF_HANDSHAPES;
						}
						
						if (optionPaneResult == JOptionPane.OK_OPTION) {			
							// merge current start and end handshapes into one
							if (glsEntity.getDependentHandShapeEntity() != null) 
								glsEntity.getDependentHandShapeEntity().mergeEvents(movieTime);			
												
							if (glsEntity.getDependentGlossEntity() != null && glsEntity.getDependentGlossEntity().getDependentHandShapeEntity() != null)
								glsEntity.getDependentGlossEntity().getDependentHandShapeEntity().mergeEvents(movieTime);								
							
							properties.setOneFrameGloss(true);
							properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
							properties.setStartEndHandshapeLeftHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
							properties.setStartEndHandshapeRightHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
							
							return null;
						} else {	// not possible to merge, due to different start/end handshapes
							// first undo time moves, always for selectedEvent
				            TimeInfo timeInfo1 = originalSelectedStartTimeInfo.cloneTimeInfo();
				            selectedEvent.setStartTimeInfo(timeInfo1);

				            // and check if we should undo move for prevEvent as well 
				            if (prevEvent != null) {	            
				                TimeInfo timeInfo2 = originalPrevEndTimeInfo.cloneTimeInfo();
								prevEvent.setEndTimeInfo(timeInfo2);            	
				            }
				            
				            return ""; // this is a dummy return code, which returns with no action being taken, and prevents a popup window in calling function
						}
					}
				}
			}
		}
		
		return null;
	}

	public String setSelectedEventEndMovieTime(int movieTime) {
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).getSelectedEvent();
		
        TimeInfo originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo().cloneTimeInfo();
        TimeInfo originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo().cloneTimeInfo();

		int packetDuration = (int) getField().getSignStreamSegmentPanel()
				.getMultipleMovieController().getPacketDuration();
		int minEndMovieTime = selectedEvent.getStartTimeInfo().getMovieTime()
				+ packetDuration;
		
		// gloss value events are allowed to have the same start/end time
		if (this instanceof GlossChainedEventsEntity
				&& selectedEvent.equals(manager.getChainedEventsCollection().getValueOrTextValueEvent())) {
			minEndMovieTime -= packetDuration;
		}

		if (minEndMovieTime > movieTime) {
			return ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME;
		}
		
		if(nextEntity != null && nextEntity.getStartTimeInfo().getMovieTime() < movieTime) {
			return ErrorMessages.CHAINS_CANNOT_OVERLAP;
		}

		ChainedEvent onset = manager.getChainedEventsCollection().getOnsetEvent();
		ChainedEvent initialHold = manager.getChainedEventsCollection().getInitialHoldEvent();
		ChainedEvent value = manager.getChainedEventsCollection().getValueOrTextValueEvent();
		ChainedEvent finalHold = manager.getChainedEventsCollection().getFinalHoldEvent();	
		
		Event nextEvent = manager.getChainedEventsCollection().getNextEvent(selectedEvent);
		
		TimeInfo originalNextStartTimeInfo = null;
		
		if (nextEvent != null)		
			originalNextStartTimeInfo = nextEvent.getStartTimeInfo().cloneTimeInfo();
		
		if(nextEvent != null) {				
			int maxNextEventStartTime = nextEvent.getEndTimeInfo().getMovieTime()-packetDuration;
			if(maxNextEventStartTime >= movieTime) {
	            TimeInfo timeInfo1 = selectedEvent.getEndTimeInfo().cloneTimeInfo();
	            timeInfo1.setMovieTime(movieTime);
				selectedEvent.setEndTimeInfo(timeInfo1);
				
                TimeInfo timeInfo2 = nextEvent.getStartTimeInfo().cloneTimeInfo();
                timeInfo2.setMovieTime(movieTime);
				nextEvent.setStartTimeInfo(timeInfo2);
			} else {
				if(selectedEvent.equals(onset)) {
					if(initialHold!= null) {
						return ErrorMessages.SET_END_ONSET_CANT_DELETE_INITIAL_HOLD;
					} else {
						return ErrorMessages.SET_END_ONSET_CANT_DELETE_VALUE;
					}
				} else if(selectedEvent.equals(initialHold)) {
					return ErrorMessages.SET_END_INITIAL_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(value)) {
					if(finalHold != null) {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_FINAL_HOLD;
					} else {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_OFFSET;
					}
				} else {
					return ErrorMessages.SET_END_FINAL_HOLD_CANT_DELETE_OFFSET;
				}
			} 
		} else {
            TimeInfo timeInfo1 = selectedEvent.getEndTimeInfo().cloneTimeInfo();
            timeInfo1.setMovieTime(movieTime);
			selectedEvent.setEndTimeInfo(timeInfo1);
		}
		
		// check for one-frame logic of gloss events
		if(this instanceof GlossChainedEventsEntity
				&& selectedEvent.equals(manager.getChainedEventsCollection().getValueOrTextValueEvent())) {
			GlossChainedEventsEntity  glsEntity = (GlossChainedEventsEntity)this;
			
			int startMovieTime = originalSelectedStartTimeInfo.getMovieTime();
			int endMovieTime = originalSelectedEndTimeInfo.getMovieTime();
			GlossEntityProperties properties = glsEntity.getGlossEntityProperties();
			
			// current start and end times same? it means we are moving to a 1 or more frame difference, so set oneFrameGloss to false
			if (startMovieTime == endMovieTime) {
				properties.setOneFrameGloss(false);
				
				if (glsEntity.getDependentHandShapeEntity() != null) 
					glsEntity.getDependentHandShapeEntity().cloneEvents(movieTime);
				
				if (glsEntity.getDependentGlossEntity() != null && glsEntity.getDependentGlossEntity().getDependentHandShapeEntity() != null)
					glsEntity.getDependentGlossEntity().getDependentHandShapeEntity().cloneEvents(movieTime);
							
			}
			else {
				// end is moving into start - intention is to recreate a one frame gloss
				if (movieTime == startMovieTime) {
					if (glsEntity.getDependentHandShapeEntity() != null
							|| (glsEntity.getDependentGlossEntity() != null && glsEntity.getDependentGlossEntity().getDependentHandShapeEntity() != null) ) {
						boolean isPossibleToMerge = false;	// can we merge handshapes at all? (i.e. is the first the same as the last handshape?)
						
						if (glsEntity.getDependentHandShapeEntity() != null) {
							isPossibleToMerge = glsEntity.getDependentHandShapeEntity().isPossibleToMergeEvents();
						}
						
						if (isPossibleToMerge) {
							if (glsEntity.getDependentGlossEntity() != null && glsEntity.getDependentGlossEntity().getDependentHandShapeEntity() != null) {
								isPossibleToMerge = glsEntity.getDependentGlossEntity().getDependentHandShapeEntity().isPossibleToMergeEvents();
							}
						}

						int optionPaneResult = -1;
						if (isPossibleToMerge) {
							String warningDialogText = "Warning: this action will create a one-frame gloss\n\n"
									+ "and merge the start and end handshapes into one handshape.\n\n"
									+ "If this is okay, please press Proceed";
							String warningTitleText = "WARNING";
							int optionType = JOptionPane.YES_NO_OPTION;
							int messageType = JOptionPane.ERROR_MESSAGE;
							String options[] = {"Proceed", "Cancel"};
							// indeed this action is not an error, and is in fact a warning;
							// but nonetheless a red stop sign messageType is louder than a yellow triangle
							
							try {
								optionPaneResult = JOptionPane.showOptionDialog(this, warningDialogText, warningTitleText, optionType, 
										messageType, null, options,	options[1]);
							} catch (HeadlessException he) {
								System.out.println(he.getMessage());
							}
						} else {
							// first undo time moves, always for selectedEvent
				            TimeInfo timeInfo1 = originalSelectedEndTimeInfo.cloneTimeInfo();
				            selectedEvent.setEndTimeInfo(timeInfo1);
				            
				            // and check if we should undo move for nextEvent as well
				            if (nextEvent != null) {	            
				                TimeInfo timeInfo2 = originalNextStartTimeInfo.cloneTimeInfo();
								nextEvent.setStartTimeInfo(timeInfo2);            	
				            }
							return ErrorMessages.SET_ONE_FRAME_GLOSS_DIFF_HANDSHAPES;							
						}
						
						if (optionPaneResult == JOptionPane.OK_OPTION) {			
							// merge current start and end handshapes into one
							if (glsEntity.getDependentHandShapeEntity() != null) 
								glsEntity.getDependentHandShapeEntity().mergeEvents(movieTime);			
												
							if (glsEntity.getDependentGlossEntity() != null && glsEntity.getDependentGlossEntity().getDependentHandShapeEntity() != null)
								glsEntity.getDependentGlossEntity().getDependentHandShapeEntity().mergeEvents(movieTime);								
							
							properties.setOneFrameGloss(true);
							properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
							properties.setStartEndHandshapeLeftHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
							properties.setStartEndHandshapeRightHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
							
							return null;
						} else {	// not possible to merge, due to different start/end handshapes
							// first undo time moves, always for selectedEvent
				            TimeInfo timeInfo1 = originalSelectedEndTimeInfo.cloneTimeInfo();
				            selectedEvent.setEndTimeInfo(timeInfo1);
				            
				            // and check if we should undo move for nextEvent as well
				            if (nextEvent != null) {	            
				                TimeInfo timeInfo2 = originalNextStartTimeInfo.cloneTimeInfo();
								nextEvent.setStartTimeInfo(timeInfo2);            	
				            }
				            
				            return ""; // this is a dummy return code, which returns with no action being taken, and prevents a popup window in calling function
						}
					}
				}
			}
		}
		
		return null;
	}

	public ChainedEventsEntity getPreviousEntity() {
		return previousEntity;
	}

	public void setPreviousEntity(ChainedEventsEntity previousEntity) {
		this.previousEntity = previousEntity;
	}

	public ChainedEventsEntity getNextEntity() {
		return nextEntity;
	}

	public void setNextEntity(ChainedEventsEntity nextEntity) {
		this.nextEntity = nextEntity;
	}

	public String getText() {
		return getTextValue().getText();
	}

	public Event getNextEvent(Event event) {
		return manager.getChainedEventsCollection().getNextEvent(event);
	}

	public Event getPreviousEvent(Event event) {
		return manager.getChainedEventsCollection().getPreviousEvent(event);
	}

	public void setField(NonManualField nmField) {
		super.setField(nmField);
	}

	public ChainedEventsEntity clone(String fieldID) {
		ChainedEventsCollection cll = manager.getChainedEventsCollection()
				.cloneChainedEventsCollection();
		ChainedEventsEntity ret = new ChainedEventsEntity(cll, segmentPanel, getField());
		ret.determineParentEntity();
		return ret;
	}

	@Override
	public SS3Entity getSS3Entity() {
		SS3Entity entity = new SS3Entity();
		String id = getID();
		String entID = id.trim().length() > 0 ? id : Util.getLongUniqueNumber();
		entity.setID(entID);
		entity.setRefEntityId(getRefEntityId());

		ChainedEventsCollection collection = manager
				.getChainedEventsCollection();

		if (collection.getOnsetEvent() != null) {
			SS3Item onsetItem = createSS3Item(collection.getOnsetEvent());
			entity.addItem(onsetItem);
		}

		if (collection.getInitialHoldEvent() != null) {
			SS3Item inHoldItem = createSS3Item(collection.getInitialHoldEvent());
			entity.addItem(inHoldItem);
		}

		if (collection.getValueEvent() != null) {
			SS3Item valueItem = createSS3Item(collection.getValueEvent());
			entity.addItem(valueItem);
		}

		if (collection.getTextValueEvent() != null) {
			SS3Item txtValueItem = createSS3Item(collection.getTextValueEvent());
			entity.addItem(txtValueItem);
		}

		if (collection.getFinalHoldEvent() != null) {
			SS3Item fnHoldItem = createSS3Item(collection.getFinalHoldEvent());
			entity.addItem(fnHoldItem);
		}

		if (collection.getOffsetEvent() != null) {
			SS3Item offsetItem = createSS3Item(collection.getOffsetEvent());
			entity.addItem(offsetItem);
		}
		
		boolean lockedGlossPhoneAlignments = isLockedGlossPhonAlignments();
		entity.setLockedGlossPhoneAlignments(lockedGlossPhoneAlignments);
		
		if(this instanceof GlossChainedEventsEntity) {
			GlossChainedEventsEntity  glsEntity = (GlossChainedEventsEntity)this;
			boolean isLockedLeftRightAlignments = glsEntity.isLockedLeftRightAlignments();
			
			GlossChainedEventsEntity dependentGlossEntity = glsEntity.getDependentGlossEntity();
			ChainedEventsEntity dependentHandEntity = glsEntity.getDependentHandEntity();
			HandShapeEntity dependentHandShapeEntity = glsEntity.getDependentHandShapeEntity();
						
			entity.setLockedLeftRightAlignments(isLockedLeftRightAlignments);
			if(dependentGlossEntity != null) {
				entity.setDependentGlossEntityID(dependentGlossEntity.getID());
			}
			if(dependentHandEntity != null) {
				entity.setDependentHandEntityID(dependentHandEntity.getID());
			}
			if(dependentHandShapeEntity != null) {
				//String hndID = dependentHandShapeEntity.getID();
				entity.setDependentHandShapeEntityID(dependentHandShapeEntity.getID());
			}

            Compoundable startCompound = glsEntity.getStartCompound();
            if(startCompound != null) {
                entity.setStartCompaundEntityID(startCompound.getID());
            }

            Compoundable endCompound = glsEntity.getEndCompound();
            if(endCompound != null) {
                entity.setEndCompaundEntityID(endCompound.getID());
            }
			
			GlossEntityProperties properties = glsEntity.getGlossEntityProperties();
			entity.setGlossEntityProperties(properties);
		} else if (this instanceof FreeTextEntity) {
			FreeTextEntity freeTextEntity = (FreeTextEntity) this;
			if (freeTextEntity.isValidTime()) {
				entity.setValidTime(true);
				
//				int entityStartTime = entity.getStartTime();
//				int entityEndTime = entity.getEndTime();
//				
//				int freeTextEntityStartTime = this.getStartTimeInfo().getMovieTime();
//				int freeTextEntityEndTime = this.getEndTimeInfo().getMovieTime();
//				
//				// continue on here if we want to set valid time only if the times are aligned exactly
			}
		}

		return entity;
	}

	private SS3Item createSS3Item(ChainedEvent event) {
		SS3Item item = new SS3Item();
		item.setId(event.getID());
		item.setText(event.getText());
		item.setStartTimeInfo(event.getStartTimeInfo());
		item.setEndTimeInfo(event.getEndTimeInfo());
		return item;
	}

	public Event getValueOrTextValueEvent() {
		return manager.getChainedEventsCollection().getValueOrTextValueEvent();
	}

	public ChainedEventsCollection getChainedEventsCollection() {
		return manager.getChainedEventsCollection();
	}

	public ChainedEvent getValue() {
		return manager.getChainedEventsCollection().getValueEvent();
	}

	public ChainedEvent getTextValue() {
		return manager.getChainedEventsCollection().getTextValueEvent();
	}

	public GlossChainedEvent getGlossChainedEvent() {
		return manager.getChainedEventsCollection().getTextValueEvent();
	}

	public void setStartTime(int startTime) {
		ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
		if (event != null)
			event.getStartTimeInfo().setMovieTime(startTime);
	}

	@Override
	public void setStartTimeInfo(TimeInfo startTimeInfo) {
		ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
		if (event != null)
			event.setStartTimeInfo(startTimeInfo);
	}

	@Override
	public void setEndTimeInfo(TimeInfo endTimeInfo) {
		ChainedEvent event = (ChainedEvent) manager.getLastEvent();
		if (event != null)
			event.setEndTimeInfo(endTimeInfo);
	}

	public void setEndTime(int endTime) {
		ChainedEvent event = (ChainedEvent) manager.getLastEvent();
		if (event != null)
			event.getEndTimeInfo().setMovieTime(endTime);
	}

	@Override
	public Event getFirstEvent() {
		return manager.getFirstEvent();
	}

	public ChainedEvent getInitialHold() {
		return manager.getChainedEventsCollection().getInitialHoldEvent();
	}

	public ChainedEvent getFinalHold() {
		return manager.getChainedEventsCollection().getFinalHoldEvent();
	}

	public ChainedEvent getOnset() {
		return manager.getChainedEventsCollection().getOnsetEvent();
	}

	public void deleteOnset() {
		manager.deleteOnsetEventNoWarning();
	}

	public void deleteInitialHold() {
		manager.deleteInitialHoldEventNoWarning();
	}

	public void deleteFinalHold() {
		manager.deleteFinalHoldEventNoWarning();
	}

	public void deleteOffset() {
		manager.deleteOffsetEventNoWarning();
	}

	public ChainedEvent getOffset() {
		return manager.getChainedEventsCollection().getOffsetEvent();
	}

	@Override
	public Event getLastEvent() {
		return manager.getLastEvent();
	}

	@Override
	public TimeInfo getStartTimeInfo() {
		ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
		if (event == null)
			return null;
		return event.getStartTimeInfo();
	}

	@Override
	public TimeInfo getEndTimeInfo() {
		ChainedEvent event = (ChainedEvent) manager.getLastEvent();
		if (event == null)
			return null;
		return event.getEndTimeInfo();
	}

	@Override
	public boolean selectEntityIfEventSelected() {
		if (manager.getSelectedEvent() != null) {
			select();
			return true;
		}
		return false;
	}

	@Override
	public boolean selectEventIfEntitySelected() {
		if (selected) {
			manager.unselectEntity();
			ChainedEvent valueEvent = manager.getChainedEventsCollection()
					.getValueEvent();
			manager.selectChainedEvent(valueEvent);
			selected = false;
			return true;
		}
		return false;
	}

	@Override
	public boolean select() {
		selected = true;
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		unhighlight();
		manager.unselectEntity();
		return true;
	}

	@Override
	public boolean adjustEntity(int x1, int x2) {
		double pixelInTimeFrame = segmentPanel.getZoomer()
				.getPixelsInTimeFrame();
		double startTime = x1 / pixelInTimeFrame;
		double endTime = x2 / pixelInTimeFrame;
		return manager.adjustEntity((int) startTime, (int) endTime);
	}

	@Override
	public Event getSelectedEvent() {		
		if (selected)
			return this;
		return manager.getSelectedEvent();
	}

	@Override
	public void reset() {
		selected = false;
		// mToolBar.setEnable(false);
		manager.resetAll();
	}

	@Override
	public boolean delete() {
		return delete(false);
	}

	public boolean delete(boolean showWarningMsg) {
		return manager.deleteEntity();
	}

	public void setValueID(String valueID) {
		ChainedEventsCollection collection = manager
				.getChainedEventsCollection();
		ChainedEvent event = collection.getValueEvent();
		event.setID(valueID);
	}

	@Override
	public Event getEventAt(int x) {
		return manager.getEventAt(x);
	}

	@Override
	public ArrayList<String> getEventsToAdd() {
		ArrayList<String> list = new ArrayList<String>();

		ChainedEventsCollection collection = manager
				.getChainedEventsCollection();

		if (collection.getOnsetEvent() == null)
			list.add(Constants.ONSET);

		if (collection.getInitialHoldEvent() == null)
			list.add(Constants.INITIAL_HOLD);

		if (collection.getFinalHoldEvent() == null)
			list.add(Constants.FINAL_HOLD);

		if (collection.getOffsetEvent() == null)
			list.add(Constants.OFFSET);

		return list;
	}

	@Override
	public String getToolTipText(int x) {
		ChainedEvent event = (ChainedEvent) manager.getEventAt(x);

		if (event != null)
			return event.getToolTipText();
		return null;
	}

	@Override
	public boolean selectEvent(int x) {
		Event event = manager.getEventAt(x);
		return event.select();
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);

		segmentPanel.getPainter().drawEntity(this, g2d, Y, fontMetrics,
				getField().getFieldID(), previousEntity, nextEntity);
	}

	public void setInitialHoldEvent(ChainedEvent event) {
		if (event != null)
			event.setID(Constants.TYPE_HOLD);
		manager.getChainedEventsCollection().setInitialHoldEvent(event);
	}

	public void setFinalHoldEvent(ChainedEvent event) {
		if (event != null)
			event.setID(Constants.TYPE_HOLD);
		manager.getChainedEventsCollection().setFinalHoldEvent(event);
	}

	public void setOnsetEvent(ChainedEvent event) {
		if (event != null)
			event.setID(Constants.TYPE_ONSET);
		manager.getChainedEventsCollection().setOnsetEvent(event);
	}

	public void setOffsetEvent(ChainedEvent event) {
		if (event != null)
			event.setID(Constants.TYPE_OFFSET);
		manager.getChainedEventsCollection().setOffsetEvent(event);
	}

	public String addOnsetEvent() {
		if (previousEntity != null) {
			int endMovieTime = previousEntity.getEndTimeInfo().getMovieTime();
			int packetDuration = (int) segmentPanel
					.getMultipleMovieController().getPacketDuration();

			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
			int selectedEventStartTime = event.getStartTimeInfo()
					.getMovieTime() - packetDuration;

			if (selectedEventStartTime < endMovieTime) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOnsetEvent();
		return null;
	}

	public String addInitialHoldEvent() {
		int packetDuration = (int) segmentPanel.getMultipleMovieController()
				.getPacketDuration();

		ChainedEvent onset = getOnset();
		if (onset != null) {
			int onsetStartTime = onset.getStartTimeInfo().getMovieTime();
			int onsetEndTime = onset.getEndTimeInfo().getMovieTime();
			int onsetDuration = onsetEndTime - onsetStartTime;

			if (onsetDuration == packetDuration) {
				return ErrorMessages.ADD_INITIAL_HOLD_CANT_ADJUST_ONSET;
			} else {
				manager.addInitialHoldEvent();
				return null;
			}
		}

		if (previousEntity != null) {
			int endMovieTime = previousEntity.getEndTimeInfo().getMovieTime();

			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();

			int selectedEventStartTime = event.getStartTimeInfo()
					.getMovieTime() - packetDuration;

			if (selectedEventStartTime < endMovieTime) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addInitialHoldEvent();
		return null;
	}

	public String addFinalHoldEvent() {
		int packetDuration = (int) segmentPanel.getMultipleMovieController()
				.getPacketDuration();

		if (nextEntity != null) {
			int startMovieTime = nextEntity.getStartTimeInfo().getMovieTime();

			ChainedEvent event = (ChainedEvent) manager.getLastEvent();
			int selectedEventEndTime = event.getEndTimeInfo().getMovieTime()
					+ packetDuration;

			if (selectedEventEndTime > startMovieTime) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		ChainedEvent offset = getOffset();
		if (offset != null) {
			int offsetStartTime = offset.getStartTimeInfo().getMovieTime();
			int offsetEndTime = offset.getEndTimeInfo().getMovieTime();
			int offsetDuration = offsetEndTime - offsetStartTime;

			if (offsetDuration == packetDuration) {
				return ErrorMessages.ADD_FINAL_HOLD_CANT_ADJUST_OFFSET;
			}
		}

		manager.addFinalHoldEvent();
		return null;
	}

	public String addOffsetEvent() {
		if (nextEntity != null) {
			int startMovieTime = nextEntity.getStartTimeInfo().getMovieTime();
			int packetDuration = (int) segmentPanel
					.getMultipleMovieController().getPacketDuration();

			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
			int selectedEventEndTime = event.getEndTimeInfo().getMovieTime()
					+ packetDuration;

			if (selectedEventEndTime > startMovieTime) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOffsetEvent();
		return null;
	}
}