package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

/**
 * The nom-manual fields fall into two categories: anatomical and functional.
 * 
 * Anatomical fields refer to particular physical muscle movement.
 * Functional/interpretive fields provide a functional interpretation for a
 * particular movement or set of movements.
 * */

public class NonManualField extends Field {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected ChainedEventsEntityCollection collection = new ChainedEventsEntityCollection(new ArrayList<ChainedEventsEntity>(), segmentPanel);

	public NonManualField(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		Track track = new Track();
		createTrackEntity(track);
	}

	public NonManualField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.track = track;
		createTrackEntity(track);
	}

	public NonManualField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.ss3Track = ss3track;
		createTrackEntity(ss3track);
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {
		ChainedEventsEntity cEntity = (ChainedEventsEntity) entity;
		cEntity.setField(this);
		collection.addEntity(cEntity);
	}

	@Override
	public EventsEntity getEntity(String entityID) {
		return collection.getEntity(entityID);
	}

	// return ChainedEventsEntities
	@Override
	public ArrayList getEntities() {
		return collection.getEntities();
	}

	@Override
	public EventsEntity getEntityWithRefID(String refEntityId) {
		return collection.getEntityWithRefID(refEntityId);
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		if (!highlight) {
			ArrayList<ChainedEventsEntity> entities = collection.getEntities();
			for (ChainedEventsEntity entity : entities) {
				entity.highlight(false);
			}
		} else {
			EventsEntity entity = collection.getEntityWithRefID(refID);
			if (entity != null) {
                entity.highlight(true);
            }
		}
	}

	@Override
	public int getFieldSize() {
		return collection.size();
	}

	@Override
	public void deleteEntity(EventsEntity entity) {
		collection.deleteEntity((ChainedEventsEntity)entity);
	}

	public Track getTrack() {
		return track;
	}

	@Override
	public EventsEntity getEntityAt(int x) {
		return collection.getChainedEventsEntityAt(x);
	}

	@Override
	public Event getEventAt(int x) {
		return collection.getEventAt(x);
	}

	@Override
	public EventsEntity getSelectedEntity() {
		return collection.getSelectedEntity();
	}

	@Override
	public int getStartTime() {
		if (collection == null)
			return 0;
		return collection.getStartTime();
	}

	@Override
	public int getEndTime() {
		if (collection == null)
			return 0;
		return collection.getEndTime();
	}

	@Override
	public int getStartTimeCoordinate() {
		if (collection == null)
			return 0;
		return collection.getStartTimeCoordinate();
	}

	@Override
	public int getEndTimeCoordinate() {
		if (collection == null)
			return 0;
		return collection.getEndTimeCoordinate();
	}

	@Override
	public void unselect() {
		collection.unselect();
	}

	public void reset() {
		collection.reset();
	}

	public void setNewChainedEvent(NewChainedEvent event) {
		collection.setNewChainedEvent(event);
	}

	public boolean canDrawNewEvent(int x) {
		if (contains(x)) {
			ChainedEvent event = collection.getEventAt(x);
			return (event == null);
		}

		return false;
	}

	public void selectEntity(int x) {
		if (contains(x)) {
            collection.selectEntity(x);
        }
	}

	@Override
	public Event getSelectedEvent() {
		return collection.getSelectedEvent();
	}

	public NewChainedEvent getNewChainedEvent() {
		return collection.getNewChainedEvent();
	}

	@Override
	public String getFieldID() {
		if (track != null)
			return track.getFieldId();

		if (ss3Track != null)
			return ss3Track.getId();

		return null;
	}

	private void createTrackEntity(SS3Track track) {
		ArrayList chainedEvents = new ArrayList();
		ArrayList<SS3Entity> entities = track.getEntities();
        for (SS3Entity entity : entities) {
            ChainedEventsCollection collection = new ChainedEventsCollection();

            ArrayList<SS3Item> items = entity.getItemsAsList();

            for (SS3Item vItem : items) {
                String vID = vItem.getId();
                if (!vID.equalsIgnoreCase("" + Constants.ONSET_CODE) && !vID.equalsIgnoreCase("" + Constants.OFFSET_CODE) && !vID.equalsIgnoreCase("" + Constants.HOLD_CODE)) {
                    String fieldID = track.getId();

                    if (fieldID.equalsIgnoreCase("10501") || fieldID.equalsIgnoreCase("10502")) {
                        collection.setTextValueEvent(new GlossChainedEvent(vItem, segmentPanel));
                    } else {
                        collection.setValueEvent(new ChainedEvent(vItem, segmentPanel));
                    }
                }
            }

            for (SS3Item item : items) {
                String id = item.getId();

                if (id.equalsIgnoreCase("" + Constants.ONSET_CODE)) {
                    collection.setOnsetEvent(new ChainedEvent(item,
                            segmentPanel));
                } else if (id.equalsIgnoreCase("" + Constants.OFFSET_CODE)) {
                    collection.setOffsetEvent(new ChainedEvent(item,
                            segmentPanel));
//					} else if (!id.equalsIgnoreCase("" + Constants.HOLD_CODE)) {
//						collection.setValueEvent(new ChainedEvent(item,
//								segmentPanel));
                } else {
                    if (collection.getValueEvent() == null) {
                        System.out.println("no value event!!!" + id);
                    }
                    ChainedEvent valueEvent = collection.getValueEvent();
                    if (valueEvent == null) {
                        valueEvent = collection.getTextValueEvent();
                    }
                    int startTime = valueEvent.getStartTimeInfo().getMovieTime();
                    int endTime = valueEvent.getEndTimeInfo().getMovieTime();
                    if (startTime != endTime) {
                        if (item.getStartTimeInfo().getMovieTime() == endTime) {
                            collection.setFinalHoldEvent(new ChainedEvent(item,
                                    segmentPanel));
                        } else if (item.getEndTimeInfo().getMovieTime() == startTime) {
                            collection.setInitialHoldEvent(new ChainedEvent(
                                    item, segmentPanel));
                        }
                    }
                }
            }

            ChainedEventsEntity cEntity = new ChainedEventsEntity(collection, segmentPanel, this);
            cEntity.setID(entity.getID());
            cEntity.setField(this);
            cEntity.setRefEntityId(entity.getRefEntityId());
            cEntity.setParentEntity(getParentField().getEntity(entity.getParentEntityID()));

            boolean isLockedGlossPhonAlignments = entity.isLockedGlossPhoneAlignments();
            cEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);
            chainedEvents.add(cEntity);
        }

		collection = new ChainedEventsEntityCollection(chainedEvents, segmentPanel);
	}

	private void createTrackEntity(Track track) {
		ArrayList<ChainedEventsEntity> chainedEvents = new ArrayList<ChainedEventsEntity>();
		ArrayList trackValues = track.getTrackValues();

		ArrayList<TrackValue> values = new ArrayList<TrackValue>();
		HashMap<String, TrackValue> onset = new HashMap<String, TrackValue>();
		HashMap<String, TrackValue> onsetHold = new HashMap<String, TrackValue>();
		HashMap<String, TrackValue> offset = new HashMap<String, TrackValue>();
		HashMap<String, TrackValue> offsetHold = new HashMap<String, TrackValue>();

        for (Object trackValue1 : trackValues) {
            TrackValue trackValue = (TrackValue) trackValue1;
            int valueId = trackValue.getValueId();

            if (valueId == Constants.HOLD_CODE) {
                int startFrame = trackValue.getStartTimeInfo().getMovieTime();
                int endFrame = trackValue.getEndTimeInfo().getMovieTime();
                onsetHold.put("" + endFrame, trackValue);
                offsetHold.put("" + startFrame, trackValue);
            } else if (valueId == Constants.ONSET_CODE) {
                int endFrame = trackValue.getEndTimeInfo().getMovieTime();
                onset.put("" + endFrame, trackValue);
            } else if (valueId == Constants.OFFSET_CODE) {
                int startFrame = trackValue.getStartTimeInfo().getMovieTime();
                offset.put("" + startFrame, trackValue);
            } else {
                values.add(trackValue);
            }
        }

        for (TrackValue trackValue : values) {
            int startingFrame = trackValue.getStartTimeInfo().getMovieTime();
            int endingFrame = trackValue.getEndTimeInfo().getMovieTime();

            TrackValue onsetHoldValue = onsetHold.get("" + startingFrame);
            TrackValue offsetHoldValue = offsetHold.get("" + endingFrame);
            TrackValue onsetValue, offsetValue;

            if (onsetHoldValue == null) {
                onsetValue = onset.get("" + startingFrame);
            } else {
                onsetValue = onset.get(""
                        + onsetHoldValue.getStartTimeInfo().getMovieTime());
            }

            if (offsetHoldValue == null) {
                offsetValue = onset.get("" + endingFrame);
            } else {
                offsetValue = onset.get(""
                        + offsetHoldValue.getEndTimeInfo().getMovieTime());
            }

            ChainedEventsCollection collection = new ChainedEventsCollection();
            collection
                    .setOnsetEvent(new ChainedEvent(onsetValue, segmentPanel));
            collection.setInitialHoldEvent(new ChainedEvent(onsetHoldValue,
                    segmentPanel));
            collection
                    .setValueEvent(new ChainedEvent(trackValue, segmentPanel));
            collection.setFinalHoldEvent(new ChainedEvent(offsetHoldValue,
                    segmentPanel));
            collection.setOffsetEvent(new ChainedEvent(offsetValue,
                    segmentPanel));

            ChainedEventsEntity chainedEvent = new ChainedEventsEntity(collection, segmentPanel, this);
            chainedEvent.determineParentEntity();

            chainedEvent.setField(this);
            chainedEvents.add(chainedEvent);
        }

		collection = new ChainedEventsEntityCollection(chainedEvents,
				segmentPanel);
	}

	public void addChainedEventEntity(ChainedEventsEntity entity) {
		entity.setField(this);
		collection.addEntity(entity);
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		super.paint(g2d, Y, font);
		collection.paint(g2d, Y, font);
	}
	
	public ChainedEventsEntity getNextEntity(int coord) {
		return collection.getNextEntity(coord);
	}
	
	public ChainedEventsEntity getPreviousEntity(int coord) {
		return collection.getPreviousEntity(coord);
	}

	@Override
	public void deleteEmptyEntities() {
		collection.removeEmptyEntities();
		
		collection.deleteInvalidEntities(getFieldID());		
	}

	@Override
	public boolean isEmptyInterval(int startTime, int endTime) {
		return collection.getEntitiesInInterval(startTime, endTime).isEmpty();
	}
}
