package edu
        .bu
        .signstream
        .grepresentation
        .fields
        .nonManualField
        .mouseDelegateListeners;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Level;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class DeleteActionListener implements ActionListener {

    /**
     * Selected chained events entity
     */
    private ChainedEventsEntity entity = null;

    /**
     * Sets selected chained events entity.
     * @param entity selected chained events entity
     */
    public void setChainedEventsEntity(ChainedEventsEntity entity) {
        this.entity = entity;
    }

    /**
     * Executes when an annotator wants to delete selected chained entity.
     * If selected entity is GlossChainedEventsEntity, then all related entities will be deleted.
     *
     * Field IDs
     * gloss 10000
     * hand  10501
     * hand shape 1000
     *
     * gloss 10001
     * hand 10502
     * hand shape 2000
     */
    @Override
	public void actionPerformed(ActionEvent e) {
        final JPopupMenu popup = new JPopupMenu();

        JLabel msgLabel = new JLabel("Are you sure you want to delete selected entity?");
        UserInterfaceUtil.setLabelLNF(msgLabel);

        final JButton okBtn = new JButton("Proceed");
        final JButton cancelBtn = new JButton("Cancel");

        JPanel msgPanel = new JPanel();
        GridBagConstraints gbc = new GridBagConstraints();

        msgPanel.setLayout(new GridBagLayout());
        gbc.anchor = GridBagConstraints.WEST;

        gbc.gridwidth = 3;
        gbc.gridx = 0;
        gbc.gridy = 0;
        msgPanel.add(msgLabel, gbc);

        gbc.gridy = 1;
        gbc.gridx = 1;
        msgPanel.add(new JLabel(" "), gbc);

        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.gridy = 3;
        gbc.gridx = 0;
        msgPanel.add(new JLabel(" "), gbc);
        gbc.gridx = 1;
        msgPanel.add(okBtn, gbc);
        gbc.gridx = 2;
        msgPanel.add(cancelBtn, gbc);

        cancelBtn.addActionListener(new ActionListener() {
            @Override
			public void actionPerformed(ActionEvent e) {
                popup.setVisible(false);
            }
        });

        okBtn.addActionListener(new ActionListener() {
            @Override
			public void actionPerformed(ActionEvent e) {
                try {
                    entity.getField().deleteEntity(entity);

                    if (entity instanceof GlossChainedEventsEntity) {
                        GlossChainedEventsEntity glossChainedEventsEntity = (GlossChainedEventsEntity) entity;
                        Field field = glossChainedEventsEntity.getField();
                        String fieldID = field.getFieldID();
                        FieldWrappersCollection fieldWrappersCollection 
                        		= entity.getSignStreamSegmentPanel().getUtteranceView().getSegmentTemplate(field.getSegmentID()).getBottomView().getFieldWrappersCollection();
                        deleteGlossChainedEventsEntity(glossChainedEventsEntity, fieldID, fieldWrappersCollection);

                        GlossChainedEventsEntity depGlossChainedEventsEntity = glossChainedEventsEntity.getDependentGlossEntity();
                        if (depGlossChainedEventsEntity != null) {
                            String ndFieldID = fieldID.equalsIgnoreCase("10000") ? "10001" : "10000";

                            Field ndField = fieldWrappersCollection.getFieldWrapperWithFieldID(ndFieldID).getField();
                            ndField.deleteEntity(depGlossChainedEventsEntity);
                            deleteGlossChainedEventsEntity(depGlossChainedEventsEntity, ndFieldID, fieldWrappersCollection);
                        }
                    }

                    SS3Singleton.getMediaToolBar(null).getSelectedField().repaint();
                    SS3Singleton.getSignStreamApplication().repaint();
					
                    // refresh the utterance lists to reflect the absence of this gloss now
					SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().updateLoadedDatabase();
					SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getUtteranceView().updateUtteranceLists();
                } catch (NullPointerException ex) {
                    // hide the popup and log the exception.
                    SS3Singleton.getSignStreamApplication().logger.log(Level.WARNING, ex.getMessage());
                    popup.setVisible(false);
                }

                popup.setVisible(false);
            }
        });

        SwingUtilities.invokeLater(new Runnable() {
            @Override
			public void run() {
                okBtn.requestFocus();
            }
        });

        int width = msgPanel.getPreferredSize().width + 30;
        int height = msgPanel.getPreferredSize().height + 20;

        popup.add(msgPanel);
        popup.setPopupSize(new Dimension(width, height));
        popup.setLocation(250, 250);
        popup.setVisible(true);
		
		SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
    }

    /**
     * Deletes GlossChainedEventsEntity and if available corresponding entities from Hand Field and from Hand Shape Field.
     *
     * @param glossChainedEventsEntity to delete
     * @param fieldID                  ID of entity's field, in current implementation it is 10000 or 10001
     * @param fieldWrappersCollection  a collection of loaded FieldWrappers
     */
    private void deleteGlossChainedEventsEntity(GlossChainedEventsEntity glossChainedEventsEntity, String fieldID, FieldWrappersCollection fieldWrappersCollection) {

        ChainedEventsEntity dependentHandEntity = glossChainedEventsEntity.getDependentHandEntity();
        HandShapeEntity dependentHandShapeEntity = glossChainedEventsEntity.getDependentHandShapeEntity();

        if (dependentHandEntity != null) {
            if (fieldID.equalsIgnoreCase("10000")) {
                FieldWrapper rHandWrapper = fieldWrappersCollection.getFieldWrapperWithFieldID("10501");
                rHandWrapper.getField().deleteEntity(dependentHandEntity);
                if (dependentHandShapeEntity != null) {
                    FieldWrapper rHandShapeWrapper = fieldWrappersCollection.getFieldWrapperWithFieldID("1000");
                    rHandShapeWrapper.getField().deleteEntity(dependentHandShapeEntity);
                }
            } else {
                FieldWrapper rHandWrapper = fieldWrappersCollection.getFieldWrapperWithFieldID("10502");
                rHandWrapper.getField().deleteEntity(dependentHandEntity);
                if (dependentHandShapeEntity != null) {
                    FieldWrapper rHandShapeWrapper = fieldWrappersCollection.getFieldWrapperWithFieldID("2000");
                    rHandShapeWrapper.getField().deleteEntity(dependentHandShapeEntity);
                }
            }
        }
    }
}
