package edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class LockUnlockLeftRightHandAlignmentsListener implements ActionListener {

	@Override
	public void actionPerformed(ActionEvent e) {
		
		GlossChainedEventsEntity selectedEntity  = (GlossChainedEventsEntity)SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();
		
		boolean locked = !selectedEntity.isLockedLeftRightAlignments();
		GlossChainedEventsEntity ndEntity = selectedEntity
				.getDependentGlossEntity();

		// if we are at this point, both entities are guaranteed to exist, so enforce that they both are aware of the other hand
		selectedEntity.getGlossEntityProperties().setTwoHanded(true);
		ndEntity.getGlossEntityProperties().setTwoHanded(true);

		if (locked) {
			ChainedEvent value = selectedEntity.getTextValue();
			ChainedEvent onset = selectedEntity.getOnset();
			ChainedEvent initialHold = selectedEntity.getInitialHold();
			ChainedEvent offset = selectedEntity.getOffset();
			ChainedEvent finalHold = selectedEntity.getFinalHold();

			ChainedEvent ndValue = ndEntity.getTextValue();
			ChainedEvent ndOnset = ndEntity.getOnset();
			ChainedEvent ndInitialHold = ndEntity.getInitialHold();
			ChainedEvent ndOffset = ndEntity.getOffset();
			ChainedEvent ndFinalHold = ndEntity.getFinalHold();

			GlossChainedEventsEntity ndPrevEntity = ndEntity
					.getPreviousEntity();
			GlossChainedEventsEntity ndNextEntity = ndEntity.getNextEntity();

			int desiredStartTime = value.getStartTimeInfo().getMovieTime();
			int desiredEndTime = value.getEndTimeInfo().getMovieTime();
			
			if (initialHold != null)
				desiredStartTime = initialHold.getStartTimeInfo().getMovieTime();

			// onsets always precede initial holds
			if (onset != null) {
				desiredStartTime = onset.getStartTimeInfo().getMovieTime();
			}
			
			if (finalHold != null) {
				desiredEndTime = finalHold.getEndTimeInfo().getMovieTime();
			}

			// offsets always follow final holds
			if (offset != null) {
				desiredEndTime = offset.getEndTimeInfo().getMovieTime();
			}

			// check if possible to perform the action
			if ((ndPrevEntity == null || ndPrevEntity.getEndTimeInfo()
					.getMovieTime() <= desiredStartTime)
					&& (ndNextEntity == null || ndNextEntity.getStartTimeInfo()
							.getMovieTime() >= desiredEndTime)) {
				// update
				update(selectedEntity, ndEntity);

				ndEntity.setLockedLeftRightAlignments(locked);
				selectedEntity.setLockedLeftRightAlignments(locked);
				SS3Singleton.getSignStreamApplication()
						.getSignStreamSegmentPanel().repaint();

			} else {
				// try to adjust to second gloss
				GlossChainedEventsEntity prevEntity = selectedEntity
						.getPreviousEntity();
				GlossChainedEventsEntity nextEntity = selectedEntity
						.getNextEntity();

				desiredStartTime = ndValue.getStartTimeInfo().getMovieTime();
				desiredEndTime = ndValue.getEndTimeInfo().getMovieTime();
				
				if (ndInitialHold != null)
					desiredStartTime =  ndInitialHold.getStartTimeInfo().getMovieTime();
				
				// onsets always precede initial holds
				if (ndOnset != null) {
					desiredStartTime = ndOnset.getStartTimeInfo().getMovieTime();
				}
				
				if (ndFinalHold != null) {
					desiredEndTime = ndFinalHold.getEndTimeInfo().getMovieTime();
				}
				
				// offsets always follow final holds
				if (ndOffset != null) {
					desiredEndTime = ndOffset.getEndTimeInfo().getMovieTime();
				}

				// check if possible to perform the action
				if ((prevEntity == null || prevEntity.getEndTimeInfo()
						.getMovieTime() <= desiredStartTime)
						&& (nextEntity == null || nextEntity.getStartTimeInfo()
								.getMovieTime() >= desiredEndTime)) {
					// update
					update(ndEntity, selectedEntity);

					ndEntity.setLockedLeftRightAlignments(locked);
					selectedEntity.setLockedLeftRightAlignments(locked);
					SS3Singleton.getSignStreamApplication()
							.getSignStreamSegmentPanel().repaint();
				} else {
					// warning!!!
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CANT_LOCK_GLOSSES);
					return;
				}
			}
		} else {
			ndEntity.setLockedLeftRightAlignments(locked);
			selectedEntity.setLockedLeftRightAlignments(locked);
			SS3Singleton.getSignStreamApplication()
					.getSignStreamSegmentPanel().repaint();
		}
	}

	private void update(GlossChainedEventsEntity selectedEntity,
			GlossChainedEventsEntity ndEntity) {

		ChainedEvent value = selectedEntity.getTextValue();
		ChainedEvent onset = selectedEntity.getOnset();
		ChainedEvent initialHold = selectedEntity.getInitialHold();
		ChainedEvent offset = selectedEntity.getOffset();
		ChainedEvent finalHold = selectedEntity.getFinalHold();

		ChainedEvent ndValue = ndEntity.getTextValue();
		ChainedEvent ndOnset = ndEntity.getOnset();
		ChainedEvent ndInitialHold = ndEntity.getInitialHold();
		ChainedEvent ndOffset = ndEntity.getOffset();
		ChainedEvent ndFinalHold = ndEntity.getFinalHold();

		ndValue.getStartTimeInfo().setMovieTime(
				value.getStartTimeInfo().getMovieTime());
		ndValue.getEndTimeInfo().setMovieTime(
				value.getEndTimeInfo().getMovieTime());

		if (ndInitialHold != null) {
			ndInitialHold.getEndTimeInfo().setMovieTime(
					ndValue.getStartTimeInfo().getMovieTime());
		} else if (ndOnset != null) {
			ndOnset.getEndTimeInfo().setMovieTime(
					ndValue.getStartTimeInfo().getMovieTime());
		}

		if (initialHold != null && ndInitialHold != null) {
			ndInitialHold.getStartTimeInfo().setMovieTime(
					initialHold.getStartTimeInfo().getMovieTime());
			if (ndOnset != null) {
				ndOnset.getEndTimeInfo().setMovieTime(
						ndInitialHold.getStartTimeInfo().getMovieTime());
			}
		}

		if (onset != null && ndOnset != null) {
			ndOnset.getStartTimeInfo().setMovieTime(
					onset.getStartTimeInfo().getMovieTime());
		}

		if (ndFinalHold != null) {
			ndFinalHold.getStartTimeInfo().setMovieTime(
					ndValue.getEndTimeInfo().getMovieTime());
		} else if (ndOffset != null) {
			ndOffset.getStartTimeInfo().setMovieTime(
					ndValue.getEndTimeInfo().getMovieTime());
		}

		if (finalHold != null && ndFinalHold != null) {
			ndFinalHold.getEndTimeInfo().setMovieTime(
					finalHold.getEndTimeInfo().getMovieTime());
			if (ndOffset != null) {
				ndOffset.getStartTimeInfo().setMovieTime(
						finalHold.getEndTimeInfo().getMovieTime());
			}
		}

		if (offset != null && ndOffset != null) {
			ndOffset.getEndTimeInfo().setMovieTime(
					offset.getEndTimeInfo().getMovieTime());
		}

		if (ndEntity.isLockedGlossPhonAlignments()) {

			ChainedEventsEntity depHandEntity = ndEntity
					.getDependentHandEntity();
			HandShapeEntity depHandShapeEntity = ndEntity
					.getDependentHandShapeEntity();

			int startTime = ndEntity.getValueOrTextValueEvent().getStartTimeInfo().getMovieTime();
			int endTime = ndEntity.getValueOrTextValueEvent().getEndTimeInfo().getMovieTime();

			SignStreamSegmentPanel panel = SS3Singleton.getSignStreamApplication()
					.getSignStreamSegmentPanel();
			
			if (initialHold != null) {
				if (ndInitialHold != null) {
					ndInitialHold.getStartTimeInfo().setMovieTime(
							initialHold.getStartTimeInfo().getMovieTime());
					ndInitialHold.getEndTimeInfo().setMovieTime(
							initialHold.getEndTimeInfo().getMovieTime());
				} else {
					ndInitialHold = new ChainedEvent(panel);
					ndInitialHold.setStartTimeInfo(initialHold
							.getStartTimeInfo().cloneTimeInfo());
					ndInitialHold.setEndTimeInfo(initialHold.getEndTimeInfo()
							.cloneTimeInfo());
					ndEntity.setInitialHoldEvent(ndInitialHold);
				}
				
				startTime = initialHold.getStartTimeInfo().getMovieTime();
			} else {
				ndEntity.setInitialHoldEvent(null);
			}
			
			if (finalHold != null) {
				if (ndFinalHold != null) {
					ndFinalHold.getStartTimeInfo().setMovieTime(
							finalHold.getStartTimeInfo().getMovieTime());
					ndFinalHold.getEndTimeInfo().setMovieTime(
							finalHold.getEndTimeInfo().getMovieTime());
				} else {
					ndFinalHold = new ChainedEvent(panel);
					ndFinalHold.setStartTimeInfo(finalHold.getStartTimeInfo()
							.cloneTimeInfo());
					ndFinalHold.setEndTimeInfo(finalHold.getEndTimeInfo()
							.cloneTimeInfo());
					ndEntity.setFinalHoldEvent(ndFinalHold);
				}
				
				endTime = finalHold.getEndTimeInfo().getMovieTime();
			} else {
				ndEntity.setFinalHoldEvent(null);
			}
			
			// NOTE: we are not setting start/end times according to onsets/offsets as the handshapes do not track to those times

			if (depHandEntity != null) {
				ChainedEventsEntity prevEntity = depHandEntity
						.getPreviousEntity();
				ChainedEventsEntity nextEntity = depHandEntity.getNextEntity();

				boolean possibleToUpdateStartTime = prevEntity == null 
						|| (prevEntity != null && prevEntity.getEndTimeInfo().getMovieTime() <= startTime);
				boolean possibleToUpdateEndTime = nextEntity == null 
						| (nextEntity != null && nextEntity.getStartTimeInfo().getMovieTime() >= endTime);
				
				if (possibleToUpdateStartTime && possibleToUpdateEndTime) {
					depHandEntity.getValueOrTextValueEvent().getStartTimeInfo()
							.setMovieTime(startTime);
					depHandEntity.getValueOrTextValueEvent().getEndTimeInfo()
							.setMovieTime(endTime);

					if (depHandShapeEntity != null) {
						depHandShapeEntity.setStartTime(startTime);
						depHandShapeEntity.setEndTime(endTime);
					}
				} else {
					// warning!!!
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CANT_LOCK_GLOSSES);
					return;
				}
			}
		}
	}
}
