package edu.bu.signstream.grepresentation.fields.presentation;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class PresentationField extends Field {

	private static final long serialVersionUID = 1L;

	private ArrayList<PresentationEventsEntity> collection = new ArrayList<PresentationEventsEntity>();
	private NewEvent event;
	private String label = "";
	private String objectID = ""; 
	
	public static int DISPLAY_TIME_PERIOD = 0;
	public static int PARTITION = 1;
	public static int PARTITION_SEGMENT = 2;
	
	private int loadedFieldType = 0;
	
	private SS3TemporalPartition temporalPartition = null;
	private SS3GlossWindowSegment glossWindowSegment = null;
	
	public PresentationField(SS3TemporalPartition temporalPartition, SS3GlossWindowSegment glossWindowSegment, 
			SignStreamSegmentPanel segmentPanel) {
		super(segmentPanel, null);	// PresentationField should not have a parentField, so pass in null
		this.label = glossWindowSegment.getLabel();
		this.glossWindowSegment = glossWindowSegment;
		this.temporalPartition = temporalPartition;
		
		this.loadedFieldType = PresentationField.DISPLAY_TIME_PERIOD;

		String id = glossWindowSegment.getID();
		if (id == null || id.length() == 0)
			glossWindowSegment.setID("" + Util.getUniqueNumber());
		
		this.objectID = id;

		ArrayList<SS3GlossWindowItem> glossWindowItems = glossWindowSegment.getSS3GlossWindowItems();
		for (SS3GlossWindowItem glossWindowItem : glossWindowItems) {
			PresentationEvent event = new PresentationEvent(glossWindowItem, segmentPanel);
			PresentationEventsEntity entity = new PresentationEventsEntity(event, segmentPanel, this);
			collection.add(entity);
			
			Collections.sort(collection, new Comparator<PresentationEventsEntity>() {
				@Override
				public int compare(PresentationEventsEntity o1, PresentationEventsEntity o2) {
					int startTime1 = o1.getStartTimeInfo().getMovieTime();
					int startTime2 = o2.getStartTimeInfo().getMovieTime();
					
					return startTime1 - startTime2;
				}			
			});
		}
	}
	
	public PresentationEventsEntity getPreviousEntity(int startTime) {
		PresentationEventsEntity sEntity = null;
		ArrayList<PresentationEventsEntity> lEntities = new ArrayList<PresentationEventsEntity>();
		
		for (PresentationEventsEntity ent : collection) {
			
			if (ent.getEndTimeInfo() != null && startTime >= ent.getEndTimeInfo().getMovieTime())
				lEntities.add(ent);
		}
		
		if (lEntities.size() == 0)
			return sEntity;
		
		sEntity = lEntities.get(0);
		
		for (int j = 0; j < lEntities.size(); j++) {
			PresentationEventsEntity ent2 = lEntities.get(j);
			if (sEntity.getEndTimeInfo().getMovieTime() < ent2.getEndTimeInfo()
				.getMovieTime())
				sEntity = ent2;
		}
		return sEntity;
	}
	
	public PresentationEventsEntity getNextEntity(int endTime) {
		PresentationEventsEntity sEntity = null;
		ArrayList<PresentationEventsEntity> rEntities = new ArrayList<PresentationEventsEntity>();

		for(PresentationEventsEntity ent : collection) {	
			
			if (ent.getStartTimeInfo() != null && endTime <= ent.getStartTimeInfo().getMovieTime())
				rEntities.add(ent);
		}
		
		if (rEntities.size() == 0)
			return null;
		
		sEntity = rEntities.get(0);
		
		for (int j = 0; j < rEntities.size(); j++) {
			PresentationEventsEntity ent2 = rEntities.get(j);
			if (sEntity.getStartTimeInfo().getMovieTime() > ent2
				.getStartTimeInfo().getMovieTime())
				sEntity = ent2;
		}
		
		return sEntity;
	}
	
	public PresentationEventsEntity getEntityWithStartTime(int movieTime) {
		for(PresentationEventsEntity entity : collection) {
			if(entity.getStartTimeInfo().getMovieTime() == movieTime) {
				return entity;
			}
		}
		return null;
	}
	
	public PresentationEventsEntity getEntityWithEndTime(int movieTime) {
		for(PresentationEventsEntity entity : collection) {
			if(entity.getEndTimeInfo().getMovieTime() == movieTime) {
				return entity;
			}
		}
		return null;
	}
	
	public SS3TemporalPartition getTemporalPartition() {
		return temporalPartition;
	}
	
	public SS3GlossWindowSegment getSS3GlossWindowSegment() { 
		return glossWindowSegment;
	}

	public PresentationField(SS3TemporalPartition temporalPartition,
			SignStreamSegmentPanel segmentPanel) {
		super(segmentPanel, null);	// PresentationField should not have a parentField, so pass in null
		this.label = temporalPartition.getLabel();
		this.temporalPartition = temporalPartition;
		this.glossWindowSegment = temporalPartition.getGlossWindowSegment();
		
		this.loadedFieldType = PresentationField.PARTITION;

		String id = temporalPartition.getID();
		if (id == null || id.length() == 0)
			temporalPartition.setID("" + Util.getUniqueNumber());
		
		this.objectID = id;

		ArrayList<SS3GlossWindowItem> items = glossWindowSegment.getSS3GlossWindowItems();
		for (SS3GlossWindowItem item : items) {
			PresentationEvent event = new PresentationEvent(item, segmentPanel);
			PresentationEventsEntity entity = new PresentationEventsEntity(
					event, segmentPanel, this);
					
			collection.add(entity);
			Collections.sort(collection, new Comparator<PresentationEventsEntity>() {
				@Override
				public int compare(PresentationEventsEntity o1, PresentationEventsEntity o2) {
					int startTime1 = o1.getStartTimeInfo().getMovieTime();
					int startTime2 = o2.getStartTimeInfo().getMovieTime();
					
					return startTime1 - startTime2;
				}			
			});
		}
	}

	public void deleteAllEntities() {
		collection.clear();
	}

	public boolean isEventLabelUnique(String label) {
		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);
			PresentationEvent event = entity.getEvent();
			if (event.getText().equals(label))
				return false;
		}
		return true;
	}

	public ArrayList<SS3GlossWindowItem> getSS3GlossWindowItems() {
		ArrayList<SS3GlossWindowItem> statements = new ArrayList<SS3GlossWindowItem>();
		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);
			statements.add(entity.getEvent().getSS3GlossWindowItem());
		}
		return statements;
	}
	
	@Override
	public ArrayList getEntities() {
		return collection;
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {		
		collection.add((PresentationEventsEntity) entity);
		Collections.sort(collection, new Comparator<PresentationEventsEntity>() {
			@Override
			public int compare(PresentationEventsEntity o1, PresentationEventsEntity o2) {
				int startTime1 = o1.getStartTimeInfo().getMovieTime();
				int startTime2 = o2.getStartTimeInfo().getMovieTime();
				
				return startTime1 - startTime2;
			}			
		});
	}
	
	public void removeAllEntities() {
		collection.clear();
	}

	@Override
	public EventsEntity getEntityWithRefID(String refEntityId) {
		return null;
	}

	@Override
	public PresentationEventsEntity getEntity(String entityID) {
		for (PresentationEventsEntity entity: collection) {
			PresentationEvent event = entity.getEvent();
			
			if (entityID.equals(event.getID())) {
				return entity;
			}
		}
		
		return null;
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		// to be implemented when needed
	}

	public void setNewEvent(NewEvent event) {
		this.event = event;
	}

	public NewEvent getNewEvent() {
		return event;
	}

	@Override
	public int getFieldSize() {
		return collection.size();
	}

	@Override
	public String getName() {
		return label;
	}

	@Override
	public String getFieldID() {
		if (track != null)
			return track.getFieldId();
		if (ss3Track != null)
			return ss3Track.getId();
		return null;
	}

	@Override
	public Event selectEvent(int x) {
		Event event = getEventAt(x);
		if (event != null)
			event.select();
		return event;
	}

	@Override
	public boolean contains(int y) {		
		return y >= Y - 14 && y <= Y + 5;
	}

	// removes deleted entities from collection
	public void clean() {
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		super.paint(g2d, Y, font);

		if (event != null) {
			segmentPanel.getPainter().drawNewEvent(event, g2d, Y);
		}

		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);
			entity.paint(g2d, Y, font);
		}
	}

	@Override
	public int getStartTime() {
		int startTime = 0;
		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);
			startTime = startTime < entity.getStartTime() ? startTime : entity
					.getStartTime();
		}
		return startTime;
	}

	@Override
	public void deleteEntity(EventsEntity entity) {
		collection.remove(entity);
	}

	@Override
	public int getEndTime() {
		int endTime = 0;
		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);
			endTime = endTime > entity.getEndTime() ? endTime : entity
					.getEndTime();
		}
		return endTime;
	}

	@Override
	public int getStartTimeCoordinate() {
		return 0;
	}

	@Override
	public int getEndTimeCoordinate() {
		int endCoord = 0;
		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);
			endCoord = endCoord > entity.getEndTimeCoordinate() ? endCoord
					: entity.getEndTimeCoordinate();
		}
		return endCoord;
	}

	@Override
	public void unselect() {
		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);
			entity.unselect();
		}
	}

	@Override
	public void deleteEmptyEntities() {
	}

	@Override
	public Event getSelectedEvent() {
		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);

			if (entity.isSelected())
				return entity.getEvent();
		}
		return null;
	}

	@Override
	public EventsEntity getSelectedEntity() {
		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);

			if (entity.isSelected())
				return entity;
		}
		return null;
	}

	@Override
	public Event getEventAt(int x) {
		for (int i = 0; i < collection.size(); i++) {
			PresentationEventsEntity entity = collection
					.get(i);
			if (entity.contains(x))
				return entity;
		}
		return null;
	}
	
	public Event getFirstEventAfter(int x) {
		int size = collection.size();
		PresentationEventsEntity firstEntity = null;
		
		if (size > 0)
			firstEntity = collection.get(0);
		else
			return null;
		
		int firstEntityEndCoord = firstEntity.getEndTimeCoordinate();
		if (x <= firstEntityEndCoord)
			return firstEntity;
		
		for (int i = 0; i < size - 1; i++) {
			PresentationEventsEntity entity = collection.get(i);
			
			// inclusive bounds check inside entity
			if (entity.contains(x))
				return entity;

			PresentationEventsEntity nextEntity = collection.get(i + 1);
			
			if (nextEntity.contains(x))
				return nextEntity;
			
			int entityEndCoord = entity.getEndTimeCoordinate();
			int nextEntityStartCoord = nextEntity.getStartTimeCoordinate();
			
			// exclusive bounds check between entities
			if (entityEndCoord < x && x < nextEntityStartCoord)
				return nextEntity;
		}
		
		return null;
	}

	@Override
	public boolean isEmptyInterval(int startCoord, int endCoord) {
		Line2D.Double line = new Line2D.Double(startCoord, 0, endCoord, 0);
				
		for(PresentationEventsEntity entity:collection) {
			Line2D.Double l = new Line2D.Double(entity.getStartTimeCoordinate(), 0, entity.getEndTimeCoordinate(), 0);
			if(line.intersectsLine(l)) {
				return false;
			}
		}
		return true;
	}

	@Override
	public EventsEntity getEntityAt(int x) {
		return null;
	}

	public boolean setSelectedEventStartTime(int time) {
		return false;
	}

	public boolean setSelectedEventEndTime(int time) {
		return false;
	}
}
