package edu.bu.signstream.grepresentation.fields.rightLeftField;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class RightLeftHandField extends Field {

	private ArrayList<ChainedEventsEntity> entities = new ArrayList<ChainedEventsEntity>();

	protected String segmentID = "";

	public RightLeftHandField(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.segmentPanel = segmentPanel;
	}

	@Override
	public ArrayList getEntities() {
		return entities;
	}

	@Override
	public EventsEntity getEntityWithRefID(String refEntityID) {
		if (refEntityID != null)
			return null;

		for (ChainedEventsEntity entity : entities) {
			if (refEntityID.equalsIgnoreCase(entity.getRefEntityId())) {
				return entity;
			}
		}
		return null;
	}

	@Override
	public EventsEntity getEntity(String entityID) {
		for (ChainedEventsEntity entity : entities) {
			if (entity.getID().equalsIgnoreCase(entityID)) {
				return entity;
			}
		}
		return null;
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		EventsEntity entity = getEntityWithRefID(refID);
		if (entity != null)
			entity.highlight(highlight);
	}

	@Override
	public Event getEventAt(int x) {
		ChainedEventsEntity entity = (ChainedEventsEntity) getEntityAt(x);
		if (entity == null)
			return null;

		return entity.getChainedEventsCollection().getEventAt(x);
	}

	@Override
	public EventsEntity getEntityAt(int coordX) {
		for (ChainedEventsEntity entity : entities) {
			if (entity.contains(coordX))
				return entity;
		}
		return null;
	}

	public void deleteEntity(ChainedEventsEntity entity) {
		ChainedEventsEntity previousEntity = entity.getPreviousEntity();
		ChainedEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		entities.remove(entity);
	}

	@Override
	public int getFieldSize() {
		return entities.size();
	}

	@Override
	public void deleteEmptyEntities() {
		ArrayList<ChainedEventsEntity> entitiesToDelete = new ArrayList<ChainedEventsEntity>();

		for (ChainedEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null
					|| entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (ChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}

	@Override
	public boolean isEmptyInterval(int startTime, int endTime) {
		ArrayList<ChainedEventsEntity> list = new ArrayList<ChainedEventsEntity>();

		segmentPanel.getZoomer().getCoordinateTimeConvertor();

		int X1 = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(startTime);
		int X2 = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(endTime);

		for (ChainedEventsEntity entity : entities) {
			if ((entity.isEntityInsideElement(X1, X2))
					|| (entity.contains(X1) || entity.contains(X2))) {
				list.add(entity);
			}
		}

		return list.isEmpty();
	}

	@Override
	public void addEventsEntity(EventsEntity genEentity) {

		ChainedEventsEntity entity = (ChainedEventsEntity) genEentity;
		int startMovieTime = entity.getStartTimeInfo().getMovieTime();

		ChainedEventsEntity previousEntity = null;
		ChainedEventsEntity nextEntity = null;

		for (ChainedEventsEntity ent : entities) {
			int entMovieStartTime = ent.getStartTimeInfo().getMovieTime();

			if (entMovieStartTime <= startMovieTime) {
				if (previousEntity == null) {
					previousEntity = ent;
				} else if (entMovieStartTime > previousEntity
						.getStartTimeInfo().getMovieTime()) {
					previousEntity = ent;
				}
			} else {
				if (nextEntity == null) {
					nextEntity = ent;
				} else if (entMovieStartTime < nextEntity.getStartTimeInfo()
						.getMovieTime()) {
					nextEntity = ent;
				}
			}
		}

		if (previousEntity != null) {
			ChainedEventsEntity next = previousEntity.getNextEntity();
			if (next != null) {
				entity.setNextEntity(next);
				next.setPreviousEntity(entity);
			}
			previousEntity.setNextEntity(entity);
			entity.setPreviousEntity(previousEntity);

		} else if (nextEntity != null) {
			entity.setNextEntity(nextEntity);
			nextEntity.setPreviousEntity(entity);
		}

		entities.add(entity);
	}

	@Override
	public int getStartTime() {
		removeEmptyEntities();
		int startTime = 0;
		for (ChainedEventsEntity entity : entities) {
			startTime = startTime < entity.getStartTimeInfo().getMovieTime() ? startTime
					: entity.getStartTimeInfo().getMovieTime();
		}
		return startTime;
	}

	@Override
	public int getEndTime() {
		removeEmptyEntities();
		int endTime = 0;
		for (ChainedEventsEntity entity : entities) {
			endTime = endTime > entity.getEndTimeInfo().getMovieTime() ? endTime
					: entity.getEndTimeInfo().getMovieTime();
		}
		return endTime;
	}

	public void removeEmptyEntities() {
		ArrayList<ChainedEventsEntity> entitiesToDelete = new ArrayList<ChainedEventsEntity>();

		for (ChainedEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null
					|| entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (ChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}

	@Override
	public void unselect() {
		for (ChainedEventsEntity entity : entities) {
			entity.unselect();
		}
	}

	@Override
	public int getStartTimeCoordinate() {
		int time = getStartTime();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeToCoordinate(time);
	}

	@Override
	public int getEndTimeCoordinate() {
		int time = getEndTime();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeToCoordinate(time);
	}

	@Override
	public Event getSelectedEvent() {
		ChainedEventsEntity selectedEntity = (ChainedEventsEntity) getSelectedEntity();
		if (selectedEntity != null) {
			return selectedEntity.getSelectedEvent();
		}
		return null;
	}

	@Override
	public EventsEntity getSelectedEntity() {
		for (ChainedEventsEntity entity : entities) {
			if (entity.getSelectedEvent() != null)
				return entity;
		}
		return null;
	}

	@Override
	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

	@Override
	public void setSegmentID(String segmentID) {
		this.segmentID = segmentID;
	}

	@Override
	public String getSegmentID() {
		return segmentID;
	}

	@Override
	public String getFieldID() {
		if (track != null)
			return track.getFieldId(); // fieldID;
		if (ss3Track != null)
			return ss3Track.getId();
		return null;
	}

	// update creation && update time here
	@Override
	public SS3Track getSS3Track() {
		if (ss3Track != null)
			ss3Track.removeEntities();

		for (ChainedEventsEntity entity : entities) {
			ss3Track.addEntity(entity.getSS3Entity());
		}

		return ss3Track;
	}

	@Override
	public int getY() {
		return (int) Y;
	}

	@Override
	public Event selectEvent(int x) {
		Event event = getEventAt(x);
		if (event != null)
			event.select();
		return event;
	}

	@Override
	public boolean contains(int y) {
		int y1 = (int) (Y - SegmentGraphReprUtil.cellHeight);
		int y2 = (int) (Y + SegmentGraphReprUtil.valueHeight);
		return y1 <= y && y <= y2;
	}

	@Override
	public Color getColor() {
		return color;
	}

	@Override
	public void setColor(Color color) {
		this.color = color;
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		super.paint(g2d, Y, font);
		for (ChainedEventsEntity entity : entities) {
			entity.paint(g2d, Y, font);
		}
	}

	@Override
	public void deleteEntity(EventsEntity entityToDelete) {
		ChainedEventsEntity entity = (ChainedEventsEntity) entityToDelete;

		ChainedEventsEntity previousEntity = entity.getPreviousEntity();
		ChainedEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		entities.remove(entity);
	}
}
