package edu.bu.signstream.grepresentation.listener;

import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.view.GlossFieldsLabelView;
import edu.bu.signstream.grepresentation.view.NonManualFieldsLabelView;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.Slider;

public abstract class SignStreamMouseListener
   implements MouseListener, MouseMotionListener {

   protected SignStreamSegmentPanel segmentPanel;

   protected SegmentGraphicRepresentation nonManualRep;

   protected SegmentGraphicRepresentation glossRep;

   protected NonManualFieldsLabelView nmLabelView;

   protected GlossFieldsLabelView glsLabelView;

   protected Slider slider;

   public SignStreamMouseListener(
      SignStreamSegmentPanel segmentPanel,
      SegmentGraphicRepresentation nonManualRep,
      SegmentGraphicRepresentation glossRep,
      NonManualFieldsLabelView nmLabelView,
      GlossFieldsLabelView glsLabelView) {

      this.segmentPanel = segmentPanel;
      this.nonManualRep = nonManualRep;
      this.glossRep = glossRep;
      this.nmLabelView = nmLabelView;
      this.glsLabelView = glsLabelView;
      this.slider = segmentPanel.getSlider();
   }
   
   public boolean minifyAllHandShapeIcons(FieldWrapper wrapper) {
	      boolean minified = false;
	      try {
	         ArrayList<EventsEntity> entities = wrapper.getField().getEntities();
	         for (int i = 0; i < entities.size(); i++) {
	            HandShapeEntity entity = (HandShapeEntity) entities.get(i);
	            if (entity.minifyAllIcons()) {
	               minified = true;
	            }
	         }
	      } catch (NullPointerException e) {
	         ;
	      }
	      return minified;
	   }

   protected void unselectAll() {
	   if(nonManualRep != null)
		   nonManualRep.getFieldWrappersCollection().unselectAll();
      
      if (glossRep != null) 
            glossRep.getFieldWrappersCollection().unselectAll();
      
      SS3Singleton.getMediaToolBar(segmentPanel).setSelectedEntity(null);
      SS3Singleton.getMediaToolBar(segmentPanel).setSelectedEvent(null);
   }

   protected void repaintAll() {
	   if(nonManualRep != null) {
		   nonManualRep.repaint();
	   }
	   if(glossRep != null) {
		   glossRep.repaint();
	   }
   }

   protected abstract void highlightLabel(FieldWrapper wrapper);

   protected boolean isCtrlPressed(MouseEvent e) {
      return (e.getModifiers() & InputEvent.CTRL_MASK) == InputEvent.CTRL_MASK;
   }
   
   protected boolean isShiftPressed(MouseEvent e) {
	   boolean pressed =  (e.getModifiers() & InputEvent.SHIFT_DOWN_MASK) == InputEvent.SHIFT_DOWN_MASK;
	   if(pressed) {
		   return pressed;
	   } else { 
		 return  (e.getModifiers() & InputEvent.SHIFT_MASK) == InputEvent.SHIFT_MASK;
	   }
   }  
   
   /*The Meta key is a special key on MIT keyboards, such as the Space-cadet keyboard, 
    * and on Sun Microsystems keyboards, marked as a solid diamond.
	* The key may be considered equivalent to the Macintosh's command key, which has the same location and similar function. 
	* On keyboards that lack it, meta may be emulated with the Alt key or with the Windows key. 
	* Alternatively, some programs (prominent example being Emacs) allow to emulate meta by pressing and releasing the Esc key.
    * */
   protected boolean isCommandPressed(MouseEvent e) {
	   boolean pressed =  (e.getModifiers() & InputEvent.META_DOWN_MASK) == InputEvent.META_DOWN_MASK;
	   if(pressed) {
		   return pressed;
	   } else { 
		 return  (e.getModifiers() & InputEvent.META_MASK) == InputEvent.META_MASK;
	   }
   }

   protected boolean isAltPressed(MouseEvent e) {
      return (e.getModifiers() & InputEvent.ALT_MASK) == InputEvent.ALT_MASK;
   }
}
