package edu.bu.signstream.grepresentation.reguestsHandler;

import java.awt.geom.Line2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3StatementField;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntityCollection;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeSubEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.rightLeftField.RightLeftHandFieldWrapper;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.RootLabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class RequestHandler {

    private SS3Database database;
    private SignStreamSegmentPanel segmentPanel;
    private FieldsFactory factory;
    
    public void init(SignStreamSegmentPanel segmentPanel) {
    		this.segmentPanel = segmentPanel;
    		this.factory = new FieldsFactory();
    		this.database = segmentPanel.getSS3Database();
    }
    
    // take a panel object with the Segment objects already loaded and load utterance
    public void loadPredefinedPanelSelection() {
		segmentPanel.loadUtterance(segmentPanel.getSegments());
    }
	
	public Segment convertMultipleItemsAndAddToSegmentPanel(ArrayList<SS3GlossWindowItem> items, 
			PresentationField parentField) {
		
		Segment segment = convertItemsToSegment(items, parentField);
		segmentPanel.getSegments().add(segment);
		
		return segment;
	}
	
	public Segment convertMultipleItemsAndAddToSegmentPanelAtIndex(ArrayList<SS3GlossWindowItem> items, 
			PresentationField parentField, int indexToInsert) {
		
		Segment segment = convertItemsToSegment(items, parentField);
		
		if (indexToInsert > -1 && indexToInsert <= segmentPanel.getSegments().size())
			segmentPanel.getSegments().add(indexToInsert, segment);
		
		return segment;
	}
	
	public Segment convertItemsToSegment(ArrayList<SS3GlossWindowItem> items, PresentationField parentField) {	
		
		// check if we are adding the same segment tier (parentField) as an existing one
		// because, if so, just make a duplicate reference to it, so any changes made in one reflect in the other
		if ( !segmentPanel.getSegments().isEmpty()) {
			for (Segment seg: segmentPanel.getSegments()) {
				if (seg.getParentField().equals(parentField)) {
					Segment segment = seg;
					return segment;
				}
			}
		}
		
		RootLabelObject root = getCodingSchemaAsTreeNode(items, parentField);
		Segment segment = new Segment(segmentPanel, parentField);
		segment.setSS3GlossWindowItems(items);
		segment.setRootLabelObject(root);
		
		return segment;
	}
	
	private RootLabelObject getCodingSchemaAsTreeNode(ArrayList<SS3GlossWindowItem> items, PresentationField parentField) {
		
        RootLabelObject root = new RootLabelObject();
        LabelObject nonManObject = new LabelObject("", "");
        LabelObject glossNRelatedObject = new LabelObject("", "");
        root.setNonManuelLabelObject(nonManObject);
        root.setGlossNRelatedLabelObject(glossNRelatedObject);
        SS3CodingScheme schema = SS3Singleton.getSS3CodingSchemeCollection()
                .getDefaultCodingScheme();

        int rowNumber = 1;
        
        // add non-manual fields
        ArrayList<SS3Field> nmFields = schema
                .getFields(Constants.TYPE_NON_MANUAL);

        for (SS3Field nmField : nmFields) {
            addField(nmField, rowNumber, nonManObject, false, items, parentField);     
            rowNumber++;
        }

        rowNumber = 1;

        GlossFieldWrapper dGlsWrapper = null;
        GlossFieldWrapper ndGlsWrapper = null;
        RightLeftHandFieldWrapper rhWrapper = null;
        RightLeftHandFieldWrapper lhWrapper = null;
        HandShapeFieldWrapper rHsWrapper = null;
        HandShapeFieldWrapper lHsWrapper = null;

        // add dominant gloss and related fields
        SS3Field dGlsField = schema.getField(schema
                .getFieldID(Constants.TYPE_DOM_GLOSS));
        
        LabelObject dGlsObj = addField(dGlsField, rowNumber, glossNRelatedObject, false, items, parentField);
        dGlsWrapper = (GlossFieldWrapper) dGlsObj.getWrapper();
        rowNumber++;

        // add nondominant gloss and related fields
        SS3Field ndGlsField = schema.getField(schema
                .getFieldID(Constants.TYPE_NON_DOM_GLOSS));
        
        LabelObject ndGlsObj = addField(ndGlsField, rowNumber, glossNRelatedObject, false, items, parentField);
        ndGlsWrapper = (GlossFieldWrapper) ndGlsObj.getWrapper();
        rowNumber++;

        boolean visible = ndGlsWrapper.getField().getEntities().size() > 0;
        ndGlsObj.setVisible(visible);

        //if (ndGlsField != null) {
        // add Literal Translation
        SS3Field ndLtField = schema.getField(schema
                .getFieldID(Constants.TYPE_NON_DOM_TEXT));
        
        SS3Track track = database.getTrack(ndLtField.getId(), items);
        
        ArrayList<SS3Entity> entities = track.getEntities();

        if (entities.size() == 1) {
            SS3Entity ent = entities.get(0);
            ArrayList<SS3Item> entItems = ent.getItems();
            if (entItems.size() == 1) {
                SS3Item itm = entItems.get(0);
                String text = itm.getText();

                if (text.indexOf(" ") > 0) {
                    // this data should be in English Translation field.
                    // Remove it from here.

                    track.removeEntities();
                    track.removeEntities();
                    track.clear();
                }
            }
        }

        // set dependency, gloss entities
        if (dGlsWrapper != null && ndGlsWrapper != null) {
            GlossField glsField = (GlossField) dGlsWrapper.getField();

            GlossChainedEventsEntityCollection collection = glsField
                    .getGlossChainedEventsEntityCollection();
            ArrayList<GlossChainedEventsEntity> glsEntities = collection
                    .getEntities();

            for (GlossChainedEventsEntity cEntity : glsEntities) {
                String entID = cEntity.getDependentGlossEntityID();

                GlossField glsFld = ndGlsWrapper.getGlossField();

                GlossChainedEventsEntityCollection glsCollection = glsFld
                        .getGlossChainedEventsEntityCollection();
                EventsEntity ndEntity = glsCollection.getEntity(entID);

                if (ndEntity != null) {
                    GlossChainedEventsEntity ndcEntity = (GlossChainedEventsEntity) ndEntity;
                    cEntity.setDependentGlossEntity(ndcEntity);
                    ndcEntity.setDependentGlossEntity(cEntity);
                }
            }
        }

        /**
         * right hand
         * */
        // add right hand
        SS3Field rhField = schema.getField(schema
                .getFieldID(Constants.TYPE_RIGHT_HAND));
        LabelObject rhObj = addField(rhField, rowNumber, glossNRelatedObject, false, items, parentField);
        // RightLeftHandFieldWrapper
        rhWrapper = (RightLeftHandFieldWrapper) rhObj.getWrapper();
        rowNumber++;

        if (rhObj != null) {
            // add right hand shape
            SS3Field rhShapeField = schema.getField(schema
                    .getFieldID(Constants.TYPE_RIGHT_HAND_SHAPE));
            LabelObject rhShapeObj = addField(rhShapeField, rowNumber, rhObj, true, items, parentField);

            rHsWrapper = (HandShapeFieldWrapper) rhShapeObj.getWrapper();
            rowNumber++;
        }

        /**
         * left hand
         */
        // add left hand
        SS3Field lhField = schema.getField(schema
                .getFieldID(Constants.TYPE_LEFT_HAND));
        LabelObject lhObj = addField(lhField, rowNumber, glossNRelatedObject, false, items, parentField);
        lhWrapper = (RightLeftHandFieldWrapper) lhObj.getWrapper();
        rowNumber++;

        if (lhObj != null) {
            // add left hand shape
            SS3Field lhShapeField = schema.getField(schema
                    .getFieldID(Constants.TYPE_LEFT_HAND_SHAPE));
            LabelObject lhShapeObj = addField(lhShapeField, rowNumber, lhObj, true, items, parentField);
            lHsWrapper = (HandShapeFieldWrapper) lhShapeObj.getWrapper();
            rowNumber++;
        }

        setGlossAndRelated(dGlsWrapper, ndGlsWrapper, rhWrapper, rHsWrapper);
        setGlossAndRelated(ndGlsWrapper, dGlsWrapper, lhWrapper, lHsWrapper);

        // add free text
        SS3Field ftField1 = schema.getField("31");
        SS3Field ftField2 = schema.getField("20001");

        if (ftField2 != null && ftField2.getValuesAsList().size() > 0) {
            LabelObject obj = addField(ftField2, rowNumber, glossNRelatedObject, false, items, parentField);
            obj.setVisible(obj.getWrapper().getField().getEntities().size() > 0);
            rowNumber++;
        } else if (ftField1 != null) {
            LabelObject obj = addField(ftField1, rowNumber, glossNRelatedObject, false, items, parentField);
            obj.setVisible(obj.getWrapper().getField().getEntities().size() > 0);
            rowNumber++;
        }
        SS3Field etField = schema.getField("30000");
        LabelObject obj = addField(etField, rowNumber, glossNRelatedObject, false, items, parentField);
        obj.setVisible(true);

        setDependency(dGlsWrapper, ndGlsWrapper, rhWrapper, lhWrapper, rHsWrapper, lHsWrapper);

        return root;
	}

    private RootLabelObject getCodingSchemaAsTreeNode(SS3GlossWindowItem item, PresentationField parentField) {

        RootLabelObject root = new RootLabelObject();
        LabelObject nonManObject = new LabelObject("", "");
        LabelObject glossNRelatedObject = new LabelObject("", "");
        root.setNonManuelLabelObject(nonManObject);
        root.setGlossNRelatedLabelObject(glossNRelatedObject);
        SS3CodingScheme schema = SS3Singleton.getSS3CodingSchemeCollection()
                .getDefaultCodingScheme();

        int rowNumber = 1;

        int itemStartTime = item.getStartTimeInfo().getMovieTime();
        int itemEndTime = item.getEndTimeInfo().getMovieTime();

        // add non-manual fields
        ArrayList<SS3Field> nmFields = schema
                .getFields(Constants.TYPE_NON_MANUAL);

        for (SS3Field nmField : nmFields) {
            String fieldID = nmField.getId();

            boolean isVisible = false;

            SS3StatementField statementField = item
                    .getSS3StatementField(fieldID);

            // validate values
            if (statementField != null) {
            		ArrayList<SS3Entity> ss3Entities = statementField.getSS3Entities();
            		isVisible = ss3Entities.size() > 0;
            }

            addField(nmField, statementField, rowNumber, nonManObject,
                    isVisible, false, itemStartTime, itemEndTime, parentField);
        }

        rowNumber = 1;

        GlossFieldWrapper dGlsWrapper;
        GlossFieldWrapper ndGlsWrapper;
        RightLeftHandFieldWrapper rhWrapper;
        RightLeftHandFieldWrapper lhWrapper;
        HandShapeFieldWrapper rHsWrapper = null;
        HandShapeFieldWrapper lHsWrapper = null;

        // add dominant gloss and related fields
        SS3Field dGlsField = schema.getField(schema
                .getFieldID(Constants.TYPE_DOM_GLOSS));

        SS3StatementField field = item.getSS3StatementField(dGlsField.getId());

        LabelObject dGlsObj = addField(dGlsField, field, rowNumber,
                glossNRelatedObject, false, itemStartTime, itemEndTime, parentField);
        dGlsWrapper = (GlossFieldWrapper) dGlsObj.getWrapper();

        // add nondominant gloss and related fields
        SS3Field ndGlsField = schema.getField(schema
                .getFieldID(Constants.TYPE_NON_DOM_GLOSS));
        LabelObject ndGlsObj = addField(ndGlsField,
                item.getSS3StatementField(ndGlsField.getId()), rowNumber,
                glossNRelatedObject, false, itemStartTime, itemEndTime, parentField);

        ndGlsWrapper = (GlossFieldWrapper) ndGlsObj.getWrapper();

        boolean visible = ndGlsWrapper.getField().getEntities().size() > 0;
        ndGlsObj.setVisible(visible);

        // add Literal Translation
        SS3Field ndLtField = schema.getField(schema
                .getFieldID(Constants.TYPE_NON_DOM_TEXT));

        SS3StatementField sField = item.getSS3StatementField(ndLtField
                .getId());

        SS3Track track = database.getTrack(sField, ndLtField.getId(),
                itemStartTime, itemEndTime);
        ArrayList<SS3Entity> entities = track.getEntities();

        if (entities.size() == 1) {
            SS3Entity ent = entities.get(0);
            ArrayList<SS3Item> items = ent.getItems();
            if (items.size() == 1) {
                SS3Item itm = items.get(0);
                String text = itm.getText();

                if (text.indexOf(" ") > 0) {
                    // this data should be in English Translation field.
                    // Remove it from here.

                    track.removeEntities();
                    database.getTrack(sField, ndLtField.getId(),
                            itemStartTime, itemEndTime).removeEntities();
                    database.getTrack(sField, ndLtField.getId(),
                            itemStartTime, itemEndTime).clear();

                    sField.getSS3Entities().clear();
                }
            }
        }

        // set dependency, gloss entities
        if (dGlsWrapper != null && ndGlsWrapper != null) {
            GlossField glsField = (GlossField) dGlsWrapper.getField();

            GlossChainedEventsEntityCollection collection = glsField
                    .getGlossChainedEventsEntityCollection();
            ArrayList<GlossChainedEventsEntity> glsEntities = collection
                    .getEntities();

            for (GlossChainedEventsEntity cEntity : glsEntities) {
                String entID = cEntity.getDependentGlossEntityID();

                GlossField glsFld = ndGlsWrapper.getGlossField();

                GlossChainedEventsEntityCollection glsCollection = glsFld
                        .getGlossChainedEventsEntityCollection();
                EventsEntity ndEntity = glsCollection.getEntity(entID);

                if (ndEntity != null) {
                    GlossChainedEventsEntity ndcEntity = (GlossChainedEventsEntity) ndEntity;
                    cEntity.setDependentGlossEntity(ndcEntity);
                    ndcEntity.setDependentGlossEntity(cEntity);
                }
            }
        }

        /**
         * right hand
         * */
        // add right hand
        SS3Field rhField = schema.getField(schema
                .getFieldID(Constants.TYPE_RIGHT_HAND));
        LabelObject rhObj = addField(rhField,
                item.getSS3StatementField(rhField.getId()), rowNumber,
                glossNRelatedObject, false, itemStartTime, itemEndTime, parentField);
        // RightLeftHandFieldWrapper
        rhWrapper = (RightLeftHandFieldWrapper) rhObj.getWrapper();
        if (rhObj != null) {
            // add right hand shape
            SS3Field rhShapeField = schema.getField(schema
                    .getFieldID(Constants.TYPE_RIGHT_HAND_SHAPE));
            LabelObject rhShapeObj = addField(rhShapeField,
                    item.getSS3StatementField(rhShapeField.getId()), rowNumber,
                    rhObj, true, itemStartTime, itemEndTime, parentField);

            rHsWrapper = (HandShapeFieldWrapper) rhShapeObj.getWrapper();
        }

        /**
         * left hand
         */
        // add left hand
        SS3Field lhField = schema.getField(schema
                .getFieldID(Constants.TYPE_LEFT_HAND));
        LabelObject lhObj = addField(lhField,
                item.getSS3StatementField(lhField.getId()), rowNumber,
                glossNRelatedObject, false, itemStartTime, itemEndTime, parentField);
        lhWrapper = (RightLeftHandFieldWrapper) lhObj.getWrapper();

        if (lhObj != null) {
            // add left hand shape
            SS3Field lhShapeField = schema.getField(schema
                    .getFieldID(Constants.TYPE_LEFT_HAND_SHAPE));
            LabelObject lhShapeObj = addField(lhShapeField,
                    item.getSS3StatementField(lhShapeField.getId()), rowNumber,
                    lhObj, true, itemStartTime, itemEndTime, parentField);
            lHsWrapper = (HandShapeFieldWrapper) lhShapeObj.getWrapper();
        }

        setGlossAndRelated(dGlsWrapper, ndGlsWrapper, rhWrapper, rHsWrapper);
        setGlossAndRelated(ndGlsWrapper, dGlsWrapper, lhWrapper, lHsWrapper);

        // add free text
        SS3Field ftField1 = schema.getField("31");
        SS3Field ftField2 = schema.getField("20001");

        if (ftField2 != null && ftField2.getValuesAsList().size() > 0) {
            LabelObject obj = addField(ftField2,
                    item.getSS3StatementField(ftField2.getId()), rowNumber,
                    glossNRelatedObject, false, false, itemStartTime,
                    itemEndTime, parentField);
            obj.setVisible(obj.getWrapper().getField().getEntities().size() > 0);
        } else if (ftField1 != null) {
            LabelObject obj = addField(ftField1,
                    item.getSS3StatementField(ftField1.getId()), rowNumber,
                    glossNRelatedObject, false, false, itemStartTime,
                    itemEndTime, parentField);
            obj.setVisible(obj.getWrapper().getField().getEntities().size() > 0);
        }

        SS3Field etField = schema.getField("30000");
        SS3StatementField statementField = item.getSS3StatementField(etField
                .getId());
        LabelObject obj = addField(etField, statementField, rowNumber,
                glossNRelatedObject, false, false, itemStartTime, itemEndTime, parentField);
        obj.setVisible(true);

        setDependency(dGlsWrapper, ndGlsWrapper, rhWrapper, lhWrapper, rHsWrapper, lHsWrapper);

        return root;
    }


    private void setGlossAndRelated(GlossFieldWrapper glsWrp1,
                                    GlossFieldWrapper glsWrp2,
                                    RightLeftHandFieldWrapper handWrp,
                                    HandShapeFieldWrapper handShapeWrp)  {

        GlossChainedEventsEntityCollection glsCollection = glsWrp1
                .getGlossField().getGlossChainedEventsEntityCollection();

        if (glsCollection != null) {
            ArrayList<GlossChainedEventsEntity> handGlosses = glsCollection.getEntities();

            for (GlossChainedEventsEntity gloss : handGlosses) {

                setDependentGlossEntity(gloss, glsWrp2);

                setDependencies(gloss, handWrp, handShapeWrp);
            }
        }
    }

    private void setDependencies(GlossChainedEventsEntity gloss, RightLeftHandFieldWrapper handWrp, HandShapeFieldWrapper handShapeWrp) {
	    	String depHandID = gloss.getDependentHandEntityID();
	    	String depHandShapeID = gloss.getDependentHandShapeEntityID();
	
	    	// block concerning dependencies for hand ID (depHandID) and hand wrapper (handWrp)
	    	if (depHandID != null && !depHandID.isEmpty()) {
	    		EventsEntity hndEnt = handWrp.getField().getEntity(depHandID);
	
	    		// if hand entity was properly constructed and is in the wrapper already, set dependencies using that one
	    		if (hndEnt != null) {
	    			ChainedEventsEntity hndEntity = (ChainedEventsEntity) hndEnt;
	    			hndEnt.setRefEntityId(gloss.getID());
	    			gloss.setDependentHandEntity(hndEntity);
	    		} else {
	    			// else: gloss has hand entity ID saved, but there is not a corresponding hand entity object in database/track (and thus the wrapper)
	    			String fieldID = handWrp.getField().getFieldID();
	    			SS3Track hndTrack = database.getTrack(fieldID);
	
	    			ChainedEventsEntity handEntity = null;
	
	    			// try to re-create hand entity strictly from db track
	    			SS3Entity handSS3Entity = hndTrack.getSS3Entity(depHandID);
	    			if(handSS3Entity != null) {
	    				handEntity = createTrackEntity(handSS3Entity, handWrp.getField());
	    			}
	
	    			// if db recreation failed, make a new hand entity object and give it ID of depHandID
	    			if (handEntity == null) {
	    				ChainedEventsCollection collection = new ChainedEventsCollection();
	    				GlossChainedEvent glossTextValueEvent = (GlossChainedEvent) gloss.getTextValue();
	    				collection.setTextValueEvent(glossTextValueEvent.cloneGlossChainedEvent());
	    				handEntity = new ChainedEventsEntity(collection, segmentPanel, handWrp.getField());
	
	    				handEntity.setID(depHandID);
	    				handEntity.setField(handWrp.getField());
	    				handEntity.determineParentEntity();
	
	    				boolean isLockedGlossPhonAlignments = gloss.isLockedGlossPhonAlignments();
	    				handEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);
	
	    				handEntity.setStartTimeInfo(gloss.getStartTimeInfo());
	    				handEntity.setEndTimeInfo(gloss.getEndTimeInfo());
	    			}
	
	    			// add hand entity into the already created hand wrapper
	    			handWrp.addEventsEntity(handEntity);
	    			//                    handWrp.getField().addEventsEntity(handEntity);	// commenting this - it does the same thing as the above line
	
	    			// set reference id's in both directions (gloss entity <--> hand entity)
	    			handEntity.setRefEntityId(gloss.getID());
	    			gloss.setDependentHandEntity(handEntity);
	    		}
	    	}
	
	    	// block concerning dependencies for hs ID (depHandShapeID) and hs wrapper (handShapeWrp)
	    	if(depHandShapeID != null && !depHandShapeID.isEmpty()) {
	    		EventsEntity hndShapeEnt = handShapeWrp.getHandShapeField().getEntity(depHandShapeID);
	
	    		// if hs entity was properly constructed and is in the wrapper already, set dependencies using that one
	    		if (hndShapeEnt != null) {
	    			HandShapeEntity hndShapeEntity = (HandShapeEntity) hndShapeEnt;
	    			hndShapeEntity.setRefEntityId(gloss.getDependentHandEntity() != null ? gloss.getDependentHandEntityID() : null);
	    			gloss.setDependentHandShapeEntity(hndShapeEntity);
	    		} else {
	    			// else: gloss has hs entity ID saved, but there is not a corresponding hs entity object in database/track (and thus the wrapper)
	    			String fieldID = handShapeWrp.getField().getFieldID();
	    			SS3Track handShapeTrack = database.getTrack(fieldID);
	
	    			HandShapeEntity handShapeEntity = null;
	
	    			SS3Entity handShapeSS3Entity = handShapeTrack.getSS3Entity(depHandShapeID);
	
	    			// try to re-create hs entity strictly from db track
	    			if (handShapeSS3Entity != null) {
	    				handShapeEntity = new HandShapeEntity(fieldID,
	    						handShapeSS3Entity, segmentPanel, handShapeWrp.getHandShapeField());
	    				handShapeEntity.setField(handShapeWrp.getField());
	
	    				handShapeEntity.setID(handShapeSS3Entity.getID());
	    				handShapeEntity.setRefEntityId(handShapeSS3Entity.getRefEntityId());
	    				handShapeEntity.determineParentEntity();
	    			}
	
	    			// if db recreation failed, make a new hs entity object and give it ID of depHandShapeID
	    			if (handShapeEntity == null) {
	    				handShapeEntity = new HandShapeEntity(segmentPanel, handShapeWrp.getHandShapeField());
	
	    				handShapeEntity.setID(depHandShapeID);
	    				handShapeEntity.setField(handShapeWrp.getHandShapeField());
	
	    				HandShapeSubEntity subEntity = new HandShapeSubEntity(segmentPanel, handShapeWrp.getHandShapeField());
	    				subEntity.setID(handShapeEntity.getID());
	    				handShapeEntity.addHandShapeSubEntity(subEntity);
	
	    				handShapeEntity.determineParentEntity();
	    			}
	
	    			handShapeWrp.addEventsEntity(handShapeEntity);
	
	    			handShapeEntity.setRefEntityId(gloss.getDependentHandEntity() != null ? gloss.getDependentHandEntityID() : null);
	    			gloss.setDependentHandShapeEntity(handShapeEntity);
	    		}
	    	}
    }

    private ChainedEventsEntity createTrackEntity(SS3Entity entity, Field field) {

        String fieldID = field.getFieldID();

        ChainedEventsCollection collection = new ChainedEventsCollection();
        ArrayList<SS3Item> items = entity.getItemsAsList();

        for (SS3Item vItem : items) {
            String vID = vItem.getId();
            if (!vID.equalsIgnoreCase("" + Constants.ONSET_CODE) && !vID.equalsIgnoreCase("" + Constants.OFFSET_CODE) && !vID.equalsIgnoreCase("" + Constants.HOLD_CODE)) {
                if (fieldID.equalsIgnoreCase("10501") || fieldID.equalsIgnoreCase("10502")) {
                    collection.setTextValueEvent(new GlossChainedEvent(vItem, segmentPanel));
                } else {
                    collection.setValueEvent(new ChainedEvent(vItem, segmentPanel));
                }
            }
        }

        for (SS3Item item : items) {
            String id = item.getId();

            if (id.equalsIgnoreCase("" + Constants.ONSET_CODE)) {
                collection.setOnsetEvent(new ChainedEvent(item, segmentPanel));
            } else if (id.equalsIgnoreCase("" + Constants.OFFSET_CODE)) {
                collection.setOffsetEvent(new ChainedEvent(item, segmentPanel));
            } else {
                if (collection.getValueEvent() == null) {
                    System.out.println("no value event!!!" + id);
                }
                ChainedEvent valueEvent = collection.getValueEvent();
                if (valueEvent == null) {
                    valueEvent = collection.getTextValueEvent();
                }
                int startTime = valueEvent.getStartTimeInfo().getMovieTime();
                int endTime = valueEvent.getEndTimeInfo().getMovieTime();
                if (startTime != endTime) {
                    if (item.getStartTimeInfo().getMovieTime() == endTime) {
                        collection.setFinalHoldEvent(new ChainedEvent(item,
                                segmentPanel));
                    } else if (item.getEndTimeInfo().getMovieTime() == startTime) {
                        collection.setInitialHoldEvent(new ChainedEvent(
                                item, segmentPanel));
                    }
                }
            }
        }

        ChainedEventsEntity cEntity = new ChainedEventsEntity(collection, segmentPanel, field);
        cEntity.setID(entity.getID());
        cEntity.setField(field);
        cEntity.setRefEntityId(entity.getRefEntityId());
        cEntity.determineParentEntity();

        boolean isLockedGlossPhonAlignments = entity.isLockedGlossPhoneAlignments();
        cEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);

        return cEntity;
    }

    private ChainedEventsEntity  getHandEventsEntity(GlossChainedEventsEntity gloss, String id, RightLeftHandFieldWrapper handWrp) {

        ChainedEventsEntity handEntity = gloss.clone(handWrp.getField().getFieldID());

        String uniqueID = id;

        if(uniqueID == null || uniqueID.isEmpty()) {
            uniqueID = Util.getUniqueNumber();
        }

        handEntity.setID(uniqueID);

        return handEntity;
    }

    private void setDependentGlossEntity(GlossChainedEventsEntity entity, GlossFieldWrapper glsWrp) {
        String id = entity.getDependentGlossEntityID();
        if (id != null && !id.isEmpty()) {
            EventsEntity ndEntity = glsWrp.getGlossField().getEntity(id);
            if (ndEntity != null) {
                GlossChainedEventsEntity dependentGlossEntity = (GlossChainedEventsEntity) ndEntity;
                entity.setDependentGlossEntity(dependentGlossEntity);
            }
        }
    }

    private HandShapeEntity getHandShapeEntity(String entityID, String fieldType, HandShapeFieldWrapper handShapeWrp) {

        String fieldID = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme().getFieldID(fieldType);

        SS3Track track = database.getTrack(fieldID);
        SS3Entity entity = track.getSS3Entity(entityID);

        if(entity != null) {
            HandShapeEntity handShapeEntity = new HandShapeEntity(fieldID, entity, segmentPanel, handShapeWrp.getHandShapeField());
            handShapeWrp.addEventsEntity(handShapeEntity);
            handShapeEntity.determineParentEntity();
            return handShapeEntity;
        }

        return null;
    }

    private LabelObject addField(SS3Field field,
                                 SS3StatementField statementField, int rowNumber,
                                 LabelObject parent, boolean addDependentWrapper, int itemStartTime,
                                 int itemEndTime, PresentationField parentField) {
        return addField(field, statementField, rowNumber, parent, true,
                addDependentWrapper, itemStartTime, itemEndTime, parentField);
    }

    private LabelObject addField(SS3Field field,
                                 SS3StatementField statementField, int rowNumber,
                                 LabelObject parent, boolean isVisible, boolean addDependentWrapper,
                                 int itemStartTime, int itemEndTime, PresentationField parentField) {

        if (field != null) {

            LabelObject obj = new LabelObject(field.getId(),
                    field.getLabelOrName());
            obj.setRowNumber(rowNumber);
            obj.setVisible(isVisible);

            SS3Track track = database.getTrack(statementField, field.getId(),
                    itemStartTime, itemEndTime);

            FieldWrapper wrapper = factory.createFieldWrapper(track,
                    segmentPanel, parentField);
            wrapper.setVisible(isVisible);

            if (statementField != null) {
                wrapper.getField().setColor(statementField.getColor());
            }

            if (addDependentWrapper) {
                parent.getWrapper().addDependentWrapper(wrapper);
            }

            obj.setWrapper(wrapper);
            rowNumber++;
            parent.addChild(obj);
            return obj;
        }

        return null;
    }
    
    private LabelObject addField(SS3Field field, int rowNumber, LabelObject parent,
    		boolean addDependentWrapper, ArrayList<SS3GlossWindowItem> items, PresentationField parentField) {

        if (field != null) {

            LabelObject obj = new LabelObject(field.getId(),
                    field.getLabelOrName());
            obj.setRowNumber(rowNumber);

            SS3Track track = database.getTrack(field.getId(), items);

            FieldWrapper wrapper = factory.createFieldWrapper(track,
                    segmentPanel, parentField);
            
            boolean isVisible = ! track.getEntities().isEmpty();
            obj.setVisible(isVisible);
            wrapper.setVisible(isVisible);
            
            wrapper.getField().setColor(field.getColor());

            if (addDependentWrapper) {
                parent.getWrapper().addDependentWrapper(wrapper);
            }

            obj.setWrapper(wrapper);
            rowNumber++;
            parent.addChild(obj);
            return obj;
        }

        return null;
    }


    private void setOneHandDependency(GlossFieldWrapper glsWrp,
                                      RightLeftHandFieldWrapper hndWrp,
                                      HandShapeFieldWrapper hsWrp) {

        ArrayList hsEntities = hsWrp.getHandShapeField().getEntities();

        ArrayList hnEntities = hndWrp.getField().getEntities();

        ArrayList<HandShapeEntity> validHsEntities = new ArrayList<HandShapeEntity>();
        ArrayList<ChainedEventsEntity> validHndEntities = new ArrayList<ChainedEventsEntity>();

        for (Object hsObject : hsEntities) {
            HandShapeEntity hsEntity = (HandShapeEntity) hsObject;

            ChainedEventsEntity hndEntity = getHandEntity(hsEntity, hndWrp);
            GlossChainedEventsEntity glsEntity = getGlossEntity(hndEntity, glsWrp);

            if (hndEntity != null && glsEntity != null) {
                validHsEntities.add(hsEntity);
                validHndEntities.add(hndEntity);

                setDependencies(glsEntity, hndEntity, hsEntity);
            } else {
                glsEntity = getGlossEntity(hsEntity, glsWrp.getGlossField().getGlossChainedEventsEntityCollection());

                if (glsEntity != null) {
                    hndEntity = glsEntity.clone(hndWrp.getField().getFieldID());
                    hndWrp.getField().addEventsEntity(hndEntity);

                    setDependencies(glsEntity, hndEntity, hsEntity);

                    validHsEntities.add(hsEntity);
                    validHndEntities.add(hndEntity);
                }
            }
        }

        // clean up hs entities
        if(validHsEntities.size() < hsEntities.size()) {
            ArrayList<HandShapeEntity> delHsEntities = new ArrayList<HandShapeEntity>();

            for (Object hsEntity1 : hsEntities) {
                HandShapeEntity hsEntity = (HandShapeEntity) hsEntity1;
                if (!validHsEntities.contains(hsEntity)) {
                    delHsEntities.add(hsEntity);
                }
            }

            for(HandShapeEntity del : delHsEntities) {
                hsEntities.remove(del);
            }
        }

        // clean up hand entities
        if(validHndEntities.size() < hnEntities.size()) {
            ArrayList<ChainedEventsEntity> delHndEntities = new ArrayList<ChainedEventsEntity>();

            for (Object hnEntity : hnEntities) {
                ChainedEventsEntity hEntity = (ChainedEventsEntity) hnEntity;
                if (!validHndEntities.contains(hEntity)) {
                    delHndEntities.add(hEntity);
                }
            }

            for(ChainedEventsEntity del : delHndEntities) {
                hnEntities.remove(del);
            }
        }
    }

    private void setDependency(GlossFieldWrapper glsWrp,
                               GlossFieldWrapper ndGlsWrp,
                               RightLeftHandFieldWrapper rightHandWrp,
                               RightLeftHandFieldWrapper leftHandWrp,
                               HandShapeFieldWrapper rightHandShapeWrp,
                               HandShapeFieldWrapper leftHandShapeWrp) {

        setOneHandDependency(glsWrp, rightHandWrp, rightHandShapeWrp);
        setOneHandDependency(ndGlsWrp, leftHandWrp, leftHandShapeWrp);

        GlossChainedEventsEntityCollection glsColl = glsWrp.getGlossField().getGlossChainedEventsEntityCollection();
        GlossChainedEventsEntityCollection ndGlsColl = ndGlsWrp.getGlossField().getGlossChainedEventsEntityCollection();

        ArrayList<GlossChainedEventsEntity> ndGlss = ndGlsColl.getEntities();
        for (GlossChainedEventsEntity gls : ndGlss) {
            String refID = gls.getRefEntityId();
            GlossChainedEventsEntity nGls = glsColl.getEntity(refID);
            if (nGls != null) {
                gls.setDependentGlossEntity(nGls);
                nGls.setDependentGlossEntity(gls);
            } else {
                String txt = gls.getText();

                ArrayList<GlossChainedEventsEntity> glsEntities = glsColl
                        .getEntities();
                for (GlossChainedEventsEntity entity : glsEntities) {
                    String txt1 = entity.getText();
                    if (txt.equalsIgnoreCase(txt1)) {
                        GlossChainedEvent txtEvent = (GlossChainedEvent) entity
                                .getTextValue();
                        boolean oneHanded = txtEvent.getGlossClassifications()
                                .isOneHanded();
                        if (!oneHanded) {
                        }

                        int startTime1 = gls.getTextValue().getStartTimeInfo()
                                .getMovieTime();
                        int startTime = txtEvent.getStartTimeInfo()
                                .getMovieTime();

                        int endTime1 = gls.getTextValue().getEndTimeInfo()
                                .getMovieTime();
                        int endTime = txtEvent.getEndTimeInfo().getMovieTime();

                        if (startTime1 == startTime && endTime1 == endTime) {
                            gls.setDependentGlossEntity(entity);
                            entity.setDependentGlossEntity(gls);
                        }
                    }
                }
            }
        }
    }

    private ChainedEventsEntity getHandEntity(HandShapeEntity handShapeEntity, RightLeftHandFieldWrapper handWrp) {

        String refHandID = handShapeEntity.getRefEntityId();

        EventsEntity entity = null;

        // first try to get the hand entity based on the hs entity ref entity id
        if(refHandID != null)
            entity = handWrp.getField().getEntity(refHandID);

        // if the ref entity was blank or the hand entity in question is missing from the wrapper for whatever, try by midpoint coord of handShapeEntity
        if(entity == null) {
        	       	
            int startCoordinate = handShapeEntity.getStartTimeCoordinate();
            int endCoordinate = handShapeEntity.getEndTimeCoordinate();
            int midCoordinate = startCoordinate + (endCoordinate - startCoordinate)/2;

            entity = handWrp.getField().getEntityAt(midCoordinate);	// this eventually makes use of checking start == end (one frame)
        } 
        
        // if we still haven't found our hand entity, look in the wrapper for an entity matching the start/end points of this handShapeEntity
        if(entity == null) {
        	ArrayList list = handWrp.getField().getEntities();
        	
        	for(int i = 0; i < list.size(); i++) {
        		EventsEntity ent = (EventsEntity)list.get(i);
        		int startTime = ent.getStartTimeInfo().getMovieTime();
        		int endTime = ent.getEndTimeInfo().getMovieTime();
        		
        		int hsStartTime =  handShapeEntity.getStartTime();
        		int hsEndTime = handShapeEntity.getEndTime();
        		
        		if(startTime == hsStartTime && endTime == hsEndTime) {
        			entity = ent;
        		}
        	}
        }

        if(entity != null && isValid(handShapeEntity, entity))
            return (ChainedEventsEntity)entity;

        return null;
    }

    private boolean isValid(EventsEntity entity1, EventsEntity entity2) {
        if(entity2 == null)
            return false;

        int startTime = entity1.getStartTimeInfo().getMovieTime();
        int endTime = entity1.getEndTimeInfo().getMovieTime();

        Line2D domLine = new Line2D.Double(startTime, 0, endTime, 0);

        int startMovieTime = entity2.getStartTimeInfo().getMovieTime();
        int w = entity2.getEndTimeInfo().getMovieTime() - startMovieTime;
        
        boolean isValid = domLine.intersects(startMovieTime, 0, w, 1);
        
        if(startTime == startMovieTime) {
        	isValid = true;
        }

        return isValid;
    }

    private GlossChainedEventsEntity getGlossEntity(EventsEntity hsEntity, GlossChainedEventsEntityCollection glsColl) {

        int hsStartTime = hsEntity.getStartTimeInfo().getMovieTime();
        int hsEndTime = hsEntity.getEndTimeInfo().getMovieTime();

        Line2D domLine = new Line2D.Double(hsStartTime, 0, hsEndTime, 0);

        ArrayList<GlossChainedEventsEntity> glosses = glsColl.getEntities();

        GlossChainedEventsEntity entity = null;
        
        ArrayList<GlossChainedEventsEntity> entitiesToDelete = new ArrayList<GlossChainedEventsEntity>();
        for(GlossChainedEventsEntity gloss : glosses) {        	
        	if(gloss == null || gloss.getStartTimeInfo() == null || gloss.getEndTimeInfo() == null) {
        		entitiesToDelete.add(gloss);
        	} 
        }
        
        for(GlossChainedEventsEntity toDelete : entitiesToDelete) {        	
        	glsColl.deleteEntity(toDelete);
        }

        for(GlossChainedEventsEntity gloss : glosses) {   
        	
            int startMovieTime = gloss.getStartTimeInfo().getMovieTime();
            int endMovieTime = gloss.getEndTimeInfo().getMovieTime();
            int w = endMovieTime - startMovieTime;

            if(domLine.intersects(startMovieTime, 0, w, 1)) {
                entity = gloss;
            }
        }
        return entity;
    }

    private GlossChainedEventsEntity getGlossEntity(ChainedEventsEntity handEntity, GlossFieldWrapper glsFieldWrapper) {

        if(handEntity == null)
            return null;

        GlossChainedEventsEntityCollection coll = glsFieldWrapper.getGlossField().getGlossChainedEventsEntityCollection();

        GlossChainedEventsEntity entity = coll.getEntity(handEntity.getRefEntityId());

        // first try to get the gloss entity based on the hand entity ref entity id
        if(entity == null)
            entity = coll.getEntity(handEntity.getRefEntityId());

        // if it was not found by ref entity id, try next by coordinate
        if(entity == null) {
            EventsEntity ent = glsFieldWrapper.getGlossField().getEntityAt(handEntity.getValueOrTextValueEvent().getStartTimeCoordinate());

            if(ent != null)
                entity = (GlossChainedEventsEntity)ent;
        }

        // if still not found, call getGlossEntity
        if(entity == null) {
            entity = getGlossEntity(handEntity, coll);
        }

        if(isValid(handEntity, entity))
            return entity;

        return null;
    }

    private void setDependencies(GlossChainedEventsEntity glsEntity, ChainedEventsEntity handEntity, HandShapeEntity handShapeEntity) {

        glsEntity.setDependentHandEntity(handEntity);
        glsEntity.setDependentHandShapeEntity(handShapeEntity);

        // check if gloss and hand entities are displayed correctly
        Event value = glsEntity.getValueOrTextValueEvent();
        ChainedEvent initialHold = glsEntity.getInitialHold();
        ChainedEvent finalHold = glsEntity.getFinalHold();

        Event hndValue = handEntity.getValueOrTextValueEvent();

        ChainedEvent hndText = (ChainedEvent)handEntity.getValueOrTextValueEvent();
        hndText.setText(glsEntity.getText());
        
        int valueStartTime = value.getStartTimeInfo().getMovieTime();
        int valueEndTime = value.getEndTimeInfo().getMovieTime();        
        int initialHoldStartTime, finalHoldEndTime;

        // if initial hold exists, then set the entity in the RightLeftHandFieldWrapper to its start time
        if(initialHold != null) {
            initialHoldStartTime = initialHold.getStartTimeInfo().getMovieTime();
            hndValue.getStartTimeInfo().setMovieTime(initialHoldStartTime);
            handEntity.setInitialHoldEvent(initialHold.cloneChainedEvent());
        } else {
            hndValue.getStartTimeInfo().setMovieTime(valueStartTime);
        }

        // if final hold exists, then set the entity in the RightLeftHandFieldWrapper to its end time
        if(finalHold != null) {  
        		finalHoldEndTime = finalHold.getEndTimeInfo().getMovieTime();
            hndValue.getEndTimeInfo().setMovieTime(finalHoldEndTime);
            handEntity.setFinalHoldEvent(finalHold.cloneChainedEvent());
        } else {
            hndValue.getEndTimeInfo().setMovieTime(valueEndTime);
        }

        glsEntity.setLockedGlossPhonAlignments(true);
        handEntity.setLockedGlossPhonAlignments(true);
        handShapeEntity.setLockedGlossPhonAlignments(true);
    }
}
