package edu.bu.signstream.grepresentation.view;

import java.util.ArrayList;

import javax.swing.tree.MutableTreeNode;

import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;

public class LabelObject {

   private LabelObject parent;

   private ArrayList<LabelObject> children = new ArrayList<LabelObject>();

   private int rowNumber = 0;

   private String fieldID;
   private String label;
   private boolean updatable = false;
   private boolean visible = true;  
   private boolean isDefaultSegmentTier = false;

   private FieldWrapper wrapper;
   
   private MutableTreeNode mutableTreeNode = null;

   public LabelObject() {}

   public LabelObject(String fieldID, String label) {
      this.fieldID = fieldID;
      this.label = label;
   }
   
   public ArrayList<LabelObject> getVisibleObjects() {      
      ArrayList<LabelObject> allChildren = getAllChildren();     
      ArrayList<LabelObject> visibleObjects = new ArrayList<LabelObject>();
      
      for(int i = 0; i < allChildren.size(); i++) {
         LabelObject object = allChildren.get(i);
         
         if(object.isVisible()) {
        	 	visibleObjects.add(object);
        	 	object.getWrapper().setVisible(true);
         }         
      }
      return visibleObjects;
   } 
   
   public ArrayList<LabelObject> getNonEmptyObjectsBetween(int startTime, int endTime) {	      
      ArrayList<LabelObject> allChildren = getAllChildren();     
      ArrayList<LabelObject> nonEmptyObjects = new ArrayList<LabelObject>();
      
      for(int i = 0; i < allChildren.size(); i++) {
         LabelObject object = allChildren.get(i);
         
         if(!object.getWrapper().getField().getEntities().isEmpty()) {
        	 	ArrayList<EventsEntity> entities = object.getWrapper().getField().getEntities();
        	 	
        	 	for (EventsEntity entity: entities) {
        	 		int entityStartTime = entity.getStartTimeInfo().getMovieTime();
        	 		int entityEndTime = entity.getEndTimeInfo().getMovieTime();
        	 		
        	 		// if any part of entity is visible, then include its field
        	 		if ( (entityStartTime >= startTime && entityStartTime <= endTime)
        	 			|| (entityEndTime >= startTime && entityEndTime <= endTime) ) {
            	 		nonEmptyObjects.add(object);
                    object.setVisible(true);
                    object.getWrapper().setVisible(true);
        	 		
        	 			break;
        	 		}
        	 	}
         }         
      }
      return nonEmptyObjects;
   }

   public String getFieldID() {
      return fieldID;
   }

   public String getLabel() {
      return label;
   }

   public void setFieldID(String string) {
      fieldID = string;
   }

   public void setLabel(String string) {
      label = string;
   }

   @Override
public String toString() {
      return label;
   }

   public void setVisible(boolean visible) {
      this.visible = visible;
   }

   public boolean isVisible() {
      return visible;
   }

   public void setUpdatable(boolean updatable) {
      this.updatable = updatable;
   }

   public boolean isUpdatable() {
      return updatable;
   }
   
   public void setDefaultSegmentTier(boolean isDefault) {
	   this.isDefaultSegmentTier = isDefault;
   }
   
   public boolean isDefaultSegmentTier() {
	   return isDefaultSegmentTier;
   }

   public void setRowNumber(int rowNumber) {
      this.rowNumber = rowNumber;
   }

   public int getRowNumber() {
      return rowNumber;
   }

   /**
    * Returns the parent of the receiver.
    */
   public LabelObject getParent() {
      return parent;
   }

   /**
    * Returns true if the receiver represents a leaf, that is it is
    * isn't a directory.
    */
   public boolean isLeaf() {
	  
	  for(int i = 0; i < children.size(); i++) {
		  LabelObject obj = children.get(i);
		  if(obj.isVisible() || !obj.isLeaf())
			  return false;
	  }

      return true; //children.size() == 0;
   }

   /**
    * Loads the children, caching the results in the children
    * instance variable.
    */
   public LabelObject[] getChildren() {
      LabelObject[] obj = new LabelObject[children.size()];
      for (int i = 0; i < children.size(); i++) {
         obj[i] = children.get(i);
      }
      return obj;
   }

   public ArrayList<LabelObject> getAllChildren() {
      ArrayList<LabelObject> list = new ArrayList<LabelObject>();
      for (int i = 0; i < children.size(); i++) {
         LabelObject obj = children.get(i);
         list.add(obj);
         
         ArrayList<LabelObject> children = obj.getAllChildren();
         if(children != null && !children.isEmpty()) {
        	 list.addAll(obj.getAllChildren()); 
         }
      }
      return list;
   }

   public ArrayList<LabelObject> getChildrenAsList() {
      return children;
   }

   public void addChild(LabelObject node) {
      children.add(node);
   }

   public void removeChild(LabelObject node) {
      children.remove(node);
   }

   /**
    * Returns true if the children have been loaded.
    */
   protected boolean loadedChildren() {
      return (children != null);
   }

   public FieldWrapper getWrapper() {
      return wrapper;
   }

   public void setWrapper(FieldWrapper wrapper) {
      this.wrapper = wrapper;
   }

   public MutableTreeNode getMutableTreeNode() {
      return mutableTreeNode;
   }

   public void setMutableTreeNode(MutableTreeNode node) {
      mutableTreeNode = node;
   }
}
