package edu.bu.signstream.grepresentation.view;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.geom.Line2D;

import javax.swing.JComponent;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class LinenView extends JComponent {
	private SignStreamSegmentPanel segmentPanel;

   private int length = 0;
   public int SIZE = 16; //22; // 33
 
   public static final int INCH =
      Toolkit.getDefaultToolkit().getScreenResolution();
   public static final int HORIZONTAL = 0;
   public static final int VERTICAL = 1;

   public int orientation;
   public boolean isMetric;
   private int increment;
   private int units;

   public LinenView(int o, boolean m, int length, SignStreamSegmentPanel segmentPanel) {
      this.length = length;
      this.orientation = o;
      this.isMetric = m;
      this.segmentPanel = segmentPanel;
      setIncrementAndUnits();
   }
   
   public void update(int length) {
	   this.length = length;
	   repaint();
   }

   public void setIsMetric(boolean isMetric) {
      //this.isMetric = isMetric;
      setIncrementAndUnits();
      repaint();
   }

   public void setPreferredHeight(int ph) {
      setPreferredSize(new Dimension(SIZE, ph));
   }

   public void setPreferredWidth(int pw) {
      setPreferredSize(new Dimension(pw, SIZE));
   }

   private void setIncrementAndUnits() {
      if (isMetric) {
         units = (int) (INCH / 2.54);
         // dots per centimeter
         increment = units;
      } else {
         units = INCH;
         increment = units / 2;
      }
   }

   public boolean isMetric() {
      return this.isMetric;
   }

   public int getIncrement() {
      return increment;
   }

   @Override
protected void paintComponent(Graphics g) {
	      length = getWidth();
	      
	      Graphics2D g2d = (Graphics2D) g;
	      
	      Font largeFont = new Font(g2d.getFont().getName(), Font.PLAIN, 10);
	      Font smallFont = new Font(g2d.getFont().getName(), Font.PLAIN, 8);
	      
	      g2d.setColor(UIPalette.getBackground());
	      g2d.fillRect(0, 0, getWidth(), SIZE);

	      int SIZE1 = SIZE+5; //-10;
	      g2d.setColor(UIPalette.getLinenBackgroundColor());
	      g2d.fillRect(0, 0, getWidth(), SIZE1);

	      g2d.setColor(UIPalette.getLinenFontColor());
	      g2d.setFont(new Font(g2d.getFont().getName(), Font.PLAIN, 10));
	      g2d.draw(new Line2D.Double(0, SIZE1, getWidth(), SIZE1));
	      g2d.draw(new Line2D.Double(0, 0, 0, SIZE1));

	      int stickNumber = 30;
	      double interval = 33.33333 * segmentPanel.getZoomer().getPixelsInTimeFrame();
	      double yyy = length / interval;
	      
	      int mSec = 0;
	      
	      int X = 0;

	      for (int i = 1; i < yyy + 1; i++) {
	         X = (int) Math.floor(interval * i);
	     
	         if(interval <= 1) {
	        	 if (i % 30 == 0) {
	        		 if(i % 300 == 0) {
	            			 g2d.draw(new Line2D.Double(X, SIZE1, X, 6));
	            			 g.drawString("" + i / stickNumber, X + 2, 10);
	            	   	 } else {
	            			 g2d.draw(new Line2D.Double(X, SIZE1, X, 11));
	        		 }
	            } 
	         } else if(interval <= 1.1) { 
	        	 if (i % 30 == 0) {
	        		 if(i % 60 == 0) {
	            			 g2d.draw(new Line2D.Double(X, SIZE1, X, 6));
	            			 g.drawString("" + i / stickNumber, X + 2, 10);
	        		 } else {
	            			 g2d.draw(new Line2D.Double(X, SIZE1, X, 11));
	        		 }
	            } else if(i % 10 == 0) {
	            	 g2d.draw(new Line2D.Double(X, SIZE1, X, 11));
	    		}
	         } else if(interval <= 2) {
	        	 if (i % 30 == 0) {        		 
	            	g2d.draw(new Line2D.Double(X, SIZE1, X, 6));
	            	g.drawString("" + i / stickNumber, X + 2, 10);        	
	            } else if(i % 5 == 0) {
	            	 g2d.draw(new Line2D.Double(X, SIZE1, X, 11));
	    		}
	         } else if(interval < 3) {
	        	 if (i % 30 == 0) {
	        		 g2d.draw(new Line2D.Double(X, SIZE1, X, 6));
	        		 g.drawString("" + i / stickNumber, X + 2, 10);
	            }  else if (i % 15 == 0) {
	                //g2d.draw(new Line2D.Double(X, SIZE1, X, 11)); //13
	                g2d.draw(new Line2D.Double(X, 0, X, 6));
	            } else if(i % 2 == 0) {
	            	 g2d.draw(new Line2D.Double(X, SIZE1, X, 11));
	    		} 
	         } else if(interval < 20) {
	        	 if (i % 30 == 0) {
	        		 g2d.draw(new Line2D.Double(X, SIZE1, X, 6));
	        		 g.drawString("" + i / stickNumber, X + 2, 10);
	            }  else if (i % 15 == 0) {
	                g2d.draw(new Line2D.Double(X, SIZE1, X, 13)); //13
	                //g.drawString("" + i / stickNumber+".5", X-7, 12);
	                g.setFont(smallFont);
	                g.drawString("" + 30, X-7, 12);
	                g.setFont(largeFont);
	            } else {
	            	 g2d.draw(new Line2D.Double(X, SIZE1, X, 11));
	    		} 
	         } else {
	        	 if (i % 30 == 0) {
	        		 g2d.draw(new Line2D.Double(X, SIZE1, X, 6));
	        		 g.drawString("" + i / stickNumber, X + 2, 10);
	        		 mSec = 0;
	            }  else if (i % 15 == 0) {
	                g2d.draw(new Line2D.Double(X, SIZE1, X, 13)); //13
	                g.setFont(smallFont);
	                g.drawString("" + 30, X-7, 12);
	                g.setFont(largeFont);
	                mSec = 30;
	            } else if(i % 5 == 0 ){
	            	g2d.draw(new Line2D.Double(X, SIZE1, X, 13)); //13
	                g.setFont(smallFont);
	                mSec = mSec + 10;
	                g.drawString("" + mSec, X-7, 12);
	                g.setFont(largeFont);
	            } else {
	            	 g2d.draw(new Line2D.Double(X, SIZE1, X, 11));
	    		} 
	         }
	      }
   } 
}
