package edu.bu.signstream.grepresentation.view;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.panels.codingScheme.CSFieldDialog;

public class NonManualFieldsButtonPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private NonManualFieldsLabelView nonManualFieldsLabelView = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	private JButton plus, minus, expand;

	public NonManualFieldsButtonPanel(SignStreamSegmentPanel segmentPanel,
			NonManualFieldsLabelView nonManualFieldsLabelView) {
		this.segmentPanel = segmentPanel;
		this.nonManualFieldsLabelView = nonManualFieldsLabelView;
		
		setBackground(UIPalette.getBackground());
		setPreferredSize(new Dimension(getWidth(),
				SegmentGraphReprUtil.axesBottomHeight));
		setLayout(new FlowLayout(FlowLayout.LEFT, 20, 0));
		
		addIcons();
	}

	/**
	 * This helper method adds the plus, minus, and expand icons
	 * 
	 */
	private void addIcons() {		
		File file;

		String plusFilePath = "icons/plus.gif";
		file = new File(plusFilePath);
		if (file.exists()) {
			plus = new JButton(new ImageIcon(plusFilePath));
		} else {
			plus = new JButton("+");
			System.out.println("File '" + plusFilePath + "' was not found");
		}
		AddNewFieldListener addNewFieldListener = new AddNewFieldListener(this);
		plus.addActionListener(addNewFieldListener);
		setFieldsButtonsLNF(plus);

		String minusFilePath = "icons/minus.gif";
		file = new File(minusFilePath);
		if (file.exists()) {
			minus = new JButton(new ImageIcon(minusFilePath));
		} else {
			minus = new JButton("-");
			System.out.println("File '" + minusFilePath + "' was not found");
		}
		HideFieldListener hideFieldListener = new HideFieldListener(this);
		minus.addActionListener(hideFieldListener);
		setFieldsButtonsLNF(minus);

		String expandFilePath = "icons/group.gif";
		file = new File(expandFilePath);
		if (file.exists()) {
			expand = new JButton(new ImageIcon(expandFilePath));
		} else {
			expand = new JButton("O");
			System.out.println("File '" + expandFilePath + "' was not found");
		}
		ShowAllFieldListener showAllFieldListener = new ShowAllFieldListener(this);
		expand.addActionListener(showAllFieldListener);
		setFieldsButtonsLNF(expand);
		
		add(plus);
		add(minus);
		add(expand);
	}

	private static void setFieldsButtonsLNF(JButton btn) {
		Font btnFont = new Font(btn.getFont().getName(), Font.PLAIN, 8);
		FontMetrics btnFontMetrics = btn.getFontMetrics(btnFont);
		int addBtnHeight = btnFontMetrics.getHeight();
		int addBtnWidth = btnFontMetrics.stringWidth(btn.getText());

		btn.setFont(btnFont);

		Dimension addBtnMinSize = new Dimension(addBtnWidth + 15,
				addBtnHeight);
		btn.setMaximumSize(addBtnMinSize);
		btn.setPreferredSize(addBtnMinSize);
		btn.setMaximumSize(addBtnMinSize);
	}
	
	public NonManualFieldsLabelView getNonManualFieldsLabelView() {
		return nonManualFieldsLabelView;
	}
	
	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}
	
	void clickExpandButton() {
		expand.doClick();		// triggers ShowAllFieldListener.actionPerformed()
	}
}

class AddNewFieldListener implements ActionListener {
	private NonManualFieldsButtonPanel buttonPanel = null;
	private CSFieldDialog csFieldDialog = null;
	
	public AddNewFieldListener(NonManualFieldsButtonPanel parent) {
		this.buttonPanel = parent;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		LabelObject selectedFieldLabel = buttonPanel.getNonManualFieldsLabelView().getSelectedField();
		String fieldID = null;
		
		if (selectedFieldLabel != null)
			fieldID = selectedFieldLabel.getFieldID();
		
		if (csFieldDialog == null)
			csFieldDialog = buttonPanel.getNonManualFieldsLabelView().getCSFieldDialog();

		csFieldDialog.setVisible(true, fieldID);
	}	
}

class HideFieldListener implements ActionListener {
	private NonManualFieldsButtonPanel buttonPanel = null;
	private CSFieldDialog csFieldDialog = null;
	
	public HideFieldListener(NonManualFieldsButtonPanel parent) {
		this.buttonPanel = parent;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		LabelObject selectedFieldLabel = buttonPanel.getNonManualFieldsLabelView().getSelectedField();
		
		if (selectedFieldLabel == null)
			return;
		
		if (csFieldDialog == null)
			csFieldDialog = buttonPanel.getNonManualFieldsLabelView().getCSFieldDialog();

		// pass in false and set that field to unchecked, and the enter will hide it from view
		csFieldDialog.setSelectedField(selectedFieldLabel.getFieldID(), false);
		csFieldDialog.enter();
	}
	
}

class ShowAllFieldListener implements ActionListener {
	private NonManualFieldsButtonPanel buttonPanel = null;
	private CSFieldDialog csFieldDialog = null;
	
	public ShowAllFieldListener(NonManualFieldsButtonPanel parent) {
		this.buttonPanel = parent;
	}

	@Override
	public void actionPerformed(ActionEvent e) {		
		if (csFieldDialog == null)
			csFieldDialog = buttonPanel.getNonManualFieldsLabelView().getCSFieldDialog();
	      
		int startTime = SegmentGraphReprUtil.utteranceViewVisibleMovieOnset;
		int endTime = SegmentGraphReprUtil.utteranceViewVisibleMovieOffset;
		
		LabelObject nonManualLabelObject = csFieldDialog.getSegment().getRootLabelObject().getNonManuelLabelObject();

		ArrayList<LabelObject> emptyFieldLabels = new ArrayList<LabelObject>(nonManualLabelObject.getChildrenAsList());
		ArrayList<LabelObject> nonEmptyFieldLabels = nonManualLabelObject.getNonEmptyObjectsBetween(startTime, endTime);
		
		for (LabelObject nonEmptyFieldLabel: nonEmptyFieldLabels) {
			nonEmptyFieldLabel.getWrapper().setSelectedNHighlighted(false);
			csFieldDialog.setSelectedField(nonEmptyFieldLabel.getFieldID(), true);
			
			emptyFieldLabels.remove(nonEmptyFieldLabel);
		}
		
		for (LabelObject emptyFieldLabel: emptyFieldLabels) {
			csFieldDialog.setSelectedField(emptyFieldLabel.getFieldID(), false);
		}
		
		csFieldDialog.enter();
	}
	
}