package edu.bu.signstream.grepresentation.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.observer.SSObservable;
import edu.bu.signstream.observer.SSObserver;
import edu.bu.signstream.ui.panels.codingScheme.CSFieldDialog;

public class NonmanualFieldsView extends JPanel implements SSObserver {

	private static final long serialVersionUID = 1L;

	protected LinenView linenView;
	protected NonManualFieldsLabelView labelView;
	protected JScrollPane pictureScrollPane;

	private Segment segment;
	private SignStreamSegmentPanel panel;
	private SegmentGraphicRepresentation rep = null;
	private Corner upperLeft = null;
	private PresentationField parentField = null;

	public NonmanualFieldsView(Segment segment, SignStreamSegmentPanel panel,
			SegmentControlPanel controlPanel, PresentationField parentField) {
		this.segment = segment;
		this.panel = panel;
		this.parentField = parentField;

		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		CSFieldDialog csFieldDialog = new CSFieldDialog(panel, segment, this, parentField);
		labelView = new NonManualFieldsLabelView(segment, panel, controlPanel, csFieldDialog);
		rep = labelView.getSegmentGraphicRepresentation();
		upperLeft = new UpperLeftCorner(panel);

		int mDuration = SegmentGraphReprUtil.utteranceMovieTimeOffset
				- SegmentGraphReprUtil.utteranceMovieTimeOnset;
		int length = panel.getZoomer().getCoordinateTimeConvertor()
				.getScreenWidth(mDuration);

		length = length > rep.getPreferredDimension().width ? length : rep
				.getPreferredDimension().width;

		rep.setPreferredSize(new Dimension(length, (int) rep
				.getPreferredDimension().getHeight()));

		if (rep != null) {
			labelView.setPreferredSize(new Dimension(
					SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH, 
					rep.getPreferredDimension().height));
		} else {
			labelView.setPreferredHeight(480);
		}

		pictureScrollPane = new JScrollPane(rep,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		pictureScrollPane.getHorizontalScrollBar().setPreferredSize(new Dimension(0, 0));

		pictureScrollPane.setPreferredSize(new Dimension(300, 250));

		pictureScrollPane.setViewportBorder(BorderFactory
				.createLineBorder(Color.black));
		
		int l = rep.getPreferredDimension().width;
		
		pictureScrollPane.setRowHeaderView(labelView);
		pictureScrollPane.setCorner(ScrollPaneConstants.UPPER_LEFT_CORNER, upperLeft);
		pictureScrollPane
				.setCorner(ScrollPaneConstants.LOWER_LEFT_CORNER, new Corner());
		pictureScrollPane.setCorner(ScrollPaneConstants.UPPER_RIGHT_CORNER,
				new UpperRightCorner());

		int eventStartTime = SegmentGraphReprUtil.utteranceMovieTimeOnset;
		int x = panel.getZoomer().getCoordinateTimeConvertor().convertTimeToCoordinate(eventStartTime);
		scrollRectToVisible(new Point(x, 10));
		
		linenView = new LinenView(LinenView.HORIZONTAL, true, rep.getPreferredDimension().width, panel);

		add(pictureScrollPane);
	}

	public void update(Segment segment) {
		this.segment = segment;
		update();

		int mDuration = SegmentGraphReprUtil.utteranceMovieTimeOffset
				- SegmentGraphReprUtil.utteranceMovieTimeOnset;
		
		int length = panel.getZoomer().getCoordinateTimeConvertor()
				.getScreenWidth(mDuration);

		length = length > rep.getPreferredDimension().width ? length : rep
				.getPreferredDimension().width;

		rep.setPreferredSize(new Dimension(length, (int) rep
				.getPreferredDimension().getHeight()));
		
		linenView.update(length);

		if (rep != null) {
			linenView.setPreferredWidth(length);
			labelView.setPreferredSize(new Dimension(125, rep
					.getPreferredDimension().height));
		} else {
			linenView.setPreferredWidth(320);
			labelView.setPreferredHeight(480);
		}

	}

	public JScrollPane getJScrollPane() {
		return pictureScrollPane;
	}

	public Segment getSegment() {
		return segment;
	}

	public void setScrollPaneHeight(int height) {
		rep.setPreferredSize(new Dimension(rep.getPreferredDimension().width,	height));
		labelView.setPreferredHeight(height);

		pictureScrollPane.getViewport().setView(rep);
	}

	public void scrollRectToVisible(Point point) {

		if (point == null)
			return;

		JViewport vp = pictureScrollPane.getViewport();
		if (vp != null) {
			vp.setViewPosition(point);
			pictureScrollPane.invalidate();
			pictureScrollPane.revalidate();
		}
	}

	public FieldWrappersCollection getFieldWrappersCollection() {
		return segment.getFieldWrappersCollection();
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return panel;
	}

	public Corner getUpperLeftCorner() {
		return upperLeft;
	}

	public NonManualFieldsLabelView getLabelView() {
		return labelView;
	}

	public SegmentGraphicRepresentation getSegmentGraphicRep() {
		return rep;
	}

	public CSFieldDialog getCSFieldDialog() {
		return getLabelView().getCSFieldDialog();
	}

	public void update() {
		labelView.update(segment);
		
		// these multipliers work well
		int height = labelView.getLabels().size() * SegmentGraphReprUtil.axesTopHeight * 2;

		setScrollPaneHeight(height);
	}

	public LinenView getLinenView() {
		return linenView;
	}

	@Override
	public void update(SSObservable subj, Object o) {
	}
	
	void clickExpandButton() {
		labelView.clickExpandButton();
	}
}
