package edu.bu.signstream.grepresentation.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class SegmentControlPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	
	private SignStreamSegmentPanel segmentPanel;
	private Segment segment;
	private NonmanualFieldsView nonManualView;
	private GlossNRelatedView glossView;
	
	private JComboBox<String> partitionComboBox = new JComboBox<String>();
	private String currentPartitionString = null;
	private SS3TemporalPartition currentPartition = null;
	
	private JComboBox<String> segmentComboBox = new JComboBox<String>();
	private String currentSegmentString= null;
	private SS3GlossWindowSegment currentSegment = null;
	
	private JList<String> utteranceList = null;
	private JScrollPane utteranceListScroller = null;
	private UtteranceListSelectionListener utteranceListSelectionListener = null;
	
	private JButton prevUtteranceBtn = null;
	private JButton nextUtteranceBtn = null;
	
	private JLabel annotatorLabel = new JLabel("  Participant: N/A  ");
	private JButton notesBtn = new JButton(" Notes ");
	
	private SS3Participant selectedParticipant = null;
	private SS3Participant selectedAnnotator = null;
	
	private static final String DOMINANT_GLOSS_FIELD_ID = "10000";

	public static final int PREV_BTN = 1;
	public static final int NEXT_BTN = 2;
	public final int NOTES_BTN = 4;

	// objects to discard
	private JDialog notesDialog = null;
	private NotesPanel notesPanel = null;

	public SegmentControlPanel(SignStreamSegmentPanel segmentPanel,
			Segment segment, NonmanualFieldsView topView, GlossNRelatedView bottomView) {
		this.segmentPanel = segmentPanel;
		this.segment = segment;
		setNonManualView(topView);
		setGlossView(bottomView);
		
		PresentationField parentField = segment.getParentField();		
		setCurrentPartition(parentField.getTemporalPartition());
		setCurrentSegment(parentField.getSS3GlossWindowSegment());
		setSelectedParticipant(parentField.getSS3GlossWindowSegment().getParticipant());
//		setSelectedAnnotator(parentField.getSS3GlossWindowSegment().getAnnotator());
		
		ArrayList<SS3TemporalPartition> temporalPartitions = segmentPanel.getSS3Database().getSS3GlossWindow().getTemporalPartitions();		
		for (SS3TemporalPartition temporalPartition: temporalPartitions) {
			String partitionLabel = temporalPartition.getLabel();
			partitionComboBox.addItem(partitionLabel);
			
			if (partitionLabel.equals(getCurrentPartitionString())) {
				partitionComboBox.setSelectedItem(partitionLabel);
			}
		}
		TemporalPartitionActionListener partitionActionListener = new TemporalPartitionActionListener(this);
		partitionComboBox.addActionListener(partitionActionListener);
		
		ArrayList<SS3GlossWindowSegment> temporalPartitionSegments = getCurrentPartition().getTemporalPartitionSegments();		
		for (SS3GlossWindowSegment temporalPartitionSegment: temporalPartitionSegments) {
			String segmentLabel = temporalPartitionSegment.getLabel();
			String participantLabel = temporalPartitionSegment.getParticipantLabel();
			String combinedLabel = participantLabel + ":" + segmentLabel;
			
			if (combinedLabel.indexOf(":") == 0)
				combinedLabel = combinedLabel.substring(1);
			
			segmentComboBox.addItem(combinedLabel);
			
			if (combinedLabel.equals(getCurrentSegmentString())) {
				segmentComboBox.setSelectedItem(combinedLabel);
			}
		}
		SegmentActionListener segmentActionListener = new SegmentActionListener(this);
		segmentComboBox.addActionListener(segmentActionListener);

		
		utteranceList = new JList();
		utteranceListScroller = new JScrollPane(utteranceList);
		utteranceListScroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		
		utteranceListScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		utteranceList.setDragEnabled(false);
		utteranceList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		utteranceListSelectionListener = new UtteranceListSelectionListener(this);
		utteranceList.getSelectionModel().addListSelectionListener(utteranceListSelectionListener);
		
		updateUtteranceList();
		
		String currentParticipant = currentSegment.getParticipantLabel();
		annotatorLabel.setText(currentParticipant.isEmpty() ? "  Participant: N/A  " : "Participant: " + currentParticipant);
		
		prevUtteranceBtn = new JButton("< Prev Utterance" );
		nextUtteranceBtn = new JButton(" Next Utterance >");
		
		prevUtteranceBtn.setActionCommand("" + PREV_BTN);
		nextUtteranceBtn.setActionCommand("" + NEXT_BTN);
		
		prevUtteranceBtn.addActionListener(new PrevNextUtteranceListener(this));
		nextUtteranceBtn.addActionListener(new PrevNextUtteranceListener(this));
		
		notesBtn.setActionCommand("" + NOTES_BTN);
		notesBtn.addActionListener(this);

		UserInterfaceUtil.setComboBoxLNF(partitionComboBox);
		partitionComboBox.setPreferredSize(null);
		partitionComboBox.setMaximumSize(null);
		UserInterfaceUtil.setComboBoxLNF(segmentComboBox);
		segmentComboBox.setPreferredSize(null);
		segmentComboBox.setMaximumSize(null);
		UserInterfaceUtil.setListLNF(utteranceList);
		UserInterfaceUtil.setLabelLNF(annotatorLabel);
		UserInterfaceUtil.setButtonsLNF(notesBtn);
		UserInterfaceUtil.setButtonsLNF(prevUtteranceBtn);
		UserInterfaceUtil.setButtonsLNF(nextUtteranceBtn);

		annotatorLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		notesBtn.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.weightx = 1.0;
		c.gridwidth = 2;
		
		c.gridy = 0;
		add(partitionComboBox, c);
		
		c.gridy = 1;
		add(segmentComboBox, c);
		
		c.gridy = 2;
		c.weighty = 1.0;
		add(utteranceListScroller, c);
		
		c.weighty = 0.1;
		c.gridwidth = 1;
		
		c.gridy = 3;
		add(prevUtteranceBtn, c);
		
		c.gridx = 1;
		add(nextUtteranceBtn, c);
		
		c.gridx = 0;
		c.gridy = 4;
		add(annotatorLabel, c);
		
		c.gridx = 1;
		add(notesBtn, c);

	}

	public void update(Segment segment) {
		this.segment = segment;
	}
	
	public void updateUtteranceList() {
		ArrayList<SS3GlossWindowItem> items = getCurrentSegment().getSS3GlossWindowItems();
		ArrayList<String> utterances = new ArrayList<String>();
		
		for (int i = 0; i < items.size(); i++) {
			SS3GlossWindowItem item = items.get(i);			
			String fieldID = DOMINANT_GLOSS_FIELD_ID;
			String combinedTextLabels = "  ";
			
			if (item.getSS3StatementField(fieldID) != null) {
				// entities are in order by startTime
				ArrayList<SS3Entity> ss3Entities = item.getSS3StatementField(fieldID).getSS3Entities();
				
				for (SS3Entity ss3Entity: ss3Entities) {
					ArrayList<SS3Item> entItems = ss3Entity.getItems();
					for (SS3Item entItem: entItems) {
						// if entItem.isTextOnly() gave back the correct/up-to-date result, then it would be used here instead
						String text = entItem.getText();
						if (text.equals("Onset") || text.equals("Offset")
								|| text.equals("Initial Hold") || text.equals("Final Hold"))
							continue;
						
						combinedTextLabels += entItem.getText() + " ";
					}
				}
			}
			
			String utteranceLabel = item.getLabel();
			utterances.add(utteranceLabel + combinedTextLabels);
		}
		
		utteranceList.setListData(Arrays.copyOf(utterances.toArray(), utterances.size(), String[].class));
	}
	
	public void updateUtteranceListSelectedItems(ArrayList<PresentationEventsEntity> utterances) {

		ArrayList<String> utteranceLabels = new ArrayList<String>();
		for (PresentationEventsEntity utterance: utterances) {
			utteranceLabels.add(utterance.getEvent().getSS3GlossWindowItem().getLabel());
		}
		
		ArrayList<Integer> indiciesList = new ArrayList<Integer>();
		
		ListModel<String> listModel = utteranceList.getModel();
		int size = listModel.getSize();
		
		for (int i = 0; i < size; i++) {
			String elementLabelWithText = listModel.getElementAt(i);
			
			int indexOfSpace = elementLabelWithText.indexOf(" ");
			String elementLabel = elementLabelWithText.substring(0, indexOfSpace);
			
			for (String utteranceLabel: utteranceLabels) {
				if (elementLabel.equals(utteranceLabel)) {
					indiciesList.add(i);
				}
			}
		}
		
		// code necessary to create an int array from an Integer arraylist
		int indiciesListSize = indiciesList.size();
		// indiciesArray must be final to be used in inner Runnable class below
		final int[] indiciesArray = new int[indiciesListSize];
		for (int i = 0; i < indiciesListSize; i++) {
			indiciesArray[i] = indiciesList.get(i);
		}

		utteranceList.clearSelection();
		
		if (indiciesListSize > 0) {
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					
					// first remove the listener from the utteranceList to avoid an infinite action firing
					utteranceList.getSelectionModel().removeListSelectionListener(utteranceListSelectionListener);
					
					utteranceList.setSelectedIndices(indiciesArray);

					// then re-add the listener after we have manually set the indicies
					utteranceList.getSelectionModel().addListSelectionListener(utteranceListSelectionListener);
					
					utteranceList.ensureIndexIsVisible(indiciesArray[0]);
				}
			});
		}
	}
	
	public JComboBox<String> getTemporalPartitionComboBox() {
		return partitionComboBox;
	}
	
	public String getCurrentPartitionString() {
		return currentPartitionString;
	}
	
	public void setCurrentPartitionString(String currentPartitionString) {
		this.currentPartitionString = currentPartitionString;
	}
	
	public SS3TemporalPartition getCurrentPartition() {
		return currentPartition;
	}
	
	public void setCurrentPartition(SS3TemporalPartition currentPartition) {
		this.currentPartition = currentPartition;
		
		if (currentPartition != null)
			setCurrentPartitionString(currentPartition.getLabel());
		else
			setCurrentPartitionString(null);
	}
	
	public JComboBox<String> getSegmentComboBox() {
		return segmentComboBox;
	}
	
	public String getCurrentSegmentString() {
		return currentSegmentString;
	}
	
	public void setCurrentSegmentString(String currentSegmentString) {
		this.currentSegmentString = currentSegmentString;
	}
	
	public SS3GlossWindowSegment getCurrentSegment() {
		return this.currentSegment;
	}
	
	public void setCurrentSegment(SS3GlossWindowSegment currentSegment) {
		this.currentSegment = currentSegment;	
		
		if (currentSegment != null) {
			String currentParticipantLabel = currentSegment.getParticipantLabel();
			String currentSegmentLabel = currentSegment.getLabel();
			String combinedLabel = currentParticipantLabel + ":" + currentSegmentLabel;	

			if (combinedLabel.indexOf(":") == 0)
				combinedLabel = combinedLabel.substring(1);
			
			setCurrentSegmentString(combinedLabel);
		}
		else
			setCurrentSegmentString(null);
	}
	
	public JList<String> getUtteranceList() {
		return utteranceList;
	}
	
	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}
	
	public Segment getSegment() {
		return segment;
	}

	public SS3Participant getSelectedParticipant() {
		return selectedParticipant;
	}

	public void setSelectedParticipant(SS3Participant selectedParticipant) {
		this.selectedParticipant = selectedParticipant;
	}

	public SS3Participant getSelectedAnnotator() {
		return selectedAnnotator;
	}

	public void setSelectedAnnotator(SS3Participant selectedAnnotator) {
		this.selectedAnnotator = selectedAnnotator;
	}

	public void setNonManualView(NonmanualFieldsView nonManualView) {
		this.nonManualView = nonManualView;
	}

	public void setGlossView(GlossNRelatedView glossView) {
		this.glossView = glossView;
	}
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();
		switch (Integer.parseInt(str)) {
		case NOTES_BTN:
			if (notesDialog == null)
				createNotesDialog();
			notesDialog.setVisible(true);
			break;
		}
	}

	private void createNotesDialog() {
		notesDialog = new JDialog();
		notesDialog.setTitle("Note Editor");
		notesDialog.setModal(true);

		notesPanel = new NotesPanel( getCurrentSegment() != null ? getCurrentSegment().getNote() : null,
				notesDialog);

		JPanel panel1 = new JPanel(new BorderLayout());
		panel1.add(notesPanel, BorderLayout.CENTER);
		panel1.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

		Container notesContentPane = notesDialog.getContentPane();
		notesContentPane.setLayout(new BorderLayout());
		notesContentPane.add(notesPanel, BorderLayout.CENTER);

		notesDialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent we) {
				notesPanel.save();
			}
		});

		notesDialog.setSize(400, 350);
		notesDialog.setPreferredSize(new Dimension(400, 350));
		notesDialog.setLocation(150, 150);
		notesDialog.pack();
	}

	public void discard() {
		if (notesDialog != null) {
			notesPanel.discard();

			WindowListener[] listeners = notesDialog.getWindowListeners();
			for (int i = 0; i < listeners.length; i++) {
				WindowListener listener = listeners[i];
				notesDialog.removeWindowListener(listener);
			}

			notesPanel = null;
			notesDialog = null;
		}

	}
	
	/**
	 * Based on an index into a list of utterances, reframe the visible viewbox on-screen to the start of the indexed utterance.
	 * This will: 1) Automatically update the utterance list in this SegmentControlPanel; 2) Set a new movie time; 
	 * 3) Move the slider to this new movie time; 4) refresh the visible nonmanual fields in every segment tier.
	 * @param utteranceIndexToSelect The index of the utterance to select, based on the parent field of this SegmentControlPanel
	 */
	void selectAndScrollToUtteranceByIndex(int utteranceIndexToSelect) {        
        PresentationField parentField = getSegment().getParentField();
        ArrayList<PresentationEventsEntity> utterances = parentField.getEntities();
        PresentationEventsEntity reframeToUtterance = utterances.get(utteranceIndexToSelect);

        SegmentGraphReprUtil.utteranceMovieTimeOnset = reframeToUtterance.getStartTime();
        SegmentGraphReprUtil.utteranceMovieTimeOffset = reframeToUtterance.getEndTime();
        
        int xcoord = reframeToUtterance.getStartTimeCoordinate();
        
        segmentPanel.getUtteranceView().scrollRectToVisible(new Point(xcoord, 0));
		
		SS3Singleton.getVideoControlManager().getMultipleMovieController()
			.setCurrentTime(SegmentGraphReprUtil.utteranceMovieTimeOnset);
		segmentPanel.getSlider().setSliderPositionTime(SegmentGraphReprUtil.utteranceMovieTimeOnset);
		
		segmentPanel.getUtteranceView().expandAllNonmanualFields();
	}
}

class PrevNextUtteranceListener implements ActionListener {
	private SegmentControlPanel segmentControlPanel = null;
	
	public PrevNextUtteranceListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		int utteranceListSize = segmentControlPanel.getUtteranceList().getModel().getSize();
        int utteranceListIndex = segmentControlPanel.getUtteranceList().getSelectedIndex();
        int utteranceIndexToSelect = -1;
        
        // just check the utterance is not out of bounds for either prev/next
		String str = e.getActionCommand();		
		switch (Integer.parseInt(str)) {
		case SegmentControlPanel.PREV_BTN:
			utteranceIndexToSelect = utteranceListIndex - 1;
			
			if (utteranceIndexToSelect < 0) {
				String msg = "<HTML>Warning: Previous utterance does not exist <BR>";
				msg = msg + "Please use \"Next Utterance\" to view other utterances</HTML>";
				SS3Singleton.getSignStreamApplication()
						.confirmationDialog("Resource Update", msg);
				return;
			}
			
			break;
		case SegmentControlPanel.NEXT_BTN:
			utteranceIndexToSelect = utteranceListIndex + 1;
			
			if (utteranceIndexToSelect >= utteranceListSize) {
				String msg = "<HTML>Warning: Next utterance does not exist <BR>";
				msg = msg + "Please use \"Prev Utterance\" to view other utterances</HTML>";
				SS3Singleton.getSignStreamApplication()
						.confirmationDialog("Resource Update", msg);
				return;
			}
			break;
		}
		
		segmentControlPanel.selectAndScrollToUtteranceByIndex(utteranceIndexToSelect);
		
		// unselect events, and unhighlight everything so that a new look at an utterance plays that utterance
		segmentControlPanel.getSignStreamSegmentPanel().getToolBarPanels().getMediaToolBar().selectEvent(null, null, null);
		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getNonmanualFieldsView().getFieldWrappersCollection().unselectAll();
		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getGlossNRelatedView().getFieldWrappersCollection().unselectAll();
		
		// gets the focus off of the utterance list and allows the left/right arrow keys to work again
		segmentControlPanel.getSignStreamSegmentPanel().requestFocus();
	}
}

class UtteranceListSelectionListener implements ListSelectionListener {
	private SegmentControlPanel segmentControlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public UtteranceListSelectionListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
		this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();		
	}
	@Override
	public void valueChanged(ListSelectionEvent e) {
		
		// this will proceed once the value is not longer adjusting, meaning the mouse click is released
		if (e.getValueIsAdjusting() == true)
			return;
		
        ListSelectionModel utteranceListSelectionModel = (ListSelectionModel) e.getSource();
        
        if (utteranceListSelectionModel.isSelectionEmpty()) {
        		return;
        } else {
	    		// go to selected utterance
            int utteranceListIndex = utteranceListSelectionModel.getMaxSelectionIndex();
            
            segmentControlPanel.selectAndScrollToUtteranceByIndex(utteranceListIndex);

            // unselect events, and unhighlight everything so that a new look at an utterance plays that utterance
            segmentControlPanel.getSignStreamSegmentPanel().getToolBarPanels().getMediaToolBar().selectEvent(null, null, null);
            segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getNonmanualFieldsView().getFieldWrappersCollection().unselectAll();
            segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getGlossNRelatedView().getFieldWrappersCollection().unselectAll();

			// gets the focus off of the utterance list and allows the left/right arrow keys to work again
			segmentPanel.requestFocus();
        }        
	}
	
}

class SegmentActionListener implements ActionListener {
	private SegmentControlPanel segmentControlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public SegmentActionListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
		this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();
	}
	@Override
	public void actionPerformed(ActionEvent e) {
		// first check that this segment tier is not visible in any other segment in the utterance view
		String selectedTemporalPartitionName = 
				(String) segmentControlPanel.getTemporalPartitionComboBox().getSelectedItem();		

		String selectedSegmentTierNameWithParticipant = 
				(String) segmentControlPanel.getSegmentComboBox().getSelectedItem();
		String selectedSegmentTierName = selectedSegmentTierNameWithParticipant.
				substring(selectedSegmentTierNameWithParticipant.indexOf(":") + 1);
		
		segmentPanel.getUtteranceView().addUtterancePane(segmentControlPanel.getSegment(),
				selectedTemporalPartitionName, selectedSegmentTierName);
	}
	
}

class TemporalPartitionActionListener implements ActionListener {
	private SegmentControlPanel segmentControlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public TemporalPartitionActionListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
		this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// first check that this segment tier is not visible in any other segment in the utterance view
		String selectedTemporalPartitionName = 
				(String) segmentControlPanel.getTemporalPartitionComboBox().getSelectedItem();		
		
		SS3TemporalPartition currentTemporalPartition = 
				segmentPanel.getSS3Database().getSS3GlossWindow().getTemporalPartitionByLabel(selectedTemporalPartitionName);
		
		SS3GlossWindowSegment defaultSegment = currentTemporalPartition.getDefaultSegment();
		String defaultSegmentName = defaultSegment.getLabel();
		
		// go to selected segment tier, with the same utterance as the current
		segmentPanel.getUtteranceView().addUtterancePane(segmentControlPanel.getSegment(),
				selectedTemporalPartitionName, defaultSegmentName);
	}
	
}