package edu.bu.signstream.grepresentation.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
//import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.NewGlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.rightLeftField.RightLeftHandFieldWrapper;
import edu.bu.signstream.grepresentation.listener.SignStreamMouseListener;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.Slider;
import edu.bu.signstream.observer.SSObservable;
import edu.bu.signstream.observer.SSObserver;
import edu.bu.signstream.ui.panels.notes.vo.NotesRepresentation;

public class SegmentGraphicRepresentation extends JPanel implements SSObserver {

	private static final long serialVersionUID = 1L;

	private SignStreamSegmentPanel segmentPanel;
	private Segment segment;
	private FieldWrappersCollection wrappers;

	// delete
	private NotesRepresentation notes = null;

	private ArrayList<JTextField> addedObjects = new ArrayList<JTextField>();

	private SignStreamMouseListener listener = null;

	public SegmentGraphicRepresentation(SignStreamSegmentPanel segmentPanel,
			Segment segment) {
		setOpaque(false);
		this.segmentPanel = segmentPanel;
		this.segment = segment;
		this.wrappers = segment.getFieldWrappersCollection();
	}

	/**
	 * Updating local FieldWrappersCollection from local Segment.
	 * Segment object needs to be set elsewhere prior to calling this function
	 */
	public void update() {
		this.wrappers = segment.getFieldWrappersCollection();

		resizePanel();
		repaint();
		resizePanel();
	}

	public void setMouseListener(SignStreamMouseListener listener) {
		this.listener = listener;
	}

	public void activateMouseListener() {
		addMouseListener(listener);
		addMouseMotionListener(listener);
	}

	public void passivateMouseListener() {
		removeMouseListener(listener);
		removeMouseMotionListener(listener);
	}

	public void setFieldWrappersCollection(FieldWrappersCollection wrappers) {
		this.wrappers = wrappers;
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

	public FieldWrappersCollection getFieldWrappersCollection() {
		return wrappers;
	}


	public void setNotesRepresentation(NotesRepresentation notes) {
		this.notes = notes;
	}

	public NotesRepresentation getNotesRepresentation() {
		return notes;
	}

	public boolean isSliderMovable(Point p) {
		int Y = (int) p.getY();
		if (Y < SegmentGraphReprUtil.axesBottomHeight)
			return true;
		return false;
	}

	public void resizePanel() {
		setPreferredSize(getPreferredDimension());
	}

	public Dimension getPreferredDimension() {
		double X = wrappers.getEndTime() - wrappers.getStartTime() + 20;
		double Y = wrappers.getSize() * (SegmentGraphReprUtil.cellHeight + 1);
		double X1 = getPreferredSize().getWidth();
		double Y1 = getPreferredSize().getHeight();

		X = X > X1 ? X : X1;
		Y = Y > Y1 ? Y : Y1;

		return new Dimension((int) X, (int) Y);
	}

	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		paint(g);
	}

	@Override
	public void paint(Graphics g) {
		for (JTextField tf : addedObjects) {
			remove(tf);
		}

		Graphics2D g2d = (Graphics2D) g;
		
		if (segment.getParentField() != null) {
			g2d.setColor(Palette.NOT_VALID_BACKGROUND_COLOR);
			g2d.fillRect(0, 0, getWidth(), getHeight());
			
			g2d.setColor(Palette.BACKGROUND_COLOR);
			
			ArrayList<PresentationEventsEntity> utterances = segment.getParentField().getEntities();
			
			for (PresentationEventsEntity utterance: utterances) {
				int utteranceStartTime = utterance.getStartTime();
				int utteranceStartCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor()
						.convertTimeToCoordinate(utteranceStartTime);
				int utteranceEndTime = utterance.getEndTime();
				int utteranceEndCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor()
						.convertTimeToCoordinate(utteranceEndTime);
				
				int width = utteranceEndCoord - utteranceStartCoord;
		
				g2d.fillRect(utteranceStartCoord, 0, width, getHeight());
			}
		} else {	// parentField will be null for DatabaseView, so ignore the utterances
			g2d.setColor(Palette.BACKGROUND_COLOR);
			g2d.fillRect(0, 0, getWidth(), getHeight());
		}

		// draw axis
		ArrayList<FieldWrapper> visibleWrappers = wrappers.getVisibleWrappers();

		if (visibleWrappers.size() <= 0)
			return;

		g2d.setColor(Palette.AXIS_COLOR);
		g2d.draw(new Line2D.Double(0, SegmentGraphReprUtil.axesBottomHeight
				+ SegmentGraphReprUtil.linenHeight, getWidth(),
				SegmentGraphReprUtil.axesBottomHeight
						+ SegmentGraphReprUtil.linenHeight));

		int ii = 0;
		for (ii = 1; ii < visibleWrappers.size(); ii++) {
			g2d.setColor(Palette.AXIS_COLOR);
			g2d.draw(new Line2D.Double(0, SegmentGraphReprUtil.cellHeight * ii
					+ SegmentGraphReprUtil.axesTopHeight
					+ SegmentGraphReprUtil.linenHeight, getWidth(),
					SegmentGraphReprUtil.cellHeight * ii
							+ SegmentGraphReprUtil.axesTopHeight
							+ SegmentGraphReprUtil.linenHeight));
		}

		g2d.draw(new Line2D.Double(0, SegmentGraphReprUtil.cellHeight * ii
				+ SegmentGraphReprUtil.axesTopHeight
				+ SegmentGraphReprUtil.linenHeight, getWidth(),
				SegmentGraphReprUtil.cellHeight * ii
						+ SegmentGraphReprUtil.axesTopHeight
						+ SegmentGraphReprUtil.linenHeight));

		// draw lines
		for (int j = 0; j < visibleWrappers.size(); j++) {
			double Y = SegmentGraphReprUtil.cellHeight * (j + 1)
					+ SegmentGraphReprUtil.valueHeight
					+ SegmentGraphReprUtil.linenHeight;
			FieldWrapper wrapper = visibleWrappers.get(j);

			if (wrapper.isSelectedNHighlighted()) {
				Color oldColor = g2d.getColor();
				g2d.setColor(UIPalette.getBackgroundSelection());

				int y = SegmentGraphReprUtil.cellHeight * j
						+ SegmentGraphReprUtil.axesTopHeight
						+ SegmentGraphReprUtil.linenHeight + 2;
				g2d.fillRect(0, y, getWidth(),
						SegmentGraphReprUtil.cellHeight - 4);

				g2d.setColor(oldColor);
			}

			wrapper.getField().paint(g2d, Y,
					SegmentGraphReprUtil.getGraphReprFont());

			if (wrapper instanceof GlossFieldWrapper
					&& !(wrapper instanceof RightLeftHandFieldWrapper)) {
				Field fld = wrapper.getField();

				if (fld instanceof GlossField) {
					GlossField gfld = (GlossField) fld;
					NewGlossChainedEvent newEvent = gfld
							.getNewGlossChainedEvent();
					if (newEvent != null) {
						JTextField txtFld = newEvent.getTextField();
						int s = newEvent.getStartTimeCoordinate();
						txtFld.setLocation(s, (int) (Y - 18 + 2));
						add(txtFld);
						addedObjects.add(txtFld);
					}
				}
			}
		}
		// draw slider
		Slider slider = segmentPanel.getSlider();
		if (slider != null) {
			g2d.setColor(Palette.SLIDER_COLOR);
			int sliderPosition = slider.getSliderPosition();
			g2d.draw(new Line2D.Double(sliderPosition, 0, sliderPosition,
					getHeight()));
		}
	}

	// for scrolling!!!
	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}

	public int getScrollableUnitIncrement(Rectangle visibleRect,
			int orientation, int direction) {

		int currentPosition = 0;
		if (orientation == SwingConstants.HORIZONTAL) {
			currentPosition = visibleRect.x;
		} else {
			currentPosition = visibleRect.y;
		}

		// Return the number of pixels between currentPosition
		// and the nearest tick mark in the indicated direction.
		if (direction < 0) {
			int newPosition = currentPosition
					- (currentPosition / SegmentGraphReprUtil.maxUnitIncrement)
					* SegmentGraphReprUtil.maxUnitIncrement;
			return (newPosition == 0) ? SegmentGraphReprUtil.maxUnitIncrement
					: newPosition;
		} else {
			return ((currentPosition / SegmentGraphReprUtil.maxUnitIncrement) + 1)
					* SegmentGraphReprUtil.maxUnitIncrement - currentPosition;
		}
	}

	public int getScrollableBlockIncrement(Rectangle visibleRect,
			int orientation, int direction) {
		if (orientation == SwingConstants.HORIZONTAL) {
			return visibleRect.width - SegmentGraphReprUtil.maxUnitIncrement;
		} else {
			return visibleRect.height - SegmentGraphReprUtil.maxUnitIncrement;
		}
	}

	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	public boolean getScrollableTracksViewportHeight() {
		return false;
	}

	@Override
	public void update(SSObservable subj, Object o) {
		repaint();
	}

	@Override
	public String toString() {
		StringBuffer bf = new StringBuffer("SegmentGraphicRepresentation: ");

		ArrayList<FieldWrapper> vWrappers = wrappers.getVisibleWrappers();

		bf.append("size = ");
		bf.append(wrappers.getWrappers().size());
		bf.append("\n");

		for (int i = 0; i < vWrappers.size(); i++) {
			FieldWrapper wrp = vWrappers.get(i);
			bf.append("\n name = ");
			bf.append(wrp.getField().getName());
			bf.append(", fieldID = ");
			bf.append(wrp.getField().getFieldID());
		}

		return bf.toString();
	}
}
