package edu.bu.signstream.grepresentation.view;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;

public class UtteranceViewHeader extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private SignStreamSegmentPanel segmentPanel;
	private UtteranceView utteranceView;
	
	private JPanel overviewWindowButtonPanel;
	private JButton overviewWindowButton;
	private TimelineView timelineView;
	
	public UtteranceViewHeader(SignStreamSegmentPanel segmentPanel,
			UtteranceView utteranceView) {
		this.segmentPanel = segmentPanel;
		this.utteranceView = utteranceView;	
		
		overviewWindowButtonPanel = new JPanel();
		overviewWindowButtonPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		overviewWindowButtonPanel.setBackground(Palette.BACKGROUND_COLOR);
		overviewWindowButtonPanel.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		overviewWindowButtonPanel.setMinimumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		overviewWindowButtonPanel.setMaximumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		
		// button to return to overview window
		String overviewWindowButtonText = "Back To Database View";
		overviewWindowButton = new JButton(overviewWindowButtonText);
	    UserInterfaceUtil.setButtonsLNF(overviewWindowButton);
	    overviewWindowButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				loadDatabaseView();				
			}	    	
	    });
	    overviewWindowButtonPanel.add(overviewWindowButton);
		
		// timeline view
		timelineView = new TimelineView(segmentPanel);
		timelineView.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH + 
				SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		timelineView.setMinimumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH + 
				SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	    add(overviewWindowButtonPanel);
		add(timelineView);
	}
	
	public void loadDatabaseView() {
		// transplanted most code below from DbNavigationPanel
		
		SegmentGraphReprUtil.utteranceMovieTimeOnset = -1;
		SegmentGraphReprUtil.utteranceMovieTimeOffset = -1;
		
		SegmentGraphReprUtil.utteranceViewVisibleMovieOnset = -1;
		SegmentGraphReprUtil.utteranceViewVisibleMovieOffset = -1;

		segmentPanel.updateLoadedDatabase();
		SS3Database db = segmentPanel.getSS3Database();
		segmentPanel.loadDatabase(db);

		// // calculate coef that will allow users to see entire
		// // duration of a video file on a screen
		int movieDuration = segmentPanel
				.getMultipleMovieController().getMovieDuration();
		int packetDuration = (int) segmentPanel
				.getMultipleMovieController().getPacketDuration();
		
		try {
			MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(null);
			mToolBar.selectOverviewPanel();
		} catch(Exception ex) {
			System.out.println(ex.getMessage());
		}

		if (movieDuration == 0) {
			movieDuration = db.getMovieDuration();
			packetDuration = db.getPacketDuration();
		}

		if (movieDuration == 0) {
			SS3GlossWindow glossWindow = db.getSS3GlossWindow();
			ArrayList<SS3TemporalPartition> temporalPartitions = glossWindow
					.getTemporalPartitions();
			for (SS3TemporalPartition temporalPartition : temporalPartitions) {
				SS3GlossWindowSegment segment = temporalPartition
						.getGlossWindowSegment();
				ArrayList<SS3GlossWindowItem> items = segment
						.getSS3GlossWindowItems();
				for (SS3GlossWindowItem item : items) {
					int time = item.getEndTimeInfo().getMovieTime();
					if (time > movieDuration)
						movieDuration = time;
				}
			}
			packetDuration = 1001;
			movieDuration = movieDuration + 20 * packetDuration;
		}

		segmentPanel.getZoomer()
				.getCoordinateTimeConvertor()
				.setPacketDuration(packetDuration);

		double pixelsInTimeframe = db
				.getTemporalPartitionPixelInTimeFrame();

		int width = segmentPanel
				.getDatabaseView().getVisibleWidth2();

		if (pixelsInTimeframe <= 0) {
			segmentPanel
					.getZoomer()
					.fillEntirePage(movieDuration, packetDuration,
							width);
		} else {
			segmentPanel.getZoomer()
					.getCoordinateTimeConvertor()
					.setPixelInTimeFrame(pixelsInTimeframe);
		}

		int movieTime = SS3Singleton.getVideoControlManager()
				.getMultipleMovieController().getCurrentTime();
		segmentPanel.getSlider()
				.setSliderPositionTime(movieTime);		
	}	
	
	public TimelineView getTimelineView() {
		return timelineView;
	}

}
