package edu.bu.signstream.grepresentation.view.main;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.border.TitledBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingSchemeCollection;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3StatementField;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Painter;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.displayTimePeriod.DisplayTimeViewWrapper;
import edu.bu.signstream.grepresentation.fields.freeTextField.FreeTextEntity;
import edu.bu.signstream.grepresentation.fields.freeTextField.FreeTextField;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredText;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntityCollection;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.Text;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.SegmentFieldWrapper;
import edu.bu.signstream.grepresentation.listener.SegmentMouseListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.DatabaseView;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.LabelViewTemporalPartitions;
import edu.bu.signstream.grepresentation.view.RootLabelObject;
import edu.bu.signstream.grepresentation.view.TreeNodeVector;
import edu.bu.signstream.grepresentation.view.UtteranceView;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;
import edu.bu.signstream.media.fileNavigation.MediaDelegate;
import edu.bu.signstream.navigation.DatabaseDelegate;
import edu.bu.signstream.navigation.DbNavigationPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.video.MultipleMovieController;
import edu.bu.signstream.ui.video.XugglerMovie;

public class SignStreamSegmentPanel extends JPanel implements ActionListener {
	private static final long serialVersionUID = 1L;

	private boolean isUtteranceLoaded = false;

	private DbNavigationPanel navigationPanel;

	private SS3Database database;

	private JFrame parentFrame;

	private final Hashtable<JDialog, java.io.File> openedMediaDialogs = new Hashtable<JDialog, java.io.File>();

	private Painter painter = null;

	private ArrayList<Segment> segments = new ArrayList<Segment>();
	
	private Segment selectedSegment = null;

	private boolean entityLevelSelected = false;

	private ToolBarPanels tbPanel = null;

	private final JSplitPane sp = new JSplitPane();

	private Event selectedEvent = null;

	private FieldWrapper selectedWrapper = null;

	// need
	SegmentMouseListener listener = null;

	// need
	private DatabaseView databaseView = null;
	private UtteranceView utteranceView = null;

	// need
	public UtteranceView getUtteranceView() {
		return utteranceView;
	}

	// need
	public DatabaseView getDatabaseView() {
		return databaseView;
	}

	// need
	public void setSS3Database(SS3Database database) {
		this.database = database;
	}

	// need
	public SignStreamSegmentPanel(DatabaseDelegate delegate, JFrame parentFrame) {
		this.parentFrame = parentFrame;

		SS3GlossWindow glsWindow = new SS3GlossWindow();

		SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
		displayTimePeriod.setLabel("Display Time Period");
		glsWindow.setDisplayTimePeriod(displayTimePeriod);

		SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
		temporalPartition.setLabel("Temporal Partition 1");

		SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
		tpSegment.setLabel("Segment Tier 1");
		tpSegment.setID("" + Util.getUniqueNumber());
		temporalPartition.addTemporalPartitionSegment(tpSegment);
		temporalPartition.setDefaultSegment(tpSegment);
		
		ArrayList<SS3TemporalPartition> partitions = new ArrayList<SS3TemporalPartition>();
		partitions.add(temporalPartition);
		glsWindow.setTemporalPartitions(partitions);

		this.database = new SS3Database(new TimeInfo(), new TimeInfo());
		database.setSS3GlossWindow(glsWindow);
		Excerpt excerpt = new Excerpt(Util.getUniqueNumber(),
				database.getFileName());
		database.setExcerptObj(excerpt);

		SS3GlossWindow window = database.getSS3GlossWindow();
		Segment segment = window.getSegment();
		if (segment == null) {
			segment = segments.size() > 0 ? segments.get(0) : new Segment(this, null);
			window.setSegment(segment);
		}
		this.selectedSegment = segment;

		RootLabelObject rootObject = new RootLabelObject();
		LabelObject presentationObject = new LabelObject();
		rootObject.setPresentationLabelObject(presentationObject);

		SS3GlossWindowSegment dislayTimePeriod = window.getDisplayTimePeriod();

		PresentationField displayTimePeriodField = new PresentationField(null,
				dislayTimePeriod, this);
		DisplayTimeViewWrapper displayTimePeriodFieldWrapper = new DisplayTimeViewWrapper(
				displayTimePeriodField, this);
		displayTimePeriodFieldWrapper.setVisible(true);

		LabelObject dObj = new LabelObject();
		dObj.setLabel(dislayTimePeriod.getLabel());
		dObj.setFieldID(displayTimePeriodField.getFieldID());
		dObj.setWrapper(displayTimePeriodFieldWrapper);

		presentationObject.addChild(dObj);

		segment.addFieldWrapper(displayTimePeriodFieldWrapper);

		for (SS3TemporalPartition partition : partitions) {
			ArrayList<SS3GlossWindowSegment> segments = partition
					.getTemporalPartitionSegments();

			PresentationField pf = new PresentationField(partition, this);
			PresentationFieldWrapper pfw = new PresentationFieldWrapper(pf,
					this);
			pfw.setVisible(true);

			LabelObject temporalPartitionObject = new LabelObject();
			temporalPartitionObject.setLabel(partition.getLabel());
			temporalPartitionObject.setFieldID(pf.getFieldID());
			temporalPartitionObject.setWrapper(pfw);
			segment.addFieldWrapper(pfw);

			for (SS3GlossWindowSegment sgm : segments) {
				PresentationField spf = new PresentationField(partition, sgm, this);

				ArrayList segmentEntities = spf.getEntities();
				for (int i = 0; i < segmentEntities.size(); i++) {
					PresentationEventsEntity pEntity = (PresentationEventsEntity) segmentEntities
							.get(i);
					PresentationEventsEntity masterEntity = pf
							.getEntityWithStartTime(pEntity.getStartTimeInfo()
									.getMovieTime());
					if (masterEntity != null) {
						masterEntity.addDependentEntity(pEntity);
					} else {
						SS3SignStreamApplication.logger.setLevel(Level.SEVERE);
						SS3SignStreamApplication.logger
								.info("Could not find master entity for entity "
										+ pEntity.getToolTipText());
					}
				}

				SegmentFieldWrapper sw = new SegmentFieldWrapper(spf, this,
						partition.getLabel(), sgm.getLabel());
				sw.setVisible(true);

				pfw.addSegmentFieldWrapper(sw);

				LabelObject temporalPartitionPresentationObject = new LabelObject();
				temporalPartitionPresentationObject
						.setFieldID(spf.getFieldID());
				temporalPartitionPresentationObject.setLabel(sgm.getParticipantLabel() + sgm.getLabel());
				temporalPartitionPresentationObject.setWrapper(sw);

				temporalPartitionObject
						.addChild(temporalPartitionPresentationObject);
				segment.addFieldWrapper(sw);
			}

			presentationObject.addChild(temporalPartitionObject);
		}

		segment.setRootLabelObject(rootObject);

		delegate = new DatabaseDelegate(this);
		this.navigationPanel = new DbNavigationPanel(this, delegate);
		databaseView = new DatabaseView(segment, this);

		TitledBorder border = new TitledBorder("");
		databaseView.setBorder(border);

		tbPanel = new ToolBarPanels(this);
		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(this);
		tbPanel.addMediaToolBar(mediaToolBar);

		sp.setLeftComponent(navigationPanel);
		sp.setRightComponent(databaseView);

		sp.setDividerLocation(200);

		setLayout(new BorderLayout());
		add(tbPanel, BorderLayout.NORTH);
		add(sp, BorderLayout.CENTER);

		updateJMenuBar();

		repaint();
	}

	// need
	public void loadDatabase(SS3Database database) {
		if (database == null) {
			SS3GlossWindow glsWindow = new SS3GlossWindow();

			SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
			displayTimePeriod.setLabel("Display Time Period");
			glsWindow.setDisplayTimePeriod(displayTimePeriod);

			SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
			temporalPartition.setLabel("Temporal Partition 1");

			SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
			tpSegment.setLabel("Segment Tier 1");
			tpSegment.setID("" + Util.getUniqueNumber());
			temporalPartition.addTemporalPartitionSegment(tpSegment);
			temporalPartition.setDefaultSegment(tpSegment);

			ArrayList<SS3TemporalPartition> partitions = new ArrayList<SS3TemporalPartition>();
			
			partitions.add(temporalPartition);
			
			glsWindow.setTemporalPartitions(partitions);

			database = new SS3Database(new TimeInfo(), new TimeInfo());
			database.setSS3GlossWindow(glsWindow);
			Excerpt excerpt = new Excerpt(Util.getUniqueNumber(),
					database.getFileName());
			database.setExcerptObj(excerpt);
		}

		// regardless of which db is being loaded, we always want to do this check below
		// in case we are switching between two different db's, from the UtteranceView perspective
		if (sp.getRightComponent() instanceof DatabaseView) {
			sp.repaint();
		} else {
			sp.remove(utteranceView);
			sp.setLeftComponent(navigationPanel);
			sp.setRightComponent(databaseView);

			sp.setDividerLocation(200);
		}
			
		if (this.database != database) {

		DefaultMutableTreeNode databaseRoot = database.getDbNavigationPanelNode();

		DefaultMutableTreeNode delegateRoot = SS3Singleton.getSignStreamApplication().
				getDatabaseDelegate().getMutableTreeNode();

		DefaultMutableTreeNode refreshRoot = databaseRoot != null ? databaseRoot : delegateRoot;

		// update dbnavigation panel	
		getDbNavigationPanel().refresh(refreshRoot);

		// update panel title
		if (refreshRoot.getChildCount() > 0) {
			DefaultMutableTreeNode node1 = (DefaultMutableTreeNode) refreshRoot
					.getChildAt(0);

			Object uo = node1.getUserObject();
			if (uo instanceof String) {
				SS3Singleton.getSignStreamApplication().setTitle((String) uo);
			}

			getDbNavigationPanel().getJTree().expandRow(1);
			getDbNavigationPanel().getJTree().setSelectionRow(2);
		}

		SS3SignStreamApplication.logger.log(Level.INFO,
				"Loading a database [" + database.getExcerpt()
				+ "] presentation view.");

		ArrayList<SS3MediaFile> movies = new ArrayList<SS3MediaFile>();
		String loadingMediaFilesMsg = "Loading media files [";

		Excerpt excerpt = database.getExcerptObj();

		ArrayList<Object> mIDs = excerpt.getMediaFiles();
		for (int i = 0; i < mIDs.size(); i++) {
			String id = (String) mIDs.get(i);

			SS3MediaFile mediaFile = database.getMediaFile(id);
			if (mediaFile != null) {
				movies.add(mediaFile);
				loadingMediaFilesMsg = loadingMediaFilesMsg
						.concat(mediaFile.getFile().getName());

				if (i < mIDs.size() - 1)
					loadingMediaFilesMsg = loadingMediaFilesMsg
					.concat(", ");
			}
		}

		loadingMediaFilesMsg = loadingMediaFilesMsg.concat("]...");

		SS3SignStreamApplication.logger.log(Level.INFO,
				loadingMediaFilesMsg);

		showVideoFiles(movies, database.getVideoOnScreenLocations());

		// update coding scheme
		SS3CodingScheme defSchema = SS3Singleton
				.getSignStreamApplication().getDefaultSS3CodingScheme();
		SS3CodingScheme localSchema = database.getLocalSS3CodingScheme();
		HashMap<String, SS3Field> localFields = localSchema.getFields();
		Set<Map.Entry<String, SS3Field>> set = localFields.entrySet();
		Iterator<Map.Entry<String, SS3Field>> i = set.iterator();
		while (i.hasNext()) {
			Map.Entry<String, SS3Field> me = i.next();
			SS3Field lField = me.getValue();
			String fieldID = lField.getId();
			SS3Field dField = defSchema.getField(fieldID);				
			if (dField == null) {
				defSchema.addField(lField);
			} else {
				ArrayList<SS3FieldValue> values = lField.getValuesAsList();
				for (SS3FieldValue value : values) {
					dField.addValue(value);
				}

				dField.setColor(lField.getColor());
			}
		}

		SS3CodingSchemeCollection csCollection = SS3Singleton
				.getSS3CodingSchemeCollection();
		csCollection.loadDefaultCodingScheme(defSchema);

		int currentTime = database.getStartTimeInfo().getMovieTime();
		MultipleMovieController contr = SS3Singleton
				.getVideoControlManager().getMultipleMovieController();

		if (currentTime > contr.getMovieDuration())
			currentTime = contr.getMovieDuration();

		contr.selectSegment(currentTime, database.getEndTimeInfo()
				.getMovieTime());
		contr.setCurrentTime(currentTime);

		int movieDuration = getMultipleMovieController().getMovieDuration();
		int packetDuration = (int) getMultipleMovieController()
				.getPacketDuration();

		if (movieDuration == 0) {
			movieDuration = database.getMovieDuration();
			packetDuration = database.getPacketDuration();
		}

		double width = databaseView.getVisibleWidth2();
		getZoomer().fillEntirePage(movieDuration, packetDuration, width);

		update(database);

		if (movieDuration == 0) {
			SS3GlossWindow glossWindow = database.getSS3GlossWindow();
			ArrayList<SS3TemporalPartition> temporalPartitions = glossWindow.getTemporalPartitions();


			for ( SS3TemporalPartition temporalPartition : temporalPartitions ) {
				SS3GlossWindowSegment segment = temporalPartition
						.getGlossWindowSegment();
				ArrayList<SS3GlossWindowItem> items = segment
						.getSS3GlossWindowItems();
				for (SS3GlossWindowItem item : items) {
					int time = item.getEndTimeInfo().getMovieTime();
					if (time > movieDuration)
						movieDuration = time;
				}
			}
		}


		databaseView.updateDrawingsAreaSize();

		}

		repaint();
		this.database = database;

		// segments are from UtteranceView - clean from memory after going back to DatabaseView
		segments.clear();

		repaint();
		updateUI();
		//

		isUtteranceLoaded = false;
	}

	// value - SS3MediaFile
	private final ArrayList<SS3MediaFile> availableMediaFiles = new ArrayList<SS3MediaFile>();

	public void setVisible(Event event) {
		int startCoord = event.getStartTimeCoordinate();
		int endCoord = event.getEndTimeCoordinate();

		if (event instanceof PresentationEventsEntity
				|| event instanceof PresentationEvent) {
			databaseView.setVisible(startCoord, endCoord);
		} else {
			utteranceView.setVisible(startCoord, endCoord);
		}
	}

	public void setVisible(int startCoord, int endCoord, boolean isPresentation) {
		if (isPresentation) {
			if(databaseView != null) {
				databaseView.setVisible(startCoord, endCoord);
			}
		} else {
			if(utteranceView!=null) {
				utteranceView.setVisible(startCoord, endCoord);
			}
		}
	}

	public void setSelectedFieldWrapper(FieldWrapper selectedWrapper) {
		this.selectedWrapper = selectedWrapper;
	}

	public FieldWrapper getSelectedFieldWrapper() {
		return selectedWrapper;
	}

	public void setSelectedEvent(Event selectedEvent) {
		this.selectedEvent = selectedEvent;
	}

	public Event getSelectedEvent() {
		return selectedEvent;
	}

	public Segment getSegment(String segmentID) {
		for (Segment segment : segments) {
			if (segment.equals(segmentID)) {
				return segment;
			}
		}
		return null;
	}

	public ToolBarPanels getToolBarPanels() {
		return tbPanel;
	}

	public boolean isEntityLevelSelected() {
		return entityLevelSelected;
	}

	public Painter getPainter() {
		if (painter == null)
			painter = new Painter();
		return painter;
	}

	public void update(SS3Database database) {
		setSS3Database(database);
		
		SS3GlossWindow window = database.getSS3GlossWindow();
		Segment segment = selectedSegment;
			
		segment.getFieldWrappersCollection().removeAll();
			
		// most of below code is transplanted from SignStreamSegmentPanel constructor
		// since when an update is performed on the view it must act like a new creation

		RootLabelObject rootObject = new RootLabelObject();
		LabelObject presentationObject = new LabelObject();
		rootObject.setPresentationLabelObject(presentationObject);

		SS3GlossWindowSegment dislayTimePeriod = window.getDisplayTimePeriod();

		PresentationField displayTimePeriodField = new PresentationField(null, 
				dislayTimePeriod, this);
		DisplayTimeViewWrapper displayTimePeriodFieldWrapper = new DisplayTimeViewWrapper(
				displayTimePeriodField, this);
		displayTimePeriodFieldWrapper.setVisible(true);

		LabelObject dObj = new LabelObject();
		dObj.setLabel(dislayTimePeriod.getLabel());
		dObj.setFieldID(displayTimePeriodField.getFieldID());
		dObj.setWrapper(displayTimePeriodFieldWrapper);

		presentationObject.addChild(dObj);

		segment.addFieldWrapper(displayTimePeriodFieldWrapper);
		
		//Hashtable<String, SS3TemporalPartition> partitions = window.getTemporalPartitions();
		ArrayList<SS3TemporalPartition> partitions = window.getTemporalPartitions();
		
		//for (SS3TemporalPartition partition : partitions.values()) {
		for (SS3TemporalPartition partition : partitions) {
			ArrayList<SS3GlossWindowSegment> segments = partition
					.getTemporalPartitionSegments();

			PresentationField pf = new PresentationField(partition, this);
			PresentationFieldWrapper pfw = new PresentationFieldWrapper(pf,	this);
			pfw.setVisible(true);

			LabelObject temporalPartitionObject = new LabelObject();
			temporalPartitionObject.setLabel(partition.getLabel());
			temporalPartitionObject.setFieldID(pf.getFieldID());
			temporalPartitionObject.setWrapper(pfw);
			segment.addFieldWrapper(pfw);

			for (SS3GlossWindowSegment sgm : segments) {
				PresentationField spf = new PresentationField(partition, sgm, this);

				ArrayList segmentEntities = spf.getEntities();
				for (int i = 0; i < segmentEntities.size(); i++) {
					PresentationEventsEntity pEntity = (PresentationEventsEntity) segmentEntities
							.get(i);
					PresentationEventsEntity masterEntity = pf
							.getEntityWithStartTime(pEntity.getStartTimeInfo()
									.getMovieTime());
					if (masterEntity != null) {
						masterEntity.addDependentEntity(pEntity);
					} else {
						SS3SignStreamApplication.logger.setLevel(Level.SEVERE);
						SS3SignStreamApplication.logger
								.info("Could not find master entity for entity "
										+ pEntity.getToolTipText());
					}
				}

				SegmentFieldWrapper sw = new SegmentFieldWrapper(spf, this,
						partition.getLabel(), sgm.getLabel());
				sw.setVisible(true);

				pfw.addSegmentFieldWrapper(sw);

				LabelObject temporalPartitionPresentationObject = new LabelObject();
				temporalPartitionPresentationObject
						.setFieldID(spf.getFieldID());
				String temporalPartitionPresentationObjectLabel = 
						sgm.getParticipantLabel() == null || sgm.getParticipantLabel().isEmpty()
						? sgm.getLabel()
						: sgm.getParticipantLabel() + Util.SEGMENT_TIER_SEPARATOR + sgm.getLabel();
				temporalPartitionPresentationObject.setLabel(temporalPartitionPresentationObjectLabel);
				temporalPartitionPresentationObject.setWrapper(sw);
				temporalPartitionPresentationObject.setDefaultSegmentTier(sgm.isDefault());

				temporalPartitionObject
						.addChild(temporalPartitionPresentationObject);
				segment.addFieldWrapper(sw);
			}

			presentationObject.addChild(temporalPartitionObject);
		}

		segment.setRootLabelObject(rootObject);
		
//		databaseView.zoom(1);
		databaseView.updatePanels();
	}
	
	/**
	 * Add a new LabelObject and Wrapper to represent a new segment tier
	 * Then call update() to reflect changes in the view.
	 */
	public void createSegment(SS3Participant participant) {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();

		Object obj = node.getUserObject();

		// if child node (segment tier) is selected, then select its temporal partition "parent"
		if (node.getLevel() == 2) {
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node
					.getParent();
			obj = parentNode.getUserObject();
			node = parentNode;
		}

		LabelObject partitionObj = null;

		if (obj instanceof LabelObject) {
			partitionObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			partitionObj = vector.getObject();
		}
		
		String partitionLabel = partitionObj.getLabel();
		ArrayList<SS3TemporalPartition> partitions = database.getSS3GlossWindow().getTemporalPartitions();
		
		SS3TemporalPartition partition = new SS3TemporalPartition();
		
		for (SS3TemporalPartition tempPartition : partitions) {
			if(tempPartition.getLabel().equals(partitionLabel))
			{
				partition = tempPartition;
			}
		}
		
		// check that no children of node (segments) have children of their own
		if (node.getDepth() <= 1) {
			String segmentLabel = "Segment Tier 1";
			if (node.getChildCount() > 0) {
				int counter = node.getChildCount() + 1;
				segmentLabel = "Segment Tier " + counter;
				
				while (partition.getTemporalPartitionSegmentByLabel(segmentLabel) != null) {
					counter++;
					segmentLabel = "Segment Tier " + counter;
				}
			}
			 
			// segment to add
			SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
			tpSegment.setParticipant(participant);
			tpSegment.setLabel(segmentLabel);
			partition.addTemporalPartitionSegment(tpSegment);
			
			// addChild() makes segmentObj visible to LabelViewTemporalPartitions.populateTree()
			LabelObject segmentObj = new LabelObject("", tpSegment.getLabel());			
			partitionObj.addChild(segmentObj);
			
			// wrappers to make the new segment row clickable, and wrap segment-wrapper inside partition-wrapper
			PresentationField spf = new PresentationField(partition, tpSegment, this); // creates unique ID here
			SegmentFieldWrapper sw = new SegmentFieldWrapper(spf, this, 
					partition.getLabel(), tpSegment.getLabel());
			
			PresentationFieldWrapper pfw = (PresentationFieldWrapper)partitionObj.getWrapper();
			PresentationField pf = (PresentationField) pfw.getField();

			// clone the entities from the partition field, along with the item inside each
			// see PresentationFieldWrapper.mouseReleased() for source of most of below code
			ArrayList<EventsEntity> entities = pf.getEntities();
			for (EventsEntity entity : entities) {
				PresentationEventsEntity pEntity = (PresentationEventsEntity)entity;	// this cast should always work
				PresentationEventsEntity clonedEntity = pEntity.clone();
				
				pEntity.getStartTimeInfo().addDependentMovieTime(
						clonedEntity.getStartTimeInfo());
				pEntity.getEndTimeInfo().addDependentMovieTime(
						clonedEntity.getEndTimeInfo());
				pEntity.addDependentEntity(clonedEntity);
				spf.addEventsEntity(clonedEntity);	// spf wrapped by sw
				
				SS3GlossWindowItem pItem = pEntity.getEvent().getSS3GlossWindowItem();
				SS3GlossWindowItem clonedItem 
					= new SS3GlossWindowItem(partition.getLabel(), tpSegment.getLabel());
				clonedItem.setId(Util.getUniqueNumber());
				clonedItem.setStartTimeInfo(pItem.getStartTimeInfo());
				clonedItem.setEndTimeInfo(pItem.getEndTimeInfo());
				clonedItem.setLabel(pEntity.getEvent().getText()); 
				
				PresentationEvent pEvent = clonedEntity.getEvent();
				pEvent.setSS3GlossWindowItem(clonedItem);
				pItem.addSS3GlossWindowItem(clonedItem);

				spf.getSS3GlossWindowSegment().addSS3GlossWindowItem(clonedItem);	// spf wrapped by sw
			}
			
			pfw.addSegmentFieldWrapper(sw);
			segmentObj.setWrapper(sw);	// sw wrapping spf
						
			update(database);
		}
	}

	/**
	 * Add a new LabelObject and Wrapper to represent a new temporal partition
	 * Then call update() to reflect changes in the view.
	 */
	public void createTemporalPartition(SS3Participant participant) {		
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();

		// if child node (segment tier) is selected, then select its temporal partition "parent"
		if (node.getLevel() == 2) {
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node
					.getParent();
			node = parentNode;
		}

		LabelObject partitionObj = new LabelObject();
		
		String partitionLabel = partitionObj.getLabel();

		
		// check that no children of node (segments) have children of their own
		if (node.getDepth() <= 1) {
			String tpLabel = "Temporal Partition 1";
			String segmentLabel = "Segment Tier 1";
			
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node.getParent();
			
			int index = parentNode.getChildCount();
			
			if (parentNode.getChildCount() > 0) {

				do
				{
					tpLabel = "Temporal Partition " + index++;
				}while(database.getSS3GlossWindow().getTemporalPartitionByLabel(tpLabel) != null);
			}
			
			SS3TemporalPartition newPartition = new SS3TemporalPartition();
			newPartition.setLabel(tpLabel);

			RootLabelObject rootObject = new RootLabelObject();
			rootObject.setPresentationLabelObject(partitionObj);
			 
			// segment to add
			SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
			tpSegment.setParticipant(participant);
			tpSegment.setLabel(segmentLabel);
			tpSegment.setID("" + Util.getUniqueNumber());
			newPartition.addTemporalPartitionSegment(tpSegment);
			newPartition.setDefaultSegment(tpSegment);	// tpSegment is the only ST in this new TP, thus it must be the default
			
			// addChild() makes segmentObj visible to LabelViewTemporalPartitions.populateTree()
			LabelObject segmentObj = new LabelObject("", tpSegment.getLabel());			
			partitionObj.addChild(segmentObj);
			
			// wrappers to make the new segment row clickable, and wrap segment-wrapper inside partition-wrapper
			PresentationField spf = new PresentationField(newPartition, tpSegment, this); // creates unique ID here
			SegmentFieldWrapper sw = new SegmentFieldWrapper(spf, this, 
					newPartition.getLabel(), tpSegment.getLabel());
			
			PresentationField pf = new PresentationField(newPartition, this);
			PresentationFieldWrapper pfw = new PresentationFieldWrapper(pf, this);
			pfw.setVisible(true);

			// clone the entities from the partition field, along with the item inside each
			// see PresentationFieldWrapper.mouseReleased() for source of most of below code
			ArrayList<EventsEntity> entities = pf.getEntities();
			for (EventsEntity entity : entities) {
				PresentationEventsEntity pEntity = (PresentationEventsEntity)entity;	// this cast should always work
				PresentationEventsEntity clonedEntity = pEntity.clone();
				
				pEntity.getStartTimeInfo().addDependentMovieTime(
						clonedEntity.getStartTimeInfo());
				pEntity.getEndTimeInfo().addDependentMovieTime(
						clonedEntity.getEndTimeInfo());
				pEntity.addDependentEntity(clonedEntity);
				spf.addEventsEntity(clonedEntity);	// spf wrapped by sw
				
				SS3GlossWindowItem pItem = pEntity.getEvent().getSS3GlossWindowItem();
				SS3GlossWindowItem clonedItem 
					= new SS3GlossWindowItem(newPartition.getLabel(), tpSegment.getLabel());
				clonedItem.setId(Util.getUniqueNumber());
				clonedItem.setStartTimeInfo(pItem.getStartTimeInfo());
				clonedItem.setEndTimeInfo(pItem.getEndTimeInfo());
				clonedItem.setLabel(pEntity.getEvent().getText()); 
				
				PresentationEvent pEvent = clonedEntity.getEvent();
				pEvent.setSS3GlossWindowItem(clonedItem);
				pItem.addSS3GlossWindowItem(clonedItem);

				spf.getSS3GlossWindowSegment().addSS3GlossWindowItem(clonedItem);	// spf wrapped by sw
			}
			
			pfw.addSegmentFieldWrapper(sw);
			segmentObj.setWrapper(sw);	// sw wrapping spf
						
			database.getSS3GlossWindow().getTemporalPartitions().add(newPartition);
			
			
			update(database);
		}
	}
	
	/**
	 * Delete the LabelObject and Wrapper representing the selected segment tier
	 * Then call update() to reflect changes in the view.
	 */
	public void deleteSegment() {
		LabelViewTemporalPartitions labelView = databaseView.getLabelView();
		DefaultMutableTreeNode node = labelView.getSelectedNode();
		
		if (node == null)
			return;

		DefaultMutableTreeNode prevSiblingNode = node.getPreviousSibling();
		DefaultMutableTreeNode nextSiblingNode = node.getNextSibling();
		DefaultMutableTreeNode prevNode = node.getPreviousNode();
		DefaultMutableTreeNode nextNode = node.getNextNode();
		
		// if selected DisplayTimePeriod layer
		System.out.println(node.getLevel());
		if (prevSiblingNode == null && node.getLevel() == 1) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
			return;
		}

		if (prevSiblingNode != null) {
			labelView.setSelectedNode(new TreePath(prevSiblingNode.getPath()));
		} else if (nextSiblingNode != null) {
			labelView.setSelectedNode(new TreePath(nextSiblingNode.getPath()));
		} else if (prevNode != null) {
			labelView.setSelectedNode(new TreePath(prevNode.getPath()));
		} else if (nextNode != null) {
			labelView.setSelectedNode(new TreePath(nextNode.getPath()));
		}

		Object obj = node.getUserObject();

		LabelObject lObj = null;

		if (obj instanceof LabelObject) {
			lObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			lObj = vector.getObject();
		}

		FieldWrapper wrp = lObj.getWrapper();

		if (wrp instanceof PresentationFieldWrapper) {
			System.out.println("Call to delete Temporal Partition layer - no action taken");
			return;
		}
		else if (wrp instanceof SegmentFieldWrapper) {
			
			ArrayList<SS3TemporalPartition> partitions = database.getSS3GlossWindow().getTemporalPartitions();
			
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node.getParent();			
			Object parentObj = parentNode.getUserObject();

			LabelObject partitionObj = null;

			if (parentObj instanceof LabelObject) {
				partitionObj = (LabelObject) parentObj;
			} else if (obj instanceof TreeNodeVector) {
				TreeNodeVector vector = (TreeNodeVector) parentObj;
				partitionObj = vector.getObject();
			}
			
			// removeChild() makes lObj invisible to LabelViewTemporalPartitions.populateTree()
			// partitionObj is inside selectedSegment object - so still being used here
			partitionObj.removeChild(lObj);
			
			String partitionLabel = partitionObj.getLabel();			
			SS3TemporalPartition partition = new SS3TemporalPartition();
			
			for (SS3TemporalPartition tempPartition : partitions) {
				if(tempPartition.getLabel().equals(partitionLabel))
				{
					partition = tempPartition;
				}
			}

			String participantAndSegmentLabel = lObj.getLabel();
			String segmentLabelOnly = participantAndSegmentLabel.substring(
					participantAndSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) + 1);
			
			SS3GlossWindowSegment segmentToRemove = partition.getTemporalPartitionSegmentByLabel(segmentLabelOnly);
			partition.removeTemporalPartitionSegment(segmentToRemove);
		}
		
		update(database);
	}
	
	/**
	 * Delete the LabelObject and Wrapper representing the selected segment tier
	 * Then call update() to reflect changes in the view.
	 */
	public void deleteTemporalPartition() {
		LabelViewTemporalPartitions labelView = databaseView.getLabelView();
		DefaultMutableTreeNode node = labelView.getSelectedNode();
		
		if (node == null)
			return;

		DefaultMutableTreeNode prevSiblingNode = node.getPreviousSibling();
		DefaultMutableTreeNode nextSiblingNode = node.getNextSibling();
		DefaultMutableTreeNode prevNode = node.getPreviousNode();
		DefaultMutableTreeNode nextNode = node.getNextNode();
		
		// if selected DisplayTimePeriod layer
		System.out.println(node.getLevel());
		if (prevSiblingNode == null && node.getLevel() < 1) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_TEMPORAL_PARTITION);
			return;
		}

		if (prevSiblingNode != null) {
			labelView.setSelectedNode(new TreePath(prevSiblingNode.getPath()));
		} else if (nextSiblingNode != null) {
			labelView.setSelectedNode(new TreePath(nextSiblingNode.getPath()));
		} else if (prevNode != null) {
			labelView.setSelectedNode(new TreePath(prevNode.getPath()));
		} else if (nextNode != null) {
			labelView.setSelectedNode(new TreePath(nextNode.getPath()));
		}

		Object obj = node.getUserObject();
		Object parentObj = null;
		
		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node.getParent();			
			parentObj = parentNode.getUserObject();
		}
		else
			parentObj = node.getUserObject();
				

		LabelObject lObj = null;

		if (obj instanceof LabelObject) {
			lObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			lObj = vector.getObject();
		}

		LabelObject partitionObj = null;

		if (parentObj instanceof LabelObject) {
			partitionObj = (LabelObject) parentObj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) parentObj;
			partitionObj = vector.getObject();
		}
		
		// removeChild() makes lObj invisible to LabelViewTemporalPartitions.populateTree()
		// partitionObj is inside selectedSegment object - so still being used here
		partitionObj.removeChild(lObj);
		
		String partitionLabel = partitionObj.getLabel();
		
		SS3TemporalPartition partitionToRemove = database.getSS3GlossWindow().getTemporalPartitionByLabel(partitionLabel);
		database.getSS3GlossWindow().removeTemporalPartition(partitionToRemove);
		
		update(database);
	}

	/**
	 * Rename the selected segment tier's label
	 * Then call update() to reflect changes in the view.
	 * 
	 * @param newLabel - The label which the selected segment will now acquire.
	 */
	public void renameSegment(String newLabel) {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();
		
		Object obj = node.getUserObject();		
		LabelObject segmentObj = null;

		if (obj instanceof LabelObject) {
			segmentObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			segmentObj = vector.getObject();
		}

		DefaultMutableTreeNode parentNode = null;
		LabelObject partitionObj = null;

		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			parentNode = (DefaultMutableTreeNode) node.getParent();
			partitionObj = (LabelObject) parentNode.getUserObject();
		}
		else {
			// this case is handled in 
			// SS3SignStreamApplication.actionPerformed() under RENAME_SEGMENT
			return;
		}
		
		SS3TemporalPartition partition = database.getSS3GlossWindow().getTemporalPartitionByLabel(partitionObj.getLabel());
		
		if (partition.getTemporalPartitionSegmentByLabel(newLabel) != null) {
			SS3Singleton.getSignStreamApplication().confirmationDialog("Rename Segment Tier",
					"The name '" + newLabel + "' has already been used.  Please choose a different name");
			return;
		}
		
		String participantAndSegmentLabel = segmentObj.getLabel();
		String segmentLabelOnly = participantAndSegmentLabel.substring(
				participantAndSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) + 1);
		
		SS3GlossWindowSegment segmentToRename = partition.getTemporalPartitionSegmentByLabel(segmentLabelOnly);
		segmentToRename.setLabel(newLabel);
		
		update(database);
	}


	/**
	 * Rename (and reset) the selected segment tier's participant
	 * Then call update() to reflect changes in the view.
	 * 
	 * @param participant - The new participant which the selected segment will now acquire.
	 */
	public void renameParticipant(SS3Participant participant) {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();
		
		Object obj = node.getUserObject();		
		LabelObject segmentObj = null;

		if (obj instanceof LabelObject) {
			segmentObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			segmentObj = vector.getObject();
		}

		DefaultMutableTreeNode parentNode = null;
		LabelObject partitionObj = null;

		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			parentNode = (DefaultMutableTreeNode) node.getParent();
			partitionObj = (LabelObject) parentNode.getUserObject();
		}
		else {
			// this case is handled in 
			// SS3SignStreamApplication.actionPerformed() under RENAME_PARTICIPANT
			return;
		}
		
		SS3TemporalPartition partition = database.getSS3GlossWindow().getTemporalPartitionByLabel(partitionObj.getLabel());

		String participantAndSegmentLabel = segmentObj.getLabel();
		String segmentLabelOnly = participantAndSegmentLabel.substring(
				participantAndSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) + 1);
		
		SS3GlossWindowSegment segmentToRename = partition.getTemporalPartitionSegmentByLabel(segmentLabelOnly);
		segmentToRename.setParticipant(participant);
		
		update(database);
	}
	
	/**
	 * Rename the selected Temporal Partition label
	 * Then call update() to reflect changes in the view.
	 * 
	 * @param newLabel - The label which the selected Temporal Partition will now acquire.
	 */
	public void renameTemporalPartition(String newLabel) {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();
		LabelObject partitionObj = null;

		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node.getParent();
			partitionObj = (LabelObject) parentNode.getUserObject();
		}
		else
			partitionObj = (LabelObject) node.getUserObject();
		
		SS3TemporalPartition partition = new SS3TemporalPartition();
		
		partition = database.getSS3GlossWindow().getTemporalPartitionByLabel(partitionObj.getLabel());
		
		partition.setLabel(newLabel);
		
		update(database);
	}

	/**
	 * Sets the selected segment tier to be the default tier for its encapsulating temporal partition
	 * Then call update() to reflect changes in the view.
	 */
	public void setDefaultSegment() {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();

		Object obj = node.getUserObject();	
		LabelObject segmentObj = null;

		if (obj instanceof LabelObject) {
			segmentObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			segmentObj = vector.getObject();
		}

		DefaultMutableTreeNode parentNode = null;
		LabelObject partitionObj = null;

		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			parentNode = (DefaultMutableTreeNode) node.getParent();
			partitionObj = (LabelObject) parentNode.getUserObject();
		}
		else {
			// this case is handled in 
			// SS3SignStreamApplication.actionPerformed() under SET_DEFAULT_SEGMENT
			return;
		}

		ArrayList< SS3TemporalPartition> partitions = database.getSS3GlossWindow().getTemporalPartitions();		
		
		SS3TemporalPartition partition = new SS3TemporalPartition();
		
		for (SS3TemporalPartition tempPartition : partitions) {
			if(tempPartition.getLabel().equals(partitionObj.getLabel()))
			{
				partition = tempPartition;
			}
		}

		String participantAndSegmentLabel = segmentObj.getLabel();
		String segmentLabelOnly = participantAndSegmentLabel.substring(
				participantAndSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) + 1);
		
		SS3GlossWindowSegment defaultSegment = partition.getTemporalPartitionSegmentByLabel(segmentLabelOnly);
				
		partition.setDefaultSegment(defaultSegment);
		
		update(database);
	}
	
	public void updateLoadedDatabase() {
		SS3SignStreamApplication.logger.log(Level.INFO,
				"Updating loaded database...");

		if (isUtteranceLoaded) {
			ArrayList<Segment> segments = utteranceView.getSegments();

			for (Segment segment : segments) {
				RootLabelObject rootLabelObject = segment.getRootLabelObject();
				LabelObject glossLabelObject = rootLabelObject.getGlossNRelatedLabelObject();
				LabelObject nonManualLabelObject = rootLabelObject.getNonManuelLabelObject();

				ArrayList<SS3GlossWindowItem> glossWindowItems = segment.getSS3GlossWindowItems();
				
				for (SS3GlossWindowItem glossWindowItem: glossWindowItems) {
					int startTime = glossWindowItem.getStartTimeInfo().getMovieTime();
					int endTime = glossWindowItem.getEndTimeInfo().getMovieTime();
					
					ArrayList<LabelObject> allObjects = nonManualLabelObject.getAllChildren();
					allObjects.addAll(glossLabelObject.getAllChildren());

					for (LabelObject labelObject : allObjects) {
						Field field = labelObject.getWrapper().getField();
						String fieldID = field.getFieldID();

						if (field.getEntitiesBetween(startTime, endTime, true).isEmpty()) {

							if (fieldID.equalsIgnoreCase("10000")
									|| fieldID.equalsIgnoreCase("10001")) { 
								SS3StatementField sField = glossWindowItem.getSS3StatementField(fieldID);

								if (sField == null) {
									sField = new SS3StatementField();
									sField.setId(fieldID);
									glossWindowItem.addField(sField);
								}

								sField.getSS3Entities().clear();

								SS3Track track = database.getTrack(fieldID);
								if (track == null) {
									track = new SS3Track();
									track.setId(fieldID);
									database.addTrack(track);
								}

								GlossField glsField = (GlossField) field;
								GlossChainedEventsEntityCollection col = glsField.getGlossChainedEventsEntityCollection();
								EnteredText enteredText = col.getEnteredText();
								if (enteredText != null) {
									ArrayList<Text> texts = enteredText
											.getEnteredTextBetween(startTime, endTime);
									for (Text text : texts) {
										SS3Item item = text.getSS3Item();
										SS3Entity ss3Entity = new SS3Entity();
										ss3Entity.addItem(item);

										String id = edu.bu.signstream.common.util.Util
												.getLongUniqueNumber();
										ss3Entity.setID(id);
										track.addEntity(ss3Entity);
										sField.addSS3Entity(ss3Entity);
									}
								}
							} else {
								SS3StatementField sField = glossWindowItem
										.getSS3StatementField(fieldID);

								if (sField == null) {
									sField = new SS3StatementField();
									sField.setId(fieldID);
									glossWindowItem.addField(sField);
								}

								sField.getSS3Entities().clear();

								SS3Track track = database.getTrack(fieldID);
								if (track == null) {
									track = new SS3Track();
									track.setId(fieldID);
									database.addTrack(track);
								}
							}
						} else {
							SS3StatementField sField = glossWindowItem.getSS3StatementField(fieldID);

							if (sField == null) {
								sField = new SS3StatementField();
								sField.setId(fieldID);
								glossWindowItem.addField(sField);
							}

							sField.getSS3Entities().clear();

							SS3Track track = database.getTrack(fieldID);
							if (track == null) {
								track = new SS3Track();
								track.setId(fieldID);
								database.addTrack(track);
							}

							ArrayList<EventsEntity> entities = field.getEntitiesBetween(startTime, endTime, true);
							for (EventsEntity entity : entities) {
								SS3Entity ss3Entity = entity.getSS3Entity();
								track.addEntity(ss3Entity);
								sField.addSS3Entity(ss3Entity);
							}

							if (fieldID.equalsIgnoreCase("10000")
									|| fieldID.equalsIgnoreCase("10001")) {
								GlossField glsField = (GlossField) field;
								GlossChainedEventsEntityCollection col = glsField
										.getGlossChainedEventsEntityCollection();
								EnteredText enteredText = col.getEnteredText();

								ArrayList<Text> texts = enteredText
										.getEnteredTextBetween(startTime, endTime);
								for (Text text : texts) {
									SS3Item item = text.getSS3Item();
									SS3Entity ss3Entity = new SS3Entity();
									ss3Entity.addItem(item);

									String id = edu.bu.signstream.common.util.Util
											.getLongUniqueNumber();
									ss3Entity.setID(id);
									track.addEntity(ss3Entity);
									sField.addSS3Entity(ss3Entity);
								}
							}

							if (fieldID.equalsIgnoreCase("30000")) {
								FreeTextField freeTextFld = (FreeTextField) field;

								SS3StatementField tField = glossWindowItem
										.getSS3StatementField(fieldID);

								if (tField == null) {
									tField = new SS3StatementField();
									tField.setId(fieldID);
									glossWindowItem.addField(tField);
								}

								tField.getSS3Entities().clear();

								SS3Track tTrack = database.getTrack(fieldID);
								if (tTrack == null) {
									tTrack = new SS3Track();
									tTrack.setId(fieldID);
									database.addTrack(tTrack);
								}

								ArrayList freeTextEntities = freeTextFld.getEntitiesBetween(startTime, endTime, true);
								for (int i = 0; i < freeTextEntities.size(); i++) {
									FreeTextEntity freeTxtEntity = (FreeTextEntity) freeTextEntities
											.get(i);

									SS3Entity ss3Entity = freeTxtEntity
											.getSS3Entity();
									String id = ss3Entity.getID();

									if (id == null || id.length() == 0) {
										id = edu.bu.signstream.common.util.Util
												.getLongUniqueNumber();
										ss3Entity.setID(id);
									}
									tTrack.addEntity(ss3Entity);
									tField.addSS3Entity(ss3Entity);
								}
							}
						}
					}
				}
			}

		} else {

			FieldWrappersCollection collection = databaseView
					.getSegmentGraphicRepresentation()
					.getFieldWrappersCollection();

			ArrayList<FieldWrapper> wrappers = collection.getWrappers();

			SS3GlossWindow window = database.getSS3GlossWindow();

			ArrayList<SS3TemporalPartition> temporalPartitions = window.getTemporalPartitions();

			SS3TemporalPartition tp = null;
			SS3GlossWindowSegment s = null;

			for (SS3TemporalPartition temporalPartition : temporalPartitions) {
				tp = temporalPartition;

				ArrayList<SS3GlossWindowSegment> segments1 = temporalPartition
						.getTemporalPartitionSegments();
				for (SS3GlossWindowSegment segment : segments1) {

					s = segment;

				}
			}

			for (FieldWrapper wrapper : wrappers) {
				ArrayList<SS3GlossWindowItem> windowItems = new ArrayList<SS3GlossWindowItem>();

				ArrayList<EventsEntity> entities = wrapper.getField()
						.getEntities();

				if (wrapper instanceof PresentationFieldWrapper) {
					for (EventsEntity ent : entities) {
						PresentationEventsEntity pentity = (PresentationEventsEntity) ent;
						PresentationEvent pevent = pentity.getEvent();
						SS3GlossWindowItem wItem = pevent
								.getSS3GlossWindowItem();
						windowItems.add(wItem);
					}
					tp.setItems(windowItems);
				} else if (wrapper instanceof SegmentFieldWrapper) {

					for (EventsEntity ent : entities) {
						PresentationEventsEntity sentity = (PresentationEventsEntity) ent;
						PresentationEvent sevent = sentity.getEvent();
						SS3GlossWindowItem sItem = sevent
								.getSS3GlossWindowItem();
						windowItems.add(sItem);
					}
					s.setItems(windowItems);
				}
			}

		}

		SS3SignStreamApplication.logger.log(Level.INFO,
				"Loaded database had been updated.");

	}

	public boolean showVideoFiles(ArrayList<SS3MediaFile> filesToLoad,
			HashMap<String, Point> locations) {
		ArrayList<java.io.File> loadedFilesToRemove = new ArrayList<java.io.File>();
		ArrayList<Point> locationsToRemove = new ArrayList<Point>();

		ArrayList<SS3MediaFile> filesToLoadAlreadyLoaded = new ArrayList<SS3MediaFile>();
		ArrayList<XugglerMovie> moviesToUnload = new ArrayList<XugglerMovie>();

		MultipleMovieController contr = SS3Singleton.getVideoControlManager()
				.getMultipleMovieController();
		ArrayList<XugglerMovie> loadedMovies = contr.getMovies();

		int currentTime = 0;
		for (XugglerMovie movie : loadedMovies) {
			for (SS3MediaFile toLoad : filesToLoad) {
				String abcName = toLoad.getFile().getAbsolutePath();
				String movieFileName = movie.getFilename();

				if (abcName.equals(movieFileName)) {
					filesToLoadAlreadyLoaded.add(toLoad);
					currentTime = contr.getCurrentTime();
				}
			}
		}

		for (SS3MediaFile file : filesToLoadAlreadyLoaded) {
			filesToLoad.remove(file);
		}

		contr.unloadVideoFiles(moviesToUnload);

		Set<Map.Entry<JDialog, java.io.File>> set = openedMediaDialogs
				.entrySet();
		Iterator<Map.Entry<JDialog, java.io.File>> i = set.iterator();
		while (i.hasNext()) {
			Map.Entry<JDialog, java.io.File> me = i.next();
			loadedFilesToRemove.add(me.getValue());
			locationsToRemove.add(me.getKey().getLocation());
		}

		ArrayList<SS3MediaFile> filesToLoad1 = new ArrayList<SS3MediaFile>();
		Iterator<SS3MediaFile> fi = filesToLoad.iterator();
		while (fi.hasNext()) {
			SS3MediaFile mFile = fi.next();

			if (loadedFilesToRemove.contains(mFile.getFile())) {
				loadedFilesToRemove.remove(mFile.getFile());
			} else {
				filesToLoad1.add(mFile);
			}
		}


		if (!filesToLoad1.isEmpty()) {
			ArrayList<SS3MediaFile> movies = new ArrayList<SS3MediaFile>();
			String loadingMediaFilesMsg = "Loading media files [";

			Iterator<SS3MediaFile> li = filesToLoad1.iterator();
			while (li.hasNext()) {
				SS3MediaFile mediaFile = li.next();
				movies.add(mediaFile);
				loadingMediaFilesMsg = loadingMediaFilesMsg.concat(mediaFile
						.getFile().getName());
				loadingMediaFilesMsg = loadingMediaFilesMsg.concat(", ");
			}

			loadingMediaFilesMsg = loadingMediaFilesMsg.concat("]...");

			SS3SignStreamApplication.logger.log(Level.INFO, "TEST 1");
			SS3SignStreamApplication.logger.log(Level.INFO,
					loadingMediaFilesMsg);
			SS3SignStreamApplication.logger.log(Level.INFO, "TEST 2");

			SS3SignStreamApplication.logger.log(Level.INFO, "movies = "
					+ movies);
			SS3SignStreamApplication.logger.log(Level.INFO,
					"locationsToRemove = " + locationsToRemove);
			SS3SignStreamApplication.logger.log(Level.INFO, "movies.size() = "
					+ movies.size());
			SS3SignStreamApplication.logger.log(
					Level.INFO,
					"locationsToRemove.size().size() = "
							+ locationsToRemove.size());

			int size = movies.size() < locationsToRemove.size() ? movies.size()
					: locationsToRemove.size();
			if (locations.size() == 0) {
				for (int ii = 0; ii < size; ii++) {
					Point point = locationsToRemove.get(ii);
					SS3MediaFile mFile = movies.get(ii);
					locations.put(mFile.getFile().getName(), point);
				}
			}

			SS3SignStreamApplication.logger.log(Level.INFO,
					"Calling loadMovies function");

			boolean loadMovies = loadMovies(movies, locations);

			if (!loadMovies)
				return false;
		}

		if (currentTime <= 0)
			currentTime = database.getStartTimeInfo().getMovieTime();

		if (currentTime > contr.getMovieDuration())
			currentTime = contr.getMovieDuration();

		contr.selectSegment(currentTime, database.getEndTimeInfo()
				.getMovieTime());

		currentTime = 1001;
		contr.setCurrentTime(currentTime);

		return true;
	}

	public void closeAllLoadedVideoFiles() {
		Set<Map.Entry<JDialog, java.io.File>> omSet = openedMediaDialogs
				.entrySet();
		Iterator<Map.Entry<JDialog, java.io.File>> omi = omSet.iterator();

		ArrayList<JDialog> dialogsToRemove = new ArrayList<JDialog>();

		while (omi.hasNext()) {
			Map.Entry<JDialog, java.io.File> me = omi.next();

			JDialog dialog = me.getKey();
			dialog.removeAll();
			dialog.removeNotify();
			dialog.setVisible(false);
			dialogsToRemove.add(dialog);
			dialog = null;
		}

		for (JDialog d : dialogsToRemove) {
			openedMediaDialogs.remove(d);
		}
	}

	public void loadUtterance(ArrayList<Segment> segments) {
		if (sp.getRightComponent() instanceof UtteranceView) {
			sp.remove(utteranceView);
		} else {
			sp.remove(databaseView);
		}

		sp.remove(navigationPanel);
		utteranceView = new UtteranceView(segments, this);
		sp.setRightComponent(utteranceView);

		isUtteranceLoaded = true;
	}

	public boolean isUtteranceLoaded() {
		return isUtteranceLoaded;
	}

	@Override
	public void repaint() {
		super.repaint();

		if (parentFrame != null) {
			parentFrame.invalidate();
			parentFrame.validate();
		}

	}

	public void updateJMenuBar() {
		availableMediaFiles.clear();

		MediaDelegate delegate = new MediaDelegate();
		ArrayList<LoadedMediaFile> mFiles = delegate.getLoadedMediaFiles(this);

		for (LoadedMediaFile lFile : mFiles) {
			if (lFile.isLocated()) {
				String fileName = lFile.getMediaFile().getFile().getName();
				JCheckBoxMenuItem item = new JCheckBoxMenuItem(fileName);
				item.setSelected(lFile.isLoaded());
				item.addActionListener(this);
				availableMediaFiles.add(lFile.getMediaFile());
				item.setActionCommand("" + (availableMediaFiles.size() - 1));
			}
		}
	}

	public SS3Database getSS3Database() {
		return database;
	}

	public Slider getSlider() {
		return SS3Singleton.getVideoControlManager().getSlider();
	}

	public ArrayList<Segment> getSegments() {
		return segments;
	}

	public DbNavigationPanel getDbNavigationPanel() {
		return navigationPanel;
	}

	public JFrame getParentFrame() {
		return parentFrame;
	}

	public Zoomer getZoomer() {
		return SS3Singleton.getVideoControlManager().getZoomer();
	}

	public MultipleMovieController getMultipleMovieController() {
		return SS3Singleton.getVideoControlManager()
				.getMultipleMovieController();
	}

	public Excerpt getExcerpt() {
		return database.getExcerptObj();
	}

	/*
	 * filesToLoad - array of SS3MediaFile objects
	 */
	private boolean loadMovies(ArrayList<SS3MediaFile> filesToLoad,
			HashMap<String, Point> locations) {
		//
		SS3SignStreamApplication.logger.log(Level.INFO,
				"Creating XugglerMovie object");
		SS3SignStreamApplication.logger.log(Level.INFO,
				"Number of files to load: " + filesToLoad.size());
		//
		if (filesToLoad.isEmpty())
			return false;

		MultipleMovieController contr = SS3Singleton.getVideoControlManager()
				.getMultipleMovieController();

		String filesNotFoundMsg = "";

		Iterator<SS3MediaFile> i = filesToLoad.iterator();
		while (i.hasNext()) {
			SS3MediaFile nmFile = i.next();

			if (!nmFile.getFile().exists()) {
				filesNotFoundMsg = filesNotFoundMsg.concat(nmFile.getFile()
						.getName());
				i.remove();
			}
		}

		if (filesNotFoundMsg.length() > 0) {
			filesNotFoundMsg = "Video files ["
					.concat(filesNotFoundMsg.substring(0,
							filesNotFoundMsg.length() - 1)).concat(
							"] could not be found");
		}

		// make sure that only one copy of every video file is loaded

		//
		SS3SignStreamApplication.logger.log(Level.INFO,
				"Creating XugglerMovie object");
		//

		for (SS3MediaFile file : filesToLoad) {
			try {
				XugglerMovie movie = new XugglerMovie(file, contr);
				contr.registerMovie(movie);
			} catch (Exception ex) {
				String msg = "<HTML>Warning: " + ex.getMessage() + "<BR>";
				msg = msg + "Please select valid video file.</HTML>";
				SS3Singleton.getSignStreamApplication()
						.confirmationDialog("Resouce Update", msg);
				return false;
			}
		}

		contr.load();


		long frameScale = contr.getPacketDuration() > 600 ? CoordinateTimeConverter.FRAME_SCALE_MPEG
				: 600;

		getZoomer().getCoordinateTimeConvertor().setTimeScale((int) frameScale);

		updateJMenuBar();

		return true;

	}

	public void closeOpenedMediaDialog(JDialog dialog) {
		openedMediaDialogs.remove(dialog);
	}

	public Hashtable<JDialog, java.io.File> getOpenedMediaDialogs() {
		return openedMediaDialogs;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		ArrayList<SS3MediaFile> filesToLoad = new ArrayList<SS3MediaFile>();
		showVideoFiles(filesToLoad, new HashMap<String, Point>());
	}
}
