package edu.bu.signstream.grepresentation.view.mediaControl;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToolBar;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.video.VideoControlManager;
import edu.bu.signstream.ui.video.VideoControlPanel;

public class MediaToolBar extends JToolBar {

	private static final long serialVersionUID = 1L;

	public static final String VIDEO = "Play Video";
	public static final String UTTERANCE = "Play Utterance";

	private JButton setStartTime = new JButton("Set");
	private JButton setEndTime = new JButton("Set");
	private JButton goBeginning = new JButton("|-");
	private JButton goEnd = new JButton("-|");
	private JButton startFrameForward = new JButton("+");
	private JButton startFrameBack = new JButton("-");
	private JButton endFrameForward = new JButton("+");
	private JButton endFrameBack = new JButton("-");

	private JButton playSelectedEvent = new JButton("Play Event");

	private JButton playSelectedEntity = new JButton("Play Entity");
	private JButton goBeginningEntity = new JButton("|-");
	private JButton goEndEntity = new JButton("-|");

	private JButton playUtterance = new JButton("Play Utterance");
	private JButton goBeginningUtterance = new JButton("|-");
	private JButton goEndUtterance = new JButton("-|");
	
	private JComboBox annotatorSelectionComboBox = null;

	private JTextField startTimeTxt = new JTextField(10);
	private JTextField endTimeTxt = new JTextField(10);

	private SignStreamSegmentPanel segmentPanel;
	private VideoControlPanel videoControlPanel;

	private Event selectedEvent = null;
	private EventsEntity entity = null;

	private Field currentParentField = null;
	private Field field = null;
	private JPanel panel = null;

	public MediaToolBar(final SignStreamSegmentPanel segmentPanel) {
		super("Media Tool Bar");
		this.segmentPanel = segmentPanel;

		VideoControlManager manager = SS3Singleton.getVideoControlManager();
		videoControlPanel = manager.createVideoControlPanel();
		playUtterance.setText(MediaToolBar.VIDEO);
		setFloatable(false);
		
		// this combobox must be populated with at least one item
		String[] annotators = {"--Annotator--"};
		annotatorSelectionComboBox = new JComboBox(annotators);
		annotatorSelectionComboBox.setSelectedIndex(0);

		JPanel masterPanel = new JPanel(new GridLayout(0, 1, 0, 0));
		panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		setButtonComponent(goBeginning, new GoFirstFrameEventListener(this), 0);
		setButtonComponent(goEnd, new GoLastFrameEventListener(this), 0);
		setButtonComponent(playSelectedEvent, new PlaySelectedEventListener(
				this), 3);
		setButtonComponent(startFrameBack, new StartFrameBackListener(this), 0); // -
		setButtonComponent(startFrameForward, new StartFrameForwardListener(
				this), 0);
		setTextFieldComponent(startTimeTxt, 0);
		setButtonComponent(setStartTime, new SetStartTimeListener(this), 3);
		setButtonComponent(setEndTime, new SetEndTimeListener(this), 0);
		setTextFieldComponent(endTimeTxt, 0);
		setButtonComponent(endFrameBack, new EndFrameBackListener(this), 0); // -
		setButtonComponent(endFrameForward, new EndFrameForwardListener(this),
				3); // +
		setButtonComponent(goBeginningEntity, new GoFirstFrameEntityListener(
				this), 0);
		setButtonComponent(goEndEntity, new GoLastFrameEntityListener(this), 0);
		setButtonComponent(playSelectedEntity, new PlayEntityListener(this), 3);
		addComponentToPanel(videoControlPanel, 3);
		setButtonComponent(goBeginningUtterance,
				new GoFirstFrameUtteranceListener(this), 0);
		setButtonComponent(goEndUtterance, new GoLastFrameUtteranceListener(
				this), 0);
		setButtonComponent(playUtterance, new PlayUtteranceListener(this), 3);
//		setComboBoxComponent(annotatorSelectionComboBox, new AnnotatorSelectionListener(this), 1);
		masterPanel.add(panel);
		masterPanel.add(videoControlPanel.getSlider());
		this.add(masterPanel);
	}

	public boolean isUtterance() {
		return playUtterance.getText().equals(MediaToolBar.UTTERANCE);
	}

	public JTextField getStartTimeTextField() {
		return startTimeTxt;
	}

	public JTextField getEndTimeTextField() {
		return endTimeTxt;
	}

	private void setButtonComponent(JButton button, ActionListener listener,
			int followingBlanks) {
		UserInterfaceUtil.setButtonsLNF(button);
		button.setFocusable(false);
		button.addActionListener(listener);
		addComponentToPanel(button, followingBlanks);
	}

	private void setTextFieldComponent(JTextField txtField, int followingBlanks) {
		Dimension dimension = new Dimension(70, 20);
		txtField.setMaximumSize(dimension);
		txtField.setEditable(false);
		txtField.setFocusable(false);
		addComponentToPanel(txtField, followingBlanks);
	}
	
	private void setComboBoxComponent(JComboBox combobox, ActionListener listener, 
			int followingBlanks) {
		UserInterfaceUtil.setComboBoxLNF(combobox);
		combobox.setFocusable(false);
		combobox.addActionListener(listener);
		addComponentToPanel(combobox, followingBlanks);
	}

	private void addComponentToPanel(JComponent component, int followingBlanks) {
		panel.add(component);
		for (int i = 0; i < followingBlanks; i++) {
			panel.add(new JLabel(" "));
		}
	}
	
	public Field getCurrentParentField() {
		return currentParentField;
	}
	
	public void setCurrentParentField(Field parentField) {
		this.currentParentField = parentField;
	}

	public Field getSelectedField() {
		return field;
	}

	public EventsEntity getSelectedEventsEntity() {
		return entity;
	}

	public void setSelectedEvent(Event selectedEvent) {
		this.selectedEvent = selectedEvent;
		if (selectedEvent != null) {
			startTimeTxt.setText(""
					+ selectedEvent.getStartTimeInfo().getMovieTime());
			endTimeTxt.setText(""
					+ selectedEvent.getEndTimeInfo().getMovieTime());
		} else {
			startTimeTxt.setText("");
			endTimeTxt.setText("");
		}
	}

	public void setPlayButtonLabel(String label) {
		if (label == MediaToolBar.VIDEO) {
			playUtterance.setText(MediaToolBar.VIDEO);
			// disable 'entity' buttons
			playSelectedEntity.setEnabled(false);
			goBeginningEntity.setEnabled(false);
			goEndEntity.setEnabled(false);
		} else if (label == MediaToolBar.UTTERANCE) {
			playUtterance.setText(MediaToolBar.UTTERANCE);
			// enable 'entity' buttons
			playSelectedEntity.setEnabled(true);
			goBeginningEntity.setEnabled(true);
			goEndEntity.setEnabled(true);
		}
	}

	public void setSelectedEntity(EventsEntity entityToSelect) {
		this.entity = entityToSelect;
	}

	public void selectEvent(Event eventToSelect, EventsEntity entityToSelect,
			Field fieldToSelect) {
		// text is selected; does not have start and end times
		if (eventToSelect == null) {
			startTimeTxt.setText("");
			endTimeTxt.setText("");
			selectedEvent = null;
			this.entity = null;
			this.field = fieldToSelect;
		} else {
			eventToSelect.select();
			if (eventToSelect.getID() == null
					|| eventToSelect.getID().length() == 0) {
				String uniqueID = edu.bu.signstream.common.util.Util
						.getUniqueNumber();
				eventToSelect.setID(uniqueID);
			}
			if (selectedEvent == null) {
				eventToSelect.select();
				selectedEvent = eventToSelect;
				entity = entityToSelect;
				field = fieldToSelect;
				startTimeTxt.setText(""
						+ selectedEvent.getStartTimeInfo().getMovieTime());
				endTimeTxt.setText(""
						+ selectedEvent.getEndTimeInfo().getMovieTime());
			} else {
				selectedEvent.unselect();
				eventToSelect.select();
				selectedEvent = eventToSelect;
				entity = entityToSelect;
				field = fieldToSelect;
				startTimeTxt.setText(""
						+ selectedEvent.getStartTimeInfo().getMovieTime());
				endTimeTxt.setText(""
						+ selectedEvent.getEndTimeInfo().getMovieTime());
			}
		}
	}

	public Event getSelectedEvent() {
		return this.selectedEvent;
	}

	public void updateEndTimeText() {
		if (selectedEvent == null)
			return;

		startTimeTxt.setText(""
				+ selectedEvent.getStartTimeInfo().getMovieTime());
		endTimeTxt.setText("" + selectedEvent.getEndTimeInfo().getMovieTime());
	}

	public void currentFrameBack() {
		segmentPanel.getMultipleMovieController().skipNumberOfFrames(-1);
	}

	public void currentFrameForward() {
		if (segmentPanel.getMultipleMovieController() == null)
			return;
		
		int currentTime = segmentPanel.getMultipleMovieController()
				.getCurrentTime();
		
		if (currentTime < 0)
			currentTime = 0;

		int duration = segmentPanel.getMultipleMovieController()
				.getMovieDuration();
		if (duration > 0 && currentTime >= duration)
			return;
		segmentPanel.getMultipleMovieController().skipNumberOfFrames(1);
	}

	public void adjustSlider(long currentMovieTime) {
		MediaToolbarUtilController.setSliderPositionTime(segmentPanel,
				(int) currentMovieTime);
		int X = segmentPanel.getSlider().getSliderPosition();
		if (field instanceof PresentationField) {
			segmentPanel.setVisible(X, X, true);
		} else {
			segmentPanel.setVisible(X, X, false);
		}
	}

	public void moveXFramesForward(int frames) {
		int currentTime = segmentPanel.getMultipleMovieController()
				.getCurrentTime();

		if (currentTime < 0)
			currentTime = 0;

		long newTime = currentTime + frames
				* segmentPanel.getMultipleMovieController().getPacketDuration();
		int duration = segmentPanel.getMultipleMovieController()
				.getMovieDuration();
		
		if (newTime >= duration)
			return;

		segmentPanel.getMultipleMovieController().skipNumberOfFrames(frames);
		// long frameForwardTime = segmentPanel.getMultipleMovieController()
		// .frameForward(newTime);
		// setSliderPositionTime((int) frameForwardTime);
		// int X = segmentPanel.getSlider().getSliderPosition();
		// segmentPanel.getView().setVisible(X, X);
	}

	public void moveXFramesBack(int x) {
		segmentPanel.getMultipleMovieController().skipNumberOfFrames(0 - x);
		// long frameBackTime =
		// segmentPanel.getMultipleMovieController().framesXBackward(x);
		// setSliderPositionTime((int) frameBackTime);
		// int X = segmentPanel.getSlider().getSliderPosition();
		// segmentPanel.getView().setVisible(X, X);
	}

	public void go2FirstMovieFrame() {
		segmentPanel.getMultipleMovieController().goToFirstFrame();

		MediaToolbarUtilController.setSliderPositionTime(segmentPanel, 1);

		if (field instanceof PresentationField) {
			segmentPanel.setVisible(1, 1, true);
		} else {
			segmentPanel.setVisible(1, 1, false);
		}
	}

	public void go2LastMovieFrame() {
		segmentPanel.getMultipleMovieController().goToLastFrame();
		segmentPanel.getMultipleMovieController().goToLastFrame();
		int cTime = segmentPanel.getMultipleMovieController()
				.getMovieDuration();
		currentFrameForward();

		MediaToolbarUtilController.setSliderPositionTime(segmentPanel, cTime);

		if (field instanceof PresentationField) {
			segmentPanel.setVisible(cTime, cTime, true);
		} else {
			segmentPanel.setVisible(cTime, cTime, false);
		}

		SS3Singleton.getVideoControlManager().moveSliderToEndFrame();
		segmentPanel.repaint();
	}

	public void moveSliderToEndOfEventPosition() {
		String endTime = endTimeTxt.getText();

		if (endTime.trim().length() > 0) {
			int endPacketTimeStamp = Integer.parseInt(endTimeTxt.getText());
			segmentPanel.getSlider().setSliderPositionTime(endPacketTimeStamp);
			segmentPanel.repaint();
		}
	}

	public void selectOverviewPanel() {
		EventsEntity selectedEntity = getSelectedEventsEntity();
		
		if (selectedEntity != null && selectedEntity.getParentEntity() != null) {
			EventsEntity selectedParentEntity = selectedEntity.getParentEntity();
			
			getSignStreamsegmentPanel().getDatabaseView().unselectAll();
			
			selectEvent(selectedParentEntity, selectedParentEntity, null);

			int startMovieTime = selectedParentEntity.getStartTimeInfo()
					.getMovieTime();
			int endMovieTime = selectedParentEntity.getEndTimeInfo().getMovieTime();
			startTimeTxt.setText("" + startMovieTime);
			endTimeTxt.setText("" + endMovieTime);
		}

		playUtterance.setText("Play Video");

		// disable 'entity' buttons
		playSelectedEntity.setEnabled(false);
		goBeginningEntity.setEnabled(false);
		goEndEntity.setEnabled(false);
	}
	
	public void updateAnnotatorList(ArrayList<String> newAnnotatorList, String currentlySelectedAnnotator) {
		annotatorSelectionComboBox.removeAllItems();
		
		Collections.sort(newAnnotatorList);
		
		for (String s: newAnnotatorList) {
			annotatorSelectionComboBox.addItem(s);
		}
		
		if (currentlySelectedAnnotator != null) {
			if (newAnnotatorList.contains(currentlySelectedAnnotator)) {
				annotatorSelectionComboBox.setSelectedItem(currentlySelectedAnnotator);
			} else if (annotatorSelectionComboBox.getItemCount() > 0) {
				annotatorSelectionComboBox.setSelectedIndex(0);
				SS3Singleton.setCurrentAnnotator((String) annotatorSelectionComboBox.getItemAt(0));
			}
		} else if (annotatorSelectionComboBox.getItemCount() > 0) {
			annotatorSelectionComboBox.setSelectedIndex(0);
		}
	}
	
	public void addToAnnotatorList(String newAnnotator) {
		annotatorSelectionComboBox.addItem(newAnnotator);
	}

	public SignStreamSegmentPanel getSignStreamsegmentPanel() {
		return segmentPanel;
	}
}