package edu.bu.signstream.grepresentation.view.mediaControl;

import java.awt.FontMetrics;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
//import com.sun.media.jfxmedia.logging.Logger;

import javax.swing.JTextField;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredText;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.Text;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeField;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class MediaToolbarCommons {

	public static Logger logger = Logger.getLogger("edu.bu.signstream.ui");
	
	public static void setHandshapeStartTime(HandShapeEntity entity, int time) {
		if (entity.getEvents().size() != 1) {
			entity.getFirstHandShapeEvent().getEndTimeInfo().setMovieTime(time);
		}
		entity.getFirstHandShapeEvent().getStartTimeInfo().setMovieTime(time);
	}

	public static void setHandshapeEndTime(HandShapeEntity entity, int time) {
		entity.getLastHandShapeEvent().getEndTimeInfo().setMovieTime(time);
		if (entity.getEvents().size() != 1) {
			entity.getLastHandShapeEvent().getStartTimeInfo().setMovieTime(time);
		}
	}

	public static void repaintPanel(int time, MediaToolBar bar) {
		bar.getSignStreamsegmentPanel().getSlider().setSliderPositionTime(time);
		bar.getSignStreamsegmentPanel().repaint();
	}

	public static boolean isUpdatablePresentationEvent(EventsEntity entity) {
		PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;
		return !pEntity.getDependentEntities().isEmpty();
	}

	public static void createNewGlossEvent(EnteredText enteredText,
			int currentTime, MediaToolBar bar) {

		ArrayList<Event> event = enteredText.createGlossChainedEvent(
				currentTime, currentTime);

		if (!event.isEmpty()) {
			bar.setSelectedEvent(event.get(0));
			bar.setSelectedEntity((ChainedEventsEntity) event.get(1));
			bar.getSignStreamsegmentPanel().setVisible(event.get(0));
		}
	}

	protected static void setSelectedEventEndTime(int time, Event event,
			EventsEntity parentEntity, MediaToolBar bar) {
		boolean utterance = bar.isUtterance();

		if (!utterance) {
			int movieDuration = bar.getSignStreamsegmentPanel()
					.getMultipleMovieController().getMovieDuration();

			if (time > movieDuration) {
				return;
			}
		}

		if (utterance && parentEntity != null) {
			int utteranceStartTime2 = parentEntity.getStartTimeInfo()
					.getMovieTime();
			int utteranceEndTime2 = parentEntity.getEndTimeInfo()
					.getMovieTime();
			if (time < utteranceStartTime2) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
				return;
			} else if (time > utteranceEndTime2) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
				return;
			}
		}

		MediaToolbarCommons.setSelectedObjectEndTime(bar, time);
	}

	protected static void setSelectedObjectEndTime(MediaToolBar bar, int time) {
		Field field = bar.getSelectedField();

		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
		EventsEntity parentEntity = selectedEntity.getParentEntity();

		EventsEntity entity = bar.getSelectedEventsEntity();

		if (entity instanceof PresentationEventsEntity) {

			PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;
			MediaToolbarCommons.setPresentationEventsEntityEndTime(time,
					pEntity);
			return;

		} else if (field instanceof GlossField) {
			GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) entity;
			GlossChainedEventsEntity nextEntity = glsEntity.getNextEntity();

			if (nextEntity != null) {
				int nextEntityEndMovieTime = nextEntity.getEndTimeInfo()
						.getMovieTime();
				int nextEntityStartMovieTime = nextEntity.getStartTimeInfo()
						.getMovieTime();

				if (nextEntityStartMovieTime < time) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CHAINS_CANNOT_OVERLAP);
					return;
				} else if (time == nextEntityStartMovieTime && nextEntityStartMovieTime == nextEntityEndMovieTime) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CHAINS_CANNOT_OVERLAP);
					return;
				}
			}
			
			MediaToolbarCommons.setGlossChainedEventsEntityEntityEndTime(time,
					bar);

		} else if (field instanceof HandShapeField) {

			HandShapeEntity hsEntity = (HandShapeEntity) entity;
			MediaToolbarCommons.setHandshapeEndTime(hsEntity, time);

		} else if (field instanceof NonManualField) {
			NonManualField nmField = (NonManualField) field;
			MediaToolbarCommons.setNonManualEntityEndTime(time, parentEntity,
					nmField);
		}
	}

	protected static void setPresentationEventsEntityEndTime(int time,
			PresentationEventsEntity entity) {
		PresentationField pField = (PresentationField) entity.getField();

		int startTime = entity.getStartTimeInfo().getMovieTime();
		int endTime = entity.getEndTimeInfo().getMovieTime();

		if (time <= startTime) {
			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
			return;
		} else if (time < endTime) {
			// update
			entity.getEndTimeInfo().setMovieTime(time);
		} else {
			EventsEntity npEntity = pField.getNextEntity(endTime);
			if (npEntity == null) {
				// update
				entity.getEndTimeInfo().setMovieTime(time);
			} else if (npEntity.getStartTimeInfo().getMovieTime() > time) {
				// update
				entity.getEndTimeInfo().setMovieTime(time);
			} else {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}
		}
	}

	public static void setTextStartEndTime(GlossField field,
			GlossChainedEventsEntity entity, EventsEntity parentEntity,
			int time, int coord, MediaToolBar bar) {
		
		EnteredText enteredText = field.getGlossChainedEventsEntityCollection()
				.getEnteredText();

		ArrayList<Text> selectedText = enteredText.getSelectedText();

		if (selectedText.isEmpty()) {
			return;
		}

		Text selectedText1 = selectedText.get(0);
		Text selectedText2 = selectedText.get(selectedText.size() - 1);

		if (selectedText1.getStartCoord() <= coord
				&& coord <= selectedText2.getEndCoord()) {
			MediaToolbarCommons.createNewGlossEvent(enteredText, time, bar);
		} else {
			field.deleteEmptyEntities();

			if (coord < selectedText1.getStartCoord()) {

				if (time < parentEntity.getStartTimeInfo().getMovieTime()) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
					return;
				}

				ArrayList<EventsEntity> entities = field.getEntities();

				for (EventsEntity ent : entities) {

					int endCoord = ent.getEndTimeCoordinate();
					if (coord < endCoord
							&& endCoord < selectedText1.getStartCoord()
							&& entity != ent) {

						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.CHAINS_CANNOT_OVERLAP);
						return;
					}
				}

				MediaToolbarCommons.createNewGlossEvent(enteredText, time, bar);
				
				if (entity == null)
				{
					try {
						entity = (GlossChainedEventsEntity)bar.getSelectedEventsEntity(); // this cast is okay - it could not be any other type
					}
					catch (Exception e)
					{
//						Logger.logMsg(Logger.INFO, "Error caught in attempting to cast the newly created EventsEntity into a GlossChainedEventsEntity");
						logger.log(Level.INFO, "Error caught in attempting to cast the newly created EventsEntity into a GlossChainedEventsEntity");
					}
				}

				GlossChainedEventsEntity prevEntity = entity
						.getPreviousEntity();

				int prevEndCoord = 0;

				if (prevEntity != null) {
					prevEndCoord = prevEntity.getEndTimeCoordinate();
				}

				ArrayList<Text> textInInterval = enteredText
						.getSortedEnteredTextInInteval2(prevEndCoord,
								selectedText1.getStartCoord());

				FontMetrics fontMetrics = bar
						.getFontMetrics(SegmentGraphReprUtil.getGraphReprFont());

				if (!textInInterval.isEmpty()) {
					int prevStartCoord = entity.getStartTimeCoordinate();
					String txt1 = entity.getText();
					int width1 = fontMetrics.stringWidth(txt1);
					prevStartCoord = prevStartCoord - width1 / 2;

					int commulativeWidht = 0;
					int dCommulativeWidht = Util.MTB_TEXT_BUFFER;

					int textSize = textInInterval.size();

					for (int i = textSize - 1; i >= 0; i--) {
						Text t = textInInterval.get(i);
						String txt = t.getText();

						int width = fontMetrics.stringWidth(txt);
						prevStartCoord = prevStartCoord - Util.MTB_TEXT_BUFFER - width;
						t.setStartCoord(prevStartCoord);

						commulativeWidht = commulativeWidht + width;
						dCommulativeWidht = dCommulativeWidht + Util.MTB_TEXT_BUFFER;
					}

					dCommulativeWidht = dCommulativeWidht + Util.MTB_TEXT_BUFFER;

					int availableWidht = entity.getStartTimeCoordinate()
							- prevEndCoord - dCommulativeWidht;

					int requestedWidht = commulativeWidht;

					if (availableWidht < requestedWidht) {

						if (availableWidht <= 0) {

							for (Text t : textInInterval) {
								int c1 = entity.getStartTimeCoordinate()
										- prevEndCoord;
								int c2 = c1 / 2;
								int c3 = prevEndCoord + c2;
								t.setStartCoord(c3);
							}

							return;
						}

						double d1 = availableWidht;
						double d2 = requestedWidht;
						double d = d1 / d2;

						for (Text t : textInInterval) {
							String txt = t.getText();

							int txtWidth = (fontMetrics.stringWidth(txt));

							double w = txtWidth * d;
							int width = (int) w;

							prevEndCoord = prevEndCoord + Util.MTB_TEXT_BUFFER;

							t.setStartCoord(prevEndCoord);
							prevEndCoord = prevEndCoord + width;
						}
					}

					return;
				}
			} else if (coord > selectedText2.getEndCoord()) {

				if (time > parentEntity.getEndTimeInfo().getMovieTime()) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
					return;
				}

				ArrayList<EventsEntity> entities = field.getEntities();

				for (EventsEntity ent : entities) {

					int startCoord = ent.getStartTimeCoordinate();
					if (coord > startCoord
							&& startCoord > selectedText2.getEndCoord()
							&& entity != ent) {

						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.CHAINS_CANNOT_OVERLAP);
						return;
					}
				}

				MediaToolbarCommons.createNewGlossEvent(enteredText, time, bar);
				
				if (entity == null)
				{
					try {
						entity = (GlossChainedEventsEntity)bar.getSelectedEventsEntity(); // this cast is okay - it could not be any other type
					}
					catch (Exception e)
					{
//						Logger.logMsg(Logger.INFO, "Error caught in attempting to cast the newly created EventsEntity into a GlossChainedEventsEntity");
						logger.log(Level.INFO, "Error caught in attempting to cast the newly created EventsEntity into a GlossChainedEventsEntity");
					}
				}
				
				GlossChainedEventsEntity nextEntity = entity.getNextEntity();

				int nextStartCoord = parentEntity.getEndTimeCoordinate();

				if (nextEntity != null) {
					nextStartCoord = nextEntity.getStartTimeCoordinate();
				}

				ArrayList<Text> textInInterval = enteredText
						.getSortedEnteredTextInInteval2(
								selectedText2.getEndCoord(), nextStartCoord);

				if (!textInInterval.isEmpty()) {
					FontMetrics fontMetrics = bar
							.getFontMetrics(SegmentGraphReprUtil
									.getGraphReprFont());

					int prevEndCoord = entity.getEndTimeCoordinate();

					String txt1 = entity.getText();
					int width1 = fontMetrics.stringWidth(txt1);
					prevEndCoord = prevEndCoord + width1 / 2;

					int commulativeWidht = 0;
					int dCommulativeWidht = Util.MTB_TEXT_BUFFER;

					for (Text t : textInInterval) {
						String txt = t.getText();
						int width = fontMetrics.stringWidth(txt);
						prevEndCoord = prevEndCoord + Util.MTB_TEXT_BUFFER;
						t.setStartCoord(prevEndCoord);

						prevEndCoord = prevEndCoord + width;

						commulativeWidht = commulativeWidht + width;
						dCommulativeWidht = dCommulativeWidht + Util.MTB_TEXT_BUFFER;
					}

					dCommulativeWidht = dCommulativeWidht + Util.MTB_TEXT_BUFFER;

					int startMoveInInterval = prevEndCoord;
					int endMoveInInterval = entity.getEndTimeCoordinate();
					int textWidht = commulativeWidht + dCommulativeWidht;
					int dWidht = endMoveInInterval - startMoveInInterval;

					if (dWidht < textWidht) {
						;
					}
				}
			}
		}
	}

	protected static void setNonManualEntityEndTime(int time,
			EventsEntity parentEntity, NonManualField field) {

		if (parentEntity != null) {
			if (parentEntity.getStartTimeInfo().getMovieTime() > time
					|| parentEntity.getEndTimeInfo().getMovieTime() < time) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
				return;
			}
		}

		ChainedEventsEntity selectedEntity = (ChainedEventsEntity) field
				.getSelectedEntity();

		String id = selectedEntity.getField().getFieldID();
		if (id.equalsIgnoreCase("10501") || id.equalsIgnoreCase("10502")) {
			if (selectedEntity.isLockedGlossPhonAlignments()) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.UNLOCK_GLOSS_PHON_ALIGNMENTS);
				return;
			}
		}

		String resp = selectedEntity.setSelectedEventEndMovieTime(time);

		if (resp != null) {
			SS3Singleton.getErrorMessages().showErrorMessage(resp);
			return;
		}
	}

	protected static void setGlossChainedEventsEntityEntityEndTime(int time,
			MediaToolBar bar) {
		Field field = bar.getSelectedField();
		Event event = bar.getSelectedEvent();
		EventsEntity entity = bar.getSelectedEventsEntity();

		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
		EventsEntity parentEntity = selectedEntity.getParentEntity();

		GlossField glsField = (GlossField) field;

		if (entity == null && event != null) {
			entity = glsField.getEntityAt(event.getStartTimeCoordinate());
		}

		if (parentEntity != null) {
			if (parentEntity.getStartTimeInfo().getMovieTime() > time
					| parentEntity.getEndTimeInfo().getMovieTime() < time) {

				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
				return;
			}
		}

		GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) entity;

		int moveFromCoord = glsEntity.getEndTimeCoordinate();
		int entityEndTime = glsEntity.getEndTimeInfo().getMovieTime();
		int eventEndTime = glsEntity.getSelectedEvent().getEndTimeInfo()
				.getMovieTime();

		if (entityEndTime == eventEndTime) {
			GlossChainedEventsEntity nextEntity = glsEntity.getNextEntity();
			if (nextEntity != null
					&& nextEntity.getStartTimeInfo().getMovieTime() < time) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}
		}

		String resp = glsEntity.setSelectedEventEndMovieTime(time);

		if (resp != null) {
			SS3Singleton.getErrorMessages().showErrorMessage(resp);
			return;
		} else {

			int packetDuration = (int) bar.getSignStreamsegmentPanel()
					.getMultipleMovieController().getPacketDuration();

			ChainedEvent onset = glsEntity.getOnset();
			ChainedEvent offset = glsEntity.getOffset();
			ChainedEvent initialHold = glsEntity.getInitialHold();
			ChainedEvent finalHold = glsEntity.getFinalHold();
			ChainedEvent text = glsEntity.getTextValue();

			GlossChainedEventsEntity depGlsEntity = glsEntity
					.getDependentGlossEntity();
			ChainedEventsEntity handEntity = glsEntity.getDependentHandEntity();
			HandShapeEntity handShapeEntity = glsEntity
					.getDependentHandShapeEntity();

			boolean glsLockedGlossPhoneAlignments = glsEntity
					.isLockedGlossPhonAlignments();
			boolean glsLockedLeftRightAlignments = glsEntity
					.isLockedLeftRightAlignments();
			// boolean ndGlsLockedGlossPhoneAlignments = false;

			ChainedEvent depGlsOffset = null;
			ChainedEvent depGlsFinalHold = null;

			if (depGlsEntity != null) {
				// ndGlsLockedGlossPhoneAlignments = depGlsEntity
				// .isLockedGlossPhonAlignments();
				depGlsOffset = depGlsEntity.getOffset();
				depGlsFinalHold = depGlsEntity.getFinalHold();
			}

			if (event.equals(onset)) {
				int minOnsetEndTime = onset.getStartTimeInfo().getMovieTime()
						+ packetDuration;
				int maxOnsetEndTime = text.getEndTimeInfo().getMovieTime()
						- packetDuration;

				if (initialHold != null) {
					maxOnsetEndTime = initialHold.getEndTimeInfo()
							.getMovieTime() - packetDuration;
				}

				if (time < minOnsetEndTime) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
					return;
				} else if (time > maxOnsetEndTime) {
					if (initialHold != null) {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.SET_END_ONSET_CANT_DELETE_INITIAL_HOLD);
						return;
					} else {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.SET_END_INITIAL_HOLD_CANT_DELETE_VALUE);
						return;
					}
				} else {
					onset.getEndTimeInfo().setMovieTime(time);

					if (initialHold != null) {
						initialHold.getStartTimeInfo().setMovieTime(time);
					} else {
						text.getStartTimeInfo().setMovieTime(time);
					}

					if (glsLockedGlossPhoneAlignments) {
						if (handEntity != null) {
							ChainedEventsEntity prevEntity = handEntity
									.getPreviousEntity();
							if (prevEntity != null
									&& prevEntity.getEndTimeInfo()
											.getMovieTime() > time) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.CHAINS_CANNOT_OVERLAP);
								return;
							} else {
								handEntity.getStartTimeInfo()
										.setMovieTime(time);
								if (handShapeEntity != null) {
									MediaToolbarCommons.setHandshapeStartTime(
											handShapeEntity, time);
								}
							}
						}
					}

					if (glsLockedLeftRightAlignments && depGlsEntity != null
							&& depGlsEntity.getOnset() != null) {

						ChainedEvent ndOnset = depGlsEntity.getOnset();
						ChainedEvent ndInitialHold = depGlsEntity
								.getInitialHold();
						ChainedEvent ndText = depGlsEntity.getTextValue();

						int ndGlsOnsetMinEndTime = ndOnset.getStartTimeInfo()
								.getMovieTime() + packetDuration;
						int ndGlsOnsetMaxEndTime = ndText.getEndTimeInfo()
								.getMovieTime() - packetDuration;

						if (ndInitialHold != null) {
							ndGlsOnsetMaxEndTime = ndInitialHold
									.getEndTimeInfo().getMovieTime()
									- packetDuration;
						}

						if (time < ndGlsOnsetMinEndTime) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return;
						} else if (time > ndGlsOnsetMaxEndTime) {
							if (ndInitialHold != null) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_END_ONSET_CANT_DELETE_INITIAL_HOLD);
								return;
							} else {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_END_INITIAL_HOLD_CANT_DELETE_VALUE);
								return;
							}
						} else {
							ndOnset.getEndTimeInfo().setMovieTime(time);

							if (ndInitialHold != null) {
								ndInitialHold.getStartTimeInfo().setMovieTime(
										time);
							} else {
								ndText.getStartTimeInfo().setMovieTime(time);
							}

							if (depGlsEntity.isLockedGlossPhonAlignments()) {
								ChainedEventsEntity ndHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity ndHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();

								if (ndHandEntity != null) {
									ChainedEventsEntity prevEntity = ndHandEntity
											.getPreviousEntity();
									if (prevEntity != null
											&& prevEntity.getEndTimeInfo()
													.getMovieTime() > time) {
										SS3Singleton
												.getErrorMessages()
												.showErrorMessage(
														ErrorMessages.CHAINS_CANNOT_OVERLAP);
										return;
									} else {
										ndHandEntity.getStartTimeInfo()
												.setMovieTime(time);
										if (ndHandShapeEntity != null) {
											MediaToolbarCommons
													.setHandshapeStartTime(
															ndHandShapeEntity,
															time);
											return;
										}
									}
								}
							}
						}
					}
				}

			} else if (event.equals(initialHold)) {

				if (glsLockedLeftRightAlignments && depGlsEntity != null
						&& depGlsEntity.getInitialHold() != null) {
					ChainedEvent ndInitialHold = depGlsEntity.getInitialHold();
					ChainedEvent ndText = depGlsEntity.getTextValue();

					int ndMinEndTime = ndInitialHold.getStartTimeInfo()
							.getMovieTime() + packetDuration;
					int ndMaxEndTime = ndText.getEndTimeInfo().getMovieTime()
							- packetDuration;

					if (ndMinEndTime > time || ndMaxEndTime < time) {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
						return;
					}

					ndInitialHold.getEndTimeInfo().setMovieTime(time);
					ndText.getStartTimeInfo().setMovieTime(time);
				}
			} else if (event.equals(text)) {
				if (finalHold == null && glsLockedGlossPhoneAlignments) {
					if (handEntity != null) {
						ChainedEventsEntity nextEntity = handEntity
								.getNextEntity();
						if (nextEntity != null) {
							int nextTimeStartTime = nextEntity
									.getStartTimeInfo().getMovieTime();

							if (nextTimeStartTime >= time) {
								handEntity.getEndTimeInfo().setMovieTime(time);
							} else {
								MediaToolbarCommons.setHandshapeEndTime(
										handShapeEntity, time);
							}
						}
						if(handEntity != null) {
							handEntity.getEndTimeInfo().setMovieTime(time);
						}
						if (handShapeEntity != null) {
							MediaToolbarCommons.setHandshapeEndTime(
									handShapeEntity, time);
						}
					}
				}

				if (glsLockedLeftRightAlignments && depGlsEntity != null) {
					if (depGlsFinalHold != null) {
						// depGlsEntity.setFinalHoldEvent(depGlsFinalHold);
						int maxStartTime = depGlsFinalHold.getEndTimeInfo()
								.getMovieTime() - packetDuration;

						if (maxStartTime < time) {
							depGlsEntity.setFinalHoldEvent(depGlsFinalHold);
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return;
						} else {
							depGlsEntity.getTextValue().getEndTimeInfo()
									.setMovieTime(time);
							depGlsFinalHold.getStartTimeInfo().setMovieTime(
									time);
							depGlsEntity.setFinalHoldEvent(depGlsFinalHold);
						}
					} else if (depGlsOffset != null) {
						int maxEndTime = depGlsOffset.getEndTimeInfo()
								.getMovieTime() - packetDuration;

						if (maxEndTime < time) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return;
						} else {
							depGlsEntity.getTextValue().getEndTimeInfo()
									.setMovieTime(time);
							depGlsOffset.getStartTimeInfo().setMovieTime(time);
							depGlsEntity.setOffsetEvent(depGlsOffset);

							if (depGlsEntity.isLockedGlossPhonAlignments()) {
								ChainedEventsEntity depHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity depHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();
								depHandEntity.getEndTimeInfo().setMovieTime(
										time);

								if (depHandShapeEntity != null) {
									MediaToolbarCommons.setHandshapeEndTime(
											depHandShapeEntity, time);
								}
							}
						}
					} else {
						GlossChainedEventsEntity ndNextEntity = depGlsEntity
								.getNextEntity();
						if (ndNextEntity != null
								&& ndNextEntity.getStartTimeInfo()
										.getMovieTime() < time) {
							SS3Singleton.getErrorMessages().showErrorMessage(
									ErrorMessages.CHAINS_CANNOT_OVERLAP);
							return;
						} else {
							depGlsEntity.getTextValue().getEndTimeInfo()
									.setMovieTime(time);
							if (depGlsEntity.isLockedGlossPhonAlignments()) {
								ChainedEventsEntity depHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity depHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();

								if (depHandEntity != null) {
									ChainedEventsEntity ndDepHandEntity = depHandEntity
											.getNextEntity();

									depHandEntity.getEndTimeInfo()
											.setMovieTime(time);

									if (depHandShapeEntity != null) {
										MediaToolbarCommons
												.setHandshapeEndTime(
														depHandShapeEntity,
														time);
									}
								}
							}
						}
					}
				}
			} else if (event.equals(finalHold)) {

				if (glsLockedGlossPhoneAlignments) {
					if (handEntity != null) {
						ChainedEventsEntity nextEntity = handEntity
								.getNextEntity();
						if (nextEntity != null
								&& nextEntity.getStartTimeInfo().getMovieTime() < time) {
							SS3Singleton.getErrorMessages().showErrorMessage(
									ErrorMessages.CHAINS_CANNOT_OVERLAP);
							return;
						} else {
							handEntity.getEndTimeInfo().setMovieTime(time);
							if (handShapeEntity != null) {
								MediaToolbarCommons.setHandshapeEndTime(
										handShapeEntity, time);
							}
						}
					}
				}

				if (glsLockedLeftRightAlignments) {
					if (depGlsEntity != null
							&& depGlsEntity.getFinalHold() != null) {

						ChainedEvent ndFinalHold = depGlsEntity.getFinalHold();
						int ndMinFinalHoldEndTime = ndFinalHold
								.getStartTimeInfo().getMovieTime()
								+ packetDuration;

						if (ndMinFinalHoldEndTime > time) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
							return;
						} else if (depGlsEntity.getOffset() != null) {
							ChainedEvent ndOffset = depGlsEntity.getOffset();
							int ndMaxFinalHoldEndTime = ndOffset
									.getEndTimeInfo().getMovieTime()
									- packetDuration;
							if (ndMaxFinalHoldEndTime < time) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
								return;
							} else {
								ndFinalHold.getEndTimeInfo().setMovieTime(time);
								ndOffset.getStartTimeInfo().setMovieTime(time);

								if (depGlsEntity.isLockedGlossPhonAlignments()) {
									ChainedEventsEntity ndHandEntity = depGlsEntity
											.getDependentHandEntity();
									HandShapeEntity ndHandShapeEntity = depGlsEntity
											.getDependentHandShapeEntity();

									if (ndHandEntity != null) {
										ChainedEventsEntity ndNextEntity = ndHandEntity
												.getNextEntity();
										if (ndNextEntity != null
												&& ndNextEntity
														.getStartTimeInfo()
														.getMovieTime() < time) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.CHAINS_CANNOT_OVERLAP);
											return;
										} else {
											ndHandEntity.getEndTimeInfo()
													.setMovieTime(time);
											if (ndHandShapeEntity != null) {
												MediaToolbarCommons
														.setHandshapeEndTime(
																ndHandShapeEntity,
																time);
											}
										}
									}
								}
							}
						} else {
							GlossChainedEventsEntity nextGlsEntity = depGlsEntity
									.getNextEntity();

							if (nextGlsEntity != null
									&& nextGlsEntity.getStartTimeInfo()
											.getMovieTime() < time) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.CHAINS_CANNOT_OVERLAP);
								return;
							} else {
								ndFinalHold.getEndTimeInfo().setMovieTime(time);

								if (depGlsEntity.isLockedGlossPhonAlignments()) {
									ChainedEventsEntity ndHandEntity = depGlsEntity
											.getDependentHandEntity();
									HandShapeEntity ndHandShapeEntity = depGlsEntity
											.getDependentHandShapeEntity();

									if (ndHandEntity != null) {
										ChainedEventsEntity ndNextEntity = ndHandEntity
												.getNextEntity();
										if (ndNextEntity != null
												&& ndNextEntity
														.getStartTimeInfo()
														.getMovieTime() < time) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.CHAINS_CANNOT_OVERLAP);
											return;
										} else {
											ndHandEntity.getEndTimeInfo()
													.setMovieTime(time);
											if (ndHandShapeEntity != null) {
												MediaToolbarCommons
														.setHandshapeEndTime(
																ndHandShapeEntity,
																time);
											}
										}
									}
								}
							}
						}
					}
				}
			} else if (event.equals(offset)) {
				if (glsLockedLeftRightAlignments) {
					GlossChainedEventsEntity nextEntity = depGlsEntity
							.getNextEntity();
					if (nextEntity != null
							&& nextEntity.getStartTimeInfo().getMovieTime() < time) {
						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.CHAINS_CANNOT_OVERLAP);
						return;
					} else {
						if (depGlsEntity != null) {
							depGlsEntity.getOffset().getEndTimeInfo()
									.setMovieTime(time);
						}
					}
				}
			}

			moveTextForward(moveFromCoord, glsEntity.getEndTimeCoordinate(),
					glsField, glsEntity, bar);

			return;
		}
	}

	private static void moveTextForward(int from, int to, GlossField glsField,
			GlossChainedEventsEntity entity, MediaToolBar bar) {
		// from: where entity end point started (old from)
		// to: where entity end point is desired to go
		// entity: its end point has already moved from --> to, so the end is here the (new from)
		
		FontMetrics fontMetrics = bar.getFontMetrics(SegmentGraphReprUtil
				.getGraphReprFont());
		
		EnteredText enteredText = glsField
				.getGlossChainedEventsEntityCollection().getEnteredText();

		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
		EventsEntity parentEntity = selectedEntity.getParentEntity();		
		GlossChainedEventsEntity nextEntity = entity.getNextEntity();
		
		if (nextEntity != null) {			
			int thisEntityEndCoord = entity.getEndTimeCoordinate();
			int nextEntityStartCoord = nextEntity.getStartTimeCoordinate();
			
			int oldDiffBetweenEntityCoords = nextEntityStartCoord - from;		// distance before entity has moved
			int newDiffBetweenEntityCoords = nextEntityStartCoord - thisEntityEndCoord;		// distance after entity has already moved
			
			ArrayList<Text> textInOldEntityInterval = enteredText
					.getSortedEnteredTextInInteval2(from, nextEntityStartCoord);	// interval starting at where entity used to be before moving
			
			int cumulativeTextsWidth = 0;		
			if (!textInOldEntityInterval.isEmpty()) {
				for (Text text: textInOldEntityInterval) {
					String str = text.getText();
					int width = fontMetrics.stringWidth(str);
					cumulativeTextsWidth += width += Util.MTB_TEXT_BUFFER;
				}
				
				if (cumulativeTextsWidth > newDiffBetweenEntityCoords) {
					// remove all text that was in between
					enteredText.removeText(textInOldEntityInterval);
					return;
				}
			}
		}
		
		ArrayList<Text> textInEndPointsInterval = enteredText
				.getSortedEnteredTextInInteval2(from, to);		// interval between old end point and new end point

		if (!textInEndPointsInterval.isEmpty()) {
			int maxEndCoord = parentEntity
					.getEndTimeCoordinate();
			if (nextEntity != null) {
				maxEndCoord = nextEntity.getStartTimeCoordinate();	
				// changed this from endCoord to startCoord - should be no difference because no text should be within a gloss
			}

			ArrayList<Text> textInInterval = enteredText
					.getSortedEnteredTextInInteval2(from, maxEndCoord);

			int cumulativeWidth = 0;
			int dCumulativeWidth = Util.MTB_TEXT_BUFFER;
			int textSize = textInInterval.size();

			// loop through texts to get their cumulative width
			for (int i = textSize - 1; i >= 0; i--) {
				Text t = textInInterval.get(i);
				String txt = t.getText();
				int width = fontMetrics.stringWidth(txt);
				cumulativeWidth = cumulativeWidth + width;
				dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;
			}

			dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;

			int availableWidth = maxEndCoord - to;
			int requestedWidth = cumulativeWidth;
			int prevEndCoord = to + Util.MTB_TEXT_BUFFER;

			if (availableWidth >= requestedWidth) {
				for (Text text : textInInterval) {
					text.setStartCoord(prevEndCoord);
					int endCoord = prevEndCoord
							+ fontMetrics.stringWidth(text.getText());
					text.setEndCoord(endCoord);
					prevEndCoord = prevEndCoord
							+ fontMetrics.stringWidth(text.getText()) + Util.MTB_TEXT_BUFFER;
				}
			} 
			// this else block is now obsolete due to the return block above involving newDiffBetweenEntityCoords
			// if I am wrong, then uncomment this
//			else {
//				double d1 = availableWidth;
//				double d2 = requestedWidth;
//				double d = d1 / d2;
//
//				for (Text text : textInInterval) {
//					text.setStartCoord(prevEndCoord);
//					double adjustedTextWidth = fontMetrics.stringWidth(text
//							.getText()) * d - 3;
//					prevEndCoord = prevEndCoord + BUFFER + (int) adjustedTextWidth;
//				}
//			}
		}
	}

	private static void moveTextBackward(int from, int to, GlossField glsField,
			GlossChainedEventsEntity entity, MediaToolBar bar) {
		// from: where entity end point started (old from)
		// to: where entity end point is desired to go
		// entity: its end point has already moved from --> to, so the end is here the (new from)
				
		FontMetrics fontMetrics = bar.getFontMetrics(SegmentGraphReprUtil
				.getGraphReprFont());
		
		EnteredText enteredText = glsField
				.getGlossChainedEventsEntityCollection().getEnteredText();

		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
		EventsEntity parentEntity = selectedEntity.getParentEntity();
		GlossChainedEventsEntity prevEntity = entity.getPreviousEntity();
		
		if (prevEntity != null) {
			int thisEntityStartCoord = entity.getStartTimeCoordinate();
			int prevEntityEndCoord = prevEntity.getEndTimeCoordinate();
			
			int oldDiffBetweenEntityCoords = from - prevEntityEndCoord;		// distance before entity has moved
			int newDiffBetweenEntityCoords = thisEntityStartCoord - prevEntityEndCoord;		// distance after entity has already moved
			
			ArrayList<Text> textInOldEntityInterval = enteredText
					.getSortedEnteredTextInInteval2(prevEntityEndCoord, from);

			int cumulativeTextsWidth = 0;		
			if (!textInOldEntityInterval.isEmpty()) {
				for (Text text: textInOldEntityInterval) {
					String str = text.getText();
					int width = fontMetrics.stringWidth(str);
					cumulativeTextsWidth += width += Util.MTB_TEXT_BUFFER;
				}
				
				if (cumulativeTextsWidth > newDiffBetweenEntityCoords) {
					// remove all text that was in between
					enteredText.removeText(textInOldEntityInterval);
					return;
				}			
			}
		}
		
		ArrayList<Text> textInEndPointsInterval = enteredText
				.getSortedEnteredTextInInteval2(to, from); // since text going backwards, need to reverse from/to

		if (!textInEndPointsInterval.isEmpty()) {
			int minStartCoord = parentEntity
					.getStartTimeCoordinate();
			if (prevEntity != null)  {
				minStartCoord = prevEntity.getEndTimeCoordinate();
				// changed this from startCoord to endCoord - should be no difference because no text should be within a gloss
			}

			ArrayList<Text> textInInterval = enteredText
					.getSortedEnteredTextInInteval2(minStartCoord, from);

			int cumulativeWidth = 0;
			int dCumulativeWidth = Util.MTB_TEXT_BUFFER;
			int textSize = textInInterval.size();
			
			// loop through texts to get their cumulative width
			for (int i = 0; i < textSize; i++) {
				Text t = textInInterval.get(i);
				String txt = t.getText();
				int width = fontMetrics.stringWidth(txt);
				cumulativeWidth = cumulativeWidth + width;
				dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;
			}

			dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;

			int availableWidth = to - minStartCoord;
			int requestedWidth = cumulativeWidth;
			int prevStartCoord = to - Util.MTB_TEXT_BUFFER;			

			if (availableWidth >= requestedWidth) {
				for (int i = textSize - 1; i >= 0; i--) {	// need to loop texts in reverse in setting start/end times
					Text text = textInInterval.get(i);
					text.setEndCoord(prevStartCoord);
					int startCoord = prevStartCoord
							- fontMetrics.stringWidth(text.getText());
					text.setStartCoord(startCoord);
					prevStartCoord = prevStartCoord
							- fontMetrics.stringWidth(text.getText()) - Util.MTB_TEXT_BUFFER;
				}
			} 
			// this else block is now obsolete due to the return block above involving newDiffBetweenEntityCoords
			// if I am wrong, then uncomment this
//			else {
//				double d1 = availableWidth;
//				double d2 = requestedWidth;
//				double d = d1 / d2;
//
//				for (Text text : textInInterval) {
//					text.setEndCoord(prevStartCoord);
//					double adjustedTextWidth = fontMetrics.stringWidth(text
//							.getText()) * d - 3;
//					prevStartCoord = prevStartCoord - BUFFER - (int) adjustedTextWidth;
//				}
//			}
		}
	}

	public static boolean setSelectedEventStartTime(MediaToolBar bar, int time) {

		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
		EventsEntity parentEntity = selectedEntity.getParentEntity();

		if (bar.isUtterance() && parentEntity != null) {
			int utteranceStartTime = parentEntity.getStartTimeInfo()
					.getMovieTime();
			int utteranceEndTime = parentEntity.getEndTimeInfo().getMovieTime();

			if (time < utteranceStartTime) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
				return false;
			} else if (time > utteranceEndTime) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
				return false;
			}
		}

		SignStreamSegmentPanel panel = bar.getSignStreamsegmentPanel();
		Event event = bar.getSelectedEvent();

		boolean success = MediaToolbarCommons.setSelectedObjectStartTime(bar,
				time);

		panel.setVisible(event);

		bar.getStartTimeTextField().setText(
				"" + event.getStartTimeInfo().getMovieTime());
		
		panel.repaint();

		return success;
	}
	
	public static boolean setSelectedEventEndTime(MediaToolBar bar, int time) {

		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
		EventsEntity parentEntity = selectedEntity.getParentEntity();

		if (bar.isUtterance() && parentEntity != null) {
			int utteranceStartTime = parentEntity.getStartTimeInfo()
					.getMovieTime();
			int utteranceEndTime = parentEntity.getEndTimeInfo().getMovieTime();

			if (time < utteranceStartTime) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
				return false;
			} else if (time > utteranceEndTime) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
				return false;
			}
		}

		SignStreamSegmentPanel panel = bar.getSignStreamsegmentPanel();
		Event event = bar.getSelectedEvent();

		MediaToolbarCommons.setSelectedObjectEndTime(bar, time);

		panel.setVisible(event);

		bar.getEndTimeTextField().setText(
				"" + event.getEndTimeInfo().getMovieTime());
		
		panel.repaint();

		return true; //success;
	}

	//

	/**
	 * This method sets selected object's start time to current movie time. If
	 * this operation cannot be performed, warning message will be shown.
	 */
	private static boolean setSelectedObjectStartTime(MediaToolBar bar, int time) {

		Field field = bar.getSelectedField();
		Event event = bar.getSelectedEvent();
		EventsEntity entity = bar.getSelectedEventsEntity();
		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
		EventsEntity parentEntity = selectedEntity.getParentEntity();
		SignStreamSegmentPanel panel = bar.getSignStreamsegmentPanel();

		// int time = panel.getMultipleMovieController().getCurrentTime();

		if (entity.getField() instanceof PresentationField) {
			// PresentationEventsEntity pEntity = (PresentationEventsEntity)
			// entity;
			// PresentationField pField = (PresentationField)
			// pEntity.getField();
			//
			// int startTime = pEntity.getStartTimeInfo().getMovieTime();
			// int endTime = pEntity.getEndTimeInfo().getMovieTime();
			//
			// if (time >= endTime) {
			// SS3Singleton.getErrorMessages().showErrorMessage(
			// ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
			// return false;
			// } else if (time > startTime) {
			// // update
			// pEntity.getStartTimeInfo().setMovieTime(time);
			// } else {
			// EventsEntity ppEntity = pField.getPreviousEntity(startTime);
			// if (ppEntity == null) {
			// // update
			// pEntity.getStartTimeInfo().setMovieTime(time);
			// } else if (ppEntity.getEndTimeInfo().getMovieTime() < time) {
			// // update
			// pEntity.getStartTimeInfo().setMovieTime(time);
			// } else {
			// SS3Singleton.getErrorMessages().showErrorMessage(
			// ErrorMessages.CHAINS_CANNOT_OVERLAP);
			// return false;
			// }
			// }
			return true;
		} else if (field instanceof GlossField) {
			GlossField glsField = (GlossField) field;
			if (parentEntity != null) {
				if (parentEntity.getStartTimeInfo().getMovieTime() > time
						|| parentEntity.getEndTimeInfo().getMovieTime() < time) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
					return false;
				}
			}

			GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) entity;
			GlossChainedEventsEntity prevEntity = glsEntity.getPreviousEntity();

			if (prevEntity != null) {
				int prevEntityEndMovieTime = prevEntity.getEndTimeInfo()
						.getMovieTime();
				int prevEntityStartMovieTime = prevEntity.getStartTimeInfo()
						.getMovieTime();

				if (prevEntityEndMovieTime > time) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CHAINS_CANNOT_OVERLAP);
					return false;
				} else if (time == prevEntityEndMovieTime && prevEntityEndMovieTime == prevEntityStartMovieTime) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CHAINS_CANNOT_OVERLAP);
					return false;
				}
			}

			int moveFromCoord = glsEntity.getStartTimeCoordinate();

			String resp = glsEntity.setSelectedEventStartMovieTime(time);

			if (resp != null) {
				SS3Singleton.getErrorMessages().showErrorMessage(resp);
				return false;
			} else {
				ChainedEvent onset = glsEntity.getOnset();
				ChainedEvent offset = glsEntity.getOffset();
				ChainedEvent initialHold = glsEntity.getInitialHold();
				ChainedEvent finalHold = glsEntity.getFinalHold();
				ChainedEvent text = glsEntity.getTextValue();

				GlossChainedEventsEntity depGlsEntity = glsEntity
						.getDependentGlossEntity();

				ChainedEventsEntity handEntity = glsEntity
						.getDependentHandEntity();

				HandShapeEntity handShapeEntity = glsEntity
						.getDependentHandShapeEntity();

				ChainedEvent ndOffset = null;

				boolean glsLockedGlossPhoneAlignments = glsEntity
						.isLockedGlossPhonAlignments();
				boolean glsLockedLeftRightAlignments = glsEntity
						.isLockedLeftRightAlignments();
				boolean ndGlsLockedGlossPhoneAlignments = false;
				if (depGlsEntity != null) {
					ndGlsLockedGlossPhoneAlignments = depGlsEntity
							.isLockedGlossPhonAlignments();
					ndOffset = glsEntity.getDependentGlossEntity().getOffset();
				}

				if (event.equals(onset)) {
					if (glsLockedLeftRightAlignments && depGlsEntity != null
							&& depGlsEntity.getOnset() != null) {
						ChainedEvent ndOnset = depGlsEntity.getOnset();
						GlossChainedEventsEntity prevEntity2 = depGlsEntity
								.getPreviousEntity();

						if (prevEntity2 != null) {
							int mTime = prevEntity2.getEndTimeInfo()
									.getMovieTime();
							if (mTime > time) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								ndOnset.getStartTimeInfo().setMovieTime(time);
							}
						} else {
							ndOnset.getStartTimeInfo().setMovieTime(time);
						}
					}
				} else if (event.equals(initialHold)) {
					if (glsLockedGlossPhoneAlignments) {
						if (handEntity != null) {
							ChainedEvent textValue = handEntity.getTextValue();
							ChainedEventsEntity prevHandEntity = handEntity
									.getPreviousEntity();

							if (prevHandEntity == null) {
								textValue.getStartTimeInfo().setMovieTime(time);
							} else {
								int prevEntMovieTime = prevHandEntity
										.getEndTimeInfo().getMovieTime();
								if (time < prevEntMovieTime) {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.CHAINS_CANNOT_OVERLAP);
									return false;
								} else {
									textValue.getStartTimeInfo().setMovieTime(
											time);
								}
							}
						}
						if (handShapeEntity != null) {
							setHandshapeStartTime(handShapeEntity, time);
						}
					}

					if (glsLockedLeftRightAlignments) {
						if (depGlsEntity != null
								&& depGlsEntity.getInitialHold() != null) {
							ChainedEvent ndInitialHold = depGlsEntity
									.getInitialHold();
							ChainedEvent ndOnset = depGlsEntity.getOnset();
							if (ndOnset != null) {
								int minOnsetEndTime = ndOnset
										.getStartTimeInfo().getMovieTime()
										+ (int) panel
												.getMultipleMovieController()
												.getPacketDuration();

								if (minOnsetEndTime > time) {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.ADD_INITIAL_HOLD_CANT_ADJUST_ONSET);
									return false;
								} else {
									ndOnset.getEndTimeInfo().setMovieTime(time);
									ndInitialHold.getStartTimeInfo()
											.setMovieTime(time);

									if (depGlsEntity
											.isLockedGlossPhonAlignments()) {
										ChainedEventsEntity depHandEntity = depGlsEntity
												.getDependentHandEntity();
										HandShapeEntity depHandShapeEntity = depGlsEntity
												.getDependentHandShapeEntity();

										if (depHandEntity != null) {

											ChainedEvent textValue = depHandEntity
													.getTextValue();
											ChainedEventsEntity prevHandEntity = depHandEntity
													.getPreviousEntity();

											if (prevHandEntity == null) {
												textValue.getStartTimeInfo()
														.setMovieTime(time);
											} else {
												int prevEntMovieTime = prevHandEntity
														.getEndTimeInfo()
														.getMovieTime();
												if (time < prevEntMovieTime) {
													SS3Singleton
															.getErrorMessages()
															.showErrorMessage(
																	ErrorMessages.CHAINS_CANNOT_OVERLAP);
													return false;
												} else {
													textValue
															.getStartTimeInfo()
															.setMovieTime(time);
												}
											}
											if (depHandShapeEntity != null) {
												setHandshapeStartTime(
														depHandShapeEntity,
														time);
											}
										}
									}
								}
							} else {
								GlossChainedEventsEntity prevEntity1 = depGlsEntity
										.getPreviousEntity();
								if (prevEntity1 != null) {
									int prevEndMovieTime = prevEntity1
											.getEndTimeInfo().getMovieTime();
									if (time < prevEndMovieTime) {
										SS3Singleton
												.getErrorMessages()
												.showErrorMessage(
														ErrorMessages.CHAINS_CANNOT_OVERLAP);
										return false;
									} else {
										ndInitialHold.getStartTimeInfo()
												.setMovieTime(time);
									}
								} else {
									ndInitialHold.getStartTimeInfo()
											.setMovieTime(time);
								}

								if (ndGlsLockedGlossPhoneAlignments) {
									ChainedEventsEntity depHandEntity = depGlsEntity
											.getDependentHandEntity();
									HandShapeEntity depHandShapeEntity = depGlsEntity
											.getDependentHandShapeEntity();

									if (depHandEntity != null) {
										ChainedEvent textValue = depHandEntity
												.getTextValue();
										ChainedEventsEntity prevHandEntity = depHandEntity
												.getPreviousEntity();
										if (prevHandEntity == null) {
											textValue.getStartTimeInfo()
													.setMovieTime(time);
										} else {
											int prevEntMovieTime = prevHandEntity
													.getEndTimeInfo()
													.getMovieTime();
											if (time < prevEntMovieTime) {
												SS3Singleton
														.getErrorMessages()
														.showErrorMessage(
																ErrorMessages.CHAINS_CANNOT_OVERLAP);
												return false;
											}

											textValue.getStartTimeInfo()
													.setMovieTime(time);
										}

										if (depHandShapeEntity != null) {
											setHandshapeStartTime(
													depHandShapeEntity, time);
										}
									}
								}
							}
						}
					}
				} else if (event.equals(text)) {
					if (glsLockedGlossPhoneAlignments) {
						if (handEntity != null) {
							if (initialHold == null) {
								ChainedEventsEntity prevEntity3 = handEntity
										.getPreviousEntity();
								if (prevEntity3 == null) {
									handEntity.getStartTimeInfo().setMovieTime(
											time);
								} else if (prevEntity3.getEndTimeInfo()
										.getMovieTime() > time) {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_VALUE__CANT_DELETE_INITIAL_HOLD);
									return false;
								} else {
									handEntity.getStartTimeInfo().setMovieTime(
											time);
								}

								if (handShapeEntity != null) {
									setHandshapeStartTime(handShapeEntity, time);
								}
							} else {
								// do not update - hand start time should be
								// equal to initial hold start time
							}
						}
					}

					if (glsLockedLeftRightAlignments && depGlsEntity != null) {

						ChainedEvent depOnset = depGlsEntity.getOnset();
						ChainedEvent depInitialHold = depGlsEntity
								.getInitialHold();

						if (depInitialHold != null) {
							int minEndTime = depInitialHold.getStartTimeInfo()
									.getMovieTime()
									+ (int) panel.getMultipleMovieController()
											.getPacketDuration();
							if (minEndTime > time) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_START_VALUE__CANT_DELETE_INITIAL_HOLD);
								return false;
							} else {
								depGlsEntity.getTextValue().getStartTimeInfo()
										.setMovieTime(time);
								depInitialHold.getEndTimeInfo().setMovieTime(
										time);
							}
						} else if (depOnset != null) {
							int minEndTime = depOnset.getStartTimeInfo()
									.getMovieTime()
									+ (int) panel.getMultipleMovieController()
											.getPacketDuration();
							if (minEndTime > time) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_START_VALUE_CANT_DELETE_ONSET);
								return false;
							} else {
								depGlsEntity.getTextValue().getStartTimeInfo()
										.setMovieTime(time);
								depOnset.getEndTimeInfo().setMovieTime(time);

								ChainedEventsEntity depHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity depHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();

								if (depGlsEntity.isLockedGlossPhonAlignments()) {
									if (depHandEntity != null) {
										ChainedEventsEntity depHandPrevEntity = depHandEntity
												.getPreviousEntity();
										if (depHandPrevEntity == null) {
											depHandEntity.getStartTimeInfo()
													.setMovieTime(time);
										} else if (depHandPrevEntity
												.getEndTimeInfo()
												.getMovieTime() > time) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.CHAINS_CANNOT_OVERLAP);
											return false;
										} else {
											depHandEntity.getStartTimeInfo()
													.setMovieTime(time);
										}

										if (depHandShapeEntity != null) {
											setHandshapeStartTime(
													depHandShapeEntity, time);
										}
									}
								}
							}
						} else {
							depGlsEntity.getTextValue().getStartTimeInfo()
									.setMovieTime(time);

							if (depGlsEntity.isLockedGlossPhonAlignments()) {
								ChainedEventsEntity depHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity depHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();

								if (depHandEntity != null) {
									ChainedEventsEntity depHandPrevEntity = depHandEntity
											.getPreviousEntity();
									if (depHandPrevEntity == null) {
										depHandEntity.getStartTimeInfo()
												.setMovieTime(time);
									} else if (depHandPrevEntity
											.getEndTimeInfo().getMovieTime() > time) {
										SS3Singleton
												.getErrorMessages()
												.showErrorMessage(
														ErrorMessages.CHAINS_CANNOT_OVERLAP);
										return false;
									} else {
										depHandEntity.getStartTimeInfo()
												.setMovieTime(time);
									}

									if (depHandShapeEntity != null) {
										setHandshapeStartTime(
												depHandShapeEntity, time);
									}
								}
							}
						}
					}
				} else if (event.equals(offset)) {

					if (glsLockedGlossPhoneAlignments) {
						if (handEntity != null) {
							ChainedEventsEntity nextHandEntity = handEntity
									.getNextEntity();
							if (nextHandEntity == null) {
								handEntity.getEndTimeInfo().setMovieTime(time);
							} else if (nextHandEntity.getStartTimeInfo()
									.getMovieTime() < time) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								handEntity.getStartTimeInfo()
										.setMovieTime(time);
							}

							if (handShapeEntity != null) {
								setHandshapeEndTime(handShapeEntity, time);
							}
						}
					}

					if (depGlsEntity != null && ndOffset != null
							&& glsLockedLeftRightAlignments) {
						ChainedEvent ndFinalHold = depGlsEntity.getFinalHold();
						GlossChainedEvent ndValue = depGlsEntity
								.getGlossChainedEvent();

						int packetDuration = (int) panel
								.getMultipleMovieController()
								.getPacketDuration();

						int maxStartTime = ndOffset.getEndTimeInfo()
								.getMovieTime() - packetDuration;

						if (time <= maxStartTime) {
							if (ndFinalHold != null) {
								int minHoldEndTime = ndFinalHold
										.getStartTimeInfo().getMovieTime()
										+ packetDuration;
								if (minHoldEndTime <= time) {
									ndOffset.getStartTimeInfo().setMovieTime(
											time);
									ndFinalHold.getEndTimeInfo().setMovieTime(
											time);

									if (depGlsEntity
											.isLockedGlossPhonAlignments()) {
										ChainedEventsEntity ndDepHandEntity = depGlsEntity
												.getDependentHandEntity();
										HandShapeEntity ndDepHandShapeEntity = depGlsEntity
												.getDependentHandShapeEntity();
										if (ndDepHandEntity != null) {
											ChainedEventsEntity ndNextEntity = ndDepHandEntity
													.getNextEntity();
											if (ndNextEntity == null) {
												ndDepHandEntity.getTextValue()
														.getEndTimeInfo()
														.setMovieTime(time);
												if (ndDepHandShapeEntity != null) {
													setHandshapeEndTime(
															ndDepHandShapeEntity,
															time);
												}
											} else {
												int nextEntityMovieTime = ndNextEntity
														.getStartTimeInfo()
														.getMovieTime();
												if (nextEntityMovieTime < time) {
													SS3Singleton
															.getErrorMessages()
															.showErrorMessage(
																	ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
													return false;
												} else {
													ndDepHandEntity
															.getTextValue()
															.getEndTimeInfo()
															.setMovieTime(time);
													if (ndDepHandShapeEntity != null) {
														setHandshapeStartTime(
																ndDepHandShapeEntity,
																time);
													}
												}
											}
										}
									}
								} else {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_OFFSET_CANT_DELETE_FINAL_HOLD);
									return false;
								}
							} else {
								int minEndTime = ndValue.getStartTimeInfo()
										.getMovieTime() + packetDuration;
								if (minEndTime <= time) {
									ndOffset.getStartTimeInfo().setMovieTime(
											time);
									ndValue.getEndTimeInfo().setTimeScale(time);

									if (depGlsEntity
											.isLockedGlossPhonAlignments()) {

										ChainedEventsEntity ndDepHandEntity = depGlsEntity
												.getDependentHandEntity();
										HandShapeEntity ndDepHandShapeEntity = depGlsEntity
												.getDependentHandShapeEntity();
										if (ndDepHandEntity != null) {
											ChainedEventsEntity ndNextEntity = ndDepHandEntity
													.getNextEntity();
											if (ndNextEntity == null) {
												ndDepHandEntity.getTextValue()
														.getEndTimeInfo()
														.setMovieTime(time);
												if (ndDepHandShapeEntity != null) {
													setHandshapeEndTime(
															ndDepHandShapeEntity,
															time);
												}
											} else {
												int nextEntityMovieTime = ndNextEntity
														.getStartTimeInfo()
														.getMovieTime();
												if (nextEntityMovieTime < time) {
													SS3Singleton
															.getErrorMessages()
															.showErrorMessage(
																	ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
													return false;
												} else {
													ndDepHandEntity
															.getTextValue()
															.getEndTimeInfo()
															.setMovieTime(time);
													if (ndDepHandShapeEntity != null) {
														setHandshapeStartTime(
																ndDepHandShapeEntity,
																time);
													}
												}
											}
										}
									}

									depGlsEntity.setOffsetEvent(ndOffset);
								} else {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_OFFSET_CANT_DELETE_VALUE);
									return false;
								}
							}
						} else {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						}
					}
				} else if (event.equals(finalHold)) {
					if (glsLockedLeftRightAlignments && depGlsEntity != null
							&& depGlsEntity.getFinalHold() != null) {
						ChainedEvent ndFinalHold = depGlsEntity.getFinalHold();
						GlossChainedEvent ndValue = depGlsEntity
								.getGlossChainedEvent();

						int packetDuration = (int) panel
								.getMultipleMovieController()
								.getPacketDuration();

						int minValueEndTime = ndValue.getStartTimeInfo()
								.getMovieTime() + packetDuration;
						int maxHoldStartTime = ndFinalHold.getEndTimeInfo()
								.getMovieTime() - packetDuration;

						if (minValueEndTime > time) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.SET_START_FINAL_HOLD_CANT_DELETE_VALUE);
							return false;
						} else if (time > maxHoldStartTime) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else {
							ndFinalHold.getStartTimeInfo().setMovieTime(time);
							ndValue.getEndTimeInfo().setMovieTime(time);
						}
					}
				}
				
				moveTextBackward(moveFromCoord, glsEntity.getStartTimeCoordinate(),
						glsField, glsEntity, bar);
			}
		} else if (field instanceof HandShapeField) {
			HandShapeEntity hsEntity = (HandShapeEntity) entity;
			if (hsEntity.isLockedGlossPhonAlignments()) {
				setHandshapeStartTime(hsEntity, time);
			} else {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.UNLOCK_GLOSS_PHON_ALIGNMENTS);
				return false;
			}

		} else if (field instanceof NonManualField) {
			if (entity != null) {
				if (parentEntity.getStartTimeInfo().getMovieTime() > time
						|| parentEntity.getEndTimeInfo().getMovieTime() < time) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
					return false;
				}
			}

			ChainedEventsEntity nmEntity = (ChainedEventsEntity) entity;
			String id = nmEntity.getField().getFieldID();
			if (id.equalsIgnoreCase("10501") || id.equalsIgnoreCase("10502")) {
				if (nmEntity.isLockedGlossPhonAlignments()) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.UNLOCK_GLOSS_PHON_ALIGNMENTS);

					return false;
				}
			}

			String errorCode = nmEntity.setSelectedEventStartMovieTime(time);
			if (errorCode != null) {
				SS3Singleton.getErrorMessages().showErrorMessage(errorCode);
				return false;
			}

			// set start time
		}
		return true;
	}
//
//	private static boolean setTtt(int time, ChainedEventsEntity entity,
//			Event event, MediaToolBar bar) {
//
//		ChainedEvent onset = entity.getOnset();
//		ChainedEvent offset = entity.getOffset();
//		ChainedEvent initialHold = entity.getInitialHold();
//		ChainedEvent finalHold = entity.getFinalHold();
//		ChainedEvent text = entity.getTextValue();
//
//		ChainedEventsEntity prevEntity = entity.getPreviousEntity();
//
//		int precedingTime = bar.getSignStreamsegmentPanel()
//				.getMultipleMovieController().getPrecedingFrameTime(time);
//
//		int followingTime = bar.getSignStreamsegmentPanel()
//				.getMultipleMovieController().getFollowingFrameTime(time);
//
//		int eventStartTime = event.getStartTimeInfo().getMovieTime();
//
//		int eventEndTime = event.getEndTimeInfo().getMovieTime();
//
//		// check if entities would not overlap
//		if (time < prevEntity.getEndTimeInfo().getMovieTime()) {
//			SS3Singleton.getErrorMessages().showErrorMessage(
//					ErrorMessages.CHAINS_CANNOT_OVERLAP);
//			return false;
//		}
//
//		// check if start time is smaller then end time
//		if (eventEndTime < time) {
//			SS3Singleton.getErrorMessages().showErrorMessage(
//					ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
//			return false;
//		}
//
//		if (event.equals(onset)) {
//
//			// onset must be at least one frame long
//			int onsetMaxStartTime = bar
//					.getSignStreamsegmentPanel()
//					.getMultipleMovieController()
//					.getPrecedingFrameTime(
//							onset.getEndTimeInfo().getMovieTime());
//
//			if (time > onsetMaxStartTime) {
//				SS3Singleton.getErrorMessages().showErrorMessage(
//						ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
//				return false;
//			}
//
//			event.getStartTimeInfo().setMovieTime(time);
//			return true;
//		} else if (event.equals(initialHold)) {
//
//			// hold must be at least one frame long
//			int holdMaxStartTime = bar
//					.getSignStreamsegmentPanel()
//					.getMultipleMovieController()
//					.getPrecedingFrameTime(
//							initialHold.getEndTimeInfo().getMovieTime());
//
//			if (time > holdMaxStartTime) {
//				SS3Singleton.getErrorMessages().showErrorMessage(
//						ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
//				return false;
//			}
//
//			if (onset != null) {
//
//				// onset must be at least one frame long
//				int onsetMinEndTime = bar
//						.getSignStreamsegmentPanel()
//						.getMultipleMovieController()
//						.getFollowingFrameTime(
//								onset.getStartTimeInfo().getMovieTime());
//
//				if (onsetMinEndTime > time) {
//					SS3Singleton
//							.getErrorMessages()
//							.showErrorMessage(
//									ErrorMessages.SET_START_INITIAL_HOLD_CANT_DELETE_ONSET);
//					return false;
//				} else {
//					event.getStartTimeInfo().setMovieTime(time);
//					onset.getEndTimeInfo().setMovieTime(time);
//					return true;
//				}
//			} else {
//				event.getStartTimeInfo().setMovieTime(time);
//				return true;
//			}
//		} else if (event.equals(text)) {
//
//			if (initialHold != null) {
//
//				// initialHold must be at least one frame long
//				int holdMinEndTime = bar
//						.getSignStreamsegmentPanel()
//						.getMultipleMovieController()
//						.getFollowingFrameTime(
//								initialHold.getStartTimeInfo().getMovieTime());
//
//				if (holdMinEndTime > time) {
//					SS3Singleton
//							.getErrorMessages()
//							.showErrorMessage(
//									ErrorMessages.SET_START_VALUE__CANT_DELETE_INITIAL_HOLD);
//					return false;
//				} else {
//					event.getStartTimeInfo().setMovieTime(time);
//					initialHold.getEndTimeInfo().setMovieTime(time);
//					return true;
//				}
//
//			} else if (onset != null) {
//				// onset must be at least one frame long
//				int onsetMinEndTime = bar
//						.getSignStreamsegmentPanel()
//						.getMultipleMovieController()
//						.getFollowingFrameTime(
//								onset.getStartTimeInfo().getMovieTime());
//
//				if (onsetMinEndTime > time) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.SET_START_VALUE_CANT_DELETE_ONSET);
//					return false;
//				} else {
//					event.getStartTimeInfo().setMovieTime(time);
//					onset.getEndTimeInfo().setMovieTime(time);
//					return true;
//				}
//
//			} else {
//				// set event start time
//				event.getStartTimeInfo().setMovieTime(time);
//				return true;
//			}
//		} else if (event.equals(finalHold)) {
//			// hold must be at least one frame
//			int holdMaxStartTime = bar
//					.getSignStreamsegmentPanel()
//					.getMultipleMovieController()
//					.getPrecedingFrameTime(
//							finalHold.getEndTimeInfo().getMovieTime());
//
//			if (holdMaxStartTime < time) {
//				SS3Singleton.getErrorMessages().showErrorMessage(
//						ErrorMessages.SET_START_FINAL_HOLD_CANT_DELETE_VALUE);
//				return false;
//			} else {
//				// text must be at least single frame (start time <= end time)
////				int minTextEndTime = bar
////						.getSignStreamsegmentPanel()
////						.getMultipleMovieController()
////						.getFollowingFrameTime(
////								text.getStartTimeInfo().getMovieTime());
//
//				if(time < text.getStartTimeInfo().getMovieTime()) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.SET_START_FINAL_HOLD_CANT_DELETE_VALUE);
//					return false;
//				}
//
//				event.getStartTimeInfo().setMovieTime(time);
//				text.getEndTimeInfo().setMovieTime(time);
//				return true;
//			}
//
//		} else if (event.equals(offset)) {
//			
//			if(finalHold != null) {
//				int minFinalHoldEndTime = bar
//						.getSignStreamsegmentPanel()
//						.getMultipleMovieController()
//						.getFollowingFrameTime(
//								finalHold.getStartTimeInfo().getMovieTime());
//				
//				if(minFinalHoldEndTime > time) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.SET_START_OFFSET_CANT_DELETE_FINAL_HOLD);
//					return false;
//				} else {
//					event.getStartTimeInfo().setMovieTime(time);
//					finalHold.getEndTimeInfo().setMovieTime(time);
//					return true;
//				}
//
//			} else {
//				if(time < text.getStartTimeInfo().getMovieTime()) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.SET_START_OFFSET_CANT_DELETE_VALUE);
//					return false;
//				}
//
//				event.getStartTimeInfo().setMovieTime(time);
//				text.getEndTimeInfo().setMovieTime(time);
//				return true;
//
//			}
//		}
//		// should not be here, invalid option
//		return false;
//	}
//
	public static void setPresentationEventEndTime(int time, Event event,
			EventsEntity entity, Field field, SignStreamSegmentPanel panel,
			JTextField txtField) {

		int startTime = event.getStartTimeInfo().getMovieTime();
		int endTime = event.getEndTimeInfo().getMovieTime();
		int movieEndTime = panel.getMultipleMovieController().getMovieDuration();
		
		if(field == null) {
			field = entity.getField();
		}
		
		PresentationField presentationField = (PresentationField) field;
		EventsEntity nextEntity = presentationField.getNextEntity(endTime);

		PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;
		ArrayList<PresentationEventsEntity> entities = pEntity
				.getDependentEntities();

		for (PresentationEventsEntity dEntity : entities) {
			int minimalEndTime = dEntity.getSS3GlossWindowItem().getMinimalEndTime();
			if (minimalEndTime != Integer.MIN_VALUE && time < minimalEndTime) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.UTTERANCE_MINIMAL_END_TIME);
				return;
			}
		}

		if (time >= movieEndTime ) {
			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.UTTERANCE_END_AFTER_MOVIE_END);
			return;
		}
		else if (endTime > time && time <= startTime) {

			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
			return;

		} else if (endTime < time) {

			if (nextEntity != null
					&& time > nextEntity.getStartTimeInfo().getMovieTime()) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}
		}

		event.getEndTimeInfo().setMovieTime(time);
		txtField.setText("" + time);

		for (PresentationEventsEntity dEntity : entities) {
			dEntity.getEndTimeInfo().setMovieTime(time);
//			dEntity.getSS3GlossWindowItem().setTranslationFieldEntityEndTime(time);
		}

		panel.getSlider().setSliderPositionTime(time);
		panel.repaint();
	}

	public static void setPresentationEventStartTime(int time, Event event,
			EventsEntity entity, Field field, SignStreamSegmentPanel panel,
			JTextField txtField) {

		int startTime = event.getStartTimeInfo().getMovieTime();
		int endTime = event.getEndTimeInfo().getMovieTime();
		int movieStartTime = 0;		// obvious, but explicit
		
		if (field == null) {
			field = entity.getField();
		}
		
		PresentationField presentationField = (PresentationField) field;
		EventsEntity prevEntity = presentationField.getPreviousEntity(startTime);

		PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;		
		ArrayList<PresentationEventsEntity> entities = pEntity
				.getDependentEntities();

		for (PresentationEventsEntity dEntity : entities) {
			int maximalStartTime = dEntity.getSS3GlossWindowItem().getMaximalStartTime();
			if (maximalStartTime != Integer.MAX_VALUE && time > maximalStartTime) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.UTTERANCE_MAXIMAL_START_TIME);
				return;
			}
		}
		
		if (time < movieStartTime) {
			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.UTTERANCE_START_BEFORE_MOVIE_START);
			return;
		}
		else if (startTime < time && time >= endTime) {

			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
			return;

		} else if (startTime > time) {
			
			if (prevEntity != null && prevEntity.getEndTimeInfo().getMovieTime() > time) {

				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}
		}

		txtField.setText("" + time);
		event.getStartTimeInfo().setMovieTime(time);

		for (PresentationEventsEntity dEntity : entities) {
			dEntity.getStartTimeInfo().setMovieTime(time);
//			dEntity.getSS3GlossWindowItem().setTranslationFieldEntityStartTime(time);
		}

		panel.getSlider().setSliderPositionTime(time);
		panel.repaint();
	}
}
