package edu.bu.signstream.grepresentation.view.signbank;

import java.util.ArrayList;
import java.util.HashMap;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.signbank.SignBankResource;
import edu.bu.signstream.common.util.signbank.SignBankResourceCollection;

public class SignBankCollection {
	
	private HashMap<String, SignBankMainGloss> mainGlossesList = null;
	
	private SignBankResourceCollection signBankResourceCollection = null;
	
	public SignBankCollection(SignBankResourceCollection resourceCollection) {
		this.mainGlossesList = new HashMap<String, SignBankMainGloss>();
		this.signBankResourceCollection = resourceCollection;
	}

	/**
	 * Gets the list of SignBankMainGloss objects
	 * @return A HashMap from MainGloss Label -> MainGloss
	 */
	public HashMap<String, SignBankMainGloss> getMainGlossesList() {
		return mainGlossesList;
	}
	
	public HashMap<String, SignBankMainGloss> getMainGlossesListFilterByResource(SignBankResource resource) {
		// TODO
		return null;
	}
	
	public SignBankResourceCollection getSignBankResourceCollection() {
		return signBankResourceCollection;
	}
	
	public boolean canAccessSignBank() {
		return signBankResourceCollection.canAccessSignBank();
	}

	/**
	 * Given the params from the lookup panel, look up in this collection object and 
	 * return a list of occurrence objects which match those params
	 * @param signLabelText
	 * @param exactWord
	 * @param searchHS
	 * @param searchOneHanded
	 * @param searchTwoHanded
	 * @param domStartHSImageLabel
	 * @param nondomStartHSImageLabel
	 * @param domEndHSImageLabel
	 * @param nondomEndHSImageLabel
	 * @return
	 */
	public ArrayList<SignBankOccurrence> lookup(String signLabelText, boolean exactWord, 
			boolean searchHS, boolean searchOneHanded, boolean searchTwoHanded,
			String domStartHSImageLabel, String nondomStartHSImageLabel, 
			String domEndHSImageLabel, String nondomEndHSImageLabel) {
		
		ArrayList<SignBankOccurrence> resultOccurrences = new ArrayList<SignBankOccurrence>();
		
		for (SignBankMainGloss mainGloss: mainGlossesList.values()) {
			//Check is main gloss matches search query.
			boolean exactMatch = mainGloss.getMainGlossLabel().toUpperCase().equals(signLabelText.toUpperCase());
			boolean closeMatch = mainGloss.getMainGlossLabel().toUpperCase().contains(signLabelText.toUpperCase());
			if (!exactWord && closeMatch || exactMatch) {
				//If so, and atleast 1 variant meets handshape/handedness requirements, add all of its variants.
				if (mainGlossMeetsRequirements(mainGloss, searchHS, searchOneHanded, searchTwoHanded, domStartHSImageLabel, nondomStartHSImageLabel, domEndHSImageLabel, nondomEndHSImageLabel)) {
					addAllVariants(mainGloss, resultOccurrences);
				}
				continue;
			} 
			
			//Otherwise, check each individual variant for the search query.
			for (SignBankVariant variant: mainGloss.getVariantsList().values()) {
				
				String variantLabel = variant.getVariantLabel();
				if (variantLabel.isEmpty() || variantLabel == "")
					continue;
				
				// check whether this variant label is (equal/contains) the text we're looking for
				if (exactWord) {
					if ( !variantLabel.toUpperCase().equals(signLabelText.toUpperCase())) {
						continue;
					}
				} else {
					if ( !variantLabel.toUpperCase().contains(signLabelText.toUpperCase())) {
						continue;
					}
				}
				//If so, and atleast 1 of this gloss's variants meets handshape/handedness requirements, add all variants.
				if (mainGlossMeetsRequirements(mainGloss, searchHS, searchOneHanded, searchTwoHanded, domStartHSImageLabel, nondomStartHSImageLabel, domEndHSImageLabel, nondomEndHSImageLabel)) {
					addAllVariants(mainGloss, resultOccurrences);
				}
				break;
			}
		}
		
		return resultOccurrences;
	}
	
	private boolean mainGlossMeetsRequirements(SignBankMainGloss mainGloss, boolean searchHS, boolean searchOneHanded, boolean searchTwoHanded,
			String domStartHSImageLabel, String nondomStartHSImageLabel, 
			String domEndHSImageLabel, String nondomEndHSImageLabel) {
		boolean meetsHandedRequirement = false;
		boolean meetsHandshapeRequirement = !searchHS;
		
		for (SignBankVariant v : mainGloss.getVariantsList().values()) {
			//If 1H, ensure atleast 1 variant is 1H, if 2H, ensure atleast 1 variant is 2H.
			if (v.getDefaultOccurrence() != null && (searchOneHanded && !v.getDefaultOccurrence().isTwoHanded() || searchTwoHanded && v.getDefaultOccurrence().isTwoHanded()))
				meetsHandedRequirement = true;
			//If searching for HandShapes, ensure atleast 1 variant matches handshapes.
			if (!meetsHandshapeRequirement) {
				meetsHandshapeRequirement = searchHandshapes(searchOneHanded, searchTwoHanded, v, domStartHSImageLabel, nondomStartHSImageLabel, domEndHSImageLabel, nondomEndHSImageLabel);
			}
		}
		
		return meetsHandedRequirement && meetsHandshapeRequirement;
	}
	
	private void addAllVariants(SignBankMainGloss mainGloss, ArrayList<SignBankOccurrence> resultOccurrences) {
		for (SignBankVariant v : mainGloss.getVariantsList().values()) {
			if (v.getDefaultOccurrence() != null)
				resultOccurrences.add(v.getDefaultOccurrence());
		}
	}
	

	private boolean searchHandshapes(boolean searchOneHanded, boolean searchTwoHanded, SignBankVariant variant, String domStartHSImageLabel, String nondomStartHSImageLabel, 
			String domEndHSImageLabel, String nondomEndHSImageLabel) {
			for (SignBankOccurrence occurrence: variant.getOccurrencesList().values()) {
				String domStartHS = occurrence.getDomStartHS();
				String domEndHS = occurrence.getDomEndHS();
				String nondomStartHS = occurrence.getNondomStartHS();
				String nondomEndHS = occurrence.getNondomEndHS();
				boolean occTwoHanded = occurrence.isTwoHanded();
				
				boolean  matchFound = false;
				
				// this case is searching both one-handed signs and two-handed signs - an either search
				if (searchOneHanded && searchTwoHanded) {
					if (occTwoHanded) {
						if (Util.matchSignBank(domStartHSImageLabel, domStartHS)
								&& Util.matchSignBank(domEndHSImageLabel, domEndHS)
								&& Util.matchSignBank(nondomStartHSImageLabel, nondomStartHS)
								&& Util.matchSignBank(nondomEndHSImageLabel, nondomEndHS) ) {
							matchFound = true;
						}								
					} else {
						if (Util.matchSignBank(domStartHSImageLabel, domStartHS)
								&& Util.matchSignBank(domEndHSImageLabel, domEndHS) ) {
							matchFound = true;
						}								
					}
				} else {
					if (searchOneHanded && !occTwoHanded) {
						if (Util.matchSignBank(domStartHSImageLabel, domStartHS)
								&& Util.matchSignBank(domEndHSImageLabel, domEndHS) ) {
							matchFound = true;
						}
					} else if (searchTwoHanded && occTwoHanded) {
						if (Util.matchSignBank(domStartHSImageLabel, domStartHS)
								&& Util.matchSignBank(domEndHSImageLabel, domEndHS)
								&& Util.matchSignBank(nondomStartHSImageLabel, nondomStartHS)
								&& Util.matchSignBank(nondomEndHSImageLabel, nondomEndHS) ) {
							matchFound = true;
						}
					}
				}
				if (matchFound) return true;
			}
			return false;
	}
	public void deleteSignBankOccurrence(SignBankOccurrence signBankOccurrence) {
		if (signBankOccurrence == null)
			return;
		
		String occID = signBankOccurrence.getOccurrenceID();
		String resourceFileName = signBankOccurrence.getSIgnBankResourceFileName();
		
		for (SignBankMainGloss mainGloss: mainGlossesList.values()) {
			for (SignBankVariant variant: mainGloss.getVariantsList().values()) {
				for (SignBankOccurrence occurrence: variant.getOccurrencesList().values()) {
					if (occurrence.getOccurrenceID().equals(occID) 
							&& occurrence.getSIgnBankResourceFileName().equals(resourceFileName)) {
						variant.getOccurrencesList().remove(occID);
						return;
					}
				}
			}
		}
	}

	/**
	 * Checks this collection for whether a sign exists with the main gloss and variant labels specified.
	 * @param mainGlossStr The main gloss label to match against
	 * @param variantStr The variant label to match against
	 * @return SignBankVariant which is the variant which matches the labels input, or null if not found
	 */
	public SignBankVariant contains(String mainGlossStr, String variantStr) {
		mainGlossStr = mainGlossStr.toUpperCase();
		variantStr = variantStr.toUpperCase();
		
		HashMap<String, SignBankVariant> variants = null;
		
		for (SignBankMainGloss currentMainGloss: mainGlossesList.values()) {
			String currentMainGlossStr = currentMainGloss.getMainGlossLabel().toUpperCase();
			if (currentMainGlossStr.equals(mainGlossStr)) {
				variants = currentMainGloss.getVariantsList();
				for (SignBankVariant currentVariant: variants.values()) {
					String currentVariantStr = currentVariant.getVariantLabel().toUpperCase();
					if (currentVariantStr.equals(variantStr)) {
						return currentVariant;
					}
				}
			}
		}
		
		return null;
	}
	
	/**
	 * <pre>
	 * Returns the SignBankOccurrence in this collection which matches all 3 ID's given, or null if such does not exist.
	 * </pre>
	 * 
	 * @param mainGlossID The main gloss ID of the SignBankOccurrence to find
	 * @param variantID The variant ID of the SignBankOccurrence to find
	 * @param occurrenceID the ID of the SignBankOccurrence to find
	 * @return the SignBankOccurrence object matching the ID given, or null if none found
	 */
	public SignBankOccurrence getOccurrenceWithID(String mainGlossID, String variantID, String occurrenceID) {
		HashMap<String, SignBankVariant> variants = null;
		String currentMainGlossID = null, currentVariantID = null;
		
		for (SignBankMainGloss currentMainGloss: mainGlossesList.values()) {
			currentMainGlossID = currentMainGloss.getMainGlossID();
			if (currentMainGlossID.equals(mainGlossID)) {
				variants = currentMainGloss.getVariantsList();
				for (SignBankVariant currentVariant: variants.values()) {
					currentVariantID = currentVariant.getVariantID();
					if (currentVariantID.equals(variantID)) {
						SignBankOccurrence occurrence = currentVariant.getOccurrenceByID(occurrenceID);
						if (occurrence != null)
							return occurrence;
					}
				}
			}
		}
		
		return null;
	}
	
	@Override
	public String toString() {
		
		String ret = "SignBankCollection[";
		for (SignBankMainGloss mainGloss: mainGlossesList.values()) {
			ret += "\n" + mainGloss.toString();
		}
		for (SignBankResource resource: signBankResourceCollection.getResourcesList()) {
			ret += "\n" + resource.toString();
		}
		ret+="\n/SignBankCollection]";
		
		return ret;
	}

}
