package edu.bu.signstream.navigation;

import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;

import javax.swing.tree.DefaultMutableTreeNode;

import edu.bu.signstream.codingScheme.ui.importLegacyCodingScheme.CodingSchemeUpdateDialog;
import edu.bu.signstream.common.codingSchema.CodingSchema;
import edu.bu.signstream.common.db.SS2toSS3Adaptor;
import edu.bu.signstream.common.db.parser.Parser;
import edu.bu.signstream.common.db.parser.SS2Parser;
import edu.bu.signstream.common.db.parser.SS3Parser;
import edu.bu.signstream.common.db.parser.SS3ZipXmlFilesParser;
import edu.bu.signstream.common.db.parser.XMLParser;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.SignStreamDocument;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingSchemeCollection;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
//import edu.bu.signstream.common.fields.vo.ss3.db.SS3DatabaseCollection;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
//import edu.bu.signstream.common.fields.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
//import edu.bu.signstream.common.fields.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class DatabaseDelegate {
	   private DefaultMutableTreeNode topNode = new DefaultMutableTreeNode("  ");

	   private SignStreamSegmentPanel ssPanel;

	   public DatabaseDelegate(SignStreamSegmentPanel ssPanel) {
	      this.ssPanel = ssPanel;
	   }

	   public DefaultMutableTreeNode getMutableTreeNode() {
	      return topNode;
	   }

	   public void openDb(File file, File zipFile) {
	      DefaultMutableTreeNode node = createNode(file, zipFile);
	      topNode.removeAllChildren();	// if we are switching databases, remove the old db name node
	      topNode.add(node);
	      
	      DefaultMutableTreeNode duplicateTopNode = new DefaultMutableTreeNode("  ");
	      DefaultMutableTreeNode duplicateChildNode = (DefaultMutableTreeNode) node.clone();
	      
	      for (Enumeration e = node.children(); e.hasMoreElements() ;) {
              DefaultMutableTreeNode entNode = (DefaultMutableTreeNode)e.nextElement();
              DefaultMutableTreeNode duplicateEntNode = (DefaultMutableTreeNode) entNode.clone();
              duplicateChildNode.add(duplicateEntNode);
	      }
          
	      duplicateTopNode.add(duplicateChildNode);
			
			// find db which was created in createNode
			for (SS3Database db: SS3Singleton.getSS3DatabaseCollection().getDatabases().values()) {
				String str1 = db.getReadFromXMLFile().toString();
				int index = str1.indexOf("/");
				String str2 = str1.substring(0, index);
				
				if (file.getPath().equals(str2)) {
				      db.setDbNavigationPanelNode(duplicateTopNode);
				}
			}
	      
	   }

	   public void addNewDatabase(String fileName) {
		      
	      DefaultMutableTreeNode fileNode = new DefaultMutableTreeNode(fileName);
	      topNode.add(fileNode);
	   }

	   private void addFile(
			   DefaultMutableTreeNode node, 
			   File file, File zipFile ) { 

	      if (file.isFile()) {
	         String fileName = file.getName();

             if(Util.isXML3File(fileName) || zipFile != null) {
                 // Execute this code if a file is in newest format, filename.3.xml and zipped.
                 // Zipped database file should have mask filename.ss3.zip or filename.ss3
	        	 SS3ZipXmlFilesParser xmlParser = new SS3ZipXmlFilesParser();
	        	 SS3Database db = xmlParser.parseSS3Database(file.getPath());
	        	 
	        	 if(db == null)
	        		 return;

	        	 try {
	        		 File parentFile = file.getParentFile();
	        		 String directory = parentFile.getName();
	        		 String parentDirectoryPath = parentFile.getParent();
	        		 
	        		 if(zipFile!=null) {
	            		 directory = zipFile.getName();
	            		 parentDirectoryPath = zipFile.getParent();
	        		 }
	        		 
	        		 db.setFileName(fileName);
	        		 db.setDirectory(directory);
	        		 db.setParentDirectoryPath(parentDirectoryPath);
	        	 } catch(Exception ex) {
	        		 ex.printStackTrace();
	        	 }

	             ArrayList<String> mediaFileIDs = new ArrayList<String>();
	             ArrayList<SS3MediaFile> mFiles = db.getMediaFilesCollection();
	             Iterator<SS3MediaFile> iterator = mFiles.iterator();
	             while (iterator.hasNext()) {
	                SS3MediaFile mFile = iterator.next();
	                mediaFileIDs.add(mFile.getId());
	             }

	             db.getExcerptObj().setMediaFiles(mediaFileIDs);

	 			// update coding scheme
	 			SS3CodingScheme defSchema = SS3Singleton
	 					.getSignStreamApplication().getDefaultSS3CodingScheme();
	 			SS3CodingScheme localSchema = db.getLocalSS3CodingScheme();
	 			HashMap<String, SS3Field> localFields = localSchema.getFields();
	 			Set<Map.Entry<String, SS3Field>> set = localFields.entrySet();
	 			Iterator<Map.Entry<String, SS3Field>> iter = set.iterator();
	 			while (iter.hasNext()) {
	 				Map.Entry<String, SS3Field> me = iter.next();
	 				SS3Field lField = me.getValue();
	 				String fieldID = lField.getId();
	 				SS3Field dField = defSchema.getField(fieldID);				
	 				if (dField == null) {
	 					defSchema.addField(lField);
	 				} else {
	 					ArrayList<SS3FieldValue> values = lField.getValuesAsList();
	 					for (SS3FieldValue value : values) {
	 						dField.addValue(value);
	 					}
	 					
	 					dField.setColor(lField.getColor());
	 				}
	 			}

	 			SS3CodingSchemeCollection csCollection = SS3Singleton
	 					.getSS3CodingSchemeCollection();
	 			csCollection.loadDefaultCodingScheme(defSchema);

	             SS3Singleton.getSS3DatabaseCollection().addSS3SDatabase(db);
	             db.setSS3CodingScheme(
	                SS3Singleton
	                   .getSS3CodingSchemeCollection()
	                   .getDefaultCodingScheme());
	             db.setReadFromXMLFile(file);

	             DefaultMutableTreeNode[] nodes = new DefaultMutableTreeNode[30];
	             
	             for(int i = 0; i < 30; i++) {
	            	 nodes[i] = null;
	             }
	             
	             String dbNumber = db.getNumber();

	             if(dbNumber == null || dbNumber.trim().length()==0)
	            	 dbNumber = "1"; 
	             
	             int newNodeNumber = Integer.valueOf(dbNumber);
	             
	             nodes[newNodeNumber] = new DefaultMutableTreeNode(db.getExcerptObj()); 
	             
	             for (Enumeration e = node.children(); e.hasMoreElements() ;) {
	                 DefaultMutableTreeNode entNode = (DefaultMutableTreeNode)e.nextElement();
	                 Excerpt excerpt = (Excerpt)entNode.getUserObject();
	                 
	                 SS3Database sDb = SS3Singleton.getSS3DatabaseCollection().getSS3Database(excerpt);
	                 String number = sDb.getNumber();
	                 int n = Integer.valueOf(number);
	                 nodes[n] = entNode; 
	             }
	             
	             node.removeAllChildren();
	             
	             for(int j = 0; j < 30; j++) {
	            	 if(nodes[j] != null)
	            		 node.add(nodes[j]);
	             }
	         } else if (Util.isXMLFile(fileName)) {
                // Execute if a database is in older format, unzipped files.
                // Filename should have mask filename.xml

	            SS3Parser parser = new SS3Parser();
	            SS3Database db = parser.parseSS3Database(file.getPath());

	            Excerpt excerpt = new Excerpt(db.getNumber(), db.getExcerpt());

	            ArrayList<String> mediaFileIDs = new ArrayList<String>();
	            ArrayList<SS3MediaFile> mFiles = db.getMediaFilesCollection();
	            Iterator<SS3MediaFile> iterator = mFiles.iterator();
	            while (iterator.hasNext()) {
	               SS3MediaFile mFile = iterator.next();
	               mediaFileIDs.add(mFile.getId());
	            }

	            excerpt.setMediaFiles(mediaFileIDs);

	            SS3Singleton.getSS3DatabaseCollection().addSS3SDatabase(db);
	            db.setSS3CodingScheme(
	               SS3Singleton
	                  .getSS3CodingSchemeCollection()
	                  .getDefaultCodingScheme());
	            db.setReadFromXMLFile(file);

	            DefaultMutableTreeNode cNode = new DefaultMutableTreeNode(excerpt);
	           
	            node.add(cNode);
	         } else {
	            SS3SignStreamApplication.logger.log(
	               Level.INFO,
	               "Parsing binary file [" + file.getName() + "].");
	            try {
	               SignStreamDocument ssDoc = createSSDocument(file.getAbsolutePath());

	               loadLegacyCodingScheme(fileName, ssDoc.getCodingScheme());

	               SS3CodingScheme scheme =
	                  ssPanel.getSS3Database().getSS3CodingScheme();
	               SS3CodingScheme lScheme =
	                  ssPanel.getSS3Database().getLocalSS3CodingScheme();
	               
	               SS3SignStreamApplication.logger.log(
	                  Level.INFO,
	                  "Legacy Coding Scheme has ["
	                     + lScheme.getFields().size()
	                     + "] fields.");

	               if (ssDoc != null) {
	                  SS2toSS3Adaptor adaptor = new SS2toSS3Adaptor(ssDoc);
	                  ArrayList<Excerpt> excerptList = ssDoc.getExcerpts();

	                  for (int j = 0; j < excerptList.size(); j++) {
	                     Excerpt excerpt = excerptList.get(j);
	                     excerpt.setPath(file.getAbsolutePath());

	                     SS3Database db = adaptor.loadSS2Database(excerpt);
	                     SS3Singleton.getSS3DatabaseCollection().addSS3SDatabase(db);
	                     db.setSS3CodingScheme(scheme);
	                     db.setLocalSS3CodingScheme(lScheme);
	                     db.setExcerpt(excerpt.getExcerpt());

	                     SS3SignStreamApplication.logger.log(
	                        Level.INFO,
	                        "Database ["
	                           + db.getShortDescription()
	                           + "] had been added to the database collection. Updating database references.");

	                     updateDatabaseReferencesToCodingScheme(scheme, db);

	                     SS3SignStreamApplication.logger.log(
	                        Level.INFO,
	                        "Database references had been updated. Creating TreeNode that represents the database.");

	                     DefaultMutableTreeNode cNode =
	                        new DefaultMutableTreeNode(excerpt);

	                     SS3SignStreamApplication.logger.log(
	                        Level.INFO,
	                        "Tree node had been created.");
	                     
	                     node.insert(cNode, j);
	                     
	                     SS3SignStreamApplication.logger.log(
	                        Level.INFO,
	                        "Excerpt [" + excerpt + "] Tree node had been added.");
	                  }
	                  SS3SignStreamApplication.logger.log(
	                     Level.INFO,
	                     "Legacy file had been loaded.");
	               }
	            } catch (IllegalArgumentException ex) {
	               SS3SignStreamApplication.logger.log(
	                  Level.SEVERE,
	                  "File ["
	                     + fileName
	                     + "] is not a valid file name."
	                     + ex.getStackTrace());
	            }
	         }
	      }
	   }

    private SignStreamDocument createSSDocument(String path)
            throws IllegalArgumentException {
        Parser parser;

        if (edu.bu.signstream.common.util.Util.isXMLFile(path)) {
            parser = new XMLParser();
        } else if (
                !edu.bu.signstream.common.util.Util.isMovieFile(path)
                        && !edu.bu.signstream.common.util.Util.isTextFile(path)) {
            parser = new SS2Parser();
        } else {
            throw new IllegalArgumentException();
        }

        SignStreamDocument doc = parser.loadSSDocument(path);

        if (doc != null) {
            CodingSchema codingSchema = new CodingSchema();
            codingSchema.loadCodingSchema(doc.getCodingScheme());
            SS3Singleton.loadCodingSchema(codingSchema);
        }

        return doc;
    }

	   private void updateDatabaseReferencesToCodingScheme(
	      SS3CodingScheme cScheme,
	      SS3Database db) {

	      HashMap fields = cScheme.getFields();
	      Set set = fields.keySet();
	      Iterator iterator = set.iterator();

	      while (iterator.hasNext()) {
	         String key = (String) iterator.next();
	         Object value = fields.get(key);
	         SS3Field field = (SS3Field) value;
	         String fieldLegacyID = field.getLegacyID();

	         // update value references. Do not log.
	         Iterator valuesIterator = field.getValuesAsList().iterator();
	         while (valuesIterator.hasNext()) {
	            SS3FieldValue fValue = (SS3FieldValue) valuesIterator.next();
	            String legacyValueID = fValue.getLegacyID();

	            if (legacyValueID != null && legacyValueID.length() > 0) {

	               SS3Track track = db.getTrack(fieldLegacyID);
	               if (track != null) {
	                  ArrayList entities = track.getEntities();
	                  for (int i = 0; i < entities.size(); i++) {
	                     SS3Entity entity = (SS3Entity) entities.get(i);
	                     ArrayList items = entity.getItemsAsList();

	                     for (int j = 0; j < items.size(); j++) {
	                        SS3Item item = (SS3Item) items.get(j);
	                        if (item.getId().equalsIgnoreCase(legacyValueID)) {
	                           item.setId(fValue.getId());
	                           j = items.size();
	                           i = entities.size();
	                        }
	                     }
	                  }
	               }
	            }
	         }

	         // update field references
	         if (fieldLegacyID != null && fieldLegacyID.length() > 0) {
	            SS3Track track = db.getTrack(fieldLegacyID);
	            if (track != null)
	               track.setId(field.getId());
	         }
	      }
	   }

	   private void loadLegacyCodingScheme(String fileName, HashMap codingScheme) {

	      CodingSchemeUpdateDialog updateDialog = new CodingSchemeUpdateDialog(
                  fileName,
                  codingScheme,
                  ssPanel,
                  ssPanel.getParentFrame());
	      updateDialog.setVisible(true);
	   }

	   private boolean isExcerptLoaded(Excerpt excerpt) {
	      Enumeration e = topNode.children();

	      while (e.hasMoreElements()) {
	         Object obj = e.nextElement();
	         try {
	            DefaultMutableTreeNode ex = (DefaultMutableTreeNode) obj;
	            Object uo = ex.getUserObject();
	            Excerpt ex1 = (Excerpt) uo;

	            if (ex1.getExcerpt().equals(excerpt.getExcerpt()))
	               return true;
	         } catch (Exception ex) {
	            ;
	         }
	      }
	      return false;
	   }

	   public void addExcerpt(Excerpt excerpt) {
	      if (isExcerptLoaded(excerpt))
	         return;

	      DefaultMutableTreeNode cNode = new DefaultMutableTreeNode(excerpt);
	      topNode.add(cNode);
	   }

	   public void adddNewDatabase(File file) {
	      DefaultMutableTreeNode node = new DefaultMutableTreeNode(file.getName());
	      topNode.add(node);
	   }

	   private void addDirectory(DefaultMutableTreeNode node, File file, File zipFile) {
	      File[] files = file.listFiles();

	      for (int i = 0; i < files.length; i++) {
	         File nFile = files[i];
	       
	         if (nFile.isFile()
	            && !edu.bu.signstream.common.util.Util.isMovieFile(nFile.getName())
	            && !edu.bu.signstream.common.util.Util.isTextFile(nFile.getName())) {

	            addFile(node, nFile, zipFile);
	         } else if (
	            nFile.isDirectory()
	               && !nFile.getName().equalsIgnoreCase("movies")) {

	            DefaultMutableTreeNode dirNode =
	               new DefaultMutableTreeNode(nFile.getName());
	            addDirectory(dirNode, nFile, zipFile);
	            node.add(dirNode);
	         }
	      }
	   }

	   public DefaultMutableTreeNode createNode(File file, File zipFile) {
		  
		  String cDirName = file.getName();
			
			if(cDirName.startsWith("tmp_")) {
				cDirName = cDirName.substring(4);				
			}
			
			if(cDirName.endsWith(".zip")) {			
				cDirName = cDirName.substring(0, cDirName.length()-4);
			}
			
			if(cDirName.endsWith(".ss3")) {
				cDirName = cDirName.substring(0, cDirName.length()-4);
			}
		  
	      DefaultMutableTreeNode node = new DefaultMutableTreeNode(cDirName);

	      if (file.isDirectory()) {
	         addDirectory(node, file, zipFile);
	      } else if (file.isFile()) {
	         addFile(node, file, zipFile);
	      }
	      return node;
	   }
	}