package edu.bu.signstream.navigation;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3DatabaseCollection;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class DbNavigationPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JTree tree = new JTree();
	private DatabaseDelegate dbDelegate = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public DbNavigationPanel(SignStreamSegmentPanel segmentPanel,
			DatabaseDelegate dbDelegate) {
		this.segmentPanel = segmentPanel;

		if (dbDelegate == null) {
			this.dbDelegate = new DatabaseDelegate(segmentPanel);
		} else {
			this.dbDelegate = dbDelegate;
		}

		try {
			DefaultMutableTreeNode treeNode = dbDelegate.getMutableTreeNode();
			this.tree = new JTree(treeNode);

			initTreeUI();
			setTree();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void initTreeUI() {
		Color treeColor = new Color(255, 247, 200);

		int fontSize = 12, fontStyle = Font.PLAIN;
		String fontFamily = "Times-Roman";
		Font font = new Font(fontFamily, fontStyle, fontSize);

		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		renderer.setLeafIcon(null);
		renderer.setIcon(null);
		renderer.setClosedIcon(null);
		renderer.setOpenIcon(null);

		renderer.setBackground(treeColor);
		renderer.setBackgroundNonSelectionColor(treeColor);
		renderer.setBackgroundSelectionColor(treeColor);
		renderer.setBorderSelectionColor(treeColor);
		renderer.setTextSelectionColor(Color.RED);
		renderer.setFont(font);
		
		tree.getInputMap().put(KeyStroke.getKeyStroke("LEFT"), "none");
		tree.getInputMap().put(KeyStroke.getKeyStroke("RIGHT"), "none");

		tree.addKeyListener(SS3Singleton.getVideoControlKeyListener());		
		tree.setCellRenderer(renderer);
		tree.setBackground(treeColor);
		tree.setForeground(treeColor);
		tree.putClientProperty("JTree.lineStyle", "Horizontal");

		UIManager.put("Tree.line", Palette.AXIS_COLOR);
	}

	public DbNavigationPanel(TreeModel model) {
		this.tree = new JTree(model);
		this.dbDelegate = new DatabaseDelegate(segmentPanel);
		initTreeUI();

		try {
			setTree();
		} catch (Exception ex) {
			SS3SignStreamApplication.logger.log(Level.INFO, ex.getMessage());
		}
	}

	public void setSignStreamSegmentPanel(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

	public DatabaseDelegate getDatabaseDelegate() {
		return dbDelegate;
	}

	public JTree getJTree() {
		return tree;
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

	public void refresh(TreeNode treeNode) {
		tree = new JTree(treeNode);
		initTreeUI();
		setTree();
		updateUI();
	}

	public void setTree() {
		removeAll();

		JScrollPane hashPane = new JScrollPane(tree);
		hashPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);

		tree.addMouseListener(new TreeMouseListener(this));

		tree.addTreeSelectionListener(new TreeSelectionListener() {
			@Override
			public void valueChanged(TreeSelectionEvent e) {
			}
		});

		tree.expandPath(new TreePath(tree.getModel().getRoot()));

		setLayout(new BorderLayout());

		add(hashPane, BorderLayout.CENTER);
	}
}

class TreeMouseListener implements MouseListener {
	private final DbNavigationPanel panel;

	public TreeMouseListener(DbNavigationPanel panel) {
		this.panel = panel;
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mouseClicked(MouseEvent e) {

		if (e.getClickCount() >= 2) {

			final JTree tree = panel.getJTree();

			DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree
					.getLastSelectedPathComponent();

			SS3SignStreamApplication.logger.log(Level.INFO, "Node [" + node
					+ "] had been selected.");

			SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();

			if (node != null) {
				SegmentGraphReprUtil.utteranceMovieTimeOnset = 0;
				SegmentGraphReprUtil.utteranceMovieTimeOffset = -1;

				segmentPanel.updateLoadedDatabase();

				Object obj = node.getUserObject();

				if (obj instanceof Excerpt) {
					Excerpt excerpt = (Excerpt) obj;
					SS3DatabaseCollection dbCol = SS3Singleton
							.getSS3DatabaseCollection();
					SS3Database db = dbCol.getSS3Database(excerpt);

					panel.getSignStreamSegmentPanel().loadDatabase(db);

					// // calculate coef that will allow users to see entire
					// // duration of a video file on a screen
					int movieDuration = panel.getSignStreamSegmentPanel()
							.getMultipleMovieController().getMovieDuration();
					int packetDuration = (int) panel
							.getSignStreamSegmentPanel()
							.getMultipleMovieController().getPacketDuration();
					
					try {
						MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(null);
						mToolBar.selectOverviewPanel();
					} catch(Exception ex) {
						;
					}

					if (movieDuration == 0) {
						movieDuration = db.getMovieDuration();
						packetDuration = db.getPacketDuration();
					}

					if (movieDuration == 0) {
						SS3GlossWindow glossWindow = db.getSS3GlossWindow();
						ArrayList<SS3TemporalPartition> temporalPartitions = glossWindow
								.getTemporalPartitions();
						for (SS3TemporalPartition temporalPartition : temporalPartitions) {
							SS3GlossWindowSegment segment = temporalPartition
									.getGlossWindowSegment();
							ArrayList<SS3GlossWindowItem> items = segment
									.getSS3GlossWindowItems();
							for (SS3GlossWindowItem item : items) {
								int time = item.getEndTimeInfo().getMovieTime();
								if (time > movieDuration)
									movieDuration = time;
							}
						}
						packetDuration = 1001;
						movieDuration = movieDuration + 20 * packetDuration;
					}

					panel.getSignStreamSegmentPanel().getZoomer()
							.getCoordinateTimeConvertor()
							.setPacketDuration(packetDuration);

					double pixelsInTimeframe = db
							.getTemporalPartitionPixelInTimeFrame();

					int width = panel.getSignStreamSegmentPanel()
							.getDatabaseView().getVisibleWidth2();

					if (pixelsInTimeframe <= 0) {
						panel.getSignStreamSegmentPanel()
								.getZoomer()
								.fillEntirePage(movieDuration, packetDuration,
										width);
					} else {
						panel.getSignStreamSegmentPanel().getZoomer()
								.getCoordinateTimeConvertor()
								.setPixelInTimeFrame(pixelsInTimeframe);
					}

					int movieTime = SS3Singleton.getVideoControlManager()
							.getMultipleMovieController().getCurrentTime();
					panel.getSignStreamSegmentPanel().getSlider()
							.setSliderPositionTime(movieTime);
				}
			}
		}
    }
}

class Perspective {
	// Construct the panel with the toggle buttons.

	public Perspective() {
		JRadioButton showDescendant = new JRadioButton("segmentsnts", true);
		JRadioButton showAncestor = new JRadioButton("utterances");

		ButtonGroup bGroup = new ButtonGroup();
		bGroup.add(showDescendant);
		bGroup.add(showAncestor);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(showDescendant);
		buttonPanel.add(showAncestor);
	}
}
