package edu.bu.signstream.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.ui.panels.newDatabase.VideoPanel;
import edu.bu.signstream.ui.panels.newDatabase.VideoPanelScrolledView;

public class DirectoryLookupScrolledView extends JPanel implements
		ActionListener {

	private static final long serialVersionUID = 1L;

	private ArrayList<SerchableDirectory> directories = new ArrayList<SerchableDirectory>();

	private VideoPanelScrolledView videoPanelView = null;
	private JFrame parentFrame = null;
	private JDialog dialog = null;
	private JScrollPane scroller = null;
	private DirectoryLookupPanel lookupPane = null;

	private JButton cancelBtn = new JButton("Cancel");
	private JButton submitBtn = new JButton("Submit");

	private Dimension d = null;

	private final int CANCEL = 1;
	private final int SUBMIT = 2;

	public DirectoryLookupScrolledView(VideoPanelScrolledView videoPanelView,
			ArrayList<SerchableDirectory> directories, Dimension d,
			JFrame parentFrame, JDialog dialog) {
		super(new BorderLayout());
		this.videoPanelView = videoPanelView;
		this.directories = directories;
		this.d = d;
		this.parentFrame = parentFrame;
		this.dialog = dialog;

		cancelBtn.setActionCommand("" + CANCEL);
		cancelBtn.addActionListener(this);

		submitBtn.setActionCommand("" + SUBMIT);
		submitBtn.addActionListener(this);

		initUI();
	}

	public void initUI() {
		removeAll();

		lookupPane = new DirectoryLookupPanel(directories, parentFrame, this);
		lookupPane.setOpaque(true);
		lookupPane.setBackground(Color.white);

		scroller = new JScrollPane(lookupPane);
		scroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		scroller.setPreferredSize(d);

		scroller.setBorder(BorderFactory.createEmptyBorder(3, 10, 0, 10));

		ButtonPanel bPanel = new ButtonPanel();
		bPanel.add(cancelBtn);
		bPanel.add(submitBtn);

		setLayout(new BorderLayout(5, 5));

		String str1 = "  Choose the directories in which SignStream should look for movie files";
		String str2 = " that have not yet been located (and the priority order for searching).";

		JLabel lbl1 = new JLabel(str1);
		lbl1.setBackground(Color.WHITE);
		UserInterfaceUtil.setLabelLNF(lbl1);

		JLabel lbl2 = new JLabel(str2);
		lbl2.setBackground(Color.WHITE);
		UserInterfaceUtil.setLabelLNF(lbl2);

		JPanel northPanel = new JPanel();
		northPanel.setBackground(Color.white);
		northPanel.setBorder(BorderFactory.createEmptyBorder(5, 1, 1, 1));
		northPanel.setLayout(new GridLayout(0, 1));
		northPanel.add(lbl1);
		northPanel.add(lbl2);

		setBackground(Color.WHITE);

		add(northPanel, BorderLayout.NORTH);
		add(scroller, BorderLayout.CENTER);
		add(bPanel, BorderLayout.SOUTH);

		setPreferredSize(new Dimension(395, 300));
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {

		case CANCEL:
			dialog.dispose();

			break;

		case SUBMIT:
			ArrayList<String> notFound = new ArrayList<String>();
			ArrayList<VideoPanel> videoPanels = videoPanelView.getVideoPanels();

			for (VideoPanel vp : videoPanels) {
				ArrayList<File> selectedFiles = new ArrayList<File>();
				selectedFiles.clear();

				if (vp.getLookupCb().isSelected()) {
					for (SerchableDirectory directory : directories) {
						File file = new File(directory.getDirectory());
						ArrayList<File> found = findAllFiles(file, vp
								.getFileNameLbl().getText());
						if (found != null && found.size() > 0) {
							selectedFiles.addAll(found);
						}
					}
					if (selectedFiles.size() > 0) {
						vp.locateViaLookup(selectedFiles.get(0));
					} else {
						notFound.add(vp.getFileNameLbl().getText());
					}
				}
			}

			if (notFound.size() > 0) {
				StringBuffer bf = new StringBuffer("Warning: ");
				if (notFound.size() == 1) {
					bf.append("File ");
					bf.append(notFound.get(0));
					bf.append(" was not found.");
				} else {
					bf.append("Files ");
					int i = 0;
					for (String notFoundFileName : notFound) {
						bf.append(notFoundFileName);
						if (i < notFound.size() - 1)
							bf.append(", ");
					}
					bf.append(" were not found.");
				}

				SS3Singleton.getSignStreamApplication().confirmationDialog("Video Lookup", bf.toString());
			}
			dialog.dispose();

			break;
		}
	}

	private ArrayList<File> findAllFiles(File directory, final String fileName) {
		if (directory == null ) {
			return null;
		}
		
		if(!directory.isDirectory()) {
			return null;
		}

		ArrayList<File> res = new ArrayList<File>();

		File[] files = directory.listFiles(new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				boolean equals = fileName.trim().equalsIgnoreCase(name.trim());
				return equals;
			}
		});
		if (files != null) {
			res.addAll(Arrays.asList(files));
		}

		if ((files = directory.listFiles()) != null) {
			for (int i = 0; i < files.length; i++) {
				if (files[i].isDirectory()) {
					res.addAll(this.findAllFiles(files[i], fileName));
				}
			}
		}
		return res;
	}
}

class DirectoryLookupPanel extends JPanel implements ActionListener {
	private static final long serialVersionUID = 1L;

	private ArrayList<SerchableDirectory> directories = null;

	private JFrame parentFrame = null;

	private JButton selectBtn = new JButton("Select");
	private JButton removeBtn = new JButton("Remove");
	private MoveToTop moveBtn = new MoveToTop();

	private JLabel label = new JLabel();
	private JCheckBox displayCb = new JCheckBox();

	private JLabel directoryLbl = new JLabel("Directory");
	private JLabel lookupLbl = new JLabel("Lookup");
	private JLabel searchPriorityLbl = new JLabel("Move to Top");
	// private JLabel searchPriority2Lbl = new JLabel("to Top");
	private JLabel directoryNameLbl = new JLabel("Directory Name");

	private DirectoryLookupScrolledView lookupView = null;

	private final int SELECT = 0;

	private String lastTimeOpenedDirectory = "";

	public DirectoryLookupPanel(ArrayList<SerchableDirectory> directories,
			JFrame parentFrame, DirectoryLookupScrolledView lookupView) {
		this.directories = directories;
		this.parentFrame = parentFrame;
		this.lookupView = lookupView;

		for (SerchableDirectory directory : directories) {
			directory.setDirectoryLookupPanel(this);
		}

		UserInterfaceUtil.setLabelLNF(directoryLbl);
		UserInterfaceUtil.setLabelLNF(lookupLbl);
		UserInterfaceUtil.setLabelLNF(searchPriorityLbl);
		// UserInterfaceUtil.setLabelLNF(searchPriority2Lbl);
		UserInterfaceUtil.setLabelLNF(directoryNameLbl);

		moveBtn.setEnabled(false);
		removeBtn.setEnabled(false);
		label.setEnabled(false);
		displayCb.setEnabled(false);

		UserInterfaceUtil.setLabelLNF(label);
		UserInterfaceUtil.setButtonsLNF(selectBtn);
		UserInterfaceUtil.setButtonsLNF(removeBtn);

		selectBtn.setActionCommand("" + SELECT);
		selectBtn.addActionListener(this);

		initUI();
	}

	public void reArrangeDirectories() {
		ArrayList<SerchableDirectory> checkedDir = new ArrayList<SerchableDirectory>();
		ArrayList<SerchableDirectory> unCheckedDir = new ArrayList<SerchableDirectory>();
		for (SerchableDirectory directory : directories) {
			if (directory.getDisplayCb().isSelected()) {
				checkedDir.add(directory);
			} else {
				unCheckedDir.add(directory);
			}
		}

		directories.clear();
		directories.addAll(checkedDir);
		directories.addAll(unCheckedDir);

		initUI();
	}

	public void moveToTop(SerchableDirectory directory) {
		directories.remove(directory);
		directories.add(0, directory);
		initUI();
	}

	public JFrame getParentFrame() {
		return parentFrame;
	}

	public void removeDirectory(SerchableDirectory directory) {
		directories.remove(directory);
		directories.trimToSize();
	}

	public void initUI() {
		removeAll();
		invalidate();
		repaint();
		revalidate();

		JPanel gbPanel = new JPanel(new GridBagLayout());
		gbPanel.setBackground(Color.WHITE);
		gbPanel.setForeground(Color.white);

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(0, 1, 0, 1);
		c.weightx = 0;

		c.anchor = GridBagConstraints.NORTHWEST;

		c.gridy = 0;
		c.gridx = 0;
		c.gridwidth = 2;
		gbPanel.add(directoryLbl, c);
		c.gridx = 2;
		c.gridwidth = 1;
		gbPanel.add(lookupLbl, c);
		c.gridx = 3;
		c.gridwidth = 2;
		gbPanel.add(searchPriorityLbl, c);
		c.gridx = 5;
		c.gridwidth = 1;
		gbPanel.add(directoryNameLbl, c);

		int i = 1;
		for (SerchableDirectory directory : directories) {
			directory.setPriority(i);
			c.anchor = GridBagConstraints.WEST;
			c.gridy = i;
			c.gridx = 0;
			gbPanel.add(directory.getSelectBtn(), c);
			c.gridx = 1;
			gbPanel.add(directory.getRemoveBtn(), c);
			c.gridx = 2;
			gbPanel.add(directory.getDisplayCb(), c);
			c.gridx = 3;
			gbPanel.add(directory.getMoveToTopBtn(), c);
			c.gridx = 4;
			gbPanel.add(directory.getPriorityLbl(), c);
			c.gridx = 5;
			gbPanel.add(directory.getLabel(), c);
			i++;
		}

		c.gridy = directories.size() + 2;
		c.gridx = 0;
		gbPanel.add(selectBtn, c);
		c.gridx = 1;
		gbPanel.add(removeBtn, c);
		c.gridx = 2;
		gbPanel.add(displayCb, c);
		c.gridx = 3;
		gbPanel.add(moveBtn, c);
		c.gridx = 4;
		gbPanel.add(label, c);
		c.gridx = 5;
		gbPanel.add(new JLabel(), c);

		setLayout(new FlowLayout(FlowLayout.LEFT));
		add(gbPanel);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case SELECT:

			File file = new File(lastTimeOpenedDirectory);
			File selectedFile = UserInterfaceUtil.selectDirectory(parentFrame,
					file, this, "Locate Video File");
			if (selectedFile != null && selectedFile.exists()) {
				lastTimeOpenedDirectory = selectedFile.getPath();
				SerchableDirectory directory = new SerchableDirectory(
						selectedFile.getPath(), true);
				directory.setDirectoryLookupPanel(this);
				directories.add(directory);

				reArrangeDirectories();

				initUI();
			}

			break;
		}
	}
}
