package edu.bu.signstream.ui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.HeadlessException;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.filechooser.FileView;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

//import com.apple.eawt.AppEvent.QuitEvent;
//import com.apple.eawt.Application;
//import com.apple.eawt.QuitHandler;
//import com.apple.eawt.QuitResponse;

import edu.bu.signstream.common.db.parser.SS3Parser;
import edu.bu.signstream.common.filesLocators.FilesLocation;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.ImageMaker;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3DatabaseCollection;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.presentation.dialog.RenameSegmentDialog;
import edu.bu.signstream.grepresentation.fields.presentation.dialog.RenameTemporalPartitionDialog;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;
import edu.bu.signstream.navigation.DatabaseDelegate;
import edu.bu.signstream.navigation.DbNavigationPanel;
import edu.bu.signstream.ui.panels.AnnotatorSelectionDialog;
import edu.bu.signstream.ui.panels.newDatabase.CreateUpdateMacroUnit;
import edu.bu.signstream.ui.panels.newDatabase.CreateUpdateMacroUnitDialog;
import edu.bu.signstream.ui.panels.search.SearchDialog;

/**
 * DO not use JOptionPane as it disables MenuItems on Mac.
 * */
public class SS3SignStreamApplication extends JFrame implements ActionListener,
		KeyListener {

	private static final long serialVersionUID = 1L; 

	public static Logger logger = Logger.getLogger("edu.bu.signstream.ui");

	private final int NEW_COLLECTION = 0;
	private final int CLOSE_DB = 3;
	private final int SAVE_DB = 4;
	private final int SAVE_AS_DB = 5;
	private final int OPEN_AS_SS3 = 6;
	private final int NEW_SEARCH = 50;
	private final int SAVED_SEARCH = 51;

	private final int NEW_MACRO_UNIT = 7;
	private final int DELETE_MACRO_UNIT = 8;
	private final int RENAME_MACRO_UNIT = 9;

	private final int CREATE_SEGMENT = 10;
	private final int RENAME_SEGMENT = 11;
	private final int RENAME_PARTICIPANT = 12;
	private final int DELETE_SEGMENT = 13;
	private final int SET_DEFAULT_SEGMENT = 14;
	
	private final int CREATE_TEMPORAL_PARTITION = 20;
	private final int RENAME_TEMPORAL_PARTITION = 21;
	private final int DELETE_TEMPORAL_PARTITION = 22;

	private final int EXIT = 30;

	private SS3Parser parser = null;
	private DatabaseDelegate dbDelegate = null;
	private SignStreamSegmentPanel panel = null;
	private SearchDialog searchDialog = null;
	
	private SignBankCollection signBankCollection = null;

	private boolean isWindowsOS = false;

	public static String path = "newXMLFiles";
	public static String globalParticipantsFileName = "globalParticipants.xml";
	public static String localParticipantsFileName = "localParticipants.xml";
	public static String globalAnnotatorsFileName = "globalAnnotators.xml";
	public static String localAnnotatorsFileName = "localAnnotators.xml";
	public static String mediaLibraryFileName = "mediaLibrary.xml";

	private String scPath = null;

	JMenu fileMenu = new JMenu("Collection");
	JMenuItem newItem = new JMenuItem(" New Blank Collection ");
	JMenuItem openAsItem = new JMenuItem(" Open...");
	JMenuItem openLegacyItem = new JMenuItem(" Open Legacy...");
	JMenuItem closeItem = new JMenuItem(" Close ");
	JMenuItem saveItem = new JMenuItem(" Save ");
	JMenuItem saveAsItem = new JMenuItem(" Save As... ");
	JMenuItem newSearch = new JMenuItem("New Search");
	JMenuItem savedSearch = new JMenuItem("Open Saved Search");

	private CreateUpdateMacroUnitDialog macroUnitDialog = null;
	private SaveAndOpenCollectionDialog saveCollectionANdOpenCollectionDialog = null;

	@Override
	public boolean isFocusable() {
		return true;
	}
	
	/**
	 * Due to a recurrent problem on OSX applications, JPopupMenu objects do not disappear when
	 * the user switches to another application, and remains visible until the user switches back to the
	 * Signstream application and clicks away from the JPopupMenu, which closes it.
	 * This method is meant to be called by any client class in Signstream which uses a JPopupMenu object 
	 * and is experiencing this issue - here it is forced to invisible when the window focus switches.
	 * @param popup The JPopupMenu which requires a forced setVisible(false) call on window focus switch
	 */
	public void forcePopupInvisibleOnWindowLostFocus(final JPopupMenu popup) {
		final JFrame frame = this;	// need a final reference to this SS3SignStreamApplication object for the inner class below
		frame.addWindowFocusListener(new WindowFocusListener() {
		    @Override public void windowGainedFocus(WindowEvent arg0) {}
		    @Override public void windowLostFocus(WindowEvent arg0) {
		        popup.setVisible(false);
		        frame.removeWindowFocusListener(this);
		    }
		});
		
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return panel;
	}

	public DatabaseDelegate getDatabaseDelegate() {
		return dbDelegate;
	}
	
	public CreateUpdateMacroUnitDialog getMacroUnitDialog() {
		return macroUnitDialog;
	}

	public SS3SignStreamApplication() {

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception ex) {
			;
			logger.log(Level.INFO, "Could not set system look and feel", ex);
		}

		System.setProperty("apple.laf.useScreenMenuBar", "true");

		isWindowsOS = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
				.equalsIgnoreCase(UIManager.getSystemLookAndFeelClassName());

		System.setProperty("com.apple.eawt.CocoaComponent.CompatibilityMode",
				"false");

		KeyStroke nextFrameKeyStroke = KeyStroke.getKeyStroke(39, 0);
		KeyStroke prevFrameKeyStroke = KeyStroke.getKeyStroke(37, 0);	

		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				nextFrameKeyStroke, "nextFrame");
		
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				prevFrameKeyStroke, "prevFrame");
	
		getRootPane().getActionMap().put("nextFrame", new AbstractAction() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				SS3Singleton.getMediaToolBar(null).currentFrameForward();			}
		});
		
		getRootPane().getActionMap().put("prevFrame", new AbstractAction() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				SS3Singleton.getMediaToolBar(null).currentFrameBack();
			}
		});

		try {
			FileHandler fh = new FileHandler("SignStreamLog.txt");
			logger.addHandler(fh);
		} catch (Exception ex) {
			ex.printStackTrace();
		}

		logger.log(Level.INFO, "Starting the Sign Stream application... ");

		String dv = System.getProperty("file.separator", "/");
		path = "newXMLfiles" + dv;
		scPath = path + "defCodingScheme.xml";

		this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

		parser = new SS3Parser();

		SS3Database ss3db = new SS3Database(new TimeInfo(), new TimeInfo());
		ss3db.setSS3GlossWindow(createNewGlossWindow());
		Excerpt excerpt = new Excerpt(Util.getUniqueNumber(),
				ss3db.getFileName());
		ss3db.setExcerptObj(excerpt);
		
		// parse sign bank collection on startup as well
		parseSignBank();
		
		MenuItemsCollection menuItemsCollection = new MenuItemsCollection();
		JMenuBar mb = new JMenuBar();
		JMenu fileMenu = createFileMenu(menuItemsCollection);
		mb.add(fileMenu);

		JMenu muMenu = new JMenu("Macro Unit");
		JMenuItem newItem = new JMenuItem(" New Macro Unit ");
		JMenuItem renameItem = new JMenuItem(" Edit Macro Unit ");

		menuItemsCollection.setMacroUnitMenu(fileMenu);
		menuItemsCollection.setNewMacroUnitItem(newItem);
		menuItemsCollection.setEditMacroUnitItem(renameItem);
		muMenu.add(renameItem);

		newItem.setActionCommand("" + NEW_MACRO_UNIT);
		renameItem.setActionCommand("" + RENAME_MACRO_UNIT);

		newItem.addActionListener(this);
		renameItem.addActionListener(this);
		
		muMenu.addSeparator();
		
		JMenu sMenu = createSegmentMenu(menuItemsCollection);
		muMenu.add(sMenu);

		muMenu.addSeparator();
		
		JMenu tpMenu = createTemporalPartitionMenu(menuItemsCollection);
		muMenu.add(tpMenu);
			
		mb.add(muMenu);

		setJMenuBar(mb);

		panel = new SignStreamSegmentPanel(null, this); //SignStreamSegmentPanel(excerpt, ss3db, null, this);
		dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();
		
		loadNewMacroUnitDialog(ss3db);

		logger.log(Level.INFO, "Starting the Sign Stream application... 1");

		SignStreamWindowListener windowListener = new SignStreamWindowListener(this);
		addWindowListener(windowListener);		// handles save/dont-save on clicking 'x' to close window

		logger.log(Level.INFO, "Starting the Sign Stream application... 2");

		logger.log(Level.INFO, "Starting the Sign Stream application... 3");

		SS3Singleton.loadSignStreamApplication(this);

		logger.log(Level.INFO, "Starting the Sign Stream application... 4");

		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(panel, BorderLayout.CENTER);
		
		logger.log(Level.INFO,
				"The Sign Stream application started successfully");
	}

	public SS3CodingScheme getDefaultSS3CodingScheme() {
		return parser.parseSS3CodingScheme(scPath);
	}

	private void loadNewMacroUnitDialog(SS3Database ss3db) {
		Point location = new Point(300, 100);

		if (macroUnitDialog != null) {
			location = macroUnitDialog.getLocation();
			macroUnitDialog.dispose();
		}

		Calendar rightNow = Calendar.getInstance();
		int dayOfMonth = rightNow.get(Calendar.DAY_OF_MONTH);
		int month = rightNow.get(Calendar.MONTH) + 1;
		int year = rightNow.get(Calendar.YEAR);
		int timeHrs = rightNow.get(Calendar.HOUR_OF_DAY);
		int timeMin = rightNow.get(Calendar.MINUTE);
		int timeSec = rightNow.get(Calendar.SECOND);

		String defCollectionCreatedOnLoad = "Collection_" + month + "_"
				+ dayOfMonth + "_" + year + "_" + timeHrs + "_" + timeMin + "_"
				+ timeSec;

		setTitle(defCollectionCreatedOnLoad);

		DbNavigationPanel navigation = panel.getDbNavigationPanel();
		navigation.getDatabaseDelegate().addNewDatabase(
				defCollectionCreatedOnLoad);
		navigation.refresh(navigation.getDatabaseDelegate()
				.getMutableTreeNode());
		navigation.getJTree().repaint();

		String globalParticipantsFileName = path + "globalParticipants.xml";
		String localParticipantsFileName = path + "localParticipants.xml";
		String globalAnnotatorsFileName = path + "globalAnnotators.xml";
		String localAnnotatorsFileName = path + "localAnnotators.xml";
		String mediaLibraryFileName = path + "mediaLibrary.xml";
		
		SS3Singleton.getManager().loadParticipantsLibrary(globalParticipantsFileName, localParticipantsFileName);
		SS3Singleton.getManager().loadAnnotatorsLibrary(globalAnnotatorsFileName, localAnnotatorsFileName);	
		
		ParticipantsLibrary participantsLibrary = SS3Singleton.getManager().getLoadedParticipantsLibrary();
		ParticipantsLibrary annotatorsLibrary = SS3Singleton.getManager().getLoadedAnnotatorsLibrary();

		SS3MediaLibrary mLibrary = parser
				.parseMediaLibrary(mediaLibraryFileName);

		if(macroUnitDialog == null) {
		macroUnitDialog = new CreateUpdateMacroUnitDialog(
				CreateUpdateMacroUnit.FUNCTION_ADD, ss3db,
				ss3db.getExcerptObj(), new ArrayList<LoadedMediaFile>(),
				mLibrary, participantsLibrary, annotatorsLibrary,
				signBankCollection.getSignBankResourceCollection(),
				panel, this);
		} else {
			macroUnitDialog.update(
					CreateUpdateMacroUnit.FUNCTION_ADD, ss3db,
					ss3db.getExcerptObj(), new ArrayList<LoadedMediaFile>(),
					mLibrary, participantsLibrary, annotatorsLibrary,
					signBankCollection.getSignBankResourceCollection());
		}

		macroUnitDialog.setVisible(true);
		macroUnitDialog.setLocation(location);
	}

	@Override
	public void keyTyped(KeyEvent e) {
		displayInfo(e, "KEY TYPED: ");
	}

	/** Handle the key-pressed event from the text field. */
	@Override
	public void keyPressed(KeyEvent e) {
		displayInfo(e, "KEY PRESSED: ");
	}

	/** Handle the key-released event from the text field. */
	@Override
	public void keyReleased(KeyEvent e) {
		displayInfo(e, "KEY RELEASED: ");
	}

	private void displayInfo(KeyEvent e, String keyStatus) {
		int modifiersEx = e.getModifiersEx();
		String modString = "extended modifiers = " + modifiersEx;
		String tmpString = InputEvent.getModifiersExText(modifiersEx);
		if (tmpString.length() > 0) {
			modString += " (" + tmpString + ")";
		} else {
			modString += " (no extended modifiers)";
		}

		String actionString = "action key? ";
		if (e.isActionKey()) {
			actionString += "YES";
		} else {
			actionString += "NO";
		}

		String locationString = "key location: ";
		int location = e.getKeyLocation();
		if (location == KeyEvent.KEY_LOCATION_STANDARD) {
			locationString += "standard";
		} else if (location == KeyEvent.KEY_LOCATION_LEFT) {
			locationString += "left";
		} else if (location == KeyEvent.KEY_LOCATION_RIGHT) {
			locationString += "right";
		} else if (location == KeyEvent.KEY_LOCATION_NUMPAD) {
			locationString += "numpad";
		} else { // (location == KeyEvent.KEY_LOCATION_UNKNOWN)
			locationString += "unknown";
		}
	}

	private JMenu createSegmentMenu(MenuItemsCollection menuItemsCollection) {
		JMenu sMenu = new JMenu("Segment Tier");
		JMenuItem newItem = new JMenuItem(" New Segment Tier");
		JMenuItem renameItem = new JMenuItem(" (Re)name Segment Tier");
		JMenuItem renameParticipantItem = new JMenuItem(" (Re)name Participant");
		JMenuItem deleteItem = new JMenuItem(" Delete Segment Tier");
		JMenuItem setDefaultItem = new JMenuItem(" Set Default Segment Tier");

		menuItemsCollection.setSegmentMenu(sMenu);
		menuItemsCollection.setNewSegmentItem(newItem);
		menuItemsCollection.setRenameSegmentItem(renameItem);
		menuItemsCollection.setRenameParticipantItem(renameParticipantItem);
		menuItemsCollection.setDeleteSegmentItem(deleteItem);
		menuItemsCollection.setSetDefaultSegmentItem(setDefaultItem);

		sMenu.add(newItem);
		sMenu.add(renameItem);
		sMenu.add(renameParticipantItem);
		sMenu.add(deleteItem);
		sMenu.add(setDefaultItem);

		newItem.setActionCommand("" + CREATE_SEGMENT);
		renameItem.setActionCommand("" + RENAME_SEGMENT);
		renameParticipantItem.setActionCommand("" + RENAME_PARTICIPANT);
		deleteItem.setActionCommand("" + DELETE_SEGMENT);
		setDefaultItem.setActionCommand("" + SET_DEFAULT_SEGMENT);

		newItem.addActionListener(this);
		renameItem.addActionListener(this);
		renameParticipantItem.addActionListener(this);
		deleteItem.addActionListener(this);
		setDefaultItem.addActionListener(this);

		return sMenu;
	}
	
	private JMenu createTemporalPartitionMenu(MenuItemsCollection menuItemsCollection) {
		JMenu tpMenu = new JMenu("Temporal Partition");
		JMenuItem newItem = new JMenuItem(" New Temporal Partition");
		JMenuItem renameItem = new JMenuItem(" Rename Temporal Partition");
		JMenuItem deleteItem = new JMenuItem(" Delete Temporal Partition");

		menuItemsCollection.setSegmentMenu(tpMenu);
		menuItemsCollection.setNewSegmentItem(newItem);
		menuItemsCollection.setRenameSegmentItem(renameItem);
		menuItemsCollection.setDeleteSegmentItem(deleteItem);

		tpMenu.add(newItem);
		tpMenu.add(renameItem);
		tpMenu.add(deleteItem);

		newItem.setActionCommand("" + CREATE_TEMPORAL_PARTITION);
		deleteItem.setActionCommand("" + DELETE_TEMPORAL_PARTITION);
		renameItem.setActionCommand("" + RENAME_TEMPORAL_PARTITION);

		newItem.addActionListener(this);
		renameItem.addActionListener(this);
		deleteItem.addActionListener(this);

		return tpMenu;
	}

	private JMenu createFileMenu(MenuItemsCollection menuItemsCollection) {

		menuItemsCollection.setCollectionMenu(fileMenu);
		menuItemsCollection.setNewCollectionItem(newItem);
		menuItemsCollection.setOpenCoolectionItem(openAsItem);
		menuItemsCollection.setOpenLegacyItem(openLegacyItem);
		menuItemsCollection.setCloseCollectionItem(closeItem);
		menuItemsCollection.setSaveCollectionItem(saveItem);
		menuItemsCollection.setSaveAsCollectionItem(saveAsItem);
		menuItemsCollection.setNewSearchCollectionItem(newSearch);
		menuItemsCollection.setSavedSearchCollectionItem(savedSearch);


		fileMenu.add(newItem);
		fileMenu.add(openAsItem);
		fileMenu.addSeparator();
		fileMenu.add(closeItem);
		fileMenu.add(saveItem);

		saveItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S,
                																			java.awt.Event.META_MASK));
		fileMenu.add(saveAsItem);
		fileMenu.addSeparator();
		fileMenu.add(newSearch);
		fileMenu.add(savedSearch);
		savedSearch.setEnabled(false);

		newItem.setActionCommand("" + NEW_COLLECTION);
		newItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N,
																						java.awt.Event.META_MASK));
		
		openAsItem.setActionCommand("" + OPEN_AS_SS3);

        openAsItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O,
                																				java.awt.Event.META_MASK));
		closeItem.setActionCommand("" + CLOSE_DB);
		closeItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_W,
																						java.awt.Event.META_MASK));
		saveItem.setActionCommand("" + SAVE_DB);
		saveAsItem.setActionCommand("" + SAVE_AS_DB);
		newSearch.setActionCommand("" + NEW_SEARCH);
		savedSearch.setActionCommand("" + SAVED_SEARCH);

		newItem.addActionListener(this);
		openAsItem.addActionListener(this);
		closeItem.addActionListener(this);
		saveItem.addActionListener(this);
		saveAsItem.addActionListener(this);
		newSearch.addActionListener(this);
		savedSearch.addActionListener(this);

		return fileMenu;
	}

	private SS3GlossWindow createNewGlossWindow() {
		SS3GlossWindow glsWindow = new SS3GlossWindow();

		SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
		displayTimePeriod.setLabel("Display Time Period");
		glsWindow.setDisplayTimePeriod(displayTimePeriod);

		SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
		temporalPartition.setLabel("Temporal Partition 1");

		SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
		tpSegment.setLabel("Segment Tier 1");
		temporalPartition.addTemporalPartitionSegment(tpSegment);
		temporalPartition.setDefaultSegment(tpSegment);

		ArrayList<SS3TemporalPartition> partitions = new ArrayList<SS3TemporalPartition>();
		partitions.add(temporalPartition);
		glsWindow.setTemporalPartitions(partitions);

		return glsWindow;
	}

	public File openZipDatabase() {

		File selectedFile = null;
		FilesLocation location = SS3Singleton.getFilesLocation();

		if ("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
				.equalsIgnoreCase(UIManager.getSystemLookAndFeelClassName())) {
			JFileChooser fc = new JFileChooser();
			fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES); // .DIRECTORIES_ONLY);
																			// //
																			// .FILES_AND_DIRECTORIES);
			String dbLocation = location.getLastOpenedDatabaseLocation();

			fc.addChoosableFileFilter(new SignStream3FilesFilter());

			File file = new File(dbLocation);
			String absPath = file.getAbsolutePath();

			fc.setSelectedFile(new File(absPath));
			fc.setFileView(new ImageFileView());

			int returnVal = fc.showOpenDialog(this);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				selectedFile = fc.getSelectedFile();

				logger.log(Level.INFO, "Opening [" + selectedFile.getName()
						+ "]");
			} else {
				logger.log(Level.INFO, "Open command cancelled by user.");
			}
		} else {

			FileDialog fd = new FileDialog(this, "Open a Collection",
					FileDialog.LOAD);
			String dbLocation = location.getLastOpenedDatabaseLocation();
			File file = new File(dbLocation);
			String absPath = file.getAbsolutePath();
			fd.setFilenameFilter(new ZipFormatFilenameFilter());
			fd.setDirectory(absPath);
			fd.setFile(absPath);

			fd.setVisible(true);

			System.setProperty("apple.awt.fileDialogForDirectories", "false");

			if (fd.getFile() != null) {
				String selectedFileName = fd.getDirectory() + File.separator
						+ fd.getFile();
				selectedFile = new File(selectedFileName);
			}
		}
		return selectedFile;
	}

	private void saveChangesToTheCollection(final boolean loadNewDatabase) {
		final JTree tree = panel.getDbNavigationPanel().getJTree();

		final DefaultMutableTreeNode rootNode = dbDelegate.getMutableTreeNode();

		if (SS3Singleton.getApplicationStateController()
				.isLoadedCollectionUpdated()) {

			final JDialog newCollectionDialog = new JDialog(this,
					"Save Resource", true);
			JButton dontSaveBtn = new JButton("Don't Save");
			JButton cancelBtn = new JButton("Cancel");
			JButton saveBtn = new JButton("Save");

			dontSaveBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					SS3DatabaseCollection dbCol = SS3Singleton
							.getSS3DatabaseCollection();
					dbCol.removeAll();
					SS3Singleton.getApplicationStateController()
							.setLoadedCollectionUpdated(false);

					panel.closeAllLoadedVideoFiles();

					rootNode.removeAllChildren();
					((DefaultTreeModel) tree.getModel()).reload(rootNode);

					SS3Singleton.getSS3DatabaseCollection().removeAll();

					if (loadNewDatabase)
						panel.loadDatabase(null);

					newCollectionDialog.dispose();
					System.exit(0);
				}
			});

			cancelBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					newCollectionDialog.dispose();
				}

			});

			saveBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					DefaultMutableTreeNode node = null;
					int childCount = rootNode.getChildCount();

					if (childCount > 0) {
						node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

						if (node.getChildCount() > 0) {
							DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
									.getChildAt(0);
							Object uo = dNode.getUserObject();
							if (uo instanceof String) {
								node = dNode;
							}
						}
					}
					
					saveLoadedDatabase(node);
					
					SS3Singleton.getApplicationStateController()
							.setLoadedCollectionUpdated(false);

					panel.closeAllLoadedVideoFiles();

					panel.getMultipleMovieController().getMovieDuration();

					rootNode.removeAllChildren();
					((DefaultTreeModel) tree.getModel()).reload(rootNode);

					SS3Singleton.getSS3DatabaseCollection().removeAll();

					if (loadNewDatabase)
						panel.loadDatabase(null);

					newCollectionDialog.dispose();

					System.exit(0);
				}
			});

			ButtonPanel newCollectionButtonPanel = new ButtonPanel();
			newCollectionButtonPanel.add(dontSaveBtn);
			newCollectionButtonPanel.add(cancelBtn);
			newCollectionButtonPanel.add(saveBtn);

			JPanel newCollectionPanel = new JPanel();
			newCollectionPanel.setLayout(new BorderLayout());
			JLabel newCollectionLabel = new JLabel(
					" Do you want to save the changes you made to Collection?");
			newCollectionPanel.add(newCollectionLabel, BorderLayout.CENTER);
			newCollectionPanel
					.add(newCollectionButtonPanel, BorderLayout.SOUTH);

			newCollectionPanel.add(new JLabel("  "), BorderLayout.WEST);

			Dimension d = newCollectionPanel.getPreferredSize();
			Dimension nD = new Dimension(d.width + 10, d.height + 40);
			newCollectionPanel.setPreferredSize(nD);

			Container contentPane = newCollectionDialog.getContentPane();
			contentPane.setLayout(new BorderLayout(40, 40));
			contentPane.add(newCollectionPanel, BorderLayout.CENTER);

			newCollectionDialog.setLocation(new Point(150, 150));
			newCollectionDialog.pack();
			newCollectionDialog.setVisible(true);
		}
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		panel.updateLoadedDatabase();

		String str = ae.getActionCommand();

		final JTree tree = panel.getDbNavigationPanel().getJTree();
		dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();
		final DefaultMutableTreeNode rootNode = dbDelegate.getMutableTreeNode();
		int childCount = rootNode.getChildCount();
		DefaultMutableTreeNode node = null;

		if (childCount > 0) {
			node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

			if (node.getChildCount() > 0) {
				DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
						.getChildAt(0);
				Object uo = dNode.getUserObject();
				if (uo instanceof String) {
					node = dNode;
				}
			}
		}

		switch (Integer.parseInt(str)) {

		case OPEN_AS_SS3:

			logger.log(Level.INFO, "Opening a collection...");

			if (macroUnitDialog != null)
				macroUnitDialog.setVisible(false);

			if (!SS3Singleton.getApplicationStateController().isLoadedCollectionUpdated()) {				
				openDatabaseSS3(tree, rootNode, node);
			} else {
				if (saveCollectionANdOpenCollectionDialog == null) {
					saveCollectionANdOpenCollectionDialog = new SaveAndOpenCollectionDialog(this);
				}
				saveCollectionANdOpenCollectionDialog.setVisible(true);
			}

			break;

		case NEW_COLLECTION:
			panel.closeAllLoadedVideoFiles();
			rootNode.removeAllChildren();
			((DefaultTreeModel) tree.getModel()).reload(rootNode);
			SS3Singleton.getSS3DatabaseCollection().removeAll();

			panel.loadDatabase(null);
			
			macroUnitDialog = null;
			
			// below taken from constructor - mimicking functionality
			parser = new SS3Parser();
			
			SS3Database ss3db = new SS3Database(new TimeInfo(), new TimeInfo());
			ss3db.setSS3GlossWindow(createNewGlossWindow());
			Excerpt excerpt = new Excerpt(Util.getUniqueNumber(),
					ss3db.getFileName());
			ss3db.setExcerptObj(excerpt);
			
			panel.loadDatabase(ss3db);
			
			// parse sign bank collection on startup as well
			parseSignBank();

			loadNewMacroUnitDialog(ss3db);

			break;
		case CLOSE_DB:

			if (node == null) {
				confirmationDialog("Save Resource", "Collection not available.");
				return;
			}

			final DefaultMutableTreeNode dNode = node;

			if (SS3Singleton.getApplicationStateController()
					.isLoadedCollectionUpdated()) {
				final JDialog newCollectionDialog = new JDialog(this,
						"Save Resource", true);
				final JButton dontSaveBtn = new JButton("Don't Save");
				final JButton cancelBtn = new JButton("Cancel");
				final JButton saveBtn = new JButton("Save");

				dontSaveBtn.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent arg0) {
						panel.closeAllLoadedVideoFiles();
						rootNode.removeAllChildren();
						((DefaultTreeModel) tree.getModel()).reload(rootNode);
						SS3Singleton.getSS3DatabaseCollection().removeAll();
						setTitle("");
						panel.loadDatabase(null);
						newCollectionDialog.dispose();
						dontSaveBtn.removeActionListener(this);
					}
				});

				cancelBtn.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent arg0) {
						newCollectionDialog.dispose();
						cancelBtn.removeActionListener(this);
					}
				});

				saveBtn.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent arg0) {
						String name = (String) dNode.getUserObject();
						File dirToDel = new File(name);

						if (dirToDel.exists())
							dirToDel.delete();

						Enumeration enum4 = dNode.children();

						boolean saved = true;

						String directory = null;

						while (enum4.hasMoreElements()) {
							DefaultMutableTreeNode node1 = (DefaultMutableTreeNode) enum4
									.nextElement();
							Excerpt ex = (Excerpt) node1.getUserObject();
							SS3DatabaseCollection dbCol = SS3Singleton
									.getSS3DatabaseCollection();
							SS3Database db = dbCol
									.getSS3Database(ex);

							if (!ex.getExcerpt().equals(db.getExcerpt())) {
								// delete old file
								db.deleteFile();
								db.setExcerpt(ex.getExcerpt());
							}

							db.setNumber(ex.getID());
							db.setExcerpt(ex.getExcerpt());
							db.setDirectory(name);
							db.setFileName(ex.getExcerpt() + ".xml");

							directory = db.getParentDirectoryPath()
									+ File.separator + db.getDirectory();

							if (!saveDatabase(db)) {
								saved = false;
							}
						}

						boolean ziped = zipDirectory(directory);

						if (!ziped) {
							removeDirectory(directory);
						}

						if (saved && ziped) {
							confirmationDialog("Saved Resource", "Collection "
									+ name + " has been saved.");
							return;
						}

						panel.closeAllLoadedVideoFiles();
						rootNode.removeAllChildren();
						((DefaultTreeModel) tree.getModel()).reload(rootNode);
						SS3Singleton.getSS3DatabaseCollection().removeAll();

						panel.loadDatabase(null);
						newCollectionDialog.dispose();
						saveBtn.removeActionListener(this);
					}
				});

				ButtonPanel newCollectionButtonPanel = new ButtonPanel();
				newCollectionButtonPanel.add(dontSaveBtn);
				newCollectionButtonPanel.add(cancelBtn);
				newCollectionButtonPanel.add(saveBtn);

				JPanel newCollectionPanel = new JPanel();
				newCollectionPanel.setLayout(new BorderLayout());
				JLabel newCollectionLabel = new JLabel(
						" Do you want to save the changes you made to Collection?");
				newCollectionPanel.add(newCollectionLabel, BorderLayout.CENTER);
				newCollectionPanel.add(newCollectionButtonPanel,
						BorderLayout.SOUTH);

				newCollectionPanel.add(new JLabel("  "), BorderLayout.WEST);

				Dimension d = newCollectionPanel.getPreferredSize();
				Dimension nD = new Dimension(d.width + 10, d.height + 40);
				newCollectionPanel.setPreferredSize(nD);

				Container contentPane = newCollectionDialog.getContentPane();
				contentPane.setLayout(new BorderLayout(40, 40));
				contentPane.add(newCollectionPanel, BorderLayout.CENTER);

				newCollectionDialog.setLocation(new Point(150, 150));
				newCollectionDialog.pack();
				newCollectionDialog.setVisible(true);
				//
			} else {
				panel.closeAllLoadedVideoFiles();
				rootNode.removeAllChildren();
				((DefaultTreeModel) tree.getModel()).reload(rootNode);
				SS3Singleton.getSS3DatabaseCollection().removeAll();

				panel.loadDatabase(null);
			}
			break;

		case SAVE_DB:
			if (node == null) {
				confirmationDialog("Save Resource", "Collection is not available.");
				return;
			}

			saveLoadedDatabase(node);
			break;

		case SAVE_AS_DB:
			if (node == null) {
				confirmationDialog("Save Resource", "Collection not available.");
				return;
			}

			FileDialog fd = new FileDialog(this, "Save Collection As...",
					FileDialog.SAVE);
			fd.setVisible(true);
			String directoryName = fd.getFile();
			String parentDirectoryName = fd.getDirectory();

			if (directoryName == null) {
				return;
			}

			panel.updateLoadedDatabase();

			String name1 = (String) node.getUserObject();
			File dirToDel1 = new File(name1);

			if (dirToDel1.exists())
				dirToDel1.delete();

			Enumeration enum41 = node.children();

			boolean saved1 = true;

			while (enum41.hasMoreElements()) {
				DefaultMutableTreeNode node11 = (DefaultMutableTreeNode) enum41
						.nextElement();

				Excerpt ex1 = (Excerpt) node11.getUserObject();
				SS3DatabaseCollection dbCol1 = SS3Singleton
						.getSS3DatabaseCollection();
				SS3Database db1 = dbCol1.getSS3Database(ex1);

				if (!ex1.getExcerpt().equals(db1.getExcerpt())) {
					// delete old file
					db1.deleteFile();
					db1.setExcerpt(ex1.getExcerpt());
				}

				db1.setNumber(ex1.getID());
				db1.setExcerpt(ex1.getExcerpt());
				db1.setParentDirectoryPath(parentDirectoryName);
				db1.setDirectory(directoryName);
				db1.setFileName(ex1.getExcerpt() + ".xml");

				if (!saveDatabase(db1)) {
					saved1 = false;
				}
			}

			String dirName = parentDirectoryName + directoryName;
			boolean ziped1 = zipDirectory(dirName);

			if (!ziped1) {
				removeDirectory(dirName);
			}

			if (saved1 && ziped1) {
				node.setUserObject(directoryName);
				tree.repaint();
				String title = directoryName;
				setTitle(title);
	
			}
			
			break;
			
		case NEW_SEARCH:
			if (searchDialog == null || searchDialog.getSearchPanel() == null) {
				searchDialog = new SearchDialog(this);
				searchDialog.setVisible(true);
			}
			break;

		case CREATE_SEGMENT:
			logger.log(Level.INFO, "Create Segment");	
			if (node == null) {
				confirmationDialog("Create Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				DefaultMutableTreeNode labelViewSelectedNode 
					= panel.getDatabaseView().getLabelView().getSelectedNode();
				DefaultMutableTreeNode prevSiblingNode
					= labelViewSelectedNode.getPreviousSibling();
				int level = labelViewSelectedNode.getLevel();
				
				if (prevSiblingNode == null && level == 1) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SELECT_DISPLAY_TIME_PERIOD);
					return;
				}
				else {					
					// select participant screen	
					AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
							SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedParticipants(), 
							macroUnitDialog.getMacroUnitPanel().getParticipantsDialog());
					annotatorSelectionDialog.setLocation(250, 250);
					annotatorSelectionDialog.setVisible(true);
					
					// get last selected item in list of participants before confirmation, then pass it along to createSegment()
					SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
					
					panel.createSegment(participant);					
				}
			}
			break;

		case RENAME_PARTICIPANT:
			logger.log(Level.INFO, "Rename Participant");	
			if (node == null) {
				confirmationDialog("Rename Participant Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
					return;
				}
				else {		
					// select participant screen	
					AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
							SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedParticipants(), 
							macroUnitDialog.getMacroUnitPanel().getParticipantsDialog());
					annotatorSelectionDialog.setLocation(250, 250);
					annotatorSelectionDialog.setVisible(true);
					
					// get last selected item in list of participants before confirmation, then pass it along to createSegment()
					SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
					
					panel.renameParticipant(participant);
				}
			}
			break;
		
		case RENAME_SEGMENT:
			logger.log(Level.INFO, "Rename Segment");
			
			if (node == null) {
				confirmationDialog("Rename Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
				}
				else {
					RenameSegmentDialog renameDialog = new RenameSegmentDialog(panel);
					renameDialog.setLocation(150, 150);
					renameDialog.setVisible(true);
					
					if (renameDialog.getText() == null || renameDialog.getText().isEmpty())
						break;
					
					String txt = renameDialog.getText();
					
					panel.renameSegment(txt);				
				}
			}

			break;

		case DELETE_SEGMENT:
			logger.log(Level.INFO, "Delete Segment");	
			if (node == null) {
				confirmationDialog("Delete Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
				}
				else {
					DefaultMutableTreeNode parentOfSelectedNode = 
							(DefaultMutableTreeNode) panel.getDatabaseView().getLabelView().getSelectedNode().getParent();
				
					// there is only one segment tier child of this temporal partition - can't delete it
					if(parentOfSelectedNode.getChildCount() == 1)	{
						confirmationDialog("Delete Segment Tier",
								"You cannot delete the last Segment Tier of any given Temporal Partition");
					} 
					else {
						LabelObject selectedLabelObject = (LabelObject) panel.getDatabaseView().getLabelView().getSelectedNode().getUserObject();
						String label = selectedLabelObject.getLabel();
						
						String warningDialogText =
								"All annotated data for segment \""+ label + "\" will be DELETED.\n\n"
								+ "Are you absolutely sure you want to delete segment \"" + label + "\"?";
						String warningTitleText = "WARNING";
						int optionType = JOptionPane.YES_NO_OPTION;
						int messageType = JOptionPane.ERROR_MESSAGE;
						String options[] = {"Proceed", "Cancel"};
						
						int result = -1;
						try {
							result = JOptionPane.showOptionDialog(this, 
									warningDialogText, 
									warningTitleText, 
									optionType, 
									messageType, 
									null, 
									options, 
									options[1]);
						} catch (HeadlessException he) {
							logger.log(Level.INFO, he.getMessage());
						}
						
						if (result == JOptionPane.OK_OPTION)
							panel.deleteSegment();
					}					
				}
			}
			break;
			
		case SET_DEFAULT_SEGMENT:
			logger.log(Level.INFO, "Set Default Segment");
			if (node == null) {
				confirmationDialog("Set Default Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
				}
				else {
					panel.setDefaultSegment();
				}
			}
			
			break;

		case CREATE_TEMPORAL_PARTITION:
			logger.log(Level.INFO, "Create Temporal Partition");	
			if (node == null) {
				confirmationDialog("Create Temporal Partition Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				DefaultMutableTreeNode labelViewSelectedNode 
					= panel.getDatabaseView().getLabelView().getSelectedNode();
				DefaultMutableTreeNode prevSiblingNode
					= labelViewSelectedNode.getPreviousSibling();
				int level = labelViewSelectedNode.getLevel();
				
				if (prevSiblingNode == null && level == 1) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SELECT_DISPLAY_TIME_PERIOD);
					return;
				}
				else {
					// select participant screen	
					AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
							SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedParticipants(), 
							macroUnitDialog.getMacroUnitPanel().getParticipantsDialog());
					annotatorSelectionDialog.setLocation(250, 250);
					annotatorSelectionDialog.setVisible(true);
					
					// get last selected item in list of participants before confirmation, then pass it along to createSegment()
					SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
					
					panel.createTemporalPartition(participant);					
				}
			}
			break;
		case RENAME_TEMPORAL_PARTITION:
			logger.log(Level.INFO, "Rename Temporal Partition");
			
			if (node == null) {
				confirmationDialog("Rename Temporal Partition Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 1 && level !=2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_TEMPORAL_PARTITION);
				}
				else {
					DefaultMutableTreeNode labelViewSelectedNode = panel.getDatabaseView().getLabelView().getSelectedNode();
					DefaultMutableTreeNode prevSiblingNode = null;
					if(labelViewSelectedNode.getLevel() == 1)
						prevSiblingNode = labelViewSelectedNode.getPreviousSibling();
					else{
						DefaultMutableTreeNode parent = (DefaultMutableTreeNode)labelViewSelectedNode.getParent();
						prevSiblingNode = parent.getPreviousSibling();
					}
					
					if( prevSiblingNode == null )
					{
						confirmationDialog("Rename Temporal Partiton",
								"You cannot rename the Display Time Period");
					}
					else
					{
						RenameTemporalPartitionDialog renameDialog = new RenameTemporalPartitionDialog(panel);
						renameDialog.setLocation(150, 150);
						renameDialog.setVisible(true);
						
						
						if (renameDialog.getText() == null || renameDialog.getText().isEmpty())
							break;
						
						if(panel.getSS3Database().getSS3GlossWindow().getTemporalPartitionByLabel(renameDialog.getText()) != null)
						{
							confirmationDialog("Rename Temporal Partiton",
									"The name '" + renameDialog.getText() + "' has already been used.  Please choose a different name");
						}
						else{
							panel.renameTemporalPartition(renameDialog.getText());
						}
					}
				}
			}

			break;
		case DELETE_TEMPORAL_PARTITION:
			logger.log(Level.INFO, "Delete Temporal Partiton");	
			if (node == null) {
				confirmationDialog("Delete Temporal Partiton",
						"Please select a Macro Unit and try again");
			} else {
				
				DefaultMutableTreeNode labelViewSelectedNode = panel.getDatabaseView().getLabelView().getSelectedNode();
				DefaultMutableTreeNode prevSiblingNode = null;
				if(labelViewSelectedNode.getLevel() == 1)
					prevSiblingNode = labelViewSelectedNode.getPreviousSibling();
				else{
					DefaultMutableTreeNode parent = (DefaultMutableTreeNode)labelViewSelectedNode.getParent();
					prevSiblingNode = parent.getPreviousSibling();
				}

			
				DefaultMutableTreeNode databaseViewRootNode = (DefaultMutableTreeNode)panel.getDatabaseView().getLabelView().getSelectedNode().getRoot();
				if(databaseViewRootNode.getChildCount() <= 2 || prevSiblingNode == null)
				{
					confirmationDialog("Delete Temporal Partiton",
							"You cannot delete the last Temporatoral Partition or the Display Time Period");
				}
				else
				{	
					int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
					if (level != 1 && level !=2) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_TEMPORAL_PARTITION);
					}
					else {
						LabelObject selectedLabelObject = null;
						String label = "";
						
						if(level == 2)
						{
							DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) panel.getDatabaseView().getLabelView().getSelectedNode().getParent();
							selectedLabelObject = (LabelObject)parentNode.getUserObject();
							
							label = selectedLabelObject.getLabel();
						}
						else
						{
							selectedLabelObject = (LabelObject) panel.getDatabaseView().getLabelView().getSelectedNode().getUserObject();
							label = selectedLabelObject.getLabel();
						}
						
						String warningDialogText =
								"All annotated data for Temporal Partition  \""+ label + "\" will be DELETED.\n\n"
								+ "Are you absolutely sure you want to delete Temporal Partition \"" + label + "\"?";
						String warningTitleText = "WARNING";
						int optionType = JOptionPane.YES_NO_OPTION;
						int messageType = JOptionPane.ERROR_MESSAGE;
						String options[] = {"Proceed", "Cancel"};
						
						int result = -1;
						try {
							result = JOptionPane.showOptionDialog(this, 
									warningDialogText, 
									warningTitleText, 
									optionType, 
									messageType, 
									null, 
									options, 
									options[1]);
						} catch (HeadlessException he) {
							logger.log(Level.INFO, he.getMessage());
						}
						
						if (result == JOptionPane.OK_OPTION)
							panel.deleteTemporalPartition();
					}
				}
			}
			break;
			
		case EXIT:
			exit();
			break;

		case NEW_MACRO_UNIT:
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.FUNCTIONALITY_NOT_YET_IMPLEMENTED);
			break;

		case RENAME_MACRO_UNIT:

			Object sObject1 = tree.getLastSelectedPathComponent();

			if (sObject1 == null) {
				confirmationDialog("Rename Macro Unit Warning",
						"Please select a Macro Unit to rename.");
				return;
			}

			DefaultMutableTreeNode node6 = (DefaultMutableTreeNode) sObject1;

			if (!node6.isLeaf()) {
				confirmationDialog("Rename Macro Unit Warning",
						"Please select a Macro Unit to rename.");
				return;
			}

			Object robj = node6.getUserObject();
			Excerpt excerpt1 = (Excerpt) robj;

			SS3DatabaseCollection dbCol123 = SS3Singleton
					.getSS3DatabaseCollection();
			SS3Database db = dbCol123.getSS3Database(excerpt1);
			db.setExcerpt(excerpt1.getExcerpt());

			ArrayList<LoadedMediaFile> lmFiles1 = new ArrayList<LoadedMediaFile>();
			HashMap<String, SS3MediaFile> mFiles1 = db.getMediaFiles();
			Collection<SS3MediaFile> files = mFiles1.values();

			for (SS3MediaFile mFile : files) {
				LoadedMediaFile mf = new LoadedMediaFile();
				mf.setMediaFile(mFile);
				mf.setDisplay(true);
				lmFiles1.add(mf);
			}

			String globalParticipantsFileName1 = path
					+ "globalParticipants.xml";
			String localParticipantsFileName1 = path + "localParticipants.xml";
			String globalAnnotatorsFileName1 = path + "globalAnnotators.xml";
			String localAnnotatorsFileName1 = path + "localAnnotators.xml";
			String mediaLibraryFileName1 = path + "mediaLibrary.xml";
			
			ParticipantsLibrary participantsLibrary1 = parser
					.parseParticipantsLibrary(globalParticipantsFileName1,
							localParticipantsFileName1);
			ParticipantsLibrary annotatorsLibrary1 = parser
					.parseAnnotatorsLibrary(globalAnnotatorsFileName1,
							localAnnotatorsFileName1);
			SS3MediaLibrary mLibrary1 = parser
					.parseMediaLibrary(mediaLibraryFileName1);
			
			if(macroUnitDialog == null) {
			macroUnitDialog = new CreateUpdateMacroUnitDialog(
					CreateUpdateMacroUnit.FUNCTION_UPDATE, db, excerpt1,
					lmFiles1, mLibrary1, participantsLibrary1,
					annotatorsLibrary1,
					signBankCollection.getSignBankResourceCollection(),
					panel, this);
			} else {
				macroUnitDialog = new CreateUpdateMacroUnitDialog(
						CreateUpdateMacroUnit.FUNCTION_UPDATE, db, excerpt1,
						lmFiles1, mLibrary1, participantsLibrary1,
						annotatorsLibrary1,
						signBankCollection.getSignBankResourceCollection(),
						panel, this);
			}

			macroUnitDialog.setVisible(true);
			break;

		case DELETE_MACRO_UNIT:
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.FUNCTIONALITY_NOT_YET_IMPLEMENTED);
			break;
		}

		fileMenu.setEnabled(true);
		newItem.setEnabled(true);
		openAsItem.setEnabled(true);
		openLegacyItem.setEnabled(true);

		closeItem.setEnabled(true);
		saveItem.setEnabled(true);
		saveAsItem.setEnabled(true);

		repaint();

		fileMenu.setEnabled(true);
		newItem.setEnabled(true);
		openAsItem.setEnabled(true);
		openLegacyItem.setEnabled(true);

		closeItem.setEnabled(true);
		saveItem.setEnabled(true);
		saveAsItem.setEnabled(true);
	}
	
	/**
	 * ((This is a package level method for now, with the intention of only being used by SS3SignStreamApplication and contained classes.))
	 * This code was originally called in the OPEN_AS_SS3 block of the local actionPerformed().
	 * Similar code was found in the SAVE and DONT_SAVE blocks of the SaveAndOpenCollectionDialog listener,
	 * so this method is standardized between them all for easy readability. 
	 * @param tree JTree from the db navigational panel
	 * @param rootNode Root node from the db navigational panel
	 * @param node Sub node from the db navigational panel, under the root node
	 */
	void openDatabaseSS3(JTree tree, DefaultMutableTreeNode rootNode, DefaultMutableTreeNode node) {
		File file = openZipDatabase();
		
		if(file != null) {
			File unzippedFiles = unzipDirectory(file);
			if (unzippedFiles != null) {
				
				panel.closeAllLoadedVideoFiles();
				rootNode.removeAllChildren();
				
				((DefaultTreeModel) tree.getModel()).reload(rootNode);
				
				panel.loadDatabase(null);
				SS3Singleton.getSS3DatabaseCollection().removeAll();
				
				try {
					dbDelegate.openDb(unzippedFiles, file);
					SS3Singleton.getFilesLocation()
							.setLastOpenedDatabaseLocation(file.getPath());
					panel.getDbNavigationPanel().refresh(
							dbDelegate.getMutableTreeNode());

					if (rootNode.getChildCount() > 0) {
						DefaultMutableTreeNode node1 = (DefaultMutableTreeNode) rootNode.getChildAt(0);
                        DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) node1.getFirstChild();

                        panel.getDbNavigationPanel().getJTree().requestFocus();

                        panel.getDbNavigationPanel().getJTree().expandRow(0);
                        panel.getDbNavigationPanel().getJTree().expandRow(1);
                        panel.getDbNavigationPanel().getJTree().setSelectionRow(2);

                        SS3SignStreamApplication.logger.log(Level.INFO, "Node [" + node
                                + "] had been selected.");

                        SignStreamSegmentPanel segmentPanel = SS3Singleton
                                .getSignStreamApplication().getSignStreamSegmentPanel();

                        if (selectedNode != null) {
                            SegmentGraphReprUtil.utteranceMovieTimeOnset = 0;
                            SegmentGraphReprUtil.utteranceMovieTimeOffset = -1;

                            Object obj = selectedNode.getUserObject();

                            if (obj instanceof Excerpt) {
                                Excerpt excerpt = (Excerpt) obj;
                                SS3DatabaseCollection dbCol = SS3Singleton
                                        .getSS3DatabaseCollection();
                                SS3Database db = dbCol.getSS3Database(excerpt);

                                panel.loadDatabase(db);

                                // // calculate coef that will allow users to see entire
                                // // duration of a video file on a screen
                                int movieDuration = panel.getMultipleMovieController().getMovieDuration();
                                int packetDuration = (int) panel
                                        .getMultipleMovieController().getPacketDuration();

                                try {
                                    MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(null);
                                    mToolBar.selectOverviewPanel();
                                } catch(Exception ex) {
                                    ;
                                }

                                if (movieDuration == 0) {
                                    movieDuration = db.getMovieDuration();
                                    packetDuration = db.getPacketDuration();
                                }

                                if (movieDuration == 0) {
                                    SS3GlossWindow glossWindow = db.getSS3GlossWindow();
                                    ArrayList<SS3TemporalPartition> temporalPartitions = glossWindow
                                            .getTemporalPartitions();
                                    for (SS3TemporalPartition temporalPartition : temporalPartitions) {
                                        SS3GlossWindowSegment segment = temporalPartition
                                                .getGlossWindowSegment();
                                        ArrayList<SS3GlossWindowItem> items = segment
                                                .getSS3GlossWindowItems();
                                        for (SS3GlossWindowItem item : items) {
                                            int time = item.getEndTimeInfo().getMovieTime();
                                            if (time > movieDuration)
                                                movieDuration = time;
                                        }
                                    }
                                    packetDuration = 1001;
                                    movieDuration = movieDuration + 20 * packetDuration;
                                }

                                panel.getZoomer().getCoordinateTimeConvertor().setPacketDuration(packetDuration);

                                double pixelsInTimeframe = db
                                        .getTemporalPartitionPixelInTimeFrame();

                                int width = panel.getDatabaseView().getVisibleWidth2();

                                if (pixelsInTimeframe <= 0) {
                                    panel.getZoomer()
                                            .fillEntirePage(movieDuration, packetDuration,
                                                    width);
                                } else {
                                    panel.getZoomer()
                                            .getCoordinateTimeConvertor()
                                            .setPixelInTimeFrame(pixelsInTimeframe);
                                }

                                int movieTime = SS3Singleton.getVideoControlManager()
                                        .getMultipleMovieController().getCurrentTime();
                                panel.getZoomer().fillEntirePage(movieDuration, packetDuration, width);
                                panel.getSlider().setSliderPositionTime(movieTime);
                            }

						Object uo = node1.getUserObject();
						if (uo instanceof String) {
							setTitle((String) uo);
						}
					}
                    }
					SS3Singleton.getApplicationStateController()
							.setLoadedCollectionUpdated(false);

					logger.log(Level.INFO, "Database [" + file.getName()
							+ "] had been loaded.");
					
					System.out.println(unzippedFiles.getPath());
					
					removeDirectory(unzippedFiles.getPath());	// remove the unzipped "tmp_<file>.ss3" directory
					
					// select annotator screen	
					AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
							SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedAnnotators(), 
							macroUnitDialog.getMacroUnitPanel().getAnnotatorsDialog());
					annotatorSelectionDialog.setLocation(250, 250);
					annotatorSelectionDialog.setVisible(true);
					
				} catch (Exception ex) {
					logger.log(Level.SEVERE, "Could not open collection ["
							+ file.getName() + "].", ex);
				}
			} else {
				logger.log(Level.INFO, "Wrong file format.");
				
				String txt = "<HTML>SignStream is unable to open this file.  Either it was not " + "<BR>" +
						"created by SIgnStream or it has been corrupted.</HTML>";
				
				confirmationDialog("File format", txt);
			}
		}
	}

	public SS3Database openDatabaseSS3() {

		File file = openZipDatabase();
		File unzippedFiles = unzipDirectory(file);

		if (unzippedFiles != null) {
			try {
				dbDelegate.openDb(unzippedFiles, file);

				// set correct path!!!
				SS3Singleton.getFilesLocation().setLastOpenedDatabaseLocation(
						file.getPath());

				logger.log(Level.INFO, "Database [" + file.getName()
						+ "] had been loaded.");
				
				// find db which was created in dbDelegate.openDB
				for (SS3Database db: SS3Singleton.getSS3DatabaseCollection().getDatabases().values()) {
					String str1 = db.getReadFromXMLFile().toString();
					int index = str1.indexOf("/");
					String str2 = str1.substring(0, index);
					
					if (unzippedFiles.getPath().equals(str2)) {
						return db;
					}
				}
				
			} catch (Exception ex) {
				logger.log(Level.SEVERE,
						"Could not open collection [" + file.getName() + "].",
						ex);
			}
		}
		
		return null;
	}

	public void saveLoadedDatabase(DefaultMutableTreeNode node) {
		
		if(panel.isUtteranceLoaded()) {
			panel.updateLoadedDatabase();
		}
		
		String name = (String) node.getUserObject();
		File dirToDel = new File(name);

		if (dirToDel.exists())
			dirToDel.delete();

		Enumeration enum4 = node.children();

		boolean saved = true;

		String directory = null;

		while (enum4.hasMoreElements()) {
			DefaultMutableTreeNode node1 = (DefaultMutableTreeNode) enum4
					.nextElement();
			Excerpt ex = (Excerpt) node1.getUserObject();
			SS3DatabaseCollection dbCol = SS3Singleton
					.getSS3DatabaseCollection();
			SS3Database db = dbCol.getSS3Database(ex);

			if (!ex.getExcerpt().equals(db.getExcerpt())) {
				// delete old file
				db.deleteFile();
				db.setExcerpt(ex.getExcerpt());
			}

			db.setNumber(ex.getID());
			db.setExcerpt(ex.getExcerpt());
			db.setDirectory(name);
			db.setFileName(ex.getExcerpt() + ".xml");

			directory = db.getParentDirectoryPath() + File.separator
					+ db.getDirectory();

			saved = saveDatabase(db);
			
			if(!saved) {
				return;
			}
		}
		
		boolean ziped = zipDirectory(directory);
		if (ziped) {
			removeDirectory(directory);
		} else {
			return;
		}

		if (saved && ziped) {
			SS3Singleton.getApplicationStateController()
					.setLoadedCollectionUpdated(false);
			confirmationDialog("Saved Resource", "Collection " + name
					+ " has been saved.");
			return;
		}
	}

	public File unzipDirectory(File zFile) {

		try {
			String name = zFile.getName();

			if (name.endsWith(".zip")) {
				name = name.substring(0, name.length() - 4);
			}

			FileInputStream fin = new FileInputStream(zFile.getPath());
			// create temporary directory
			File tmpDir = new File("tmp_" + name);
			tmpDir.mkdir();

			// create ZipInputStream from FileInputStream object
			ZipInputStream zin = new ZipInputStream(fin);

			int i = 0;
			while ((zin.getNextEntry()) != null) {
				// create OutputStream to extract the entry from zip file
				String fileName = tmpDir.getName() + File.separator + "file"
						+ i + ".xml";
				OutputStream os = new FileOutputStream(fileName);

				byte[] buffer = new byte[1024];
				int length;

				// read the entry from zip file and extract it to disk
				while ((length = zin.read(buffer)) > 0) {
					os.write(buffer, 0, length);
				}

				// close the streams
				os.close();
				i++;
			}
			// close the zip file
			zin.close();

			return tmpDir;
		} catch (IOException e) {
			System.out.println("IOException :" + e);
			return null;
		}
	}

	public boolean zipDirectory(String dirName) {
		try {
			File dirFile = new File(dirName);
			String cDirName = dirFile.getName();

			if (cDirName.startsWith("tmp_")) {
				cDirName = cDirName.substring(4);
			}

			if (!cDirName.endsWith(".ss3")) {
				if (cDirName.endsWith(".ss3.zip")) {
					cDirName = cDirName.substring(0, cDirName.length() - 4);
				} else {
					cDirName = cDirName + ".ss3";
				}
			}

			String zipFile = dirFile.getParentFile().getPath()
					+ File.separatorChar + cDirName;

			byte[] buffer = new byte[1024];

			try {
				FileOutputStream fout = new FileOutputStream(zipFile);
				ZipOutputStream zout = new ZipOutputStream(fout);

				File dir = new File(dirName);

				File[] children = dir.listFiles();
				if (children != null) {
					for (int i = 0; i < children.length; i++) {
						File file = children[i];

						FileInputStream fin = new FileInputStream(file);
						zout.putNextEntry(new ZipEntry(file.getName()));
						int length;

						while ((length = fin.read(buffer)) > 0) {
							zout.write(buffer, 0, length);
						}

						zout.closeEntry();
						fin.close();
					}
				}

				// close the ZipOutputStream
				zout.close();
				fout.close();

				// remove unzipped files
				if (children != null) {
					for (int i = 0; i < children.length; i++) {
						File file = children[i];
						file.delete();
					}
				}
				dir.delete();

				if (zipFile != null) {
					String fileToDelete = zipFile + ".zip";
					File file = new File(fileToDelete);
					if (file.exists()) {
						file.delete();
					}
				}

			} catch (FileNotFoundException ex) {
				ex.printStackTrace();

				String[] str = new String[4];
				str[0] = "Collection could not be saved as .ss3 file";
				str[1] = "because access is denied.";
				str[2] = "Please close zip file and try again.";
				str[3] = "";

				SS3Singleton.getErrorMessages()
						.showSaveCollectionErrorMessage(str);
				return false;
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
			
			String[] str = new String[4];
			str[0] = "Collection could not be saved as .ss3 file";
			str[1] = ioe.getMessage();
			str[2] = "";
			str[3] = "";

			SS3Singleton.getErrorMessages()
					.showSaveCollectionErrorMessage(str);
			
			return false;
		}
		return true;
	}

	private boolean removeDirectory(String dirName) {
		File dir = new File(dirName);
		File[] children = dir.listFiles();

		// remove files
		if (children != null) {
			for (int i = 0; i < children.length; i++) {
				File file = children[i];
				file.delete();
			}
		}
		dir.delete();
		return true;
	}

	public void confirmationDialog(String title, String msg) {
		final JDialog collectionSavedDialog = new JDialog(this, title, true);
		JButton okBtn = new JButton("OK");

		ButtonPanel collectionSavedButtonPanel = new ButtonPanel();
		collectionSavedButtonPanel.add(okBtn);

		JPanel collectionSavedPanel = new JPanel();
		collectionSavedPanel.setLayout(new BorderLayout());
		JLabel collectionSavedLabel = new JLabel(msg);

		okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				collectionSavedDialog.dispose();
			}
		});

		collectionSavedPanel.add(collectionSavedLabel, BorderLayout.CENTER);
		collectionSavedPanel
				.add(collectionSavedButtonPanel, BorderLayout.SOUTH);

		collectionSavedPanel.add(new JLabel("  "), BorderLayout.WEST);

		Dimension d = collectionSavedPanel.getPreferredSize();
		Dimension nD = new Dimension(d.width + 20, d.height + 40);
		collectionSavedPanel.setPreferredSize(nD);

		Container contentPane = collectionSavedDialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(collectionSavedPanel, BorderLayout.CENTER);

		collectionSavedDialog.setLocation(new Point(150, 150));
		collectionSavedDialog.pack();
		collectionSavedDialog.setVisible(true);

	}

	public void saveCollection() {
		panel.updateLoadedDatabase();

		Object root = panel.getDbNavigationPanel().getJTree().getModel()
				.getRoot();

		if (root == null)
			return;

		DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
		int childCount = rootNode.getChildCount();
		DefaultMutableTreeNode node = null;

		if (childCount > 0) {
			node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

			if (node.getChildCount() > 0) {
				DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
						.getChildAt(0);
				Object uo = dNode.getUserObject();
				if (uo instanceof String) {
					node = dNode;
				}
			}
		}

		if (node == null) {
			confirmationDialog("Save Collection Warning",
					"Collection not available");
			return;
		}

		String name = (String) node.getUserObject();
		File dirToDel = new File(name);

		if (dirToDel.exists())
			dirToDel.delete();

		Enumeration enum4 = node.children();

		boolean saved = true;

		while (enum4.hasMoreElements()) {
			DefaultMutableTreeNode node1 = (DefaultMutableTreeNode) enum4
					.nextElement();
			Excerpt ex = (Excerpt) node1.getUserObject();
			SS3DatabaseCollection dbCol = SS3Singleton
					.getSS3DatabaseCollection();
			SS3Database db = dbCol.getSS3Database(ex);

			if (!ex.getExcerpt().equals(db.getExcerpt())) {
				// delete old file
				db.deleteFile();
				db.setExcerpt(ex.getExcerpt());
			}

			db.setNumber(ex.getID());
			db.setExcerpt(ex.getExcerpt());
			db.setDirectory(name);
			db.setFileName(ex.getExcerpt() + ".xml");

			if (!saveDatabase(db)) {
				saved = false;
			}
		}

		if (saved) {
			confirmationDialog("Save Resource", "Collection " + name
					+ " has been saved.");
		}
	}

	private boolean saveDatabase(SS3Database db) {
		try {
			return db.save();
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	public void exit() {

		if (SS3Singleton.getApplicationStateController()
				.isLoadedCollectionUpdated()) {
			saveChangesToTheCollection(false);
		} else {
			dispose();
			System.exit(0);
		}

		return;
	}
	
	public SignBankCollection getSignBankCollection() {
		return signBankCollection;
	}
	
	public SignBankCollection parseSignBank() {
		signBankCollection =  SS3Singleton.getSignBankParser().parseSignBank();
		return signBankCollection;
	}
	
	public void writeSignBankCollection() {
		SS3Singleton.getSignBankWriter().writeCollectionToSignBank(signBankCollection);
	}
}

class SignStreamWindowListener implements WindowListener {

	private SS3SignStreamApplication appl = null;

	public SignStreamWindowListener(SS3SignStreamApplication appl) {
		this.appl = appl;
	}

	@Override
	public void windowClosing(WindowEvent e) {
		appl.exit();
	}

	@Override
	public void windowClosed(WindowEvent e) {
	}

	@Override
	public void windowOpened(WindowEvent e) {
	}

	@Override
	public void windowIconified(WindowEvent e) {
	}

	@Override
	public void windowDeiconified(WindowEvent e) {
	}

	@Override
	public void windowActivated(WindowEvent e) {
	}

	@Override
	public void windowDeactivated(WindowEvent e) {
	}
}

class ImageFileView extends FileView {

	@Override
	public String getName(File f) {
		return null; 
	}

	@Override
	public String getDescription(File f) {
		return null; 
	}

	@Override
	public Boolean isTraversable(File f) {
		return null; 
	}

	@Override
	public String getTypeDescription(File f) {
		String type = null;

		if (f.isDirectory()) {
			FileFilter filter = new SignStreamFileFilter();
			File[] filteredFiles = f.listFiles(filter);
			if (filteredFiles != null && filteredFiles.length > 0) {
				type = "SignStream file";
			}
		}

		return type;
	}

	@Override
	public Icon getIcon(File f) {
		Icon icon = null;

		if (f.isDirectory()) {
			FileFilter filter = new SignStreamFileFilter();
			File[] filteredFiles = f.listFiles(filter);

			if (filteredFiles != null && filteredFiles.length > 0) {
				icon = new ImageIcon(ImageMaker.PALETTE_ICON, "SignStream");
			} 
		}

		return icon;
	}
}

class SignStreamFileFilter implements FileFilter {
	@Override
	public boolean accept(File file) {
		if (Util.isXML3File(file.getName())) {
			return true;
		}
		return false;
	}
}

class PopupListener extends MouseAdapter {
	@Override
	public void mousePressed(MouseEvent e) {
		maybeShowPopup(e);
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		maybeShowPopup(e);
	}

	private void maybeShowPopup(MouseEvent e) {
	}
}

class SignStream3FilesFilter extends javax.swing.filechooser.FileFilter {

	@Override
	public boolean accept(File f) {
		if (f.isDirectory()) {
			return true;
		} else {
			return f.getName().endsWith(".ss3.zip");
		}
	}

	@Override
	public String getDescription() {
		return "Accepts SignStream3 files only.";
	}
}

class ZipFormatFilenameFilter implements FilenameFilter {
	String ext1;
	String ext2;

	public ZipFormatFilenameFilter() {
		this.ext1 = ".ss3.zip";
		this.ext2 = ".ss3";
	}

	@Override
	public boolean accept(File dir, String name) {
		if (name.endsWith(ext1)) {
			return true;
		} else {
			return name.endsWith(ext2);
		}
	}
}

class SaveAndOpenCollectionDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;

	private SS3SignStreamApplication ssApl;

	private final int SAVE = 0;
	private final int DONT_SAVE = 1;
	private final int CANCEL = 2;

	public SaveAndOpenCollectionDialog(SS3SignStreamApplication ssApl) {
		super(ssApl, "Save Resource", true);
		this.ssApl = ssApl;

		JButton dontSaveBtn = new JButton("Don't Save");
		JButton cancelBtn = new JButton("Cancel");
		JButton saveBtn = new JButton("Save");

		dontSaveBtn.setActionCommand("" + DONT_SAVE);
		saveBtn.setActionCommand("" + SAVE);
		cancelBtn.setActionCommand("" + CANCEL);

		dontSaveBtn.addActionListener(this);
		saveBtn.addActionListener(this);
		cancelBtn.addActionListener(this);

		ButtonPanel btnPanel = new ButtonPanel();
		btnPanel.add(dontSaveBtn);
		btnPanel.add(cancelBtn);
		btnPanel.add(saveBtn);

		JLabel saveChangesLbl = new JLabel(
				"Do you want to save the changes you made?");

		JPanel newCollectionPanel = new JPanel();
		newCollectionPanel.setLayout(new BorderLayout());
		newCollectionPanel.add(saveChangesLbl, BorderLayout.CENTER);
		newCollectionPanel.add(btnPanel, BorderLayout.SOUTH);
		newCollectionPanel.add(new JLabel("  "), BorderLayout.WEST);

		Dimension d = newCollectionPanel.getPreferredSize();
		Dimension nD = new Dimension(d.width + 10, d.height + 40);
		newCollectionPanel.setPreferredSize(nD);

		Container contentPane = this.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(newCollectionPanel, BorderLayout.CENTER);

		this.setLocation(new Point(150, 150));
		this.pack();
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		SignStreamSegmentPanel segmentPanel = ssApl.getSignStreamSegmentPanel();
		DatabaseDelegate dbDelegate = ssApl.getDatabaseDelegate();

		JTree tree = segmentPanel.getDbNavigationPanel().getJTree();
		final DefaultMutableTreeNode rootNode = dbDelegate.getMutableTreeNode();

		DefaultMutableTreeNode node = null;
		int childCount = rootNode.getChildCount();

		if (childCount > 0) {
			node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

			if (node.getChildCount() > 0) {
				DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
						.getChildAt(0);
				Object uo = dNode.getUserObject();
				if (uo instanceof String) {
					node = dNode;
				}
			}
		}

		switch (Integer.parseInt(str)) {
		case SAVE:
			if (node == null) {
				SS3Singleton
				.getSignStreamApplication().confirmationDialog("Save Resource", "Collection is not available.");
				return;
			}
			ssApl.saveLoadedDatabase(node);
		case DONT_SAVE:
			this.setVisible(false);
			
			ssApl.openDatabaseSS3(tree, rootNode, node);
			
			break;
		case CANCEL:
			this.setVisible(false);
			break;
		}
	}
}
