package edu.bu.signstream.ui.panels;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.newDatabase.ParticipantView;
import edu.bu.signstream.ui.panels.newDatabase.ParticipantsPaneScrolledView;

public class AnnotatorsPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;

	private ParticipantView participantView = null;
	private ParticipantsPaneScrolledView participantsPaneScrolledView = null;
	private AnnotatorsDialog parentDialog = null;
	private NewAnnotatorDialog newAnnotatorDialog = null;

	private SS3Participant selectedParticipant = null;
	private SS3Participant newSelectedParticipant = null;
	private ArrayList<SS3Participant> selectedParticipants = new ArrayList<SS3Participant>();

	private JTextField nameTxt = new JTextField(14);
	private JTextField labelTxt = new JTextField(14);
	private JTextArea commentsTa = new JTextArea();

	// list of SS3Participant objects
	private JList globalParticipantsList = null;
	private DefaultListModel globalParticipantsListModel = new DefaultListModel();
	// list of SS3Participant objects
	private JList localParticipantsList = null;
	private DefaultListModel localParticipantsListModel = new DefaultListModel();

	private JScrollPane listScrollPane = null;
	private JScrollPane localParticipantsListScrollPane = null;

	private JPanel panel = new JPanel();

	private JButton newBtn = new JButton("New");
	private JButton editBtn = new JButton("Edit");
	private JButton deleteBtn = new JButton("Remove");
	private JButton cancelBtn = new JButton("Cancel");
	private JButton submitBtn = new JButton("Add Selection");
	private JButton discardCancelBtn = new JButton("Discard");
	private JButton saveCancelBtn = new JButton("Save");
	private JButton cancelDeleteBtn = new JButton("Cancel");
	private JButton discardDeleteBtn = new JButton("Discard");

	private JLabel globalAnnotatorsLbl = null;
	private JLabel localAnnotatorsLbl = null;
	private JLabel annotatorLabelLbl = null;
	private JLabel annotatorNameLbl = null;
	private JLabel commentsLbl = new JLabel("Comments");

	private final int NEW = 0;
	private final int EDIT = 1;
	private final int DELETE = 2;
	private final int CANCEL = 3;
	private final int SUBMIT = 4;
	private final int DISCARD_CANCEL = 5;
	private final int CANCEL_DELETE = 6;
	private final int DISCARD_DELETE = 7;

	public AnnotatorsPanel(SS3Participant selectedParticipant,
			final ParticipantsPaneScrolledView participantsPaneScrolledView,
			ParticipantView participantView, AnnotatorsDialog parentDialog) {

		this.selectedParticipant = selectedParticipant;
		this.participantsPaneScrolledView = participantsPaneScrolledView;
		this.participantView = participantView;
		this.parentDialog = parentDialog;

		ParticipantsLibrary annotatorsLibrary = null;
		ArrayList<ParticipantView> views = null;

		if ((participantView != null && participantView.isParticipant())
				|| (participantsPaneScrolledView != null && participantsPaneScrolledView
						.isParticipant())) {
			globalAnnotatorsLbl = new JLabel("Global Participants");
			localAnnotatorsLbl = new JLabel("Local Participants");
			annotatorLabelLbl = new JLabel("Participant Label");
			annotatorNameLbl = new JLabel("Particicpant Full Name");

			if (participantView != null && participantView.isParticipant()) {
				annotatorsLibrary = participantView.getParticipantsLibrary();
				views = participantView.getEnteredViews();
			} else {
				annotatorsLibrary = participantsPaneScrolledView
						.getParticipantLibrary();
				views = participantsPaneScrolledView.getParticipantViews();
			}
		} else {
			if (participantView != null && !participantView.isParticipant()) {
				annotatorsLibrary = participantView.getAnnotatorsLibrary();
				views = participantView.getEnteredViews();
			} else {
				annotatorsLibrary = participantsPaneScrolledView
						.getParticipantLibrary();
				views = participantsPaneScrolledView.getParticipantViews();
			}

			globalAnnotatorsLbl = new JLabel("Global Annotators");
			localAnnotatorsLbl = new JLabel("Local Annotators");
			annotatorLabelLbl = new JLabel("Annotator Label");
			annotatorNameLbl = new JLabel("Annotator Full Name");
		}

		globalParticipantsList = new JList(globalParticipantsListModel);
		globalParticipantsList
				.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		localParticipantsList = new JList(localParticipantsListModel);
		localParticipantsList
				.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		if (selectedParticipant != null) {
			submitBtn.setEnabled(false);
			nameTxt.setText(selectedParticipant.getFullName());
			labelTxt.setText(selectedParticipant.getLabel());
			commentsTa.setText(selectedParticipant.getComments());
		}

		if (annotatorsLibrary != null) {
			HashMap<String, SS3Participant> globalAnnotators = annotatorsLibrary
					.getGlobalParticipants();
			for (String glbAnnotID : globalAnnotators.keySet()) {
				SS3Participant glbParticipant = globalAnnotators
						.get(glbAnnotID);
				glbParticipant.setSelected(false);
				globalParticipantsListModel.addElement(glbParticipant);

				if (glbParticipant.equals(selectedParticipant)) {
					globalParticipantsList.setSelectedValue(glbParticipant,
							true);
				}
			}

			HashMap<String, SS3Participant> localAnnotators = annotatorsLibrary
					.getLocalParticipants();
			for (String lclAnnotID : localAnnotators.keySet()) {
				SS3Participant lclParticipant = localAnnotators.get(lclAnnotID);
				lclParticipant.setSelected(false);
				localParticipantsListModel.addElement(lclParticipant);

				if (lclParticipant.equals(selectedParticipant)) {
					localParticipantsList
							.setSelectedValue(lclParticipant, true);
				}
			}
		}

		// already selected participants
		for (ParticipantView view : views) {
			SS3Participant prt = view.getParticipant();
			prt.setSelected(true);
			selectedParticipants.add(prt);
		}

		nameTxt.setEnabled(false);
		labelTxt.setEnabled(false);
		commentsTa.setEnabled(false);
		deleteBtn.setEnabled(false);

		if (selectedParticipant == null) {
			editBtn.setEnabled(false);
		}

		globalParticipantsList
				.addListSelectionListener(new ListSelectionListener() {
					@Override
					public void valueChanged(ListSelectionEvent evt) {
						Object gObj = globalParticipantsList.getSelectedValue();

						if (gObj != null) {
							editBtn.setEnabled(true);
							submitBtn.setEnabled(true);
							SS3Participant sPrt = (SS3Participant) gObj;

							if (sPrt.isSelected())
								submitBtn.setEnabled(false);

							deleteBtn.setEnabled(false);

							newSelectedParticipant = sPrt;
							nameTxt.setText(sPrt.getFullName());
							labelTxt.setText(sPrt.getLabel());
							commentsTa.setText(sPrt.getComments());
							localParticipantsList.clearSelection();
						}
					}
				});

		localParticipantsList
				.addListSelectionListener(new ListSelectionListener() {
					@Override
					public void valueChanged(ListSelectionEvent evt) {
						Object obj = localParticipantsList.getSelectedValue();

						if (obj != null) {
							editBtn.setEnabled(true);
							submitBtn.setEnabled(true);
							SS3Participant sPrt = (SS3Participant) obj;

							if (sPrt.isSelected())
								submitBtn.setEnabled(false);

							newSelectedParticipant = sPrt;
							globalParticipantsList.clearSelection();
							nameTxt.setText(sPrt.getFullName());
							labelTxt.setText(sPrt.getLabel());
							commentsTa.setText(sPrt.getComments());

							deleteBtn.setEnabled(sPrt.isJustCreated() && !sPrt.isSelected());
						}
					}
				});

		newBtn.setActionCommand("" + NEW);
		editBtn.setActionCommand("" + EDIT);
		deleteBtn.setActionCommand("" + DELETE);
		cancelBtn.setActionCommand("" + CANCEL);
		submitBtn.setActionCommand("" + SUBMIT);
		discardCancelBtn.setActionCommand("" + DISCARD_CANCEL);
		saveCancelBtn.setActionCommand("" + SUBMIT);
		cancelDeleteBtn.setActionCommand("" + CANCEL_DELETE);
		discardDeleteBtn.setActionCommand("" + DISCARD_DELETE);

		newBtn.addActionListener(this);
		editBtn.addActionListener(this);
		deleteBtn.addActionListener(this);
		cancelBtn.addActionListener(this);
		submitBtn.addActionListener(this);
		discardCancelBtn.addActionListener(this);
		saveCancelBtn.addActionListener(this);
		cancelDeleteBtn.addActionListener(this);
		discardDeleteBtn.addActionListener(this);

		UserInterfaceUtil.setLabelLNF(globalAnnotatorsLbl);
		UserInterfaceUtil.setLabelLNF(localAnnotatorsLbl);
		UserInterfaceUtil.setLabelLNF(annotatorLabelLbl);
		UserInterfaceUtil.setLabelLNF(annotatorNameLbl);
		UserInterfaceUtil.setLabelLNF(commentsLbl);

		UserInterfaceUtil.setTextAreaLNF(commentsTa);
		UserInterfaceUtil.setTextFieldLNF(nameTxt);
		UserInterfaceUtil.setTextFieldLNF(labelTxt);

		UserInterfaceUtil.setListLNF(globalParticipantsList);
		UserInterfaceUtil.setListLNF(localParticipantsList);

		initUI();
	}
	
	public void updateAnnotatorsPanel() {
		this.selectedParticipant = selectedParticipant;
		this.participantsPaneScrolledView = participantsPaneScrolledView;
		this.participantView = participantView;
		this.parentDialog = parentDialog;

		ParticipantsLibrary annotatorsLibrary = null;
		ArrayList<ParticipantView> views = null;

		if ((participantView != null && participantView.isParticipant())
				|| (participantsPaneScrolledView != null && participantsPaneScrolledView
						.isParticipant())) {
			globalAnnotatorsLbl = new JLabel("Global Participants");
			localAnnotatorsLbl = new JLabel("Local Participants");
			annotatorLabelLbl = new JLabel("Participant Label");
			annotatorNameLbl = new JLabel("Particicpant Full Name");

			if (participantView != null && participantView.isParticipant()) {
				annotatorsLibrary = participantView.getParticipantsLibrary();
				views = participantView.getEnteredViews();
			} else {
				annotatorsLibrary = participantsPaneScrolledView
						.getParticipantLibrary();
				views = participantsPaneScrolledView.getParticipantViews();
			}
		} else {
			if (participantView != null && !participantView.isParticipant()) {
				annotatorsLibrary = participantView.getAnnotatorsLibrary();
				views = participantView.getEnteredViews();
			} else {
				annotatorsLibrary = participantsPaneScrolledView
						.getParticipantLibrary();
				views = participantsPaneScrolledView.getParticipantViews();
			}

			globalAnnotatorsLbl = new JLabel("Global Annotators");
			localAnnotatorsLbl = new JLabel("Local Annotators");
			annotatorLabelLbl = new JLabel("Annotator Label");
			annotatorNameLbl = new JLabel("Annotator Full Name");
		}

		globalParticipantsList = new JList(globalParticipantsListModel);
		globalParticipantsList
				.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		localParticipantsList = new JList(localParticipantsListModel);
		localParticipantsList
				.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		if (selectedParticipant != null) {
			submitBtn.setEnabled(false);
			nameTxt.setText(selectedParticipant.getFullName());
			labelTxt.setText(selectedParticipant.getLabel());
			commentsTa.setText(selectedParticipant.getComments());
		}

		if (annotatorsLibrary != null) {
			HashMap<String, SS3Participant> globalAnnotators = annotatorsLibrary
					.getGlobalParticipants();
			for (String glbAnnotID : globalAnnotators.keySet()) {
				SS3Participant glbParticipant = globalAnnotators
						.get(glbAnnotID);
				glbParticipant.setSelected(false);
				globalParticipantsListModel.addElement(glbParticipant);

				if (glbParticipant.equals(selectedParticipant)) {
					globalParticipantsList.setSelectedValue(glbParticipant,
							true);
				}
			}

			HashMap<String, SS3Participant> localAnnotators = annotatorsLibrary
					.getLocalParticipants();
			for (String lclAnnotID : localAnnotators.keySet()) {
				SS3Participant lclParticipant = localAnnotators.get(lclAnnotID);
				lclParticipant.setSelected(false);
				localParticipantsListModel.addElement(lclParticipant);

				if (lclParticipant.equals(selectedParticipant)) {
					localParticipantsList
							.setSelectedValue(lclParticipant, true);
				}
			}
		}

		// already selected participants
		for (ParticipantView view : views) {
			SS3Participant prt = view.getParticipant();
			prt.setSelected(true);
			selectedParticipants.add(prt);
		}

		nameTxt.setEnabled(false);
		labelTxt.setEnabled(false);
		commentsTa.setEnabled(false);
		deleteBtn.setEnabled(false);

		if (selectedParticipant == null) {
			editBtn.setEnabled(false);
		}

		globalParticipantsList
				.addListSelectionListener(new ListSelectionListener() {
					@Override
					public void valueChanged(ListSelectionEvent evt) {
						Object gObj = globalParticipantsList.getSelectedValue();

						if (gObj != null) {
							editBtn.setEnabled(true);
							submitBtn.setEnabled(true);
							SS3Participant sPrt = (SS3Participant) gObj;

							if (sPrt.isSelected())
								submitBtn.setEnabled(false);

							deleteBtn.setEnabled(false);

							newSelectedParticipant = sPrt;
							nameTxt.setText(sPrt.getFullName());
							labelTxt.setText(sPrt.getLabel());
							commentsTa.setText(sPrt.getComments());
							localParticipantsList.clearSelection();
						}
					}
				});

		localParticipantsList
				.addListSelectionListener(new ListSelectionListener() {
					@Override
					public void valueChanged(ListSelectionEvent evt) {
						Object obj = localParticipantsList.getSelectedValue();

						if (obj != null) {
							editBtn.setEnabled(true);
							submitBtn.setEnabled(true);
							SS3Participant sPrt = (SS3Participant) obj;

							if (sPrt.isSelected())
								submitBtn.setEnabled(false);

							newSelectedParticipant = sPrt;
							globalParticipantsList.clearSelection();
							nameTxt.setText(sPrt.getFullName());
							labelTxt.setText(sPrt.getLabel());
							commentsTa.setText(sPrt.getComments());

							deleteBtn.setEnabled(sPrt.isJustCreated() && !sPrt.isSelected());
						}
					}
				});

		newBtn.setActionCommand("" + NEW);
		editBtn.setActionCommand("" + EDIT);
		deleteBtn.setActionCommand("" + DELETE);
		cancelBtn.setActionCommand("" + CANCEL);
		submitBtn.setActionCommand("" + SUBMIT);
		discardCancelBtn.setActionCommand("" + DISCARD_CANCEL);
		saveCancelBtn.setActionCommand("" + SUBMIT);
		cancelDeleteBtn.setActionCommand("" + CANCEL_DELETE);
		discardDeleteBtn.setActionCommand("" + DISCARD_DELETE);

		newBtn.addActionListener(this);
		editBtn.addActionListener(this);
		deleteBtn.addActionListener(this);
		cancelBtn.addActionListener(this);
		submitBtn.addActionListener(this);
		discardCancelBtn.addActionListener(this);
		saveCancelBtn.addActionListener(this);

		UserInterfaceUtil.setLabelLNF(globalAnnotatorsLbl);
		UserInterfaceUtil.setLabelLNF(localAnnotatorsLbl);
		UserInterfaceUtil.setLabelLNF(annotatorLabelLbl);
		UserInterfaceUtil.setLabelLNF(annotatorNameLbl);
		UserInterfaceUtil.setLabelLNF(commentsLbl);

		UserInterfaceUtil.setTextAreaLNF(commentsTa);
		UserInterfaceUtil.setTextFieldLNF(nameTxt);
		UserInterfaceUtil.setTextFieldLNF(labelTxt);

		UserInterfaceUtil.setListLNF(globalParticipantsList);
		UserInterfaceUtil.setListLNF(localParticipantsList);
	}

	
	public ParticipantsPaneScrolledView getParticipantsPaneScrolledView() {
		return participantsPaneScrolledView;
	}

	private void initUI() {
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		panel.setLayout(gbl);

		int width = labelTxt.getPreferredSize().width;

		listScrollPane = new JScrollPane(globalParticipantsList);
		listScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		listScrollPane.setPreferredSize(new Dimension(width,
				globalParticipantsList.getPreferredSize().height));

		localParticipantsListScrollPane = new JScrollPane(localParticipantsList);
		localParticipantsListScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		localParticipantsListScrollPane.setPreferredSize(new Dimension(width,
				96));

		JScrollPane commentsScrollPane = new JScrollPane(commentsTa);
		commentsScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		commentsScrollPane.setPreferredSize(new Dimension(width, 56));

		ButtonPanel tPanel = new ButtonPanel();
		tPanel.add(newBtn);
		tPanel.add(new JLabel(" "));
		tPanel.add(editBtn);
		tPanel.add(new JLabel(" "));
		tPanel.add(deleteBtn);

		UserInterfaceUtil.setButtonsLNF(newBtn);
		UserInterfaceUtil.setButtonsLNF(editBtn);
		UserInterfaceUtil.setButtonsLNF(deleteBtn);
		newBtn.setPreferredSize(deleteBtn.getPreferredSize());
		editBtn.setPreferredSize(deleteBtn.getPreferredSize());

		gbc.insets = new Insets(1, 2, 1, 2);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 0, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(localAnnotatorsLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 0, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(globalAnnotatorsLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 1, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(tPanel, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 2, 1, 5, // 2,
				GridBagConstraints.BOTH);
		panel.add(localParticipantsListScrollPane, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 1, 1, 6, // 3,
				GridBagConstraints.BOTH);
		panel.add(listScrollPane, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 7, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(annotatorNameLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 8, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(nameTxt, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 9, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(annotatorLabelLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 10, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(labelTxt, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 7, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(commentsLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 8, 1, 3,
				GridBagConstraints.BOTH);
		panel.add(commentsScrollPane, gbc);

		panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));

		setLayout(new BorderLayout(30, 30));
		add(panel, BorderLayout.CENTER);

		ButtonPanel bPanel = new ButtonPanel();
		bPanel.add(cancelBtn);
		bPanel.add(submitBtn);

		add(bPanel, BorderLayout.SOUTH);
	}

	public ParticipantView getParticipantView() {
		return participantView;
	}

	public SS3Participant getNewSelectedParticipant() {
		return newSelectedParticipant;
	}

	public JTextField getNameTxt() {
		return nameTxt;
	}

	public JTextField getLabelTxt() {
		return labelTxt;
	}

	public JTextArea getCommentsTa() {
		return commentsTa;
	}

	public JList getGlobalParticipantsList() {
		return globalParticipantsList;
	}

	public DefaultListModel getGlobalParticipantsListModel() {
		return globalParticipantsListModel;
	}

	public JList getLocalParticipantsList() {
		return localParticipantsList;
	}

	public DefaultListModel getLocalParticipantsListModel() {
		return localParticipantsListModel;
	}

	private boolean isEdited() {
		Object obj = globalParticipantsList.getSelectedValue() != null ? globalParticipantsList
				.getSelectedValue() : localParticipantsList.getSelectedValue();

		if (obj == null || obj.equals(selectedParticipant))
			return false;

		return true;
	}

	private JDialog cancelDialog = null;

	public void cancelEntry() {
		newSelectedParticipant = null;	// if you hit cancel, then clear what was selected on screen
		
		if (isEdited()) {
			if (cancelDialog == null) {
				cancelDialog = new JDialog(parentDialog, "Unsaved data");

				ButtonPanel buttonPanel = new ButtonPanel();
				buttonPanel.add(discardCancelBtn);
				buttonPanel.add(saveCancelBtn);

				JPanel centerPanel = new JPanel();
				centerPanel.setLayout(new GridLayout(1, 1));
				JLabel l1 = new JLabel(
						"Warning:  Unless you save this information");
				JLabel l2 = new JLabel(
						"before proceeding, it will be discarded.");
				centerPanel.add(l1);
				centerPanel.add(l2);

				JPanel panel = new JPanel();
				panel.setLayout(new BorderLayout());
				panel.add(centerPanel, BorderLayout.CENTER);
				panel.add(buttonPanel, BorderLayout.SOUTH);
				panel.add(new JLabel(" "), BorderLayout.WEST);

				Dimension d = panel.getPreferredSize();
				Dimension nD = new Dimension(d.width + 10, d.height + 40);
				panel.setPreferredSize(nD);

				Container contentPane = cancelDialog.getContentPane();
				contentPane.setLayout(new BorderLayout(40, 40));
				contentPane.add(panel, BorderLayout.CENTER);

				cancelDialog.setLocation(new Point(150, 150));
				cancelDialog.pack();
			}
			cancelDialog.setVisible(true);
		} else {
			parentDialog.setVisible(false);
		}
	}

	private void enterEntry() {
		Object obj = globalParticipantsList.getSelectedValue() != null ? globalParticipantsList
				.getSelectedValue() : localParticipantsList.getSelectedValue();

		if (obj != null) {
			SS3Participant participant = (SS3Participant) obj;

			if (participantView != null) {
				participantView.setParticipant(participant);
			} else {
				participantsPaneScrolledView.addParticipant(participant);
			}
		}

		parentDialog.setVisible(false);
	}

	ParticipantDialog participantDialog = null;

	private void showParticipantDialog(SS3Participant participant) {
		if (participantDialog == null) {
			participantDialog = new ParticipantDialog(participant, null, this);
			participantDialog.setSize(280, 310);
			participantDialog.setLocation(150, 150);

		} else {
			participantDialog.updateParticipant(participant, null, this);
		}

		participantDialog.setVisible(true);
	}

	private JDialog deleteDialog = null;
	private JLabel deleteLbl = null;

	private void showDeleteDialog() {
		Object obj = localParticipantsList.getSelectedValue();
		if (obj != null) {
			SS3Participant participant = (SS3Participant) obj;

			if (deleteDialog == null) {
				deleteDialog = new JDialog(parentDialog, "Deletion Confirmation");
				ButtonPanel buttonPanel = new ButtonPanel();
				buttonPanel.add(cancelDeleteBtn);
				buttonPanel.add(discardDeleteBtn);

				JPanel panel = new JPanel();
				panel.setLayout(new BorderLayout());

				JPanel centerPanel = new JPanel();
				centerPanel.setLayout(new GridLayout(1, 1));
				JLabel l1 = new JLabel(
						"Warning: Are you sure you wish to discard");
				deleteLbl = new JLabel("participant/annotator "
						+ participant.getLabel() + "?");
				centerPanel.add(l1);
				centerPanel.add(deleteLbl);

				panel.add(centerPanel, BorderLayout.CENTER);
				panel.add(buttonPanel, BorderLayout.SOUTH);
				panel.add(new JLabel("  "), BorderLayout.WEST);

				Dimension d = panel.getPreferredSize();
				Dimension nD = new Dimension(d.width + 10, d.height + 40);
				panel.setPreferredSize(nD);

				Container contentPane = deleteDialog.getContentPane();
				contentPane.setLayout(new BorderLayout(40, 40));
				contentPane.add(panel, BorderLayout.CENTER);

				deleteDialog.setLocation(new Point(150, 150));
				deleteDialog.pack();
			} else {
				String txt = "participant/annotator " + participant.getLabel()
						+ "?";
				deleteLbl.setText(txt);
				deleteLbl.repaint();
				deleteDialog.getContentPane().repaint();
			}
			deleteDialog.setVisible(true);
		}
	}

	private void showNewAnnotatorDialog(SS3Participant participant) {
		if (newAnnotatorDialog == null) {
			newAnnotatorDialog = new NewAnnotatorDialog(participant, participantView, this);
			newAnnotatorDialog.setLocation(150, 150);
			newAnnotatorDialog.setSize(280, 165);
		}
		newAnnotatorDialog.update(participant, participantView, this);	// passing in this instead  of null, part of fixing save new annotator bug
		newAnnotatorDialog.setVisible(true);
	}
	
	public NewAnnotatorDialog getNewAnnotatorDialog() {
		return newAnnotatorDialog;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case NEW:
			if (isParticipant()) {
				showParticipantDialog(null);
			} else {
				showNewAnnotatorDialog(null);
			}
			break;

		case EDIT:
			SS3Participant participant = newSelectedParticipant != null ? newSelectedParticipant
					: selectedParticipant;

			if (isParticipant()) {
				showParticipantDialog(participant);
			} else {
				showNewAnnotatorDialog(participant);
			}
			break;

		case DELETE:
			showDeleteDialog();
			break;
		case CANCEL:
			cancelEntry();
			break;
		case SUBMIT:
			enterEntry();
			break;
		case DISCARD_CANCEL:
			if (cancelDialog != null) {
				cancelDialog.setVisible(false);
			}
			parentDialog.setVisible(false);
			break;
		case CANCEL_DELETE:
			deleteDialog.setVisible(false);
			break;
		case DISCARD_DELETE:
			Object obj = localParticipantsList.getSelectedValue();
			if (obj != null) {
				localParticipantsListModel.removeElement(obj);
				deleteBtn.setEnabled(false);
				deleteDialog.setVisible(false);
				if (obj instanceof SS3Participant) {
					SS3Participant participantToDelete = (SS3Participant) obj;
					ParticipantsLibrary library = getParticipantsPaneScrolledView().getParticipantLibrary();
					library.removeLocalParticipant(participantToDelete.getId());
					edu.bu.signstream.common.db.UtilityFilesXMLWriter writer = new edu.bu.signstream.common.db.UtilityFilesXMLWriter();
					try {
						if (isParticipant()) {
							writer.saveParticipantsLibrary(library);
						} else {
							writer.saveAnnotatorsLibrary(library);
						}
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			}
			break;
		}
	}

	private boolean isParticipant() {
		return (participantView != null && participantView.isParticipant())
				|| (participantsPaneScrolledView != null && participantsPaneScrolledView
						.isParticipant());
	}
}

class NameFieldListener implements DocumentListener {
	private JTextField nameTxt = null;
	private JList participantsList = null;

	NameFieldListener(JTextField nameTxt, JList participantsList) {
		super();
		this.nameTxt = nameTxt;
		this.participantsList = participantsList;
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		update();
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		update();
	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		update();
	}

	private void update() {
		Object obj = participantsList.getSelectedValue();

		if (obj != null) {
			SS3Participant participant = (SS3Participant) obj;
			participant.setFullName(nameTxt.getText().trim());
		}
	}
}

class LabelFieldListener implements DocumentListener {
	private JTextField labelTxt = null;
	private JList participantsList = null;

	LabelFieldListener(JTextField labelTxt, JList participantsList) {
		super();
		this.labelTxt = labelTxt;
		this.participantsList = participantsList;
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		update();
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		update();
	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		update();
	}

	private void update() {
		Object obj = participantsList.getSelectedValue();

		if (obj != null) {
			SS3Participant participant = (SS3Participant) obj;
			participant.setLabel(labelTxt.getText().trim());
		}
	}
}

class CommentsFieldListener implements DocumentListener {
	private JTextArea commentsTa = null;
	private JList participantsList = null;

	CommentsFieldListener(JTextArea commentsTa, JList participantsList) {
		super();
		this.commentsTa = commentsTa;
		this.participantsList = participantsList;
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		update();
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		update();
	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		update();
	}

	private void update() {
		Object obj = participantsList.getSelectedValue();

		if (obj != null) {
			SS3Participant participant = (SS3Participant) obj;
			participant.setComments(commentsTa.getText().trim());
		}
	}
}
