package edu.bu.signstream.ui.panels;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaLibrary;
import edu.bu.signstream.ui.ButtonPanel;

public class MediaLibraryPanel extends JPanel implements ActionListener {

	private SS3MediaLibrary library;

	private JTable videoLibraryTable = null;

	private JFrame frame = null;

	private JList searchList = new JList();

	private JTextField defLocTxt = new JTextField();

	private DefaultListModel listModel = new DefaultListModel();

	private JButton updateDefFilesLocationBtn = new JButton("Update");

	private JButton removeSearchLocationBtn = new JButton("Remove");

	private JButton addSearchLocationBtn = new JButton("Add");

	private JButton updateVideoFileLocationBtn = new JButton("Update");

	private JButton removeVideoFilesBtn = new JButton("Remove");

	private JButton addVideoFileBtn = new JButton("Add");

	private final int UPDATE_DEF_LOCATION = 0;

	private final int REMOVE_SEARCH_LOCATION = 1;

	private final int ADD_SEARCH_LOCATION = 2;

	private final int UPDATE_VIDEO_FILES_LOCATION = 3;

	private final int REMOVE_VIDEO_FILES_LOCATION = 4;

	private final int ADD_VIDEO_FILES_LOCATION = 5;

	private MediaLibraryTableModel tableModel;

	private JTextField defFilesLocationTxt = new JTextField();

	public MediaLibraryPanel(SS3MediaLibrary library, JFrame frame) {
		this.library = library;
		this.frame = frame;

		initUI(library);
	}

	private void initUI(SS3MediaLibrary library) {

		updateDefFilesLocationBtn.setActionCommand("" + UPDATE_DEF_LOCATION);
		removeSearchLocationBtn.setActionCommand("" + REMOVE_SEARCH_LOCATION);
		addSearchLocationBtn.setActionCommand("" + ADD_SEARCH_LOCATION);
		updateVideoFileLocationBtn.setActionCommand(""
				+ UPDATE_VIDEO_FILES_LOCATION);
		removeVideoFilesBtn.setActionCommand("" + REMOVE_VIDEO_FILES_LOCATION);
		addVideoFileBtn.setActionCommand("" + ADD_VIDEO_FILES_LOCATION);

		updateDefFilesLocationBtn.addActionListener(this);
		removeSearchLocationBtn.addActionListener(this);
		addSearchLocationBtn.addActionListener(this);
		updateVideoFileLocationBtn.addActionListener(this);
		removeVideoFilesBtn.addActionListener(this);
		addVideoFileBtn.addActionListener(this);

		// populate search list
//		Iterator iter = library.getSearchLocations().iterator();
//		while (iter.hasNext()) {
//			listModel.addElement(iter.next());
//		}
		searchList.setModel(listModel);

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				searchList.ensureIndexIsVisible(listModel.getSize() - 1);
			}
		});

		// create video table
		videoLibraryTable = createVideoLibraryTable();

		defLocTxt.setPreferredSize(new Dimension(300, defLocTxt
				.getPreferredSize().height));

		// create default files location panel
		JPanel defLocPane = new JPanel();
		defLocPane.setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(0, 0, 0, 0);
		defLocPane.add(new JLabel("Default files location"), gbc);
		defLocPane.add(Box.createHorizontalStrut(10));
		defLocPane.add(defLocTxt, gbc);
		defLocPane.add(Box.createHorizontalStrut(10));
		defLocPane.add(updateDefFilesLocationBtn, gbc);

		// create search locations panel
		JPanel searchLocPane = new JPanel();
		searchLocPane.setLayout(new BorderLayout());
		searchLocPane.setBorder(new TitledBorder("Search locations"));
		JScrollPane jsp1 = new JScrollPane(searchList);
		jsp1.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		searchLocPane.add(jsp1, BorderLayout.CENTER);
		// searchList.setLayoutOrientation(JList.HORIZONTAL_WRAP);
		ButtonPanel searchBtnPanel = new ButtonPanel();
		searchBtnPanel.add(removeSearchLocationBtn);
		searchBtnPanel.add(addSearchLocationBtn);
		searchLocPane.add(searchBtnPanel, BorderLayout.SOUTH);
		searchLocPane.setPreferredSize(new Dimension(defLocPane
				.getPreferredSize().width, 250));
		// searchLocPane.getPreferredSize().height));

		// create video library panel
		JPanel videoPanel = new JPanel();
		videoPanel.setLayout(new BorderLayout());
		videoPanel.setBorder(new TitledBorder("Media Library"));
		JScrollPane jsp2 = new JScrollPane(videoLibraryTable);
		jsp2.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		videoPanel.add(jsp2, BorderLayout.CENTER);
		ButtonPanel videoButtonPanel = new ButtonPanel();
		videoButtonPanel.add(removeVideoFilesBtn);
		videoButtonPanel.add(updateVideoFileLocationBtn);
		videoButtonPanel.add(addVideoFileBtn);
		videoPanel.add(videoButtonPanel, BorderLayout.SOUTH);
		videoPanel.setPreferredSize(new Dimension(
				defLocPane.getPreferredSize().width, 250));

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints cgbc = new GridBagConstraints();

		setLayout(gbl);

		cgbc.anchor = GridBagConstraints.NORTHWEST;
		cgbc.gridwidth = GridBagConstraints.REMAINDER;
		cgbc.insets = new Insets(0, 0, 10, 0);
		add(defLocPane, cgbc);

		cgbc.anchor = GridBagConstraints.NORTHWEST;
		cgbc.gridwidth = GridBagConstraints.REMAINDER;
		add(searchLocPane, cgbc);

		cgbc.anchor = GridBagConstraints.NORTHWEST;
		cgbc.gridwidth = GridBagConstraints.REMAINDER;
		add(videoPanel, cgbc);
	}

	private JPanel addVideoPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());

		JTextField txt = new JTextField("");

		panel.add(new JLabel("Media File:"), BorderLayout.WEST);
		panel.add(txt, BorderLayout.CENTER);
		panel.add(new JButton("Add"), BorderLayout.EAST);

		return panel;
	}

	private JTable createVideoLibraryTable() {

		Object[] columnNames = { "ID", "File name", "File location",
				"Description" };

		Collection collection = library.getMediaFilesList();
		Object[][] data = new Object[collection.size()][];

		int j = 0;
		Iterator iter = collection.iterator();
		while (iter.hasNext()) {
			SS3MediaFile mFile = (SS3MediaFile) iter.next();

			Object[] obj = { mFile.getId(), mFile.getFile().getName(),
					mFile.getFile().getPath(), mFile.getDescription() };
			data[j] = obj;
			j++;
		}

		tableModel = new MediaLibraryTableModel(columnNames, data);
		MediaLibrarySorter sorter = new MediaLibrarySorter(tableModel);
		final JTable table = new JTable(sorter);
		sorter.setTableHeader(table.getTableHeader());

		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		table.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					@Override
					public void valueChanged(ListSelectionEvent e) {

						if (e.getValueIsAdjusting()) {
							MediaLibrarySorter sorter1 = (MediaLibrarySorter) table
									.getModel();
							sorter1.update(table.getSelectedRow(), table
									.getSelectedColumn());
						}

						// setPreferredSize();
					}
				});

		return table;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();
		String defPath = defLocTxt.getText();

		switch (Integer.parseInt(str)) {

		case UPDATE_DEF_LOCATION:
			FileDialog fd = new FileDialog(frame, "Open a database",
					FileDialog.LOAD);
			

			if (defPath != null && defPath.length() > 0)
				fd.setDirectory(defPath);

			fd.show();

			if (fd.getFile() != null) {
				String selectedFileName = fd.getDirectory() + File.separator
						+ fd.getFile();
				defLocTxt.setText(new File(selectedFileName).getAbsolutePath());
			}

			break;

		case REMOVE_SEARCH_LOCATION:
			int[] selected = searchList.getSelectedIndices();
			for (int i = 0; i < selected.length; ++i) {
				listModel.removeElementAt(selected[i] - i);
			}
			break;

		case ADD_SEARCH_LOCATION:
			JFileChooser addSrchFc = new JFileChooser();

			if (defPath != null && defPath.length() > 0)
				addSrchFc.setCurrentDirectory(new File(defPath));

			addSrchFc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

			int searchReturnVal = addSrchFc.showOpenDialog(this);
			File searchFile = addSrchFc.getSelectedFile();

			if (searchReturnVal == JFileChooser.APPROVE_OPTION
					&& searchFile != null) {
				// defLocTxt.setText(file.getAbsolutePath());
				listModel.addElement(searchFile.getAbsolutePath());
			}

			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					searchList.ensureIndexIsVisible(listModel.getSize() - 1);
				}
			});
			break;
		case UPDATE_VIDEO_FILES_LOCATION:
			;
			break;
		case REMOVE_VIDEO_FILES_LOCATION:
			;
			break;
		case ADD_VIDEO_FILES_LOCATION:
			JFileChooser addVideoFc = new JFileChooser();

			if (defPath != null && defPath.length() > 0)
				addVideoFc.setCurrentDirectory(new File(defPath));

			addVideoFc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

			int videoReturnVal = addVideoFc.showOpenDialog(this);
			File videoFile = addVideoFc.getSelectedFile();

			if (videoReturnVal == JFileChooser.APPROVE_OPTION
					&& videoFile != null) {
				// khjjk
				System.out.println("add video files location: "
						+ videoFile.getAbsolutePath());
				// listModel.addElement(videoFile.getAbsolutePath());
			}
			break;
		}
	}
}
