package edu.bu.signstream.ui.panels;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.newDatabase.ParticipantView;

public class NewAnnotatorPanel extends JPanel implements ActionListener {
	private static final long serialVersionUID = 1L;

	private SS3Participant participant = null;
	private AnnotatorsPanel annotatorPanel = null;
	private ParticipantView participantView = null;
	private ArrayList<String> enteredAnnotatorNames = new ArrayList<String>();
	private ArrayList<String> enteredAnnotatorLabels = new ArrayList<String>();

	private JDialog parentDialog = null;

	private JTextField nameTxt = new JTextField(11);
	private JTextField labelTxt = new JTextField(11);
	private JTextArea commentsTa = new JTextArea();

	private JLabel nameLbl = new JLabel("Annotator Full Name");
	private JLabel labelLbl = new JLabel("Annotator Label");
	private JLabel cmtLbl = new JLabel("Comments");

	private JButton cancelBtn = new JButton("Cancel");
	private JButton submitBtn = new JButton("Save");
	private JButton discardBtn = new JButton("Discard");
	private JButton saveBtn = new JButton("Save");

	private boolean isGlobal = true;;

	private final int CANCEL = 2;
	private final int SUBMIT = 3;
	private final int DISCARD = 4;
	private final int SAVE = 5;

	DefaultListModel globalParticipantsListModel = null;
	DefaultListModel localParticipantsListModel = null;

	public NewAnnotatorPanel(SS3Participant participant,
			ParticipantView participantView, AnnotatorsPanel annotatorPanel,
			JDialog parentDialog) {
		this.participant = participant;
		this.participantView = participantView;
		this.annotatorPanel = annotatorPanel;
		this.parentDialog = parentDialog;

		if (annotatorPanel != null) {
			this.isGlobal = !annotatorPanel.getGlobalParticipantsList()
					.isSelectionEmpty();
			globalParticipantsListModel = annotatorPanel
					.getGlobalParticipantsListModel();
			localParticipantsListModel = annotatorPanel
					.getLocalParticipantsListModel();

			for (int i = 0; i < globalParticipantsListModel.getSize(); i++) {
				SS3Participant gObj = (SS3Participant) globalParticipantsListModel
						.get(i);
				enteredAnnotatorNames.add(gObj.getFullName());
				enteredAnnotatorLabels.add(gObj.getLabel());
			}

			for (int j = 0; j < localParticipantsListModel.getSize(); j++) {
				SS3Participant lObj = (SS3Participant) localParticipantsListModel
						.get(j);
				if (!lObj.equals(participant)) {
					enteredAnnotatorNames.add(lObj.getFullName());
					enteredAnnotatorLabels.add(lObj.getLabel());
				}
			}
		} else {
			this.isGlobal = false;
		}

		cancelBtn.setActionCommand("" + CANCEL);
		submitBtn.setActionCommand("" + SUBMIT);
		discardBtn.setActionCommand("" + DISCARD);
		saveBtn.setActionCommand("" + SAVE);

		cancelBtn.addActionListener(this);
		submitBtn.addActionListener(this);
		discardBtn.addActionListener(this);
		saveBtn.addActionListener(this);

		submitBtn.setEnabled(false);

		if (participant != null) {
			nameTxt.setText(participant.getFullName());
			labelTxt.setText(participant.getLabel());
			commentsTa.setText(participant.getComments());
			submitBtn.setEnabled(true);
		}

		if (participant != null && isGlobal) {
			nameTxt.setEnabled(false);
			labelTxt.setEnabled(false);
		}

		UserInterfaceUtil.setTextFieldLNF(nameTxt);
		UserInterfaceUtil.setTextFieldLNF(labelTxt);
		UserInterfaceUtil.setTextAreaLNF(commentsTa);

		UserInterfaceUtil.setLabelLNF(nameLbl);
		UserInterfaceUtil.setLabelLNF(labelLbl);
		UserInterfaceUtil.setLabelLNF(cmtLbl);

		// to enable/disable buttons/fields based on full name field
		nameTxt.getDocument().addDocumentListener(
				new DocumentModelListener(nameTxt, submitBtn));

		JPanel panel = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		panel.setLayout(gbl);

		int width = labelTxt.getPreferredSize().width;

		gbc.insets = new Insets(0, 3, 0, 3);

		JScrollPane commentsScrollPane = new JScrollPane(commentsTa);
		commentsScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		commentsScrollPane.setPreferredSize(new Dimension(width, commentsTa
				.getPreferredSize().height * 3));

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 0, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(nameLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 1, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(nameTxt, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 2, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(labelLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 3, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(labelTxt, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 0, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(cmtLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 1, 1, 3,
				GridBagConstraints.BOTH);
		panel.add(commentsScrollPane, gbc);

		setLayout(new BorderLayout());
		add(panel, BorderLayout.CENTER);

		ButtonPanel bPanel = new ButtonPanel();
		bPanel.add(cancelBtn);
		bPanel.add(submitBtn);

		add(bPanel, BorderLayout.SOUTH);
	}

	public void update(SS3Participant participant,
			ParticipantView participantView,
			AnnotatorsPanel annotatorPanel) {
		this.participant = participant;
		this.annotatorPanel = annotatorPanel;
		this.participantView = participantView;

		if (annotatorPanel != null) {
			this.isGlobal = !annotatorPanel.getGlobalParticipantsList()
					.isSelectionEmpty();

			globalParticipantsListModel = annotatorPanel
					.getGlobalParticipantsListModel();
			localParticipantsListModel = annotatorPanel
					.getLocalParticipantsListModel();

			enteredAnnotatorNames.clear();
			enteredAnnotatorLabels.clear();

			for (int i = 0; i < globalParticipantsListModel.getSize(); i++) {
				SS3Participant gObj = (SS3Participant) globalParticipantsListModel
						.get(i);
				enteredAnnotatorNames.add(gObj.getFullName());
				enteredAnnotatorLabels.add(gObj.getLabel());
			}

			for (int j = 0; j < localParticipantsListModel.getSize(); j++) {
				SS3Participant lObj = (SS3Participant) localParticipantsListModel
						.get(j);
				if (!lObj.equals(participant)) {
					enteredAnnotatorNames.add(lObj.getFullName());
					enteredAnnotatorLabels.add(lObj.getLabel());
				}
			}
		} else {
			this.isGlobal = false;
		}

		submitBtn.setEnabled(false);

		if (participant != null) {
			nameTxt.setText(participant.getFullName());
			labelTxt.setText(participant.getLabel());
			commentsTa.setText(participant.getComments());
			submitBtn.setEnabled(true);
		}
		else {
			nameTxt.setText(null);
			labelTxt.setText(null);
			commentsTa.setText(null);
			submitBtn.setEnabled(true);
		}

		if (participant != null && isGlobal) {
			nameTxt.setEnabled(false);
			labelTxt.setEnabled(false);
		}
		repaint();
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case SUBMIT:
			enterEntry();
			break;
		case CANCEL:
			cancelEntry();
			break;
		case DISCARD:
			unsavedDataDialog.setVisible(false);
			parentDialog.setVisible(false);
			break;
		case SAVE:
			enterEntry();
			break;
		}
	}

	public void enterEntry() {
		String annotatorsName = nameTxt.getText().trim();
		String annotatorsLabel = labelTxt.getText().trim();

		if (annotatorsName.length() == 0) {

			StringBuffer bf = new StringBuffer();
			bf.append("<HTML>Warning:  You must specify a name for the <BR>");
			bf.append("annotator or else this information cannot be saved. <BR>");
			SS3Singleton.getSignStreamApplication()
					.confirmationDialog("Warning", bf.toString());
			
			return;
		} else if (participant == null) {
			// assign name to label field if label field is empty
			if (annotatorsLabel.length() == 0)
				annotatorsLabel = annotatorsName;
			
			// check if entered name or label is unique. Only for new annotators!
			if (enteredAnnotatorNames.contains(annotatorsName)
					|| enteredAnnotatorLabels.contains(annotatorsLabel)) {
				StringBuffer bf1 = new StringBuffer();
				bf1.append("<HTML>Warning:  There is already an annotator with this name. <BR>");
				bf1.append("If you still wish to create a new annotator, <BR>");
				bf1.append("please assign a unique name.</HTML>");
				SS3Singleton.getSignStreamApplication()
						.confirmationDialog("Warning", bf1.toString());
				return;
			}
		}
		
		// validate no separator characters
		if (annotatorsName.indexOf(Util.SEGMENT_TIER_SEPARATOR) != -1
				|| annotatorsLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) != -1) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.TEXT_CONTAINS_SEGMENT_TIER_SEPARATOR);
			return;
		}

		if (participant == null) {
			// create new annotator and add to the Annotator selection panel,
			// local annotators
			participant = new SS3Participant();
			participant.setId(Util.getUniqueNumber());
			participant.setJustCreated(true);
			participant.setFullName(annotatorsName);
			participant.setLabel(annotatorsLabel);
			participant.setComments(commentsTa.getText());
//			participantView.setParticipant(participant);

			annotatorPanel.getLocalParticipantsListModel().addElement(
					participant);
			annotatorPanel.getLocalParticipantsList().setSelectedValue(
					participant, true);
			annotatorPanel.getParticipantsPaneScrolledView()
					.getParticipantLibrary().addLocalParticipant(participant);

			ParticipantsLibrary library = annotatorPanel
					.getParticipantsPaneScrolledView().getParticipantLibrary();

			edu.bu.signstream.common.db.UtilityFilesXMLWriter writer = new edu.bu.signstream.common.db.UtilityFilesXMLWriter();
			try {
				writer.saveAnnotatorsLibrary(library);
			} catch (Exception ex) {
				ex.printStackTrace();
			}

		} else if (!isGlobal) {
			participant.setFullName(annotatorsName);
			participant.setLabel(annotatorsLabel);
			participant.setComments(commentsTa.getText());

			if (participantView != null) {
				participantView.setParticipant(participant);
			} else {
				annotatorPanel.getNameTxt().setText(annotatorsName);
				annotatorPanel.getLabelTxt().setText(annotatorsLabel);
				annotatorPanel.getCommentsTa().setText(commentsTa.getText());

				annotatorPanel.getLocalParticipantsList().setSelectedIndex(
						annotatorPanel.getLocalParticipantsList()
								.getSelectedIndex());
			}

		} else if (isGlobal) {
			participant.setComments(commentsTa.getText());
			annotatorPanel.getCommentsTa().setText(commentsTa.getText());

			annotatorPanel.getGlobalParticipantsList().setSelectedIndex(
					annotatorPanel.getGlobalParticipantsList()
							.getSelectedIndex());
		}

		parentDialog.setVisible(false);
	}

	private boolean isEdited() {
		if (participant == null) {
			return nameTxt.getText().length() > 0
					|| labelTxt.getText().length() > 0
					|| commentsTa.getText().length() > 0;
		} else {
			if (!participant.getFullName().equals(nameTxt.getText())) {
				return true;
			} else if (!participant.getLabel().equals(labelTxt.getText())) {
				return true;
			} else if (!commentsTa.getText().equals(participant.getComments())) {
				return true;
			}
		}
		return false;
	}
	
	public SS3Participant getParticipantJustCreated() {
		return participant;
	}

	private JDialog unsavedDataDialog = null;

	public void cancelEntry() {
		if (isEdited()) {
			if (unsavedDataDialog == null) {
				unsavedDataDialog = new JDialog(parentDialog, "Unsaved data",
						true);

				ButtonPanel buttonPanel = new ButtonPanel();
				buttonPanel.add(discardBtn);
				buttonPanel.add(saveBtn);

				JPanel centerPanel = new JPanel();
				centerPanel.setLayout(new GridLayout(1, 1));
				JLabel l1 = new JLabel(
						"Warning:  Unless you save this information");
				JLabel l2 = new JLabel(
						"before proceeding, it will be discarded.");
				centerPanel.add(l1);
				centerPanel.add(l2);

				JPanel panel = new JPanel();
				panel.setLayout(new BorderLayout());
				panel.add(centerPanel, BorderLayout.CENTER);
				panel.add(buttonPanel, BorderLayout.SOUTH);
				panel.add(new JLabel(" "), BorderLayout.WEST);

				Dimension d = panel.getPreferredSize();
				Dimension nD = new Dimension(d.width + 10, d.height + 40);
				panel.setPreferredSize(nD);

				Container contentPane = unsavedDataDialog.getContentPane();
				contentPane.setLayout(new BorderLayout(40, 40));
				contentPane.add(panel, BorderLayout.CENTER);

				unsavedDataDialog.setLocation(new Point(150, 150));
				unsavedDataDialog.pack();
			}
			unsavedDataDialog.setVisible(true);
		} else {
			parentDialog.setVisible(false);
		}
	}
}
