package edu.bu.signstream.ui.panels;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.newDatabase.ParticipantView;

public class ParticipantPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;

	private SS3Participant participant = null;
	private AnnotatorsPanel annotatorPanel = null;
	private ParticipantView participantView = null;

	private ArrayList<String> enteredParticipantNames = new ArrayList<String>();
	private ArrayList<String> enteredParticipantLabels = new ArrayList<String>();

	private boolean isGlobal = false;

	private JDialog parentDialog = null;

	private JLabel participantFullNameLbl = new JLabel("Full Name");
	private JLabel participantLabelLbl = new JLabel("Participant Label");
	private JLabel participantCommentLbl = new JLabel("Comments");
	private JLabel yearOfBirthlLbl = new JLabel("Year of birth");
	private JLabel genderLbl = new JLabel("Gender");
	private JLabel languageLbl = new JLabel("Native Language");
	private JLabel dHandLbl = new JLabel("Dominant Hand");
	private JLabel backgroundLbl = new JLabel("Background");
	private JLabel parentInfoLbl = new JLabel("Parent Info");

	private JTextField nameTxt = new JTextField(11);
	private JTextField labelTxt = new JTextField(11);
	private JTextField yearOfBirthTxt = new JTextField(11);
	private JTextField nativeLanguageTxt = new JTextField(11);

	private JTextArea commentsTa = new JTextArea();
	private JTextArea parentInfoTa = new JTextArea();
	private JTextArea backgroundTa = new JTextArea();

	private JRadioButton genderMaleBtn = new JRadioButton("M");
	private JRadioButton genderFemaleBtn = new JRadioButton("F");

	private JRadioButton dHandRightBtn = new JRadioButton("Right");
	private JRadioButton dHandLeftBtn = new JRadioButton("Left");

	private JButton cancelBtn = new JButton("Cancel");
	private JButton submitBtn = new JButton("Save");
	
	private JButton discardBtn = new JButton("Discard");
	private JButton saveBtn = new JButton("Save");

	private final int CANCEL = 0;
	private final int SUBMIT = 1;
	private final int DISCARD = 2;
	private final int SAVE = 3;
	
	private JDialog cancelEntryDialog = null;
	private DocumentModelListener documentModelListener = null;

	public ParticipantPanel(SS3Participant participant,
			ParticipantView participantView, AnnotatorsPanel annotatorPanel,
			JDialog parentDialog) {
		this.participant = participant;
		this.participantView = participantView;
		this.annotatorPanel = annotatorPanel;
		this.parentDialog = parentDialog;

		cancelBtn.setActionCommand("" + CANCEL);
		submitBtn.setActionCommand("" + SUBMIT);
		cancelBtn.addActionListener(this);
		submitBtn.addActionListener(this);

		ButtonGroup group1 = new ButtonGroup();
		group1.add(genderMaleBtn);
		group1.add(genderFemaleBtn);

		ButtonGroup group2 = new ButtonGroup();
		group2.add(dHandRightBtn);
		group2.add(dHandLeftBtn);

		initUI();
		
		documentModelListener = new DocumentModelListener(nameTxt, submitBtn);

		// to enable/disable buttons/fields based on full name field
		nameTxt.getDocument().addDocumentListener(documentModelListener);

		update(participant, participantView, annotatorPanel); //, participantView, annotatorPanel);
	}

	public void update(SS3Participant participant, ParticipantView participantView, AnnotatorsPanel annotatorPanel) { 
		// , ParticipantView participantView, AnnotatorsPanel annotatorPanel) {
		this.participant = participant;
		this.participantView = participantView;
		this.annotatorPanel = annotatorPanel;
		
		if (participant == null) {
			this.isGlobal = false;
			submitBtn.setEnabled(false);
		
			nameTxt.setText("");
			labelTxt.setText("");
			yearOfBirthTxt.setText("");
			nativeLanguageTxt.setText("");

			commentsTa.setText("");
			parentInfoTa.setText("");
			backgroundTa.setText("");

			genderMaleBtn.setSelected(false);
			genderFemaleBtn.setSelected(false);

			dHandRightBtn.setSelected(false);
			dHandLeftBtn.setSelected(false);
			
		} else {
			if (participantView != null) {
				this.isGlobal = false;
			} else {
				this.isGlobal = !annotatorPanel.getGlobalParticipantsList()
						.isSelectionEmpty();
			}

			submitBtn.setEnabled(true);

			nameTxt.setText(participant.getFullName());
			labelTxt.setText(participant.getLabel());
			yearOfBirthTxt.setText(participant.getYearOfBirth());
			nativeLanguageTxt.setText(participant.getNativeLanguage());
			commentsTa.setText(participant.getComments());
			parentInfoTa.setText(participant.getParentsInfo());
			backgroundTa.setText(participant.getBackground());

			if (participant.getGender() == 'M') {
				genderMaleBtn.setSelected(true);
			} else {
				genderFemaleBtn.setSelected(true);
			}

			if (participant.isDominantHandRight()) {
				dHandRightBtn.setSelected(true);
			} else {
				dHandLeftBtn.setSelected(true);
			}

			if (this.isGlobal) {
				nameTxt.setEnabled(false);
				labelTxt.setEnabled(false);
				yearOfBirthTxt.setEnabled(false);
				nativeLanguageTxt.setEnabled(false);
				parentInfoTa.setEnabled(false);
				backgroundTa.setEnabled(false);

				genderMaleBtn.setEnabled(false);
				genderFemaleBtn.setEnabled(false);
				dHandRightBtn.setEnabled(false);
				dHandLeftBtn.setEnabled(false);
			}
		}

		if (annotatorPanel != null) {
			DefaultListModel globalParticipantsListModel = annotatorPanel
					.getGlobalParticipantsListModel();
			DefaultListModel localParticipantsListModel = annotatorPanel
					.getLocalParticipantsListModel();

			for (int i = 0; i < globalParticipantsListModel.getSize(); i++) {
				SS3Participant gObj = (SS3Participant) globalParticipantsListModel
						.get(i);
				enteredParticipantNames.add(gObj.getFullName());
				enteredParticipantLabels.add(gObj.getLabel());
			}

			for (int j = 0; j < localParticipantsListModel.getSize(); j++) {
				SS3Participant lObj = (SS3Participant) localParticipantsListModel
						.get(j);
				if (!lObj.equals(participant)) {
					enteredParticipantNames.add(lObj.getFullName());
					enteredParticipantLabels.add(lObj.getLabel());
				}
			}
		}

		repaint();
	}

	private void initUI() {

		UserInterfaceUtil.setTextFieldLNF(nameTxt);
		UserInterfaceUtil.setTextFieldLNF(labelTxt);
		UserInterfaceUtil.setTextFieldLNF(yearOfBirthTxt);
		UserInterfaceUtil.setTextFieldLNF(nativeLanguageTxt);
		UserInterfaceUtil.setTextAreaLNF(commentsTa);
		UserInterfaceUtil.setTextAreaLNF(parentInfoTa);
		UserInterfaceUtil.setTextAreaLNF(backgroundTa);

		UserInterfaceUtil.setLabelLNF(participantFullNameLbl);
		UserInterfaceUtil.setLabelLNF(participantLabelLbl);
		UserInterfaceUtil.setLabelLNF(participantCommentLbl);
		UserInterfaceUtil.setLabelLNF(yearOfBirthlLbl);
		UserInterfaceUtil.setLabelLNF(genderLbl);
		UserInterfaceUtil.setLabelLNF(languageLbl);
		UserInterfaceUtil.setLabelLNF(dHandLbl);
		UserInterfaceUtil.setLabelLNF(backgroundLbl);
		UserInterfaceUtil.setLabelLNF(parentInfoLbl);

		UserInterfaceUtil.setRadioBtnLNF(genderMaleBtn);
		UserInterfaceUtil.setRadioBtnLNF(genderFemaleBtn);
		UserInterfaceUtil.setRadioBtnLNF(dHandRightBtn);
		UserInterfaceUtil.setRadioBtnLNF(dHandLeftBtn);

		JPanel panel = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		panel.setLayout(gbl);

		int width = labelTxt.getPreferredSize().width;

		JScrollPane commentsScrollPane = new JScrollPane(commentsTa);
		commentsScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		commentsScrollPane.setPreferredSize(new Dimension(width, 42));

		JScrollPane parentInfoScrollPane = new JScrollPane(parentInfoTa);
		parentInfoScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		parentInfoScrollPane.setPreferredSize(new Dimension(width, 42));

		JScrollPane backgroundInfoScrollPane = new JScrollPane(backgroundTa);
		backgroundInfoScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		backgroundInfoScrollPane.setPreferredSize(new Dimension(width, 42));

		gbc.insets = new Insets(0, 3, 0, 3);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 0, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(participantFullNameLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 0, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(participantCommentLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 1, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(nameTxt, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 1, 1, 3,
				GridBagConstraints.BOTH);
		panel.add(commentsScrollPane, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 2, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(participantLabelLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 3, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(labelTxt, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 4, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(yearOfBirthlLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 4, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(parentInfoLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 5, 1, 3,
				GridBagConstraints.BOTH);
		panel.add(parentInfoScrollPane, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 5, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(yearOfBirthTxt, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 6, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(genderLbl, gbc);

		JPanel genderPanel = new JPanel();
		genderPanel.setLayout(new GridLayout(1, 1, 0, 0));
		genderPanel.add(genderMaleBtn);
		genderPanel.add(genderFemaleBtn);

		genderPanel.setPreferredSize(yearOfBirthTxt.getPreferredSize());

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 7, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(genderPanel, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 8, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(languageLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 9, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(nativeLanguageTxt, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 10, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(dHandLbl, gbc);

		JPanel dHandPanel = new JPanel();
		dHandPanel.setLayout(new GridLayout(1, 1));
		dHandPanel.add(dHandRightBtn);
		dHandPanel.add(dHandLeftBtn);

		UserInterfaceUtil.setGridBagConstraints(gbc, 0, 12, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(dHandPanel, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 8, 1, 1,
				GridBagConstraints.BOTH);
		panel.add(backgroundLbl, gbc);

		UserInterfaceUtil.setGridBagConstraints(gbc, 1, 9, 1, 4,
				GridBagConstraints.BOTH);
		panel.add(backgroundInfoScrollPane, gbc);

		setLayout(new BorderLayout());
		add(panel, BorderLayout.CENTER);

		ButtonPanel bPanel = new ButtonPanel();
		bPanel.add(cancelBtn);
		bPanel.add(submitBtn);

		add(bPanel, BorderLayout.SOUTH);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case CANCEL:
			if (isEdited()) {
				showCancelEntryDialog();
			} else {
				parentDialog.setVisible(false);
			}
			break;
		case SUBMIT:
			enterEntry();
			break;
		case DISCARD:
			parentDialog.setVisible(false);
			break;
		case SAVE:
			enterEntry();
			break;
		}
	}

	private void enterEntry() {
		String participantName = nameTxt.getText().trim();
		String participantLabel = labelTxt.getText().trim();
		String comments = commentsTa.getText().trim();

		// assign name to label field if label field is empty
		if (participantLabel.length() == 0)
			participantLabel = participantName;

		if (participantName.length() == 0) {
			StringBuffer bf = new StringBuffer();
			bf.append("<HTML>Warning:  You must specify a name for the participant <BR>");
			bf.append("or else this information cannot be saved. </HTML>");
			SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", bf.toString());
			//bf.append("Do you want to discard what has been entered or resume editing?");
			//discard(bf.toString());

		} else if (participant == null) {
			// check if entered name is unique. Only for new participant!
			if (enteredParticipantNames.contains(participantName)) {
				StringBuffer bf1 = new StringBuffer();
				bf1.append("<HTML>Warning:  There is already a participant with this name. <BR>");
				bf1.append("If you still wish to create a new participant, <BR>");
				bf1.append("please assign a unique name.</HTML>");
				SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", bf1.toString());
				//discard(bf1.toString());
			} else

			// check if entered label is unique
			if (enteredParticipantLabels.contains(participantLabel)) {
				StringBuffer bf2 = new StringBuffer();
				bf2.append("<HTML>Warning:  There is already a participant with this label. <BR>");
				bf2.append("If you still wish to create a new participant, <BR>");
				bf2.append("please assign a unique label.</HTML>");
				SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", bf2.toString());
				//discard(bf2.toString());
			}
		}
		
		// validate no separator characters
		if (participantName.indexOf(Util.SEGMENT_TIER_SEPARATOR) != -1
				|| participantLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) != -1) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.TEXT_CONTAINS_SEGMENT_TIER_SEPARATOR);
			return;
		}
		
		// validate data
		boolean isValidEntry = true;
		StringBuffer validationWarnings = new StringBuffer("<HTML>Warning: ");

		if (!genderMaleBtn.isSelected() && !genderFemaleBtn.isSelected()) {
			isValidEntry = false;
			validationWarnings
					.append("Please select participant's gender. <BR>");
		}

		if (!dHandRightBtn.isSelected() && !dHandLeftBtn.isSelected()) {
			isValidEntry = false;
			validationWarnings
					.append("Please select participant's dominant hand. <BR>");
		}

		if (!isValidEntry) {
			validationWarnings.append("</HTML>");
			SS3Singleton
					.getSignStreamApplication()
					.confirmationDialog("Adding new participant",
							validationWarnings.toString());
			return;
		}

		if (participant == null) {
			// create new participant and add to the Annotator selection panel,
			participant = new SS3Participant();
			participant.setId(Util.getUniqueNumber());
			participant.setJustCreated(true);
			participant.setFullName(participantName);
			participant.setLabel(participantLabel);
			participant.setYearOfBirth(yearOfBirthTxt.getText());
			participant.setNativeLanguage(nativeLanguageTxt.getText());
			participant.setComments(comments);
			participant.setParentsInfo(parentInfoTa.getText());
			participant.setBackground(backgroundTa.getText());

			if (genderMaleBtn.isSelected()) {
				participant.setGenderMale();
			}

			if (genderFemaleBtn.isSelected()) {
				participant.setGenderFemale();
			}

			if (dHandRightBtn.isSelected()) {
				participant.setDominantHand("R");
			} else {
				participant.setDominantHand("L");
			}

			if (genderMaleBtn.isSelected()) {
				participant.setGenderMale();
			} else if (genderFemaleBtn.isSelected()) {
				participant.setGenderFemale();
			}

			if (dHandRightBtn.isSelected()) {
				participant.setDominantHand("R");
			} else if (dHandLeftBtn.isSelected()) {
				participant.setDominantHand("L");
			}

			if (participantView != null) {
				participantView.setParticipant(participant);

			} else {
				annotatorPanel.getLocalParticipantsListModel().addElement(
						participant);
				annotatorPanel.getLocalParticipantsList().setSelectedValue(
						participant, true);
				annotatorPanel.getParticipantsPaneScrolledView()
						.getParticipantLibrary()
						.addLocalParticipant(participant);

				annotatorPanel.getNameTxt().setText(participantName);
				annotatorPanel.getLabelTxt().setText(participantLabel);
				annotatorPanel.getCommentsTa().setText(comments);

				ParticipantsLibrary library = annotatorPanel
						.getParticipantsPaneScrolledView()
						.getParticipantLibrary();

				edu.bu.signstream.common.db.UtilityFilesXMLWriter writer = new edu.bu.signstream.common.db.UtilityFilesXMLWriter();

				// if(participantView.isParticipant()) {
				try {
					writer.saveParticipantsLibrary(library);
				} catch (Exception ex) {
					ex.printStackTrace();
				}
				// } else {
				// try {
				// writer.saveAnnotatorsLibrary(library);
				// } catch (Exception ex) {
				// ex.printStackTrace();
				// }
				// }
			}
		} else if (!isGlobal) {
			participant.setFullName(participantName);
			participant.setLabel(participantLabel);
			participant.setYearOfBirth(yearOfBirthTxt.getText());
			// String language = nativeLanguageTxt.getText();
			participant.setNativeLanguage(nativeLanguageTxt.getText());
			participant.setComments(comments);
			participant.setParentsInfo(parentInfoTa.getText());
			participant.setBackground(backgroundTa.getText());

			if (genderMaleBtn.isSelected()) {
				participant.setGenderMale();
			}

			if (genderFemaleBtn.isSelected()) {
				participant.setGenderFemale();
			}

			if (dHandRightBtn.isSelected()) {
				participant.setDominantHand("R");
			} else {
				participant.setDominantHand("L");
			}

			if (genderMaleBtn.isSelected()) {
				participant.setGenderMale();
			} else if (genderFemaleBtn.isSelected()) {
				participant.setGenderFemale();
			}

			if (dHandRightBtn.isSelected()) {
				participant.setDominantHand("R");
			} else if (dHandLeftBtn.isSelected()) {
				participant.setDominantHand("L");
			}

			if (participantView != null) {
				participant.setFullName(nameTxt.getText());
				participant.setLabel(labelTxt.getText());
				participant.setYearOfBirth(yearOfBirthTxt.getText());
				participant.setNativeLanguage(nativeLanguageTxt.getText());
				participant.setComments(commentsTa.getText());
				participant.setParentsInfo(parentInfoTa.getText());
				participant.setBackground(backgroundTa.getText());

				if (genderMaleBtn.isSelected()) {
					participant.setGenderMale();
				} else {
					participant.setGenderFemale();
				}

				if (dHandRightBtn.isSelected()) {
					participant.setDominantHand("R");
				} else {
					participant.setDominantHand("L");
				}

				participantView.setParticipant(participant);

				edu.bu.signstream.common.db.UtilityFilesXMLWriter writer = new edu.bu.signstream.common.db.UtilityFilesXMLWriter();

				if (participantView.isParticipant()) {
					try {
						writer.saveParticipantsLibrary(participantView
								.getParticipantsLibrary());
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				} else {
					try {
						writer.saveAnnotatorsLibrary(participantView
								.getAnnotatorsLibrary());
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			} else {
				annotatorPanel.getLocalParticipantsList().setSelectedIndex(
						annotatorPanel.getLocalParticipantsList()
								.getSelectedIndex());

				annotatorPanel.getNameTxt().setText(participantName);
				annotatorPanel.getLabelTxt().setText(participantLabel);
				annotatorPanel.getCommentsTa().setText(comments);

				annotatorPanel.repaint();
				annotatorPanel.updateUI();
			}

		} else if (isGlobal) {
			participant.setComments(commentsTa.getText());
			annotatorPanel.getCommentsTa().setText(comments);

			annotatorPanel.getGlobalParticipantsList().setSelectedIndex(
					annotatorPanel.getGlobalParticipantsList()
							.getSelectedIndex());

			annotatorPanel.getNameTxt().setText(participantName);
			annotatorPanel.getLabelTxt().setText(participantLabel);
			annotatorPanel.getCommentsTa().setText(comments);
		}
		parentDialog.setVisible(false);
	}
             
//	private void discard(String msg) {
//		final JDialog dialog = new JDialog(parentDialog, "Discard Resource",
//				true);
//		JButton cancelBtn = new JButton("Resume Editing");
//		JButton discardBtn = new JButton("Discard");
//
//		discardBtn.addActionListener(new ActionListener() {
//
//			public void actionPerformed(ActionEvent arg0) {
//				parentDialog.dispose();
//				dialog.dispose();
//			}
//
//		});
//
//		cancelBtn.addActionListener(new ActionListener() {
//
//			public void actionPerformed(ActionEvent arg0) {
//				dialog.dispose();
//			}
//		});
//
//		ButtonPanel buttonPanel = new ButtonPanel();
//		buttonPanel.add(cancelBtn);
//		buttonPanel.add(discardBtn);
//
//		JPanel panel = new JPanel();
//		panel.setLayout(new BorderLayout());
//		JLabel label = new JLabel(msg);
//		panel.add(label, BorderLayout.CENTER);
//		panel.add(buttonPanel, BorderLayout.SOUTH);
//
//		panel.add(new JLabel("  "), BorderLayout.WEST);
//
//		Dimension d = panel.getPreferredSize();
//		Dimension nD = new Dimension(d.width + 10, d.height + 40);
//		panel.setPreferredSize(nD);
//
//		Container contentPane = dialog.getContentPane();
//		contentPane.setLayout(new BorderLayout(40, 40));
//		contentPane.add(panel, BorderLayout.CENTER);
//
//		dialog.setLocation(new Point(150, 150));
//		dialog.pack();
//		dialog.setVisible(true);
//	}

	public boolean isEdited() {
		if (participant == null) {
			if (nameTxt.getText().length() > 0)
				return true;

			if (labelTxt.getText().length() > 0)
				return true;

			if (yearOfBirthTxt.getText().length() > 0)
				return true;

			if (nativeLanguageTxt.getText().length() > 0)
				return true;

			if (commentsTa.getText().length() > 0)
				return true;

			if (parentInfoTa.getText().length() > 0)
				return true;

			if (backgroundTa.getText().length() > 0)
				return true;
		} else {
			if (!nameTxt.getText().equals(participant.getFullName()))
				return true;

			if (!labelTxt.getText().equals(participant.getLabel()))
				return true;

			if (!commentsTa.getText().equals(participant.getComments()))
				return true;

			if (!yearOfBirthTxt.getText().equals(participant.getYearOfBirth()))
				return true;

			if (!nativeLanguageTxt.getText().equals(
					participant.getNativeLanguage()))
				return true;

			if (!parentInfoTa.getText().equals(participant.getParentsInfo()))
				return true;

			if (!parentInfoTa.getText().equals(participant.getBackground()))
				return true;
		}
		return false;
	}

	public void showCancelEntryDialog() {
		if (isEdited()) {
			if (cancelEntryDialog != null) {
				
				cancelEntryDialog = new JDialog();

				discardBtn.setActionCommand("" + DISCARD);
				saveBtn.setActionCommand("" + SAVE);
				discardBtn.addActionListener(this);
				saveBtn.addActionListener(this);

				ButtonPanel buttonPanel = new ButtonPanel();
				buttonPanel.add(cancelBtn);
				buttonPanel.add(saveBtn);

				JPanel centerPanel = new JPanel();
				centerPanel.setLayout(new GridLayout(1, 1));
				JLabel l1 = new JLabel(
						"Warning:  Unless you save this information");
				JLabel l2 = new JLabel(
						"before proceeding, it will be discarded.");
				centerPanel.add(l1);
				centerPanel.add(l2);

				JPanel panel = new JPanel();
				panel.setLayout(new BorderLayout());
				panel.add(centerPanel, BorderLayout.CENTER);
				panel.add(buttonPanel, BorderLayout.SOUTH);
				panel.add(new JLabel(" "), BorderLayout.WEST);

				Dimension d = panel.getPreferredSize();
				Dimension nD = new Dimension(d.width + 10, d.height + 40);
				panel.setPreferredSize(nD);

				Container contentPane = cancelEntryDialog.getContentPane();
				contentPane.setLayout(new BorderLayout(40, 40));
				contentPane.add(panel, BorderLayout.CENTER);

				cancelEntryDialog.setLocation(new Point(150, 150));
				cancelEntryDialog.pack();
			}

			cancelEntryDialog.setVisible(true);
		} else {
			parentDialog.setVisible(false);
		}

	}
}
