package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.jdesktop.swingx.JXTreeTable;
import org.jdesktop.swingx.treetable.AbstractMutableTreeTableNode;
import org.jdesktop.swingx.treetable.DefaultTreeTableModel;
import org.jdesktop.swingx.treetable.MutableTreeTableNode;
import org.jdesktop.swingx.treetable.TreeTableNode;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.FieldsGroup;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.NonmanualFieldsView;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class CSFieldPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JDialog parent = null;

	private SignStreamSegmentPanel segmentPanel;
	private Segment segment;
	private PresentationField parentField;
	private NonmanualFieldsView nonManualView;
	private SegmentGraphicRepresentation rep;
	private CodingSchemaFieldNode fieldsRootNode;
	private JXTreeTable fieldsTable;
	
	private JDialog editFieldDialog = null; 
	private AddOrUpdateField editFieldPanel = null;

	private SelectionListener selectionListener = null;
	private MutableTreeTableNode selectedNode = null;

	private JButton newFieldBtn = new JButton("New Field");
	private JButton deleteBtn = new JButton("Delete");
	private JButton editBtn = new JButton("Edit");
	private JButton cancelBtn = new JButton("Cancel");
	private JButton doneBtn = new JButton("Done");

	private final int NEW_FIELD = 8;
	private final int DELETE = 9;
	private final int EDIT = 10;
	private final int CANCEL = 11;
	private final int DONE = 12;

	private GridBagConstraints gbc = new GridBagConstraints();

	// field ids that might have been used to code data
	private ArrayList<String> fieldIDs = new ArrayList<String>();
	
	// fields recently created and added to the coding schema
	private ArrayList<SS3Field> recentlyAddedFields = new ArrayList<SS3Field>();
	
	private JDialog newFieldDialog = null; 
	private AddOrUpdateField newFieldPanel = null;

	public CSFieldPanel(SignStreamSegmentPanel segmentPanel, Segment segment, NonmanualFieldsView nonManualView, JDialog dialog,
			PresentationField parentField) {
		this.segmentPanel = segmentPanel;
		this.nonManualView = nonManualView;
		this.segment = segment;
		this.parent = dialog;
		this.parentField = parentField;

		deleteBtn.setEnabled(false);
		editBtn.setEnabled(false);

		newFieldBtn.setActionCommand("" + NEW_FIELD);
		newFieldBtn.addActionListener(this);
		deleteBtn.setActionCommand("" + DELETE);
		deleteBtn.addActionListener(this);
		editBtn.setActionCommand("" + EDIT);
		editBtn.addActionListener(this);
		cancelBtn.setActionCommand("" + CANCEL);
		cancelBtn.addActionListener(this);
		doneBtn.setActionCommand("" + DONE);
		doneBtn.addActionListener(this);

		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		selectionListener = new SelectionListener(this, scheme, deleteBtn, editBtn);

		JScrollPane scrollPane = getTreeTableScrollPanel();
		Dimension d = new Dimension(scrollPane.getPreferredSize().width + 260, 250);

		scrollPane.setPreferredSize(d);
		scrollPane.setMinimumSize(d);
		scrollPane.setMaximumSize(d);

		setLayout(new GridBagLayout());

		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(1, 1, 1, 1);

		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 9;
		add(scrollPane, gbc);
		gbc.gridwidth = 1;

		gbc.gridx = 10;
		add(newFieldBtn, gbc);

		gbc.gridx = 0;
		gbc.gridy = 14;
		add(deleteBtn, gbc);

		gbc.gridx = 1;
		add(editBtn, gbc);
		gbc.gridx = 2;
		add(Box.createHorizontalStrut(365), gbc);
		gbc.gridx = 8;
		add(cancelBtn, gbc);
		gbc.gridx = 10;
		add(doneBtn, gbc);
	}

	public void setSelectedTreeTableNode(MutableTreeTableNode selectedNode) {
		this.selectedNode = selectedNode;

		deleteBtn.setEnabled(false);
		editBtn.setEnabled(false);

		if (selectedNode != null && selectedNode.isLeaf()) {
			if (fieldsRootNode.equals(selectedNode.getParent())) {
				cancelBtn.setEnabled(true);
				doneBtn.setEnabled(true);
			} else {
				boolean doNext = true;
				while (doNext) {
					TreeTableNode pNode = selectedNode.getParent();
					if (fieldsRootNode.equals(pNode.getParent())) {
						doNext = false;
						int selectedGroup = getGroup(pNode);
						if (selectedGroup != 4) {
							editBtn.setEnabled(true);
						}
					}
				}
			}
		}
	}

	public boolean isFieldNameAndLabelUnique(String fieldName,
			String fieldLabel, int groupID) {
		MutableTreeTableNode node = groupID >= 0 && groupID < 4 ? (MutableTreeTableNode) fieldsRootNode
				.getChildAt(groupID)
				: fieldsRootNode;

		int numberOfChildren = node.getChildCount();
		for (int i = 0; i < numberOfChildren; i++) {
			MutableTreeTableNode childNode = (MutableTreeTableNode) node
					.getChildAt(i);
			String existingFieldName = (String) childNode.getValueAt(0);
			String existingFieldLabel = (String) childNode.getValueAt(1);

			if (existingFieldName.equals(fieldName)
					|| existingFieldLabel.equals(fieldLabel)) {
				return false;
			}
		}
		return true;
	}
	
	public void addNewField(SS3Field field) {
		recentlyAddedFields.add(field);
		
		segmentPanel.getSS3Database().getLocalSS3CodingScheme().addField(field);

		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		scheme.addField(field);

		int groupID = field.getGroupID();

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable
				.getTreeTableModel();
		MutableTreeTableNode node = groupID >= 0 && groupID < 4 ? (MutableTreeTableNode) fieldsRootNode
				.getChildAt(groupID) : fieldsRootNode;
				
		Object[] newFieldObject = new Object[] { field.getName(), field.getLabel(),	new Boolean(true), " ", field.getColor(), "" + field.getId() }; 
		CodingSchemaFieldNode newFieldNode = new CodingSchemaFieldNode(newFieldObject);	
		fieldsModel.insertNodeInto(newFieldNode, node, node.getChildCount());

		scrollToAndSelectNode(newFieldNode);
	} 
	
	public void updateField(SS3Field field) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		
		// check if field is still in the same group
		int legacyGroupID = getGroup(selectedNode.getParent());
		int groupID = field.getGroupID();
		
		segmentPanel.getSS3Database().getLocalSS3CodingScheme().addField(field);
		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		scheme.addField(field);
		
		Object[] newFieldObject = new Object[] { field.getName(), field.getLabel(),	new Boolean(true), " ", field.getColor(), "" + field.getId() }; 
		fieldsModel.setUserObject(selectedNode, newFieldObject);			 
		
		if(legacyGroupID != groupID) {
			fieldsModel.removeNodeFromParent(selectedNode);			
			addNewField(field);
		}
	}
			
	private void createNewFieldDialog() {
		if(newFieldDialog == null) {
			newFieldDialog = new JDialog(parent, "Field Specifications Editor", true);
		}			

		newFieldPanel = new AddOrUpdateField(segmentPanel, segment, 
				getGroup(selectedNode), null, this, newFieldDialog, parentField);
		newFieldPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

		newFieldDialog.pack();

		Container newFieldContentPane = newFieldDialog.getContentPane();
		newFieldContentPane.setLayout(new BorderLayout());
		newFieldContentPane.add(newFieldPanel, BorderLayout.CENTER);

		newFieldDialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent we) {
				newFieldDialog.setVisible(false);
			}
		});

		newFieldDialog.setSize(newFieldDialog.getPreferredSize());
		newFieldDialog.setLocation(150, 150);
		newFieldDialog.setVisible(true);
	}	

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();
		
		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		switch (Integer.parseInt(str)) {
		case NEW_FIELD:			
			createNewFieldDialog();
			break;

		case EDIT:

			SS3Field fieldToEdit = null;
			if (selectedNode != null) {
				String id = (String) selectedNode.getValueAt(5);
				fieldToEdit = scheme.getField(id);
			}

			if(editFieldDialog == null) {
				editFieldDialog = new JDialog(parent,
					"Field Specifications Editor", true);
			}
			
			editFieldPanel = new AddOrUpdateField(
					segmentPanel, segment, getGroup(selectedNode.getParent()),
					fieldToEdit, this, editFieldDialog, parentField);			
			editFieldPanel.setBorder(BorderFactory.createEmptyBorder(15, 15,
					15, 15));

			editFieldDialog.pack();			
			editFieldDialog.setLayout(new BorderLayout());
			editFieldDialog.add(editFieldPanel, BorderLayout.CENTER);
			
			editFieldDialog.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosing(WindowEvent we) {
					if(editFieldPanel != null) {
						editFieldPanel.discard();
						editFieldPanel = null;
					}
					editFieldDialog.dispose();
				}
			});

			editFieldDialog.setSize(editFieldDialog.getPreferredSize());
			editFieldDialog.setLocation(150, 150);
			editFieldDialog.setVisible(true);

			break;

		case CANCEL:
			parent.dispose();
			break;
		case DONE:
			enter();
            break;
		}
	}
	
	public void enter() {
        ArrayList<String> slctFieldIDs = getSelectedFieldIDsNUpdateCodingSchema(); 

        LabelObject object = segment.getRootLabelObject().getNonManuelLabelObject(); 
        
        ArrayList<LabelObject> children = object.getAllChildren();  
        
        // make all not visible
        for (LabelObject child : children) {
        	child.setVisible(false);
		}

        // set to true only those fields that should be visible
        for (LabelObject child : children) {
            String id = child.getFieldID();

            if (containsID(slctFieldIDs, id)) {
               slctFieldIDs.remove(id);
               child.setVisible(true);
            }
        }
        
        if (segmentPanel.getUtteranceView() != null) {
            // for each segment currently in view in UtteranceView, delete empty entities from its nonmanual fields
            ArrayList<Segment> segmentsList = segmentPanel.getUtteranceView().getSegments();
            for (Segment uvSegment: segmentsList) {
            		LabelObject uvObject = uvSegment.getRootLabelObject().getNonManuelLabelObject();
            		ArrayList<LabelObject> uvChildren = uvObject.getAllChildren();
                    
                // delete empty entities from nonmanual fields
                for (LabelObject uvChild: uvChildren) {
                		FieldWrapper wrapper = uvChild.getWrapper();
                		wrapper.getField().deleteEmptyEntities();
                }
            }
        }
        
        nonManualView.update(segment);
        nonManualView.getLabelView().loadWrappers();
        nonManualView.getLabelView().updateUI();
        nonManualView.getLabelView().repaint();
        
        parent.dispose();
	}
	
	private boolean containsID(ArrayList<String> slctFieldIDs, String id) {
		for (int i = 0; i < slctFieldIDs.size(); i++) {
			String fID = slctFieldIDs.get(i);
		    id.trim();
		    fID.trim();     
		    if (fID.equals(id))
		    	return true;
		}
		return false;
	}

	public JScrollPane getTreeTableScrollPanel() {
		populateFieldsTable();

		String[] cNames = { " Field  Names   ", "Field Labels", "Display", "Data",
				"Color", " " };

		DefaultTreeTableModel fieldsModel = new DefaultTreeTableModel(
				fieldsRootNode, Arrays.asList(cNames)) {
			@Override
			public Class<?> getColumnClass(int column) {

				if (column == 2) {
					return Boolean.class;
				} else if (column == 4) {
					return Color.class;
				} else {
					return String.class;
				}
			}

			@Override
			public boolean isCellEditable(Object node, int column) {
				return column == 4 || column == 2;
			}
			
			@Override
			public int getChildCount(Object parent) {
				//Only override count if at the root of the tree.
				if (parent == fieldsRootNode) {
					CodingSchemaFieldNode root = (CodingSchemaFieldNode) parent;
					int children = 0;
					for (int i=0;i<root.getChildCount();i++) {
						TreeTableNode child = root.getChildAt(i);
						//Nonmanual Fields contain an ArrayList of child IDs in their 5th column
						if (child.getValueAt(5) instanceof ArrayList) 
							children++;
					}
					return children;
				}
				else return super.getChildCount(parent);
			}
		};

		fieldsTable = new JXTreeTable(fieldsModel) {
			private static final long serialVersionUID = 1L;

			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
				if (column == 4) {
					return new ColorRenderer(true);
				}
				// else...
				return super.getCellRenderer(row, column);
			}
		};

		TableColumn customCol = fieldsTable.getColumnModel().getColumn(4);
		customCol.setCellEditor(new ColorEditor());

		fieldsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		fieldsTable.setHorizontalScrollEnabled(true);
		fieldsTable.setFillsViewportHeight(false);
		fieldsTable.setRootVisible(false);

		// do not show field id column
		fieldsTable.getColumnModel().removeColumn(
				fieldsTable.getColumnModel().getColumn(5));
		fieldsTable.setAutoCreateColumnsFromModel(false);
		
		UIDefaults uiDefs = UIManager.getDefaults();
		uiDefs.put("Tree.leftChildIndent", new Integer(3));

		fieldsTable.setOpenIcon(null);
		fieldsTable.setClosedIcon(null);
		fieldsTable.setLeafIcon(null);

		fieldsTable.setRowHeight(SegmentGraphReprUtil.treeTableCellHeight);
		fieldsTable.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);

		fieldsTable.addTreeSelectionListener(selectionListener);

		fieldsTable.packAll();

		JScrollPane jsp = new JScrollPane(fieldsTable);
		jsp.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		return jsp;
	}
	

	public void updateField(SS3Field field, int groupIndex,
			MutableTreeTableNode selectedNode) {
	}

	public int getGroup(TreeTableNode node) {
		if (node == null)
			return 4;

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable
				.getTreeTableModel();
		TreeTableNode rootNode = fieldsModel.getRoot();
		TreeTableNode headNode = rootNode.getChildAt(0);
		TreeTableNode faceNode = rootNode.getChildAt(1);
		TreeTableNode bodyNode = rootNode.getChildAt(2);
		TreeTableNode gramNode = rootNode.getChildAt(3);

		if (node.equals(headNode))
			return 0;
		if (node.equals(faceNode))
			return 1;
		if (node.equals(bodyNode))
			return 2;
		if (node.equals(gramNode))
			return 3;
		return 4;
	}

	
	private class CodingSchemaFieldNode extends AbstractMutableTreeTableNode {
		public CodingSchemaFieldNode(Object[] data) {
			super(data);
			if (data == null) {
				throw new IllegalArgumentException("Node data cannot be null");
			}
		}

		@Override
		public int getColumnCount() {
			return getData().length;
		}

		@Override
		public Object getValueAt(int column) {
			return getData()[column];
		}

		public Object[] getData() {
			return (Object[]) getUserObject();
		}

		private boolean isGroup(TreeTableNode node) {
			DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable
					.getTreeTableModel();
			TreeTableNode rootNode = fieldsModel.getRoot();
			TreeTableNode headNode = rootNode.getChildAt(0);
			TreeTableNode faceNode = rootNode.getChildAt(1);
			TreeTableNode bodyNode = rootNode.getChildAt(2);
			TreeTableNode gramNode = rootNode.getChildAt(3);

			return node.equals(headNode) || node.equals(faceNode)
					|| node.equals(bodyNode) || node.equals(gramNode);
		}

		@Override
		public void setValueAt(Object value, int column) {
			super.setValueAt(value, column);
			if (column == 2) {
				Boolean isChecked = (Boolean) value;
				getData()[column] = isChecked;

				// for head, face, body, and grammatical nodes - select/unselect
				// all children if group is selected/unselected
				if (isGroup(this)) {
					int childCount = getChildCount();
					for (int i = 0; i < childCount; i++) {
						TreeTableNode node = getChildAt(i);
						if (node != null)
							node.setValueAt(value, column);
					}
				} else if (!isChecked) {
					// if parent node is unselected, all children nodes should
					// be unselected as well
					int childCount = getChildCount();
					for (int i = 0; i < childCount; i++) {
						TreeTableNode node = getChildAt(i);
						node.setValueAt(value, column);
					}
				} else {
					DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable
							.getTreeTableModel();
					TreeTableNode rootNode = fieldsModel.getRoot();
					TreeTableNode parentNode = this;
					while (!parentNode.getParent().equals(rootNode)) {
						parentNode = parentNode.getParent();
					}

					if (parentNode != this && !isGroup(parentNode)) {
						getParent().setValueAt(value, column);
					}
				}
			} else if (column == 4) {
				getData()[column] = value;
			}
		}
	}

	private HashMap<String, CodingSchemaFieldNode> getFieldsList(
			Collection<SS3Field> fieldsToDisplay,
			SegmentGraphicRepresentation rep) {

		HashMap<String, SS3Field> fields = new HashMap<String, SS3Field>();
		Iterator<SS3Field> iter = fieldsToDisplay.iterator();
		while (iter.hasNext()) {
			SS3Field csField = iter.next();
			fields.put(csField.getId(), csField);
		}

		LabelObject object = segment.getRootLabelObject()
				.getNonManuelLabelObject();
		ArrayList<LabelObject> leafsOnly = object.getAllChildren();
		ArrayList<String> available = new ArrayList<String>();
		ArrayList<String> visible = new ArrayList<String>();

		Hashtable<String, Color> colors = new Hashtable<String, Color>();
		HashMap<String, CodingSchemaFieldNode> nodes = new HashMap<String, CodingSchemaFieldNode>();

		for (int i = 0; i < leafsOnly.size(); i++) {
			LabelObject obj = leafsOnly.get(i);
			FieldWrapper wrapper = obj.getWrapper();
			String id = wrapper.getField().getFieldID();

			Color color = wrapper.getField().getColor();
			if (color != null)
				colors.put(id, color);

			if (obj.isVisible())
				visible.add(id);

			if (!available.contains(id)
					&& wrapper.getField().getFieldSize() > 0)
				available.add(id);
		}

		if (fields != null) {
			Set<String> csKeys = fields.keySet();
			Object[] sortedKeys = Sorter.sortIntegerSet(csKeys);

			for (int i = 0; i < sortedKeys.length; i++) {
				SS3Field csField = fields
						.get(sortedKeys[i]);

				String fieldID = "" + csField.getId();

				Boolean checked = visible.contains(fieldID) ? new Boolean(true)
						: new Boolean(false);

				String dt = available.contains(fieldID) ? "  *  " : "";

				Color color = csField.getColor();
				Object col = colors.get(fieldID);

				if (col != null)
					color = (Color) col;

				Object[] values = new Object[] { csField.getName(),
						csField.getLabel(), checked, dt, color, fieldID };

				CodingSchemaFieldNode node = new CodingSchemaFieldNode(values);
				nodes.put(fieldID, node);
			}
		}
		return nodes;
	}

	private void populateFieldsTable() {
		fieldsRootNode = new CodingSchemaFieldNode(new Object[] { "Field Name",
				"Field Label", "true", "Data", "Color.BLUE", "" });

		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		HashMap<String, SS3Field> fields = scheme.getDependentRootFields();
		HashMap<String, CodingSchemaFieldNode> nmNodes = getFieldsList(scheme
				.getFields().values(), rep);
		//
		scheme.loadFieldsGroups();
		//
		ArrayList<FieldsGroup> groups = scheme.getFieldsGroups();

		for (int i = 0; i < groups.size(); i++) {
			FieldsGroup group = groups.get(i);
			boolean checked = false;
			String dt = "";
			ArrayList<CodingSchemaFieldNode> nodes = new ArrayList<CodingSchemaFieldNode>();
			ArrayList<String> ids = group.getFieldsID();

			for (String id : ids) {
				//
				fieldIDs.add(id);
				//
				CodingSchemaFieldNode node = nmNodes.get(id);
				if (node != null) {
					Object[] values = (Object[]) node.getUserObject();
					Boolean checked1 = (Boolean) values[2];
					String dt1 = (String) values[3];

					if (checked1.booleanValue()) {
						checked = true;
					}

					if (dt1.trim().length() > 0) {
						dt = dt1;
					}

					nodes.add(node);
				}
			}
			
			Color color = Color.BLACK;
			
			Object[] value = new Object[] { group.getGroupName(), "", checked,
					dt, Color.BLACK, group.getFieldsID() };
			CodingSchemaFieldNode gNode = new CodingSchemaFieldNode(value);
			fieldsRootNode.add(gNode);

			for (CodingSchemaFieldNode node1 : nodes) {
				gNode.add(node1);
			}
		}

		if (fields != null) {
			Set<String> csKeys = fields.keySet();
			Object[] sortedKeys = Sorter.sortIntegerSet(csKeys);

			for (int i = 0; i < sortedKeys.length; i++) {
				SS3Field childField = fields
						.get(sortedKeys[i]);
				addChild(fieldsRootNode, childField);
			}
		}
	}

	private void addChild(CodingSchemaFieldNode parent, SS3Field childField) {
		CodingSchemaFieldNode child = createFieldNode(childField);
		parent.add(child);
		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		ArrayList<SS3Field> depFields = scheme.getDependentFields(childField
				.getId());

		for (int i = 0; i < depFields.size(); i++) {
			SS3Field f = depFields.get(i);
			addChild(child, f);
		}
	}

	private CodingSchemaFieldNode createFieldNode(SS3Field field) {
		LabelObject object = segment.getRootLabelObject()
				.getGlossNRelatedLabelObject();
		ArrayList<LabelObject> list = object.getAllChildren();
		ArrayList<String> available = new ArrayList<String>();
		ArrayList<String> visible = new ArrayList<String>();

		for (int i = 0; i < list.size(); i++) {
			LabelObject obj = list.get(i);
			FieldWrapper wrapper = obj.getWrapper();
			String id = wrapper.getField().getFieldID();

			if (obj.isVisible())
				visible.add(id);

			if (!available.contains(id)
					&& wrapper.getField().getFieldSize() > 0)
				available.add(id);
		}

		String fieldID = "" + field.getId();
		boolean checked = visible.contains(fieldID) ? true : false;
		boolean data = available.contains(fieldID);
		String dataStr = data ? "*" : "";

		Object[] fieldNodeObj = new Object[] { field.getName(),
				field.getLabel(), checked, dataStr, field.getColor(),
				field.getId() };
		return new CodingSchemaFieldNode(fieldNodeObj);
	}
	
	public void setSelectedField(String fieldID, boolean selected) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < 4; groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode
					.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode
						.getChildAt(i);
				String nodeID = (String) childNode.getValueAt(5);
				
				if (nodeID.equals(fieldID)) {
					// this will either check or uncheck the field, then refresh the panel to reflect the change
					childNode.setValueAt(selected, 2);
				}
			}
		}
	}
	
	public void scrollToAndSelectFieldByID(String fieldID) {
		if (fieldID == null || fieldID.isEmpty())
			return;
		
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < 4; groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode
					.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode
						.getChildAt(i);
				String nodeID = (String) childNode.getValueAt(5);
				
				if (nodeID.equals(fieldID)) {					
					scrollToAndSelectNode(childNode);
				}
			}
		}
	}
	
	private void scrollToAndSelectNode(MutableTreeTableNode node) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
	    TreeNode[] nodes = fieldsModel.getPathToRoot(node); 
        TreePath path = new TreePath(nodes); 

	    fieldsTable.expandPath(path.getParentPath());
        int row = fieldsTable.getRowForPath(path);
	    fieldsTable.scrollRowToVisible(row);
	    fieldsTable.scrollPathToVisible(path);	
	    fieldsTable.setRowSelectionInterval(row, row);
	}

	public ArrayList<String> getSelectedFieldIDsNUpdateCodingSchema() {
		ArrayList<String> visibleFieldIDs = new ArrayList<String>();

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) fieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < 4; groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode
					.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				MutableTreeTableNode childNode = (MutableTreeTableNode) groupNode
						.getChildAt(i);
				Boolean display = (Boolean) childNode.getValueAt(2);

				if (display.booleanValue()) {
					visibleFieldIDs.add((String) childNode.getValueAt(5));
				}
			}
		}

		// always set gloss fields to visible
		getSelectedGlossFldIDs(4, visibleFieldIDs);
		getSelectedGlossFldIDs(5, visibleFieldIDs);
		getSelectedHandFields(6, visibleFieldIDs);
		getSelectedHandFields(7, visibleFieldIDs);

		int numberOfChildrenAfterGroupedNonManuals = fieldsRootNode
				.getChildCount();
		for (int counter = 8; counter < numberOfChildrenAfterGroupedNonManuals; counter++) {
			MutableTreeTableNode node = (MutableTreeTableNode) fieldsRootNode
					.getChildAt(counter);
			Boolean display = (Boolean) node.getValueAt(2);
			if (display.booleanValue()) {
				visibleFieldIDs.add((String) node.getValueAt(5));
			}
		}
		return visibleFieldIDs;
	}

	private void getSelectedGlossFldIDs(int index, ArrayList<String> ids) {
		MutableTreeTableNode node = (MutableTreeTableNode) fieldsRootNode
				.getChildAt(index);
		Object obj = node.getValueAt(2);
		Boolean display = (Boolean) obj;
		if (display.booleanValue()) {
			ids.add((String) node.getValueAt(5));
			int childCount = node.getChildCount();
			for (int count = 0; count < childCount; count++) {
				MutableTreeTableNode cNode = (MutableTreeTableNode) node
						.getChildAt(count);
				Boolean cDisplay = (Boolean) cNode.getValueAt(2);

				if (cDisplay.booleanValue()) {
					ids.add((String) cNode.getValueAt(5));
				}
			}
		}
	}

	private void getSelectedHandDependentFieldIDs(MutableTreeTableNode node,
			ArrayList<String> ids) {
		int childCount = node.getChildCount();
		for (int count = 0; count < childCount; count++) {
			MutableTreeTableNode cNode = (MutableTreeTableNode) node
					.getChildAt(count);
			Boolean cDisplay = (Boolean) cNode.getValueAt(2);

			if (cDisplay.booleanValue()) {
				ids.add((String) cNode.getValueAt(5));
				if (cNode.getChildCount() > 0) {
					getSelectedHandDependentFieldIDs(cNode, ids);
				}
			}
		}
	}

	private void getSelectedHandFields(int index, ArrayList<String> ids) {
		MutableTreeTableNode node = (MutableTreeTableNode) fieldsRootNode.getChildAt(index);
		Object obj = node.getValueAt(2);
		Boolean display = (Boolean) obj;
		if (display.booleanValue()) {
			ids.add((String) node.getValueAt(5));
			int childCount = node.getChildCount();
			for (int count = 0; count < childCount; count++) {
				MutableTreeTableNode cNode = (MutableTreeTableNode) node
						.getChildAt(count);
				Boolean cDisplay = (Boolean) cNode.getValueAt(2);

				if (cDisplay.booleanValue()) {
					ids.add((String) cNode.getValueAt(5));
					getSelectedHandDependentFieldIDs(cNode, ids);
				}
			}
		}
	}
	
	public void discard() {		
		newFieldBtn.removeActionListener(this);
		deleteBtn.removeActionListener(this);
		editBtn.removeActionListener(this);
		cancelBtn.removeActionListener(this);
		doneBtn.removeActionListener(this);
		
		fieldsTable.removeTreeSelectionListener(selectionListener);		
		if(newFieldPanel != null) {
			newFieldPanel.discard();
		}
		
		if(editFieldPanel != null) {
			editFieldPanel.discard();
			editFieldPanel = null;
		}
		
		if(editFieldDialog != null) {
			editFieldDialog.dispose();
			editFieldDialog = null;
		}
	}
	
	public Segment getSegment() {
		return segment;
	}
}

class SelectionListener implements TreeSelectionListener {
	private CSFieldPanel panel = null;
	private SS3CodingScheme scheme = null;

	public SelectionListener(CSFieldPanel panel, SS3CodingScheme scheme,
			JButton deleteBtn, JButton editBtn) {
		this.panel = panel;
		this.scheme = scheme;
	}

	@Override
	public void valueChanged(TreeSelectionEvent se) {
		JTree tree = (JTree) se.getSource();
		MutableTreeTableNode selectedNode = (MutableTreeTableNode) tree
				.getLastSelectedPathComponent();

		panel.setSelectedTreeTableNode(selectedNode);

		if (selectedNode == null) {
			return;
		} else if (selectedNode.isLeaf()) {
			String fieldID = (String) selectedNode.getValueAt(5);
			SS3Field field = scheme.getField(fieldID);

			Color selectedColor = (Color)selectedNode.getValueAt(4);			
			Segment segment = panel.getSegment();
			field.setColor(selectedColor);
			
			LabelObject object = segment.getRootLabelObject().getNonManuelLabelObject();
			ArrayList<LabelObject> leafsOnly = object.getAllChildren();

			for (int i = 0; i < leafsOnly.size(); i++) {
				LabelObject obj = leafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if(id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);									
				}
			}
			
			LabelObject gObject = segment.getRootLabelObject().getGlossNRelatedLabelObject();
			ArrayList<LabelObject> gLeafsOnly = gObject.getAllChildren();

			for (int i = 0; i < gLeafsOnly.size(); i++) {
				LabelObject obj = gLeafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if(id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);
				}
			}
			
			if (selectedNode.isLeaf())
				selectedNode = (MutableTreeTableNode) selectedNode.getParent();

			panel.updateField(field, panel.getGroup(selectedNode),
							selectedNode);
		} else {
			panel.updateField(null, panel.getGroup(selectedNode), selectedNode);
		}
	}
}

class TableSelectionListener implements ListSelectionListener {
	JTable table = null;
	JTextField newValueTxt = null;
	JTextField newLabelTxt = null;
	JTextField selectedValueNameTxt = null;
	JButton removeValueBtn = null;

	TableSelectionListener(JTable table, JTextField newValueTxt,
			JTextField newLabelTxt, JTextField selectedValueNameTxt,
			JButton removeValueBtn) {
		this.table = table;
		this.newValueTxt = newValueTxt;
		this.newLabelTxt = newLabelTxt;
		this.selectedValueNameTxt = selectedValueNameTxt;
		this.removeValueBtn = removeValueBtn;
	}

	@Override
	public void valueChanged(ListSelectionEvent event) {
		int row = table.getSelectedRow();

		if (row >= 0) {
			String name = (String) table.getValueAt(row, 0);
			String label = (String) table.getValueAt(row, 1);
			newValueTxt.setText(name);
			newLabelTxt.setText(label);
			selectedValueNameTxt.setText(name);
			removeValueBtn.setEnabled(true);
		}
	}
}
