package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.ui.panels.table.sortable.CSFieldTableSorter;
import edu.bu.signstream.ui.panels.table.sortable.SortableTableModel;

public class CSIndependentFieldsPanel extends JPanel {
   private ArrayList valueList = new ArrayList();
   private JTable table;
   private Object[] columnNames;
   private SS3CodingScheme scheme;

   private SortableTableModel tableModel = null;

   public CSIndependentFieldsPanel(
      ArrayList valueList,
      Object[] columnNames,
      SS3CodingScheme scheme) {
      this.valueList = valueList;
      this.columnNames = columnNames;
      this.scheme = scheme;
      initUI();
   }

   private void initUI() {
      JScrollPane scrollPane = createValuesTable();

      GridBagLayout gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();
      setLayout(gbl);

      gbc.anchor = GridBagConstraints.NORTHWEST;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbc.anchor = GridBagConstraints.WEST;

      add(scrollPane, gbc);

      table.getColumnModel().removeColumn(table.getColumnModel().getColumn(5));

      table.setAutoCreateColumnsFromModel(false);
      //table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

      // add color renderer/editor for color column
      table.setDefaultRenderer(Color.class, new ColorRenderer(true));
      table.setDefaultEditor(Color.class, new ColorEditor());
   }

   private JScrollPane createValuesTable() {
      tableModel = new SortableTableModel(columnNames, createTableModelData());

      CSFieldTableSorter sorter = new CSFieldTableSorter(tableModel);

      table = new JTable(sorter);
      sorter.setTableHeader(table.getTableHeader());

      table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

      table
         .getSelectionModel()
         .addListSelectionListener(new ListSelectionListener() {
         @Override
		public void valueChanged(ListSelectionEvent e) {

            if (e.getValueIsAdjusting()) {
               CSFieldTableSorter sorter1 =
                  (CSFieldTableSorter) table.getModel();
               sorter1.update(
                  table.getSelectedRow(),
                  table.getSelectedColumn());
            }

            setPreferredSize();
         }
      });

      setPreferredSize();
      JScrollPane scrollPane = new JScrollPane(table);
      scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
      Dimension d =
         new Dimension(scrollPane.getPreferredSize().width + 50, 350);
      scrollPane.setMaximumSize(d);
      scrollPane.setPreferredSize(d);

      return scrollPane;
   }

   private Object[][] createTableModelData() {
      Object[][] data = new Object[valueList.size()][];
      for (int j = 0; j < valueList.size(); j++) {
         data[j] = (Object[]) valueList.get(j);
      }

      return data;
   }

   public void addNewField(SS3Field csField) {
      Boolean checked = new Boolean(true);
      String dt = "";

      Object[] row =
         {
            checked,
            dt,
            csField.getName(),
            csField.getLabel(),
            csField.getColor(),
            csField.getId()};

      valueList.add(row);

      tableModel.update(valueList);
      tableModel.fireTableDataChanged();

      // add to loaded coding scheme
      scheme.addField(csField);
   }

   private void setPreferredSize() {
      TableColumn checkBox = table.getColumn(columnNames[0]);
      TableColumn data = table.getColumn(columnNames[1]);
      TableColumn fieldName = table.getColumn(columnNames[2]);
      TableColumn fieldLabel = table.getColumn(columnNames[3]);
      TableColumn fieldColor = table.getColumn(columnNames[4]);

      int checkBoxPrefSize = getPreferredWidthForColumn(checkBox);
      int fieldDataPrefSize = getPreferredWidthForColumn(data);
      int fieldNamePrefSize = getPreferredWidthForColumn(fieldName);
      int fieldLabelPrefSize = getPreferredWidthForColumn(fieldLabel);
      int fieldColorPrefSize = 40;

      checkBox.setMinWidth(checkBoxPrefSize);
      data.setMinWidth(fieldDataPrefSize);
      fieldName.setMinWidth(fieldNamePrefSize);
      fieldLabel.setMinWidth(fieldLabelPrefSize);
      fieldColor.setMinWidth(fieldColorPrefSize);
   }

   public SortableTableModel getTableModel() {
      return tableModel;
   }

   public ArrayList getSelectedFieldIDs() {

      // call update on table model here 

      ArrayList selectedFieldIDs = new ArrayList();
      ArrayList selectedFields = tableModel.getSelectedFields();

      for (int i = 0; i < selectedFields.size(); i++) {
         String fieldID = (String) selectedFields.get(i);
         selectedFieldIDs.add(fieldID);
      }
      return selectedFieldIDs;
   }

   public int getPreferredWidthForColumn(TableColumn col) {
         int hw = columnHeaderWidth(col), // hw = header width
   cw = widestCellInColumn(col); // cw = column width

      return hw > cw ? hw : cw;
   }

   private int columnHeaderWidth(TableColumn col) {
      TableCellRenderer renderer = col.getHeaderRenderer();
      return 75;
   }

   private int widestCellInColumn(TableColumn col) {
      int c = col.getModelIndex(), width = 0, maxw = 0;

      for (int r = 0; r < table.getRowCount(); ++r) {
         TableCellRenderer renderer = table.getCellRenderer(r, c);

         Component comp =
            renderer.getTableCellRendererComponent(
               table,
               table.getValueAt(r, c),
               false,
               false,
               r,
               c);

         width = comp.getPreferredSize().width;
         maxw = width > maxw ? width : maxw;
      }
      return maxw + 10;
   }
}
