package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.ui.panels.AnnotatorsDialog;
import edu.bu.signstream.ui.panels.AnnotatorsPanel;
import edu.bu.signstream.ui.panels.NewAnnotatorDialog;
import edu.bu.signstream.ui.panels.ParticipantDialog;

public class ParticipantListener implements ActionListener {

	public static final int INFO = 1;
	public static final int EDIT = 2;
	public static final int ADD = 3;
	public static final int REMOVE = 4;

	JTextField fullNameTxt = new JTextField(11);
	JTextField labelTxt = new JTextField(11);
	JTextField ageTxt = new JTextField(11);
	JTextField genderTxt = new JTextField(11);
	JTextField languageTxt = new JTextField(11);
	JTextField domHandTxt = new JTextField(11);
	JTextArea commentsTf = new JTextArea(4, 24);
	JTextArea parentsTf = new JTextArea(4, 24);
	JTextArea backgroundTf = new JTextArea(4, 24);

	JDialog showInformationDialog = null; // new JDialog();
	JPanel informationPanel = new JPanel();
	
	private ParticipantsPaneScrolledView participantsPaneScrolledView = null;
	private AnnotatorsDialog annotatorsDialog = null;

	public ParticipantListener(ParticipantsPaneScrolledView participantsPaneScrolledView, AnnotatorsDialog annotatorsDialog) {	
		this.participantsPaneScrolledView = participantsPaneScrolledView;
		this.annotatorsDialog = annotatorsDialog;
	}
	
	public void setParticipantsPaneScrolledView(ParticipantsPaneScrolledView participantsPaneScrolledView) {
		this.participantsPaneScrolledView = participantsPaneScrolledView;	
	}

	private void showInformation(SS3Participant participant,
			boolean isParticipant) {
		if (showInformationDialog == null) {
			showInformationDialog = new JDialog();
			
			JLabel participantName = new JLabel("");
			JLabel participantLabel = new JLabel("");

			if (isParticipant) {
				participantName.setText("Participant's Full Name: ");
				participantLabel.setText("Participant's Label: ");
			} else {
				participantName.setText("Annotator's Full Name: ");
				participantLabel.setText("Annotator's Label: ");
			}

			JLabel l3 = new JLabel("Age: ");
			JLabel l4 = new JLabel("Gender: ");
			JLabel l5 = new JLabel("Native Language: ");
			JLabel l6 = new JLabel("Dominant Hand: ");

			fullNameTxt.setEnabled(false);
			fullNameTxt.setText(participant.getFullName());

			labelTxt.setEnabled(false);
			labelTxt.setText(participant.getLabel());

			ageTxt.setEnabled(false);
			ageTxt.setText(participant.getYearOfBirth());

			genderTxt.setEnabled(false);
			genderTxt.setText("" + participant.getGender());

			languageTxt.setEnabled(false);
			languageTxt.setText("" + participant.getNativeLanguage());

			domHandTxt.setEnabled(false);
			domHandTxt.setText("" + participant.getDominantHand());

			JLabel l7 = new JLabel("Comments:");
			commentsTf.setEnabled(false);
			commentsTf.setText(participant.getComments());
			JScrollPane commentsInfo = new JScrollPane(commentsTf);
			commentsInfo.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

			JLabel l8 = new JLabel("Parent Info:");
			parentsTf.setEnabled(false);
			parentsTf.setText(participant.getParentsInfo());
			JScrollPane parentsInfo = new JScrollPane(parentsTf);
			parentsInfo.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

			JLabel l9 = new JLabel("Background:");
			backgroundTf.setEnabled(false);
			backgroundTf.setText(participant.getBackground());
			JScrollPane backgroundInfo = new JScrollPane(backgroundTf);
			backgroundInfo.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

			GridBagConstraints c = new GridBagConstraints();
			informationPanel.setLayout(new GridBagLayout());

			c.anchor = GridBagConstraints.WEST;
			c.insets = new Insets(1, 1, 1, 1);

			addComponent(c, 0, 0, 2, participantName, 2, 0, 1, fullNameTxt);
			addComponent(c, 0, 1, 2, participantLabel, 2, 1, 1, labelTxt);

			if (isParticipant) {
				addComponent(c, 0, 2, 2, l3, 2, 2, 1, ageTxt);
				addComponent(c, 0, 3, 2, l4, 2, 3, 1, genderTxt);
				addComponent(c, 0, 4, 2, l5, 2, 4, 1, languageTxt);
				addComponent(c, 0, 5, 2, l6, 2, 5, 1, domHandTxt);
			}

			addComponent(c, 0, 6, 3, l7, 0, 7, 3, commentsInfo);

			if (isParticipant) {
				addComponent(c, 0, 8, 3, l8, 0, 9, 3, parentsInfo);
				addComponent(c, 0, 10, 3, l9, 0, 11, 3, backgroundInfo);
			}

			informationPanel.setBorder(BorderFactory.createEmptyBorder(20, 20,
					30, 20));

			JPanel panel = new JPanel(new BorderLayout(20, 20));
			panel.add(informationPanel, BorderLayout.CENTER);

			Container contentPane = showInformationDialog.getContentPane();
			contentPane.setLayout(new BorderLayout());
			contentPane.add(panel, BorderLayout.CENTER);

			if (isParticipant) {
				showInformationDialog.setTitle("Participant Information");
			} else {
				showInformationDialog.setTitle("Annotator Information");
			}

			showInformationDialog.pack();
			showInformationDialog.setLocation(250, 250);
		} else {			
			fullNameTxt.setText(participant.getFullName());
			ageTxt.setText(participant.getYearOfBirth());
			commentsTf.setText(participant.getComments());
			
			if (isParticipant) {
				labelTxt.setText(participant.getFullName());
				genderTxt.setText("" + participant.getGender());
				languageTxt.setText("" + participant.getNativeLanguage());
				domHandTxt.setText("" + participant.getDominantHand());
				parentsTf.setText(participant.getParentsInfo());
				backgroundTf.setText(participant.getBackground());
			}
			
			informationPanel.repaint();
		}
		showInformationDialog.setVisible(true);
	}


	private void addComponent(GridBagConstraints c, int gridx1, int gridy1,
			int gridwidth1, JComponent component1, int gridx2, int gridy2,
			int gridwidth2, JComponent component2) {
		c.gridy = gridy1;
		c.gridx = gridx1;
		c.gridwidth = gridwidth1;
		informationPanel.add(component1, c);
		c.gridy = gridy2;
		c.gridx = gridx2;
		c.gridwidth = gridwidth2;
		informationPanel.add(component2, c);
	}

	private ParticipantDialog participantDialog = null;
	
	private void showParticipantDialog(ParticipantButton btn) {
		SS3Participant participant = btn.getParticipant();
		ParticipantView participantView = btn.getParticipantView();
		AnnotatorsPanel annotatorPanel = btn.getAnnotatorPanel();
		
		if(participantDialog == null) {
			participantDialog = new ParticipantDialog(participant, participantView, annotatorPanel);
			participantDialog.setSize(280, 310);
			participantDialog.setLocation(150, 150);
			
		} else {
			participantDialog.updateParticipant(participant, participantView, annotatorPanel);
		}
		
		participantDialog.setVisible(true);
	}
	
	private NewAnnotatorDialog newAnnotatorDialog = null;
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();
		ParticipantButton btn = (ParticipantButton) ae.getSource();
		SS3Participant participant = btn.getParticipant();
		boolean isParticipant = btn.isParticipant();

		switch (Integer.parseInt(str)) {
		case INFO:
			showInformation(participant, isParticipant);
			break;

		case EDIT:
			if (isParticipant) {
				showParticipantDialog(btn);
			} else {
				if(newAnnotatorDialog == null) {
					newAnnotatorDialog = new NewAnnotatorDialog(participant, btn.getParticipantView(), btn.getAnnotatorPanel()); 				
					newAnnotatorDialog.setSize(280, 165);
				}
				newAnnotatorDialog.update(participant, btn.getParticipantView(), btn.getAnnotatorPanel());
				newAnnotatorDialog.setVisible(true);
			}
			break;
		case ADD:
			annotatorsDialog.setLocation(250, 250);
			annotatorsDialog.setVisible(true);
			
			break;

		case REMOVE:

			if (participantsPaneScrolledView != null)
				participantsPaneScrolledView.removeParticipantView(btn.getParticipantView());
			break;
		}
	}
}
