package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;
import edu.bu.signstream.ui.video.MultipleMovieController;
import edu.bu.signstream.ui.video.XugglerMovie;

public class VideoPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;

	private LoadedMediaFile mediaFile = null;
	private JButton selectBtn = new JButton("Select");
	private JButton removeBtn = new JButton("Remove");
	private JButton infoBtn = new JButton("Info");

	private JCheckBox lookupCb = new JCheckBox();
	private JCheckBox displayCb = new JCheckBox();

	private JLabel fileNameLbl = new JLabel();
	private JFrame parentFrame = null;

	private JDialog proceedDialog = null;
	private JDialog proceedRemoveDialog = null;
	private JDialog infoDialog = null;
	
	private RemoveListener removeListener = null;
	private ProceedListener proceedListener = null;
	private InfoListener infoListener = null;
	
	private JButton infoCloseBtn = new JButton("Close");
	private JButton proceedSelectBtn = new JButton("Proceed");
	private JButton proceedRemoveBtn = new JButton("Proceed");
	
	private JLabel l9 = new JLabel("");
	private JLabel l10 = new JLabel("");
	private JLabel fileLbl = new JLabel("");
	private JLabel folderLbl = new JLabel("");
	private JLabel descriptionLbl = new JLabel("");
	
	private VideoPanel vp = null;

	private final int SELECT = 1;
	private final int REMOVE = 2;
	private final int INFO = 3;

	public VideoPanel(LoadedMediaFile mediaFile, JFrame parentFrame) {
		this.mediaFile = mediaFile;
		this.parentFrame = parentFrame;
		this.vp = this;

		selectBtn.setEnabled(true);

		if (mediaFile != null) {
			removeBtn.setEnabled(true);
			infoBtn.setEnabled(true);
			lookupCb.setEnabled(true);
			displayCb.setEnabled(true);
			
			String fn = mediaFile.getMediaFile().getFile().getAbsolutePath();
			MultipleMovieController contr = SS3Singleton
					.getVideoControlManager().getMultipleMovieController();

			ArrayList<XugglerMovie> loadedMovies = contr.getMovies();

			for (XugglerMovie movie : loadedMovies) {
				if (fn.equals(movie.getFilename())) {
					displayCb.setSelected(true);
				}
			}

			String fileName = mediaFile.getMediaFile().getFile().getName();

			fileNameLbl.setText(fileName);

			boolean isExists = mediaFile.getMediaFile().getFile().exists();
			
			if (!isExists) {
				fileNameLbl.setEnabled(false);
				lookupCb.setSelected(true);
			} else {
				lookupCb.setSelected(false);
				lookupCb.setEnabled(false);
			}

		} else {
			removeBtn.setEnabled(false);
			infoBtn.setEnabled(false);
			lookupCb.setEnabled(false);
			displayCb.setEnabled(false);
		}

		selectBtn.setActionCommand("" + SELECT);
		selectBtn.addActionListener(this);
		removeBtn.setActionCommand("" + REMOVE);
		removeBtn.addActionListener(this);
		infoBtn.setActionCommand("" + INFO);
		infoBtn.addActionListener(this);

		UserInterfaceUtil.setButtonsLNF(selectBtn);
		UserInterfaceUtil.setButtonsLNF(removeBtn);
		UserInterfaceUtil.setButtonsLNF(infoBtn);
		UserInterfaceUtil.setLabelLNF(fileNameLbl);		
		lookupCb.setBackground(Color.white);
		displayCb.setBackground(Color.white);
	}
	
	public void setMediaFile(LoadedMediaFile mediaFile) {
		this.mediaFile = mediaFile;
		
		if (mediaFile != null) {
			removeBtn.setEnabled(true);
			infoBtn.setEnabled(true);
			lookupCb.setEnabled(true);
			displayCb.setEnabled(true);
			
			String fn = mediaFile.getMediaFile().getFile().getAbsolutePath();
			MultipleMovieController contr = SS3Singleton
					.getVideoControlManager().getMultipleMovieController();

			ArrayList<XugglerMovie> loadedMovies = contr.getMovies();
			
			displayCb.setSelected(false);
			
			for (XugglerMovie movie : loadedMovies) {
				if (fn.equals(movie.getFilename())) {
					displayCb.setSelected(true);
				}
			}

			String fileName = mediaFile.getMediaFile().getFile().getName();

			fileNameLbl.setText(fileName);

			boolean isExists = mediaFile.getMediaFile().getFile().exists();
			
			if (!isExists) {
				fileNameLbl.setEnabled(false);
				lookupCb.setSelected(true);
			} else {
				lookupCb.setSelected(false);
				lookupCb.setEnabled(false);
			}

		} else {
			removeBtn.setEnabled(false);
			infoBtn.setEnabled(false);
			lookupCb.setEnabled(false);
			displayCb.setEnabled(false);
		}
	}

	public void locateViaLookup(File file) {
		if (file != null && file.exists()) {
			mediaFile.getMediaFile().setFile(file);
			fileNameLbl.setText("L:" + file.getName());
			fileNameLbl.setEnabled(true);
			lookupCb.setSelected(false);
			lookupCb.setEnabled(false);
			displayCb.setEnabled(true);
			displayCb.setSelected(true);
		}
	}

	private void showSelectDialog() {
		if (proceedDialog != null) {
			if (mediaFile.getMediaFile().getFile() != null) {
				String filename = "filename: "
						+ mediaFile.getMediaFile().getFile().getName();
				l9.setText(filename);

				if (mediaFile.getMediaFile().getFile().getParentFile() != null) {
					String pathname = "pathname: "
							+ mediaFile.getMediaFile().getFile()
									.getParentFile().getPath();
					l10.setText(pathname);
				}
			}
			proceedDialog.getContentPane().repaint();
		} else {
			proceedListener = new ProceedListener();
			proceedSelectBtn.addActionListener(proceedListener);						

			if (mediaFile.getMediaFile().getFile() != null) {
				String filename = "filename: "
						+ mediaFile.getMediaFile().getFile().getName();
				l9.setText(filename);

				if (mediaFile.getMediaFile().getFile().getParentFile() != null) {
					String pathname = "pathname: "
							+ mediaFile.getMediaFile().getFile()
									.getParentFile().getPath();
					l10.setText(pathname);
				}
			}

			JPanel proceedWarningPanel = new JPanel();
			GridBagConstraints c = new GridBagConstraints();
			proceedWarningPanel.setLayout(new GridBagLayout());

			c.insets = new Insets(1, 1, 1, 1);

			String txt1 = " Please be sure that you locate the same movie file that";
			addTextComponent(txt1, GridBagConstraints.WEST, 0, 0, 3, c,
					proceedWarningPanel);

			String txt2 = "was originally associated with this Macro Unit. If you";
			addTextComponent(txt2, -1, 0, 1, 3, c, proceedWarningPanel);

			String txt3 = "select a different file, you could encounter problems,";
			addTextComponent(txt3, -1, 0, 2, 3, c, proceedWarningPanel);

			String txt4 = "as the new file will most likely not have the same frame";
			addTextComponent(txt4, -1, 0, 3, 3, c, proceedWarningPanel);

			String txt5 = "numbers as the original file. As a result, any data that has";
			addTextComponent(txt5, -1, 0, 4, 3, c, proceedWarningPanel);

			String txt6 = "already been coded may end up being linked to frames that";
			addTextComponent(txt6, -1, 0, 5, 3, c, proceedWarningPanel);

			String txt7 = "don't exist in the new file.";
			addTextComponent(txt7, -1, 0, 6, 3, c, proceedWarningPanel);

			c.gridy = 7;
			c.gridx = 2;
			c.gridwidth = 1;
			c.anchor = GridBagConstraints.EAST;
			proceedWarningPanel.add(proceedSelectBtn, c);

			String txt8 = "Previously selected movie file:";
			addTextComponent(txt8, GridBagConstraints.WEST, 0, 8, 3, c,
					proceedWarningPanel);

			c.gridy = 9;
			c.gridx = 0;
			c.gridwidth = 3;
			proceedWarningPanel.add(l9, c);

			c.gridy = 10;
			c.gridx = 0;
			c.gridwidth = 3;
			proceedWarningPanel.add(l10, c);

			proceedWarningPanel.setBorder(BorderFactory.createEmptyBorder(20,
					20, 30, 20));

			JPanel panel = new JPanel(new BorderLayout(20, 20));
			panel.add(proceedWarningPanel, BorderLayout.CENTER);

			proceedDialog = new JDialog();
			Container contentPane = proceedDialog.getContentPane();
			contentPane.setLayout(new BorderLayout());
			contentPane.add(panel, BorderLayout.CENTER);
			proceedDialog.setTitle("Video File Lookup");
			proceedDialog.pack();
			proceedDialog.setLocation(350, 300);
		}
		proceedDialog.setVisible(true);
	}

	private void showRemoveMediaFile() {
		if (proceedRemoveDialog == null) {
			removeListener = new RemoveListener();
			proceedRemoveBtn.addActionListener(removeListener);

			JPanel warningPanel = new JPanel();
			GridBagConstraints c = new GridBagConstraints();
			warningPanel.setLayout(new GridBagLayout());

			c.insets = new Insets(1, 1, 1, 1);

			String txt1 = " Are you sure you want to remove this media so";
			addTextComponent(txt1, GridBagConstraints.WEST, 0, 0, 3, c,
					warningPanel);

			String txt2 = "that it is no longer associated with this Macro Unit?";
			addTextComponent(txt2, -1, 0, 1, 3, c, warningPanel);

			String txt4 = "(This operation cannot be undone.)";
			addTextComponent(txt4, -1, 0, 3, 3, c, warningPanel);

			c.gridy = 4;
			c.gridx = 2;
			c.gridwidth = 1;
			c.anchor = GridBagConstraints.EAST;
			warningPanel.add(proceedRemoveBtn, c);

			addTextComponent("", GridBagConstraints.WEST, 0, 5, 3, c,
					warningPanel);

			warningPanel.setBorder(BorderFactory.createEmptyBorder(20, 20, 30,
					20));
			JPanel panel = new JPanel(new BorderLayout(20, 20));
			panel.add(warningPanel, BorderLayout.CENTER);

			proceedRemoveDialog = new JDialog();
			Container contentPane = proceedRemoveDialog.getContentPane();
			contentPane.setLayout(new BorderLayout());
			contentPane.add(panel, BorderLayout.CENTER);
			proceedRemoveDialog.setTitle("Remove Video File");
			proceedRemoveDialog.pack();
			proceedRemoveDialog.setLocation(380, 300);
		}
		proceedRemoveDialog.setVisible(true);
	}
	
	private void showInfoMediaFile() {
		if(infoDialog==null) {
			infoDialog = new JDialog();
			infoListener = new InfoListener();
			infoCloseBtn.addActionListener(infoListener);	

			JPanel infoPanel = new JPanel();
			GridBagConstraints c = new GridBagConstraints();
			infoPanel.setLayout(new GridBagLayout());

			c.insets = new Insets(1, 1, 1, 1);
			
			String txt1 = "File: " + mediaFile.getMediaFile().getFile().getName();
			fileLbl.setText(txt1);

			c.anchor = GridBagConstraints.WEST;
			c.gridy = 1;
			c.gridx = 0;
			c.gridwidth = 3;
			infoPanel.add(fileLbl, c);
			
			String txt2 = "Folder: " + mediaFile.getMediaFile().getFile().getParentFile().getPath();
			folderLbl.setText(txt2);

			c.gridy = 2;
			c.gridx = 0;
			c.gridwidth = 3;
			infoPanel.add(folderLbl, c);
			
			String txt3 = "Description: " + mediaFile.getMediaFile().getDescription();
			descriptionLbl.setText(txt3);

			c.gridy = 3;
			c.gridx = 0;
			c.gridwidth = 3;
			infoPanel.add(descriptionLbl, c);

			c.gridy = 7;
			c.gridx = 2;
			c.gridwidth = 3;
			c.anchor = GridBagConstraints.EAST;
			infoPanel.add(infoCloseBtn, c);

			infoPanel.setBorder(BorderFactory.createEmptyBorder(20,20, 30, 20));
			
			infoPanel.setSize(infoPanel.getPreferredSize());

			JPanel panel = new JPanel(new BorderLayout(20, 20));
			panel.add(infoPanel, BorderLayout.CENTER);

			Container contentPane = infoDialog.getContentPane();
			contentPane.setLayout(new BorderLayout());
			contentPane.add(panel, BorderLayout.CENTER);
			infoDialog.setTitle("Video File Information");
			infoDialog.pack();
			infoDialog.setLocation(420, 300);
		} else {
			String fileTxt = "File: " + mediaFile.getMediaFile().getFile().getName();
			String folderTxt = "Folder: " + mediaFile.getMediaFile().getFile().getParentFile().getPath();
			String descriptionTxt = "Description: " + mediaFile.getMediaFile().getDescription();
			fileLbl.setText(fileTxt);
			folderLbl.setText(folderTxt);
			descriptionLbl.setText(descriptionTxt);
			infoDialog.getContentPane().repaint();
		}
		infoDialog.setVisible(true);
	}
	
	private void addTextComponent(String txt, int anchor, int gridx, int gridy,
			int gridwidth, GridBagConstraints c, JPanel panel) {
		JLabel label = new JLabel(txt);
		if (anchor > -1) {
			c.anchor = anchor;
		}
		c.gridy = gridy;
		c.gridx = gridx;
		c.gridwidth = gridwidth;
		panel.add(label, c);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case SELECT:
			if (mediaFile != null) {
				showSelectDialog();
			} else {
				// use to select video files
				try {
					File file = new File("");
					File selectedFile = UserInterfaceUtil.selectFile(
							parentFrame, file, this, -1, "Locate Video File");

					if (selectedFile != null && selectedFile.exists()) {
						if (mediaFile == null) {
							mediaFile = new LoadedMediaFile();
							SS3MediaFile mf = new SS3MediaFile();
							mf.setFile(selectedFile);
							mf.setId(Util.getUniqueNumber());
							mediaFile.setMediaFile(mf);
						}

						mediaFile.getMediaFile().setFile(selectedFile);

						fileNameLbl.setText(selectedFile.getName());
						fileNameLbl.setEnabled(true);
						lookupCb.setSelected(false);
						lookupCb.setEnabled(false);
						displayCb.setEnabled(true);
						displayCb.setSelected(true);
						removeBtn.setEnabled(true);
						infoBtn.setEnabled(true);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			break;

		case REMOVE:
			showRemoveMediaFile();
			break;

		case INFO:
			showInfoMediaFile();
			break;
		}
	}
	
	public void destroy() {		
		infoCloseBtn.removeActionListener(infoListener);
		proceedSelectBtn.removeActionListener(proceedListener);
		proceedRemoveBtn.removeActionListener(removeListener);
		
		removeListener = null;
		proceedListener = null;
		infoListener = null;
		
		proceedDialog = null;
		proceedRemoveDialog = null;
		infoDialog = null;
		
		selectBtn.removeActionListener(this);
		removeBtn.removeActionListener(this);
		infoBtn.removeActionListener(this);
	}

	public LoadedMediaFile getMediaFile() {
		if (mediaFile != null)
			mediaFile.setDisplay(displayCb.isSelected());
		return mediaFile;
	}

	public JButton getSelectBtn() {
		return selectBtn;
	}

	public JButton getRemoveBtn() {
		return removeBtn;
	}

	public JButton getInfoBtn() {
		return infoBtn;
	}

	public JCheckBox getLookupCb() {
		return lookupCb;
	}

	public JCheckBox getDisplayCb() {
		return displayCb;
	}

	public JLabel getFileNameLbl() {
		return fileNameLbl;
	}

	private class ProceedListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			proceedDialog.setVisible(false);
			String fName = mediaFile.getMediaFile().getFile().getPath();
			File file1 = new File(fName);
			File selectedFile1 = UserInterfaceUtil.selectFile(parentFrame,
					file1, vp, UserInterfaceUtil.VIDEO_FILES_FILTER,
					"Please locate movie '"
							+ mediaFile.getMediaFile().getFile().getName()
							+ "'");

			if (selectedFile1 != null && selectedFile1.exists()) {
				if (mediaFile == null) {
					mediaFile = new LoadedMediaFile();
					SS3MediaFile mf = new SS3MediaFile();
					mf.setFile(selectedFile1);
					mf.setId(Util.getUniqueNumber());
					mediaFile.setMediaFile(mf);
				}

				mediaFile.getMediaFile().setFile(selectedFile1);
				fileNameLbl.setText(selectedFile1.getName());
				fileNameLbl.setEnabled(true);
				lookupCb.setSelected(false);
				lookupCb.setEnabled(true);
				displayCb.setEnabled(true);
				displayCb.setSelected(true);
				removeBtn.setEnabled(true);
				infoBtn.setEnabled(true);
			}
		}
	}

	private class RemoveListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			proceedRemoveDialog.setVisible(false);
			mediaFile = null;
			fileNameLbl.setText("");
			removeBtn.setEnabled(false);
			infoBtn.setEnabled(false);
			displayCb.setSelected(false);
			lookupCb.setSelected(false);
			displayCb.setEnabled(false);
			lookupCb.setEnabled(false);
		}
	}
	
	private class InfoListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			infoDialog.setVisible(false);
		}
	}
}