package edu.bu.signstream.ui.panels.search;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;

public class NonBinarySearchGroup extends SearchGroup implements ActionListener {

	private static final long serialVersionUID = 1L;
	
	// LIST OF EXPRESSIONS (CHILDREN)
	private ArrayList<SearchExpression> expressionsList;
	
	// GUI COMPONENTS
	
	private JPanel leftPanel;
	private JPanel rightPanel;
	
	private JPanel operatorPanel;
	private JRadioButton andButton;
	private JRadioButton orButton;
	
	private JPanel expressionsPanel;
	
	private JPanel addButtonPanel;
	private JButton addRuleButton;
	private JButton addNonBinaryGroupButton;
	private JButton addBinaryGroupButton;
	
	// ACTION COMMANDS
	
	private final int REMOVE = 0;
	private final int NEGATE_GROUP = 10;
	private final int AND = 20;
	private final int OR = 30;
	private final int ADD_RULE = 40;
	private final int ADD_NONBINARY = 50;
	private final int ADD_BINARY = 60;
	
	// LABELS
	
	private final String origGroupButtonLabel = "";
	private final String negateGroupButtonLabel = "NOT";
	
	private final String andRadioLabel = SearchOperator.AND.toString();
	private final String orRadioLabel = SearchOperator.OR.toString();
	
	private final String addRuleLabel = "Add Rule";
	private final String addNonBinaryGroupLabel = "Add Logical Group (AND, OR)";
	private final String addBinaryGroupLabel = "Add Temporal Group (WITH, BEFORE, AFTER)";

	public NonBinarySearchGroup(SS3Database database, SearchGroup parentGroup) {
		super(database, parentGroup);
		
		expressionsList = new ArrayList<SearchExpression>();
		
		searchTreeNode = new SearchTreeNode(null, SearchOperator.AND);	// all nonbinary search groups default to AND
		
		createUI();
	}

	@Override
	public void createUI() {
		
		// color background even/odd according to parentGroup && disable remove button if root
		
		int panelBorderWidth = 2;

		java.awt.Color backgroundColor = java.awt.Color.ORANGE.darker();
		java.awt.Color parentColor = null;
		
		if (parentGroup != null)
			parentColor = parentGroup.getBackground(); 
		else
			removeButton.setEnabled(false);	// no parentGroup? we are root, so disable remove button
		
		if (parentColor != null && parentColor.equals(backgroundColor))
			backgroundColor = java.awt.Color.YELLOW.darker();
		
		setBackground(backgroundColor);
		setOpaque(true);
		setBorder(BorderFactory.createLineBorder(backgroundColor, panelBorderWidth));
		
		// action listener for removeButton
		
		removeButton.addActionListener(this);
		removeButton.setActionCommand("" + REMOVE);
		
		// instantiate and add components for operatorPanel
		
		operatorPanel = new JPanel();
		operatorPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		negateGroupButton = new JButton("");
		negateGroupButton.addActionListener(this);
		negateGroupButton.setActionCommand("" + NEGATE_GROUP);
		
		andButton = new JRadioButton(andRadioLabel);
		andButton.setSelected(true);
		andButton.addActionListener(this);
		andButton.setActionCommand("" + AND);
		orButton = new JRadioButton(orRadioLabel);
		orButton.addActionListener(this);
		orButton.setActionCommand("" + OR);

		ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(andButton);
		buttonGroup.add(orButton);
		
		operatorPanel.add(negateGroupButton);
		operatorPanel.add(andButton);
		operatorPanel.add(orButton);
		operatorPanel.setBackground(backgroundColor);
		operatorPanel.setOpaque(true);
		
		// instantiate and add components for expressionsPanel
		
		expressionsPanel = new JPanel();
		expressionsPanel.setLayout(new BoxLayout(expressionsPanel, BoxLayout.PAGE_AXIS));
		expressionsPanel.setOpaque(false);
		
		addSearchRule();	// adds to newly created expressionsPanel
		
		// instantiate and add components for addButtonPanel
		
		addButtonPanel = new JPanel();
		addButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		addRuleButton = new JButton(addRuleLabel);
		addRuleButton.addActionListener(this);
		addRuleButton.setActionCommand("" + ADD_RULE);
		
		addNonBinaryGroupButton = new JButton(addNonBinaryGroupLabel);
		addNonBinaryGroupButton.addActionListener(this);
		addNonBinaryGroupButton.setActionCommand("" + ADD_NONBINARY);
		
		addBinaryGroupButton = new JButton(addBinaryGroupLabel);
		addBinaryGroupButton.addActionListener(this);
		addBinaryGroupButton.setActionCommand("" + ADD_BINARY);
		
		addButtonPanel.add(addRuleButton);
		addButtonPanel.add(addNonBinaryGroupButton);
		addButtonPanel.add(addBinaryGroupButton);
		addButtonPanel.setBackground(backgroundColor);
		addButtonPanel.setOpaque(true);
		
		// add sub-panels to this nonbinary search group panel
		
		leftPanel = new JPanel();
		leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.PAGE_AXIS));
		removeButton.setAlignmentX(LEFT_ALIGNMENT);
		leftPanel.add(removeButton);
		leftPanel.setOpaque(false);
		
		rightPanel = new JPanel();
		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.PAGE_AXIS));
		operatorPanel.setAlignmentX(LEFT_ALIGNMENT);
		operatorPanel.setAlignmentY(TOP_ALIGNMENT);
		rightPanel.add(operatorPanel);
		expressionsPanel.setAlignmentX(LEFT_ALIGNMENT);
		expressionsPanel.setAlignmentY(TOP_ALIGNMENT);
		rightPanel.add(expressionsPanel);
		addButtonPanel.setAlignmentX(LEFT_ALIGNMENT);
		addButtonPanel.setAlignmentY(TOP_ALIGNMENT);
		rightPanel.add(addButtonPanel);
		rightPanel.setOpaque(false);
		rightPanel.setBorder(BorderFactory.createLineBorder(backgroundColor, 10));
		
		setLayout(new BorderLayout());
		add(leftPanel, BorderLayout.WEST);
		add(rightPanel, BorderLayout.CENTER);		
	}

	@Override
	public ArrayList<SearchExpression> getAllExpressions() {
		return expressionsList;
	}
	
	public void addSearchRule() {
		SearchRule searchRule = new SearchRule(database, this);		
		addExpression(searchRule);
	}
	
	public void addBinarySearchGroup() {
		BinarySearchGroup binarySearchGroup = new BinarySearchGroup(database, this);		
		addExpression(binarySearchGroup);
	}
	
	public void addNonBinarySearchGroup() {
		NonBinarySearchGroup nonBinarySearchGroup = new NonBinarySearchGroup(database, this);		
		addExpression(nonBinarySearchGroup);
	}
	
	private void addExpression(SearchExpression expression) {
		if (expression == null)
			return;
		
		searchTreeNode.add(expression.getSearchTreeNode());
		
		expressionsList.add(expression);
		
		expression.setAlignmentX(LEFT_ALIGNMENT);
		expression.setAlignmentY(TOP_ALIGNMENT);
		expressionsPanel.add(expression);
	}
	
	@Override
	public void removeExpression(SearchExpression expression) {
		if (expression == null)
			return;
		
		searchTreeNode.remove(expression.getSearchTreeNode());
		
		expressionsList.remove(expression);
		
		expressionsPanel.remove(expression);
		
		revalidate();
		repaint();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		
		switch(Integer.parseInt(command)) {
		case REMOVE:
			parentGroup.removeExpression(this);
			break;
		case NEGATE_GROUP:			
			String text = negateGroupButton.getText();
			if (text.equals("")) {
				negateGroupButton.setText(negateGroupButtonLabel);
				searchTreeNode.setNegateGroupFlag(true);
				
			} else {
				negateGroupButton.setText(origGroupButtonLabel);
				searchTreeNode.setNegateGroupFlag(false);
			}
			break;
		case AND:
			searchTreeNode.setSearchOperator(SearchOperator.AND);
			break;
		case OR:
			searchTreeNode.setSearchOperator(SearchOperator.OR);
			break;
		case ADD_RULE:
			addSearchRule();
			revalidate();
			repaint();
			break;
		case ADD_NONBINARY:
			addNonBinarySearchGroup();
			revalidate();
			repaint();
			repaint();
			break;
		case ADD_BINARY:
			addBinarySearchGroup();
			revalidate();
			repaint();
			repaint();
			break;
		}
		
	}
}
