package edu.bu.signstream.ui.panels.search;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.logging.Level;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;

import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.reguestsHandler.Manager;
import edu.bu.signstream.grepresentation.reguestsHandler.RequestHandler;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.UtteranceView;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.Zoomer;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SearchPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	
	private static final String DOMINANT_GLOSS_FIELD_ID = "10000";
	
	private JLabel searchDomain = null;
	private JButton changeDomainButton = null;
	
	private JLabel searchQueryBuilderLabel = new JLabel("Query Builder: \n");
	private SearchQueryBuilderPanel searchQueryBuilderPanel = null;
	
	private JLabel searchQueryTextLabel = new JLabel("Query: \n");
	private JTextArea searchQueryTextPanel = null;
	
	private JButton searchButton = null;
	private JButton returnOriginalState = null;
	
	private JList<String> resultsList = null;
	private JScrollPane resultsListScroller = null;
	private ResultsListSelectionListener resultsListSelectionListener = null;
	
	private SearchEngine searchEngine = null;
	
	private SearchSavedState originalSavedState;
	private HashMap<Integer, SearchSavedState> searchSavedStates;
	private String currentResultVisible = null;
	private int currentResultIndexVisible = -1;
	
	private SS3SignStreamApplication application;
	private SS3Database database;
	
	private final int CHANGE_DOMAIN = 10;
	private final int SEARCH = 20;
	private final int RETURN_ORIGINAL_STATE = 30;
	
	public SearchPanel(SS3SignStreamApplication application) {
		this.application = application;
		
		if (application.getSignStreamSegmentPanel() != null) {
			if (application.getSignStreamSegmentPanel().getSS3Database() != null) {
				this.database = application.getSignStreamSegmentPanel().getSS3Database();
				
				// only save original state once per each search window session
				if (application.getSignStreamSegmentPanel().isUtteranceLoaded()) {
					originalSavedState = application.getSignStreamSegmentPanel().getUtteranceView().saveCurrentState();
				} else {	// if we are looking at database view when opening search panel
//					// generic saved state to point to the beginning of TP1;ST1;U1 in utterance view
				}
			}
		}
		
		searchEngine = new SearchEngine(database);
		
		searchSavedStates = new HashMap<Integer, SearchSavedState>();
		
		// utteranceView could be null here, which would indicate that we are looking at the databaseview
		
		createUI();
	}
	
	private void createUI() {
		SignStreamSegmentPanel segmentPanel = application.getSignStreamSegmentPanel();
		if (segmentPanel == null) {
			return;
		}
		
		searchDomain = new JLabel("Search Domain: " + database.getDirectory());
		
		changeDomainButton = new JButton("Change Search Domain");
		changeDomainButton.addActionListener(this);
		changeDomainButton.setActionCommand("" + CHANGE_DOMAIN);
		
		searchQueryBuilderPanel = new SearchQueryBuilderPanel(database);
		
		searchQueryTextPanel = new JTextArea();
		
		searchButton = new JButton("Search");
		searchButton.addActionListener(this);
		searchButton.setActionCommand("" + SEARCH);
		
		returnOriginalState = new JButton("Return To Original State");
		returnOriginalState.addActionListener(this);
		returnOriginalState.setActionCommand("" + RETURN_ORIGINAL_STATE);
		
		resultsList = new JList<String>();
		resultsListScroller = new JScrollPane(resultsList);
		resultsListScroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		
		resultsListScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		resultsList.setDragEnabled(false);
		resultsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		resultsListSelectionListener = new ResultsListSelectionListener(this, resultsList);
		resultsList.getSelectionModel().addListSelectionListener(resultsListSelectionListener);
		
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();

		c.gridy = 0;
		
		c.weightx = 1.0;
		c.gridwidth = 1;
		
		c.gridx = 0;
		c.anchor = GridBagConstraints.NORTHWEST;
		add(searchDomain, c);
		
		c.gridx = 1;	
		c.anchor = GridBagConstraints.NORTHEAST;
		add(changeDomainButton, c);
		
		c.gridx = 0;
		c.gridwidth = 2;		
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.BOTH;

		c.gridy++;
		JSeparator sep = new JSeparator(JSeparator.HORIZONTAL);
		sep.setPreferredSize(new Dimension(5, 5));
		add(sep, c);
		
		c.gridy++;
		add(searchQueryBuilderLabel, c);
		c.weighty = 0.8;
		c.gridy++;
		add(searchQueryBuilderPanel, c);

		c.weighty = 0;
		
		c.gridy++;
		c.fill = GridBagConstraints.NONE;
		add(searchButton, c);
		
		c.gridx = 1;
		c.anchor = GridBagConstraints.EAST;
		add(returnOriginalState, c);
		
		c.gridx = 0;
		c.anchor = GridBagConstraints.WEST;
		
		c.gridy++;
		c.weighty = 0.2;
		c.anchor = GridBagConstraints.SOUTH;
		c.fill = GridBagConstraints.BOTH;
		add(resultsListScroller, c);
	}
	
	private void saveCurrentState() {
		UtteranceView utteranceView = application.getSignStreamSegmentPanel().getUtteranceView();
		if (utteranceView == null) 
			return;
		
		SearchSavedState tempState = utteranceView.saveCurrentState();
		
		if (currentResultIndexVisible > -1) {
			searchSavedStates.put(currentResultIndexVisible, tempState);
		}
	}
	
	private void revertToSavedState(SearchSavedState searchSavedState) {
		
		SignStreamSegmentPanel panel = application.getSignStreamSegmentPanel();
		
		SS3Database savedStateDatabase = searchSavedState.getDatabase();
		
		if ( !savedStateDatabase.equals(database)) {
			
			// must perform this first - in case we don't actually want to switch then stop here
			boolean success = changeDomain(savedStateDatabase);
			
			if (success) {
				panel.closeAllLoadedVideoFiles();
				
				// update db nav here because we did not open this savedStateDatabase from zipfile
				// that logic is in application.openDatabaseSS3();
				
				resetPanelToDatabase(panel, savedStateDatabase);
			}	
		}
		
		UtteranceView utteranceView = panel.getUtteranceView();
		if (utteranceView == null)
			return;
		
		utteranceView.revertToSavedState(searchSavedState);
	}
	
	private void loadSearchResult() {
		
		SignStreamSegmentPanel panel = application.getSignStreamSegmentPanel();
		
		if ( !panel.getSS3Database().equals(database)) {
			if (database != null) {
				panel.closeAllLoadedVideoFiles();
			}
		}
		
		resetPanelToDatabase(panel, database);
		
		UtteranceView utteranceView = panel.getUtteranceView();
		if (utteranceView == null)
			return;

		Manager mngr = SS3Singleton.getManager();
		RequestHandler handler = mngr.getRequestHandler();
		handler.init(panel);
		
		SearchSavedState selectedSavedState = searchSavedStates.get(currentResultIndexVisible);
		
		if (selectedSavedState != null) {
			revertToSavedState(selectedSavedState);
			return;
		} else {			
			utteranceView.parseSegmentStringAndReloadView(currentResultVisible);
		}
	}
	
	private void resetPanelToDatabase(SignStreamSegmentPanel panel, SS3Database database) {
		// calls segmentPanel.update() from within if db is new
		panel.loadDatabase(database);		
		
		// block below is to set the proper zooming before loading the utterance
		// copied code from SegmentFieldWrapper.mouseCntrAltDblLeftClicked()
        double w = panel.getDatabaseView().getVisibleWidth2();
		long packetDuration = SS3Singleton.getVideoControlManager().getMultipleMovieController().getPacketDuration();
		int clipDuration = 100000; // since we have no specfiic event's timeframe, just use a static value each time
		
		Zoomer zoomer = panel.getZoomer();
		double dbPixelInTimeFrame = zoomer.getCoordinateTimeConvertor().getPixelInTimeFrame();
		database.setTemporalPartitionPixelInTimeFrame(dbPixelInTimeFrame);
		
		double pixelsInTimeFrame = zoomer.getPixelsInTimeFrame();
		
		double adder = .02;
		double factor = 8.01;
		
		if ( ! zoomer.getCoordinateTimeConvertor().isTimeScaleMPEG()) {
			adder = zoomer.getMinPixelInTimeFramePresentation();
			factor = zoomer.getMaxPixelInTimeFramePresentation()
						- zoomer.getMinPixelInTimeFramePresentation();
		}
		
		double value = (pixelsInTimeFrame - adder) * 100 / factor;
		// somewhat hackish - load an empty utterance view in order to reset the utteranceView reference inside the segmentPanel below
		// and also to create the panel infrastructure which the later calls (revert/parse) require
		panel.loadUtterance(new ArrayList());	

		panel.getZoomer().fillEntireUtterancePage(clipDuration, (int)packetDuration, w);
		
		if (panel.getUtteranceView() != null)
			panel.getUtteranceView().setZoomInOutValue((int)value);
		
		panel.getParentFrame().toFront();
	}
	
	private ArrayList<SS3GlossWindowItem> search(SearchTreeNode searchTreeNode) {
		return searchEngine.search(searchTreeNode);
	}
	
	private boolean changeDomain(SS3Database optionalDatabase) {
		// first ask the user whether they want to save the database open in the SSSP
		String warningDialogText =
				"You are about to open a new SignStream collection (in this search window).\n\n"
				+ "Do you wish to save the collection currently open in the main\n\n"
				+ "SignStream window before proceeding?\n\n";
		String warningTitleText = "WARNING";
		int optionType = JOptionPane.YES_NO_OPTION;
		int messageType = JOptionPane.WARNING_MESSAGE;
		String options[] = {"Save", "Do Not Save"};
		
		int result = JOptionPane.NO_OPTION;
		try {
			result = JOptionPane.showOptionDialog(this, warningDialogText,	warningTitleText, 
					optionType, 	messageType,	null, options, options[1]);
		} catch (HeadlessException he) {
			SS3SignStreamApplication.logger.log(Level.INFO, he.getMessage());
		}

		// if user chose to save the open database
		if (result == JOptionPane.OK_OPTION) {
			// need to first grab the DBnav node if we want to save the database
			Object root = application.getSignStreamSegmentPanel().getDbNavigationPanel().getJTree().getModel()
					.getRoot();

			if (root == null)
				return false;

			DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
			int childCount = rootNode.getChildCount();
			DefaultMutableTreeNode node = null;

			if (childCount > 0) {
				node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

				if (node.getChildCount() > 0) {
					DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
							.getChildAt(0);
					Object uo = dNode.getUserObject();
					if (uo instanceof String) {
						node = dNode;
					}
				}
			}

			if (node == null) {
				application.confirmationDialog("Save Collection Warning",
						"Collection not available");
				return false;
			}
			
			// save currently open database
			application.saveLoadedDatabase(node);
		}
			
		// switch databases
		if (optionalDatabase != null) {
			database = optionalDatabase;
		} else {
			SS3Database openedDatabase = application.openDatabaseSS3();
			if (openedDatabase != null)
				database = openedDatabase;
		}

		searchDomain.setText("Search Domain: " + database.getDirectory());
		
		updateQueryBuilderPanel();
		
		searchEngine = new SearchEngine(database);
		
		return true;
	}
	
	public SearchTreeNode getQueryBuilderPanelRootNode() {
		return searchQueryBuilderPanel.getRootSearchTreeNode();
	}
	
	public void updateQueryBuilderPanel() {
		searchQueryBuilderPanel.updateQuery(database);
	}
	
	public String getQueryTextPanelText() {
		return searchQueryTextPanel.getText();
	}
	
	public void updateQueryTextPanel(String text) {
		
	}
	
	private void updateResultPanel(ArrayList<SS3GlossWindowItem> resultItems) {
		ArrayList<String> utterances = new ArrayList<String>();
		
		for (int i = 0; i < resultItems.size(); i++) {
			SS3GlossWindowItem item = resultItems.get(i);			
			String fieldID = DOMINANT_GLOSS_FIELD_ID;
			String combinedTextLabels = "  ";
			
			if (item.getSS3StatementField(fieldID) != null) {
				// entities are in order by startTime
				ArrayList<SS3Entity> ss3Entities = item.getSS3StatementField(fieldID).getSS3Entities();
				
				for (SS3Entity ss3Entity: ss3Entities) {
					ArrayList<SS3Item> entItems = ss3Entity.getItems();
					for (SS3Item entItem: entItems) {
						// if entItem.isTextOnly() gave back the correct/up-to-date result, then it would be used here instead
						String text = entItem.getText();
						if (text.equals("Onset") || text.equals("Offset")
								|| text.equals("Initial Hold") || text.equals("Final Hold"))
							continue;
						
						combinedTextLabels += entItem.getText() + " ";
					}
				}
			}
			
			String temporalPartitionLabel = item.getPartitionLabel();
			String segmentLabel = item.getSegmentLabel();
			String utteranceLabel = item.getLabel();
			
			String combinedItemLabel = temporalPartitionLabel + ";" + segmentLabel + ";" + utteranceLabel;
			
			utterances.add(combinedItemLabel + combinedTextLabels);
		}
		
		resultsList.setListData(Arrays.copyOf(utterances.toArray(), utterances.size(), String[].class));
		
		// clear out the old
		searchSavedStates.clear();
		setCurrentResultVisible(null);
		setCurrentResultIndexVisible(-1);
	}
	
	private void clearResultPanel() {
		// pass in empty arraylist to clear the entire results list
		updateResultPanel(new ArrayList());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		switch (Integer.parseInt(str)) {
		
		case CHANGE_DOMAIN:
			
			changeDomain(null);
			
			/*
			 * update the query panel and query box, to reflect the new database
			 */			
			
			clearResultPanel();
		
			break;
			
		case SEARCH:			
			updateResultPanel(search(getQueryBuilderPanelRootNode()));
			
			break;
			
		case RETURN_ORIGINAL_STATE:
			if (originalSavedState != null) {
				saveCurrentState();
				revertToSavedState(originalSavedState);
				resultsList.clearSelection();
			}
			
			// need to reset these here in case we are switching back to original state from within the same database
			// in that case, we shouldn't mistake the original state for an element of the searchSavedStates map
			setCurrentResultVisible(null);
			setCurrentResultIndexVisible(-1);
			
			break;
		
		}
	}
	
	public void setCurrentResultVisible(String currentResultVisible) {
		this.currentResultVisible = currentResultVisible;
	}
	
	public String getCurrentResultVisible() {
		return currentResultVisible;
	}
	
	public void setCurrentResultIndexVisible(int currentResultIndexVisible) {
		this.currentResultIndexVisible = currentResultIndexVisible;
	}
	
	public int getCurrentResultIndexVisible() {
		return currentResultIndexVisible;
	}

	private class ResultsListSelectionListener implements ListSelectionListener {
		private SearchPanel searchPanel = null;
		private JList resultsList = null;
	
		public ResultsListSelectionListener(SearchPanel searchPanel, JList resultsList) {
			this.searchPanel = searchPanel;
			this.resultsList = resultsList;
		}
	
		@Override
		public void valueChanged(ListSelectionEvent e) {
			// this will proceed once the value is not longer adjusting, meaning the mouse click is released
			if (e.getValueIsAdjusting() == true)
				return;
			
	        ListSelectionModel resultsListSelectionModel = (ListSelectionModel) e.getSource();
	        
	        if (resultsListSelectionModel.isSelectionEmpty()) {
	        		return;
	        } else {
	        		int resultsListIndex = resultsListSelectionModel.getMaxSelectionIndex();
	        		String resultsListString = (String) resultsList.getModel().getElementAt(resultsListIndex);
	        		
	        		int currentIndexVisible = searchPanel.getCurrentResultIndexVisible();
	        		
	        		if (resultsListIndex == currentIndexVisible) {
	        			// do nothing, since we have selected the result we are already viewing
	        			return;
	        		} else {
	        			searchPanel.saveCurrentState();
	        			
	        			searchPanel.setCurrentResultIndexVisible(resultsListIndex);
	        			searchPanel.setCurrentResultVisible(resultsListString);
	        			
	        			searchPanel.loadSearchResult();	        			
	        		}
	        }
			
		}
		
	}

}
