package edu.bu.signstream.ui.panels.signbank;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.signbank.SignBankResource;
import edu.bu.signstream.common.util.signbank.SignBankResourceCollection;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossClassifications;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankMainGloss;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankVariant;

public class SignBankAddVariantPanel extends JPanel implements ActionListener{
	
	private static final long serialVersionUID = 1L;
	
	private SignBankPanel parent = null;
	
	private SignBankCollection collection = null;
	private SignBankResourceCollection resourceCollection = null;
	
	private String domStartHS, nondomStartHS, domEndHS, nondomEndHS;
	private String signType, participant, twoHanded, passiveBaseArm, markedHands;
	private String leftRightHS, startEndHS, startEndRightHS, startEndLeftHS;
	
	private JLabel instructionsLabel = null;
	private JLabel variantLabel = null;
	private JLabel mainGlossLabel = null;
	private JLabel resourceLabel = null;
	
	private JTextArea variantTextArea = null;
	private JTextArea mainGlossTextArea = null;
	private JTextArea extraDataText = null;
	
	private JComboBox resourceSelector = null;
	
	private JButton addButton = null;
	
	private final int ADD = 10;

	public SignBankAddVariantPanel(SignBankPanel parent, SignBankCollection collection) {
		this.parent = parent;
		this.collection = collection;
		this.resourceCollection = collection.getSignBankResourceCollection();
		
		createUI();
	}
	
	private void createUI() {
		
		instructionsLabel = new JLabel("<html><b>Enter the appropriate labels for this new gloss.</b></html>");
		variantLabel = new JLabel("Sign Variant Name:");
		mainGlossLabel = new JLabel("Main Sign Name:");
		resourceLabel = new JLabel("Local Sign Bank File:");
		
		variantTextArea = new JTextArea();
		mainGlossTextArea = new JTextArea();
		extraDataText = new JTextArea();
		extraDataText.setEditable(false);
		
		ArrayList<SignBankResource> editableResourcesList = resourceCollection.getEditableResourcesList();
		ArrayList<String> resourceFilenames = new ArrayList<String>();
		for (SignBankResource resource: editableResourcesList) {
			resourceFilenames.add(resource.getSignBankFileName());
		}
		resourceSelector = new JComboBox(resourceFilenames.toArray());
		
		addButton = new JButton("Add to Sign Bank");
		addButton.addActionListener(this);
		addButton.setActionCommand("" + ADD);
		
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.fill = GridBagConstraints.BOTH;
		c.gridy = 0;
		add(instructionsLabel, c);
		c.gridy++;
		add(variantLabel, c);
		c.gridy++;
		add(variantTextArea, c);
		c.gridy++;
		add(mainGlossLabel, c);
		c.gridy++;
		add(mainGlossTextArea, c);
		c.gridy++;
		add(resourceSelector, c);
		c.gridy++;
		add(extraDataText, c);
		c.gridy++;
		add(addButton, c);
		
	}
	
	/**
	 * 
	 * @param label
	 * @param glsEntity
	 * @param domStartHS
	 * @param nondomStartHS
	 * @param domEndHS
	 * @param nondomEndHS
	 */
	public void update(String label, GlossChainedEventsEntity glsEntity, boolean twoHandedSelected,
			String domStartHS, String nondomStartHS, String domEndHS, String nondomEndHS) {
		
		variantTextArea.setText("");
		
		this.domStartHS = domStartHS;
		this.nondomStartHS = nondomStartHS;
		this.domEndHS = domEndHS;
		this.nondomEndHS = nondomEndHS;
		
		String participant = "";
		String collection = "";
		
		GlossChainedEvent selectedTextEvent = (GlossChainedEvent) glsEntity.getTextValue();
		GlossClassifications glsClassification = selectedTextEvent.getGlossClassifications();
		GlossEntityProperties properties = glsEntity.getGlossEntityProperties();
		
		signType = null;
		int classification = glsClassification.getSignClassification();
		switch (classification) {
		case GlossClassifications.SIGN_LEXICAL:
			signType = "LEXICAL";
			break;
		case GlossClassifications.SIGN_FINGERSPELLED:
			signType = "FINGERSPELLED";
			break;
		case GlossClassifications.SIGN_LOAN:
			signType = "LOAN";
			break;
		case GlossClassifications.SIGN_NUMBERS:
			signType = "NUMBER";
			break;
		case GlossClassifications.SIGN_CLASSIFIER:
			signType = "CLASSIFIER";
			break;
		case GlossClassifications.SIGN_GESTURE:
			signType = "GESTURE";
			break;
		}
		
		boolean isPassiveBaseArmFlag = glsClassification.isPassiveBaseArm();
		passiveBaseArm = isPassiveBaseArmFlag ? "T" : "F";
		
		boolean markedHandsFlag = glsClassification.isMarkedNamberOfHands();
		markedHands = markedHandsFlag ? "T" : "F";
		
		// this applies to onehanded and twohanded alike
		if (domStartHS.equals(domEndHS)) {
			startEndRightHS = "SAME";
		} else {
			startEndRightHS = "DIFFERENT";
		}
		
		if (twoHandedSelected) {
			twoHanded = "T";
			
			// for twohanded, verify same/different for nondom hand
			if (nondomStartHS.equals(nondomEndHS)) {
				startEndLeftHS = "SAME";
			} else {
				startEndLeftHS = "DIFFERENT";
			}
			
			// left-right same/different compare start-to-start and end-to-end
			if(domStartHS.equals(nondomStartHS) && domEndHS.equals(nondomEndHS)) {
				leftRightHS = "SAME";
			} else {
				leftRightHS = "DIFFERENT";
			}
			
		} else {
			twoHanded = "F";
			
			// for one handed, there is no left hand so left start/end hs is empty
			startEndLeftHS = "";
		}
		
		// call it SAME start/end for the sign if both left and right are SAME
		if (startEndRightHS.equals("SAME") && startEndLeftHS != null && startEndLeftHS.equals("SAME")) {
			startEndHS = "SAME";
		} else {
			startEndHS = "DIFFERENT";
		}
		
		// lexical text box
		String twoHandedStr = "", passiveBaseArmStr = "", markedHandsStr = "";
		if (participant == null || participant.isEmpty())
			participant = "n/a";
		if (signType == null || participant.isEmpty()) 
			signType = "n/a";
		if (twoHanded == null || twoHanded.equals("F")) 
			twoHandedStr = "No";
		else
			twoHandedStr = "Yes";
		if (passiveBaseArm == null || passiveBaseArm.equals("F"))
			passiveBaseArmStr = "No";
		else
			passiveBaseArmStr = "Yes";
		if (markedHands == null || markedHands.equals("F")) 
			markedHandsStr = "No";
		else
			markedHandsStr = "Yes";
		
		String text = "";
		text += "Participant: " + participant;
		text += ", SignType: " + signType;
		text += ", \nTwoHanded: " + twoHandedStr;
		text += ", PassiveBaseArm: " + passiveBaseArmStr;
		text += ", \nMarkedHands: " + markedHandsStr;
		text += ", LeftRightHandShapes: " + leftRightHS;
		text += ", \nStartEndHandShapes: " + startEndHS;
		text += ", StartEndLeftHandShapes: " + startEndLeftHS;
		text += ", \nStartEndRightHandShapes: " + startEndRightHS;
		
		extraDataText.setText(text);
		
		mainGlossTextArea.setText(label);
		
		SwingUtilities.getWindowAncestor(this).pack();
	}
	
	public void addToSignBank() {
		
		HashMap<String, SignBankMainGloss> mainGlossesList = null;
		HashMap<String, SignBankVariant> variants = null;
		
		// save this for below when saving, if a new variant on an existing main gloss we want to append to it
		SignBankMainGloss mainGloss = null;

		String mainGlossStr = mainGlossTextArea.getText().toUpperCase();
		String variantStr = variantTextArea.getText().toUpperCase();
		
		// check first if this sign exists in collection already - we will disallow if so
		mainGlossesList = collection.getMainGlossesList();
		for (SignBankMainGloss currentMainGloss: mainGlossesList.values()) {
			if (currentMainGloss.getMainGlossLabel().equals(mainGlossStr)) {
				mainGloss = currentMainGloss;
			}
			
			variants = currentMainGloss.getVariantsList();
			if (variants.get(variantStr) != null) {
				String warningDialogText =
						"Sign already found in bank.\nPlease enter a different set of values";
				String warningTitleText = "Duplicate Sign";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String options[] = {"OK"};
				
				try {
					JOptionPane.showOptionDialog(this, 
							warningDialogText, 
							warningTitleText, 
							optionType, 
							messageType, 
							null, 
							options, 
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}
				
				return;
			}
		}
		
		SignBankResource resource;
		String resourceFilename = (String) resourceSelector.getSelectedItem();
		resource = resourceCollection.getResourceByFileName(resourceFilename);
		
		String mgID = Util.getUniqueNumber();
		String varID = Util.getUniqueNumber();
		String occID = Util.getUniqueNumber();
		
		// if we did not find an existing match above, then create a new main gloss object to save
		if (mainGloss == null) {
			mainGloss = new SignBankMainGloss(mgID, mainGlossStr, varID);
		}
		SignBankVariant newVariant = new SignBankVariant(mainGloss, varID, variantStr, occID);
		SignBankOccurrence newOccurrence = new SignBankOccurrence(newVariant, occID);
		
		newOccurrence.setSignBankResourceFileName(resourceFilename);
		newOccurrence.setDomStartHS(domStartHS);
		newOccurrence.setNondomStartHS(nondomStartHS);
		newOccurrence.setDomEndHS(domEndHS);
		newOccurrence.setNondomEndHS(nondomEndHS);
		newOccurrence.setParticipant(participant);
		newOccurrence.setSignType(signType);
		newOccurrence.setTwoHanded(twoHanded);
		newOccurrence.setPassiveBaseArm(passiveBaseArm);
		newOccurrence.setMarkedHands(markedHands);
		newOccurrence.setLeftRightHS(leftRightHS);
		newOccurrence.setStartEndHS(startEndHS);
		newOccurrence.setStartEndLeftHS(startEndLeftHS);
		newOccurrence.setStartEndRightHS(startEndRightHS);
		
		newVariant.getOccurrencesList().put(newOccurrence.getOccurrenceID(), newOccurrence);
		mainGloss.getVariantsList().put(newVariant.getVariantLabel(), newVariant);
		
		// no harm in overwriting the same main gloss with an additional variant
		collection.getMainGlossesList().put(mainGloss.getMainGlossLabel(), mainGloss);
		
		// write file now
		SS3Singleton.getSignBankWriter().writeResourceToSignBank(collection, resourceFilename);
		
		// re-parse the entire sign bank collection fresh, and reset references
		SignBankCollection newCollection = SS3Singleton.getSignStreamApplication().parseSignBank();
		parent.setSignBankCollection(newCollection);
		
		SwingUtilities.getWindowAncestor(this).dispose();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		switch (Integer.parseInt(str)) {
		
		case ADD:
			String mainGlossStr = mainGlossTextArea.getText();
			String variantStr = variantTextArea.getText();
			
			// first check empty labels
			if (mainGlossStr == null || mainGlossStr.isEmpty() || variantStr == null || variantStr.isEmpty()) {

				String warningDialogText =
						"Enter values for both variant and main gloss";
				String warningTitleText = "Enter Values";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String options[] = {"OK"};
				
				try {
					JOptionPane.showOptionDialog(this, 
							warningDialogText, 
							warningTitleText, 
							optionType, 
							messageType, 
							null, 
							options, 
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}
				
				return;
			}
			
			// then check to sanitize input text
			// disallow these characters: +  "  '  ,  !  <  >
			String regex = ".*[\\+\"',!<>].*";
			
			if (mainGlossStr.matches(regex) || variantStr.matches(regex)) {
				String warningDialogText =
						"Cannot enter these characters for main gloss or variant label:\n" +
						" +  \"  '  ,  !  <  >";
				String warningTitleText = "Invalid Label";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String options[] = {"OK"};
				
				try {
					JOptionPane.showOptionDialog(this, 
							warningDialogText, 
							warningTitleText, 
							optionType, 
							messageType, 
							null, 
							options, 
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}
				
				return;
			}
			
			addToSignBank();
			break;
		}
	}

	public void setSignBankCollection(SignBankCollection collection) {
		this.collection = collection;
	}
}
