package edu.bu.signstream.ui.panels.signbank;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
//import javax.swing.JScrollPane;
import javax.swing.tree.DefaultMutableTreeNode;

import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.AssociatedFieldsPanel;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;

public class SignBankPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private SignBankLookupPanel signBankLookupPanel = null;
	private SignBankResultsListPanel signBankResultsListPanel = null;
	private SignBankVariantPanel signBankVariantPanel = null;
	
	private JPanel topPanel = null;
	//private JScrollPane jsp = null;
	
	private static final double resultListToLookupPanelWidthRatio = 1.5;
	private static final double variantPanelToLookupPanelHeightRatio = 2.2;
	private static final double variantPanelWidth = 0.4;
	private static final double variantPanelHeight = 0.6;
	private static final double videoWidth = 1 - variantPanelWidth;
	private static final double videoHeight = variantPanelHeight;
	private static final double imageHeight = 1 - variantPanelHeight;
	private static final double horizontalPadding = 0.1;

	private SignBankCollection signBankCollection = null;
	
	public SignBankPanel(AssociatedFieldsPanel assocFieldsPanel, String variantLabel) {
		signBankCollection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
		
		signBankLookupPanel = new SignBankLookupPanel(this, assocFieldsPanel, variantLabel);
		signBankLookupPanel.setSignBankCollection(signBankCollection);
		
		signBankResultsListPanel = new SignBankResultsListPanel(this, assocFieldsPanel);
		
		signBankVariantPanel = new SignBankVariantPanel(this, assocFieldsPanel);
		
		createUI();
	}
	
	private void createUI() {
		
		//signBankLookupPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		signBankLookupPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.BLACK), BorderFactory.createEmptyBorder(0, 20, 0, 20)));
		
		Dimension dLookup = signBankLookupPanel.getPreferredSize();
		
		signBankResultsListPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		signBankResultsListPanel.setMinimumSize(new Dimension((int)(dLookup.width * resultListToLookupPanelWidthRatio), dLookup.height));
		signBankResultsListPanel.setMaximumSize(new Dimension((int)(dLookup.width * resultListToLookupPanelWidthRatio), dLookup.height));
		signBankResultsListPanel.setPreferredSize(new Dimension((int)(dLookup.width * resultListToLookupPanelWidthRatio), dLookup.height));
		
		topPanel = new JPanel();
		topPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		topPanel.add(signBankLookupPanel, c);
		c.gridx = 1;
		topPanel.add(signBankResultsListPanel, c);
		
		//jsp = new JScrollPane(signBankVariantPanel);
		//jsp.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		//jsp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		//jsp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		
		int width = (int) (dLookup.getWidth() * (1 + resultListToLookupPanelWidthRatio));
		int height = (int) (dLookup.getHeight() * variantPanelToLookupPanelHeightRatio);
		Dimension dVariant = new Dimension(width, height);
		//Dimension startingSize = new Dimension(width, 0);
		signBankVariantPanel.setPreferredSize(dVariant);
		signBankVariantPanel.setMinimumSize(dVariant);
		
		signBankVariantPanel.imageHeight = imageHeight;
		signBankVariantPanel.variantPanelHeight = variantPanelHeight;
		signBankVariantPanel.variantPanelWidth = variantPanelWidth;
		signBankVariantPanel.videoHeight = videoHeight;
		signBankVariantPanel.videoWidth = videoWidth;
		signBankVariantPanel.horizontalPadding = horizontalPadding;
		//jsp.setPreferredSize(dVariant);
		//jsp.setMinimumSize(dVariant);
		
		setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.gridy = 0;
		add(topPanel, c);
		c.gridy = 1;
		add(signBankVariantPanel , c);
	}
	
	public SignBankCollection getSignBankCollection() {
		return signBankCollection;
	}
	
	public void setSignBankCollection(SignBankCollection collection) {
		this.signBankCollection = collection;
		signBankLookupPanel.setSignBankCollection(collection);
	}
	
	public void updateResultsListPanel(DefaultMutableTreeNode rootNode) {
		signBankResultsListPanel.updateList(rootNode);
	}
	
	public void populateSignBankVariantPanel(SignBankOccurrence signBankOccurrence) {
		signBankVariantPanel.populate(signBankOccurrence);
	}
	
	public void setVariantPanelVisible(boolean val) {
		signBankVariantPanel.setVisible(val);
	}
}
