package edu.bu.signstream.ui.panels.signbank;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.signbank.SignBankResource;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.AssociatedFieldsPanel;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankResultNode;

public class SignBankResultsListPanel extends JPanel implements TreeSelectionListener, ActionListener {

	private static final long serialVersionUID = 1L;
	
	private SignBankPanel parent = null;
	
	private AssociatedFieldsPanel assocFieldsPanel = null;
	
	private JScrollPane jsp = null;
	
	private JTree resultsTree = null;
	private DefaultTreeCellRenderer resultsRenderer = null;
	
	private DefaultMutableTreeNode rootNode = null;
	private DefaultMutableTreeNode selectedNode = null;
	
	private JPanel labelDisplayPanel = null;
	
	private JButton deleteButton = null;
	private JButton insertButton = null;	
	
	private JLabel mainGlossLabel = null;
	private JLabel variantLabel = null;
	private JTextField variantLabelField = null;
	
	private final int DELETE = 10;
	private final int INSERT = 20;
	
	private final String insertPrimaryEntry = "Insert Labels of Primary Entry and Variant";
	private final String mainGlossLabelString = "Primary Entry: ";
	private final String variantLabelString = "Entry/Variant: ";
	
	public SignBankResultsListPanel(SignBankPanel parent, AssociatedFieldsPanel assocFieldsPanel) {
		this.parent = parent;
		this.assocFieldsPanel = assocFieldsPanel;
		
		createUI();
	}
	
	private void createTree(DefaultMutableTreeNode rootNode) {
		this.rootNode = rootNode;
		
		resultsTree = new JTree(rootNode);
		resultsTree.setCellRenderer(resultsRenderer);		
		resultsTree.addTreeSelectionListener(this);
		resultsTree.getSelectionModel().setSelectionMode
			(TreeSelectionModel.SINGLE_TREE_SELECTION);

		jsp = new JScrollPane(resultsTree);
		jsp.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		jsp.setName("Results:");
		jsp.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
	}

	private void createUI() {
		rootNode = new DefaultMutableTreeNode();
		selectedNode = rootNode;
		resultsRenderer = new MyRenderer();
		
		createTree(rootNode);
		
		int fontSize = 10;
		Font font = new Font("Arial", Font.PLAIN, fontSize);
		int fieldSpaces = 25;
		
		deleteButton = new JButton("Delete");
		deleteButton.setFont(font);
		deleteButton.addActionListener(this);
		deleteButton.setActionCommand("" + DELETE);
		
		insertButton = new JButton(insertPrimaryEntry);
		insertButton.setFont(font);
		insertButton.addActionListener(this);
		insertButton.setActionCommand("" + INSERT);
		
		mainGlossLabel = new JLabel(mainGlossLabelString);
		
		variantLabel = new JLabel(variantLabelString);
		
		variantLabelField = new JTextField(fieldSpaces);
		variantLabelField.setFont(font);
		variantLabelField.setText("<variant>");
		
		labelDisplayPanel = new JPanel();
		labelDisplayPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.gridy = 0;
		c.gridx = 0;
		c.insets = new Insets(0, 5, 0, 5);
		labelDisplayPanel.add(deleteButton, c);
		c.gridx = 1;
		labelDisplayPanel.add(insertButton, c);
		
		c.gridx = 0;
		c.gridwidth = 2;
		c.insets = new Insets(0, 0, 0, 0);
		c.anchor = GridBagConstraints.WEST;
		
		c.gridy++;
		labelDisplayPanel.add(mainGlossLabel, c);
		
		c.gridy++;
		labelDisplayPanel.add(variantLabel, c);
		c.gridx = 1;
		labelDisplayPanel.add(variantLabelField, c);
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(jsp);
		add(labelDisplayPanel);
		
		updateList(null);		
	}
	
	/**
	 * Update the existing tree list in place with the new root passed in
	 * @param root The new root to reset this tree with
	 */
	public void updateList(DefaultMutableTreeNode root) {	
		// if there are no results to display
		if (root == null || root.getChildCount() == 0) {
			root = new DefaultMutableTreeNode("No Results");
		}
		
		// reload the tree with the root passed into the function
		createTree(root);
		
		DefaultTreeModel model = (DefaultTreeModel) resultsTree.getModel();
		model.reload(root);
		
		//// NOTE: commenting this for now - because of the custom renderer with images this takes too long to display ////
		
		// expand all rows to display all variants	    
//		expandAll(resultsTree, true);
		
		// alternately: this way is less comprehensible but equally efficient (in terms of time it takes):
//		int j = resultsTree.getRowCount();
//	    int i = 0;
//	    while(i < j) {
//	    	System.out.println(i);
//	    		resultsTree.expandRow(i);
//	        i += 1;
//	        j = resultsTree.getRowCount();
//	    }
		
		removeAll();
		add(jsp);
		add(labelDisplayPanel);
		revalidate();
		repaint();
	}

	/**
	 * Expand all the node paths of the given JTree
	 * @param tree The JTree containing all of the sub-nodes to be expanded
	 * @param expand Flag to indicate whether to expand all nodes
	 */
	private void expandAll(JTree tree, boolean expand) {
		TreeNode root = (TreeNode)tree.getModel().getRoot();

		// Traverse tree from root
		expandAll(tree, new TreePath(root), expand);
	}

    /**
     * @return Whether an expandPath was called for the last node in the parent path
     */
    private boolean expandAll(JTree tree, TreePath parent, boolean expand) {
        // traverse children
        TreeNode node = (TreeNode)parent.getLastPathComponent();
        if (node.getChildCount() > 0) {
            boolean childExpandCalled = false;
            for (Enumeration e=node.children(); e.hasMoreElements(); ) {
                TreeNode n = (TreeNode)e.nextElement();
                TreePath path = parent.pathByAddingChild(n);
                childExpandCalled = expandAll(tree, path, expand) || childExpandCalled; 
                // the OR order is important here, don't let childExpand first. func calls will be optimized out !
            }

            if (!childExpandCalled) { // only if one of the children hasn't already called expand
                // expansion or collapse must be done bottom-up, BUT only for non-leaf nodes
                if (expand) {
                    tree.expandPath(parent);
                } else {
                    tree.collapsePath(parent);
                }
            }
            return true;
        } else {
            return false;
        }
    }

	/**
	 * Delete the currently selected occurrence node (the sign) and write its origin resource file back to disk.
	 * Also re-parse SignBankCollection object completely from all files, and update references.
	 */
	public void deleteSign() {
		if ( !(selectedNode instanceof SignBankResultNode)) {
			return;
		}
		
		String warningDialogText =
				"The selected sign will be permanently deleted from the local sign bank.\n\n"
				+ "Are you absolutely sure you want to delete this sign?";
		String warningTitleText = "WARNING";
		int optionType = JOptionPane.YES_NO_OPTION;
		int messageType = JOptionPane.ERROR_MESSAGE;
		String options[] = {"Proceed", "Cancel"};
		
		int result = -1;
		try {
			result = JOptionPane.showOptionDialog(this, 
					warningDialogText, 
					warningTitleText, 
					optionType, 
					messageType, 
					null, 
					options, 
					options[1]);
		} catch (HeadlessException he) {
			System.out.println("");
		}
		
		if (result != JOptionPane.OK_OPTION)
			return;
		
		SignBankResultNode signBankResultNode = (SignBankResultNode) selectedNode;
		SignBankOccurrence signBankOccurrence = (SignBankOccurrence) signBankResultNode.getUserObject();
		String resourceFileName = signBankOccurrence.getSIgnBankResourceFileName();
		SignBankCollection signBankCollection = parent.getSignBankCollection();
		
		// first check whether we can delete this occurrence, by validating it is from a local/editable resource
		SignBankResource resource = signBankCollection.getSignBankResourceCollection().getResourceByFileName(resourceFileName);
		boolean nonEditable = resource.isNonEditableResource();
		if (nonEditable) {
			warningDialogText =
					"This sign is global and cannot be deleted.";
			warningTitleText = "Global Sign";
			optionType = JOptionPane.YES_NO_OPTION;
			messageType = JOptionPane.ERROR_MESSAGE;
			String arr[] = {"OK"};
			
			try {
				JOptionPane.showOptionDialog(this, 
						warningDialogText, 
						warningTitleText, 
						optionType, 
						messageType, 
						null, 
						arr, 
						arr[0]);
			} catch (HeadlessException he) {
				System.out.println("");
			}
			
			return;
		}
		
		// first delete sign from local signbank memory collection
		signBankCollection.deleteSignBankOccurrence(signBankOccurrence);
		
		// write the resource file this sign came from back to disk 
		SS3Singleton.getSignBankWriter().writeResourceToSignBank(signBankCollection, resourceFileName);
		// TODO think about altenatively passing in resource object (and grab str from there)

		// re-parse the entire sign bank collection fresh, application class reference will be set
		SS3Singleton.getSignStreamApplication().parseSignBank();
		
		selectedNode = null;
		
		// update the tree, simulate re-pressing lookup on the newly parsed collection
		Enumeration<DefaultMutableTreeNode> mainGlossNodes = rootNode.children();
		while (mainGlossNodes.hasMoreElements()) {
			DefaultMutableTreeNode mainGlossNode = mainGlossNodes.nextElement();
			
			// if just one child, delete the main gloss node as well
			int childCount = mainGlossNode.getChildCount();
			if (childCount == 1 && mainGlossNode.getChildAt(0).equals(signBankResultNode)) {
				mainGlossNode.remove((MutableTreeNode) mainGlossNode.getChildAt(0));				
				rootNode.remove(mainGlossNode);
				
			} else {	// more than one child, don't delete the main gloss node
				int indexToDelete = -1;
				for (int i = 0; i < childCount; i++) {
					if (mainGlossNode.getChildAt(i).equals(signBankResultNode)) {
						indexToDelete = i;
						break;
					}
				}
				
				if (indexToDelete != -1) {
					mainGlossNode.remove(indexToDelete);		
					break;
				}
			}	
		}
		
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				updateList(rootNode);		
			}
		});
//		updateList(rootNode);		
	}
	
	/** 
	 * Take the currently selected node and check which type it is.
	 * If it is an occurrence node, grab the parent variant and parent main gloss, get its label.
	 * If it is a main gloss node, just get its label.
	 * Then in either case, take that label plus the label for the variant and pass it back to the MP.
	 */
	public void insert() {
		// ignore selection on rootNode
		if (selectedNode.equals(rootNode)) {
			return;
		}
		
		String mainGlossLabel = null;
		if (selectedNode instanceof SignBankResultNode) {	// first case, we have selected a sign bank result node
			SignBankResultNode signBankResultNode = (SignBankResultNode) selectedNode;
			DefaultMutableTreeNode mainGlossParentNode = (DefaultMutableTreeNode) signBankResultNode.getParent();
			mainGlossLabel = (String) mainGlossParentNode.getUserObject();
		} else {	// second case, we have selected a main gloss node 
			if (selectedNode.isRoot())
				return;
			
			mainGlossLabel = (String) selectedNode.getUserObject();
		}
		
		String variantLabel = variantLabelField.getText();
		
		assocFieldsPanel.populateMainGlossAndVariantLabels(mainGlossLabel, variantLabel);

		SwingUtilities.getWindowAncestor(this).dispose();
	}

	@Override
	public void valueChanged(TreeSelectionEvent e) {
		JTree tree = (JTree) e.getSource();
		this.selectedNode = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
		
		// ignore selection on rootNode
		if (selectedNode.equals(rootNode)) {
			return;
		}
		
		if ( !(selectedNode instanceof SignBankResultNode)) {
			String nodeMainGlossLabel = (String) selectedNode.getUserObject();
			variantLabelField.setText(nodeMainGlossLabel);
			mainGlossLabel.setText(mainGlossLabelString + nodeMainGlossLabel);
			
			return;
		}
		
		SignBankResultNode signBankResultNode = (SignBankResultNode) selectedNode;
		SignBankOccurrence signBankOccurrence = (SignBankOccurrence) signBankResultNode.getUserObject();
		
		String variantLabel = signBankOccurrence.getParentVariant().getVariantLabel();
		variantLabelField.setText(variantLabel);
		String nodeMainGlossLabel = signBankOccurrence.getParentVariant().getParentMainGloss().getMainGlossLabel();
		mainGlossLabel.setText(mainGlossLabelString + nodeMainGlossLabel);
		
		parent.setVariantPanelVisible(true);
		parent.populateSignBankVariantPanel(signBankOccurrence);		
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		switch (Integer.parseInt(str)) {
		
		case INSERT:
			insert();
			break;
			
		case DELETE:
			deleteSign();
			break;
			
		}
	}
	
	private class MyRenderer extends DefaultTreeCellRenderer {

		private static final long serialVersionUID = 1L;
		
		@Override
		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel,
				boolean expanded, boolean leaf, int row, boolean hasFocus) {

	        super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
			
			if (leaf && value instanceof SignBankResultNode) {
				SignBankResultNode node = (SignBankResultNode) value;
				SignBankOccurrence occurrence = (SignBankOccurrence) node.getUserObject();
//				System.out.println(occurrence);	// debugging

				String hsDir = "handShapes/";
				String domStartHSFilePath = "", domEndHSFilePath = "", nondomStartHSFilePath = "", nondomEndHSFilePath = "";

				String domStartHS = occurrence.getDomStartHS();
				String domEndHS = occurrence.getDomEndHS();
				String nondomStartHS = occurrence.getNondomStartHS();
				String nondomEndHS = occurrence.getNondomEndHS();

		        SS3CodingScheme cs = SS3Singleton
		                .getSS3CodingSchemeCollection().getDefaultCodingScheme();
		        
		        String rightHandShapeFieldID = "1000";
		        String leftHandShapeFieldID = "2000";
		        
		        SS3Field rightHandShapeField = cs.getField(rightHandShapeFieldID);
		        SS3Field leftHandShapeField = cs.getField(leftHandShapeFieldID);
		        
				ArrayList<SS3FieldValue> rightFieldValues = cs.getFieldImg(rightHandShapeFieldID);
				ArrayList<SS3FieldValue> leftFieldValues = cs.getFieldImg(leftHandShapeFieldID);
		        
		        SS3FieldValue rightHandStartValue = null, rightHandEndValue = null, leftHandStartValue = null, leftHandEndValue = null;

				// right start hs		
				rightHandStartValue = Util.matchSignBank(rightFieldValues, domStartHS);
				
				// right end hs
				rightHandEndValue = Util.matchSignBank(rightFieldValues, domEndHS);

				// left start hs
				leftHandStartValue = Util.matchSignBank(leftFieldValues, nondomStartHS);

				// left end hs
				leftHandEndValue = Util.matchSignBank(leftFieldValues, nondomEndHS);
	            
	            SS3FieldTypeImg domFirstImage = null, domLastImage = null, nondomFirstImage = null, nondomLastImage = null;
	            
	            if (rightHandStartValue != null) {
	                domFirstImage = cs.getImage(rightHandStartValue.getIid());
	                domStartHSFilePath = hsDir + domFirstImage.getPath1();
	            }
	            
	            if (rightHandEndValue != null) {
	                domLastImage = cs.getImage(rightHandEndValue.getIid());
	                domEndHSFilePath = hsDir + domLastImage.getPath1();
	            }
	            
	            if (leftHandStartValue != null) {
	                nondomFirstImage = cs.getImage(leftHandStartValue.getIid());
	                nondomStartHSFilePath = hsDir + nondomFirstImage.getPath1();
	            }
	            
	            if (leftHandEndValue != null) {
	                nondomLastImage = cs.getImage(leftHandEndValue.getIid());
	                nondomEndHSFilePath = hsDir + nondomLastImage.getPath1();
	            }
	            
	            BufferedImage mergedImage = mergeImagesByPath(domStartHSFilePath, nondomStartHSFilePath,
	            			domEndHSFilePath, nondomEndHSFilePath);	
	            
	            if (mergedImage == null)
	            		return this;
				
				setIcon(new ImageIcon(mergedImage));
			}
			
			return this;
		}

		public BufferedImage mergeImagesByPath(String domStartHSFilePath, String nondomStartHSFilePath,
				String domEndHSFilePath, String nondomEndHSFilePath) {
			
			if (domStartHSFilePath == null || domEndHSFilePath == null)
				return null;
			
			// first get BufferedImage objects from file paths
			BufferedImage img1 = null, img2 = null, img3 = null, img4 = null;

			File domStartHSFile = new File(domStartHSFilePath);
			if (domStartHSFile.exists() && !domStartHSFile.isDirectory()) {
				try {
					img1 = ImageIO.read(domStartHSFile);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

			File nondomStartHSFile = new File(nondomStartHSFilePath);
			if (nondomStartHSFile.exists() && !nondomStartHSFile.isDirectory()) {
				try {
					img2 = ImageIO.read(nondomStartHSFile);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

			File domEndHSFile = new File(domEndHSFilePath);
			if (domEndHSFile.exists() && !domEndHSFile.isDirectory()) {
				try {
					img3 = ImageIO.read(domEndHSFile);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

			File nondomEndHSFile = new File(nondomEndHSFilePath);
			if (nondomEndHSFile.exists() && !nondomEndHSFile.isDirectory()) {
				try {
					img4 = ImageIO.read(nondomEndHSFile);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			
			int imgSmallWidth = 25;
			int imgSmallHeight = 20;
			
			BufferedImage img1Small = new BufferedImage(imgSmallWidth, imgSmallHeight, BufferedImage.TYPE_INT_ARGB);
			BufferedImage img2Small = new BufferedImage(imgSmallWidth, imgSmallHeight, BufferedImage.TYPE_INT_ARGB);
			BufferedImage img3Small = new BufferedImage(imgSmallWidth, imgSmallHeight, BufferedImage.TYPE_INT_ARGB);
			BufferedImage img4Small = new BufferedImage(imgSmallWidth, imgSmallHeight, BufferedImage.TYPE_INT_ARGB);
			
			Graphics2D img1SmallGraphics = img1Small.createGraphics();
			if (img1 != null) {
				img1SmallGraphics.drawImage(img1.getScaledInstance(
						imgSmallWidth, 10, Image.SCALE_SMOOTH), 0, 0, imgSmallWidth, imgSmallHeight, null);
			}
			img1SmallGraphics.dispose();
			
			Graphics2D img2SmallGraphics = img2Small.createGraphics();
			if (img2 != null) {
				img2SmallGraphics.drawImage(img2.getScaledInstance(
						imgSmallWidth, 10, Image.SCALE_SMOOTH), 0, 0, imgSmallWidth, imgSmallHeight, null);				
			}
			img2SmallGraphics.dispose();
			
			Graphics2D img3SmallGraphics = img3Small.createGraphics();
			if (img3 != null) {
				img3SmallGraphics.drawImage(img3.getScaledInstance(
						imgSmallWidth, 10, Image.SCALE_SMOOTH), 0, 0, imgSmallWidth, imgSmallHeight, null);	
			}
			img3SmallGraphics.dispose();
			
			Graphics2D img4SmallGraphics = img4Small.createGraphics();
			if (img4 != null) {
				img4SmallGraphics.drawImage(img4.getScaledInstance(
						imgSmallWidth, 10, Image.SCALE_SMOOTH), 0, 0, imgSmallWidth, imgSmallHeight, null);	
			}
			img4SmallGraphics.dispose();
						

			// width and height
			int offset  = 1;
			int width = img1Small.getWidth() + img2Small.getWidth()  + img3Small.getWidth() + img4Small.getWidth() + offset;
			int height = Math.max(img1Small.getHeight(), img2Small.getHeight()) + offset;

			// create a new buffered image and draw two image into the new image
			BufferedImage newImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
			
			newImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
			Graphics2D g2 = newImage.createGraphics();
			Color oldColor = g2.getColor();

			// fill background	
			g2.setPaint(Color.WHITE);
			g2.fillRect(0, 0, width, height);

			// draw image
			g2.setColor(oldColor);
			g2.drawImage(img1Small, null, 0, 0);
			g2.drawImage(img2Small, null, (img1Small.getWidth() + offset), 0);
			g2.drawImage(img3Small, null, (img1Small.getWidth() + img2Small.getWidth() + offset*5),  0);
			g2.drawImage(img4Small, null, (img1Small.getWidth() + img2Small.getWidth() + img3Small.getWidth() + offset*6), 0);
			g2.dispose();

			return newImage;
		}
		
	}	// private class 

}
