package edu.bu.signstream.ui.panels.signbank;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;

import com.xuggle.xuggler.Global;
import com.xuggle.xuggler.ICodec;
import com.xuggle.xuggler.IContainer;
import com.xuggle.xuggler.IPacket;
import com.xuggle.xuggler.IPixelFormat;
import com.xuggle.xuggler.IStream;
import com.xuggle.xuggler.IStreamCoder;
import com.xuggle.xuggler.IVideoPicture;
import com.xuggle.xuggler.IVideoResampler;
import com.xuggle.xuggler.video.ConverterFactory;
import com.xuggle.xuggler.video.IConverter;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.AssociatedFieldsPanel;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.video.MultipleMovieController;
import edu.bu.signstream.ui.video.XugglerMovie;

public class SignBankVariantPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	
	private SignBankPanel parent = null;
	
	private SignBankOccurrence currentlyDisplayingOccurrence = null;
	
	private AssociatedFieldsPanel assocFieldsPanel = null;
	
	private JLabel startHSLabel = null;
	private JLabel endHSLabel = null;
	private JLabel domStartHSLabel = null;
	private JLabel nondomStartHSLabel = null;
	private JLabel domEndHSLabel = null;
	private JLabel nondomEndHSLabel = null;
	
	private JPanel domStartHSPanel = null;
	private JPanel nondomStartHSPanel = null;
	private JPanel domEndHSPanel = null;
	private JPanel nondomEndHSPanel = null;
	private JPanel framesPanel = null;
	
	private JPanel variantDataPanel = null;
	private JPanel videoPanel = null;
	
	private BufferedImage domStartHSImage = null;
	private BufferedImage nondomStartHSImage = null;
	private BufferedImage domEndHSImage = null;
	private BufferedImage nondomEndHSImage = null;
	private BufferedImage framesImage = null;
	
	private JLabel mainGlossLabel = null;
	private JLabel variantLabel = null;
    private JLabel frameImagesLabel = null;
	
	private JButton insertOccurrence = null;
	private JButton playVideoButton = null;
	private MultipleMovieController_SB controller = null;
	private XugglerMovie_SB xugglerMovie = null;
	private CardMediaPanel xugglerVideoPanel = null;
	private ArrayList<MultipleMovieController_SB> controllerList = new ArrayList<MultipleMovieController_SB>();
	private JTextArea extraDataText = null;
	private JScrollPane extraDataTextScrollPane = null;
	private JPanel extraDataContainer = null;
	
	private JPanel mainGlossLabelPanel = null;
	private JPanel variantLabelPanel = null;
	private JPanel hsPanelRow = null;
	
	private String participant = null;
	private String collection = null;
	private String signType = null;
	private String twoHanded = null;
	private String passiveBaseArm = null;
	private String markedHands = null;
	private String leftRightHS = null;
	private String startEndHS = null;
	private String startEndLeftHS = null;
	private String startEndRightHS = null;	

	private final int INSERT = 10;
	private final int PLAY_VIDEO = 20;
	
	private final String insertEntryVariant = "Insert All Data";
	
	//Percentage width/height each panel should take up in variant panel.
	//These values are set by the Sign Bank Panel.
	protected double variantPanelWidth;
	protected double variantPanelHeight;
	protected double videoWidth;
	protected double videoHeight;
	protected double imageHeight;
	protected double horizontalPadding;
	
	public SignBankVariantPanel(SignBankPanel parent, AssociatedFieldsPanel assocFieldsPanel) {
		this.parent = parent;
		this.assocFieldsPanel = assocFieldsPanel;
		
		createUI();
	}
	
	private void createUI() {

		startHSLabel = new JLabel("  Start HS  ");
		domStartHSLabel = new JLabel("  Dom  ");
		nondomStartHSLabel = new JLabel(  "Non-Dom  ");
		
		endHSLabel = new JLabel("  End HS  ");
		domEndHSLabel = new JLabel("  Dom  ");
		nondomEndHSLabel = new JLabel("  Non-Dom  ");
		
		frameImagesLabel = new JLabel("Frame Images");
		frameImagesLabel.setAlignmentX(CENTER_ALIGNMENT);
		
		domStartHSPanel = new JPanel();
		configureHSPanel(domStartHSPanel);
		
		nondomStartHSPanel = new JPanel();
		configureHSPanel(nondomStartHSPanel);
		
		domEndHSPanel = new JPanel();
		configureHSPanel(domEndHSPanel);
		
		nondomEndHSPanel = new JPanel();
		configureHSPanel(nondomEndHSPanel);
		
		JPanel framesContainerPanel = new JPanel();
		framesContainerPanel.setLayout(new BoxLayout(framesContainerPanel, BoxLayout.Y_AXIS));
		framesPanel = new JPanel();
		framesPanel.setLayout(new BoxLayout(framesPanel, BoxLayout.X_AXIS));
		framesPanel.setAlignmentX(CENTER_ALIGNMENT);
		framesContainerPanel.add(framesPanel);
		
		mainGlossLabel = new JLabel("Primary Entry: n/a");
		variantLabel = new JLabel("Entry/Variant: n/a");
		
		insertOccurrence = new JButton(insertEntryVariant);
		insertOccurrence.addActionListener(this);
		insertOccurrence.setActionCommand("" + INSERT);
		insertOccurrence.setVisible(false);
		
		playVideoButton = new JButton("Play Video");
		playVideoButton.addActionListener(this);
		playVideoButton.setActionCommand("" + PLAY_VIDEO);
		playVideoButton.setVisible(true);
		
		xugglerVideoPanel = new CardMediaPanel("Sign Videos");
		
		extraDataText =  new JTextArea();
		extraDataText.setEditable(false);
		extraDataText.setMargin(new Insets(10,10,0,10));
		extraDataTextScrollPane = new JScrollPane(extraDataText);
		extraDataTextScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		extraDataContainer = new JPanel();
		extraDataContainer.setLayout(new BorderLayout());
		extraDataContainer.add(extraDataTextScrollPane, BorderLayout.CENTER);
		extraDataContainer.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		// HS PANEL		
		hsPanelRow = new JPanel();
		JPanel startHSPanel = new JPanel();
		JPanel endHSPanel = new JPanel();
		
		startHSPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.gridwidth = 2;
		
		c.gridx = 0;
		c.gridy = 0;
		startHSPanel.add(startHSLabel, c);
		
		c.gridwidth = 1;
		
		c.gridy++;
		c.gridx = 0;
		startHSPanel.add(domStartHSLabel, c);
		c.gridx = 1;
		startHSPanel.add(nondomStartHSLabel, c);

		c.gridy++;
		c.gridx = 0;
		startHSPanel.add(domStartHSPanel, c);
		c.gridx = 1;
		startHSPanel.add(nondomStartHSPanel, c);
				
		
		endHSPanel.setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		
		c.gridwidth = 2;
		
		c.gridx = 0;
		c.gridy = 0;
		endHSPanel.add(endHSLabel, c);
		
		c.gridwidth = 1;
		
		c.gridy++;
		c.gridx = 0;
		endHSPanel.add(domEndHSLabel, c);
		c.gridx = 1;
		endHSPanel.add(nondomEndHSLabel, c);

		c.gridy++;
		c.gridx = 0;
		endHSPanel.add(domEndHSPanel, c);
		c.gridx = 1;
		endHSPanel.add(nondomEndHSPanel, c);
		
		
		hsPanelRow.setLayout(new GridLayout(1, 2));
		hsPanelRow.add(startHSPanel);
		hsPanelRow.add(endHSPanel);
		
		// VARIANT DATA PANEL
		variantDataPanel = new JPanel();
		variantDataPanel.setLayout(new BoxLayout(variantDataPanel, BoxLayout.Y_AXIS));
		
		mainGlossLabelPanel = new JPanel();
		mainGlossLabelPanel.add(mainGlossLabel);
		mainGlossLabelPanel.setAlignmentX(CENTER_ALIGNMENT);
		
		variantLabelPanel = new JPanel();
		variantLabelPanel.add(variantLabel);
		variantLabelPanel.setAlignmentX(CENTER_ALIGNMENT);
		
		extraDataContainer.setAlignmentX(CENTER_ALIGNMENT);
		insertOccurrence.setAlignmentX(CENTER_ALIGNMENT);
		
		mainGlossLabelPanel.setMaximumSize(mainGlossLabelPanel.getPreferredSize());
		variantLabelPanel.setMaximumSize(variantLabelPanel.getPreferredSize());
		hsPanelRow.setMaximumSize(hsPanelRow.getPreferredSize());
		
		variantDataPanel.add(mainGlossLabelPanel);
		variantDataPanel.add(variantLabelPanel);
		variantDataPanel.add(hsPanelRow);
		variantDataPanel.add(extraDataContainer);
		variantDataPanel.add(insertOccurrence);
		//variantDataPanel.add(frameImagesLabel);
		//variantDataPanel.add(framesPanel);
		
		// VIDEO PANEL
		videoPanel = new JPanel();
		videoPanel.setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.CENTER;
		
		c.gridx = 0;
		c.gridy = 0;
		
		c.gridy++;
		
		c.gridy++;
		videoPanel.add(xugglerVideoPanel, c);

		setLayout(new BorderLayout());
		
		JPanel center = new JPanel();
		center.setLayout(new BoxLayout(center, BoxLayout.X_AXIS));
		variantDataPanel.setBorder(BorderFactory.createLineBorder(Color.black));
		videoPanel.setBorder(BorderFactory.createLineBorder(Color.black));
		center.add(variantDataPanel);
		center.add(videoPanel);
		
		add(center, BorderLayout.CENTER);
		add(framesContainerPanel, BorderLayout.SOUTH);
		
		setVisible(false);
		
	}
	
	private void configureHSPanel(JPanel panel) {
		int size = 50;
		panel.setSize(new Dimension(size, size));
		panel.setMaximumSize(new Dimension(size, size));
		panel.setMinimumSize(new Dimension(size, size));
		panel.setPreferredSize(new Dimension(size, size));			
		panel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
	}
	
	public SignBankOccurrence getCurrentlyDisplayingOccurrence() {
		return currentlyDisplayingOccurrence;
	}
	
	public void populate(SignBankOccurrence occurrence) {
		
		//Show the Sign Bank Variant Panel if hidden.
		variantDataPanel.setMaximumSize(new Dimension((int)(getMinimumSize().width * variantPanelWidth), (int)(getMinimumSize().height * variantPanelHeight)));
		variantDataPanel.setPreferredSize(new Dimension((int)(getMinimumSize().width * variantPanelWidth), (int)(getMinimumSize().height * variantPanelHeight)));
		videoPanel.setMinimumSize(new Dimension((int)(getMinimumSize().width * videoWidth), (int)(getMinimumSize().height * videoHeight)));
		SwingUtilities.getWindowAncestor(this).pack();
		
		this.currentlyDisplayingOccurrence = occurrence;
		
		ArrayList<String> frameImageURLs = occurrence.getFrameImageURLs();
		ArrayList<String> signVideoURLs = occurrence.getSignVideoURLs();
		ArrayList<String> utteranceVideoURLs = occurrence.getUtteranceVideoURLs();
		
		String domStartHS = occurrence.getDomStartHS();
		String domEndHS = occurrence.getDomEndHS();
		String nondomStartHS = occurrence.getNondomStartHS();
		String nondomEndHS = occurrence.getNondomEndHS();

        SS3CodingScheme cs = SS3Singleton
                .getSS3CodingSchemeCollection().getDefaultCodingScheme();
		
		String signLabelText = "", mainGlossLabelText = "";

		String hsDir = "handShapes/";
		String domStartHSFilePath = "", domEndHSFilePath = "", nondomStartHSFilePath = "", nondomEndHSFilePath = "";

		signLabelText = occurrence.getParentVariant().getVariantLabel();
		mainGlossLabelText = occurrence.getParentVariant().getParentMainGloss().getMainGlossLabel();
		variantLabel.setText(generateWrappedText(variantLabel, "Entry/Variant: " + signLabelText));
		mainGlossLabel.setText(generateWrappedText(mainGlossLabel, "Primary Entry: " + mainGlossLabelText));
			
		insertOccurrence.setText(generateWrappedText(insertOccurrence, insertEntryVariant + ": " + signLabelText));
		insertOccurrence.setMaximumSize(insertOccurrence.getMinimumSize());
		
		String rightHandFieldID = "1000";
		String leftHandFieldID = "2000";
		
		ArrayList<SS3FieldValue> rightFieldValues = cs.getFieldImg(rightHandFieldID);
		ArrayList<SS3FieldValue> leftFieldValues = cs.getFieldImg(leftHandFieldID);

		SS3FieldValue rightHandStartValue = null, rightHandEndValue = null, leftHandStartValue = null, leftHandEndValue = null;

		// right start hs		
		rightHandStartValue = Util.matchSignBank(rightFieldValues, domStartHS);
		
		// right end hs
		rightHandEndValue = Util.matchSignBank(rightFieldValues, domEndHS);

		// left start hs
		leftHandStartValue = Util.matchSignBank(leftFieldValues, nondomStartHS);

		// left end hs
		leftHandEndValue = Util.matchSignBank(leftFieldValues, nondomEndHS);
		
		SS3FieldTypeImg domFirstImage = null, domLastImage = null, nondomFirstImage = null, nondomLastImage = null;

		if (rightHandStartValue != null) {
			domFirstImage = cs.getImage(rightHandStartValue.getIid());
            domStartHSFilePath = hsDir + domFirstImage.getPath1();
		}

		if (rightHandEndValue != null) {
			domLastImage = cs.getImage(rightHandEndValue.getIid());
            domEndHSFilePath = hsDir + domLastImage.getPath1();
		}

		if (leftHandStartValue != null) {
			nondomFirstImage = cs.getImage(leftHandStartValue.getIid());
            nondomStartHSFilePath = hsDir + nondomFirstImage.getPath1();
		}

		if (leftHandEndValue != null) {
			nondomLastImage = cs.getImage(leftHandEndValue.getIid());
            nondomEndHSFilePath = hsDir + nondomLastImage.getPath1();
		}
		
		domStartHSImage = null;
		nondomStartHSImage = null;
		domEndHSImage = null;
		nondomEndHSImage = null;

		File domStartHSFile = new File(domStartHSFilePath);
		if (domStartHSFile.exists() && !domStartHSFile.isDirectory()) {
			try {
				domStartHSImage = ImageIO.read(domStartHSFile);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		File nondomStartHSFile = new File(nondomStartHSFilePath);
		if (nondomStartHSFile.exists() && !nondomStartHSFile.isDirectory()) {
			try {
				nondomStartHSImage = ImageIO.read(nondomStartHSFile);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		File domEndHSFile = new File(domEndHSFilePath);
		if (domEndHSFile.exists() && !domEndHSFile.isDirectory()) {
			try {	
				domEndHSImage = ImageIO.read(domEndHSFile);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		File nondomEndHSFile = new File(nondomEndHSFilePath);
		if (nondomEndHSFile.exists() && !nondomEndHSFile.isDirectory()) {
			try {	
				nondomEndHSImage = ImageIO.read(nondomEndHSFile);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		domStartHSPanel.removeAll();
		nondomStartHSPanel.removeAll();
		domEndHSPanel.removeAll();
		nondomEndHSPanel.removeAll();

		framesPanel.removeAll();

		if (domStartHSImage != null)
			domStartHSPanel.add(new JLabel(new ImageIcon(domStartHSImage)));

		if (nondomStartHSImage != null)
			nondomStartHSPanel.add(new JLabel(new ImageIcon(nondomStartHSImage)));

		if (domEndHSImage != null)
			domEndHSPanel.add(new JLabel(new ImageIcon(domEndHSImage)));

		if (nondomEndHSImage != null)
			nondomEndHSPanel.add(new JLabel(new ImageIcon(nondomEndHSImage)));

		// xuggler video from the web
		xugglerVideoPanel.removeAllMedia();
		xugglerVideoPanel.resetCurrentIndex();
		controllerList.clear();
		String currentSignVideoURL = "";	// for the exception below to know where it failed

		playVideoButton.setEnabled(false);
		
		try {
			for (String signVideoURL: signVideoURLs) {
				currentSignVideoURL = signVideoURL;

				controller = new MultipleMovieController_SB();
				xugglerMovie = new XugglerMovie_SB(signVideoURL, controller);
				xugglerVideoPanel.addMedia(xugglerMovie.getVideoImage());
				controller.registerMovie(xugglerMovie);
				controllerList.add(controller);

				playVideoButton.setEnabled(true);
			}
		} catch (IllegalArgumentException e) {
			SS3SignStreamApplication.logger.log(Level.INFO, "Could not open video URL: " + currentSignVideoURL);
		}
		
		
		// lexical text box
		participant = occurrence.getParticipant();
		if (participant == null || participant.isEmpty())
			participant = "n/a";
		signType = occurrence.getSignType();
		if (signType == null || signType.isEmpty()) 
			signType = "n/a";
		if (occurrence.isTwoHanded())
			twoHanded = "Yes";
		else
			twoHanded = "No";
		if (occurrence.isPassiveBaseArm())
			passiveBaseArm = "Yes";
		else
			passiveBaseArm = "No";
		if (occurrence.isMarkedHands())
			markedHands = "Yes";
		else
			markedHands = "No";
		leftRightHS = occurrence.getLeftRightHS();
		startEndHS = occurrence.getStartEndHS();
		startEndLeftHS  = occurrence.getStartEndLeftHS();
		startEndRightHS = occurrence.getStartEndRightHS();
		
		String text = "";
		text += "SignType: " + signType;
		text += ", TwoHanded: " + twoHanded;
		text += ", \nPassiveBaseArm: " + passiveBaseArm;
		text += ", MarkedHands: " + markedHands;
		if (!leftRightHS.isEmpty())
			text += ", \nLeftRightHandShapes: " + leftRightHS;
		if (!startEndHS.isEmpty())
			text += ", \nStartEndHandShapes: " + startEndHS;
		if (!startEndLeftHS.isEmpty())
			text += ", \nStartEndLeftHandShapes: " + startEndLeftHS;
		if (!startEndRightHS.isEmpty())
			text += ", \nStartEndRightHandShapes: " + startEndRightHS;
		
		extraDataText.setText(text);
		extraDataContainer.setPreferredSize(new Dimension(getWidth()/2, 100));
		extraDataContainer.setMaximumSize(new Dimension(getWidth()/2, 200));
		mainGlossLabelPanel.setMaximumSize(mainGlossLabelPanel.getPreferredSize());
		variantLabelPanel.setMaximumSize(variantLabelPanel.getPreferredSize());
		hsPanelRow.setMaximumSize(hsPanelRow.getPreferredSize());
		
		insertOccurrence.setVisible(true);

		revalidate();
		repaint();
		SwingUtilities.getWindowAncestor(this).pack();
		
		//Keep track of total image width to enforce panel's width.
		int totalWidth = 0;
		//Keep track of previous width/height for any error panels
		int previousHeight = (int)(this.getPreferredSize().height * imageHeight);
		int previousWidth = this.getWidth() / 2;
		// start end frame images
		if (frameImageURLs.size() == 0) {
				JPanel errorPanel = new JPanel();
				JLabel errorMessage = new JLabel("No images.");
				errorPanel.setPreferredSize(new Dimension(previousWidth, previousHeight));
				errorPanel.setMaximumSize(new Dimension(previousWidth, previousHeight));
				errorPanel.setLayout(new GridBagLayout());
				errorPanel.add(errorMessage);
				errorPanel.setBackground(Color.GRAY);
				errorPanel.setAlignmentX(CENTER_ALIGNMENT);
				framesPanel.add(errorPanel);
				totalWidth=previousWidth;
		} else try {
			//If occurrence start/end image URLs are set, use those instead of the arraylist of images.
			if (occurrence.getStartImageURL() != null || occurrence.getEndImageURL() != null) {
				frameImageURLs = new ArrayList<String>();
				if (occurrence.getStartImageURL() != null) {
					frameImageURLs.add(occurrence.getStartImageURL());
				} if (occurrence.getEndImageURL() != null) {
					frameImageURLs.add(occurrence.getEndImageURL());
				}
			}	
			for (String frameImageURL: frameImageURLs) {
				URL url = new URL(frameImageURL);
				try {
 					framesImage = ImageIO.read(url);
 				} catch (IOException e) {
 					framesImage = null;
 				}
 				if (framesImage == null) {
 					JPanel errorPanel = new JPanel();
 					JLabel errorMessage = new JLabel("Image not found.");
 					errorPanel.setMaximumSize(new Dimension(previousWidth , previousHeight));
 					errorPanel.setLayout(new GridBagLayout());
 					errorPanel.add(errorMessage);
 					errorPanel.setBackground(Color.GRAY);
 					errorPanel.setAlignmentX(CENTER_ALIGNMENT);
 					framesPanel.add(errorPanel);
 					totalWidth += previousWidth;
 					continue;
 				}

				int width = framesImage.getWidth();
				int height = framesImage.getHeight();
				
				int maxWidth = getMinimumSize().width / frameImageURLs.size();
				int maxHeight = (int)(getMinimumSize().height  * imageHeight);
				
				if (width > maxWidth) {
					height = height * maxWidth / width;
					width = maxWidth;
				}
				if (height > maxHeight) {
					width = width * maxHeight / height;
					height = maxHeight;
				}
				previousHeight = height;
				previousWidth = width;
				BufferedImage resizedImage = new BufferedImage(width, height, framesImage.getType());
				Graphics2D g = resizedImage.createGraphics();
				g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
						RenderingHints.VALUE_INTERPOLATION_BILINEAR);
				g.drawImage(framesImage, 0, 0, width, height, 0, 0, framesImage.getWidth(),
						framesImage.getHeight(), null);
				g.dispose();

				if (resizedImage != null) {
					JLabel picture = new JLabel(new ImageIcon(resizedImage));
					framesPanel.add(picture);
					totalWidth += width;
				}
			}
		} catch (MalformedURLException e) {
			System.out.println("MalformedURLException caught");
		} catch (IllegalArgumentException e) {
			System.out.println("IllegalArgumentException caught");
		}
		framesPanel.setMinimumSize(new Dimension(totalWidth, framesPanel.getPreferredSize().height));
		framesPanel.setPreferredSize(new Dimension(totalWidth, framesPanel.getPreferredSize().height));
		loadPreviews();
	}
	
	private String generateWrappedText(Component c, String original) {
		Font btnFont = c.getFont();
		FontMetrics btnFontMetrics = c.getFontMetrics(btnFont);
		String newText = "<html>";
		int width = btnFontMetrics.stringWidth(original);
		
		int rows =  (int) (width/(getWidth() * variantPanelWidth * (1-horizontalPadding)) + 1);
		if (rows <= 1) {
			newText = original;
		} else {
			for (int i=0;i<rows;i++) {
				if (i>0) newText += "<br>";
				int startIndex = i*original.length() / rows;
				int endIndex = (i+1) * original.length() / rows;
				newText += original.substring(startIndex, endIndex);
			}
			newText += "</html>";
		}
		return newText;
	}
	
	private void insertOccurrence() {
		if (currentlyDisplayingOccurrence == null)
			return;
		
		assocFieldsPanel.populateFromSignBankOccurrence(currentlyDisplayingOccurrence);	

		SwingUtilities.getWindowAncestor(this).dispose();	
	}
	
	private void loadPreviews() {
		for (MultipleMovieController_SB controller : controllerList) {
			try {
				controller.actionDelegate.goToPacketTimeStamp(0);
				controller.actionDelegate.play(0, 2, null);
			} catch (Exception e) {
				System.out.println("Error loading previews.");
			}
			
		}
	}


	private void playVideo() {
		if (controllerList.size() <= 0) {
			return;
		}
		// based on which video "card" is displaying, get the corresponding controller to play it
		int currentMediaIndex = xugglerVideoPanel.getCurrentIndex();
		MultipleMovieController_SB controller = controllerList.get(currentMediaIndex);
		controller.playSegment(0, Integer.MAX_VALUE, null);	// just pass max_value to ensure entire video runs	
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		switch (Integer.parseInt(str)) {
		
		case INSERT:
			insertOccurrence();
			break;
			
		case PLAY_VIDEO:
			playVideo();
			break;
		
		}
	}
	
	/**
	 * <pre>
	 * Wrapper class to implement a CardLayout for a multitude of media (swing Components).
	 * Add Components to the panel via addMedia() and cycle through the media with prev/next buttons.
	 * You're able to retrieve the currently displaying media card via
	 * getCurrentIndex() and/or getCurrentCardData()
	 * 
	 * Sources:
	 * https://docs.oracle.com/javase/tutorial/uiswing/layout/card.html
	 * http://stackoverflow.com/questions/24866220/how-do-i-stop-the-cardlayout-next-and-previous-from-looping
	 * </pre>
	 */
	private class CardMediaPanel extends JPanel{
		private static final long serialVersionUID = 1L;
		
		private JLabel titleLabel = null;
		private JPanel mediaPanel = null;
		private JButton prevButton = null;
		private JButton nextButton = null;
		private JPanel buttonPanel = null;
		
		private ArrayList<Component> mediaComponents = null;
		private int currentIndex;
		
		private CardLayout cl = new CardLayout();
		
		public CardMediaPanel(String title) {
			titleLabel = new JLabel(title);
			titleLabel.setAlignmentX(CENTER_ALIGNMENT);
			
			mediaComponents = new ArrayList<Component>();
			currentIndex = 0;
			
			mediaPanel = new JPanel();
			
			mediaPanel.setLayout(cl);
			cl.show(mediaPanel, "");
			
			buttonPanel = new JPanel();
			
			prevButton = new JButton("Previous");
			prevButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					cl.previous(mediaPanel);
					currentIndex--;
					updatePrevNextButtons();
					
					buttonPanel.repaint();
				}   
			});

			nextButton = new JButton("Next");	
			nextButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					cl.next(mediaPanel);
					currentIndex++;
					updatePrevNextButtons();
					
					buttonPanel.repaint();
				}
			});
			
			if (getCardsSize() == 0) {
				prevButton.setEnabled(false);
				nextButton.setEnabled(false);
			}
			
			buttonPanel.add(prevButton);
			buttonPanel.add(playVideoButton);
			buttonPanel.add(nextButton);
			
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			add(titleLabel);
			add(mediaPanel);
			add(buttonPanel);			
		} 
		
		private void updatePrevNextButtons() {
			if (currentIndex <= 0)
				prevButton.setEnabled(false);
			else
				prevButton.setEnabled(true);

			if (currentIndex >= getCardsSize() - 1)
				nextButton.setEnabled(false);
			else
				nextButton.setEnabled(true);
		}
		
		public String getTitle() {
			return titleLabel.getText();
		}
		
		public int getCurrentIndex() {
			return currentIndex;
		}
		
		public void resetCurrentIndex() {
			currentIndex = 0;
		}
		
		public Component getCurrentCardData() {
			if (getCardsSize() == 0)
				return null;
			else
				return mediaComponents.get(getCurrentIndex());
		}
		
		public int getCardsSize() {
			return mediaComponents.size();
		}
		
		public void addMedia(Component c) {
			mediaComponents.add(c);
			mediaPanel.add(c, "" + mediaComponents.size());	// add via CardLayout schema
			
			if (getCardsSize() > 0) {
				updatePrevNextButtons();
			}
			
		}
		
		public void removeMedia(int index) {
			mediaComponents.remove(index);
			updatePrevNextButtons();
			mediaPanel.remove(index);
			mediaPanel.repaint();
		}
		
		public void removeMedia(Component c) {
			mediaComponents.remove(c);
			updatePrevNextButtons();
			mediaPanel.remove(c);
			mediaPanel.repaint();
		}
		
		public void removeAllMedia() {
			mediaComponents.clear();
			updatePrevNextButtons();
			mediaPanel.removeAll();
			mediaPanel.repaint();
		}		
	}
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// NOTE: EVERYTHING BELOW THIS LINE IS AUXILIARY XUGGLER HELPER CODE COPIED FROM THEIR ORIGINAL CLASSES //
	// EVERY CLASS WAS APPENDED WITH "_SB" (Sign Bank) SO AS TO AVOID CONFUSION //
	// SOME MINOR EDITS WERE MADE IN EACH CLASS TO AVOID COUPLING WITH THE SIGNSTREAMSEGMENTPANEL //

	public class XugglerMovie_SB {
		private VideoImage_SB mScreen = null;
		private BufferedImage javaImage = null;
		private String filename = "";
		private IContainer container = null;
		private int videoStreamId = -1;
		private IStreamCoder videoCoder = null;
		private IVideoResampler resampler = null;
		
		private MultipleMovieController_SB controller;
		
		public XugglerMovie_SB(String filename, MultipleMovieController_SB controller) {
			this.filename = filename;
			this.controller = controller;
			load();
		}
		
		public VideoImage_SB getVideoImage() {
			return mScreen;
		}
	
		public String getFilename() {
			return filename;
		}
	
		public IContainer getContainer() {
			return container;
		}
	
		public int getVideoStreamId() {
			return videoStreamId;
		}
	
		public IStreamCoder getVideoCoder() {
			return videoCoder;
		}
	
		public IVideoResampler getResampler() {
			return resampler;
		}
	
		public void load() {
			SS3SignStreamApplication.logger.log(Level.INFO, "First Xuggler call...");
	
			container = IContainer.make();
			
			SS3SignStreamApplication.logger.log(Level.INFO, "First Xuggler call went OK!!! ");
	
			// Open up the container
			if (container.open(filename, IContainer.Type.READ, null) < 0)
				throw new IllegalArgumentException("could not open file: "
						+ filename);
	
			// query how many streams the call to open found
			int numStreams = container.getNumStreams();
	
			// and iterate through the streams to find the first video stream
			for (int i = 0; i < numStreams; i++) {
				// Find the stream object
				IStream stream = container.getStream(i);
				// Get the pre-configured decoder that can decode this stream;
				IStreamCoder coder = stream.getStreamCoder();
	
				if (coder.getCodecType() == ICodec.Type.CODEC_TYPE_VIDEO) {
					videoStreamId = i;
					videoCoder = coder;
					break;
				}
			}
			if (videoStreamId == -1)
				throw new RuntimeException(
						"could not find video stream in container: " + filename);
	
			// open up the decoder so it can do work.
			if (videoCoder.open() < 0)
				throw new RuntimeException(
						"could not open video decoder for container: " + filename);
	
			if (videoCoder.getPixelType() != IPixelFormat.Type.BGR24) {
				// if this stream is not in BGR24, we're going to need to
				// convert it. The VideoResampler does that for us.
				resampler = IVideoResampler.make(videoCoder.getWidth(), videoCoder
						.getHeight(), IPixelFormat.Type.BGR24, videoCoder
						.getWidth(), videoCoder.getHeight(), videoCoder
						.getPixelType());
				if (resampler == null)
					throw new RuntimeException("could not create color space "
							+ "resampler for: " + filename);
			}
	
			openJavaWindow();
		}
	
		public void loadNextFrameImage(BufferedImage aJavaImage) {
			if(this.javaImage != null) {			
				this.javaImage.flush();
				this.javaImage = null;
				//System.gc();
			}
			
			this.javaImage = aJavaImage;
			aJavaImage.flush();
			aJavaImage = null;
			mScreen.setImage(this.javaImage, false);
		}
	
		public void updateJavaWindow(boolean ignoreRepaint) {
			mScreen.setImage(javaImage, ignoreRepaint);
		}
	
		/**
		 * Draws the video on a screen
		 */
		public void updateJavaWindow(BufferedImage javaImage, boolean ignoreRepaint) {
			mScreen.setImage(javaImage, ignoreRepaint);
		}
	
		/**
		 * Opens a Swing window on screen.
		 */
		public void openJavaWindow() {
			mScreen = new VideoImage_SB(controller, null);	
		}
	}
	
	public class VideoImage_SB extends JPanel implements ComponentListener {
	
		private static final long serialVersionUID = -4752966848100689153L;
		private final ImageComponent mOnscreenPicture;
	
		private int videoWidth = getWidth();
		private int videoHeight = getHeight();
	
		private int originalVideoWidth = getWidth();
		private int originalVideoHeight = getHeight();
		
		private int minVideoWidth = videoWidth;
		private int minVideoHeight = videoHeight;
	
		public VideoImage_SB(MultipleMovieController_SB controller, XugglerMovie_SB movie) {
			super();
			mOnscreenPicture = new ImageComponent();
			
			setLayout(new BorderLayout());
			add(mOnscreenPicture, BorderLayout.CENTER);
			this.setVisible(true);
			
			this.setFocusable(true);
		}
	
		@Override
		public void componentHidden(ComponentEvent e) {
			;
		}
	
		@Override
		public void componentMoved(ComponentEvent e) {
			Dimension d = this.getSize();
	
			if(d.width < 300) {
				videoWidth = 300;
				
				Dimension dd = new Dimension(300, videoHeight);
				this.setPreferredSize(dd);
				this.setMaximumSize(dd);
				this.setMinimumSize(dd);
	
			} else {
				videoWidth = d.width;
				videoHeight = d.height;
				this.setPreferredSize(d);
			}
		}
		
		public void goToMinSize() {		
			videoWidth = minVideoWidth; 
			videoHeight = minVideoHeight;
			
			Dimension d = new Dimension(minVideoWidth, minVideoHeight);				
			mOnscreenPicture.setPreferredSize(d);		
		
			Dimension dd = new Dimension(minVideoWidth, minVideoHeight + 60);
			this.setPreferredSize(dd);
			this.setMaximumSize(dd);
			this.setMinimumSize(dd);
			this.invalidate();
			this.update(getGraphics());
			
			setPreferredSize(dd);		
			mOnscreenPicture.repaint();		
			SwingUtilities.getWindowAncestor(this).pack();
		}
		
		public void goToMaxSize() {			
			videoWidth = originalVideoWidth; 
			videoHeight = originalVideoHeight;
					
			Dimension d = new Dimension(videoWidth, videoHeight);				
			mOnscreenPicture.setPreferredSize(d);		
		
			Dimension dd = new Dimension(videoWidth, videoHeight + 60);
			this.setPreferredSize(dd);
			this.setMaximumSize(dd);
			this.setMinimumSize(dd);
			this.invalidate();
			this.update(getGraphics());
			
			setPreferredSize(dd);		
			mOnscreenPicture.repaint();		
			SwingUtilities.getWindowAncestor(this).pack();	
		}
		
		@Override
		public void componentResized(ComponentEvent e) {
			Dimension d = this.getSize();
			
			if(d.width < 300) {
				videoWidth = 300;
				
				Dimension dd = new Dimension(300, videoHeight);
				this.setPreferredSize(dd);
				this.setMaximumSize(dd);
				this.setMinimumSize(dd);
	
			} else {
				videoWidth = d.width;
				videoHeight = d.height;
				this.setPreferredSize(d);
			}
		}
	
		@Override
		public void componentShown(ComponentEvent e) {}
	
		private boolean loaded = false;
	
		public void setImage(final BufferedImage aImage, boolean ignoreRepaint) {
			if(!loaded) {
				loaded = true;
				
				mOnscreenPicture.setImage(aImage, ignoreRepaint);
				
				int width = aImage.getWidth();
				int height = aImage.getHeight();
				
				int maxWidth = (int)(SignBankVariantPanel.this.getWidth() * SignBankVariantPanel.this.videoWidth * (1-horizontalPadding));
				//Maintain Height-to-width ratio for max height.
				//int maxHeight = (int) (1.0 * height/width * maxWidth);
				
				int maxHeight = (int)(SignBankVariantPanel.this.getHeight() * SignBankVariantPanel.this.videoHeight * (1-horizontalPadding) - playVideoButton.getHeight() * 2);
				
				if (width > maxWidth) {
					height = height * maxWidth / width;
					width = maxWidth;
				}
				if (height > maxHeight) {
					width = width * maxHeight / height;
					height = maxHeight;
				}
				
				videoWidth = width;
				videoHeight = height;
						
				originalVideoWidth = videoWidth;
				originalVideoHeight = videoHeight;
				
				minVideoHeight = (minVideoWidth * originalVideoHeight)/originalVideoWidth;		
				this.setPreferredSize(new Dimension(videoWidth, videoHeight));
			} else {
				loaded = true;
				mOnscreenPicture.setImage(aImage, ignoreRepaint);
				Dimension d = this.getPreferredSize();
				videoWidth = d.width;
				videoHeight = d.height;
				this.setSize(d);
				if (SwingUtilities.getWindowAncestor(this) != null) {
					SwingUtilities.getWindowAncestor(this).pack();
				}
			}
	
		}
	
		public class ImageComponent extends JComponent {
	
			private static final long serialVersionUID = 1L;
	
			private Image image;
	
			public void setImage(Image image, boolean ignoreRepaint) {
				SwingUtilities.invokeLater(new ImageRunnable(image));
			}
	
			private class ImageRunnable implements Runnable {
	
				private final Image newImage;
	
				public ImageRunnable(Image newImage) {
					super();
					this.newImage = newImage;
				}
	
				@Override
				public void run() {
					ImageComponent.this.image = null;
					ImageComponent.this.image = newImage;
					repaint();
				}
			}
	
			@Override
			public synchronized void paint(Graphics g) {
				if (image != null) {
					g.drawImage(image, (getWidth() - videoWidth) / 2,
							(getHeight() - videoHeight) / 2, videoWidth,
							videoHeight, this);
					
					image.flush();
				}
			}
		}
	}

	public class MultipleMovieController_SB {
	
		private ArrayList<XugglerMovie_SB> movies = new ArrayList<XugglerMovie_SB>();
		private VideoSegment selectedVideoSegment = new VideoSegment(0, 0);
		private long currentPacketTimeStamp = 0;
		private int packetDuration = 2000; 
	
		private int speedPointer = 2;
	
		private int dragToTime = -1;
		private int skippedFrames = 0;
		private boolean play = false;
		private boolean doneSkipping = true;
	
		public static final int NUMBER_OF_SLIDER_TICKS = 10000;
	
		private MovieActionDelegate_SB actionDelegate = null;
	
		public MultipleMovieController_SB() {
			actionDelegate = new MovieActionDelegate_SB(this);
		}
	
		MoviePlayer_SB player = null;
	
		public MovieActionDelegate_SB getMovieActionDelegate() {
			return actionDelegate;
		}
	
		public void load() {
			if (movies.isEmpty())
				return;
	
			for (XugglerMovie_SB movie : movies) {
				String filename = movie.getFilename();
				System.out.println("FILENAME = " + filename);
			}
	
			if (!play) {
				play = true;
				// calculate packet duration
				long timeStamp = actionDelegate.playNextFrame();
				currentPacketTimeStamp = actionDelegate.playNextFrame();
				packetDuration = (int) (currentPacketTimeStamp - timeStamp);
				play = false;
			}
		}
	
		public void startPlayingMovie() {
			play = true;
		}
	
		public int getSpeedPointer() {
			return speedPointer;
		}
	
		public String speeding(boolean slover) {
			if (slover) {
				if (speedPointer < 5) {
					speedPointer++;
				} else {
					speedPointer = 5;
				}
			} else {
				if (speedPointer > 0) {
					speedPointer--;
				} else {
					speedPointer = 0;
				}
			}
	
			String label = "";
	
			switch (speedPointer) {
			case 0:
				label = "/4";
				break;
	
			case 1:
				label = "/2";
				break;
	
			case 3:
				label = "2X";
				break;
	
			case 4:
				label = "4X";
				break;
	
			case 5:
				label = "8X";
				break;
			}
	
			return label;
		}
	
		public void playSegment(int startMovieTime, int endMovieTime,
				SignStreamSegmentPanel segmentPanel) {
			if (!play) {
				play = true;
				player = new MoviePlayer_SB(this, startMovieTime,
						endMovieTime);
				player.execute();
			}
		}
	
		public long getPacketDuration() {
			return packetDuration;
		}
	
		public void setDoneSkipping(boolean done) {
			this.doneSkipping = done;
		}
	
		public void skipNumberOfFrames(int moveNumberOfFrames) {
			if (!play) {
				play = true;
				if (this.doneSkipping) {
					skippedFrames = skippedFrames + moveNumberOfFrames;
					FrameSkipper fs = new FrameSkipper(this, skippedFrames);
					this.doneSkipping = false;
					this.skippedFrames = 0;
					fs.execute();
				} else {
					skippedFrames = skippedFrames + moveNumberOfFrames;
				}
				play = false;
			}
		}
	
		public long frameBackward(long packetTimeStamp) {
			long goToPacketTimeStamp = packetTimeStamp - packetDuration;
			actionDelegate.goToPacketTimeStamp(goToPacketTimeStamp);
	
			return currentPacketTimeStamp;
		}
	
		public long framesXBackward(int frames) {
			long newTime = currentPacketTimeStamp - (frames * packetDuration);
			newTime = newTime < 0 ? 0 : newTime;
			actionDelegate.goToPacketTimeStamp(newTime);
			return currentPacketTimeStamp;
		}
	
		public long frameForward(long packetTimeStamp) {
			actionDelegate.goToPacketTimeStamp(packetTimeStamp);
	
			return currentPacketTimeStamp;
		}
	
		private void goToFrame(int frameNumber) {
	
			if (movies.size() < 1 || frameNumber <= 0)
				return;
	
			int targetMovieTime = frameNumber * packetDuration;
			actionDelegate.goToPacketTimeStamp(targetMovieTime);
		}
	
		public int getCurrentFrame() {
			return (int) currentPacketTimeStamp / packetDuration;
		}
	
		public int getPercent() {
			long y = MultipleMovieController_SB.NUMBER_OF_SLIDER_TICKS
					* currentPacketTimeStamp / getMovieDuration();
	
			return (int) y;
		}
	
	
		public void setCurrentTime(int currentTime) {
			actionDelegate.goToPacketTimeStamp(currentTime);
		}
	
		public int getDragToTime() {
			return dragToTime;
		}
	
		public void setDragToTime(int dragToTime) {
			this.dragToTime = dragToTime;
		}
	
		public void dragToCurrentTime(int currentTime) {
			if (!play) {
				play = true;
				if (currentTime <= getMovieDuration()) {
					if (dragToTime == -1) {
						this.dragToTime = currentTime;
						actionDelegate.executeDragging();
					} else {
						this.dragToTime = currentTime;
					}
				}
			}
			play = false;
		}
	
		public void goToFirstFrame() {
			goToFrame(1);
			goToFrame(1);
		}
	
		public void goToLastFrame() {
			if (movies.size() < 1)
				return;
	
			int targetMovieTime = getMovieDuration() - packetDuration;
			actionDelegate.goToPacketTimeStamp(targetMovieTime);
		}
	
		public void removeAllMovies() {
			movies.clear();
		}
	
		public void setCurrentPacketTimeStamp(long currentPacketTimeStamp) {
			this.currentPacketTimeStamp = currentPacketTimeStamp;
		}
	
		public int getMovieDuration() {
			if (movies.size() < 1)
				return 0;
	
			XugglerMovie_SB movie = movies.get(0);
			long numFrames = movie.getContainer()
					.getStream(movie.getVideoStreamId()).getNumFrames();
	
			return (int) numFrames * packetDuration;
		}
	
		public void selectSegment(int startTime, int endTime) {
			selectedVideoSegment.setStartTime(startTime);
			selectedVideoSegment.setEndTime(endTime);
		}
	
		public int getPrecedingFrameTime() {
			return (int) currentPacketTimeStamp - packetDuration;
		}
	
		public int getFollowingFrameTime() {
			return (int) currentPacketTimeStamp + packetDuration;
		}
	
		public int getPrecedingFrameTime(int time) {
			return time - packetDuration;
		}
	
		public int getFollowingFrameTime(int time) {
			return time + packetDuration;
		}
	
		public int getCurrentTime() {
			return (int) currentPacketTimeStamp;
		}
	
		public void registerMovie(XugglerMovie_SB movie) {
			movies.add(movie);
		}
	
		public void removeMovie(XugglerMovie_SB movie) {
			movies.remove(movie);
		}
	
		public void goToPacketTimeStamp(long goToPacketTime) {
			actionDelegate.goToPacketTimeStamp(goToPacketTime);
		}
	
		public ArrayList<XugglerMovie_SB> getMovies() {
			return movies;
		}
	
		public boolean isPlaying() {
			return play;
		}
	
		public void stopPlayingMovie() {
			play = false;
		}
	
		private class FrameSkipper extends
				SwingWorker<Integer, ArrayList<XugglerMovie>> {
	
			private MultipleMovieController_SB controller = null;
			private int numberOfFramesToSkip = 0;
	
			FrameSkipper(final MultipleMovieController_SB controller,
					int numberOfFramesToSkip) {
				this.controller = controller;
				this.numberOfFramesToSkip = numberOfFramesToSkip;
			}
	
			private void goToFrame() {
				long move = numberOfFramesToSkip * packetDuration;
				long goToPacketTime = currentPacketTimeStamp + move;
	
				controller.getMovieActionDelegate().goToPacketTimeStamp(
						goToPacketTime);
			}
	
			@Override
			protected Integer doInBackground() throws Exception {
				goToFrame();
				return new Integer(1);
			}
	
			@Override
			protected void done() {
				controller.setDoneSkipping(true);
			}
		}
	}
	
	class VideoSegment {
		int startTime, endTime;
	
		VideoSegment(int startTime, int endTime) {
			this.startTime = startTime;
			this.endTime = endTime;
		}
	
		public int getStartTime() {
			return startTime;
		}
	
		public void setStartTime(int startTime) {
			this.startTime = startTime;
		}
	
		public int getEndTime() {
			return endTime;
		}
	
		public void setEndTime(int endTime) {
			this.endTime = endTime;
		}
	}


public class MovieActionDelegate_SB {

	private MultipleMovieController_SB controller = null;

	public MovieActionDelegate_SB(MultipleMovieController_SB controller) {
		this.controller = controller;
	}

	public long playNextFrame() {
		long currentPacketTimeStamp = controller.getCurrentTime();
		long packetDuration = controller.getPacketDuration();

		ArrayList<XugglerMovie_SB> movies = controller.getMovies();

		if (movies.isEmpty())
			return -1;

		boolean b = false;
		while (!b) {

			for (XugglerMovie_SB movie : movies) {
				IPacket packet = IPacket.make();
				int readPacket = 0;

				try {
					readPacket = movie.getContainer().readNextPacket(packet);

					long packetTimeStamp = packet.getTimeStamp();

					if (packetTimeStamp < 0) {
						currentPacketTimeStamp = currentPacketTimeStamp
								+ packetDuration;
					} else {
						currentPacketTimeStamp = packetTimeStamp;
					}

				} catch (Exception ex1) {
					return -1;
				}

				if (readPacket < 0)
					return -1;

				if (packet.getStreamIndex() == movie.getVideoStreamId()) {
					b = true;

					IVideoPicture picture = IVideoPicture.make(movie
							.getVideoCoder().getPixelType(), movie
							.getVideoCoder().getWidth(), movie.getVideoCoder()
							.getHeight());
					

					int offset = 0;
					while (offset < packet.getSize()) {

						int bytesDecoded = movie.getVideoCoder().decodeVideo(
								picture, packet, offset);
						if (bytesDecoded < 0)
							throw new RuntimeException(
									"got error decoding video in: "
											+ movie.getFilename());
						offset += bytesDecoded;

						if (picture.isComplete()) {
							IVideoPicture newPic = picture;

							if (movie.getResampler() != null) {

								newPic = IVideoPicture
										.make(movie.getResampler()
												.getOutputPixelFormat(),
												picture.getWidth(), picture
														.getHeight());
								if (movie.getResampler().resample(newPic,
										picture) < 0)
									throw new RuntimeException(
											"could not resample video from: "
													+ movie.getFilename());
							}
							if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
								throw new RuntimeException(
										"could not decode video"
												+ " as BGR 24 bit data in: "
												+ movie.getFilename());

							// convert the BGR24 to an Java buffered image
							BufferedImage image = new BufferedImage(
									picture.getWidth(), picture.getHeight(),
									BufferedImage.TYPE_3BYTE_BGR);
							IConverter converter = ConverterFactory
									.createConverter(image,
											IPixelFormat.Type.BGR24);
							BufferedImage javaImage = converter.toImage(newPic);
							image.flush();
							image = null;
							movie.loadNextFrameImage(javaImage);
							movie.updateJavaWindow(true);

							newPic.delete();
							converter.delete();
						} else {
							playNextFrame();
						}
					}

					if (movie.equals(movies.get(movies.size() - 1))) {
						controller.setCurrentPacketTimeStamp(packet
								.getTimeStamp());
					} else {
						// not a video stream, silently drop it.
						do {
						} while (false);
					}

					packet.delete();
					picture.delete();
				}
			}
		}
		return currentPacketTimeStamp;
	}

	public long goToPreviousKeyFrame(long goToPacketTime, XugglerMovie_SB movie) {

		if (goToPacketTime == -1) {
			return -1;
		}

		long packetDuration = controller.getPacketDuration();

		long p = packetDuration * 30;
		long minTimeStamp = goToPacketTime - p;
		long targetTimeStamp = goToPacketTime - 2 * packetDuration;
		long maxTimeStamp = goToPacketTime - packetDuration;

		if (minTimeStamp < 0) {
			minTimeStamp = 0;
		}

		if (maxTimeStamp < 0) {
			maxTimeStamp = 0;
		}

		long packetTimeStamp = -1;

		movie.getContainer().seekKeyFrame(movie.getVideoStreamId(), -1, 0);
		movie.getContainer().seekKeyFrame(movie.getVideoStreamId(),
				minTimeStamp, targetTimeStamp, maxTimeStamp,
				IContainer.SEEK_FLAG_FRAME | IContainer.SEEK_FLAG_BACKWARDS);

		IPacket packet = IPacket.make();

		int readPacket = -1;

		boolean b = false;
		while (!b) {
			packet = IPacket.make();

			try {
				readPacket = movie.getContainer().readNextPacket(packet);

				if (packet.getStreamIndex() == movie.getVideoStreamId()) {
					b = true;
				}
			} catch (Exception ex1) {
				ex1.printStackTrace();
			}
		}

		// isKeyFrame
		if (packet.isKeyPacket()) {

			if (readPacket < 0)
				return -1;

			packetTimeStamp = packet.getTimeStamp();

			IVideoPicture picture = IVideoPicture.make(movie.getVideoCoder()
					.getPixelType(), movie.getVideoCoder().getWidth(), movie
					.getVideoCoder().getHeight());

			int offset = 0;

			int bytesDecoded = movie.getVideoCoder().decodeVideo(picture,
					packet, offset);

			if (bytesDecoded < 0)
				throw new RuntimeException("got error decoding video in: "
						+ movie.getFilename());
			offset += bytesDecoded;
		
			if (picture.isComplete()) {
				IVideoPicture newPic = picture;

				if (movie.getResampler() != null) {

					newPic = IVideoPicture.make(movie.getResampler()
							.getOutputPixelFormat(), picture.getWidth(),
							picture.getHeight());
					if (movie.getResampler().resample(newPic, picture) < 0)
						throw new RuntimeException(
								"could not resample video from: "
										+ movie.getFilename());
				}

				if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
					throw new RuntimeException("could not decode video"
							+ " as BGR 24 bit data in: " + movie.getFilename());

				// convert the BGR24 to an Java buffered image
				BufferedImage image = new BufferedImage(picture.getWidth(),
						picture.getHeight(), BufferedImage.TYPE_3BYTE_BGR);

				IConverter converter = ConverterFactory.createConverter(image,
						IPixelFormat.Type.BGR24);

				BufferedImage javaImage = converter.toImage(newPic);
				image.flush();
				image = null;

				if (packetTimeStamp == goToPacketTime) {
					movie.loadNextFrameImage(javaImage);
					movie.updateJavaWindow(true);
				}

				picture.delete();
				packet.delete();

			} else {
				// not a video stream, silently drop it.
				do {
				} while (false);
			}
			return packetTimeStamp;
		} else {
			return goToPreviousKeyFrame(goToPacketTime - packetDuration, movie);
		}
	}

	public void goToPacketTimeStamp(long goToPacketTime) {
		long packetDuration = controller.getPacketDuration();
		long frameNumberLong = goToPacketTime/packetDuration;
		int frameNumber = (int)frameNumberLong;
				
		goToPacketTime = frameNumber*packetDuration;
		
		if(goToPacketTime < 1) {
			goToPacketTime = 0;
		}

		ArrayList<XugglerMovie_SB> movies = controller.getMovies();
		
		if(goToPacketTime == 0) {
			for (XugglerMovie_SB movie : movies) {			
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(), -1, 0);
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(), 0, 0, 0, IContainer.SEEK_FLAG_FRAME);
			}
			
			playNextFrame();
			return;
		}
				
		for (XugglerMovie_SB movie : movies) {

			long currentPacketTimeStamp = goToPreviousKeyFrame(goToPacketTime, movie);
			
			controller
			.setCurrentPacketTimeStamp(currentPacketTimeStamp);

			if (currentPacketTimeStamp < goToPacketTime) {

				boolean toScroll = true;
				long cPacketTimeStamp = currentPacketTimeStamp;

				long newCurrentPacketTimeStamp = currentPacketTimeStamp;
				long movieDuration = controller.getMovieDuration();

				while (toScroll) {
					// scroll frame by frame
					boolean b = false;
					while (!b) {
						IPacket packet = IPacket.make();
						int readPacket = 0;

						try {
							readPacket = movie.getContainer().readNextPacket(packet);
							long packetTimeStamp = packet.getTimeStamp();	

							if (packetTimeStamp < 0) {
								cPacketTimeStamp = cPacketTimeStamp
										+ packetDuration;
							} else {
								cPacketTimeStamp = packetTimeStamp;
							}

						} catch (Exception ex1) {
							;
						}

						if (readPacket < 0)
							return;

						if (packet.getStreamIndex() == movie.getVideoStreamId()) {
							b = true;

							IVideoPicture picture = IVideoPicture.make(movie
									.getVideoCoder().getPixelType(), movie
									.getVideoCoder().getWidth(), movie
									.getVideoCoder().getHeight());

							int offset = 0;
							while (offset < packet.getSize()) {

								int bytesDecoded = movie.getVideoCoder()
										.decodeVideo(picture, packet, offset);
								if (bytesDecoded < 0)
									throw new RuntimeException(
											"got error decoding video in: "
													+ movie.getFilename());
								offset += bytesDecoded;

								if (cPacketTimeStamp >= goToPacketTime
										|| movieDuration <= goToPacketTime) {

									if (picture.isComplete()) {
										IVideoPicture newPic = picture;

										if (movie.getResampler() != null) {

											newPic = IVideoPicture.make(movie
													.getResampler()
													.getOutputPixelFormat(),
													picture.getWidth(), picture
															.getHeight());
											if (movie.getResampler().resample(
													newPic, picture) < 0)
												throw new RuntimeException(
														"could not resample video from: "
																+ movie.getFilename());
										}

										if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
											throw new RuntimeException(
													"could not decode video"
															+ " as BGR 24 bit data in: "
															+ movie.getFilename());

										// convert the BGR24 to an Java buffered
										// image
										BufferedImage image = new BufferedImage(
												picture.getWidth(),
												picture.getHeight(),
												BufferedImage.TYPE_3BYTE_BGR);

										IConverter converter = ConverterFactory
												.createConverter(image,
														IPixelFormat.Type.BGR24);

										BufferedImage javaImage = converter
												.toImage(newPic);
										image.flush();
										image = null;

										movie.loadNextFrameImage(javaImage);
										movie.updateJavaWindow(true);

										toScroll = false;
										newCurrentPacketTimeStamp = cPacketTimeStamp;
										newPic.delete();
										converter.delete();
									}
								}
							}
							// converter.delete();
							picture.delete();
							packet.delete();

						} else {
							// not a video stream, silently drop it.
							do {
							} while (false);
						}
					}
				}

				if (movie.equals(movies.get(movies.size() - 1))) {
					controller
							.setCurrentPacketTimeStamp(newCurrentPacketTimeStamp);
				}
			}
		}
	}
	
	
	
	public Integer play(long startMovieTime, long endMovieTime,
			SignStreamSegmentPanel segmentPanel) throws Exception {
		ArrayList<XugglerMovie_SB> movies = controller.getMovies();
		if (movies.isEmpty())
			return new Integer(0);

		if (startMovieTime >= 0) {
			goToPacketTimeStamp(startMovieTime);
		} else if (controller.getPercent() >= MultipleMovieController.NUMBER_OF_SLIDER_TICKS) {
			goToPacketTimeStamp(startMovieTime);
		}

		// get packet that belongs to our video
		long firstTimestampInStream = Global.NO_PTS;
		long systemClockStartTime = 0;
		long packetTimeStamp = startMovieTime;
		int counter = 0;

		boolean updateImage = false;

		
		while (controller.isPlaying()) {
			systemClockStartTime = System.currentTimeMillis();
			counter++;
			updateImage = false;

			switch (controller.getSpeedPointer()) {
			case 0:
			case 1:
			case 2:
				updateImage = true;
				break;

			case 3:
				if (counter == 2 || counter == 4 || counter == 6) {
					updateImage = true;
				}
				break;

			case 4:
				if (counter == 4) {
					updateImage = true;
				}
				break;

			case 5:
				// 8
				if (counter == 6) {
					updateImage = true;
				}
				break;
			}

			if (counter >= 6) {
				counter = 0;
			}
			for (XugglerMovie_SB movie : movies) {
				try {
					IPacket packet = IPacket.make();
					int readPacket = 0;

					try {
						readPacket = movie.getContainer()
								.readNextPacket(packet);
						packetTimeStamp = packet.getTimeStamp();

						if (packetTimeStamp >= endMovieTime) {
							controller.stopPlayingMovie();
						}

						controller.setCurrentPacketTimeStamp(packet
								.getTimeStamp());

					} catch (Exception ex1) {
						return new Integer(0);
					}

					if (readPacket < 0) {
						return new Integer(0);
					}

					if (packet.getStreamIndex() == movie.getVideoStreamId()) {

						IVideoPicture picture = IVideoPicture.make(movie
								.getVideoCoder().getPixelType(), movie
								.getVideoCoder().getWidth(), movie
								.getVideoCoder().getHeight());

						int offset = 0;
						while (offset < packet.getSize()) {

							int bytesDecoded = movie.getVideoCoder()
									.decodeVideo(picture, packet, offset);
							if (bytesDecoded < 0) {
								throw new RuntimeException(
										"got error decoding video in: "
												+ movie.getFilename());
							}
							// packet = null;
							offset += bytesDecoded;
							if (updateImage) {
								if (picture.isComplete()) {
									IVideoPicture newPic = picture;

									if (movie.getResampler() != null) {

										final int WIDTH = picture.getWidth();
										final int HEIGHT = picture.getHeight();
										final IPixelFormat.Type TYPE = movie
												.getResampler()
												.getOutputPixelFormat();

										newPic = IVideoPicture.make(TYPE,
												WIDTH, HEIGHT);
										if (movie.getResampler().resample(
												newPic, picture) < 0) {
											throw new RuntimeException(
													"could not resample video from: "
															+ movie.getFilename());
										}
										newPic.setComplete(true, TYPE, WIDTH,
												HEIGHT, 0);
									}

									if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
									{
										throw new RuntimeException(
												"could not decode video"
														+ " as BGR 24 bit data in: "
														+ movie.getFilename());
									}
									// convert the BGR24 to an Java buffered
									// image
									BufferedImage image = new BufferedImage(
											picture.getWidth(),
											picture.getHeight(),
											BufferedImage.TYPE_3BYTE_BGR);

									IConverter converter = ConverterFactory
											.createConverter(image,
													IPixelFormat.Type.BGR24);

									BufferedImage javaImage = converter
											.toImage(newPic);
									movie.loadNextFrameImage(javaImage);

									image.flush();
									image = null;
									converter.delete();
									converter = null;
								}
							}
						}
						if (movie.equals(movies.get(movies.size() - 1))) {
							controller.setCurrentPacketTimeStamp(packet
									.getTimeStamp());

							if (firstTimestampInStream == Global.NO_PTS) {
								firstTimestampInStream = picture.getTimeStamp();
							} else {
								long systemClockCurrentTime = System
										.currentTimeMillis();

								long millisecondsToSleepDif = systemClockCurrentTime
										- systemClockStartTime; // + 50;

								double frameRate = movie.getContainer()
										.getStream(packet.getStreamIndex())
										.getStreamCoder().getFrameRate()
										.getDouble();

								long millisecondsToSleep = (long) (1000 / frameRate);

								// speed[] = { 0.25, 0.5, 1, 2, 4, 8 };
								// 0, 1, 2, 3, 4, 5

								switch (controller.getSpeedPointer()) {

								case 0:
									// 0.25
									millisecondsToSleep = millisecondsToSleep
											* 4 - millisecondsToSleepDif;
									break;
								case 1:
									// 0.5
									millisecondsToSleep = millisecondsToSleep
											* 2 - millisecondsToSleepDif;
									break;
								case 2:
									// 1
									millisecondsToSleep = millisecondsToSleep
											- millisecondsToSleepDif;
									break;
								case 3:
									// 2x
									millisecondsToSleep = millisecondsToSleep
											/ 2 - millisecondsToSleepDif;
									break;
								case 4:
									// 4x
									millisecondsToSleep = millisecondsToSleep
											/ 3 - millisecondsToSleepDif;
									break;
								case 5:
									// 8x
									millisecondsToSleep = millisecondsToSleep
											/ 4 - millisecondsToSleepDif;
									break;
								}

								if (millisecondsToSleep > 0) {
									try {
										Thread.sleep(millisecondsToSleep);
									} catch (InterruptedException e) {
										// we might get this when the user
										// closes the dialog box,
										// so just return from the method.
										return new Integer(0);
									}
								}
								for (XugglerMovie_SB m : movies) {
									m.updateJavaWindow(true);
								}
							}
							packet.delete();
							picture.delete();
						} else {
							// not a video stream, silently drop it.
							do {
							} while (false);
						}
					}
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		}
		return new Integer(1);
	}

	public void executeDragging() {
		int dragToTime = controller.getDragToTime();

		if (dragToTime == -1)
			return;

		long currentPacketTimeStamp = controller.getCurrentTime();
		long newCurrentPacketTimeStamp = currentPacketTimeStamp;
		long packetDuration = controller.getPacketDuration();

		long dFrame = -1;

		if (currentPacketTimeStamp == dragToTime) {
			dFrame = 0;
		} else if (currentPacketTimeStamp < dragToTime) {
			dFrame = (dragToTime - currentPacketTimeStamp) / packetDuration;
		}

		int time = dragToTime;
		dragToTime = -1;
		controller.setDragToTime(dragToTime);

		ArrayList<XugglerMovie_SB> movies = controller.getMovies();

		for (XugglerMovie_SB movie : movies) {

			if (dFrame == -1 || dFrame > 20) {
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(), -1,
						0);
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(),
						time - packetDuration, IContainer.SEEK_FLAG_BACKWARDS);
			}

			boolean toScroll = true;

			long cPacketTimeStamp = currentPacketTimeStamp;

			while (toScroll) {
				// scroll frame by frame
				boolean b = false;
				while (!b) {
					IPacket packet = IPacket.make();
					int readPacket = 0;

					try {
						readPacket = movie.getContainer()
								.readNextPacket(packet);
						long packetTimeStamp = packet.getTimeStamp();

						if (packetTimeStamp < 0) {
							cPacketTimeStamp = cPacketTimeStamp
									+ packetDuration;
						} else {
							cPacketTimeStamp = packetTimeStamp;
						}

					} catch (Exception ex1) {
						;
					}

					if (readPacket < 0)
						return;

					if (packet.getStreamIndex() == movie.getVideoStreamId()) {
						b = true;

						IVideoPicture picture = IVideoPicture.make(movie
								.getVideoCoder().getPixelType(), movie
								.getVideoCoder().getWidth(), movie
								.getVideoCoder().getHeight());

						int offset = 0;
						while (offset < packet.getSize()) {

							int bytesDecoded = movie.getVideoCoder()
									.decodeVideo(picture, packet, offset);
							if (bytesDecoded < 0)
								throw new RuntimeException(
										"got error decoding video in: "
												+ movie.getFilename());
							offset += bytesDecoded;

							if (picture.isComplete()) {
								IVideoPicture newPic = picture;

								if (movie.getResampler() != null) {

									newPic = IVideoPicture.make(movie
											.getResampler()
											.getOutputPixelFormat(), picture
											.getWidth(), picture.getHeight());
									if (movie.getResampler().resample(newPic,
											picture) < 0)
										throw new RuntimeException(
												"could not resample video from: "
														+ movie.getFilename());
								}

								if (cPacketTimeStamp >= time
										|| controller.getMovieDuration() <= time) {

									if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
										throw new RuntimeException(
												"could not decode video"
														+ " as BGR 24 bit data in: "
														+ movie.getFilename());

									// convert the BGR24 to an Java buffered
									// image
									BufferedImage image = new BufferedImage(
											picture.getWidth(),
											picture.getHeight(),
											BufferedImage.TYPE_3BYTE_BGR);
									IConverter converter = ConverterFactory
											.createConverter(image,
													IPixelFormat.Type.BGR24);
									BufferedImage javaImage = converter
											.toImage(newPic);
									movie.loadNextFrameImage(javaImage);
									movie.updateJavaWindow(true);
									image.flush();
									image = null;
									toScroll = false;
									newCurrentPacketTimeStamp = cPacketTimeStamp;
									converter.delete();
								}
							}
						}

						picture.delete();
						packet.delete();
					} else {
						// not a video stream, silently drop it.
						do {
						} while (false);
					}
				}
			}

			if (movie.equals(movies.get(movies.size() - 1))) {
				controller.setCurrentPacketTimeStamp(newCurrentPacketTimeStamp);
			}

			executeDragging();
		}
	}
}

public class MoviePlayer_SB extends SwingWorker<Integer, ArrayList<XugglerMovie_SB>> {

	private MultipleMovieController_SB controller = null;
	private int startMovieTime, endMovieTime;

	public MoviePlayer_SB(final MultipleMovieController_SB controller,
			int startMovieTime, int endMovieTime
			) {

		this.controller = controller;
		this.startMovieTime = startMovieTime;
		this.endMovieTime = endMovieTime;
	}

	@Override
	protected Integer doInBackground() throws Exception {
		return controller.getMovieActionDelegate().play(startMovieTime, endMovieTime, null);
	}

	@Override
	protected void done() {
		
		if(controller.isPlaying()) {
			controller.stopPlayingMovie();
		}

		if (controller.getCurrentTime() < 0) {
			SS3Singleton.getVideoControlManager().moveSliderTo(
					MultipleMovieController.NUMBER_OF_SLIDER_TICKS);
		}
	}
}



}
