package edu.bu.signstream.ui.panels.table.sortable;

import java.util.*;
import javax.swing.table.*;

public class SortableTableModel extends AbstractTableModel {
	protected Object[] columnNames = null;

	protected Object[][] data = null;

	public SortableTableModel(Object[] columnNames, Object[][] data) {
		this.columnNames = columnNames;
		this.data = data;
	}

	public ArrayList getData() {
		ArrayList list = new ArrayList();
		for (int i = 0; i < data.length; i++) {
			list.add(data[i]);
		}
		return list;
	}

	@Override
	public int getColumnCount() {
		return columnNames.length;
	}

	@Override
	public int getRowCount() {
		return data == null ? 0 : data.length;
	}

	@Override
	public String getColumnName(int col) {
		return (String) columnNames[col];
	}

	@Override
	public Object getValueAt(int row, int col) {
		return data[row][col];
	}

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
      data[rowIndex][columnIndex] = aValue;
      fireTableCellUpdated(rowIndex, columnIndex);
	}
	
	public void update(ArrayList valueList) {
		Object[][] data = new Object[valueList.size()][];

		for (int j = 0; j < valueList.size(); j++) {
			data[j] = (Object[]) valueList.get(j);
		}
      
      reloadData(data);

	}

	/*
	 * JTable uses this method to determine the default renderer/ editor for
	 * each cell. If we didn't implement this method, then the last column would
	 * contain text ("true"/"false"), rather than a check box.
	 */
	@Override
	public Class getColumnClass(int c) {
		return getValueAt(0, c).getClass();
	}

	/*
	 * Don't need to implement this method unless your table's editable.
	 */
	@Override
	public boolean isCellEditable(int row, int col) {
		// Note that the data/cell address is constant,
		// no matter where the cell appears onscreen.      
      return col == 4;
	}

	public void update(int row, int col) {      
		if (col == 0) {
			Boolean selected = (Boolean) data[row][col];
			data[row][col] = new Boolean(!selected.booleanValue());
		}
	}

	public void reloadData(Object[][] dataToUpdate) {
		for (int i = 0; i < data.length; i++) {
			Object[] obj = data[i];
			Boolean selected = (Boolean) obj[0];
			String name = (String) obj[2];

			if (selected.booleanValue()) {
				for (int j = 0; j < dataToUpdate.length; j++) {
					Object[] o = dataToUpdate[j];
					String n = (String) o[2];

					if (n.equals(name))
						dataToUpdate[j][0] = selected;
				}
			}
		}
		this.data = dataToUpdate;
	}

	public ArrayList getSelectedFields() {
		ArrayList selectedFields = new ArrayList();

		for (int i = 0; i < data.length; i++) {
			Object[] obj = data[i];
			Boolean selected = (Boolean) obj[0];

			if (selected.booleanValue()) {
				selectedFields.add(obj[5]);
			}
		}
		return selectedFields;
	}
}
