package edu.bu.signstream.ui.video;

import java.awt.image.BufferedImage;
import java.util.ArrayList;

import com.xuggle.xuggler.Global;
import com.xuggle.xuggler.IContainer;
import com.xuggle.xuggler.IPacket;
import com.xuggle.xuggler.IPixelFormat;
import com.xuggle.xuggler.IVideoPicture;
import com.xuggle.xuggler.video.ConverterFactory;
import com.xuggle.xuggler.video.IConverter;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class MovieActionDelegate {

	private MultipleMovieController controller = null;

	public MovieActionDelegate(MultipleMovieController controller) {
		this.controller = controller;
	}

	public long playNextFrame() {
		long currentPacketTimeStamp = controller.getCurrentTime();
		long packetDuration = controller.getPacketDuration();

		ArrayList<XugglerMovie> movies = controller.getMovies();

		if (movies.isEmpty())
			return -1;

		boolean b = false;
		while (!b) {

			for (XugglerMovie movie : movies) {
				IPacket packet = IPacket.make();
				int readPacket = 0;

				try {
					readPacket = movie.getContainer().readNextPacket(packet);

					long packetTimeStamp = packet.getTimeStamp();

					if (packetTimeStamp < 0) {
//					if (packetTimeStamp <= 0) {
						currentPacketTimeStamp = 0;
						// we cannot use packetDuration as a way to step forward, since we can't rely that it has been set properly by this point
						// also b/c this method is used by MultipleMovieController.load() to calculate packetDuration in the first place
//						currentPacketTimeStamp = currentPacketTimeStamp
//								+ packetDuration;
					} else {
						currentPacketTimeStamp = packetTimeStamp;
					}

				} catch (Exception ex1) {
					return -1;
				}

				if (readPacket < 0)
					return -1;

				if (packet.getStreamIndex() == movie.getVideoStreamId()) {
					b = true;

					IVideoPicture picture = IVideoPicture.make(movie
							.getVideoCoder().getPixelType(), movie
							.getVideoCoder().getWidth(), movie.getVideoCoder()
							.getHeight());

					int offset = 0;
					while (offset < packet.getSize()) {

						int bytesDecoded = movie.getVideoCoder().decodeVideo(
								picture, packet, offset);
						if (bytesDecoded < 0)
							throw new RuntimeException(
									"got error decoding video in: "
											+ movie.getFilename());
						offset += bytesDecoded;

						if (picture.isComplete()) {
							IVideoPicture newPic = picture;

							if (movie.getResampler() != null) {

								newPic = IVideoPicture
										.make(movie.getResampler()
												.getOutputPixelFormat(),
												picture.getWidth(), picture
														.getHeight());
								if (movie.getResampler().resample(newPic,
										picture) < 0)
									throw new RuntimeException(
											"could not resample video from: "
													+ movie.getFilename());
							}
							if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
								throw new RuntimeException(
										"could not decode video"
												+ " as BGR 24 bit data in: "
												+ movie.getFilename());

							// convert the BGR24 to an Java buffered image
							BufferedImage image = new BufferedImage(
									picture.getWidth(), picture.getHeight(),
									BufferedImage.TYPE_3BYTE_BGR);
							IConverter converter = ConverterFactory
									.createConverter(image,
											IPixelFormat.Type.BGR24);
							BufferedImage javaImage = converter.toImage(newPic);
							image.flush();
							image = null;
							movie.loadNextFrameImage(javaImage);
							movie.updateJavaWindow(true);

							newPic.delete();
							converter.delete();
						}
					}

					if (movie.equals(movies.get(movies.size() - 1))) {
						controller.setCurrentPacketTimeStamp(packet
								.getTimeStamp());
//						controller.setCurrentPacketTimeStamp(currentPacketTimeStamp);
					} else {
						// not a video stream, silently drop it.
						do {
						} while (false);
					}

					packet.delete();
					picture.delete();
				}
			}
		}
		return currentPacketTimeStamp;
	}

	public long goToPreviousKeyFrame(long goToPacketTime, XugglerMovie movie) {

		if (goToPacketTime == -1) {
			return -1;
		}

		long packetDuration = controller.getPacketDuration();

		long p = packetDuration * 30;
		long minTimeStamp = goToPacketTime - p;
		long targetTimeStamp = goToPacketTime - 2 * packetDuration;
		long maxTimeStamp = goToPacketTime - packetDuration;

		if (minTimeStamp < 0) {
			minTimeStamp = 0;
		}

		if (maxTimeStamp < 0) {
			maxTimeStamp = 0;
		}

		long packetTimeStamp = -1;

		movie.getContainer().seekKeyFrame(movie.getVideoStreamId(), -1, 0);
		movie.getContainer().seekKeyFrame(movie.getVideoStreamId(),
				minTimeStamp, targetTimeStamp, maxTimeStamp,
				IContainer.SEEK_FLAG_FRAME | IContainer.SEEK_FLAG_BACKWARDS);

		IPacket packet = IPacket.make();

		int readPacket = -1;

		boolean b = false;
		while (!b) {
			packet = IPacket.make();

			try {
				readPacket = movie.getContainer().readNextPacket(packet);

				if (packet.getStreamIndex() == movie.getVideoStreamId()) {
					b = true;
				}
			} catch (Exception ex1) {
				ex1.printStackTrace();
			}
		}

		// isKeyFrame
		if (packet.isKeyPacket()) {

			if (readPacket < 0)
				return -1;

			packetTimeStamp = packet.getTimeStamp();

			IVideoPicture picture = IVideoPicture.make(movie.getVideoCoder()
					.getPixelType(), movie.getVideoCoder().getWidth(), movie
					.getVideoCoder().getHeight());

			int offset = 0;

			int bytesDecoded = movie.getVideoCoder().decodeVideo(picture,
					packet, offset);

			if (bytesDecoded < 0)
				throw new RuntimeException("got error decoding video in: "
						+ movie.getFilename());
			offset += bytesDecoded;
		
			if (picture.isComplete()) {
				IVideoPicture newPic = picture;

				if (movie.getResampler() != null) {

					newPic = IVideoPicture.make(movie.getResampler()
							.getOutputPixelFormat(), picture.getWidth(),
							picture.getHeight());
					if (movie.getResampler().resample(newPic, picture) < 0)
						throw new RuntimeException(
								"could not resample video from: "
										+ movie.getFilename());
				}

				if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
					throw new RuntimeException("could not decode video"
							+ " as BGR 24 bit data in: " + movie.getFilename());

				// convert the BGR24 to an Java buffered image
				BufferedImage image = new BufferedImage(picture.getWidth(),
						picture.getHeight(), BufferedImage.TYPE_3BYTE_BGR);

				IConverter converter = ConverterFactory.createConverter(image,
						IPixelFormat.Type.BGR24);

				BufferedImage javaImage = converter.toImage(newPic);
				image.flush();
				image = null;

				if (packetTimeStamp == goToPacketTime) {
					movie.loadNextFrameImage(javaImage);
					movie.updateJavaWindow(true);
				}

				picture.delete();
				packet.delete();

			} else {
				// not a video stream, silently drop it.
				do {
				} while (false);
			}
			return packetTimeStamp;
		} else {
			return goToPreviousKeyFrame(goToPacketTime - packetDuration, movie);
		}
	}

	public void goToPacketTimeStamp(long goToPacketTime) {
		long packetDuration = controller.getPacketDuration();

		long movieDuration = controller.getMovieDuration();

		long frameNumberLong = goToPacketTime/packetDuration;
		int frameNumber = (int)frameNumberLong;
				
		goToPacketTime = frameNumber*packetDuration;
		
		if(goToPacketTime < 1) {
			goToPacketTime = 0;
		}
		
		if (goToPacketTime > movieDuration) {
			return;
		}

		ArrayList<XugglerMovie> movies = controller.getMovies();
		
		if(goToPacketTime == 0) {
			for (XugglerMovie movie : movies) {			
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(), -1, 0);
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(), 0, 0, 0, IContainer.SEEK_FLAG_FRAME);
			}
			
			playNextFrame();
			return;
		}
				
		for (XugglerMovie movie : movies) {

			long currentPacketTimeStamp = goToPreviousKeyFrame(goToPacketTime, movie);
			
//			controller
//			.setCurrentPacketTimeStamp(currentPacketTimeStamp);	// doing this here results in looping at end of video; see readNextPacket call below

			if (currentPacketTimeStamp < goToPacketTime) {

				boolean toScroll = true;
				long cPacketTimeStamp = currentPacketTimeStamp;

				long newCurrentPacketTimeStamp = currentPacketTimeStamp;

				while (toScroll) {
					// scroll frame by frame
					boolean b = false;
					while (!b) {
						IPacket packet = IPacket.make();
						int readPacket = 0;

						try {
							readPacket = movie.getContainer().readNextPacket(packet);		// when last packet is read, return negative, then returns below
							// https://groups.google.com/forum/#!topic/xuggler-users/FXgmW4dViF8
							// read there, if later we want to make a more permanent fix as to to reading that last packet fully
							long packetTimeStamp = packet.getTimeStamp();	

							if (packetTimeStamp < 0) {
								cPacketTimeStamp = cPacketTimeStamp
										+ packetDuration;
							} else {
								cPacketTimeStamp = packetTimeStamp;
							}

						} catch (Exception ex1) {
							;
						}

						if (readPacket < 0)
							return;

						if (packet.getStreamIndex() == movie.getVideoStreamId()) {
							b = true;

							IVideoPicture picture = IVideoPicture.make(movie
									.getVideoCoder().getPixelType(), movie
									.getVideoCoder().getWidth(), movie
									.getVideoCoder().getHeight());

							int offset = 0;
							while (offset < packet.getSize()) {

								int bytesDecoded = movie.getVideoCoder()
										.decodeVideo(picture, packet, offset);
								if (bytesDecoded < 0)
									throw new RuntimeException(
											"got error decoding video in: "
													+ movie.getFilename());
								offset += bytesDecoded;

								if (cPacketTimeStamp >= goToPacketTime
										|| movieDuration < goToPacketTime) {

									if (picture.isComplete()) {
										IVideoPicture newPic = picture;

										if (movie.getResampler() != null) {

											newPic = IVideoPicture.make(movie
													.getResampler()
													.getOutputPixelFormat(),
													picture.getWidth(), picture
															.getHeight());
											if (movie.getResampler().resample(
													newPic, picture) < 0)
												throw new RuntimeException(
														"could not resample video from: "
																+ movie.getFilename());
										}

										if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
											throw new RuntimeException(
													"could not decode video"
															+ " as BGR 24 bit data in: "
															+ movie.getFilename());

										// convert the BGR24 to an Java buffered
										// image
										BufferedImage image = new BufferedImage(
												picture.getWidth(),
												picture.getHeight(),
												BufferedImage.TYPE_3BYTE_BGR);

										IConverter converter = ConverterFactory
												.createConverter(image,
														IPixelFormat.Type.BGR24);

										BufferedImage javaImage = converter
												.toImage(newPic);
										image.flush();
										image = null;

										movie.loadNextFrameImage(javaImage);
										movie.updateJavaWindow(true);

										toScroll = false;
										newCurrentPacketTimeStamp = cPacketTimeStamp;
										newPic.delete();
										converter.delete();
									}
								}
							}
							// converter.delete();
							picture.delete();
							packet.delete();

						} else {
							// not a video stream, silently drop it.
							do {
							} while (false);
						}
					}
				}

				if (movie.equals(movies.get(movies.size() - 1))) {
					controller
							.setCurrentPacketTimeStamp(newCurrentPacketTimeStamp);
				}
			}
		}
	}
	
//	private boolean stop = false;
//	
////	public void stopPlayingMovie(boolean stop) {
//		this.stop = stop;
//	}

	public Integer play(long startMovieTime, long endMovieTime,
			SignStreamSegmentPanel segmentPanel) throws Exception {
		ArrayList<XugglerMovie> movies = controller.getMovies();

		if (movies.isEmpty())
			return new Integer(0);

		if (startMovieTime >= 0) {
			goToPacketTimeStamp(startMovieTime);
		} else if (controller.getPercent() >= MultipleMovieController.NUMBER_OF_SLIDER_TICKS) {
			goToPacketTimeStamp(startMovieTime);
		}

		// get packet that belongs to our video
		long firstTimestampInStream = Global.NO_PTS;
		long systemClockStartTime = 0;
		long packetTimeStamp = startMovieTime;
		int counter = 0;

		boolean updateImage = false;

		while (controller.isPlaying()) {
			systemClockStartTime = System.currentTimeMillis();
			counter++;
			updateImage = false;

			switch (controller.getSpeedPointer()) {
			case 0:
			case 1:
			case 2:
				updateImage = true;
				break;

			case 3:
				if (counter == 2 || counter == 4 || counter == 6) {
					updateImage = true;
				}
				break;

			case 4:
				if (counter == 4) {
					updateImage = true;
				}
				break;

			case 5:
				// 8
				if (counter == 6) {
					updateImage = true;
				}
				break;
			}

			if (counter >= 6) {
				SS3Singleton.getVideoControlManager().getSlider()
						.setSliderPositionTime((int) packetTimeStamp);
				segmentPanel.repaint();
				counter = 0;
			}

			for (XugglerMovie movie : movies) {
				try {
					IPacket packet = IPacket.make();
					int readPacket = 0;

					try {
						readPacket = movie.getContainer()
								.readNextPacket(packet);
						packetTimeStamp = packet.getTimeStamp();

						if (packetTimeStamp >= endMovieTime) {
							controller.stopPlayingMovie();
							SS3Singleton
									.getVideoControlManager()
									.getSlider()
									.setSliderPositionTime(
											(int) packetTimeStamp);
							segmentPanel.repaint();
						}

						controller.setCurrentPacketTimeStamp(packet
								.getTimeStamp());

					} catch (Exception ex1) {
						return new Integer(0);
					}

					if (readPacket < 0)
						return new Integer(0);

					if (packet.getStreamIndex() == movie.getVideoStreamId()) {

						IVideoPicture picture = IVideoPicture.make(movie
								.getVideoCoder().getPixelType(), movie
								.getVideoCoder().getWidth(), movie
								.getVideoCoder().getHeight());

						int offset = 0;
						while (offset < packet.getSize()) {

							int bytesDecoded = movie.getVideoCoder()
									.decodeVideo(picture, packet, offset);
							if (bytesDecoded < 0)
								throw new RuntimeException(
										"got error decoding video in: "
												+ movie.getFilename());
							// packet = null;
							offset += bytesDecoded;
							if (updateImage) {
								if (picture.isComplete()) {
									IVideoPicture newPic = picture;

									if (movie.getResampler() != null) {

										final int WIDTH = picture.getWidth();
										final int HEIGHT = picture.getHeight();
										final IPixelFormat.Type TYPE = movie
												.getResampler()
												.getOutputPixelFormat();

										newPic = IVideoPicture.make(TYPE,
												WIDTH, HEIGHT);
										if (movie.getResampler().resample(
												newPic, picture) < 0)
											throw new RuntimeException(
													"could not resample video from: "
															+ movie.getFilename());

										newPic.setComplete(true, TYPE, WIDTH,
												HEIGHT, 0);
									}

									if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
										throw new RuntimeException(
												"could not decode video"
														+ " as BGR 24 bit data in: "
														+ movie.getFilename());

									// convert the BGR24 to an Java buffered
									// image
									BufferedImage image = new BufferedImage(
											picture.getWidth(),
											picture.getHeight(),
											BufferedImage.TYPE_3BYTE_BGR);

									IConverter converter = ConverterFactory
											.createConverter(image,
													IPixelFormat.Type.BGR24);

									BufferedImage javaImage = converter
											.toImage(newPic);
									movie.loadNextFrameImage(javaImage);

									image.flush();
									image = null;
									converter.delete();
									converter = null;
								}
							}
						}

						if (movie.equals(movies.get(movies.size() - 1))) {
							controller.setCurrentPacketTimeStamp(packet
									.getTimeStamp());

							if (firstTimestampInStream == Global.NO_PTS) {
								firstTimestampInStream = picture.getTimeStamp();
							} else {
								long systemClockCurrentTime = System
										.currentTimeMillis();

								long millisecondsToSleepDif = systemClockCurrentTime
										- systemClockStartTime; // + 50;

								double frameRate = movie.getContainer()
										.getStream(packet.getStreamIndex())
										.getStreamCoder().getFrameRate()
										.getDouble();

								long millisecondsToSleep = (long) (1000 / frameRate);

								// speed[] = { 0.25, 0.5, 1, 2, 4, 8 };
								// 0, 1, 2, 3, 4, 5

								switch (controller.getSpeedPointer()) {

								case 0:
									// 0.25
									millisecondsToSleep = millisecondsToSleep
											* 4 - millisecondsToSleepDif;
									break;
								case 1:
									// 0.5
									millisecondsToSleep = millisecondsToSleep
											* 2 - millisecondsToSleepDif;
									break;
								case 2:
									// 1
									millisecondsToSleep = millisecondsToSleep
											- millisecondsToSleepDif;
									break;
								case 3:
									// 2x
									millisecondsToSleep = millisecondsToSleep
											/ 2 - millisecondsToSleepDif;
									break;
								case 4:
									// 4x
									millisecondsToSleep = millisecondsToSleep
											/ 3 - millisecondsToSleepDif;
									break;
								case 5:
									// 8x
									millisecondsToSleep = millisecondsToSleep
											/ 4 - millisecondsToSleepDif;
									break;
								}

								if (millisecondsToSleep > 0) {
									try {
										Thread.sleep(millisecondsToSleep);
									} catch (InterruptedException e) {
										// we might get this when the user
										// closes the dialog box,
										// so just return from the method.
										return new Integer(0);
									}
								}

								for (XugglerMovie m : movies) {
									m.updateJavaWindow(true);
								}
							}
							packet.delete();
							picture.delete();
						} else {
							// not a video stream, silently drop it.
							do {
							} while (false);
						}
					}
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		}
		
		//
		//System.err.println("STOPED");
		//
		
		return new Integer(1);
	}

	public void executeDragging() {
		int dragToTime = controller.getDragToTime();

		if (dragToTime == -1)
			return;

		long currentPacketTimeStamp = controller.getCurrentTime();
		long newCurrentPacketTimeStamp = currentPacketTimeStamp;
		long packetDuration = controller.getPacketDuration();

		long dFrame = -1;

		if (currentPacketTimeStamp == dragToTime) {
			dFrame = 0;
		} else if (currentPacketTimeStamp < dragToTime) {
			dFrame = (dragToTime - currentPacketTimeStamp) / packetDuration;
		}

		int time = dragToTime;
		dragToTime = -1;
		controller.setDragToTime(dragToTime);

		ArrayList<XugglerMovie> movies = controller.getMovies();

		for (XugglerMovie movie : movies) {

			if (dFrame == -1 || dFrame > 20) {
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(), -1,
						0);
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(),
						time - packetDuration, IContainer.SEEK_FLAG_BACKWARDS);
			}

			boolean toScroll = true;

			long cPacketTimeStamp = currentPacketTimeStamp;

			while (toScroll) {
				// scroll frame by frame
				boolean b = false;
				while (!b) {
					IPacket packet = IPacket.make();
					int readPacket = 0;

					try {
						readPacket = movie.getContainer()
								.readNextPacket(packet);
						long packetTimeStamp = packet.getTimeStamp();

						if (packetTimeStamp < 0) {
							cPacketTimeStamp = cPacketTimeStamp
									+ packetDuration;
						} else {
							cPacketTimeStamp = packetTimeStamp;
						}

					} catch (Exception ex1) {
						;
					}

					if (readPacket < 0)
						return;

					if (packet.getStreamIndex() == movie.getVideoStreamId()) {
						b = true;

						IVideoPicture picture = IVideoPicture.make(movie
								.getVideoCoder().getPixelType(), movie
								.getVideoCoder().getWidth(), movie
								.getVideoCoder().getHeight());

						int offset = 0;
						while (offset < packet.getSize()) {

							int bytesDecoded = movie.getVideoCoder()
									.decodeVideo(picture, packet, offset);
							if (bytesDecoded < 0)
								throw new RuntimeException(
										"got error decoding video in: "
												+ movie.getFilename());
							offset += bytesDecoded;

							if (picture.isComplete()) {
								IVideoPicture newPic = picture;

								if (movie.getResampler() != null) {

									newPic = IVideoPicture.make(movie
											.getResampler()
											.getOutputPixelFormat(), picture
											.getWidth(), picture.getHeight());
									if (movie.getResampler().resample(newPic,
											picture) < 0)
										throw new RuntimeException(
												"could not resample video from: "
														+ movie.getFilename());
								}

								if (cPacketTimeStamp >= time
										|| controller.getMovieDuration() < time) {

									if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
										throw new RuntimeException(
												"could not decode video"
														+ " as BGR 24 bit data in: "
														+ movie.getFilename());

									// convert the BGR24 to an Java buffered
									// image
									BufferedImage image = new BufferedImage(
											picture.getWidth(),
											picture.getHeight(),
											BufferedImage.TYPE_3BYTE_BGR);
									IConverter converter = ConverterFactory
											.createConverter(image,
													IPixelFormat.Type.BGR24);
									BufferedImage javaImage = converter
											.toImage(newPic);
									movie.loadNextFrameImage(javaImage);
									movie.updateJavaWindow(true);
									image.flush();
									image = null;
									toScroll = false;
									newCurrentPacketTimeStamp = cPacketTimeStamp;
									converter.delete();
								}
							}
						}

						picture.delete();
						packet.delete();
					} else {
						// not a video stream, silently drop it.
						do {
						} while (false);
					}
				}
			}

			if (movie.equals(movies.get(movies.size() - 1))) {
				controller.setCurrentPacketTimeStamp(newCurrentPacketTimeStamp);
			}

			executeDragging();
		}
	}
}
