package edu.bu.signstream.ui.video;

import java.util.ArrayList;

import javax.swing.SwingWorker;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.Slider;
import edu.bu.signstream.ui.video.actions.MoviePlayer;

public class MultipleMovieController {

	private ArrayList<XugglerMovie> movies = new ArrayList<XugglerMovie>();
	private VideoSegment selectedVideoSegment = new VideoSegment(0, 0);
	private long currentPacketTimeStamp = 0;
	private int packetDuration = 2000; // 4000;

	private int speedPointer = 2;
	// private double speed[] = { 0.25, 0.5, 1, 2, 4, 8 };

	private int dragToTime = -1;
	private int skippedFrames = 0;
	private boolean play = false;
	private boolean doneSkipping = true;

	public static int NUMBER_OF_SLIDER_TICKS = 10000;

	private MovieActionDelegate actionDelegate = null;

	public MultipleMovieController() {
		actionDelegate = new MovieActionDelegate(this);
	}

	MoviePlayer player = null;

	public void stopPalayingVideoUpdateVideoControlPanel() {
		if (isPlaying()) {
			play = false;
			if (player != null) {
				player.cancel(true);
			}
			ArrayList<VideoControlPanel> videoPanels = SS3Singleton
					.getVideoControlManager().getVideoControlPanels();
			for (VideoControlPanel videoPanel : videoPanels) {
				videoPanel.setStopVideoControls();
			}
		}
	}

	public MovieActionDelegate getMovieActionDelegate() {
		return actionDelegate;
	}

	public void load() {
		if (movies.isEmpty())
			return;

		for (XugglerMovie movie : movies) {
			String filename = movie.getFilename();
			System.out.println("FILENAME = " + filename);
		}

		if (!play) {
			play = true;
			// calculate packet duration
//			long timeStamp = actionDelegate.playNextFrame();
//			currentPacketTimeStamp = actionDelegate.playNextFrame();
//			packetDuration = (int) (currentPacketTimeStamp - timeStamp);
			
			// new method: oftentimes the playNextFrame will return 0 as the timestamp, when IPacket.getTimeStamp() returns
			// a negative number (previously this was using packetDuration as a means of advancing the timestamp automatically,
			// but since packetDuration is being calculated on the fly here, we cannot do it that way)
			// so: playNextFrame() until we have positive timestamps (twice) and then calculate the time between them
			long timeStamp, nextTimeStamp;
			
			do {
				timeStamp = actionDelegate.playNextFrame();
			} while (timeStamp <= 0);
			
			// theoretically if the above do-while already passed the 0/negative timestamps, then the next call to 
			// playNextFrame() should be fine, but just wrapping nextTimeStamp calulation in a do-while as well out of an abundance of caution
			do {
				nextTimeStamp = actionDelegate.playNextFrame();
			} while (nextTimeStamp <= 0);
				
			packetDuration = (int) (nextTimeStamp - timeStamp);
			
			play = false;
		}
	}

	public void startPlayingMovie() {
		play = true;
	}

	public int getSpeedPointer() {
		return speedPointer;
	}

	public String speeding(boolean slover) {
		if (slover) {
			if (speedPointer < 5) {
				speedPointer++;
			} else {
				speedPointer = 5;
			}
		} else {
			if (speedPointer > 0) {
				speedPointer--;
			} else {
				speedPointer = 0;
			}
		}

		String label = "";

		switch (speedPointer) {
		case 0:
			label = "/4";
			break;

		case 1:
			label = "/2";
			break;

		case 3:
			label = "2X";
			break;

		case 4:
			label = "4X";
			break;

		case 5:
			label = "8X";
			break;
		}

		return label;
	}

	public void playSegment(int startMovieTime, int endMovieTime,
			SignStreamSegmentPanel segmentPanel) {
		if (!play) {
			play = true;
			ArrayList<VideoControlPanel> videoPanels = SS3Singleton
					.getVideoControlManager().getVideoControlPanels();
			player = new MoviePlayer(this, videoPanels, startMovieTime,
					endMovieTime, segmentPanel);
			player.execute();
		}
	}

	public long getPacketDuration() {
		return packetDuration;
	}

	public void setDoneSkipping(boolean done) {
		this.doneSkipping = done;
	}

	public void skipNumberOfFrames(int moveNumberOfFrames) {
		if (!play) {
			play = true;
			if (this.doneSkipping) {
				skippedFrames = skippedFrames + moveNumberOfFrames;
				FrameSkipper fs = new FrameSkipper(this, skippedFrames);
				this.doneSkipping = false;
				this.skippedFrames = 0;
				fs.execute();
			} else {
				skippedFrames = skippedFrames + moveNumberOfFrames;
			}
			play = false;
		}
	}

	public long frameBackward(long packetTimeStamp) {
		long goToPacketTimeStamp = packetTimeStamp - packetDuration;
		actionDelegate.goToPacketTimeStamp(goToPacketTimeStamp);

		return currentPacketTimeStamp;
	}

	public long framesXBackward(int frames) {
		long newTime = currentPacketTimeStamp - (frames * packetDuration);
		newTime = newTime < 0 ? 0 : newTime;
		actionDelegate.goToPacketTimeStamp(newTime);
		return currentPacketTimeStamp;
	}

	public long frameForward(long packetTimeStamp) {
		actionDelegate.goToPacketTimeStamp(packetTimeStamp);

		return currentPacketTimeStamp;
	}

	private void goToFrame(int frameNumber) {

		if (movies.size() < 1 || frameNumber <= 0)
			return;

		int targetMovieTime = frameNumber * packetDuration;
		actionDelegate.goToPacketTimeStamp(targetMovieTime);
	}

	public int getCurrentFrame() {
		return (int) currentPacketTimeStamp / packetDuration;
	}

	public int getPercent() {
		long y = MultipleMovieController.NUMBER_OF_SLIDER_TICKS
				* currentPacketTimeStamp / getMovieDuration();

		return (int) y;
	}

	public long goToVideoPercent(int value) {
		if (movies.size() < 1)
			return -1;

		stopPalayingVideoUpdateVideoControlPanel();

		play = true;

		double numberOfSticks = MultipleMovieController.NUMBER_OF_SLIDER_TICKS;
		int movieDuration = getMovieDuration();

		double numberOfFrames = movieDuration / packetDuration;
		double framesInOneStick = numberOfFrames / numberOfSticks;
		double goToFrameNumber = value * framesInOneStick;

		int frameToGo = (int) goToFrameNumber;
		int movieTimeToSeek = frameToGo * packetDuration;

		if (value <= 2) {
			// go to first frame
			goToFirstFrame();
		} else if (value + 2 >= numberOfSticks) {
			// go to last frame
			goToLastFrame();
		} else {
			for (XugglerMovie movie : movies) {
				actionDelegate.goToPreviousKeyFrame(movieTimeToSeek, movie);
			}

			actionDelegate.playNextFrame();
		}

		play = false;
		return currentPacketTimeStamp;
	}

	public void setCurrentTime(int currentTime) {
		actionDelegate.goToPacketTimeStamp(currentTime);
	}

	public int getDragToTime() {
		return dragToTime;
	}

	public void setDragToTime(int dragToTime) {
		this.dragToTime = dragToTime;
	}

	public void dragToCurrentTime(int currentTime) {
		if (!play) {
			play = true;
			if (currentTime <= getMovieDuration()) {
				if (dragToTime == -1) {
					this.dragToTime = currentTime;
					actionDelegate.executeDragging();
				} else {
					this.dragToTime = currentTime;
				}
			}
		}
		play = false;
	}

	public void goToFirstFrame() {
		goToFrame(1);
		goToFrame(1);
		// SS3Singleton.getVideoControlManager().synchronizeSlider();
	}

	public void goToLastFrame() {
		if (movies.size() < 1)
			return;

		int targetMovieTime = getMovieDuration() - packetDuration;
		actionDelegate.goToPacketTimeStamp(targetMovieTime);
	}

	public void removeAllMovies() {
		movies.clear();
	}

	public void setCurrentPacketTimeStamp(long currentPacketTimeStamp) {
		this.currentPacketTimeStamp = currentPacketTimeStamp;
	}

	public int getMovieDuration() {
		if (movies.size() < 1)
			return 0;

		XugglerMovie movie = movies.get(0);
		long numFrames = movie.getContainer()
				.getStream(movie.getVideoStreamId()).getNumFrames();

		return (int) numFrames * packetDuration;
	}

	public void selectSegment(int startTime, int endTime) {
		selectedVideoSegment.setStartTime(startTime);
		selectedVideoSegment.setEndTime(endTime);
	}

	public int getPrecedingFrameTime() {
		return (int) currentPacketTimeStamp - packetDuration;
	}

	public int getFollowingFrameTime() {
		return (int) currentPacketTimeStamp + packetDuration;
	}

	public int getPrecedingFrameTime(int time) {
		return time - packetDuration;
	}

	public int getFollowingFrameTime(int time) {
		return time + packetDuration;
	}

	public int getCurrentTime() {
		return (int) currentPacketTimeStamp;
	}

	public void registerMovie(XugglerMovie movie) {
		movies.add(movie);
	}

	public void removeMovie(XugglerMovie movie) {
		movies.remove(movie);
	}

	public void goToPacketTimeStamp(long goToPacketTime) {
		actionDelegate.goToPacketTimeStamp(goToPacketTime);
	}

	public void unloadVideoFiles(ArrayList<XugglerMovie> movies) {
		// for(XugglerMovie movie:movies) {
		// JDialog dialog = movie.getVideoImage();
		// dialog.removeAll();
		// dialog.removeNotify();
		// dialog.setVisible(false);
		// movies.remove(movie);
		// }
	}

	public ArrayList<XugglerMovie> getMovies() {
		return movies;
	}

	public boolean isPlaying() {
		return play;
	}

	public void stopPlayingMovie() {
		play = false;

		// player.cancel(true);
		// while(!player.isDone()) {
		// System.out.println("wait to end playing the video file");
		// }

		// if(player != null) {
		// player.cancel(true);
		// }
	}

	private class FrameSkipper extends
			SwingWorker<Integer, ArrayList<XugglerMovie>> {

		private MultipleMovieController controller = null;
		private int numberOfFramesToSkip = 0;

		FrameSkipper(final MultipleMovieController controller,
				int numberOfFramesToSkip) {
			this.controller = controller;
			this.numberOfFramesToSkip = numberOfFramesToSkip;
		}

		private void goToFrame() {
			long move = numberOfFramesToSkip * packetDuration;
			long goToPacketTime = currentPacketTimeStamp + move;

			controller.getMovieActionDelegate().goToPacketTimeStamp(
					goToPacketTime);

			SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();

			Slider slider = segmentPanel.getSlider();
			int time = getCurrentTime();
			slider.setSliderPositionTime(time);
			segmentPanel.repaint();

			int X = segmentPanel.getSlider().getSliderPosition();

			if (segmentPanel.isUtteranceLoaded()) {
				segmentPanel.getUtteranceView().setVisible(X, X);
			} else {
				segmentPanel.getDatabaseView().setVisible(X, X);
			}
		}

		@Override
		protected Integer doInBackground() throws Exception {
			goToFrame();
			return new Integer(1);
		}

		@Override
		protected void done() {
			controller.setDoneSkipping(true);
		}
	}
}

class VideoSegment {
	int startTime, endTime;

	VideoSegment(int startTime, int endTime) {
		this.startTime = startTime;
		this.endTime = endTime;
	}

	public int getStartTime() {
		return startTime;
	}

	public void setStartTime(int startTime) {
		this.startTime = startTime;
	}

	public int getEndTime() {
		return endTime;
	}

	public void setEndTime(int endTime) {
		this.endTime = endTime;
	}
}
