package edu.bu.signstream.ui.video;

import java.util.ArrayList;
import javax.swing.SwingWorker;

import com.xuggle.xuggler.IContainer;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.Slider;

import edu.bu.signstream.ui.video.actions.MoviePlayer;

public class MultipleMovieControllerSimulator extends MultipleMovieController {

	private ArrayList<XugglerMovie> movies = new ArrayList<XugglerMovie>();
	private VideoSegment selectedVideoSegment = new VideoSegment(0, 0);
	private long currentPacketTimeStamp = 0;
	private int packetDuration = 10001; // 4000;

	private int speedPointer = 2;
	// private double speed[] = { 0.25, 0.5, 1, 2, 4, 8 };

	private int dragToTime = -1;
	private int skippedFrames = 0;
	private boolean play = false;
	private boolean doneSkipping = true;

	public static int NUMBER_OF_SLIDER_TICKS = 10000;	
	
	private MovieActionDelegateSimulator actionDelegate = null;
	
	public MultipleMovieControllerSimulator() {
		actionDelegate = new MovieActionDelegateSimulator(this);
	}
	
	@Override
	public MovieActionDelegate getMovieActionDelegate() {
		return actionDelegate;
	}
	
	@Override
	public void load() {
		if (movies.isEmpty())
			return;

		if (!play) {
			play = true;
			// calculate packet duration
			long timeStamp = actionDelegate.playNextFrame();
			currentPacketTimeStamp = actionDelegate.playNextFrame();
			packetDuration = (int) (currentPacketTimeStamp - timeStamp);
			play = false;
		}
	}

	@Override
	public void startPlayingMovie() {
		play = true;
	}

	@Override
	public int getSpeedPointer() {
		return speedPointer;
	}

	@Override
	public String speeding(boolean slover) {
		if (slover) {
			if (speedPointer < 5) {
				speedPointer++;
			} else {
				speedPointer = 5;
			}
		} else {
			if (speedPointer > 0) {
				speedPointer--;
			} else {
				speedPointer = 0;
			}
		}

		String label = "";

		switch (speedPointer) {
		case 0:
			label = "/4";
			break;

		case 1:
			label = "/2";
			break;

		case 3:
			label = "2X";
			break;

		case 4:
			label = "4X";
			break;

		case 5:
			label = "8X";
			break;
		}

		return label;
	}

	@Override
	public void playSegment(int startMovieTime, int endMovieTime, SignStreamSegmentPanel segmentPanel) {

		if (!play) {
			play = true;
			ArrayList<VideoControlPanel> videoPanels = SS3Singleton
					.getVideoControlManager().getVideoControlPanels();
			MoviePlayer player = new MoviePlayer(this, videoPanels,
					startMovieTime, endMovieTime, segmentPanel);
			player.execute();
		}
	}

	@Override
	public long getPacketDuration() {
		return packetDuration;
	}

	@Override
	public void setDoneSkipping(boolean done) {
		this.doneSkipping = done;
	}

	@Override
	public void skipNumberOfFrames(int moveNumberOfFrames) {
		if (!play) {
			play = true;
			if (this.doneSkipping) {
				skippedFrames = skippedFrames + moveNumberOfFrames;
				FrameSkipper fs = new FrameSkipper(this, skippedFrames);
				this.doneSkipping = false;
				this.skippedFrames = 0;
				fs.execute();
			} else {
				skippedFrames = skippedFrames + moveNumberOfFrames;
			}
			play = false;
		}
	}

	@Override
	public long frameBackward(long packetTimeStamp) {
		long goToPacketTimeStamp = packetTimeStamp - packetDuration;
		actionDelegate.goToPacketTimeStamp(goToPacketTimeStamp);
		
		return currentPacketTimeStamp;
	}

	@Override
	public long framesXBackward(int frames) {
		long newTime = currentPacketTimeStamp - (frames * packetDuration);
		newTime = newTime < 0 ? 0 : newTime;
		actionDelegate.goToPacketTimeStamp(newTime);
		return currentPacketTimeStamp;
	}

	@Override
	public long frameForward(long packetTimeStamp) {
		//
		System.out.println("packetTimeStamp = " + packetTimeStamp);
		//
		actionDelegate.goToPacketTimeStamp(packetTimeStamp);

		return currentPacketTimeStamp;
	}

	private void goToFrame(int frameNumber) {

		if (movies.size() < 1 || frameNumber <= 0)
			return;

		int targetMovieTime = frameNumber * packetDuration;
		actionDelegate.goToPacketTimeStamp(targetMovieTime);
	}

	@Override
	public int getCurrentFrame() {
		return (int) currentPacketTimeStamp / packetDuration;
	}

	@Override
	public int getPercent() {
		long y = MultipleMovieController.NUMBER_OF_SLIDER_TICKS
				* currentPacketTimeStamp / getMovieDuration();

		return (int) y;
	}

//	@Override
//	public int goToVideoPercent(int percent) {
//		if (movies.size() < 1)
//			return 0;
//
//		if (!play) {
//			play = true;
//			long goToMovieTime = (getMovieDuration() / MultipleMovieController.NUMBER_OF_SLIDER_TICKS)
//					* percent;
//
//			if (goToMovieTime <= 0 && currentPacketTimeStamp < packetDuration)
//				return 0;
//
//			for (XugglerMovie movie : movies) {
//				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(),
//						goToMovieTime, IContainer.SEEK_FLAG_BACKWARDS);
//			}
//
//			currentPacketTimeStamp =actionDelegate.playNextFrame();
//			play = false;
//		}
//		return 0;
//	}

	@Override
	public void setCurrentTime(int currentTime) {
		actionDelegate.goToPacketTimeStamp(currentTime);
	}
	
	@Override
	public int getDragToTime() {
		return dragToTime;
	}
	
	@Override
	public void setDragToTime(int dragToTime) {
		this.dragToTime = dragToTime;
	}

	@Override
	public void dragToCurrentTime(int currentTime) {
		if (!play) {
			play = true;
			if (currentTime <= getMovieDuration()) {
				if (dragToTime == -1) {
					this.dragToTime = currentTime;
					actionDelegate.executeDragging();
				} else {
					this.dragToTime = currentTime;
				}
			}
		}
		play = false;
	}

	@Override
	public void goToFirstFrame() {
		goToFrame(1);
		goToFrame(1);
		// SS3Singleton.getVideoControlManager().synchronizeSlider();
	}

	@Override
	public void goToLastFrame() {
		if (movies.size() < 1)
			return;

		int targetMovieTime = getMovieDuration() - packetDuration;
		actionDelegate.goToPacketTimeStamp(targetMovieTime);	
	}
	
	@Override
	public void removeAllMovies() {
		movies.clear();
	}

	@Override
	public void setCurrentPacketTimeStamp(long currentPacketTimeStamp) {
		this.currentPacketTimeStamp = currentPacketTimeStamp;
	}

	@Override
	public int getMovieDuration() {
		if (movies.size() < 1)
			return 0;

		XugglerMovie movie = movies.get(0);
		long numFrames = movie.getContainer()
				.getStream(movie.getVideoStreamId()).getNumFrames();

		return (int) numFrames * packetDuration;
	}

	@Override
	public void selectSegment(int startTime, int endTime) {
		selectedVideoSegment.setStartTime(startTime);
		selectedVideoSegment.setEndTime(endTime);
	}

	@Override
	public int getPrecedingFrameTime() {
		return (int) currentPacketTimeStamp - packetDuration;
	}

	@Override
	public int getFollowingFrameTime() {
		return (int) currentPacketTimeStamp + packetDuration;
	}

	@Override
	public int getPrecedingFrameTime(int time) {
		return time - packetDuration;
	}

	@Override
	public int getFollowingFrameTime(int time) {
		return time + packetDuration;
	}

	@Override
	public int getCurrentTime() {
		return (int) currentPacketTimeStamp;
	}

	@Override
	public void registerMovie(XugglerMovie movie) {
		movies.add(movie);
	}

	@Override
	public void removeMovie(XugglerMovie movie) {
		movies.remove(movie);
	}
	
	@Override
	public void goToPacketTimeStamp(long goToPacketTime) {
		actionDelegate.goToPacketTimeStamp(goToPacketTime);
	}

	@Override
	public void unloadVideoFiles(ArrayList<XugglerMovie> movies) {
		// for(XugglerMovie movie:movies) {
		// JDialog dialog = movie.getVideoImage();
		// dialog.removeAll();
		// dialog.removeNotify();
		// dialog.setVisible(false);
		// movies.remove(movie);
		// }
	}

	@Override
	public ArrayList<XugglerMovie> getMovies() {
		return movies;
	}

	@Override
	public boolean isPlaying() {
		return play;
	}

	@Override
	public void stopPlayingMovie() {
		play = false;
	}

	private class FrameSkipper extends
			SwingWorker<Integer, ArrayList<XugglerMovie>> {

		private MultipleMovieController controller = null;
		private int numberOfFramesToSkip = 0;

		FrameSkipper(final MultipleMovieController controller,
				int numberOfFramesToSkip) {
			this.controller = controller;
			this.numberOfFramesToSkip = numberOfFramesToSkip;
		}

		private void goToFrame() {
			long move = numberOfFramesToSkip * packetDuration;
			long goToPacketTime = currentPacketTimeStamp + move;
			
			controller.getMovieActionDelegate().goToPacketTimeStamp(goToPacketTime);
			
			SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();

			Slider slider = segmentPanel.getSlider();
			int time = getCurrentTime();
			slider.setSliderPositionTime(time);
			segmentPanel.repaint();

			int X = segmentPanel.getSlider().getSliderPosition();
			
			if(segmentPanel.isUtteranceLoaded()) {
				segmentPanel.getUtteranceView().setVisible(X, X);
			} else {
				segmentPanel.getDatabaseView().setVisible(X, X);
			}
		}

		@Override
		protected Integer doInBackground() throws Exception {
			goToFrame();
			return new Integer(1);
		}

		@Override
		protected void done() {
			controller.setDoneSkipping(true);
		}
	}
}