package edu.bu.signstream.ui.video;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class VideoImage extends JDialog implements ComponentListener {

	private static final long serialVersionUID = -4752966848100689153L;
	private final ImageComponent mOnscreenPicture;

	private int videoWidth = getWidth();
	private int videoHeight = getHeight();

	private int originalVideoWidth = getWidth();
	private int originalVideoHeight = getHeight();
	
	private int minVideoWidth = videoWidth;
	private int minVideoHeight = videoHeight;

	private VideoControlPanel controlPanel = null;
	
	private JLabel label = new JLabel();

	public VideoImage(MultipleMovieController controller, XugglerMovie movie) {
		super();
		mOnscreenPicture = new ImageComponent();
				
		KeyStroke prevFrameKeyStroke = KeyStroke.getKeyStroke(37, 0);
		//getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("R"), "videoControl");
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(prevFrameKeyStroke, "prevFrame");	 
		getRootPane().getActionMap().put("prevFrame", new AbstractAction() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				MultipleMovieController controller = SS3Singleton.getVideoControlManager().getMultipleMovieController();
				controller.skipNumberOfFrames(-1);
				
			}
		});	
		
		KeyStroke nextFrameKeyStroke = KeyStroke.getKeyStroke(39, 0);
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(nextFrameKeyStroke, "nextFrame");	 
		getRootPane().getActionMap().put("nextFrame", new AbstractAction() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				MultipleMovieController controller = SS3Singleton.getVideoControlManager().getMultipleMovieController();
				controller.skipNumberOfFrames(1);
			}
		});	
		
		KeyStroke playKeyStroke = KeyStroke.getKeyStroke(32, 0);
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(playKeyStroke, "play");	 
		getRootPane().getActionMap().put("play", new AbstractAction() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {			
				//MultipleMovieController controller = SS3Singleton.getVideoControlManager().getMultipleMovieController();
				//if(controller.isPlaying()) {
				//	controller.stopPalayingVideoUpdateVideoControlPanel(); //stopPlayingMovie();
				//} else {
					ArrayList<VideoControlPanel> videoPanels = SS3Singleton.getVideoControlManager().getVideoControlPanels();
					if(!videoPanels.isEmpty()) {
						VideoControlPanel videoControlPanel = videoPanels.get(0);
						videoControlPanel.playMovie();
					}
				//}
			}
		});	
		
		this.addComponentListener(new XugglerListener(movie));
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(mOnscreenPicture, BorderLayout.CENTER);
		this.setVisible(true);
		this.toFront();
		
		final JPanel glass = (JPanel) this.rootPane.getGlassPane();
	    glass.setVisible(true);    
	   	    
	    label.setForeground(Color.white);
	    
	    Font font = new Font(label.getFont().getName(), Font.PLAIN, 22);
	    label.setFont(font);

	    glass.setLayout(new FlowLayout(FlowLayout.LEFT));
	    glass.add(label);	    
	    // this.setAlwaysOnTop(true);

		this.addComponentListener(this);

		this.addMouseListener(new java.awt.event.MouseAdapter() {
			@Override
			public void mouseClicked(java.awt.event.MouseEvent evt) {
				// if(evt.getClickCount() == 2) swap();
			}
		});

		controlPanel = SS3Singleton.getVideoControlManager().createVideoControlPanel();
		//controlPanel.addKeyListener(listener);
		getContentPane().add(controlPanel, BorderLayout.SOUTH);
		
		this.setFocusable(true);
	}

	public void setLabel(String text) {
		label.setText(text);
	}

	public boolean isFullScreen() {
		return isFullScreen();
	}

	@Override
	public void setPreferredSize(Dimension preferredSize) {
		super.setPreferredSize(preferredSize);
	}

	public void removeVideoControlPanel(JPanel panel) {
		getContentPane().remove(panel);
	}

	@Override
	public void componentHidden(ComponentEvent e) {
		;
	}

	@Override
	public void componentMoved(ComponentEvent e) {
		Dimension d = this.getSize();

		if(d.width < 300) {
			videoWidth = 300;
			
			Dimension dd = new Dimension(300, videoHeight);
			this.setPreferredSize(dd);
			this.setMaximumSize(dd);
			this.setMinimumSize(dd);

		} else {
			videoWidth = d.width;
			videoHeight = d.height;
			this.setPreferredSize(d);
		}
	}
	
	public void goToMinSize() {		
		videoWidth = minVideoWidth; 
		videoHeight = minVideoHeight;
		
		Dimension d = new Dimension(minVideoWidth, minVideoHeight);				
		mOnscreenPicture.setPreferredSize(d);		
	
		Dimension dd = new Dimension(minVideoWidth, minVideoHeight + 60);
		this.setPreferredSize(dd);
		this.setMaximumSize(dd);
		this.setMinimumSize(dd);
		this.invalidate();
		this.update(getGraphics());
		
		setPreferredSize(dd);		
		mOnscreenPicture.repaint();		
		this.pack();
	}
	
	public void goToMaxSize() {			
		videoWidth = originalVideoWidth; 
		videoHeight = originalVideoHeight;
				
		Dimension d = new Dimension(videoWidth, videoHeight);				
		mOnscreenPicture.setPreferredSize(d);		
	
		Dimension dd = new Dimension(videoWidth, videoHeight + 60);
		this.setPreferredSize(dd);
		this.setMaximumSize(dd);
		this.setMinimumSize(dd);
		this.invalidate();
		this.update(getGraphics());
		
		setPreferredSize(dd);		
		mOnscreenPicture.repaint();		
		this.pack();	
	}
	
	@Override
	public void componentResized(ComponentEvent e) {
		Dimension d = this.getSize();
		
		if(d.width < 300) {
			videoWidth = 300;
			
			Dimension dd = new Dimension(300, videoHeight);
			this.setPreferredSize(dd);
			this.setMaximumSize(dd);
			this.setMinimumSize(dd);

		} else {
			videoWidth = d.width;
			videoHeight = d.height;
			this.setPreferredSize(d);
		}
	}

	@Override
	public void componentShown(ComponentEvent e) {}

	public void displayMessage(String str) {}
	
	private boolean loaded = false;

	public void setImage(final BufferedImage aImage, boolean ignoreRepaint) {
		if(!loaded) {
			loaded = true;
			
			mOnscreenPicture.setImage(aImage, ignoreRepaint);

			videoWidth = aImage.getWidth()
					+ controlPanel.getPreferredSize().height + 50;
			videoHeight = aImage.getHeight() > controlPanel.getPreferredSize().width ? aImage
					.getWidth()
					: controlPanel.getPreferredSize().width;
					
			originalVideoWidth = videoWidth;
			originalVideoHeight = videoHeight;
			
			Dimension prefDimention = controlPanel.getPreferredSize();
			minVideoWidth = prefDimention.width+40;		
			minVideoHeight = (minVideoWidth * originalVideoHeight)/originalVideoWidth;		
			this.setPreferredSize(new Dimension(videoWidth + 15, videoHeight));
		} else {
			loaded = true;
			mOnscreenPicture.setImage(aImage, ignoreRepaint);
			Dimension d = this.getSize();
			videoWidth = d.width;
			videoHeight = d.height;
			this.setPreferredSize(d);
		}

		this.pack();
	}

	public class ImageComponent extends JComponent {

		private static final long serialVersionUID = 1L;

		private Image image;

		public void setImage(Image image, boolean ignoreRepaint) {
			SwingUtilities.invokeLater(new ImageRunnable(image));
		}

		private class ImageRunnable implements Runnable {

			private final Image newImage;

			public ImageRunnable(Image newImage) {
				super();
				this.newImage = newImage;
			}

			@Override
			public void run() {
				ImageComponent.this.image = null;
				ImageComponent.this.image = newImage;
				repaint();
			}
		}

		@Override
		public synchronized void paint(Graphics g) {
			if (image != null) {
				g.drawImage(image, (getWidth() - videoWidth) / 2,
						(getHeight() - videoHeight) / 2, videoWidth,
						videoHeight, this);
				
				image.flush();
			}
		}
	}
}

class XugglerListener implements ComponentListener {	
	private XugglerMovie movie = null;
	
	public XugglerListener(XugglerMovie movie) {
		this.movie = movie;
	}
	
	@Override
	public void componentHidden(ComponentEvent arg0) {
		MultipleMovieController contr = SS3Singleton.getVideoControlManager().getMultipleMovieController();
		contr.removeMovie(movie);
	}

	@Override
	public void componentMoved(ComponentEvent arg0) {}
	@Override
	public void componentResized(ComponentEvent arg0) {}
	@Override
	public void componentShown(ComponentEvent arg0) {}
}
