package edu.bu.signstream.ui.video;

import java.awt.image.BufferedImage;
import java.util.logging.Level;

import com.xuggle.xuggler.ICodec;
import com.xuggle.xuggler.IContainer;
import com.xuggle.xuggler.IPixelFormat;
import com.xuggle.xuggler.IStream;
import com.xuggle.xuggler.IStreamCoder;
import com.xuggle.xuggler.IVideoResampler;

import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class XugglerMovie {
	private VideoImage mScreen = null;
	private BufferedImage javaImage = null;
	private String filename = "";
	private IContainer container = null;
	private int videoStreamId = -1;
	private IStreamCoder videoCoder = null;
	private IVideoResampler resampler = null;
	
	private SS3MediaFile mediaFile = null;

	private MultipleMovieController controller;

	public XugglerMovie(SS3MediaFile mediaFile, MultipleMovieController controller) {
		this.mediaFile = mediaFile;
		this.filename = mediaFile.getFile().getAbsolutePath();
		this.controller = controller;
		load();
	}
	
	public SS3MediaFile getSS3MediaFile() {
		return mediaFile;
	}

	public VideoImage getVideoImage() {
		return mScreen;
	}

	public VideoImage getMScreen() {
		return mScreen;
	}

	public String getFilename() {
		return filename;
	}

	public IContainer getContainer() {
		return container;
	}

	public int getVideoStreamId() {
		return videoStreamId;
	}

	public IStreamCoder getVideoCoder() {
		return videoCoder;
	}

	public IVideoResampler getResampler() {
		return resampler;
	}

	public void load() {
		SS3SignStreamApplication.logger.log(Level.INFO, "First Xuggler call...");

		container = IContainer.make();
		
		SS3SignStreamApplication.logger.log(Level.INFO, "First Xuggler call went OK!!! ");

		// Open up the container
		if (container.open(filename, IContainer.Type.READ, null) < 0)
			throw new IllegalArgumentException("could not open file: "
					+ filename);

		// query how many streams the call to open found
		int numStreams = container.getNumStreams();

		// and iterate through the streams to find the first video stream
		for (int i = 0; i < numStreams; i++) {
			// Find the stream object
			IStream stream = container.getStream(i);
			// Get the pre-configured decoder that can decode this stream;
			IStreamCoder coder = stream.getStreamCoder();

			if (coder.getCodecType() == ICodec.Type.CODEC_TYPE_VIDEO) {
				videoStreamId = i;
				videoCoder = coder;
				break;
			}
		}
		if (videoStreamId == -1)
			throw new RuntimeException(
					"could not find video stream in container: " + filename);

		// open up the decoder so it can do work.
		if (videoCoder.open() < 0)
			throw new RuntimeException(
					"could not open video decoder for container: " + filename);

		if (videoCoder.getPixelType() != IPixelFormat.Type.BGR24) {
			// if this stream is not in BGR24, we're going to need to
			// convert it. The VideoResampler does that for us.
			resampler = IVideoResampler.make(videoCoder.getWidth(), videoCoder
					.getHeight(), IPixelFormat.Type.BGR24, videoCoder
					.getWidth(), videoCoder.getHeight(), videoCoder
					.getPixelType());
			if (resampler == null)
				throw new RuntimeException("could not create color space "
						+ "resampler for: " + filename);
		}

		openJavaWindow();
	}

	public void loadNextFrameImage(BufferedImage aJavaImage) {
		if(this.javaImage != null) {			
			this.javaImage.flush();
			this.javaImage = null;
		}
		
		this.javaImage = aJavaImage;
		aJavaImage.flush();
		aJavaImage = null;
		mScreen.setImage(this.javaImage, false);
	}

	public void updateJavaWindow(boolean ignoreRepaint) {
		mScreen.setImage(javaImage, ignoreRepaint);
	}

	/**
	 * Draws the video on a screen
	 */
	public void updateJavaWindow(BufferedImage javaImage, boolean ignoreRepaint) {
		mScreen.setImage(javaImage, ignoreRepaint);
	}

	/**
	 * Opens a Swing window on screen.
	 */
	public void openJavaWindow() {
		mScreen = new VideoImage(controller, this);	
		SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();
		segmentPanel.getOpenedMediaDialogs().put(mScreen, mediaFile.getFile());
	}
}
