package edu.bu.signstream.ui.video.actions;

import java.awt.image.BufferedImage;
import java.util.ArrayList;

import javax.swing.SwingWorker;

import com.xuggle.xuggler.IContainer;
import com.xuggle.xuggler.IPacket;
import com.xuggle.xuggler.IPixelFormat;
import com.xuggle.xuggler.IVideoPicture;
import com.xuggle.xuggler.video.ConverterFactory;
import com.xuggle.xuggler.video.IConverter;

import edu.bu.signstream.ui.video.MultipleMovieController;
import edu.bu.signstream.ui.video.XugglerMovie;

public class GoToPacketTimeStampWorker extends SwingWorker<Integer, ArrayList<XugglerMovie>> {

	private MultipleMovieController controller = null;
	private long goToPacketTime = 0;

	public GoToPacketTimeStampWorker(final MultipleMovieController controller, long goToPacketTime) {
		this.controller = controller;
		this.goToPacketTime = goToPacketTime;
	}
	
	private void goToFrame() {
		long currentPacketTimeStamp = controller.getCurrentTime();
		long newCurrentPacketTimeStamp = controller.getCurrentTime(); //currentPacketTimeStamp;
		long packetDuration = controller.getPacketDuration(); 
		long dFrame = -1;

		if (currentPacketTimeStamp == goToPacketTime) {
			dFrame = 0;
		} else if (currentPacketTimeStamp < goToPacketTime) {
			dFrame = (goToPacketTime - currentPacketTimeStamp) / packetDuration;
		}
		
		ArrayList<XugglerMovie> movies = controller.getMovies();

		for (XugglerMovie movie : movies) {

			if (dFrame == -1 || dFrame > 20) {
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(), -1,
						0);
				movie.getContainer().seekKeyFrame(movie.getVideoStreamId(),
						goToPacketTime - packetDuration,
						IContainer.SEEK_FLAG_BACKWARDS);
			}

			boolean toScroll = true;

			long cPacketTimeStamp = currentPacketTimeStamp;

			while (toScroll) {
				// scroll frame by frame
				boolean b = false;
				while (!b) {
					IPacket packet = IPacket.make();
					int readPacket = 0;

					try {
						readPacket = movie.getContainer()
								.readNextPacket(packet);
						long packetTimeStamp = packet.getTimeStamp();

						if (packetTimeStamp < 0) {
							cPacketTimeStamp = cPacketTimeStamp
									+ packetDuration;
						} else {
							cPacketTimeStamp = packetTimeStamp;
						}

					} catch (Exception ex1) {
						;
					}

					if (readPacket < 0)
						return;

					if (packet.getStreamIndex() == movie.getVideoStreamId()) {
						b = true;

						IVideoPicture picture = IVideoPicture.make(movie
								.getVideoCoder().getPixelType(), movie
								.getVideoCoder().getWidth(), movie
								.getVideoCoder().getHeight());

						int offset = 0;
						while (offset < packet.getSize()) {

							int bytesDecoded = movie.getVideoCoder()
									.decodeVideo(picture, packet, offset);
							if (bytesDecoded < 0)
								throw new RuntimeException(
										"got error decoding video in: "
												+ movie.getFilename());
							offset += bytesDecoded;

							if (cPacketTimeStamp >= goToPacketTime
									|| controller.getMovieDuration() <= goToPacketTime) {

								if (picture.isComplete()) {
									IVideoPicture newPic = picture;

									if (movie.getResampler() != null) {

										newPic = IVideoPicture.make(movie
												.getResampler()
												.getOutputPixelFormat(),
												picture.getWidth(), picture
														.getHeight());
										if (movie.getResampler().resample(
												newPic, picture) < 0)
											throw new RuntimeException(
													"could not resample video from: "
															+ movie.getFilename());
									}

									// if (cPacketTimeStamp >= goToPacketTime
									// || getMovieDuration() <= goToPacketTime)
									// {

									if (newPic.getPixelType() != IPixelFormat.Type.BGR24)
										throw new RuntimeException(
												"could not decode video"
														+ " as BGR 24 bit data in: "
														+ movie.getFilename());

									// convert the BGR24 to an Java buffered
									// image
									BufferedImage image = new BufferedImage(
											picture.getWidth(),
											picture.getHeight(),
											BufferedImage.TYPE_3BYTE_BGR);
									IConverter converter = ConverterFactory
											.createConverter(image,
													IPixelFormat.Type.BGR24);
									BufferedImage javaImage = converter
											.toImage(newPic);
									movie.loadNextFrameImage(javaImage);
									movie.updateJavaWindow(true);

									toScroll = false;
									newCurrentPacketTimeStamp = cPacketTimeStamp;
								}
							}
						}

					} else {
						// not a video stream, silently drop it.
						do {
						} while (false);
					}
				}
			}

			if (movie.equals(movies.get(movies.size() - 1))) {
				controller.setCurrentPacketTimeStamp(newCurrentPacketTimeStamp);
			}
		}
	}
	
	private boolean playing = false;
	
	@Override
	protected Integer doInBackground() throws Exception {
		if(!controller.isPlaying()) {
			playing = true;
			goToFrame();
		}
		return new Integer(1);
	}

	@Override
	protected void done() {
		controller.stopPlayingMovie();
	}

}
